module automata =
type stateType = enum (stopped, running, blocked);

process printState = (? string str; integer S; !) spec (| str ^= S |)
external;

process thProducer_Automaton =
  {integer QueueSize;}
   ( ? event tick;
       event pProdStart, pProdStop;
       integer pQueueNbItem;
     ! event pMsgSend, pProdStartTimer, pProdStopTimer;
       event Complete;
       event queueReset;
       integer queueItem;
   )

  (| (| S := (#running when c1) default (#stopped when c2) default (#blocked when c3) default (#stopped when c4) default (#running when c5) default ZS 
      | ZS := S $1 init #stopped     
      | S ^= tick
      | printState("Producer_Automaton", integer(S))
      |)
 | c1 := pProdStart when ZS=#stopped
 | c2 := pProdStop when ZS=#running
 | c3 := when not (pQueueNbItem < QueueSize) when ZS=#running
 | c4 := pProdStop when ZS=#running
 | c5 := when (pQueueNbItem < QueueSize) when ZS=#blocked

 | (| Complete := tick
    | queueReset := when c1
    | queueItem :=  getItem()     
    | queueItem ^= pMsgSend
    | pMsgSend := (when (ZS=#running) when (S =#running)) default c5 | sendMessage(pMsgSend)
    | pProdStartTimer := when c3
    | pProdStopTimer :=  when c5
    |)
 |)

where
  event c1, c2, c3, c4, c5;
  stateType S, ZS;
  process  getItem =(? !integer X) external;
  process  sendMessage =(? event x ;!) external;
end;
  
process thConsumer_Automaton =
   ( ? event tick;
       event pConsStart, pConsStop;
       event pMsgRecv;       
       integer pQueueNbItem;
     ! event pConsStartTimer, pConsStopTimer;
       event Complete;
       event dequeueItem;
   )

  (| (| S := (#running when c1) default (#stopped when c2) default (#blocked when c3) default (#stopped when c4) default (#running when c5) default ZS 
      | ZS := S $1 init #stopped 
      | printState("thConsumer_Automaton", integer(S))
      | S ^= tick
      |)
   | c1 := pConsStart when ZS=#stopped
   | c2 := pConsStop when ZS=#running
   | c3 := when not (pQueueNbItem >0) when ZS=#running
   | c4 := pConsStop when ZS=#running
   | c5 := when (pQueueNbItem >0) when ZS=#blocked
   | (| Complete := tick
      | dequeueItem := (when (ZS=#running) when (S =#running)) when pMsgRecv default c5
      | pConsStartTimer := when c3
      | pConsStopTimer :=  when c5
      |)
   |)

where
  event c1, c2, c3, c4, c5;
  type stateTypeConsumer = enum (stopped, running, blocked);
  stateTypeConsumer S, ZS;
 end;
  
process Timer_Automaton = {integer vmax}
( ? event tick;
    event pStart, pStop;
  ! event pTimeOut;
    event Complete;
   )

  (| (| S := (#running when c1) default (#stopped when c2)  default ZS 
      | ZS := S $1 init #stopped
%       | printState("Timer_Automaton", integer(S))%
      | S ^= tick
      |)
   | c1 := pStart when ZS=#stopped default when pTimeOut
   | c2 := pStop when ZS=#running
   | timerValue := (0 when c1) default ((ztimerValue+1) when (ZS = S) when (S = #running))  default ztimerValue
   | ztimerValue := timerValue $ init 0
   | timerValue ^= tick
   | (| Complete := tick
      | pTimeOut := when (ztimerValue>vmax)
      |)
   |)
  where
    event c1, c2;
    type stateTypeTimer = enum (stopped, running);
    stateTypeTimer S, ZS;
    integer timerValue, ztimerValue;
  end;
    
  end;
    
   
