package fr.inria.tea.aadl2ssme.scheduling.actions;
import java.util.*;

import fr.inria.tea.aadl2ssme.scheduling.idstruct.*;

public class ExportVCD {
	public ArrayList<VcdVariable> traceVcdVariables = new ArrayList<VcdVariable>();
	public L_MSTA lMSTA;
	private int hyperPeriod;
	private int minInterval;
	String strTimeScale = "";
	I_Process iProcess;
	
	public ExportVCD(L_MSTA msta,  I_Process process, int int1, int int2, String str){
		lMSTA = msta;
		hyperPeriod = int1;
		minInterval = int2;
		strTimeScale =  str;
		iProcess = process;
	}
	
	public String generateVCD(){
		String text= new String("");
		generateVCDVariables();
		text = generateHeading() + generateBody(10);
		return text;
	}
	
	public String generateDate(){
		Date date = new Date();
		return "$date\n    " + date.toString() + "\n$end\n";
	}
	
	public String generateVersion(){
		return "$version\n    Scheduling synthesis export" + "\n$end\n";
	}
	
	public String generateTimeScale(){
		return "$timescale\n    " + getTimeScaleNumber() + " " + getTimeScale() + "\n$end\n";
	}
	
	public String generateVariableList(){
		String strVariableList = "";
		for (int i=0; i<traceVcdVariables.size();i++){
			strVariableList = strVariableList + "$var trireg 1 " + traceVcdVariables.get(i).strVCDVariableName + " " + traceVcdVariables.get(i).strVariableName + " $end\n";
		}
		return strVariableList + "$enddefinitions\n";
	}

	public String generateHeading(){
		
		return generateDate() + generateVersion() + generateTimeScale() + generateVariableList() + "\n$end\n\n";
	}

	public String generateBody(int intHyperPeriodNumber){
		String strAllInstants = new String();
		String strOneInstant = new String();
		ArrayList<Boolean> bPrevValueList = new ArrayList<Boolean>();
//		ArrayList<Boolean> bValueChangeList = new ArrayList<Boolean>();
		for (int l=0; l<traceVcdVariables.size(); l++){
			bPrevValueList.add(traceVcdVariables.get(l).indexArray[0]);
			//bValueChangeList.add(false);
		}
		
		for (int k=0; k<intHyperPeriodNumber; k++){
			//period instants list
			for (int i=0; i<hyperPeriod; i++){
				strOneInstant = "";
				//variable list
				for (int j=0; j<traceVcdVariables.size(); j++){
					if (bPrevValueList.get(j) != traceVcdVariables.get(j).indexArray[i]){
						//bValueChangeList.set(j, true);
						if (traceVcdVariables.get(j).indexArray[i]){
							strOneInstant = strOneInstant + "1" + traceVcdVariables.get(j).strVCDVariableName + "\n";
						} else {
							strOneInstant = strOneInstant + "0" + traceVcdVariables.get(j).strVCDVariableName + "\n";
						}
					} 
					bPrevValueList.set(j, traceVcdVariables.get(j).indexArray[i]);
				}
				if (strOneInstant.length()>0){
					strAllInstants = strAllInstants + "#" + Integer.toString(i + k*hyperPeriod) + "\n" + strOneInstant;
				}
			}
		}
		strOneInstant = "#0\n$dumpvars\n";
		for (int i=0; i<traceVcdVariables.size(); i++){
			if (traceVcdVariables.get(i).indexArray[0]){
				strOneInstant = strOneInstant + "1" + traceVcdVariables.get(i).strVCDVariableName + "\n";
			} else {
				strOneInstant = strOneInstant + "0" + traceVcdVariables.get(i).strVCDVariableName + "\n";	
			}
		}
		strOneInstant = strOneInstant + "$end\n";
		strAllInstants = strOneInstant + strAllInstants;
		return strAllInstants;
	}
	
	public String getTimeScale(){
		return "ns";
	}
	public int getTimeScaleNumber(){
		return 1;
	}
	
	public void generateVCDVariables(){
		int i=0;
		I_Thread cIThread;		
		int charInt = 0x21;
		
		//thread list
		for (i=0; i<iProcess.threadLists.size();i++){
			cIThread = iProcess.threadLists.get(i);

			//dispatch event
			creatThreadEvent(cIThread, EventType.ET_DISPATCH, charInt);
			charInt += 1;
			
			//start event
			creatThreadEvent(cIThread, EventType.ET_START, charInt);
			charInt += 1;
			
			//complete event
			creatThreadEvent(cIThread, EventType.ET_COMPLETE, charInt);
			charInt += 1;
			
			//deadline event
			creatThreadEvent(cIThread, EventType.ET_DEADLINE, charInt);
			charInt += 1;
			
			charInt = creatPortEvent(cIThread, charInt) + 1;
			
			charInt = creatThreadExecution(iProcess, charInt) + 1 ;
		}
		return;
	}
	public int creatThreadExecution (I_Process cIProcess, int charInt){
		int indexChar = charInt;
		int intThreadNumber = cIProcess.threadLists.size();
		VcdVariable vcdVariable;

		//in port list
		for (int k=0; k<intThreadNumber;k++){
			vcdVariable = new VcdVariable(cIProcess.threadLists.get(k).name.replace(" ", "_") + "_Execution", "*" + Character.toString((char) indexChar), hyperPeriod);
			indexChar +=1;
			traceVcdVariables.add(vcdVariable);
			for (int l=0; l<cIProcess.threadLists.get(k).threadInstanceList.size();l++){
				setVCDVariableIntervalValue(vcdVariable, cIProcess.threadLists.get(k).threadInstanceList.get(l).startMSTAIndex, cIProcess.threadLists.get(k).threadInstanceList.get(l).completeMSTAIndex);
			}
		}
		return indexChar;
	}
	public void setVCDVariableIntervalValue(VcdVariable vcdVariable, int a, int b){
		if (a<=b){
			for (int i=a; i<=b-1;i++){
				vcdVariable.indexArray[i] = true;
			}
		}
	}
	public int creatPortEvent(I_Thread cIThread, int charInt){
		int indexChar = charInt;
		int intPortNumber = cIThread.lThread.inPortList.size();
		int index = 0, ioSpecSize =0;
		VcdVariable vcdVariable;
		I_ThreadInstance cIThreadInstance;
		String strPortName = "";
		
		//in port list
		for (int k=0; k<intPortNumber;k++){
			strPortName = cIThread.lThread.inPortList.get(k).portName;
			if (strPortName == null) {
				strPortName = "In_Port_" + Integer.toString(k);
			} else {
				if (strPortName.length()<1){
					strPortName = "In_Port_" + Integer.toString(k);
				}
			}

			vcdVariable = new VcdVariable(cIThread.name.replace(" ", "_") + "_" + strPortName, "*" + Character.toString((char) indexChar), hyperPeriod);
			indexChar +=1;
			traceVcdVariables.add(vcdVariable);
			//thread instance list
			for (int j=0; j<cIThread.threadInstanceList.size();j++){
				cIThreadInstance = cIThread.threadInstanceList.get(j);
				ioSpecSize = cIThreadInstance.iPortIOTimeMSTAIndexList.get(k).alInt.size();
				//io spec list
				for (int i = 0; i<ioSpecSize; i++){
					if (cIThreadInstance.iPortIOTimeMSTAIndexList.get(k).alInt != null){
						index = cIThreadInstance.iPortIOTimeMSTAIndexList.get(k).alInt.get(i);
						if (index < vcdVariable.indexArray.length){
							vcdVariable.indexArray[index] = true;
						}						
					}
				}
			}
		}
				
				
		intPortNumber = cIThread.lThread.outPortList.size();
		index = 0;
		ioSpecSize =0;
		//out port list
		for (int k=0; k<intPortNumber;k++){
			strPortName = cIThread.lThread.outPortList.get(k).portName;
			if (strPortName == null) {
				strPortName = "Out_Port_" + Integer.toString(k);
			} else {
				if (strPortName.length()<1){
					strPortName = "Out_Port_" + Integer.toString(k);
				}
			}

			vcdVariable = new VcdVariable(cIThread.name.replace(" ", "_") + "_" + strPortName, "*" + Character.toString((char) indexChar), hyperPeriod);
			indexChar += 1;
			traceVcdVariables.add(vcdVariable);
			//thread instance list
			for (int j=0; j<cIThread.threadInstanceList.size();j++){
				cIThreadInstance = cIThread.threadInstanceList.get(j);
				ioSpecSize = cIThreadInstance.oPortIOTimeMSTAIndexList.get(k).alInt.size();
				//io spec list
				for (int i = 0; i<ioSpecSize; i++){
					if (cIThreadInstance.oPortIOTimeMSTAIndexList.get(k).alInt != null){
						index = cIThreadInstance.oPortIOTimeMSTAIndexList.get(k).alInt.get(i);
						if (index < vcdVariable.indexArray.length){
							vcdVariable.indexArray[index] = true;
						}						
					}
				}
			}
		}
		return 	indexChar;
	}
	
	public void creatThreadEvent(I_Thread cIThread, EventType eventType, int indexChar){
		int eventIndex = 0;
		VcdVariable vcdVariable = new VcdVariable(cIThread.name.replace(" ", "_") + "_" + eventType.toString().substring(3), "*" + Character.toString((char) indexChar), hyperPeriod);
		traceVcdVariables.add(vcdVariable);
		//int index = traceVcdVariables.indexOf(vcdVariable);
		I_ThreadInstance cIThreadInstance;
		for (int j=0; j<cIThread.threadInstanceList.size();j++){
			cIThreadInstance = cIThread.threadInstanceList.get(j);
			switch (eventType) {
	        case ET_DISPATCH: 
	        	eventIndex = cIThreadInstance.dispatchMSTAIndex;
	        	break;
	        case ET_START:
	        	eventIndex = cIThreadInstance.startMSTAIndex;
	        	break;
	        case ET_COMPLETE:
	        	eventIndex = cIThreadInstance.completeMSTAIndex;
	        	break;
	        case ET_DEADLINE:
	        	eventIndex = cIThreadInstance.deadlineMSTAIndex;
	        	break;      	
	        default:
	        	eventIndex = -1;
	        	break;
	        }	

			if ((eventIndex < vcdVariable.indexArray.length)&&(eventIndex>=0)){
				vcdVariable.indexArray[eventIndex]= true;
			}
		}
	}
}
