package fr.inria.tea.aadl2ssme.scheduling.actions;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.jface.dialogs.MessageDialog;

import fr.inria.tea.aadl2ssme.scheduling.actions.*;
import fr.inria.tea.aadl2ssme.scheduling.idstruct.*;
/**
 * Our sample action implements workbench action delegate.
 * The action proxy will be created by the workbench and
 * shown in the UI. When the user tries to use the action,
 * this delegate will be created and execution will be 
 * delegated to it.
 * @see IWorkbenchWindowActionDelegate
 */
public class Scheduling implements IWorkbenchWindowActionDelegate {
	
	private IWorkbenchWindow window;
	
	/**
	 * The constructor.
	 */
	public Scheduling() {
	}
	public Scheduling(L_Process process) {
		lProcess = process;
	}
	public Scheduling(SchedulingPolicy policy) {
		schedulingPolicy = policy;
	}	
	private L_MSTA lMSTA;
	private L_Process lProcess;
	private I_Process iProcess = new I_Process();
	private int hyperPeriod = 0;
	private int minumumInterval = 0;
	private L_AffineClocks lAffineClocks;
	private SchedulingPolicy schedulingPolicy;
	private boolean isDebugging = false;
	private boolean bShowProcess = false;
	private boolean bOneAllocForN = true;
	private ArrayList<I_ThreadInstance> processedTIList = new ArrayList<I_ThreadInstance>();
	public boolean shiftedDispatch = false; //TODO value by default
	public boolean shiftedStart = true; //TODO value by default
	public boolean ioEventOverlap = true; //TODO value by default
	
	/**
	 * The action has been activated. The argument of the
	 * method represents the 'real' action sitting
	 * in the workbench UI.
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {
//		if (bShowProcess){
//		MessageDialog.openInformation(
//			window.getShell(),
//			"Start the scheduling of AADL process...",
//			"Hello, Eclipse world");
//		}
		if (action.getId().indexOf("EDFSchedulingAction") > 0){
			schedulingPolicy = SchedulingPolicy.SP_EDF;
		} else if (action.getId().indexOf("dependencySchedulingAction") > 0){
			schedulingPolicy = SchedulingPolicy.SP_DEPENDENCY;
		}else if (action.getId().indexOf("RMSchedulingAction") > 0){
			schedulingPolicy = SchedulingPolicy.SP_RM;
		} else {
			schedulingPolicy = SchedulingPolicy.SP_DEPENDENCY;
		}
		
		performScheduling();
		
	}

	/**
	 * Selection in the workbench has been changed. We 
	 * can change the state of the 'real' action here
	 * if we want, but this can only happen after 
	 * the delegate has been created.
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
	}

	/**
	 * We can use this method to dispose of any system
	 * resources we previously allocated.
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
	}

	/**
	 * We will cache window object in order to
	 * be able to provide parent shell for the message dialog.
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}
	public void performScheduling() {
		//
		Initialize();
		
		//
		GetProcess();
		
		//
		SetSchedulingPolicy();
		
		//
		hyperPeriod = CalculateHyperPeriod();
		
		//
		minumumInterval = CalculateMinumumInterval();
		
		//initialize the I_Process
		InitializeIProcess(shiftedDispatch, shiftedStart, ioEventOverlap);
		
		InitializeLMSTA();
		
		//EDF_Scheduling edf_Scheduling= new EDF_Scheduling(lMSTA, lProcess);
		
		//The main algo to do the scheduling
		MainSchedulingAlgorithm(schedulingPolicy);
		
		//add all input and output ports io frozen events
		CompleteIProcessWithIOTimeSpec();
		
		//Show scheduling result for every thread instance
		ShowSchedulingResultInConsole();
		
		//Show scheduling result for every event 
		ShowIndexedSchedulingResultInConsole();
		
		//export the resulting to text
		ExportSchedule2Text(lMSTA);
				
		//export the resulting to VCD file format
		ExportSchedule2VCD(lMSTA);
				
		
		//construct Signal affine clocks
		MapToSignalAffineClocks(lMSTA, iProcess);
		
		//export the affine clocks to process behavior in ASME2SSME
		ExportASME2SSME(lAffineClocks);
		
		
		showProgressInfo(bShowProcess, " SCHEDULING COMPLETES!");
		
	}
	public void	Initialize(){
		showProgressInfo(bShowProcess, "---START--- Initialize.");
		//L_MSTA
		lMSTA = new L_MSTA();
		//L_Process
		//iProcess = new I_Process();
		lAffineClocks = new L_AffineClocks();
		//hyperPeriod = 0;
		//minumumInterval = 0;
		//schedulingPolicy= SchedulingPolicy.SP_EDF;
		if (schedulingPolicy== null){
			schedulingPolicy = SchedulingPolicy.SP_EDF;
		}
	}
	
	public void GetProcess(){
		
		TestLProcess1 testLProcess1 = new TestLProcess1();
		lProcess = testLProcess1.getLProcess();
		iProcess = testLProcess1.getIProcess();
	}

	public void SetSchedulingPolicy(){
		
		if (schedulingPolicy == null) schedulingPolicy = SchedulingPolicy.SP_EDF;
	}
	
	public void MainSchedulingAlgorithm(SchedulingPolicy schedulingPolicy){
		showProgressInfo(bShowProcess, "---START---MainSchedulingAlgorithm.");
		//make a list of thread instances
		ArrayList<AL_ThreadInstance> schedulingThreadInstanceGroupList;
		
	    AL_ThreadInstance currentThreadInstanceList;
	    I_ThreadInstance currentThreadInstance;
	    boolean isSchedulable = false;
		
		//make a ordered list of all threads (thread instance) according to the scheduling policy
		schedulingThreadInstanceGroupList = CreatePOrderedThreadInstanceList(iProcess, schedulingPolicy);
		
		if (schedulingThreadInstanceGroupList.size() < 1) {
			showDebugInfo(isDebugging, "Scheduling queue is vide, exit!");
			return;
		}
		showProgressInfo(bShowProcess, "Start to process the thread instance queue, according to " + schedulingPolicy.toString());
		
		Iterator<AL_ThreadInstance> itSTIL = schedulingThreadInstanceGroupList.iterator();
	    //use hasNext() and next() methods of Iterator to iterate through the elements
		int instanceNumberInGroup = 0;
	    
	    while(itSTIL.hasNext()){
	    	
	    	currentThreadInstanceList = (AL_ThreadInstance) itSTIL.next();
	    	String str111 = Integer.toString(currentThreadInstanceList.threadInstanceList.get(0).deadlineMSTAIndex);
	    	String str112 = Integer.toString(currentThreadInstanceList.threadInstanceList.size());
	    	showProgressInfo(bShowProcess, "Current thread instance group with deadline: " + str111 + " with " +str112 + " instance(s)");
	    	
	    	instanceNumberInGroup = currentThreadInstanceList.threadInstanceList.size();
	    	
	    	if (instanceNumberInGroup == 0) { //No tasks in the instance group: queue is vide, algorithm ends
	    		showDebugInfo(isDebugging, "Scheduling ends, check results!");
	    	} else if (instanceNumberInGroup == 1){ //only one instance in the group
	    		//Only one task in the current instance group, process this instance
    			currentThreadInstance = (I_ThreadInstance) currentThreadInstanceList.threadInstanceList.get(0);
    			
	    		isSchedulable = CalPeriodicThreadInstance(currentThreadInstance, lMSTA); 
	    		if (isSchedulable) {
		    		//Allocate this thread instance on msta
		    		//Copy tempThreadInstance to currentThreadInstance
	    			//CopySchedulableThreadInstance(tempThreadInstance, iTI);
	    			showProgressInfo(bShowProcess,"Current thread instance has been scheduled: "+ currentThreadInstance.getThreadInstanceName());
	    		} else {
	    			showDebugInfo(isDebugging, "Current thread instance is not schedulable, exit!");
	    		}
	    	} else { // instanceNumberInGroup > 1: this group has more than one instance
	    		//the instance order is not important, so the order will be changed when there is schedulability problem for one instance 
	    		//TODO here, we need to probe all the order possibility until we find one that works
	    		
	    		isSchedulable = calThreadInstanceList(currentThreadInstanceList);
	    		if (isSchedulable) {
		    		//Allocate this thread instance on msta
		    		//Copy tempThreadInstance to currentThreadInstance	
	    			//CopySchedulableThreadInstance(tempThreadInstance, currentThreadInstance);
	    			//showProgressInfo(bShowProcess,"Current thread instance has been scheduled: "+ currentThreadInstance.getThreadInstanceName());
	    		} else {
	    			//TODO ROLLBACK: (jump to the next step of this loop)
	    		}
	    	}
	    }
	}
	
//	public ArrayList<I_ThreadInstance> GetAllInstancesForSameThread(I_ThreadInstance iTI, ArrayList<AL_ThreadInstance> sTIGL){
//		ArrayList<I_ThreadInstance> resultTIList = new ArrayList<I_ThreadInstance>();
//		ArrayList<I_ThreadInstance> tempTIList = new ArrayList<I_ThreadInstance>();
//		I_ThreadInstance tempTI;
//		Iterator<AL_ThreadInstance> itSTIGL = sTIGL.iterator();
//    	while(itSTIGL.hasNext()){		
//    		AL_ThreadInstance currentThreadInstanceList = (AL_ThreadInstance) itSTIGL.next();
//    		tempTIList = currentThreadInstanceList.threadInstanceList;
//    		Iterator<I_ThreadInstance> itTIL = tempTIList.iterator();
//        	while(itTIL.hasNext()){	
//        		tempTI = (I_ThreadInstance) itTIL.next();
//        		if (tempTI.iThread == iTI.iThread){
//        			resultTIList.add(tempTI);
//        		}
//        	}
//    	}
//		return resultTIList;
//	
//	}

	public boolean CalPeriodicThreadInstance(I_ThreadInstance iCTI, L_MSTA lMSTA){
		//TODO
		boolean isSchedulable;
		if (iCTI.isAllocated) {
			 // the thread instace has already been allocated, so add it to processed threadinstancelist. 
			if (!processedTIList.contains(iCTI)) processedTIList.add(iCTI);
			showProgressInfo(bShowProcess, "Thread instance: " + iCTI.getThreadInstanceName() + " has already been processed!");
			return true;
		}

		///////////////////
		TimeAllocation timeAlloc = new TimeAllocation(iCTI, lMSTA, bOneAllocForN);
		
		//tempThreadInstance 
		isSchedulable = timeAlloc.calPeriodicThreadInstance(iCTI, lMSTA, bOneAllocForN);
		//TODO [removed] CHECK schedulability for this thread instance -> "external function"
		//isSchedulable = CheckThreadInstanceSchedulable(tempThreadInstance, lMSTA);
		if (isSchedulable) {
    		//Allocate this thread instance on msta
    		//Copy tempThreadInstance to iCTI
			//CopySchedulableThreadInstance(tempThreadInstance, iCTI);
			showProgressInfo(bShowProcess,"Current thread instance has been scheduled: "+ iCTI.getThreadInstanceName());		
			return true;
		} else {
			showDebugInfo(isDebugging, "Current thread instance is not schedulable, exit!");
			return false;
		}

	}

	public boolean 	calThreadInstanceList(AL_ThreadInstance alThreadInstance){
		Iterator<I_ThreadInstance> itALTI = alThreadInstance.threadInstanceList.iterator();
		I_ThreadInstance currentThreadInstance;
    	while(itALTI.hasNext()){
	    	currentThreadInstance = (I_ThreadInstance) itALTI.next();
	    	showProgressInfo(bShowProcess, "Current thread instance: " + currentThreadInstance.getThreadInstanceName());
    		
	    	if (currentThreadInstance.isAllocated){
	    		showDebugInfo(isDebugging, "Current thread instance is already allocated!");
	    	} else {
	    		CalPeriodicThreadInstance(currentThreadInstance, lMSTA); 
	    	}//if
    	}//while
		return true;
	}
	
//	public I_ThreadInstance CalPeriodicThreadInstance(I_ThreadInstance iThreadInstance, L_MSTA msta){
//		//TODO
//		TimeAllocation timeAlloc = new TimeAllocation(iThreadInstance, msta);
//		//TODO to change 
//		I_ThreadInstance iTI = timeAlloc.calPeriodicOtherInstances(iThreadInstance, msta);
//		return iTI;		
//	}
	
	
//	public boolean CheckThreadInstanceSchedulable(I_ThreadInstance iThreadInstance, L_MSTA msta){
//		//TODO		
//		return false;
//	}
	
	
//	public void	CopySchedulableThreadInstance(I_ThreadInstance tempThreadInstance, I_ThreadInstance currentThreadInstance){
//		currentThreadInstance.iThread = tempThreadInstance.iThread;
//		currentThreadInstance.period = tempThreadInstance.period;
//		currentThreadInstance.isAllocated = true;
//		currentThreadInstance.instanceNumber = tempThreadInstance.instanceNumber;
//		currentThreadInstance.dispatchMSTAIndex = tempThreadInstance.dispatchMSTAIndex;
//		currentThreadInstance.frozenMSTAIndex = tempThreadInstance.frozenMSTAIndex;
//		currentThreadInstance.startMSTAIndex = tempThreadInstance.startMSTAIndex;
//		currentThreadInstance.completeMSTAIndex = tempThreadInstance.completeMSTAIndex;
//		currentThreadInstance.outputMSTAIndex = tempThreadInstance.outputMSTAIndex;
//	}
	
//	public boolean CheckThreadInstanceSchedulable(){
//		//TODO		
//		return false;
//	}
	
	public ArrayList<AL_ThreadInstance> CreatePOrderedThreadInstanceList(I_Process iProcess, SchedulingPolicy schedulingPolicy){
		//make a ordered list of all threads (thread instance) according to the scheduling policy

		switch (schedulingPolicy) {
        case SP_EDF:
    		EDF_Scheduling schedulingEDF = new EDF_Scheduling(iProcess, bShowProcess);
    		return schedulingEDF.CreateEDFThreadInstanceList();
		case SP_DEPENDENCY:
			Dependency_Scheduling schedulingDependency = new Dependency_Scheduling(iProcess, bShowProcess);
			return schedulingDependency.CreateDependencyThreadInstanceList();
        case SP_RM:
			RM_Scheduling schedulingRM = new RM_Scheduling(iProcess, bShowProcess);
			return schedulingRM.CreateRMThreadInstanceList();
        case SP_OTHER:  
        	return null;                 
        default: 
        	return null;
		}
	}
	
		
	public int locateTIInSQ(ArrayList<Integer> indexList, I_ThreadInstance iTICPO){
		int value = 0;
		int deadline = iTICPO.deadlineMSTAIndex;
		if (indexList.size() < 1) {
			return 0;
		}
		for (int p=0; p<indexList.size();p++){
			value = indexList.get(p);
			if (deadline == value){
				//the same deadline, so insert in the same position
				return -p;
			} else if (deadline < value) {
				return p;
			} 
		}
		return 0;//add the element in the tail of the queue
	}
	
	
	public int CalculateHyperPeriod(){
		String str = "CalculateHyperPeriod"; 
		if ((!checkExistence(lMSTA, "MSTA",str))||(!checkExistence(lProcess,"LProcess",str))||(!checkExistence(iProcess,"IProcess",str))){
			return -1;
		}
		ArrayList<Integer> pPeriodIntegerList;
		int resultHyperPeriod = 0;
		pPeriodIntegerList = getPeriodListFromLProcess();
		resultHyperPeriod = getLeastCommonMultiple(pPeriodIntegerList);
		showProgressInfo(bShowProcess, "hyperPeriod: " + Integer.toString(resultHyperPeriod));
		return resultHyperPeriod;
	}	
	public ArrayList<Integer> getPeriodListFromLProcess(){
		//ArrayList<TimeUnit> pTimeUnitList = new ArrayList<TimeUnit>();
		ArrayList<Integer> pPeriodIntegerList = new ArrayList<Integer>();
//		for (i=0;i<lProcess.threadLists.size();i++){
//			pPeriodIntegerList.add(lProcess.threadLists.get(i).period);
//			pTimeUnitList.add(lProcess.threadLists.get(i).periodTimeUnit);
//		}
		
		for (int i=0;i<lProcess.threadLists.size();i++){
			if (lProcess.threadLists.get(i).periodTimeUnit != lProcess.threadLists.get(0).periodTimeUnit){
				showDebugInfo(isDebugging, "Period time units are not the same, exit.");
				return null;
			}
			pPeriodIntegerList.add(lProcess.threadLists.get(i).period);
		}
		return pPeriodIntegerList;
	}
	public int getLeastCommonMultiple(ArrayList<Integer> iIntegerList){
		    int o = iIntegerList.get(0);
		    for(int i = 1; i < iIntegerList.size(); i++){
		    	o = getLeastCommonMultiple(o, iIntegerList.get(i));
		    }
		    return o;
	}
	
	public int getLeastCommonMultiple(int i1, int i2){
		int o = i1 * (i2 / getGreatestCommonDivisor(i1, i2));
	    return o;
	}

	
	public int getGreatestCommonDivisor(int i1, int i2){
		int a=i1, b=i2;
	    while (b > 0){
	        int t = b;
	        b = a % b; 
	        a = t;
	    }
	    return a;
	}
	
	public int getGreatestCommonDivisor(ArrayList<Integer> iIntegerList){
	    int o = iIntegerList.get(0);
	    for(int i = 1; i < iIntegerList.size(); i++){
	    	o = getGreatestCommonDivisor(o, iIntegerList.get(i));
	    }
	    return o;
	}
	
	public int CalculateMinumumInterval(){
		//calculate the minumum interval in a hyper-period so that instants are fixed. Scheduling is based on these instants.
		String str = "CalculateMinumumInterval";
		if ((!checkExistence(lMSTA, "MSTA",str))||(!checkExistence(lProcess,"LProcess",str))||(!checkExistence(iProcess,"IProcess",str))){
			return -1;
		}
		
		int minPeriod = 0;
		int minInterval = 0;
		int maxEventNumber = 0;
		int i=0, j=0, k=0, o=0;
		ArrayList<IO_TimeSpec> ioTimeSpecList;
		
		ArrayList<Integer> pPeriodIntegerList = new ArrayList<Integer>();	
		for (i=0;i<lProcess.threadLists.size();i++){
			if (lProcess.threadLists.get(i).periodTimeUnit != lProcess.threadLists.get(0).periodTimeUnit){
				showDebugInfo(isDebugging, "Period time units are not the same, exit.");
				return -1;
			}
			pPeriodIntegerList.add(lProcess.threadLists.get(i).period);
			if (minPeriod > lProcess.threadLists.get(i).period){
				minPeriod = lProcess.threadLists.get(i).period;
			}
			if (minInterval > minPeriod){
				minInterval = minPeriod;
			}			
			pPeriodIntegerList.add(lProcess.threadLists.get(i).computationTime);
			
			for (j=0;j<lProcess.threadLists.get(i).inPortList.size();j++){
				ioTimeSpecList = lProcess.threadLists.get(i).inPortList.get(j).ioTimePropList;
				if (maxEventNumber < ioTimeSpecList.size()){
					maxEventNumber = ioTimeSpecList.size();
				}
				for (k=0;k<ioTimeSpecList.size();k++){
					pPeriodIntegerList.add(ioTimeSpecList.get(k).io_time);
					if (minInterval > ioTimeSpecList.get(k).io_time){
						minInterval = ioTimeSpecList.get(k).io_time;
					}
				}
			}
			for (j=0;j<lProcess.threadLists.get(i).outPortList.size();j++){
				ioTimeSpecList = lProcess.threadLists.get(i).outPortList.get(j).ioTimePropList;
				if (maxEventNumber < ioTimeSpecList.size()){
					maxEventNumber = ioTimeSpecList.size();
				}				
				for (k=0;k<ioTimeSpecList.size();k++){
					pPeriodIntegerList.add(ioTimeSpecList.get(k).io_time);
					if (minInterval > ioTimeSpecList.get(k).io_time){
						minInterval = ioTimeSpecList.get(k).io_time;
					}
				}
			}
		
		}
		o = getGreatestCommonDivisor(pPeriodIntegerList);
//		if ((maxEventNumber * o) > minPeriod){
//			reduce();
//		}
//		if ((minInterval * maxEventNumber) > minPeriod){
//			reduce();
//		}
//		minInterval
//		minPeriod
//		maxEventNumber
		showProgressInfo(bShowProcess, "minumumInterval: " + Integer.toString(o));
		return o;
	}

	public void ExportSchedule2Text(L_MSTA msta){
		//TODO
		showProgressInfo(bShowProcess,"---TODO---ExportSchedule2Text");
	}
	
	public void ExportSchedule2VCD(L_MSTA msta){
		String str;
		ExportVCD exportVCD = new ExportVCD(msta, iProcess, hyperPeriod, minumumInterval, "ns");
		str = exportVCD.generateVCD();
		showProgressInfo(bShowProcess,"---TODO---ExportSchedule2VCD");
		showProgressInfo(bShowProcess,"ExportSchedule2VCD\n\n" + str);
	}
	
	
	public L_AffineClocks MapToSignalAffineClocks(L_MSTA msta, I_Process iProcess){
		//TODO
		showProgressInfo(bShowProcess,"---TODO---MapToSignalAffineClocks");
		return null;
	}
	
	public void	ExportASME2SSME(L_AffineClocks lAffineClocks){
		//TODO
		showProgressInfo(bShowProcess,"---TODO---ExportASME2SSME");
		
	}
	
	public void InitializeIProcess(boolean shiftedDispatch, boolean shiftedStart, boolean ioEventOverlap){
		showProgressInfo(bShowProcess, "---START---InitializeIProcess.");
		String str = "InitializeIProcess"; 
		if ((!checkExistence(lMSTA, "MSTA",str))||(!checkExistence(lProcess,"LProcess",str))||(!checkExistence(iProcess,"IProcess",str))){
			return;
		}
		if (hyperPeriod == 0){
			showDebugInfo(isDebugging, "InitializeIProcess error: hyperPeriod is 0.");	    
			return;
		}
		if (minumumInterval == 0){
			showDebugInfo(isDebugging, "InitializeIProcess error: minumumInterval is 0.");	    
			return;
		}
		iProcess.processName = lProcess.processName;
		iProcess.processInstanceNameInAADL = lProcess.processInstanceNameInAADL;
		iProcess.processImplementationNameInAADL = lProcess.processImplementationNameInAADL;
		ArrayList<I_Thread> newThreadList = iProcess.threadLists;
		//new ArrayList<I_Thread>();
		//iProcess.threadLists = newThreadList;
		Iterator<L_Thread> itAADLThreadList = lProcess.threadLists.iterator();
		L_Thread lThread;
		int tempPeriodNumber = 0;
		//for every thread in the list
		while(itAADLThreadList.hasNext()){
			lThread = itAADLThreadList.next();
			I_Thread iThread = new I_Thread();
			iThread.lThread = lThread;
			iThread.name = lThread.threadName;
			iThread.nameInstanceInAADL = lThread.threadInstanceNameInAADL;
			iThread.nameImplementationInAADL = lThread.threadImplimentationInAADL;
			iThread.periodOnMSTA = getInstantNumberOnMSTA(lThread.period);
			iThread.computationTimeOnMSTA = getInstantNumberOnMSTA(lThread.computationTime);
			iThread.deadlineOnMSTA = getInstantNumberOnMSTA(iThread.lThread.deadline);
			tempPeriodNumber = hyperPeriod/iThread.periodOnMSTA;
			iThread.periodNumber = tempPeriodNumber;//= lThread.
			iThread.shiftedDispatch = shiftedDispatch; //TODO value by default
			iThread.shiftedStart = shiftedStart; //TODO value by default
			iThread.ioEventOverlap = ioEventOverlap; //TODO value by default
			iThread.firstReleaseTime = 0;
			iThread.isAllocated = false; //TODO value by default

			//for every thread instance in the hyper-period
			//newThreadInstanceList 
			ArrayList<I_ThreadInstance> newThreadInstanceList = new ArrayList<I_ThreadInstance>();
			for (int i=0; i< iThread.periodNumber; i++){
				I_ThreadInstance iThreadInstance = new I_ThreadInstance();
				iThreadInstance.iThread = iThread;
				iThreadInstance.period = iThread.periodOnMSTA;
				iThreadInstance.isAllocated = false;
				iThreadInstance.instanceNumber = i;
				iThreadInstance.dispatchMSTAIndex = 0; //to be calculated
				iThreadInstance.frozenMSTAIndex = 0; //to be calculated
				iThreadInstance.startMSTAIndex = 0; //to be calculated
				iThreadInstance.completeMSTAIndex = 0; //to be calculated
				iThreadInstance.outputMSTAIndex = 0; //to be calculated
				//iThreadInstance.deadlineMSTAIndex = 0; //set now
				if ((iThread.periodOnMSTA!=0)&&(iThread.deadlineOnMSTA!=0)){
					iThreadInstance.deadlineMSTAIndex = iThread.periodOnMSTA*(i) + iThread.deadlineOnMSTA;
				} else {
					iThreadInstance.deadlineMSTAIndex = 0;
				}
				newThreadInstanceList.add(iThreadInstance);
			}
			iThread.threadInstanceList = newThreadInstanceList;
			newThreadList.add(iThread);
		}
	}
	
	public void InitializeLMSTA(){
		for (int i =0; i < hyperPeriod; i++ ){
			lMSTA.flagAllocatedList.add(false);
			AL_EventList alEventList = new AL_EventList();
			lMSTA.eventList.add(alEventList);
			AL_ComputationList alComputationList = new AL_ComputationList();
			lMSTA.computationListIndex.add(alComputationList);
		}
	}
	
	public int getInstantNumberOnMSTA(int absTime){
		//TODO need to consider the time unit
		int i = absTime/minumumInterval;
		return i;
	}
	
	public boolean checkExistence(Object object, String className, String funcName){
		if (object==null){
			showDebugInfo(isDebugging, className + " does not exist in " + funcName + ", exit!");
			return false;
		}
		return true;
	}
	
//	enum SchedulingPolicy {
//		SP_EDF,
//		SP_RM,
//		SP_OTHER
//	}
	public void showDebugInfo(boolean bShow, String str){
		if (bShow) System.err.println("DI - MA: " + str);		
	}
	public void showProgressInfo(boolean bShow, String str){
		if (bShow) System.out.println("PI - MA: " + str);
	}
	
	public void ShowSchedulingResultInConsole(){
		for (int i=0; i<iProcess.threadLists.size();i++){
			I_Thread showThread = iProcess.threadLists.get(i);
			//System.out.println(showThread.name);
			for (int j=0; j<showThread.threadInstanceList.size();j++){
				I_ThreadInstance showThreadInstance = showThread.threadInstanceList.get(j);
				System.out.print(showThreadInstance.getThreadInstanceName());
				System.out.print(" -D-" + trimShowStr(showThreadInstance.dispatchMSTAIndex));
				System.out.print(" -F-" + trimShowStr(showThreadInstance.frozenMSTAIndex));
				System.out.print(" -S-" + trimShowStr(showThreadInstance.startMSTAIndex));
				System.out.print(" -C-" + trimShowStr(showThreadInstance.completeMSTAIndex));
				System.out.print(" -F-" + trimShowStr(showThreadInstance.outputMSTAIndex));
				System.out.println(" -D-" + trimShowStr(showThreadInstance.deadlineMSTAIndex));
			}
		}
	}
	
	public void ShowIndexedSchedulingResultInConsole(){
		System.out.println("---------Show results with traces-------------");
		I_Thread currentThread;
		I_ThreadInstance currentThreadInstance;
		char[] tChar = new char[hyperPeriod+1];
		
		tChar = initChar(tChar);
		System.out.print(trimString(" ", 30) + "   |");
		for (int g=0; g<(tChar.length/5);g++){
			tChar[g*5] = '|';
		}
		System.out.println(tChar);
		
		tChar = initChar(tChar);
		
		for (int i=0;i<iProcess.threadLists.size();i++){
			currentThread = iProcess.threadLists.get(i);
//			for (int k=0;k<hyperPeriod;k++){
//				str += " ";
//			}
			
			for (int j=0;j<currentThread.threadInstanceList.size();j++){
				currentThreadInstance = currentThread.threadInstanceList.get(j);
				if (tChar[currentThreadInstance.dispatchMSTAIndex] ==' '){
					tChar[currentThreadInstance.dispatchMSTAIndex]='d';
				} else {
					//deadline = dispatch
					tChar[currentThreadInstance.dispatchMSTAIndex]='D';
				}
				
				if (currentThreadInstance.dispatchMSTAIndex < currentThreadInstance.frozenMSTAIndex){
					tChar[currentThreadInstance.frozenMSTAIndex]='i';	
				} else if (currentThreadInstance.dispatchMSTAIndex == currentThreadInstance.frozenMSTAIndex){
					//dispatch = input frozen
					tChar[currentThreadInstance.frozenMSTAIndex]='I';
				}
				
				if (currentThreadInstance.frozenMSTAIndex == currentThreadInstance.startMSTAIndex ){
					//input frozen = start
					tChar[currentThreadInstance.startMSTAIndex]='b';
					if (currentThreadInstance.dispatchMSTAIndex == currentThreadInstance.startMSTAIndex){
						//dispatch = input frozen = start						
						tChar[currentThreadInstance.startMSTAIndex]='B';
					}
				}else{
					tChar[currentThreadInstance.startMSTAIndex]='S';
				}
				if (currentThreadInstance.completeMSTAIndex > (currentThreadInstance.startMSTAIndex + 1)){
					for (int l=(currentThreadInstance.startMSTAIndex + 1); l<currentThreadInstance.completeMSTAIndex;l++){
						tChar[l]='-';
					}
					tChar[currentThreadInstance.completeMSTAIndex]='e';
				}
				tChar[currentThreadInstance.outputMSTAIndex]='o';
				if (currentThreadInstance.completeMSTAIndex == currentThreadInstance.outputMSTAIndex ){
					tChar[currentThreadInstance.completeMSTAIndex]='E';
				}
				tChar[currentThreadInstance.deadlineMSTAIndex]='l';
				if (currentThreadInstance.deadlineMSTAIndex == currentThreadInstance.outputMSTAIndex ){
					tChar[currentThreadInstance.deadlineMSTAIndex]='L';	
				}
			}
			System.out.print(trimString(currentThread.name,30) + "   |");
			System.out.println(tChar);
			tChar = initChar(tChar);
		}
	}
	public String trimString(String str, int length){
		if (str.length()>length){
			return str.substring(0, length);
		} else if (str.length()<length){
			for (int i=0;i<(length-str.length()) ;i++){
			str += " ";	
			}
			return str; 
		} else {
			return str;
		}
	}
	public char[] initChar(char[] tChar){
		for (int k=0;k<tChar.length;k++){
			tChar[k]= ' ';
		}
		return tChar;
	}
	public String trimShowStr(int i){
		String str = new String(Integer.toString(i));
		int k = str.length();
		for (int j=0; j< (4-k); j++){
			str = " " + str;
		}
		return str;
	}
	
	public void CompleteIProcessWithIOTimeSpec(){
		
		int i=0, j=0, k=0, l=0, ioInt=0;
		I_Thread cIThread;
		I_ThreadInstance cIThreadInstance;
		L_Port cLPort;
		//thread list
		for (i=0; i<iProcess.threadLists.size();i++){
			cIThread = iProcess.threadLists.get(i);
			//thread instance list
			for (j=0; j<cIThread.threadInstanceList.size();j++){
				cIThreadInstance = cIThread.threadInstanceList.get(j);
				//in port list
				for (k=0; k<cIThreadInstance.iThread.lThread.inPortList.size();k++){
					AL_Int al_Int = new AL_Int();
					ArrayList<Integer> alInt = new ArrayList<Integer>();
					cIThreadInstance.iPortIOTimeMSTAIndexList.add(al_Int);
					al_Int.alInt = alInt;
					cLPort = cIThreadInstance.iThread.lThread.inPortList.get(k);
					//IN_OUT_Time list
					for (l=0; l<cLPort.ioTimePropList.size();l++){
						ioInt = getInstantNumberOnMSTA(cLPort.ioTimePropList.get(l).io_time);
						switch (cLPort.ioTimePropList.get(l).reference_time) {
			            case RTE_DISPATCH: 
			            	ioInt = cIThreadInstance.dispatchMSTAIndex + ioInt;
			            	break;
			            case RTE_START:
			            	ioInt = cIThreadInstance.startMSTAIndex + ioInt;
			            	break;
			            case RTE_COMPLETE:
			            	ioInt = cIThreadInstance.completeMSTAIndex + ioInt;
			            	break;
			            default:
			            	ioInt = cIThreadInstance.dispatchMSTAIndex + ioInt;
			            	break;
				        }
						alInt.add(ioInt);
					}

				}
				//out port list
				for (k=0; k<cIThreadInstance.iThread.lThread.outPortList.size();k++){	
					AL_Int al_Int = new AL_Int();
					ArrayList<Integer> alInt = new ArrayList<Integer>();
					cIThreadInstance.oPortIOTimeMSTAIndexList.add(al_Int);
					al_Int.alInt = alInt;
					cLPort = cIThreadInstance.iThread.lThread.outPortList.get(k);
					//IN_OUT_Time list
					for (l=0; l<cLPort.ioTimePropList.size();l++){
						ioInt = getInstantNumberOnMSTA(cLPort.ioTimePropList.get(l).io_time);
						switch (cLPort.ioTimePropList.get(l).reference_time) {
			            case RTE_DISPATCH: 
			            	ioInt = cIThreadInstance.dispatchMSTAIndex + ioInt;
			            	break;
			            case RTE_START:
			            	ioInt = cIThreadInstance.startMSTAIndex + ioInt;
			            	break;
			            case RTE_COMPLETE:
			            	ioInt = cIThreadInstance.completeMSTAIndex + ioInt;
			            	break;
			            default:
			            	ioInt = cIThreadInstance.completeMSTAIndex + ioInt;
			            	break;
				        }
						alInt.add(ioInt);
					}
				}				
			}
		}
	}
	
	
	enum SpecLanguage {
		AADL,
		SIMULINK,
		GENEAUTO,
		PERIODIC_SIGNAL
	}
	
	
	public I_Process performScheduling(L_Process lprocess1) {
		//
		Initialize();
		
		//
		GetProcess(lprocess1);
		
		//
		SetSchedulingPolicy();
		
		//
		hyperPeriod = CalculateHyperPeriod();
		
		//
		minumumInterval = CalculateMinumumInterval();
		
		//initialize the I_Process
		InitializeIProcess(shiftedDispatch, shiftedStart, ioEventOverlap);
		
		InitializeLMSTA();
		
		//EDF_Scheduling edf_Scheduling= new EDF_Scheduling(lMSTA, lProcess);
		
		//The main algo to do the scheduling
		MainSchedulingAlgorithm(schedulingPolicy);
		
		//add all input and output ports io frozen events
		CompleteIProcessWithIOTimeSpec();
		
		//Show scheduling result for every thread instance
		ShowSchedulingResultInConsole();
		
		//Show scheduling result for every event 
		ShowIndexedSchedulingResultInConsole();
		
		//export the resulting to text
		ExportSchedule2Text(lMSTA);
				
		//export the resulting to VCD file format
		ExportSchedule2VCD(lMSTA);
				
		
		//construct Signal affine clocks
		MapToSignalAffineClocks(lMSTA, iProcess);
		
		//export the affine clocks to process behavior in ASME2SSME
		ExportASME2SSME(lAffineClocks);
		
		
		showProgressInfo(bShowProcess, " SCHEDULING COMPLETES!");
		
		return iProcess;
		
	}
	
	
     public void GetProcess(L_Process lprocess1){
		
		lProcess = lprocess1;
	}
	
}


