package fr.inria.tea.aadl2ssme.scheduling.actions;

import java.util.ArrayList;
import java.util.Iterator;

import fr.inria.tea.aadl2ssme.scheduling.idstruct.*;

public class TimeAllocation {

	L_MSTA lMSTA;
	I_ThreadInstance iTI; 
	I_ThreadInstance iFirstTI;

	boolean isPeriodSpecified;
	boolean isInputTimeSpecified;
	boolean isOutputTimeSpecified;
	boolean isComputationTimeSpecified;
	
	//allocation parameters
	boolean isIOEventOverlap;
	boolean isShiftedDispatch;
	boolean isShiftedStart;
	
	boolean isDispatchAllocated = false;
	boolean isFrozenAllocated = false;
	boolean isStartAllocated = false;
	boolean isCompleteAllocated = false;
	boolean isOutputTimeAllocated = false;
	boolean bOneAllocForN = true;
	private boolean isDebugging = false;
	private boolean bShowProcess = false;
	private boolean isSchedulingShow = false;

	/**
	 * The constructor.
	 */
	public TimeAllocation() {
	}
	
	public TimeAllocation(I_ThreadInstance iThreadInstance, L_MSTA msta, boolean bool) {
		showProgressInfo(bShowProcess, "---START---Thread instance: " + iThreadInstance.getThreadInstanceName());
		iTI = iThreadInstance;
		lMSTA = msta;
		bOneAllocForN = bool;
		setPropertySpecified();
	}
	
	public I_ThreadInstance calPeriodicOtherInstances(I_ThreadInstance iTI, L_MSTA msta){
		
		return null;
		
	}
	
//	public I_ThreadInstance calPeriodicThreadInstance(I_ThreadInstance iTI, L_MSTA msta, int n){
//		I_Thread iThread = iTI.iThread;
//		I_ThreadInstance resITI = new I_ThreadInstance();
//		if (iThread.isAllocated) {
//			System.out.println("Current thread instance is already allocated:" + iThread.name);	    
//			return null;
//		}
//		isIOEventOverlap = iThread.ioEventOverlap;
//		isShiftedDispatch = iThread.shiftedDispatch;
//		isShiftedStart = iThread.shiftedStart;
//		resITI.dispatchMSTAIndex = allocFrozenEvent(n);
//		
//		return resITI;
//	}
	
	public boolean calPeriodicThreadInstance(I_ThreadInstance iTI, L_MSTA msta, boolean bOneAllocForN){	
		if ((bOneAllocForN)&&(iTI.instanceNumber==0)){
			return calPeriodicThreadAllInstances(iTI, msta);
		} else {
			//return calPeriodicThreadSingleInstance(iTI, msta);
			return true;
		}
	}
	
//	public boolean calPeriodicThreadSingleInstance(I_ThreadInstance iTI, L_MSTA msta){
//		//calcualte for the n-th instance of the thread
//		//by default all the instances of a thread is calculated when the fist one is calculated
//		I_Thread iThread = iTI.iThread;
//		//I_ThreadInstance resITI = new I_ThreadInstance();
//		if (iThread.isAllocated) {
//			showDebugInfo(isDebugging, "Current thread instance is already allocated:" + iThread.name);
//			return false;
//		}
//		int index = 0;
//		isIOEventOverlap = iThread.ioEventOverlap;
//		isShiftedDispatch = iThread.shiftedDispatch;
//		isShiftedStart = iThread.shiftedStart;
//		
//		//index = allocFrozenEvent(iTI.instanceNumber);
//		//showSchedulingInfo(isSchedulingShow, " allocFrozenEvent ", index);
//		showProgressInfo(bShowProcess, iTI.getThreadInstanceName());
//		
//		//index = allocDispatchEvent(iTI.instanceNumber);
//		showSchedulingInfo(isSchedulingShow, " DISPATCH ", index);
//		
//		index = allocFrozenEvent(iTI.instanceNumber, true);
//		showSchedulingInfo(isSchedulingShow, " IN-FROZEN ", index);
//		
//		index = allocStartEvent(iTI.instanceNumber);
//		showSchedulingInfo(isSchedulingShow, " START ", index);
//		
//		index = allocCompleteEvent(iTI.instanceNumber);
//		showSchedulingInfo(isSchedulingShow, " COMPLETE ", index);
//		
//		index = allocFrozenEvent(iTI.instanceNumber, false);
//		showSchedulingInfo(isSchedulingShow, " OUT-FROZEN ", index);
//		
//		allocDeadlineEvent();
//		System.out.println("");
//		//showSchedulingInfo(isSchedulingShow, " allocFrozenEvent ", index);
//		return true;
//	}	
	
	public boolean calPeriodicThreadAllInstances(I_ThreadInstance iTI, L_MSTA msta){
		//calcualte for the n-th instance of the thread
		//by default all the instances of a thread is calculated when the fist one is calculated
		I_Thread iThread = iTI.iThread;
		//I_ThreadInstance resITI = new I_ThreadInstance();
		if (iThread.isAllocated) {
			showDebugInfo(isDebugging, "Current thread instance is already allocated:" + iThread.name);	    
			return false;
		}
		
		int firstIndex = 0;
		int instanceNumber = iTI.iThread.periodNumber;
		isIOEventOverlap = iThread.ioEventOverlap;
		isShiftedDispatch = iThread.shiftedDispatch;
		isShiftedStart = iThread.shiftedStart;
		
		for (int i=0; i<instanceNumber; i++){
			showProgressInfo(bShowProcess, iTI.iThread.threadInstanceList.get(i).getThreadInstanceName());
			firstIndex = allocDispatchEvent(iTI.iThread, i);
			showSchedulingInfo(isSchedulingShow, " DISPATCH ", firstIndex);			
		}

		for (int i=0; i<instanceNumber; i++){
			firstIndex = allocFrozenEvent(iTI.iThread, i, true);
			showSchedulingInfo(isSchedulingShow, " IN-FROZEN ", firstIndex);
		}
		
		for (int i=0; i<instanceNumber; i++){
			firstIndex = allocStartEvent(i);
			showSchedulingInfo(isSchedulingShow, " START ", firstIndex);
		}
		for (int i=0; i<instanceNumber; i++){
			firstIndex = allocCompleteEvent(i);
			showSchedulingInfo(isSchedulingShow, " COMPLETE ", firstIndex);
		}
		for (int i=0; i<instanceNumber; i++){
			firstIndex = allocFrozenEvent(iTI.iThread, i, false);
			showSchedulingInfo(isSchedulingShow, " OUT-FROZEN ", firstIndex);
		}
		for (int i=0; i<instanceNumber; i++){
			allocDeadlineEvent(i);
			showSchedulingInfo(isSchedulingShow, " DEADLINE ", iTI.iThread.threadInstanceList.get(i).deadlineMSTAIndex);
		}
		return true;
	}
	
	public int allocDispatchEvent(I_Thread iThread, int n){
		int index  = n * iTI.period;
		if (!isShiftedDispatch){
			// Regular dispatch
			// LOOP ALLOCATE {dispatch} at [0 + period*n: 0 =< n < TI_n]
			isDispatchAllocated = true;
			allocDispatchEventInEventList(n, index);
			iThread.threadInstanceList.get(n).dispatchMSTAIndex = index;
			
			return index;
		} else { 
			isDispatchAllocated = false;
			return 0;
		}
	}
	
	public void allocDispatchEventInEventList(int n, int index){
		I_Event iEvent = new I_Event();
		iEvent.eventType = EventType.ET_DISPATCH;
		if ((n>=0)&& (n<iTI.iThread.threadInstanceList.size())){
			iEvent.iTI = iTI.iThread.threadInstanceList.get(n);
		} else {
			showDebugInfo(isDebugging, "Dispatch event could not find its  " + iTI.iThread.threadInstanceList.get(n).getThreadInstanceName());
		}
		if ((index>=0)&& (index<lMSTA.eventList.size())){		
			AL_EventList alEL = lMSTA.eventList.get(index);
			alEL.lEvent_AL.add(iEvent);
		} else {
			showDebugInfo(isDebugging, "Dispatch event index on MSTA is not valid - " + iTI.iThread.threadInstanceList.get(n).getThreadInstanceName());
		}
		
	}
	
//	public int allocFrozenEvent(int n){
//		showProgressInfo(bShowProcess, "Thread instance: " + iTI.getThreadInstanceName() + " ---@ allocFrozenEvent");
//		IO_TimeSpec ioTimeSpec = GetLastIOTimeProp(iTI.iThread, true);
//		int indexFrozenOnMSTA = getIndexFromIOTimeSpec(ioTimeSpec);;
//		int instantNumberOnMSTA = getInstanceNumberOfIOTimeSpec(ioTimeSpec);
//		if (isDispatchAllocated) {
//			indexFrozenOnMSTA = iTI.period * (n-1) + indexFrozenOnMSTA;
//			if (isIOEventOverlap){
//				return indexFrozenOnMSTA;
//			} else { // ! isIOEventOverlap
//				//VERIFY the interval [{dispatch}*(n-1), {dispatch}*(n-1)+input_time] (0 =< n < TI_n) is allocated
//				boolean bIntAllocated = lMSTA.checkIntervalAllocated(iTI.period * (n-1), indexFrozenOnMSTA);
//				if (bIntAllocated) {
//					//TODO
//					//NeedToRollOver();
//					return -1;
//				} else {
//					lMSTA.setIntervalOccupied(iTI.period * (n-1), indexFrozenOnMSTA);
//					//TODO
//					//lMSTA.setEvent();
//					//iTI.frozenMSTAIndex = indexOnMSTA;
//				}
//			}
//		} else { // ! isDispatchAllocated 
//			// if the dispatch is not allocated, i.e., the dispatch is not regular and will be allocated according to -> 
//			// the last input_time of the thread instance
//			int indexNewDispatch = 0;
//			if (isIOEventOverlap){
//				if (instantNumberOnMSTA > lMSTA.currentPoint){ // input time is later than current point
//					//dispatch at 0, frozen at instantNumberOnMSTA (for first thread instance)
//					indexNewDispatch = 0 + iTI.period*(n-1);
//					indexFrozenOnMSTA = indexNewDispatch + instantNumberOnMSTA;
//				} else {  // input time is no later than current point
//					indexNewDispatch = lMSTA.currentPoint - instantNumberOnMSTA + iTI.period*(n-1);
//					indexFrozenOnMSTA = indexNewDispatch + instantNumberOnMSTA;
//				}
//				return indexFrozenOnMSTA;
//			} else { // IO_Time Event Overlap is not allowed and dispatch is not regular
//				//boolean bIntAllocated = lMSTA.checkIntervalAllocated(iTI.period * (n-1), indexFrozenOnMSTA);
//				indexNewDispatch = iTI.period*(n-1) + lMSTA.currentPoint;
//				indexFrozenOnMSTA = indexNewDispatch + instantNumberOnMSTA;
//
//				//TODO need to check for all instances of the same thread...
//				boolean bCheckIntAllocated = checkIntervalAllocForAllInstance(iTI, lMSTA.currentPoint, instantNumberOnMSTA);
//				if (bCheckIntAllocated) return -1; //Some of needed interval(s) (for all the instances) has been allocated.
//				
//				boolean bIntAllocated = lMSTA.checkIntervalAllocated(indexNewDispatch, indexFrozenOnMSTA);
//				
//				if (bIntAllocated) {
//					//TODO
//					//NeedToRollOver();
//					return -1;
//				} else {
//					lMSTA.setIntervalOccupied(iTI.period * (n-1), indexFrozenOnMSTA);
//					//TODO
//					//lMSTA.setEvent();
//					//iTI.frozenMSTAIndex = indexOnMSTA;
//				}				
//				return indexFrozenOnMSTA;
//			}
//		}
//		return indexFrozenOnMSTA;		
//	}
	
	public int allocFrozenEvent(I_Thread iThread, int n, boolean isInput){
		showProgressInfo(bShowProcess, "Thread instance: " + iTI.iThread.threadInstanceList.get(n).getThreadInstanceName() + " ---@ allocFrozenEvent");
		if (!isInput){//only for output_time allocation
			if (isFrozenAllocated) {
				showDebugInfo(isDebugging, "Frozen is allocated but dispatch has not been allocated! - " + iTI.iThread.threadInstanceList.get(n).getThreadInstanceName());
				//System.err.println("");
				return -1;
			}
		}
		IO_TimeSpec ioTimeSpec = GetLastIOTimeProp(iTI.iThread, isInput);
		int indexFrozenOnMSTA = getIndexFromIOTimeSpec(ioTimeSpec);
		int instantNumberOnMSTA = getInstanceNumberOfIOTimeSpec(ioTimeSpec);
		int indexNewDispatch = 0;
		if (isInput){
			indexNewDispatch = iTI.period * n;
		} else {
			indexNewDispatch = iThread.threadInstanceList.get(n).completeMSTAIndex;
			if (indexNewDispatch<1){
				showDebugInfo(isDebugging, "Complete not set for out frozen! - " + iThread.threadInstanceList.get(n).getThreadInstanceName());
			}
		}
		if (isDispatchAllocated) {
			indexFrozenOnMSTA = indexNewDispatch + indexFrozenOnMSTA;
			if (isIOEventOverlap){
				//TODO alloc event list
				if (isInput){
					iTI.iThread.threadInstanceList.get(n).frozenMSTAIndex =  indexFrozenOnMSTA;	
				} else {
					iTI.iThread.threadInstanceList.get(n).outputMSTAIndex =  indexFrozenOnMSTA;
				}
				
				return indexFrozenOnMSTA;
			} else { // ! isIOEventOverlap
				//VERIFY the interval [{dispatch}*(n-1), {dispatch}*(n-1)+input_time] (0 =< n < TI_n) is allocated
				boolean bIntAllocated = lMSTA.checkIntervalAllocated(indexNewDispatch, indexFrozenOnMSTA);
				if (bIntAllocated) {
					//TODO
					//NeedToRollOver();
					showDebugInfo(isDebugging, "{Frozen} Interval allocated! [" + Integer.toString(indexNewDispatch) +"," + Integer.toString(indexFrozenOnMSTA) +"] - " + iTI.iThread.threadInstanceList.get(n).getThreadInstanceName());
					return -1;
				} else {
					lMSTA.setIntervalOccupied(indexNewDispatch, indexFrozenOnMSTA);
					if (isInput){
						iTI.iThread.threadInstanceList.get(n).frozenMSTAIndex =  indexFrozenOnMSTA;	
					} else {
						iTI.iThread.threadInstanceList.get(n).outputMSTAIndex =  indexFrozenOnMSTA;
					}
					//iTI.iThread.threadInstanceList.get(n).frozenMSTAIndex = indexFrozenOnMSTA;
					if (n==0){
						lMSTA.currentPoint = indexFrozenOnMSTA;
					}
					//TODO
					//lMSTA.setEvent();
					//iTI.frozenMSTAIndex = indexOnMSTA;
				}
			}
		} else { // ! isDispatchAllocated ***This part is only for input***
			// if the dispatch is not allocated, i.e., the dispatch is not regular and will be allocated according to -> 
			// the last input_time of the thread instance
			if (isIOEventOverlap){
				if (instantNumberOnMSTA > lMSTA.currentPoint){ // input time is later than current point
					//dispatch at 0, frozen at instantNumberOnMSTA (for first thread instance)
					//indexNewDispatch = 0 + iTI.period*n;
					indexFrozenOnMSTA = indexNewDispatch + instantNumberOnMSTA;
				} else {  // input time is no later than current point
					indexNewDispatch = lMSTA.currentPoint - instantNumberOnMSTA + iTI.period*n;
					indexFrozenOnMSTA = indexNewDispatch + instantNumberOnMSTA;
				}
				return indexFrozenOnMSTA;
			} else { // IO_Time Event Overlap is not allowed and dispatch is not regular
				//boolean bIntAllocated = lMSTA.checkIntervalAllocated(iTI.period * (n-1), indexFrozenOnMSTA);
				indexNewDispatch = iTI.period*n + lMSTA.currentPoint;
				indexFrozenOnMSTA = indexNewDispatch + instantNumberOnMSTA;

				//TODO need to check for all instances of the same thread...
				boolean bCheckIntAllocated = checkIntervalAllocForAllInstance(iTI, lMSTA.currentPoint, instantNumberOnMSTA);
				if (bCheckIntAllocated) return -1; //Some of needed interval(s) (for all the instances) has been allocated.
				
				boolean bIntAllocated = lMSTA.checkIntervalAllocated(indexNewDispatch, indexFrozenOnMSTA);
				
				if (bIntAllocated) {
					//TODO
					//NeedToRollOver();
					return -1;
				} else {
					iTI.dispatchMSTAIndex = indexNewDispatch;
					if (isInput){
						iTI.iThread.threadInstanceList.get(n).frozenMSTAIndex =  indexFrozenOnMSTA;	
					} else {
						iTI.iThread.threadInstanceList.get(n).outputMSTAIndex =  indexFrozenOnMSTA;
					}
					//iTI.frozenMSTAIndex = indexFrozenOnMSTA;

					lMSTA.setIntervalOccupied(indexNewDispatch, indexFrozenOnMSTA);

					//TODO
					//lMSTA.setEvent();
					//iTI.frozenMSTAIndex = indexOnMSTA;
				}				
				return indexFrozenOnMSTA;
			}
		}
		return indexFrozenOnMSTA;		
	}
	public IO_TimeSpec GetLastIOTimeProp(I_Thread iThread, boolean isInput){
		//get the last input time
		IO_TimeSpec ioTimeSpec= new IO_TimeSpec();
		ioTimeSpec.io_time = 0;
		IO_TimeSpec pIOTimeSpec;
		ArrayList<L_Port> allIOPortInThreadList;
		if (isInput){
			allIOPortInThreadList = iThread.lThread.inPortList;
		} else {
			allIOPortInThreadList = iThread.lThread.outPortList;
		}
		Iterator<L_Port> itAIPIT = allIOPortInThreadList.iterator();
    	while(itAIPIT.hasNext()){
    		L_Port lPort = (L_Port) itAIPIT.next();
    		pIOTimeSpec = lPort.GetLastIOTimeProp();
    		if (pIOTimeSpec.io_time > ioTimeSpec.io_time){
    			ioTimeSpec.io_time = pIOTimeSpec.io_time;
    			ioTimeSpec.reference_time = pIOTimeSpec.reference_time;
    			ioTimeSpec.timeUnit = pIOTimeSpec.timeUnit;
    		}    		
    	}
		return	ioTimeSpec;
	}
	public int getIndexFromIOTimeSpec(IO_TimeSpec ioTimeSpec){
		// get the index of MSTA from an IO_TimeSpec
		//TODO in the next version, different time unit will be considered
		int indexOnMSTA = ioTimeSpec.io_time;
		
		return indexOnMSTA;
	}
	public int getInstanceNumberOfIOTimeSpec(IO_TimeSpec ioTimeSpec){
		// get the index number of MSTA from an IO_TimeSpec
		//TODO
		int indexOnMSTA = ioTimeSpec.io_time;
		
		return indexOnMSTA;
	}

	public boolean checkIntervalAllocForAllInstance(I_ThreadInstance iTI, int start, int instantNumber){
		// get the index number of MSTA from an IO_TimeSpec
		//TODO
		int iCheckDispatch = 0;
		int iCheckFrozenOnMSTA = 0;
		boolean bCheckIntAllocated = false;
		for (int k = 1; k <= iTI.iThread.periodNumber; k++){
			iCheckDispatch = iTI.period*(k-1) + start;
			iCheckFrozenOnMSTA = iCheckDispatch + instantNumber;
			bCheckIntAllocated = bCheckIntAllocated || lMSTA.checkIntervalAllocated(iCheckDispatch, iCheckFrozenOnMSTA);
		}		
		return bCheckIntAllocated;
	}
	
	public ArrayList<L_Port> GetAllPortsFromThread(I_Thread iThread){
		return iThread.lThread.inPortList;
	}
	
	public int allocStartEvent(int n){
		showProgressInfo(bShowProcess, "Thread instance: " + iTI.iThread.threadInstanceList.get(n).getThreadInstanceName() + " ---@ allocStartEvent");
		
//		IO_TimeSpec ioTimeSpec = GetLastInputTimeProp(iTI.iThread);
//		int indexFrozenOnMSTA = getIndexFromIOTimeSpec(ioTimeSpec);;
//		int instantNumberOnMSTA = getInstanceNumberOfIOTimeSpec(ioTimeSpec);
		
		int computationTimeOnMSTA = getComputationTimeOnMSTA(iTI);
		int lastestIndexFirst = 0;		
		if (isDispatchAllocated) { //regular dispatch
//			indexFrozenOnMSTA = iTI.period * (n-1) + indexFrozenOnMSTA;
			if (isShiftedStart){ //if shift start is allowed
				//allocate next vide intervals for every period
				if (n==0){
					for (int k=0; k < iTI.iThread.periodNumber; k++){
						I_ThreadInstance currrentThreadInstance = iTI.iThread.threadInstanceList.get(k);
						int i = getVideIntervalForTI(currrentThreadInstance);
						if (i >= 0) {
							//TODO verify deadline
							if (currrentThreadInstance.startMSTAIndex<1) {
								currrentThreadInstance.startMSTAIndex = i;
								currrentThreadInstance.completeMSTAIndex = i + iTI.iThread.computationTimeOnMSTA;
								if (currrentThreadInstance.completeMSTAIndex>currrentThreadInstance.deadlineMSTAIndex){
									showDebugInfo(isDebugging, "Allocation failed for start-complete/deadline - P2: " + currrentThreadInstance.getThreadInstanceName());
									return -1;
								}
								lMSTA.setIntervalOccupied(currrentThreadInstance.startMSTAIndex, currrentThreadInstance.completeMSTAIndex);
							}
						} else {
							showDebugInfo(isDebugging, "Allocation failed for start-complete - P1: " + currrentThreadInstance.getThreadInstanceName());
							return -1;
						}
					}
//					if (iTI.startMSTAIndex<1) {
//						iTI.startMSTAIndex = lMSTA.currentPoint;
//						iTI.completeMSTAIndex = lMSTA.currentPoint + iTI.iThread.computationTimeOnMSTA;
//						
//					}
					lMSTA.currentPoint = ((I_ThreadInstance) iTI.iThread.threadInstanceList.get(0)).completeMSTAIndex;
					//TODO allocate start and complete
					//TODO: verify deadline
					return iTI.iThread.threadInstanceList.get(n).startMSTAIndex; // for the first instance
				} else { 
					if (iTI.iThread.threadInstanceList.get(n).startMSTAIndex<1) {
						showDebugInfo(isDebugging, "Allocation failed for start-complete - P2: " + iTI.iThread.threadInstanceList.get(n).getThreadInstanceName());
					} else {
						return iTI.iThread.threadInstanceList.get(n).startMSTAIndex;
					}
					//showDebugInfo(isDebugging, "Allocation failed for start-complete - P2: " + iTI.iThread.threadInstanceList.get(n).getThreadInstanceName());
				}
			} else { //if shift start is not allowed, find the lastest VideIntervalAtKthPeriod for allocation
				if (n==0){
					//get the lastest index available for allocation in each period
					int lastestIndex = 0;
					int lastestIndexTemp = 0;

					//TODO verify deadline
					for (int k=0; k < iTI.iThread.periodNumber; k++){
						I_ThreadInstance currrentThreadInstance = iTI.iThread.threadInstanceList.get(k);
						lastestIndexTemp = getVideIntervalForTI(currrentThreadInstance);
						if (lastestIndexTemp > lastestIndex){
							lastestIndex = lastestIndexTemp;
						}
						lMSTA.setIntervalOccupied(lastestIndex, lastestIndex+iTI.iThread.computationTimeOnMSTA);
						if (k==0){
							lastestIndexFirst = lastestIndex;
						}
						currrentThreadInstance.startMSTAIndex = lastestIndex;
						currrentThreadInstance.completeMSTAIndex = lastestIndex+iTI.iThread.computationTimeOnMSTA;
						if (currrentThreadInstance.completeMSTAIndex>currrentThreadInstance.deadlineMSTAIndex){
							showDebugInfo(isDebugging, "Alloc -START/Deadline- failed for " + currrentThreadInstance.getThreadInstanceName());
							return -1;
						}
						//TODO alloc event list
					}
					
					lMSTA.currentPoint = lastestIndexFirst;
					//TODO: verify deadline
					
					return lastestIndexFirst; // for the first instance
				} else {
					if (iTI.iThread.threadInstanceList.get(n).startMSTAIndex < 0){
						showDebugInfo(isDebugging, "Alloc -START- failed for " + iTI.iThread.threadInstanceList.get(n).getThreadInstanceName());
						return -1;
					}
					return iTI.iThread.threadInstanceList.get(n).startMSTAIndex;
				}
				//TODO 
				//lMSTA.setIntervalOccupied(n, n+n);
			}
		} else { // Dispatch is not allocated 
			if (isFrozenAllocated) {
				System.err.println("Allocation failed because frozen event is allocated but dispatch has not been allocated!");
				return -1;
			}
			
			if (isShiftedStart){ //if shifted start is allowed, dispatch not allocated, frozen not allocated (Input_Time is not specified)
				//allocate next vide intervals for every period
				if (n==0){
					for (int k=0; k < iTI.iThread.periodNumber; k++){
						I_ThreadInstance currrentThreadInstance = iTI.iThread.threadInstanceList.get(k);
						int i = getVideIntervalForTI(currrentThreadInstance);
						if (i >= 0) {
							if (currrentThreadInstance.startMSTAIndex<1) {
								currrentThreadInstance.startMSTAIndex = i;
								currrentThreadInstance.completeMSTAIndex = i + iTI.iThread.computationTimeOnMSTA;
								if (currrentThreadInstance.completeMSTAIndex > currrentThreadInstance.deadlineMSTAIndex){
									showDebugInfo(isDebugging, "Allocation failed for start-complete/deadline - P4 :" + currrentThreadInstance.getThreadInstanceName());
									return -1;
								}
								lMSTA.setIntervalOccupied(currrentThreadInstance.startMSTAIndex, currrentThreadInstance.completeMSTAIndex);
							}
						} else {
							showDebugInfo(isDebugging, "Allocation failed for start-complete - P4 :" + currrentThreadInstance.getThreadInstanceName());
							return -1;
						}
					}
					if (iTI.startMSTAIndex<1) {
						iTI.startMSTAIndex = lMSTA.currentPoint;
						iTI.completeMSTAIndex = lMSTA.currentPoint + iTI.iThread.computationTimeOnMSTA;
					}
					//TODO allocate start and complete
					//TODO: verify deadline
					return iTI.iThread.threadInstanceList.get(n).startMSTAIndex; // for the first instance
				} else { 
					if (iTI.iThread.threadInstanceList.get(n).startMSTAIndex <1){
						showDebugInfo(isDebugging, "Allocation failed for start-complete - P5 :" + iTI.iThread.threadInstanceList.get(n).getThreadInstanceName());
						return -1;
					}
					return iTI.iThread.threadInstanceList.get(n).startMSTAIndex;
				}
			} else { //if shift start is not allowed, find the lastest VideIntervalAtKthPeriod for allocation
				if (n==0){
					//get the lastest index available for allocation in each period
					int lastestIndex = 0;
					int lastestIndexTemp = 0;
					for (int k=0; k < iTI.iThread.periodNumber; k++){
						lastestIndexTemp = getVideIntervalForTI(iTI.iThread.threadInstanceList.get(k));
						if (lastestIndexTemp > lastestIndex){
							lastestIndex = lastestIndexTemp;
						}
//						boolean bCheckIntervalAlloc = lMSTA.checkIntervalAllocated(iTI.period*k + lastestIndex, computationTimeOnMSTA);
//						if (bCheckIntervalAlloc){
//							showDebugInfo(isDebugging, "Allocation failed for start-complete - P6 :" + iTI.getThreadInstanceName());
//							return -1;
//						}
						lMSTA.setIntervalOccupied(lastestIndex, lastestIndex+iTI.iThread.computationTimeOnMSTA);
						iTI.iThread.threadInstanceList.get(k).startMSTAIndex =  lastestIndex;
						iTI.iThread.threadInstanceList.get(k).completeMSTAIndex = lastestIndex+iTI.iThread.computationTimeOnMSTA;
						//Allocate event to event list
						if (k==0) {
							lMSTA.currentPoint = lastestIndex+iTI.iThread.computationTimeOnMSTA;
						}
					}
					//TODO: verify deadline
					return iTI.iThread.threadInstanceList.get(n).startMSTAIndex; // for the first instance
				} else {
					if (iTI.iThread.threadInstanceList.get(n).startMSTAIndex < 0){
						showDebugInfo(isDebugging, "Alloc -START- failed for " + iTI.iThread.threadInstanceList.get(n).getThreadInstanceName());
						return -1;
					}
				}
				//TODO 
				return iTI.iThread.threadInstanceList.get(n).startMSTAIndex;
			}
			
		}
		return lMSTA.currentPoint;//TODO			
	}
	public int getVideIntervalForTI(I_ThreadInstance iThreadInstance){
		//I_ThreadInstance iThreadInstance = iTI.iThread.threadInstanceList.get(k);
		int i1s = -1;
		if (iThreadInstance.dispatchMSTAIndex>i1s) i1s= iThreadInstance.dispatchMSTAIndex;
		if (iThreadInstance.frozenMSTAIndex>i1s) i1s = iThreadInstance.frozenMSTAIndex;
		int i1e = 0;
		if (iThreadInstance.completeMSTAIndex > iThreadInstance.dispatchMSTAIndex){
			i1e = iThreadInstance.completeMSTAIndex;
		}else {
			i1e = iThreadInstance.period + iThreadInstance.dispatchMSTAIndex;
		}
		return lMSTA.checkIntervalAllocInInterval(i1s, i1e, iThreadInstance.iThread.computationTimeOnMSTA);
		
	}
	
//	public int getVideIntervalForCurrentInstance(){
//		//Search a vide interval on MSTA for current thread instance
//		int i1s = getEarliestStartForCurrentInstance();
//		int i1e = 0;
//		if (iTI.completeMSTAIndex<1){
//			i1e = iTI.period + iTI.dispatchMSTAIndex;
//		}else{
//			i1e = iTI.completeMSTAIndex;
//		}
//		return lMSTA.checkIntervalAllocInInterval(i1s, i1e, iTI.iThread.computationTimeOnMSTA);
//	}
//	public int getEarliestStartForCurrentInstance(){
//		//get the earlist start index on MSTA for current thread instance
//		int i = -1;
//		if (iTI.dispatchMSTAIndex>i) i = iTI.dispatchMSTAIndex;
//		if (iTI.frozenMSTAIndex>i) i = iTI.frozenMSTAIndex;
//		return i;
//	}
//	public int getLastestVideIntervalAtKthPeriod(int k){
//		//TODO: add this function to L_MSTA class
//		return k;
//	}
	
	public int getComputationTimeOnMSTA(I_ThreadInstance iTI){
		int computationTimeOnMSTA = iTI.iThread.computationTimeOnMSTA;
		return computationTimeOnMSTA;
	}
	
	public int allocCompleteEvent(int n){
		showProgressInfo(bShowProcess, "Thread instance: " + iTI.iThread.threadInstanceList.get(n).getThreadInstanceName() + " ---@ allocCompleteEvent");
		return iTI.iThread.threadInstanceList.get(n).completeMSTAIndex;
	}
	public int allocOutputTimeEvent(int n){
		showProgressInfo(bShowProcess, "Thread instance: " + iTI.iThread.threadInstanceList.get(n).getThreadInstanceName() + " ---@ allocOutputTimeEvent");
		return this.allocFrozenEvent(iTI.iThread, n, false);
	}
	public boolean verifyDeadline(int n){
		return false;
	}
	public boolean allocDeadlineEvent(int n){
		showProgressInfo(bShowProcess, "Thread instance: " + iTI.iThread.threadInstanceList.get(n).getThreadInstanceName() + " ---@ allocDeadlineEvent");
		int deadlineIndexOnMSTA = 0;
		//int lastestIndexTemp = 0;
		ArrayList<I_ThreadInstance> threadInstanceList;
		for (int k=0; k < iTI.instanceNumber; k++){
			threadInstanceList = iTI.iThread.threadInstanceList;
			((I_ThreadInstance) threadInstanceList.get(k)).deadlineMSTAIndex = deadlineIndexOnMSTA + iTI.period * k;
		}
		return true;
	}
	
	public void setPropertySpecified(){
		if (iTI.iThread.lThread.computationTime != 0) isComputationTimeSpecified = true;
		if (iTI.iThread.lThread.period != 0) isPeriodSpecified = true; 
		isInputTimeSpecified = checkIOTimePropertySpecified(true);
		isOutputTimeSpecified = checkIOTimePropertySpecified(false);	 
	}
	
	public boolean checkIOTimePropertySpecified(boolean isInput){
		ArrayList<L_Port> ioPortList = iTI.iThread.lThread.inPortList;
		if (isInput) {
			ioPortList = iTI.iThread.lThread.inPortList;
		} else {
			ioPortList = iTI.iThread.lThread.outPortList;
		}
		 
		L_Port lPort;
		ArrayList<IO_TimeSpec> ioTimeSpecList;
		IO_TimeSpec ioTimeSpec;
		Iterator<L_Port> itIPL = ioPortList.iterator();
    	while(itIPL.hasNext()){
    		lPort = (L_Port) itIPL.next();
    		ioTimeSpecList = lPort.ioTimePropList;
    		Iterator<IO_TimeSpec> itIOTSL = ioTimeSpecList.iterator();
        	while(itIOTSL.hasNext()){
        		ioTimeSpec = (IO_TimeSpec) itIOTSL.next();
        		if (ioTimeSpec.io_time > 0)  return true;
        	}
    	}
        return false;
	}
	public void showDebugInfo(boolean bShow, String str){
		if (bShow) System.err.println("DI - TA: " + str);		
	}
	public void showProgressInfo(boolean bShow, String str){
		if (bShow) System.out.println("PI - TA: " + str);
	}
	public void showSchedulingInfo(boolean bShow, String str, int index){
		if (bShow) System.out.println(str + " @ " + Integer.toString(index));
	}
}
