 
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.io.*;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.xtext.resource.XtextResource;
import org.eclipse.xtext.ui.editor.XtextEditor;
import org.eclipse.xtext.util.concurrent.IUnitOfWork;
import org.osate.aadl2.*;
import org.eclipse.pop.ssme.impl.*;

public class ASME2SSME {
	private Shell shell;
	
	private IPath fileLocation;
	public static final String ASME2SSME_Version="v1.0";

	/**
	 * Class constructor
	 */
	public ASME2SSME() {
	}

	/**
	 * Prints the string str in the console
	 * 
	 * @param str
	 *            a string to be printed out
	 */
	public void Trace(String str) {
		// java.lang.System.out.println(str);
	}

	/**
	 * Writes an out file (aFile)
	 * 
	 * @param aFile
	 * @param aContents
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public void WriteFile(File aFile, String aContents)
			throws FileNotFoundException, IOException {

		Trace("\n ******** ASME2SSME: WriteFile");
		if (aFile == null) throw new IllegalArgumentException("File should not be null.");
		if (!aFile.exists()) throw new FileNotFoundException("File does not exist: " + aFile);
		if (!aFile.isFile()) throw new IllegalArgumentException("Should not be a directory: " + aFile);
		if (!aFile.canWrite()) throw new IllegalArgumentException("File cannot be written: " + aFile);
		// use buffering
		Writer output = new BufferedWriter(new FileWriter(aFile));
		try {
			// FileWriter always assumes default encoding is OK!
			output.write(aContents);
		} finally {
			output.close();
		}
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void startTransformation(IFile aadlFile) throws IOException {

		Trace("\n ******** ASME2SSME: start Transformation ********");

		//   IFile  InputaadlFile = aadlFile;
		IPath fileLocation1 = aadlFile.getLocation();
		fileLocation = fileLocation1;

		/************** test for aadl file transformation **********************************/
		IWorkbench wb = PlatformUI.getWorkbench();
		IWorkbenchWindow win = wb.getActiveWorkbenchWindow();
		IWorkbenchPage page = win.getActivePage();
		// IWorkbenchPart part = page.getActivePart();
		IEditorPart activeEditor = page.getActiveEditor();
		if (activeEditor == null) return;
		XtextEditor xtextEditor = (XtextEditor) activeEditor.getAdapter(XtextEditor.class);
		/*  LB ---
		 * final ISelection selection; if (xtextEditor != null) { if (part
		 * instanceof ContentOutline) selection = ((ContentOutline)
		 * part).getSelection(); else selection = (ITextSelection)
		 * xtextEditor.getSelectionProvider().getSelection(); }
		 */

		/**************************************************
		 * This piece of code is partially copied from OSATE:
		 * org.osate.xtext.aadl2.ui.handlers.execute():
		 * xtextEditor.getDocument().readOnly(), to get resource from xtext. The
		 * file must be opened in the editor of Aadl2, and a ´Clean project´
		 * should be performed to assure that the properties are connected to
		 * their definitions.
		 **************************************************/
		// I do not understand very well this code, but it works !!
		xtextEditor.getDocument().readOnly(
				new IUnitOfWork<EObject, XtextResource>() {
					public EObject exec(XtextResource resource)
							throws Exception {
						Trace("\n ******** Get resource ********");
						// Resolve references such that HREFs use symbolic
						EcoreUtil.resolveAll(resource);
						Trace("\n ******** Resolve resource ********");

						if (resource.getContents().size() > 1) {
							MessageDialog.openInformation(shell,
									"AADL2SSME transformation test Plug-in",
									"More than one root in the model.");
							return null;
						}

						/****************************************************************/
						Reousurce_Handler(resource, fileLocation);
						/****************************************************************/
						return null;
					}
				});
	}

	/**
	 * handler resource, start the transformation
	 * 
	 * @param resource
	 * @param fileLocation
	 */
	public void Reousurce_Handler(Resource resource, IPath fileLocation) {
		EList<EObject> content = resource.getContents();
		if (!content.isEmpty()) {

			MessageDialog.openInformation(shell, "AADL(v2)2SSME transformation test Plug-in",
					"Start the transformation from AADLv2 (ASME) to SSME.");

			Trace("\n ***************************************************************");
			Trace(" ****** Start the transformation from AADLv2 (ASME) to SSME ****** ");
			Trace(" *************************************************************** \n");

			for (int j = 0; j < content.size(); j++) {
				EObject obj = content.get(j);

				if (obj instanceof AadlPackage) {
					AadlPackage aadlRootModel = null;
					aadlRootModel = (AadlPackage) obj;

					Trace("\n --------------------------------------------------");
					Trace(" ------- Start the transformation of AadlPackage -- ");

					ASME2SSME_Translate_Spec asme2ssme_aadlspec = new ASME2SSME_Translate_Spec();
					ListModuleImpl RootListModule = asme2ssme_aadlspec.ASME2SSME_AadlSpec_package(aadlRootModel);

					Trace("\n --------------------------------------------------");
					Trace(" ------- End transformation of AadlPackage -- ");
					save(RootListModule, fileLocation );
					
				} else if (obj instanceof PropertySet) {
					PropertySet aadlRootModel = null;
					aadlRootModel = (PropertySet) obj;

					Trace("\n ***************************************************");
					Trace(" ****** Start the transformation of PropertySet ****** ");

					ASME2SSME_Translate_Spec asme2ssme_aadlspec = new ASME2SSME_Translate_Spec();
					ListModuleImpl RootListModule = asme2ssme_aadlspec.ASME2SSME_AadlSpec_PropertySet(aadlRootModel);

					Trace("\n **************************************************");
					Trace(" ****** End transformation of PropertySet ****** ");
					save(RootListModule, fileLocation );
				} 
				else {
					MessageDialog.openInformation(shell, "AADL2SSME transformation test Plug-in", "AadlSpec has not been gotten.");
					return;
				}
			}

			Trace("\n *********************************************************");
			Trace(" ****** End transformation from AADL (ASME) to SSME ****** ");
			Trace(" ********************************************************* \n");
		}
	}

	private void save(ListModuleImpl rootListModule, IPath fileLocation2) {
		try {
			IPath fileWithoutExtension = fileLocation.removeFileExtension();
			SSMEUtils.save((EObject) rootListModule, fileWithoutExtension.toOSString() + ".ssme");
		} catch (IOException e) {
			e.printStackTrace();
		} 
	}
	 
}
