 
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;

import org.osate.aadl2.*;
import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;

import fr.inria.tea.aadl2ssme.scheduling.actions.*;
import fr.inria.tea.aadl2ssme.scheduling.idstruct.*;

 
public class ASME2SSME_Scheduling extends ASME2SSME_Translate_Name {	
	
	private AADLCoLAPI aadlColAPI;
	private signalTreeAPI myLib;
	private ASME2SSME_Translate_Common asme2ssme_common;

	public ASME2SSME_Scheduling() {
		  aadlColAPI = new AADLCoLAPI();
    	  myLib= new signalTreeAPI();
    	  asme2ssme_common = new ASME2SSME_Translate_Common();
	}
	

	
    /**
     * Scheduling the threads of a processor
     * 
     * @param subp
     * @param p_behavior
     * @param sysimpl
     */
    public void ASME2SSME_Processor_Scheduling(ProcessorSubcomponent subp, 
    		ProcessModelImpl p_behavior, SystemImplementation sysimpl){
    	
    	Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Processor_Scheduling : " + subp.getName());
    	//get the processes which are executed on this processor
    	ArrayList<ProcessSubcomponent> list1 = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subp);
    	for(int i =0; i< list1.size(); i++){
    		ProcessSubcomponent sub1 = list1.get(i);
    		ASME2SSME_Process_Scheduling(subp, p_behavior, sysimpl, sub1);
    	}
    }
    
    
    /**
     * Scheduling a subprocess: 
     * set LProcess, 
     * perform scheduling, 
     * get scheduling result IProcess, 
     * relocate the result to affine clocks
     * 
     * @param subp the processor that this process is executed on
     * @param p_behavior the behavior subprocess of processor
     * @param sysimpl system implementation
     * @param sub1 process
     */
   private void ASME2SSME_Process_Scheduling(ProcessorSubcomponent subp, 
    		ProcessModelImpl p_behavior, SystemImplementation sysimpl, ProcessSubcomponent sub1){
    	
    	Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Process_Scheduling : " + sub1.getName());
    	Scheduling scheduling1 = new Scheduling();

    	L_Process lprocess = ASME2SSME_Set_LProcess(sub1);
    	I_Process iprocess1 = scheduling1.performScheduling(lprocess);
    	   	
    	//create a Signal process that schedules subprocess sub1
    	ProcessModelImpl pscheduler1 = ASME2SSME_Create_ProcessScheduler(subp, p_behavior, sysimpl, sub1, iprocess1);
    	myLib.STree_addProcessDeclaration(p_behavior, pscheduler1);
      	
    	//make an instance of pscheduler1: (sub1_t1_ctl1, sub1_t1_time1, ...) := Scheduling_sub1(top)
    	SignalsDefinitionImpl eq1 = ASME2SSME_Create_ProcessScheduler_Instance(subp, p_behavior, sub1);
    	myLib.STree_addExpressionOfProcessToaModel(p_behavior, eq1);  
    }
    
    
    /**
     * Set L_Process struct (input of the scheduler)
     * 
     * @param sub1 process subcomponent
     * @return L_Process
     */
   private L_Process ASME2SSME_Set_LProcess(ProcessSubcomponent sub1){
    	
    	Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Set_LProcess : " + sub1.getName());
    	
    	L_Process lProcess = new L_Process();
    	
    	ProcessImplementation p1 = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(sub1);
    	
    	//=============process basic information
    	lProcess.processName = aadlColAPI.Subcomponent_GetName(sub1);
		lProcess.processInstanceNameInAADL = aadlColAPI.Subcomponent_GetName(sub1);
		lProcess.processImplementationNameInAADL = aadlColAPI.ComponentImpl_GetName(p1);
		
		//=== threads information
		//ArrayList<L_Thread> lThreadList = new ArrayList<L_Thread>();
		ArrayList<ThreadSubcomponent> subt_list1 = aadlColAPI.Process_GetSubcomponentsThread(p1);
		for(int i=0; i<subt_list1.size(); i++){
			//each thread infromation
			ThreadSubcomponent subt1 = subt_list1.get(i);					
			L_Thread lThread1 = ASME2SSME_Set_LThread(subt1);						
			//lThreadList.add(lThread1);
			//lProcess.Get_lThreads().add(lThread1);
			lProcess.threadLists.add(lThread1);
		}
		
		//=== thread dependency
		ASME2SSME_Set_PortsDependency(sub1, lProcess);
		
		return lProcess;
    }
    
    
    
    
    /**
     * Set LThread struct
     * 
     * @param subt1 thread subcomponent
     * @return
     */
   private L_Thread ASME2SSME_Set_LThread(ThreadSubcomponent subt1){
    	
        Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Set_LThread : " + subt1.getName());
    	
    	L_Thread lThread1 = new  L_Thread();
    	
    	ThreadImplementation t1 = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(subt1);
		// ThreadType tt1 = (ThreadType) aadlColAPI.ComponentImpl_GetComponentType(t1);
		
		lThread1.threadName = aadlColAPI.Subcomponent_GetName(subt1);
		lThread1.threadInstanceNameInAADL = aadlColAPI.Subcomponent_GetName(subt1);
		lThread1.threadImplimentationInAADL = aadlColAPI.ComponentImpl_GetName(t1);
			  	    
  		//default value 
		lThread1.isPeriodic = true;
		lThread1.isAperiodic = false;
		lThread1.isSporadic = false;
		lThread1.isHybrid = false;
		lThread1.isTimed = false;
		lThread1.isBackground = false;

		lThread1.period = 20;
		lThread1.periodTimeUnit = TimeUnit.SECOND;
		lThread1.deadline = 20;
		lThread1.deadlineTimeUnit = TimeUnit.SECOND;
		lThread1.computationTime = 4;
		lThread1.computationTimeUnit = TimeUnit.SECOND;
		
		//periodic only
		String dispatchp = aadlColAPI.Thread_Get_DispatchProtocol_str(t1);      	  		
  		if(dispatchp.equals("Periodic")){
  			lThread1.isPeriodic = true;
  			
  			IntegerLiteral period = aadlColAPI.Thread_GetPeriod(t1);
  			long periodValue = period.getValue();
  			String periodUnit = aadlColAPI.IntegerLiteral_Get_Unit(period);
  			
  			//========================= period
  			lThread1.period = (int) periodValue;
			lThread1.periodTimeUnit = ASME2SSME_TimeUnit(periodUnit);
			
			IntegerLiteral deadline = aadlColAPI.Thread_GetDeadline(t1);
			if(deadline == null){
				deadline = period;
			}
			long deadlineValue = deadline.getValue();
  			String deadlineUnit = aadlColAPI.IntegerLiteral_Get_Unit(deadline);
  			
			//========================= deadline
			lThread1.deadline = (int) deadlineValue;
			lThread1.deadlineTimeUnit = ASME2SSME_TimeUnit(deadlineUnit);
			
			/*   == TODO == get execution time*/
			//========================== computation
			lThread1.computationTime = 3;
			lThread1.computationTimeUnit = TimeUnit.SECOND; 
			//==========================
			
  		}
  		else{
  			Trace("\n == thread : " + t1.getName() + " dispatch : " + dispatchp);
  			java.lang.System.err.print("\n non periodic thread scheduling not implemented ");
			throw new IllegalArgumentException("non periodic thread scheduling not implemented \n");
  		}
		
  		//======= in ports information
  		ArrayList<L_Port> inPortList1 = ASME2SSME_Set_inPortList(subt1, lThread1);		
		lThread1.inPortList = inPortList1;
		
		//======= out ports information
		ArrayList<L_Port> outPortList1 = ASME2SSME_Set_outPortList(subt1, lThread1);		
		lThread1.outPortList = outPortList1;
		
		return lThread1;
    }
    
    
    /**
     * Set in port list L_Port struct of a thread
     * 
     * @param subt1 thread subcomponent
     * @param lThread1 L_Thread struct of subt1
     * 
     * @return
     */
    private ArrayList<L_Port> ASME2SSME_Set_inPortList(ThreadSubcomponent subt1, L_Thread lThread1){
    	
        Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Set_inPortList : " + subt1.getName());
    	ArrayList<L_Port> inPortList1 = new ArrayList<L_Port>();
    	
    	ThreadImplementation t1 = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(subt1);
		ThreadType tt1 = (ThreadType) aadlColAPI.ComponentImpl_GetComponentType(t1);
    	
		//in data ports
		ArrayList<DataPort> inDPorts= aadlColAPI.Thread_GetInDataPort(tt1);	
		DataPort inDPort1;
		for(int j=0; j< inDPorts.size(); j++){
			inDPort1 = inDPorts.get(j);
			L_Port lPort1 = new L_Port();
			
			String reftime1 = aadlColAPI.Port_Get_InputTime_Time1_str(t1, inDPort1);
			String maxUnit1 = aadlColAPI.Port_Get_InputTime_Offset1_MaxUnit_str(t1, inDPort1);
			long maxValue1 = aadlColAPI.Port_Get_InputTime_Offset1_MaxValue_long(t1, inDPort1);
//			String minUnit = aadlColAPI.Port_Get_InputTime_Offset1_MinUnit_str(t1, inDPort1);
//			long minValue = aadlColAPI.Port_Get_InputTime_Offset1_MinValue_long(t1, inDPort1);
			
			//===========================
			lPort1.portType = PortType.DATA;
			lPort1.portName = aadlColAPI.Port_GetName(inDPort1);
			lPort1.portNameInAADL = aadlColAPI.Port_GetName(inDPort1);
			lPort1.dataPortType = aadlColAPI.DataPort_GetClassifier(inDPort1).getName() ;
			
			lPort1.disThreadList.add(lThread1);
			lPort1.srcThread = null;
			
			IO_TimeSpec ioTimeSpec1 = new IO_TimeSpec();			
			ioTimeSpec1.io_time = (int) maxValue1;
			ioTimeSpec1.reference_time = ASME2SSME_ReferenceTime(reftime1);
			ioTimeSpec1.timeUnit = ASME2SSME_TimeUnit(maxUnit1);
			
//			ioTimeSpec1.io_time = 1;
//			ioTimeSpec1.reference_time = ReferenceTimeEnum.RTE_DISPATCH;
//			ioTimeSpec1.timeUnit = TimeUnit.SECOND;
			
			lPort1.ioTimePropList.add(ioTimeSpec1);
			lPort1.portDirection = PortDirection.INPUT;
			//===========================
			
			inPortList1.add(lPort1);
		}
		
		//in event ports
		ArrayList<EventPort> inEPorts= aadlColAPI.Thread_GetInEventPort(tt1);	
		L_Port lPort2;
		for(int k=0; k< inEPorts.size(); k++){
			EventPort inEPort1 = inEPorts.get(k);
			lPort2 = new L_Port();
			
			String reftime2 = aadlColAPI.Port_Get_InputTime_Time1_str(t1, inEPort1);
			String maxUnit2 = aadlColAPI.Port_Get_InputTime_Offset1_MaxUnit_str(t1, inEPort1);
			long maxValue2 = aadlColAPI.Port_Get_InputTime_Offset1_MaxValue_long(t1, inEPort1);
			
			//===========================
			lPort2.portType = PortType.EVENT;
			lPort2.portName = aadlColAPI.Port_GetName(inEPort1);
			lPort2.portNameInAADL = aadlColAPI.Port_GetName(inEPort1);
			lPort2.dataPortType = "EVENT" ;
			
			lPort2.disThreadList.add(lThread1);
			lPort2.srcThread = null;
			
			IO_TimeSpec ioTimeSpec1 = new IO_TimeSpec();			
			ioTimeSpec1.io_time = (int) maxValue2;
			ioTimeSpec1.reference_time = ASME2SSME_ReferenceTime(reftime2);
			ioTimeSpec1.timeUnit = ASME2SSME_TimeUnit(maxUnit2);
			
//			ioTimeSpec1.io_time = 1;
//			ioTimeSpec1.reference_time = ReferenceTimeEnum.RTE_DISPATCH;
//			ioTimeSpec1.timeUnit = TimeUnit.SECOND;
			
			lPort2.ioTimePropList.add(ioTimeSpec1);
			lPort2.portDirection = PortDirection.INPUT;
			//===========================
			inPortList1.add(lPort2);
		}
		
		//in event data ports
		ArrayList<EventDataPort> inEDPorts= aadlColAPI.Thread_GetInEventDataPort(tt1);
		EventDataPort inEDPort1;
		for(int m=0; m< inEDPorts.size(); m++){
			inEDPort1 = inEDPorts.get(m);
			L_Port lPort3 = new L_Port();
			
			String reftime3 = aadlColAPI.Port_Get_InputTime_Time1_str(t1, inEDPort1);
			String maxUnit3 = aadlColAPI.Port_Get_InputTime_Offset1_MaxUnit_str(t1, inEDPort1);
			long maxValue3 = aadlColAPI.Port_Get_InputTime_Offset1_MaxValue_long(t1, inEDPort1);
			
			//===========================
			lPort3.portType = PortType.EVENT_DATA;
			lPort3.portName = aadlColAPI.Port_GetName(inEDPort1);
			lPort3.portNameInAADL = aadlColAPI.Port_GetName(inEDPort1);
			lPort3.dataPortType = aadlColAPI.EventDataPort_GetClassifier(inEDPort1).getName() ;
			
			lPort3.disThreadList.add(lThread1);
			lPort3.srcThread = null;
			
			IO_TimeSpec ioTimeSpec1 = new IO_TimeSpec();		
			ioTimeSpec1.io_time = (int) maxValue3;
			ioTimeSpec1.reference_time = ASME2SSME_ReferenceTime(reftime3);
			ioTimeSpec1.timeUnit = ASME2SSME_TimeUnit(maxUnit3);
			
//			ioTimeSpec1.io_time = 1;
//			ioTimeSpec1.reference_time = ReferenceTimeEnum.RTE_DISPATCH;
//			ioTimeSpec1.timeUnit = TimeUnit.SECOND;
			
			lPort3.ioTimePropList.add(ioTimeSpec1);
			lPort3.portDirection = PortDirection.INPUT;
			//===========================
			inPortList1.add(lPort3);
		}
		
		return inPortList1;
    }
    
    
    
    
    /**
     * Set out port list L_Port struct of thread subcomponent
     * 
     * @param subt1 thread subcomponent
     * @param lThread1 L_Thread struct of subt1
     * @return
     */
    private ArrayList<L_Port> ASME2SSME_Set_outPortList (ThreadSubcomponent subt1, L_Thread lThread1){
    	
        Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Set_outPortList : " + subt1.getName());
    	
    	ArrayList<L_Port> outPortList1 = new ArrayList<L_Port>();
    	
    	ThreadImplementation t1 = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(subt1);
		ThreadType tt1 = (ThreadType) aadlColAPI.ComponentImpl_GetComponentType(t1);
    	  	
		// out data ports
		ArrayList<DataPort> outDPorts = aadlColAPI.Thread_GetOutDataPort(tt1);
		L_Port lPort1;
		for(int j1=0; j1< outDPorts.size(); j1 ++){
			DataPort outDPort1 = outDPorts.get(j1);			
			lPort1 = new L_Port();
			
			String reftime1 = aadlColAPI.Port_Get_OutputTime_Time1_str(t1, outDPort1);
			String maxUnit1 = aadlColAPI.Port_Get_OutputTime_Offset1_MaxUnit_str(t1, outDPort1);
			long maxValue1 = aadlColAPI.Port_Get_OutputTime_Offset1_MaxValue_long(t1, outDPort1);
			
			//================================
			lPort1.portType = PortType.DATA;
			lPort1.portName = aadlColAPI.Port_GetName(outDPort1);
			lPort1.portNameInAADL = aadlColAPI.Port_GetName(outDPort1);
			lPort1.dataPortType = aadlColAPI.DataPort_GetClassifier(outDPort1).getName() ;
			
			//lPort2.disThreadList.add(lThread1);
			lPort1.srcThread = lThread1;
			
			IO_TimeSpec ioTimeSpec1 = new IO_TimeSpec();
			ioTimeSpec1.io_time = (int) maxValue1;
			ioTimeSpec1.reference_time = ASME2SSME_ReferenceTime(reftime1);
			ioTimeSpec1.timeUnit = ASME2SSME_TimeUnit(maxUnit1);
			
//			ioTimeSpec1.io_time = 2;
//			ioTimeSpec1.reference_time = ReferenceTimeEnum.RTE_COMPLETE;
//			ioTimeSpec1.timeUnit = TimeUnit.SECOND;
			
			lPort1.ioTimePropList.add(ioTimeSpec1);
			lPort1.portDirection = PortDirection.OUTPUT;
			//==================================
			outPortList1.add(lPort1);
		}
		
		// out event ports
		ArrayList<EventPort> outEPorts = aadlColAPI.Thread_GetOutEventPort(tt1);
		L_Port lPort2;
		for(int k1=0; k1< outEPorts.size(); k1 ++){
			EventPort outEPort1 = outEPorts.get(k1);			
			lPort2 = new L_Port();
			
			String reftime2 = aadlColAPI.Port_Get_OutputTime_Time1_str(t1, outEPort1);
			String maxUnit2 = aadlColAPI.Port_Get_OutputTime_Offset1_MaxUnit_str(t1, outEPort1);
			long maxValue2 = aadlColAPI.Port_Get_OutputTime_Offset1_MaxValue_long(t1, outEPort1);
			
			//================================
			lPort2.portType = PortType.EVENT;
			lPort2.portName = aadlColAPI.Port_GetName(outEPort1);
			lPort2.portNameInAADL = aadlColAPI.Port_GetName(outEPort1);
			lPort2.dataPortType = "EVENT";
			
			//lPort2.disThreadList.add(lThread1);
			lPort2.srcThread = lThread1;
			
			IO_TimeSpec ioTimeSpec2 = new IO_TimeSpec();
			ioTimeSpec2.io_time = (int) maxValue2;
			ioTimeSpec2.reference_time = ASME2SSME_ReferenceTime(reftime2);
			ioTimeSpec2.timeUnit = ASME2SSME_TimeUnit(maxUnit2);
			
//			ioTimeSpec2.io_time = 2;
//			ioTimeSpec2.reference_time = ReferenceTimeEnum.RTE_COMPLETE;
//			ioTimeSpec2.timeUnit = TimeUnit.SECOND;
			
			lPort2.ioTimePropList.add(ioTimeSpec2);
			lPort2.portDirection = PortDirection.OUTPUT;
			//==================================
			outPortList1.add(lPort2);
		}
		
		// out event data ports
		ArrayList<EventDataPort> outEDPorts = aadlColAPI.Thread_GetOutEventDataPort(tt1);
		L_Port lPort3;
		for(int m1=0; m1< outEDPorts.size(); m1 ++){
			EventDataPort outEDPort1 = outEDPorts.get(m1);			
			lPort3 = new L_Port();
			
			String reftime3 = aadlColAPI.Port_Get_OutputTime_Time1_str(t1, outEDPort1);
			String maxUnit3 = aadlColAPI.Port_Get_OutputTime_Offset1_MaxUnit_str(t1, outEDPort1);
			long maxValue3 = aadlColAPI.Port_Get_OutputTime_Offset1_MaxValue_long(t1, outEDPort1);
			
			//================================
			lPort3.portType = PortType.EVENT_DATA;
			lPort3.portName = aadlColAPI.Port_GetName(outEDPort1);
			lPort3.portNameInAADL = aadlColAPI.Port_GetName(outEDPort1);
			lPort3.dataPortType = aadlColAPI.EventDataPort_GetClassifier(outEDPort1).getName() ;
			
			//lPort2.disThreadList.add(lThread1);
			lPort3.srcThread = lThread1;
			
			IO_TimeSpec ioTimeSpec3 = new IO_TimeSpec();
			ioTimeSpec3.io_time = (int) maxValue3;
			ioTimeSpec3.reference_time = ASME2SSME_ReferenceTime(reftime3);
			ioTimeSpec3.timeUnit = ASME2SSME_TimeUnit(maxUnit3);
			
//			ioTimeSpec3.io_time = 2;
//			ioTimeSpec3.reference_time = ReferenceTimeEnum.RTE_COMPLETE;
//			ioTimeSpec3.timeUnit = TimeUnit.SECOND;
			
			lPort3.ioTimePropList.add(ioTimeSpec3);
			lPort3.portDirection = PortDirection.OUTPUT;
			//==================================
			outPortList1.add(lPort3);
		}
		
		return outPortList1;
    }
    
    
    
    /**
     * Set ports´ srcThread and disThread
     * 
     * @param sub1 process subcomponent
     * @param lProcess1 L_Process struct of sub1
     */
    private void ASME2SSME_Set_PortsDependency(ProcessSubcomponent sub1, L_Process lProcess1){
        
    	Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Set_inPortsrc : " + sub1.getName());
    	
    	//============ get all the port connections of p1
    	ProcessImplementation p1 = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(sub1);
    	ArrayList<PortConnection> DPConnections1 = aadlColAPI.Process_GetDataConnection(p1);
    	ArrayList<PortConnection> EPConnections1 = aadlColAPI.Process_GetEventConnection(p1);
    	ArrayList<PortConnection> EDPConnections1 = aadlColAPI.Process_GetEventDataConnection(p1);
    	
    	ArrayList<PortConnection> pconnections = new ArrayList<PortConnection>();
    	if(DPConnections1 != null){
    		pconnections.addAll(DPConnections1);
    	}
    	if(EPConnections1 != null){
    		pconnections.addAll(EPConnections1);
    	}
    	if(EDPConnections1 != null){
    		pconnections.addAll(EDPConnections1);
    	}
    	
    	for(int i =0; i< pconnections.size(); i++){
    		PortConnection pc1 = pconnections.get(i);
    		
    		Context src_context = aadlColAPI.Connection_GetSrcContext(pc1);
    		Context dst_context = aadlColAPI.Connection_GetDstContext(pc1);
    		Port src_port = (Port) aadlColAPI.PortConnection_GetSrc(pc1);
    		Port dst_port = (Port) aadlColAPI.PortConnection_GetDst(pc1);
    		
    		//============ for each port connection: 
        	if((src_context != null) && (dst_context != null)){
        		if((src_context instanceof ThreadSubcomponent) && (dst_context instanceof ThreadSubcomponent)){
        			
        			ThreadSubcomponent t1 = (ThreadSubcomponent) src_context;
        			ThreadSubcomponent t2 = (ThreadSubcomponent) dst_context;
        			
        			L_Thread lthread1 = ASME2SSME_Get_lThread(lProcess1, sub1, t1);
        			L_Thread lthread2 = ASME2SSME_Get_lThread(lProcess1, sub1, t2);
        			L_Port lport1 = ASME2SSME_Get_lPort(lProcess1, sub1, t1, src_port);
        			L_Port lport2 = ASME2SSME_Get_lPort(lProcess1, sub1, t2, dst_port);
        			
        			if((lthread1 != null) && (lthread2 != null)
        					&& (lport1 != null) && (lport2 !=null)){
        				//====================================
            			lport1.disThreadList.add(lthread2);
            			lport2.srcThread = lthread1;
            			//====================================
        			}       			
        		}
        	}
        	
    	}
    }
    
    
    
    /**
     * Return time unit of a string unit
     * 
     * @param str a string unit
     * @return
     */
    private TimeUnit ASME2SSME_TimeUnit(String str){
    	
    	TimeUnit unit1;
    	if(str.equals("Sec") || str.equals("sec")){
    		unit1 = TimeUnit.SECOND;
    	}
    	else{
    		unit1 = TimeUnit.SECOND;
    	}
    	
    	return unit1;
    	
    }
    
    
    /**
     * Return reference time
     * 
     * @param str a string reference time
     * @return a enum reference time
     */
    private ReferenceTimeEnum ASME2SSME_ReferenceTime(String str){
    	ReferenceTimeEnum ref1;
    	
    	if(str.equals("Dispatch") || str.equals("dispatch")){
    		ref1 = ReferenceTimeEnum.RTE_DISPATCH;
    	}
    	else if(str.equals("Start") || str.equals("start")){
    		ref1 = ReferenceTimeEnum.RTE_START;
    	}
    	else if(str.equals("Complete") || str.equals("complete") 
    			|| str.equals("Completion") || str.equals("completion")){
    		ref1 = ReferenceTimeEnum.RTE_COMPLETE;
    	}
    	else{
    		ref1 = ReferenceTimeEnum.RTE_DISPATCH;
    	}
    	
    	return ref1;
    }
    
	
    /**
     * returns a L_Port object that represents port p1 of thread subt1 in process sub1, 
     * 
     * @param lProcess1 a L_Process object that represents sub1
     * @param sub1 process subcomponent
     * @param t1 thread subcomponent of sub1
     * @param p1 port
     * @return
     */
    private L_Port ASME2SSME_Get_lPort(L_Process lProcess1, ProcessSubcomponent sub1, 
    		                          ThreadSubcomponent t1, Port p1){
       Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Get_lPort : " + p1.getName());
    	String pname = aadlColAPI.Port_GetName(p1);
    	
    	L_Thread lthread1 = ASME2SSME_Get_lThread(lProcess1, sub1, t1);
    	if(lthread1 != null){
    		//=================
    		ArrayList<L_Port> in_lports = lthread1.inPortList;//lthread1.Get_inPorts();
    		ArrayList<L_Port> out_lports = lthread1.outPortList;//lthread1.Get_outPorts();
    		
    		//get all lports of lthread1
    		ArrayList<L_Port> lports = new ArrayList<L_Port>();
    		if(in_lports != null){
    			lports.addAll(in_lports);
    		}
    		if(out_lports != null){
    			lports.addAll(out_lports);
    		}
    		
    		if(lports != null){
    			for(int i =0; i<lports.size(); i++){
        			L_Port lport1 = lports.get(i);
        			String pname1 = lport1.portName;//lport1.Get_PortName();
        			if(pname1 != null){
        				if(pname1.equals(pname)){
        					return lport1;
        				}
        			}
        			
        		}
    		}   		
    	}
    	
    	return null;
    }
    
    
    /**
     * Returns a L_Thread object that represents sub thread t1 in process sub1
     * 
     * @param lProcess1 a L_Process object that represents sub1
     * @param sub1 process subcomponent
     * @param subt1 thread subcomponent
     * @return
     */
    private L_Thread ASME2SSME_Get_lThread(L_Process lProcess1, ProcessSubcomponent sub1, ThreadSubcomponent t1){

        Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Get_lThread : " + t1.getName());
    	
    	String tname = aadlColAPI.Subcomponent_GetName(t1);
    	
    	ArrayList<L_Thread> lthreads = lProcess1.threadLists;//lProcess1.Get_lThreads();
    	for(int i=0; i< lthreads.size(); i++){
    		L_Thread lthread1 = lthreads.get(i);
    		String name1 = lthread1.threadName;//lthread1.Get_threadName();
    		if(name1.equals(tname)){
    			return lthread1;
    		}
    	}
        return null;
     }
    
    
    /**
     * Returns a I_Thread object that represents sub thread t1 in process sub1
     * 
     * @param iProcess1 a I_Process object that represents sub1
     * @param sub1 process subcomponent
     * @param subt1 thread subcomponent
     * @return
     */
    private I_Thread ASME2SSME_Get_iThread(I_Process iProcess1, ProcessSubcomponent sub1, ThreadSubcomponent t1){

        Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Get_iThread : " + t1.getName());
    	
    	String tname = aadlColAPI.Subcomponent_GetName(t1);
    	
    	ArrayList<I_Thread> ithreads = iProcess1.threadLists;//lProcess1.Get_lThreads();
    	for(int i=0; i< ithreads.size(); i++){
    		I_Thread ithread1 = ithreads.get(i);
    		String name1 = ithread1.name;//lthread1.Get_threadName();
    		if(name1.equals(tname)){
    			return ithread1;
    		}
    	}
    	
        return null;
     }
    
    
    /**
     * Create a Signal process that represents the scheduling of a subprocess sub1
     * 
     * @param subp processor that sub1 is executed on
     * @param p_behavior the behavior subprocess of processor
     * @param sysimpl system implementation
     * @param sub1 process subcomponent
     * @return
     */
    private ProcessModelImpl ASME2SSME_Create_ProcessScheduler(ProcessorSubcomponent subp, 
    		ProcessModelImpl p_behavior, SystemImplementation sysimpl, 
    		ProcessSubcomponent sub1, I_Process iprocess1){
    	
    	Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Create_ProcessScheduler : " + sub1.getName());
    	
    	//====================================================
    	//create a Signal process: process Scheduling_sub1(?top ! t1_ctl1, t1_time1, ...)    	
    	String sub1_name = aadlColAPI.Subcomponent_GetName(sub1);
    	ProcessModelImpl pscheduler = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(sub1_name + "_Process_Scheduler") );
    	ASME2SSME_ProcessScheduler_Interface(pscheduler, sub1);
    	ASME2SSME_ProcessScheduler_Body(pscheduler, sub1, iprocess1);
    	//====================================================
    	
    	return pscheduler;
    }
  
    
    /**
     * Create the interface of process scheduler 
     * 
     * @param pscheduler Signal process that represents the process scheduler
     * @param sub1 process subcomponent
     */
    private void ASME2SSME_ProcessScheduler_Interface(ProcessModelImpl pscheduler, ProcessSubcomponent sub1){
    	
    	Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_ProcessScheduler_Interface : " + sub1.getName());
    	
    	// add input signal: event top;
    	asme2ssme_common.ASME2SSME_add_input_to_process_interface("top", "event", pscheduler);	
    	
    	// add output signals
    	ProcessImplementation pimpl = aadlColAPI.Processsubcomponent_GetComponentImpl(sub1);
    	ArrayList<ThreadSubcomponent> subts = aadlColAPI.Process_GetSubcomponentsThread(pimpl);
    	for(int i =0; i< subts.size(); i++){
    		//for each subthread t1, add t1_ctl1, t1_time1
    		ThreadSubcomponent t1 = subts.get(i);
    		ThreadType tt1 = aadlColAPI.Threadsubcomponent_GetComponentType(t1);
    		String tt1_name = aadlColAPI.ComponentType_GetName(tt1);
    		
    		String name1 = aadlColAPI.Subcomponent_GetName(t1); 
			String CTL1_type = Name_of_CTL1_type();
			asme2ssme_common.ASME2SSME_add_output_to_process_interface(name1+ "_ctl1", CTL1_type, pscheduler);	
		    
		    if(aadlColAPI.Thread_HasPorts(tt1)){
				String TIME1_type = Name_of_ThreadTimeBundle(tt1_name);
				asme2ssme_common.ASME2SSME_add_output_to_process_interface(name1+ "_time1", TIME1_type, pscheduler);	
		    }
    	}
    }
    
    
    /**
     * Create the body of subprocess scheduler
     * 
     * @param pscheduler Signal process that represents the process scheduler
     * @param sub1 process subcomponent
     * @param iprocess1 I_Process of sub1
     */
    private void ASME2SSME_ProcessScheduler_Body(ProcessModelImpl pscheduler, ProcessSubcomponent sub1, I_Process iprocess1){
    	
        Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_ProcessScheduler_Body : " + sub1.getName());
    	
    	ProcessImplementation pimpl = aadlColAPI.Processsubcomponent_GetComponentImpl(sub1);
    	ArrayList<ThreadSubcomponent> subts = aadlColAPI.Process_GetSubcomponentsThread(pimpl);
    	for(int i =0; i< subts.size(); i++){
    		ThreadSubcomponent t1 = subts.get(i);
    		I_Thread ithread1 = ASME2SSME_Get_iThread(iprocess1, sub1, t1);
    		ASME2SSME_Thread_Scheduling(pscheduler, t1, ithread1);
    	}
    }	
    
    
    /**
     * Scheduling a subthread t1:
     * create a Signal process that represents the scheduler of thread, 
     * add an instance into the body the process scheduler pscheduler
     * 
     * @param pscheduler Signal process that represents the process scheduler
     * @param t1 thread subcomponent
     * @param ithread1 I_Thread of t1
     */
    private void ASME2SSME_Thread_Scheduling(ProcessModelImpl pscheduler, 
    		ThreadSubcomponent t1, I_Thread ithread1){
    	
    	Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Thread_Scheduling : " + t1.getName());
    	//create a Signal process that schedules subthread t1
    	ProcessModelImpl tscheduler1 = ASME2SSME_Create_ThreadScheduler(t1, ithread1);
    	myLib.STree_addProcessDeclaration(pscheduler, tscheduler1);
      	
    	//make an instance of tscheduler1: (t1_ctl1, t1_time1) := t1_Thread_Scheduler(top)
    	SignalsDefinitionImpl eq1 = ASME2SSME_Create_ThreadScheduler_Instance(pscheduler, t1);
    	myLib.STree_addExpressionOfProcessToaModel(pscheduler, eq1);  
    }
    
    
    /**
     * Create a Signal process that represents the scheduling of a subprocess sub1
     * 
     * @param t1 thread subcomponent
     * @param ithread1 I_Thread of t1
     * @return
     */
    private ProcessModelImpl ASME2SSME_Create_ThreadScheduler(ThreadSubcomponent t1, I_Thread ithread1){
    	
    	Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Create_ThreadScheduler : " + t1.getName());
    	//create a Signal process: process Scheduling_t1(?top ! ctl1, time1)    	
    	String t1_name = aadlColAPI.Subcomponent_GetName(t1);
    	ProcessModelImpl tscheduler = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(t1_name + "_Thread_Scheduler") );
    	ASME2SSME_ThreadScheduler_Interface(tscheduler, t1);
    	ASME2SSME_ThreadScheduler_Body(tscheduler, t1, ithread1);
    	return tscheduler;
    }
    
    
    /**
     * Create the interface of thread scheduler: 
     * (? event top; ! CTL1 ctl1; x_TIME_EVENT time1;)
     * 
     * @param tscheduler Signal process that represents the thread scheduler
     * @param t1 thread subcomponent
     */
    private void ASME2SSME_ThreadScheduler_Interface(ProcessModelImpl tscheduler, ThreadSubcomponent t1){
    	
    	Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_ThreadScheduler_Interface : " + t1.getName());
    	// add input signal: event top;
    	asme2ssme_common.ASME2SSME_add_input_to_process_interface("top", "event", tscheduler);	
    	
    	// add output signals
        ThreadType tt1 = aadlColAPI.Threadsubcomponent_GetComponentType(t1);
    	String tt1_name = aadlColAPI.ComponentType_GetName(tt1);
    		
    	//String name1 = aadlColAPI.Subcomponent_GetName(t1); 
		String CTL1_type = Name_of_CTL1_type();
		asme2ssme_common.ASME2SSME_add_output_to_process_interface("ctl1", CTL1_type, tscheduler);	
		    
		if(aadlColAPI.Thread_HasPorts(tt1)){
			String TIME1_type = Name_of_ThreadTimeBundle(tt1_name);
			asme2ssme_common.ASME2SSME_add_output_to_process_interface("time1", TIME1_type, tscheduler);	
		}
    }
    
    
    /**
     * Create the body of subthread scheduler
     * 
     * @param pscheduler Signal process that represent scheduler of thread t1
     * @param t1 thread subcomponent
     * @param ithread1 I_Thread of t1
     */
    private void ASME2SSME_ThreadScheduler_Body(ProcessModelImpl pscheduler, 
    		ThreadSubcomponent t1, I_Thread ithread1){
    	
        Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_ThreadScheduler_Body : " + t1.getName());
    	
    		
    	
    	ArrayList<I_ThreadInstance> ithreadinstances = ithread1.threadInstanceList;
    	//get the first ithreadinstance for computation 
    	//(suppose all the instances are scheduled in the same way)
    	if(ithreadinstances != null){
    		I_ThreadInstance ithreadinstance1 = ithreadinstances.get(0);
    		int period = ithreadinstance1.period;
    		int dispatch = ithreadinstance1.dispatchMSTAIndex;
    		int deadline = ithreadinstance1.deadlineMSTAIndex;
    		int start = ithreadinstance1.startMSTAIndex;
    		// String name1 = ithreadinstance1.iThread.name;
    		
        	//==============================
    		String ctl1_name = "ctl1"; // t1_ctl1
    		
        	//t1_ctl1.Dispatch := affine{dispatch, period}(top)    		
	  		FieldAccessImpl dispatch1 = myLib.STree_mkFieldAccess(ctl1_name, "Dispatch"); 
	  		SignalsDefinitionImpl eq1 = ASME2SSME_Create_affine_equation(dispatch, period, dispatch1);
	  		myLib.STree_addExpressionOfProcessToaModel(pscheduler, eq1);   
    		
        	//t1_ctl1.Resume := affine{start, period}(top)
	  		FieldAccessImpl start1 = myLib.STree_mkFieldAccess(ctl1_name, "Resume"); 
	  		SignalsDefinitionImpl eq2 = ASME2SSME_Create_affine_equation(start, period, start1);
	  		myLib.STree_addExpressionOfProcessToaModel(pscheduler, eq2);
	  		
        	//t1_ctl1.Deadline := affine{deadline, period}(top)
	  		FieldAccessImpl deadline1 = myLib.STree_mkFieldAccess(ctl1_name, "Deadline"); 
	  		SignalsDefinitionImpl eq3 = ASME2SSME_Create_affine_equation(deadline, period, deadline1);
	  		myLib.STree_addExpressionOfProcessToaModel(pscheduler, eq3);  
	  		
        	//t1_ctl1.Initialize := affine{dispatch, period}(top)
	  		FieldAccessImpl initialize1 = myLib.STree_mkFieldAccess(ctl1_name, "Initialize"); 
	  		SignalsDefinitionImpl eq4 = ASME2SSME_Create_affine_equation(dispatch, period, initialize1);
	  		myLib.STree_addExpressionOfProcessToaModel(pscheduler, eq4);  
	  		
	  		String time1_name = "time1"; // t1_ctl1	
	  		ArrayList<L_Port> inlports = ithread1.lThread.inPortList;
	  		ArrayList<AL_Int> iportios = ithreadinstance1.iPortIOTimeMSTAIndexList;
	  			  		
        	//t1_time1.x_Frozen_time_event
	  		for(int i=0; i<inlports.size(); i++ ){
	  			String strPortName1 = inlports.get(i).portName;
				if (strPortName1 == null) {
					strPortName1 = Integer.toString(i);
				} 

				AL_Int iportio = iportios.get(i);
				ArrayList<Integer> ints = iportio.alInt;
				if(ints != null){
					Integer in1 = ints.get(0);
					
					FieldAccessImpl inp1 = myLib.STree_mkFieldAccess(time1_name, strPortName1+"_Frozen_time_event"); 
			  		SignalsDefinitionImpl eq5 = ASME2SSME_Create_affine_equation(in1, period, inp1);
			  		myLib.STree_addExpressionOfProcessToaModel(pscheduler, eq5);  
				}
	  		}
	  		
	  		ArrayList<L_Port> outlports = ithread1.lThread.outPortList;
	  		ArrayList<AL_Int> oportios = ithreadinstance1.oPortIOTimeMSTAIndexList;
	  		
        	//t1_time1.y_Output_time_event
	  		for(int j=0; j<outlports.size(); j++ ){
	  			String strPortName2 = outlports.get(j).portName;
				if (strPortName2 == null) {
					strPortName2 = Integer.toString(j);
				} 

				AL_Int oportio = oportios.get(j);
				ArrayList<Integer> ints2 = oportio.alInt;
				if(ints2 != null){
					Integer out1 = ints2.get(0);
					
					FieldAccessImpl outp1 = myLib.STree_mkFieldAccess(time1_name, strPortName2+"_Output_time_event"); 
			  		SignalsDefinitionImpl eq6 = ASME2SSME_Create_affine_equation(out1, period, outp1);
			  		myLib.STree_addExpressionOfProcessToaModel(pscheduler, eq6);  
				}
	  		}
	  		
        	//==============================
    	}
    }
    
    
    /**
     * Create an equation that represents the process call of subthread scheduler
     * 
     * @param pscheduler the process scheduler
     * @param t1 thread subcomponent
     */
    private SignalsDefinitionImpl ASME2SSME_Create_ThreadScheduler_Instance(
    		ProcessModelImpl pscheduler, ThreadSubcomponent t1){
    	
    	Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Create_ThreadScheduler_Instance : " + t1.getName());
    	ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
    	ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
    	ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
    	
    	String t1_name = aadlColAPI.Subcomponent_GetName(t1);
    	ThreadType tt1 = aadlColAPI.Threadsubcomponent_GetComponentType(t1);
    	
    	//====================================================
    	//make an instance of tscheduler1: (t1_ctl1, t1_time1) := t1_Thread_Scheduling(top)
    	    	   			
    	//top    	
    	lins.add(myLib.mkIdentifier("top"));
    			
    	//t1_ctl1   
    	louts.add(myLib.mkIdentifier(t1_name+"_ctl1"));	
		    
    	//ti_time1
		if(aadlColAPI.Thread_HasPorts(tt1)){
			louts.add(myLib.mkIdentifier(t1_name+"_time1"));
		}
    	   	
    	SignalsDefinitionImpl eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(t1_name+"_Thread_Scheduler"), lparams, lins, louts);
    	//====================================================    	
    	
    	return eq;
    }
    
    
    /**
     * Returns a Signal equations eq that: sig1 := affine_sample{phi, d}(top)
     * @param phi
     * @param d
     * @param sig1
     * @return
     */
    private SignalsDefinitionImpl ASME2SSME_Create_affine_equation(int phi, int d, ExprSignal sig1){
    	
    	Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Create_affine_equation");
    	
		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
		
		//phi
		ConstantIntegerImpl phi1 = myLib.STree_mkIntegerConstant(Integer.toString(phi));	
		lparams.add(phi1);
		//d
		ConstantIntegerImpl d1 = myLib.STree_mkIntegerConstant(Integer.toString(d));	
		lparams.add(d1);
		//top
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		lins.add(myLib.mkIdentifier("top"));
		//sig1
		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
		louts.add(sig1);
		SignalsDefinitionImpl eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("affine_sample"), lparams, lins, louts);
    	return eq;
    }
    
    
    
    /**
     * Create an equation that represents the process call of subprocess scheduler
     * 
     * @param subp processor subcomponent
     * @param p_behavior behavior subprocess of subp
     * @param sub1 process subcomponent
     */
    private SignalsDefinitionImpl ASME2SSME_Create_ProcessScheduler_Instance(ProcessorSubcomponent subp, 
    		ProcessModelImpl p_behavior, ProcessSubcomponent sub1){
    	
    	Trace("\n ====== ASME2SSME_Scheduling: ASME2SSME_Create_ProcessScheduler_Instance : " + sub1.getName());
    	ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
    	ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
    	ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
    	
    	String p1_name = aadlColAPI.Subcomponent_GetName(sub1);
    	ProcessImplementation pimpl = aadlColAPI.Processsubcomponent_GetComponentImpl(sub1);
    	
    	//====================================================
    	//make an instance of pscheduler1: 
    	//(sub1_t1_ctl1, sub1_t1_time1, ...) := sub1_Process_Schedul(top)
    	    	   			
    	//top    	
    	lins.add(myLib.mkIdentifier("top"));
    	    	
    	ArrayList<ThreadSubcomponent> subts = aadlColAPI.Process_GetSubcomponentsThread(pimpl);
    	for(int i =0; i< subts.size(); i++){
    		//for each subthread t1, add sub1_t1_ctl1, sub1_t1_time1
    		ThreadSubcomponent t1 = subts.get(i);
    		ThreadType tt1 = aadlColAPI.Threadsubcomponent_GetComponentType(t1);
    		String t1_name = aadlColAPI.Subcomponent_GetName(t1);
    		
    		//t1_ctl1   
        	louts.add(myLib.mkIdentifier(p1_name+"_"+t1_name+"_ctl1"));
		    
        	//ti_time1
    		if(aadlColAPI.Thread_HasPorts(tt1)){
    			louts.add(myLib.mkIdentifier(p1_name+"_"+t1_name+"_time1"));
    		}
    	}
    			 	   	
    	SignalsDefinitionImpl eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(p1_name+"_Process_Scheduler"), lparams, lins, louts);
    	//====================================================    	
    	
    	return eq;
    	
    }
    	
}
