 
/*
 * //////////////////////////////////////////////////////////////////////////
 * //This class simulates the generation of deadline signal
 * //////////////////////////////////////////////////////////////////////////
 */
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import org.osate.aadl2.*;
import org.eclipse.pop.ssme.Type;
import org.eclipse.pop.ssme.impl.*;
import org.eclipse.pop.ssme.polychrony.SignalOperators;


public class ASME2SSME_Simulate_Deadline extends ASME2SSME_Simulate_Common {
	
	private signalTreeAPI myLib ;
	private AADLCoLAPI aadlColAPI ;
	/**
	 * Class constructor
	 */
	public ASME2SSME_Simulate_Deadline() {	
		  myLib= new signalTreeAPI();
		  aadlColAPI = new AADLCoLAPI();
	}
	
	
	
	
	
	/**
	 * Make a Signal process that simulates the generation of Deadline signals
	 * 
	 * @param n
	 * @return
	 */
	public ProcessModelImpl ASME2SSME_mkSimDeadline(ProcessorSubcomponent subp, ArrayList<ProcessSubcomponent> list1){
		
//		process Sim_Deadline = 
//			(? event top; event dispatch1, dispatch2, ...;
//			 ! event deadline1, deadline2, ...; )
//			(| deadline1 := dispatch1
//           | deadline2 := dispatch2
//           | ...
//			 |);
		
		Trace("\n ######## ASME2SSME_Simulate_Deadline: ASME2SSME_mkSimDeadline : " + subp.getName());
	
		ProcessModelImpl vprocess = new ProcessModelImpl();
		ArrayList<ThreadSubcomponent> list2 = new ArrayList<ThreadSubcomponent>();
		for(int n = 0; n< list1.size(); n++){
			ProcessSubcomponent sub1 = list1.get(n); 
    	  	ProcessImplementation p1_impl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(sub1);   	  	
    	  	ArrayList<ThreadSubcomponent> listx = aadlColAPI.Process_GetSubcomponentsThread(p1_impl);
    	  	if (listx.size()>0) list2.addAll(listx);
		}
		
		vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier("Sim_Deadline") );
		
		//interface:  ?  event top;  event dispach0, dispatch1, ...
		Type type = myLib.mkPredefinedType(SignalOperators.opevent);
		SignalsDeclarationImpl vdecsig = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(TOP_ID), type, null);
	   //	vdecsig = mkEventDecl(TOP_ID);
		myLib.STree_addDeclarationToInterface(vprocess, vdecsig, 2);
		
		for(int i =0; i<list2.size(); i++){
			 Type type1 = myLib.mkPredefinedType(SignalOperators.opevent);
			SignalsDeclarationImpl vdecsig1 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(DISPATCH_ID+ Integer.toString(i) ), type1, null);
			myLib.STree_addDeclarationToInterface(vprocess, vdecsig1, 2);
		}
			
		// interface: ! event deadline0, deadline1, ...;
		for(int j =0; j<list2.size(); j++){
			Type t1 = myLib.mkPredefinedType(SignalOperators.opevent);
			SignalsDeclarationImpl out1 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(DEADLINE_ID+ Integer.toString(j) ), t1, null);
			myLib.STree_addDeclarationToInterface(vprocess, out1, 3);
		}
		
		//body  
		// deadlinex := dispatchx
		for(int k =0; k<list2.size(); k++){
			IdentifierImpl leftExp = myLib.mkIdentifier(DEADLINE_ID+ Integer.toString(k) );
			IdentifierImpl rightExp = myLib.mkIdentifier(DISPATCH_ID+ Integer.toString(k) );
			SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, eq);
		}
		myLib.STree_addComment(vprocess, "Signal process simulates Deadline signals", false);
		return vprocess;
	}
}
