 
/*
 * //////////////////////////////////////////////////////////////////////////
 * //This class simulates the generation of dispatch signal
 * //////////////////////////////////////////////////////////////////////////
 */
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import org.osate.aadl2.*;

import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;
import org.eclipse.pop.ssme.polychrony.SignalOperators;


public class ASME2SSME_Simulate_Dispatch extends ASME2SSME_Simulate_Common {
	private AADLCoLAPI aadlColAPI;
	private signalTreeAPI myLib;
	private ProcessModelImpl vprocess = new ProcessModelImpl();
	/**
	 * Class constructor
	 */
	public ASME2SSME_Simulate_Dispatch() {
		myLib= new signalTreeAPI();
		aadlColAPI = new AADLCoLAPI();
		vprocess = new ProcessModelImpl();
	}



	/**
	 * Make a Signal process that simulates the generation of Dispatch signals.
	 * @param n
	 * @return
	 */
	public ProcessModelImpl ASME2SSME_mkSimDispatch_Periodic(Integer n){

		//		process Sim_Dispatch_Periodic = 
		//			{integer p1, p2, ppcm;}
		//			(? event top;
		//			 ! event h1, h2;)
		//			(| h1 := affine_sample{ppcm/p1,1}(top)
		//			 | h2 := affine_sample{ppcm/p2,1}(top)
		//			 |);

		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_mkSimDispatch_Periodic");


		vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier("Sim_Dispatch_Periodic") );

		//interface  
		//parameters: Integer p0, ..., p(n-1)
		for(int i =0; i<n; i++){
			org.eclipse.pop.ssme.Type type = myLib.mkPredefinedType(SignalOperators.opinteger);
			SignalsDeclarationImpl param1 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("p"+ Integer.toString(i) ), type, null);
			myLib.STree_addDeclarationToInterface(vprocess, param1, 1);
		}

		//		//parameter: Integer ppcm
		//		Type type1 = myLib.mkPredefinedType(SignalOperators.opinteger);
		//		SignalsDeclarationImpl ppcm = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("ppcm" ), type1, null);
		//		myLib.STree_addDeclarationToInterface(vprocess, ppcm, 1);

		//input: event top;
		org.eclipse.pop.ssme.Type type = myLib.mkPredefinedType(SignalOperators.opevent);
		SignalsDeclarationImpl vdecsig = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("top" ), type, null);
		myLib.STree_addDeclarationToInterface(vprocess, vdecsig, 2);

		//outputs: event d0, ..., d(n-1)
		for(int j =0; j<n; j++){
			org.eclipse.pop.ssme.Type t1 = myLib.mkPredefinedType(SignalOperators.opevent);
			SignalsDeclarationImpl out1 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("d"+ Integer.toString(j) ), t1, null);
			myLib.STree_addDeclarationToInterface(vprocess, out1, 3);
		}

		//body  
		// dk := affine_sample{1, pk}(top)
		for(int k =0; k<n; k++){
			SignalsDefinitionImpl eq = ASME2SSME_Call_Affine(Integer.toString(k));
			myLib.STree_addExpressionOfProcessToaModel(vprocess, eq);
		}

		myLib.STree_addComment(vprocess, "Signal process simulates Dispatch signals", false);
		return vprocess;
	}


	/**
	 * Makes a process call of affine_sample: dx := affine_sample{1, px}(top)
	 * @param x
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_Call_Affine(String x){

		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_Call_Affine");

		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
		//1
		ConstantIntegerImpl ci1 = myLib.STree_mkIntegerConstant("1");	
		lparams.add(ci1);

		//px
		lparams.add(myLib.mkIdentifier("p"+x));

		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		lins.add(myLib.mkIdentifier("top"));

		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
		louts.add(myLib.mkIdentifier("d"+x));
		SignalsDefinitionImpl eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("affine_sample"), lparams, lins, louts);

		return eq;

	}


	/**
	 * Calculates ppcm of two integers
	 * 
	 * @param x
	 * @param y
	 * @return
	 */
	public int PPCM (int x, int y) {

		Trace("\n ######## ASME2SSME_Simulate_Dispatch: PPCM");

		int product, rest, ppcm;    	
		if ((0 == x) || (0 == y)){
			Trace(" PPCM (" + x + "," + y + ") = 0");
			return 0;
		}

		product = x*y;
		rest   = x%y;
		while(rest != 0){
			x = y;
			y = rest;
			rest = x%y;
		}

		ppcm = product/y;
		Trace(" PPCM (" + x + "," + y + ") = " + ppcm);

		return ppcm;		
	}


	/**
	 * Calcules ppcm of a list of integers
	 * 
	 * @param plist
	 * @return
	 */
	public int Calcule_PPCM(ArrayList<Integer> plist ){

		Trace("\n ######## ASME2SSME_Simulate_Dispatch: Calcule_PPCM");

		int x = plist.get(0);
		int z = 1;
		int y;

		if(plist.size() == 1){
			return plist.get(0);
		}

		for (int i = 1; i < plist.size(); i++){
			y = plist.get(i);
			z = PPCM  (x, y);
			x = z;
		}		

		return z;
	}


	/**
	 * Converts a list of periods (value, unit) to the same unit, 
	 * and returns the final integer period values
	 * 
	 * @param list
	 * @return
	 */
	public ArrayList<Integer> Calcule_Int_SameUnit(ArrayList<IntegerLiteral> list){

		Trace("\n ######## ASME2SSME_Simulate_Dispatch: Calcule_Int_SameUnit");
		
		ArrayList<Integer> list2 = new ArrayList<Integer>();

		if(list.size() > 0){

			IntegerLiteral v1 = list.get(0);
			UnitLiteral sunit = aadlColAPI.IntegerLiteral_Unit(v1);

			if(list.size()>1){
				for(int j = 1; j< list.size(); j++){
					UnitLiteral u2 = aadlColAPI.IntegerLiteral_Unit(list.get(j));
					UnitLiteral u1 = Get_Small_TimeUnit (sunit, u2);
					sunit = u1;
				}
			}
			Trace("The smallest unit is " + aadlColAPI.IntegerLiteral_Get_Unit(v1));

			for(int i=0; i< list.size(); i++){
				IntegerLiteral int_value = (IntegerLiteral) list.get(i);			
				// String str_value = aadlColAPI.IntegerLiteral_Get_Value(int_value);  //getNumberAsString();
				long value = int_value.getValue();
				double factor = aadlColAPI.IntegerLiteral_Get_AbsoluteFactor(int_value, sunit);   //getUnitLiteral().getAbsoluteFactor(sunit);
				double scaled_value = value * factor;
				list2.add((int) scaled_value);
			}
		}
		return list2;
	}


	/**
	 * Returns the smaller time unit: ps, ns, us,ms,sec,min,hr
	 * @param u1
	 * @param u2
	 * @return
	 */
	public UnitLiteral Get_Small_TimeUnit (UnitLiteral u1, UnitLiteral u2){

		Trace("\n ######## ASME2SSME_Simulate_Dispatch: Get_Small_TimeUnit");

		UnitLiteral ux;

		if( aadlColAPI.UnitLiteral_Name(u1).equals("Ps")
				| aadlColAPI.UnitLiteral_Name(u1).equals("ps")){
			ux = u1;
		}
		else if( aadlColAPI.UnitLiteral_Name(u1).equals("Ns") 
				| aadlColAPI.UnitLiteral_Name(u1).equals("ns")){
			if ( aadlColAPI.UnitLiteral_Name(u2).equals("Ps")
					| aadlColAPI.UnitLiteral_Name(u2).equals("ps")){
				ux = u2;
			}
			else {
				ux = u1;
			}
		}
		else if( aadlColAPI.UnitLiteral_Name(u1).equals("Us") 
				| aadlColAPI.UnitLiteral_Name(u1).equals("us")){
			if ( aadlColAPI.UnitLiteral_Name(u2).equals("Ps") 
					| aadlColAPI.UnitLiteral_Name(u1).equals("ps")
					| aadlColAPI.UnitLiteral_Name(u2).equals("Ns")
					| aadlColAPI.UnitLiteral_Name(u1).equals("ns")){
				ux = u2;
			}
			else {
				ux = u1;
			}
		}
		else if( aadlColAPI.UnitLiteral_Name(u1).equals("Ms")
				| aadlColAPI.UnitLiteral_Name(u1).equals("ms")){
			if ( aadlColAPI.UnitLiteral_Name(u2).equals("Ps") 
					| aadlColAPI.UnitLiteral_Name(u1).equals("ps")
					| aadlColAPI.UnitLiteral_Name(u2).equals("Ns")
					| aadlColAPI.UnitLiteral_Name(u1).equals("ns")
					| aadlColAPI.UnitLiteral_Name(u2).equals("Us")
					| aadlColAPI.UnitLiteral_Name(u1).equals("us")){
				ux = u2;
			}
			else {
				ux = u1;
			}
		}
		else if( aadlColAPI.UnitLiteral_Name(u1).equals("Sec")
				| aadlColAPI.UnitLiteral_Name(u1).equals("ses")){
			if ( aadlColAPI.UnitLiteral_Name(u2).equals("Min") 
					| aadlColAPI.UnitLiteral_Name(u1).equals("min")
					| aadlColAPI.UnitLiteral_Name(u2).equals("Hr")
					| aadlColAPI.UnitLiteral_Name(u1).equals("hr")){
				ux = u1;
			}
			else {
				ux = u2;
			}
		}
		else if( aadlColAPI.UnitLiteral_Name(u1).equals("Min") 
				| aadlColAPI.UnitLiteral_Name(u1).equals("min")){
			if ( aadlColAPI.UnitLiteral_Name(u2).equals("Hr")
					| aadlColAPI.UnitLiteral_Name(u1).equals("hr")){
				ux = u1;
			}
			else {
				ux = u2;
			}
		}
		else{
			ux = u2;
		}

		Trace("the small unit of "+ aadlColAPI.UnitLiteral_Name(u1) + " " + aadlColAPI.UnitLiteral_Name(u2) + " is " + aadlColAPI.UnitLiteral_Name(ux));
		return ux;
	}


	/**
	 * Makes a process call equation that calls the Sim_Dispatch process. 
	 * () := Sim_Dispatch{p1, p2, ...}(top)
	 * 
	 * @param plist
	 * @param lins
	 * @param louts
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_Call_SimDispatch_Periodic(ArrayList<IntegerLiteral> plist,
			ArrayList<ExprSignal> lins, ArrayList<ExprSignal> louts){

		  	
		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_Call_SimDispatch_Periodic : ");

		//a list of integer period values
		ArrayList<Integer> ilist = Calcule_Int_SameUnit(plist);
		//ppcm
		int ppcm = Calcule_PPCM(ilist);
		Trace("\n ppcm = " + Integer.toString(ppcm));

		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
		for(int i= 0; i< ilist.size(); i++){ // ppcm / pi
			Integer pi = ilist.get(i);
			Integer pix = ppcm/pi;

			ConstantIntegerImpl ci1 = myLib.STree_mkIntegerConstant(Integer.toString(pix));	
			lparams.add(ci1);
		}

		SignalsDefinitionImpl eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("Sim_Dispatch_Periodic"), lparams, lins, louts);

		return eq;
	}



	/**
	 * Make a Signal process that simulates the generation of Dispatch signals of aperiodic threads
	 * 
	 * @param subp a processor subcomponent
	 * @param bprocess a Signal process that represents the behavior part of a processor
	 * @param sysimpl a system implementation which contains subp
	 * @return
	 */
	public ProcessModelImpl ASME2SSME_mkSimDispatch_Aperiodic(ProcessorSubcomponent subp, ProcessModelImpl bprocess, SystemImplementation sysimpl){

		//		process Sim_Dispatch_Aperiodic = 
		//			(? event top;
		//             event p1_t1_i1_TE, p1_e_t1_i2_TE, ...;
		//             event p1_t2_i1_TE, p1_e_t2_i2_TE, ...; ...
		//			 ! event p1_t1_dispatch, p2_d2_dispatch, ...;)
		//			(| p1_t1_dispatch := p1_t1_i1_TE default p1_t1_i2_TE default ...
		//			 | p1_t2_dispatch := p1_t2_i1_TE default p1_t2_i2_TE default ...
		//           | ...
		//			 |);

		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_mkSimDispatch_Aperiodic");
		
		
		ProcessModelImpl vprocess =  myLib.STree_mkSkeletonModel(myLib.mkIdentifier("Sim_Dispatch_Aperiodic") );
		org.eclipse.pop.ssme.Type type = myLib.mkPredefinedType(SignalOperators.opevent);
		SignalsDeclarationImpl vdecsig = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("top" ), type, null);
		myLib.STree_addDeclarationToInterface(vprocess, vdecsig, 2);

		//interface =================================================================================
		ArrayList<ProcessSubcomponent> listp = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subp);
		int cnt =0;
		for(int i =0; i<listp.size(); i++){
			ProcessSubcomponent p1 = listp.get(i);
			ProcessImplementation pimpl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(p1);
			ArrayList<ThreadSubcomponent> list1 = aadlColAPI.Process_GetSubcomponentsThread(pimpl);

			for(int j =0; j< list1.size(); j++){
				ThreadSubcomponent t1 = list1.get(j);
				ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(t1);
				ThreadType tt1= aadlColAPI.Thread_GetCompType(tt);

				String dispatch = aadlColAPI.Thread_Get_DispatchProtocol_str(tt);
				if(dispatch.equals("Aperiodic")){
					cnt++;
					ArrayList<EventPort> list2 = aadlColAPI.Component_GetInEventPort(tt1);
					ArrayList<EventDataPort> list3 = aadlColAPI.Component_GetInEventDataPort(tt1);

					//for each in event port i of sub thread t: add an event e_t_i as an output of process type
					if(list2.size()>0){
						for(int m=0; m<list2.size(); m++){
							EventPort ep1 = list2.get(m);
							String name1 = Name_of_TriggerDispatch_Event(aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_" + aadlColAPI.Port_GetName(ep1));
							org.eclipse.pop.ssme.Type id1 = myLib.mkPredefinedType(SignalOperators.opevent);
							SignalsDeclarationImpl sig1 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name1), id1, null);
							myLib.STree_addDeclarationToInterface(vprocess, sig1, 2);
						}
					}

					if(list3.size()>0){
						for(int n=0; n<list3.size(); n++){
							EventDataPort edp1 = list3.get(n);
							String name2 = Name_of_TriggerDispatch_Event(aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_" + aadlColAPI.Port_GetName(edp1));
							org.eclipse.pop.ssme.Type id2 = myLib.mkPredefinedType(SignalOperators.opevent);
							SignalsDeclarationImpl sig2 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name2), id2, null);
							myLib.STree_addDeclarationToInterface(vprocess, sig2, 2);
						}
					}

					String name3 = aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_dispatch";
					org.eclipse.pop.ssme.Type id3 = myLib.mkPredefinedType(SignalOperators.opevent);
					SignalsDeclarationImpl sig3 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name3), id3, null);
					myLib.STree_addDeclarationToInterface(vprocess, sig3, 3);

				}
			}
		}

		if(cnt == 0){
			return null;
		}

		//body ==========================================================
		// p1_t1 := p1_t1_i1_TE default p1_t1_i2_TE default ...
		for(int k =0; k<listp.size(); k++){
			ProcessSubcomponent p1 = listp.get(k);
			ProcessImplementation pimpl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(p1);
			ArrayList<ThreadSubcomponent> list1 = aadlColAPI.Process_GetSubcomponentsThread(pimpl);
			for(int j =0; j< list1.size(); j++){
				ThreadSubcomponent t1 = list1.get(j);
				ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(t1);
				// ThreadType tt1= aadlColAPI.Thread_GetCompType(tt);				
				String dispatch = aadlColAPI.Thread_Get_DispatchProtocol_str(tt);
				if(dispatch.equals("Aperiodic")){
					SignalsDefinitionImpl eq1 = ASME2SSME_mkSimDispatch_Aperiodic_definition(p1, t1);
					myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
				}

			}
		}
		return vprocess;			
	}



	/**
	 * Make a Signal equation that defines xx_dispatch: p1_t2_dispatch := p1_t2_i1_TE default p1_t2_i2_TE default ...
	 * 
	 * @param subp a processor subcomponent
	 * @param bprocess a Signal process that represents the behavior part of a processor
	 * @param sysimpl a system implementation which contains subp
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_mkSimDispatch_Aperiodic_definition(ProcessSubcomponent sub1, ThreadSubcomponent sub2){

		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_mkSimDispatch_Aperiodic_definition");
		
		
		SignalsDefinitionImpl eq1 = new SignalsDefinitionImpl();

		String name1 = aadlColAPI.Subcomponent_GetName(sub1);
		String name2 = aadlColAPI.Subcomponent_GetName(sub2);

		IdentifierImpl leftExp = myLib.mkIdentifier(name1 + "_" + name2 + "_dispatch");

		ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(sub2);
		ThreadType tt1= aadlColAPI.Thread_GetCompType(tt);
		ArrayList<EventPort> list2 = aadlColAPI.Component_GetInEventPort(tt1);
		ArrayList<EventDataPort> list3 = aadlColAPI.Component_GetInEventDataPort(tt1);
		ArrayList<IdentifierImpl> list_id = new ArrayList<IdentifierImpl>();

		for(int i =0; i< list2.size(); i++){
			EventPort p1 = list2.get(i);
			IdentifierImpl id1 = myLib.mkIdentifier(Name_of_TriggerDispatch_Event(name1 + "_" + name2 + "_" + aadlColAPI.Port_GetName(p1)));
			list_id.add(id1);
		}
		for(int j =0; j< list3.size(); j++){
			EventDataPort p2 = list3.get(j);
			IdentifierImpl id2 = myLib.mkIdentifier(Name_of_TriggerDispatch_Event(name1 + "_" + name2 + "_" + aadlColAPI.Port_GetName(p2)));
			list_id.add(id2);
		}

		if(list_id.size() == 1){
			String str = list_id.get(0).getAtomString();
			IdentifierImpl rightExp = myLib.mkIdentifier(str);
			eq1 = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		}
		else{
			String str1 = myLib.Identifier_GetStr(list_id.get(0));
			IdentifierImpl exp0 = myLib.mkIdentifier(str1);
			ExprSignal exp1 = exp0;
			IdentifierImpl exp2 = new IdentifierImpl();
			DefaultImpl rightExp = new DefaultImpl();

			for(int k =1; k< list_id.size(); k++){
				exp2 = list_id.get(k);
				rightExp = myLib.STree_mkDefault(exp1, exp2);
				exp1 = rightExp;
			}
			eq1 = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		}

		return eq1;
	}



	/**
	 * Makes a process call of Sim_Dispatch_Aperiodic process: Sim_Dispatch_Aperiodic{}
	 * 
	 * @param plist
	 * @param lins
	 * @param louts
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_Call_SimDispatch_Aperiodic(ArrayList<ExprParam> lps,
			ArrayList<ExprSignal> lins, ArrayList<ExprSignal> louts){

		   	
		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_Call_SimDispatch_Aperiodic");

		SignalsDefinitionImpl eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("Sim_Dispatch_Aperiodic"), lps, lins, louts);

		return eq;
	}




	/**
	 * Make a Signal process that simulates the generation of Dispatch signals of sporadic threads
	 * 
	 * @param subp a processor subcomponent
	 * @param bprocess a Signal process that represents the behavior part of a processor
	 * @param sysimpl a system implementation which contains subp
	 * @return
	 */
	public ProcessModelImpl ASME2SSME_mkSimDispatch_Sporadic(ProcessorSubcomponent subp, ProcessModelImpl bprocess, SystemImplementation sysimpl){

		//		process Sim_Dispatch_Sporadic = 
		//			(? event top;
		//             event p1_t1_i1_TE, p1_t1_i2_TE, ...;
		//             event p1_t2_i1_TE, p1_t2_i2_TE, ...; ...
		//			 ! event p1_t1_dispatch, p1_t2_dispatch, ...;)
		//			(| p1_t1_dispatch := p1_t1_i1_TE default p1_t1_i2_TE default ...
		//			 | p1_t2_dispatch := p1_t2_i1_TE default p1_t2_i2_TE default ...
		//           | ...
		//			 |);

		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_mkSimDispatch_Sporadic : " + subp.getName());
		
		
		ProcessModelImpl vprocess =  myLib.STree_mkSkeletonModel(myLib.mkIdentifier("Sim_Dispatch_Sporadic") );
		org.eclipse.pop.ssme.Type type = myLib.mkPredefinedType(SignalOperators.opevent);
		SignalsDeclarationImpl vdecsig = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("top" ), type, null);
		myLib.STree_addDeclarationToInterface(vprocess, vdecsig, 2);

		//interface =================================================================================
		ArrayList<ProcessSubcomponent> listp = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subp);
		int cnt =0;
		for(int i =0; i<listp.size(); i++){
			ProcessSubcomponent p1 = listp.get(i);
			ProcessImplementation pimpl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(p1);
			ArrayList<ThreadSubcomponent> list1 = aadlColAPI.Process_GetSubcomponentsThread(pimpl);
			for(int j =0; j< list1.size(); j++){
				ThreadSubcomponent t1 = list1.get(j);
				ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(t1);
				ThreadType tt1= aadlColAPI.Thread_GetCompType(tt);

				String dispatch = aadlColAPI.Thread_Get_DispatchProtocol_str(tt);
				if(dispatch.equals("Sporadic")){
					cnt++;
					ArrayList<EventPort> list2 = aadlColAPI.Component_GetInEventPort(tt1);
					ArrayList<EventDataPort> list3 = aadlColAPI.Component_GetInEventDataPort(tt1);

					//for each in event port i of sub thread t: add an event e_t_i as an output of process type
					if(list2.size()>0){
						for(int m=0; m<list2.size(); m++){
							EventPort ep1 = list2.get(m);
							String name1 = Name_of_TriggerDispatch_Event(aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_" + aadlColAPI.Port_GetName(ep1));
							org.eclipse.pop.ssme.Type id1 = myLib.mkPredefinedType(SignalOperators.opevent);
							SignalsDeclarationImpl sig1 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name1), id1, null);
							myLib.STree_addDeclarationToInterface(vprocess, sig1, 2);
						}
					}

					if(list3.size()>0){
						for(int n=0; n<list3.size(); n++){
							EventDataPort edp1 = list3.get(n);
							String name2 = Name_of_TriggerDispatch_Event(aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_" + aadlColAPI.Port_GetName(edp1));
							org.eclipse.pop.ssme.Type id2 = myLib.mkPredefinedType(SignalOperators.opevent);
							SignalsDeclarationImpl sig2 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name2), id2, null);
							myLib.STree_addDeclarationToInterface(vprocess, sig2, 2);
						}
					}

					String name3 = aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_dispatch";
					org.eclipse.pop.ssme.Type id3 = myLib.mkPredefinedType(SignalOperators.opevent);
					SignalsDeclarationImpl sig3 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name3), id3, null);
					myLib.STree_addDeclarationToInterface(vprocess, sig3, 3);

				}
			}
		}

		if(cnt == 0){
			return null;
		}

		//body ==========================================================
		// p1_t1 := p1_t1_i1_TE default p1_t1_i2_TE default ...
		for(int k =0; k<listp.size(); k++){
			ProcessSubcomponent p1 = listp.get(k);
			ProcessImplementation pimpl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(p1);
			ArrayList<ThreadSubcomponent> list1 = aadlColAPI.Process_GetSubcomponentsThread(pimpl);
			for(int j =0; j< list1.size(); j++){
				ThreadSubcomponent t1 = list1.get(j);
				ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(t1);			
				String dispatch = aadlColAPI.Thread_Get_DispatchProtocol_str(tt);
				if(dispatch.equals("Sporadic")){
					SignalsDefinitionImpl eq1 = ASME2SSME_mkSimDispatch_Sporadic_definition(p1, t1);
					myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
				}

			}
		}

		return vprocess;	

	}



	/**
	 * Make a Signal equation that defines xx_dispatch: p1_t2_dispatch := p1_t2_i1_TE default p1_t2_i2_TE default ...
	 * 
	 * @param subp a processor subcomponent
	 * @param bprocess a Signal process that represents the behavior part of a processor
	 * @param sysimpl a system implementation which contains subp
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_mkSimDispatch_Sporadic_definition(ProcessSubcomponent sub1, ThreadSubcomponent sub2){



		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_mkSimDispatch_Sporadic_definition : " + sub2.getName());
		
		
		SignalsDefinitionImpl eq1 = new SignalsDefinitionImpl();

		String name1 = aadlColAPI.Subcomponent_GetName(sub1);
		String name2 = aadlColAPI.Subcomponent_GetName(sub2);

		IdentifierImpl leftExp = myLib.mkIdentifier(name1 + "_" + name2 + "_dispatch");

		ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(sub2);
		ThreadType tt1= aadlColAPI.Thread_GetCompType(tt);
		ArrayList<EventPort> list2 = aadlColAPI.Component_GetInEventPort(tt1);
		ArrayList<EventDataPort> list3 = aadlColAPI.Component_GetInEventDataPort(tt1);
		ArrayList<IdentifierImpl> list_id = new ArrayList<IdentifierImpl>();

		for(int i =0; i< list2.size(); i++){
			EventPort p1 = list2.get(i);
			IdentifierImpl id1 = myLib.mkIdentifier(Name_of_TriggerDispatch_Event(name1 + "_" + name2 + "_" + aadlColAPI.Port_GetName(p1)));
			list_id.add(id1);
		}
		for(int j =0; j< list3.size(); j++){
			EventDataPort p2 = list3.get(j);
			IdentifierImpl id2 = myLib.mkIdentifier(Name_of_TriggerDispatch_Event(name1 + "_" + name2 + "_" + aadlColAPI.Port_GetName(p2)));
			list_id.add(id2);
		}

		if(list_id.size() == 1){
			String str = myLib.Identifier_GetStr(list_id.get(0));
			IdentifierImpl rightExp = myLib.mkIdentifier(str);
			eq1 = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		}
		else{
			String str1 = myLib.Identifier_GetStr(list_id.get(0));
			IdentifierImpl exp0 = myLib.mkIdentifier(str1);
			ExprSignal exp1 = exp0;
			IdentifierImpl exp2 = new IdentifierImpl();
			DefaultImpl rightExp = new DefaultImpl();

			for(int k =1; k< list_id.size(); k++){
				exp2 = list_id.get(k);
				rightExp = myLib.STree_mkDefault(exp1, exp2);
				exp1 = rightExp;
			}
			eq1 = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		}

		return eq1;
	}



	/**
	 * Makes a process call of Sim_Dispatch_Sporadic process: Sim_Dispatch_Sporadic{}
	 * 
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_Call_SimDispatch_Sporadic(ArrayList<ExprParam> lps,
			ArrayList<ExprSignal> lins, ArrayList<ExprSignal> louts){

		   	
		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_Call_SimDispatch_Sporadic");

		SignalsDefinitionImpl eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("Sim_Dispatch_Sporadic"), lps, lins, louts);

		return eq;
	}



	/**
	 * Make a Signal process that simulates the generation of Dispatch signals of Timed threads
	 * 
	 * @param subp a processor subcomponent
	 * @param bprocess a Signal process that represents the behavior part of a processor
	 * @param sysimpl a system implementation which contains subp
	 * @return
	 */
	public ProcessModelImpl ASME2SSME_mkSimDispatch_Timed(ProcessorSubcomponent subp, ProcessModelImpl bprocess, SystemImplementation sysimpl){

		//		process Sim_Dispatch_Timed = 
		//			(? event top;
		//             event p1_t1_i1_TE, p1_t1_i2_TE, ...;
		//             event p1_t2_i1_TE, p1_t2_i2_TE, ...; ...
		//			 ! event p1_t1_dispatch, p1_t2_dispatch, ...;)
		//			(| p1_t1_dispatch := p1_t1_i1_TE default p1_t1_i2_TE default ...
		//			 | p1_t2_dispatch := p1_t2_i1_TE default p1_t2_i2_TE default ...
		//           | ...
		//			 |);

		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_mkSimDispatch_Timed : " + subp.getName());
		
		
		ProcessModelImpl vprocess =  myLib.STree_mkSkeletonModel(myLib.mkIdentifier("Sim_Dispatch_Timed") );
		org.eclipse.pop.ssme.Type type = myLib.mkPredefinedType(SignalOperators.opevent);
		SignalsDeclarationImpl vdecsig = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("top" ), type, null);
		myLib.STree_addDeclarationToInterface(vprocess, vdecsig, 2);

		//interface =================================================================================
		ArrayList<ProcessSubcomponent> listp = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subp);
		int cnt =0;
		for(int i =0; i<listp.size(); i++){
			ProcessSubcomponent p1 = listp.get(i);
			ProcessImplementation pimpl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(p1);
			ArrayList<ThreadSubcomponent> list1 = aadlColAPI.Process_GetSubcomponentsThread(pimpl);
			for(int j =0; j< list1.size(); j++){
				ThreadSubcomponent t1 = list1.get(j);
				ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(t1);
				ThreadType tt1= aadlColAPI.Thread_GetCompType(tt);

				String dispatch = aadlColAPI.Thread_Get_DispatchProtocol_str(tt);
				if(dispatch.equals("Timed")){
					cnt++;
					ArrayList<EventPort> list2 = aadlColAPI.Component_GetInEventPort(tt1);
					ArrayList<EventDataPort> list3 = aadlColAPI.Component_GetInEventDataPort(tt1);

					//for each in event port i of sub thread t: add an event e_t_i as an output of process type
					if(list2.size()>0){
						for(int m=0; m<list2.size(); m++){
							EventPort ep1 = list2.get(m);
							String name1 = Name_of_TriggerDispatch_Event(aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_" + aadlColAPI.Port_GetName(ep1));
							org.eclipse.pop.ssme.Type id1 = myLib.mkPredefinedType(SignalOperators.opevent);
							SignalsDeclarationImpl sig1 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name1), id1, null);
							myLib.STree_addDeclarationToInterface(vprocess, sig1, 2);
						}
					}

					if(list3.size()>0){
						for(int n=0; n<list3.size(); n++){
							EventDataPort edp1 = list3.get(n);
							String name2 = Name_of_TriggerDispatch_Event(aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_" + aadlColAPI.Port_GetName(edp1));
							org.eclipse.pop.ssme.Type id2 = myLib.mkPredefinedType(SignalOperators.opevent);
							SignalsDeclarationImpl sig2 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name2), id2, null);
							myLib.STree_addDeclarationToInterface(vprocess, sig2, 2);
						}
					}

					String name3 = aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_dispatch";
					org.eclipse.pop.ssme.Type id3 = myLib.mkPredefinedType(SignalOperators.opevent);
					SignalsDeclarationImpl sig3 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name3), id3, null);
					myLib.STree_addDeclarationToInterface(vprocess, sig3, 3);

				}
			}
		}

		if(cnt == 0){
			return null;
		}

		//body ==========================================================
		// p1_t1 := p1_t1_i1_TE default p1_t1_i2_TE default ...
		for(int k =0; k<listp.size(); k++){
			ProcessSubcomponent p1 = listp.get(k);
			ProcessImplementation pimpl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(p1);
			ArrayList<ThreadSubcomponent> list1 = aadlColAPI.Process_GetSubcomponentsThread(pimpl);
			for(int j =0; j< list1.size(); j++){
				ThreadSubcomponent t1 = list1.get(j);
				ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(t1);			
				String dispatch = aadlColAPI.Thread_Get_DispatchProtocol_str(tt);
				if(dispatch.equals("Timed")){
					SignalsDefinitionImpl eq1 = ASME2SSME_mkSimDispatch_Timed_definition(p1, t1);
					myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
				}

			}
		}

		return vprocess;			
	}



	/**
	 * Make a Signal equation that defines xx_dispatch: p1_t2_dispatch := p1_t2_i1_TE default p1_t2_i2_TE default ...
	 * 
	 * @param subp a processor subcomponent
	 * @param bprocess a Signal process that represents the behavior part of a processor
	 * @param sysimpl a system implementation which contains subp
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_mkSimDispatch_Timed_definition(ProcessSubcomponent sub1, ThreadSubcomponent sub2){



		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_mkSimDispatch_Timed_definition : " + sub2.getName());
		
		
		SignalsDefinitionImpl eq1 = new SignalsDefinitionImpl();

		String name1 = aadlColAPI.Subcomponent_GetName(sub1);
		String name2 = aadlColAPI.Subcomponent_GetName(sub2);

		IdentifierImpl leftExp = myLib.mkIdentifier(name1 + "_" + name2 + "_dispatch");		
		ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(sub2);
		ThreadType tt1= aadlColAPI.Thread_GetCompType(tt);
		ArrayList<EventPort> list2 = aadlColAPI.Component_GetInEventPort(tt1);
		ArrayList<EventDataPort> list3 = aadlColAPI.Component_GetInEventDataPort(tt1);
		ArrayList<IdentifierImpl> list_id = new ArrayList<IdentifierImpl>();

		for(int i =0; i< list2.size(); i++){
			EventPort p1 = list2.get(i);
			IdentifierImpl id1 = myLib.mkIdentifier(Name_of_TriggerDispatch_Event(name1 + "_" + name2 + "_" + aadlColAPI.Port_GetName(p1)));
			list_id.add(id1);
		}
		for(int j =0; j< list3.size(); j++){
			EventDataPort p2 = list3.get(j);
			IdentifierImpl id2 = myLib.mkIdentifier(Name_of_TriggerDispatch_Event(name1 + "_" + name2 + "_" + aadlColAPI.Port_GetName(p2)));
			list_id.add(id2);
		}

		if(list_id.size() == 1){
			String str = myLib.Identifier_GetStr(list_id.get(0));
			IdentifierImpl rightExp = myLib.mkIdentifier(str);
			eq1 = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		}
		else{
			String str1 = myLib.Identifier_GetStr(list_id.get(0));
			IdentifierImpl exp0 = myLib.mkIdentifier(str1);
			ExprSignal exp1 = exp0;
			IdentifierImpl exp2 = new IdentifierImpl();
			DefaultImpl rightExp = new DefaultImpl();

			for(int k =1; k< list_id.size(); k++){
				exp2 = list_id.get(k);
				rightExp = myLib.STree_mkDefault(exp1, exp2);
				exp1 = rightExp;
			}
			eq1 = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		}

		return eq1;
	}



	/**
	 * Makes a process call of Sim_Dispatch_Timed process: Sim_Dispatch_Timed{}
	 * 
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_Call_SimDispatch_Timed(ArrayList<ExprParam> lps,
			ArrayList<ExprSignal> lins, ArrayList<ExprSignal> louts){

		    	
		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_Call_SimDispatch_Timed");

		SignalsDefinitionImpl eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("Sim_Dispatch_Timed"), lps, lins, louts);

		return eq;
	}




	/**
	 * Make a Signal process that simulates the generation of Dispatch signals of Hybrid threads
	 * 
	 * @param subp a processor subcomponent
	 * @param bprocess a Signal process that represents the behavior part of a processor
	 * @param sysimpl a system implementation which contains subp
	 * @return
	 */
	public ProcessModelImpl ASME2SSME_mkSimDispatch_Hybrid(ProcessorSubcomponent subp, ProcessModelImpl bprocess, SystemImplementation sysimpl){

		// 		process Sim_Dispatch_Hybrid = 
		// 			(? event top;
		//              event p1_t1_i1_TE, p1_t1_i2_TE, ...;
		//              event p1_t2_i1_TE, p1_t2_i2_TE, ...; ...
		// 			 ! event p1_t1_dispatch, p1_t2_dispatch, ...;)
		// 			(| p1_t1_dispatch := p1_t1_i1_TE default p1_t1_i2_TE default ...
		// 			 | p1_t2_dispatch := p1_t2_i1_TE default p1_t2_i2_TE default ...
		//            | ...
		// 			 |);

		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_mkSimDispatch_Hybrid : " + subp.getName());
		
		
		ProcessModelImpl vprocess =  myLib.STree_mkSkeletonModel(myLib.mkIdentifier("Sim_Dispatch_Hybrid") );
		org.eclipse.pop.ssme.Type type = myLib.mkPredefinedType(SignalOperators.opevent);
		SignalsDeclarationImpl vdecsig = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("top" ), type, null);
		myLib.STree_addDeclarationToInterface(vprocess, vdecsig, 2);

		//interface =================================================================================
		ArrayList<ProcessSubcomponent> listp = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subp);
		int cnt =0;
		for(int i =0; i<listp.size(); i++){
			ProcessSubcomponent p1 = listp.get(i);
			ProcessImplementation pimpl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(p1);
			ArrayList<ThreadSubcomponent> list1 = aadlColAPI.Process_GetSubcomponentsThread(pimpl);
			for(int j =0; j< list1.size(); j++){
				ThreadSubcomponent t1 = list1.get(j);
				ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(t1);
				ThreadType tt1= aadlColAPI.Thread_GetCompType(tt);

				String dispatch = aadlColAPI.Thread_Get_DispatchProtocol_str(tt);
				if(dispatch.equals("Hybrid")){
					cnt++;
					ArrayList<EventPort> list2 = aadlColAPI.Component_GetInEventPort(tt1);
					ArrayList<EventDataPort> list3 = aadlColAPI.Component_GetInEventDataPort(tt1);

					//for each in event port i of sub thread t: add an event e_t_i as an output of process type
					if(list2.size()>0){
						for(int m=0; m<list2.size(); m++){
							EventPort ep1 = list2.get(m);
							String name1 = Name_of_TriggerDispatch_Event(aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_" + aadlColAPI.Port_GetName(ep1));
							org.eclipse.pop.ssme.Type id1 = myLib.mkPredefinedType(SignalOperators.opevent);
							SignalsDeclarationImpl sig1 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name1), id1, null);
							myLib.STree_addDeclarationToInterface(vprocess, sig1, 2);
						}
					}

					if(list3.size()>0){
						for(int n=0; n<list3.size(); n++){
							EventDataPort edp1 = list3.get(n);
							String name2 = Name_of_TriggerDispatch_Event(aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_" + aadlColAPI.Port_GetName(edp1));
							org.eclipse.pop.ssme.Type id2 = myLib.mkPredefinedType(SignalOperators.opevent);
							SignalsDeclarationImpl sig2 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name2), id2, null);
							myLib.STree_addDeclarationToInterface(vprocess, sig2, 2);
						}
					}

					String name3 = aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_dispatch";
					org.eclipse.pop.ssme.Type id3 = myLib.mkPredefinedType(SignalOperators.opevent);
					SignalsDeclarationImpl sig3 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name3), id3, null);
					myLib.STree_addDeclarationToInterface(vprocess, sig3, 3);

				}
			}
		}

		if(cnt == 0) return null;

		//body  
		// p1_t1 := p1_t1_i1_TE default p1_t1_i2_TE default ...
		for(int k =0; k<listp.size(); k++){
			ProcessSubcomponent p1 = listp.get(k);
			ProcessImplementation pimpl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(p1);
			ArrayList<ThreadSubcomponent> list1 = aadlColAPI.Process_GetSubcomponentsThread(pimpl);
			for(int j =0; j< list1.size(); j++){
				ThreadSubcomponent t1 = list1.get(j);
				ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(t1);			
				String dispatch = aadlColAPI.Thread_Get_DispatchProtocol_str(tt);
				if(dispatch.equals("Hybrid")){
					SignalsDefinitionImpl eq1 = ASME2SSME_mkSimDispatch_Hybrid_definition(p1, t1);
					myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
				}

			}
		}

		return vprocess;			
	}



	/**
	 * Make a Signal equation that defines xx_dispatch: p1_t2_dispatch := p1_t2_i1_TE default p1_t2_i2_TE default ...
	 * 
	 * @param subp a processor subcomponent
	 * @param bprocess a Signal process that represents the behavior part of a processor
	 * @param sysimpl a system implementation which contains subp
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_mkSimDispatch_Hybrid_definition(ProcessSubcomponent sub1, ThreadSubcomponent sub2){

		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_mkSimDispatch_Hybrid_definition : " + sub2.getName());
		
		
		SignalsDefinitionImpl eq1 = new SignalsDefinitionImpl();

		String name1 = aadlColAPI.Subcomponent_GetName(sub1);
		String name2 = aadlColAPI.Subcomponent_GetName(sub2);

		IdentifierImpl leftExp = myLib.mkIdentifier(name1 + "_" + name2 + "_dispatch");		
		ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(sub2);
		ThreadType tt1= aadlColAPI.Thread_GetCompType(tt);
		ArrayList<EventPort> list2 = aadlColAPI.Component_GetInEventPort(tt1);
		ArrayList<EventDataPort> list3 = aadlColAPI.Component_GetInEventDataPort(tt1);
		ArrayList<IdentifierImpl> list_id = new ArrayList<IdentifierImpl>();

		for(int i =0; i< list2.size(); i++){
			EventPort p1 = list2.get(i);
			IdentifierImpl id1 = myLib.mkIdentifier(Name_of_TriggerDispatch_Event(name1 + "_" + name2 + "_" + aadlColAPI.Port_GetName(p1)));
			list_id.add(id1);
		}
		for(int j =0; j< list3.size(); j++){
			EventDataPort p2 = list3.get(j);
			IdentifierImpl id2 = myLib.mkIdentifier(Name_of_TriggerDispatch_Event(name1 + "_" + name2 + "_" + aadlColAPI.Port_GetName(p2)));
			list_id.add(id2);
		}

		if(list_id.size() == 1){
			String str = myLib.Identifier_GetStr(list_id.get(0));
			IdentifierImpl rightExp = myLib.mkIdentifier(str);
			eq1 = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		}
		else{
			String str1 = myLib.Identifier_GetStr(list_id.get(0));
			IdentifierImpl exp0 = myLib.mkIdentifier(str1);
			ExprSignal exp1 = exp0;
			IdentifierImpl exp2 = new IdentifierImpl();
			DefaultImpl rightExp = new DefaultImpl();

			for(int k =1; k< list_id.size(); k++){
				exp2 = list_id.get(k);
				rightExp = myLib.STree_mkDefault(exp1, exp2);
				exp1 = rightExp;
			}
			eq1 = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		}

		return eq1;
	}



	/**
	 * Makes a process call of Sim_Dispatch_Hybrid process: Sim_Dispatch_Hybrid{}
	 * 
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_Call_SimDispatch_Hybrid(ArrayList<ExprParam> lps,
			ArrayList<ExprSignal> lins, ArrayList<ExprSignal> louts){

		    	
		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_Call_SimDispatch_Hybrid");

		SignalsDefinitionImpl eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("Sim_Dispatch_Hybrid"), lps, lins, louts);

		return eq;
	}




	/**
	 * Make a Signal process that simulates the generation of Dispatch signals of Background threads
	 * 
	 * @param subp a processor subcomponent
	 * @param bprocess a Signal process that represents the behavior part of a processor
	 * @param sysimpl a system implementation which contains subp
	 * @return
	 */
	public ProcessModelImpl ASME2SSME_mkSimDispatch_Background(ProcessorSubcomponent subp, ProcessModelImpl bprocess, SystemImplementation sysimpl){

		//  		process Sim_Dispatch_Background = 
		//  			(? event top;
		//               event p1_t1_i1_TE, p1_t1_i2_TE, ...;
		//               event p1_t2_i1_TE, p1_t2_i2_TE, ...; ...
		//  			 ! event p1_t1_dispatch, p1_t2_dispatch, ...;)
		//  			(| p1_t1_dispatch := p1_t1_i1_TE default p1_t1_i2_TE default ...
		//  			 | p1_t2_dispatch := p1_t2_i1_TE default p1_t2_i2_TE default ...
		//             | ...
		//  			 |);

		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_mkSimDispatch_Background : " + subp.getName());
		
		
		ProcessModelImpl vprocess =  myLib.STree_mkSkeletonModel(myLib.mkIdentifier("Sim_Dispatch_Background") );
		org.eclipse.pop.ssme.Type type = myLib.mkPredefinedType(SignalOperators.opevent);
		SignalsDeclarationImpl vdecsig = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("top" ), type, null);
		myLib.STree_addDeclarationToInterface(vprocess, vdecsig, 2);

		//interface 
		ArrayList<ProcessSubcomponent> listp = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subp);
		int cnt =0;
		for(int i =0; i<listp.size(); i++){
			ProcessSubcomponent p1 = listp.get(i);
			ProcessImplementation pimpl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(p1);
			ArrayList<ThreadSubcomponent> list1 = aadlColAPI.Process_GetSubcomponentsThread(pimpl);
			for(int j =0; j< list1.size(); j++){
				ThreadSubcomponent t1 = list1.get(j);
				ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(t1);
				ThreadType tt1= aadlColAPI.Thread_GetCompType(tt);

				String dispatch = aadlColAPI.Thread_Get_DispatchProtocol_str(tt);
				if(dispatch.equals("Background")){
					cnt++;
					ArrayList<EventPort> list2 = aadlColAPI.Component_GetInEventPort(tt1);
					ArrayList<EventDataPort> list3 = aadlColAPI.Component_GetInEventDataPort(tt1);

					//for each in event port i of sub thread t: add an event e_t_i as an output of process type
					if(list2.size()>0){
						for(int m=0; m<list2.size(); m++){
							EventPort ep1 = list2.get(m);
							String name1 = Name_of_TriggerDispatch_Event(aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_" + aadlColAPI.Port_GetName(ep1));
							org.eclipse.pop.ssme.Type id1 = myLib.mkPredefinedType(SignalOperators.opevent);
							SignalsDeclarationImpl sig1 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name1), id1, null);
							myLib.STree_addDeclarationToInterface(vprocess, sig1, 2);
						}
					}

					if(list3.size()>0){
						for(int n=0; n<list3.size(); n++){
							EventDataPort edp1 = list3.get(n);
							String name2 = Name_of_TriggerDispatch_Event(aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_" + aadlColAPI.Port_GetName(edp1));
							org.eclipse.pop.ssme.Type id2 = myLib.mkPredefinedType(SignalOperators.opevent);
							SignalsDeclarationImpl sig2 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name2), id2, null);
							myLib.STree_addDeclarationToInterface(vprocess, sig2, 2);
						}
					}

					String name3 = aadlColAPI.Subcomponent_GetName(p1) +"_" + aadlColAPI.Subcomponent_GetName(t1) + "_dispatch";
					org.eclipse.pop.ssme.Type id3 = myLib.mkPredefinedType(SignalOperators.opevent);
					SignalsDeclarationImpl sig3 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name3), id3, null);
					myLib.STree_addDeclarationToInterface(vprocess, sig3, 3);

				}
			}
		}

		if(cnt == 0) return null;

		//body 
		// p1_t1 := p1_t1_i1_TE default p1_t1_i2_TE default ...
		for(int k =0; k<listp.size(); k++){
			ProcessSubcomponent p1 = listp.get(k);
			ProcessImplementation pimpl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(p1);
			ArrayList<ThreadSubcomponent> list1 = aadlColAPI.Process_GetSubcomponentsThread(pimpl);
			for(int j =0; j< list1.size(); j++){
				ThreadSubcomponent t1 = list1.get(j);
				ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(t1);
				// ThreadType tt1= aadlColAPI.Thread_GetCompType(tt);				
				String dispatch = aadlColAPI.Thread_Get_DispatchProtocol_str(tt);
				if(dispatch.equals("Background")){
					SignalsDefinitionImpl eq1 = ASME2SSME_mkSimDispatch_Background_definition(p1, t1);
					myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
				}

			}
		}
		return vprocess;	
	}



	/**
	 * Make a Signal equation that defines xx_dispatch: p1_t2_dispatch := p1_t2_i1_TE default p1_t2_i2_TE default ...
	 * 
	 * @param subp a processor subcomponent
	 * @param bprocess a Signal process that represents the behavior part of a processor
	 * @param sysimpl a system implementation which contains subp
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_mkSimDispatch_Background_definition(ProcessSubcomponent sub1, ThreadSubcomponent sub2){

		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_mkSimDispatch_Background_definition : " + sub2.getName());
		
		SignalsDefinitionImpl eq1 = new SignalsDefinitionImpl();
		String name1 = aadlColAPI.Subcomponent_GetName(sub1);
		String name2 = aadlColAPI.Subcomponent_GetName(sub2);

		IdentifierImpl leftExp = myLib.mkIdentifier(name1 + "_" + name2 + "_dispatch"); 		
		ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(sub2);
		ThreadType tt1= aadlColAPI.Thread_GetCompType(tt);
		ArrayList<EventPort> list2 = aadlColAPI.Component_GetInEventPort(tt1);
		ArrayList<EventDataPort> list3 = aadlColAPI.Component_GetInEventDataPort(tt1);
		ArrayList<IdentifierImpl> list_id = new ArrayList<IdentifierImpl>();

		for(int i =0; i< list2.size(); i++){
			EventPort p1 = list2.get(i);
			IdentifierImpl id1 = myLib.mkIdentifier(Name_of_TriggerDispatch_Event(name1 + "_" + name2 + "_" + aadlColAPI.Port_GetName(p1)));
			list_id.add(id1);
		}
		for(int j =0; j< list3.size(); j++){
			EventDataPort p2 = list3.get(j);
			IdentifierImpl id2 = myLib.mkIdentifier(Name_of_TriggerDispatch_Event(name1 + "_" + name2 + "_" + aadlColAPI.Port_GetName(p2)));
			list_id.add(id2);
		}

		if(list_id.size() == 1){
			String str = myLib.Identifier_GetStr(list_id.get(0));
			IdentifierImpl rightExp = myLib.mkIdentifier(str);
			eq1 = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		}
		else{
			String str1 = myLib.Identifier_GetStr(list_id.get(0));
			IdentifierImpl exp0 = myLib.mkIdentifier(str1);
			ExprSignal exp1 = exp0;
			IdentifierImpl exp2 = new IdentifierImpl();
			DefaultImpl rightExp = new DefaultImpl();

			for(int k =1; k< list_id.size(); k++){
				exp2 = list_id.get(k);
				rightExp = myLib.STree_mkDefault(exp1, exp2);
				exp1 = rightExp;
			}
			eq1 = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		}

		return 	eq1;
	}



	/**
	 * Makes a process call of Sim_Dispatch_Background process: Sim_Dispatch_Background{}
	 * 
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_Call_SimDispatch_Background(ArrayList<ExprParam> lps,
			ArrayList<ExprSignal> lins, ArrayList<ExprSignal> louts){

		      	
		Trace("\n ######## ASME2SSME_Simulate_Dispatch: ASME2SSME_Call_SimDispatch_Backgroundd");

		SignalsDefinitionImpl eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("Sim_Dispatch_Background"), lps, lins, louts);

		return eq;
	}


}
