
  
 /* 
 * //////////////////////////////////////////////////////////////////////////
 * //This class simulates the generation of TimeEvent signal
 * //////////////////////////////////////////////////////////////////////////
 */
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import org.osate.aadl2.*;

import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;
import org.eclipse.pop.ssme.polychrony.SignalOperators;

public class ASME2SSME_Simulate_TimeEvent extends ASME2SSME_Simulate_Common{
	private signalTreeAPI myLib;
	private AADLCoLAPI aadlColAPI;
	
	/**
	 * Class constructor
	 */
	public ASME2SSME_Simulate_TimeEvent() {
		 myLib= new signalTreeAPI();
		 aadlColAPI = new AADLCoLAPI();
		}
	
	
	/**
	 * Make a Signal process that simulates the generation of time_event signals
	 * 
	 * @param n
	 * @return
	 */
	public ProcessModelImpl ASME2SSME_mkSimTimeEvent(ProcessorSubcomponent subp, ArrayList<ProcessSubcomponent> list1){
		
//		process Sim_TimeEvent = 
//			(? event top; 
//             CTL1 xp_xt_ctl1, ...;
//             CTL2 xp_xt_ctl2, ...;
//			 ! xp_xt_TIME_EVENT xp_xt_time1; ... )
//			(| xp_xt_time1.yy_Frozen_time_event := xp_xt_ctl1.Dispatch
//           | xp_xt_time1.zz_Output_time_event := xp_xt_ctl2.Complete
//           | ...
//			 |);
		
		Trace("\n ######## ASME2SSME_Simulate_TimeEvent: ASME2SSME_mkSimTimeEvent : " + subp.getName());
		
		ProcessModelImpl vprocess = new ProcessModelImpl();		
		vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier("Sim_TimeEvent") );
		
		//interface ============================================		
		ASME2SSME_mkSimTimeEvent_Interface(subp, list1, vprocess);
		
		//body ===============================================
		ASME2SSME_mkSimTimeEvent_Body(subp, list1, vprocess);
		
		//add comments for behavior process instance
		ArrayList<String> comments3 = new ArrayList<String>();
		comments3.add("Signal process simulates time_event signals");
		myLib.STree_addComment(vprocess, comments3, false);
		
		return vprocess;
		
	}
	
	/**
	 * Make a Signal process that simulates the interface of Sim_time_event process
	 * 
	 * @param n
	 * @return
	 */
	public void ASME2SSME_mkSimTimeEvent_Interface(ProcessorSubcomponent subp, ArrayList<ProcessSubcomponent> list1, ProcessModelImpl vprocess){
		
		Trace("\n ######## ASME2SSME_Simulate_TimeEvent: ASME2SSME_mkSimTimeEvent_Interface : " + subp.getName());
		
    	
    	ASME2SSME_Translate_Name asme2ssme_name = new ASME2SSME_Translate_Name();
    	
		//input: event top;
    	org.eclipse.pop.ssme.Type type = myLib.mkPredefinedType(SignalOperators.opevent);
		SignalsDeclarationImpl vdecsig = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("top" ), type, null);
		myLib.STree_addDeclarationToInterface(vprocess, vdecsig, 2);
		
		//inputs: CTL1 xp_xt_ctl1, ...;
		for(int n = 0; n< list1.size(); n++){
			ProcessSubcomponent sub1 = list1.get(n); //sub process
    	  	String sub1_name = aadlColAPI.Subcomponent_GetName(sub1); //sub process name
    	  	ProcessImplementation p1_impl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(sub1); // process impl   	  	
    	  	ArrayList<ThreadSubcomponent> listx = aadlColAPI.Process_GetSubcomponentsThread(p1_impl);
    	  	
    	  	for (int m = 0; m<listx.size(); m++){
    	  		ThreadSubcomponent sub2 = listx.get(m);
    	  		String sub2_name = aadlColAPI.Subcomponent_GetName(sub2);
    	  		IdentifierImpl type_ctl1 = myLib.mkIdentifier(asme2ssme_name.Name_of_CTL1_type());
    	  		IdentifierImpl id1 = myLib.mkIdentifier(sub1_name + "_" + sub2_name + "_ctl1");        	  	
    			SignalsDeclarationImpl vdecsig1 = myLib.STree_mkSignalDeclaration(id1, type_ctl1, null);
    			myLib.STree_addDeclarationToInterface(vprocess, vdecsig1, 2);
    	  	}
    	  	
		}
		
		//inputs: CTL2 xp_xt_ctl2, ...;
		for(int n1 = 0; n1< list1.size(); n1++){
				ProcessSubcomponent sub1 = list1.get(n1); //sub process
		    	 String sub1_name = aadlColAPI.Subcomponent_GetName(sub1); //sub process name
		    	 ProcessImplementation p1_impl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(sub1); // process impl   	  	
		    	 ArrayList<ThreadSubcomponent> listx = aadlColAPI.Process_GetSubcomponentsThread(p1_impl);
		    	  	
		    	 for (int m1 = 0; m1<listx.size(); m1++){
		    	  		ThreadSubcomponent sub2 = listx.get(m1);
		    	  		String sub2_name = aadlColAPI.Subcomponent_GetName(sub2);
		    	  		IdentifierImpl type_ctl2 = myLib.mkIdentifier(asme2ssme_name.Name_of_CTL2_type());
		    	  		IdentifierImpl id2 = myLib.mkIdentifier(sub1_name + "_" + sub2_name + "_ctl2");        	  	
		    			SignalsDeclarationImpl vdecsig2 = myLib.STree_mkSignalDeclaration(id2, type_ctl2, null);
		    			myLib.STree_addDeclarationToInterface(vprocess, vdecsig2, 2);
		    	  	}
		    	  	
				}		
					
		//outputs: xp_xt_TIME_EVENT xp_xt_time1; ...
		for(int n2 = 0; n2< list1.size(); n2++){
			//Trace("ASME2SSME_mkSimTimeEvent_Interface() list1.size() >0");
			ProcessSubcomponent sub1 = list1.get(n2); //sub process
		    String sub1_name = aadlColAPI.Subcomponent_GetName(sub1); //sub process name
		    ProcessImplementation p1_impl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(sub1); // process impl   	  	
		    ArrayList<ThreadSubcomponent> listx = aadlColAPI.Process_GetSubcomponentsThread(p1_impl);
		    	  	
		    for (int m2 = 0; m2<listx.size(); m2++){
		    	//Trace("ASME2SSME_mkSimTimeEvent_Interface() listx.size() >0"); 
		    	ThreadSubcomponent sub2 = listx.get(m2);
		    	  String sub2_name = aadlColAPI.Subcomponent_GetName(sub2);
		    	  ThreadImplementation timpl = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(sub2);
		    	  ThreadType ttype = aadlColAPI.Thread_GetCompType(timpl);
		    	  
		    	  if(aadlColAPI.Thread_HasPorts(ttype)){
		    		  //Trace("ASME2SSME_mkSimTimeEvent_Interface() Thread_HasPorts() = true");
		    		  String t_type_name = aadlColAPI.ComponentType_GetName(ttype);
			    	  String TimeBundle_type_name = asme2ssme_name.Name_of_ThreadTimeBundle(t_type_name);
			    	  IdentifierImpl type2 = myLib.mkIdentifier(TimeBundle_type_name);
			    	  IdentifierImpl id1 = myLib.mkIdentifier(sub1_name + "_" + sub2_name + "_time1");        	  	
			    	  SignalsDeclarationImpl vdecsig1 = myLib.STree_mkSignalDeclaration(id1, type2, null);
			    	  myLib.STree_addDeclarationToInterface(vprocess, vdecsig1, 3);
		    	  }		    	 
		    }
		}				
		
		return;
	}
	
	/**
	 * Make a Signal process that simulates the body of Sim_time_event process
	 * 
	 * @param n
	 * @return
	 */
	public void ASME2SSME_mkSimTimeEvent_Body(ProcessorSubcomponent subp, ArrayList<ProcessSubcomponent> list1, ProcessModelImpl vprocess){
	
		Trace("\n ######## ASME2SSME_Simulate_TimeEvent: ASME2SSME_mkSimTimeEvent_Body : " + subp.getName());
		
    	
    	
		// xp_xt_time1.yy_Frozen_time_event := xp_xt_ctl1.Dispatch or xp_xt_time1.zz_Output_time_event := xp_xt_ctl2.Complete (by default)
		// if the Input_Time or Output_Time is specified, then the specified value should be used to define the time event (not yet implemented)
		//for each sub process
		for(int n3 = 0; n3< list1.size(); n3++){
				
			ProcessSubcomponent sub3 = list1.get(n3); //sub process
			String sub1_name = aadlColAPI.Subcomponent_GetName(sub3); //sub process name
			ProcessImplementation p1_impl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(sub3); // process impl   	  	
			ArrayList<ThreadSubcomponent> listx = aadlColAPI.Process_GetSubcomponentsThread(p1_impl);
				 
			//for each sub thread
			for (int m3 = 0; m3<listx.size(); m3++){
		        ThreadSubcomponent sub4 = listx.get(m3);
				String sub4_name = aadlColAPI.Subcomponent_GetName(sub4);
				ThreadImplementation timpl = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(sub4);
				ThreadType ttype = (ThreadType) aadlColAPI.ComponentImpl_GetComponentType(timpl);
				
				if(aadlColAPI.Thread_HasPorts(ttype)){
					//for each in port p1: defines the time event by default: xp_xt_time1.yy_Frozen_time_event := xp_xt_ctl1.Dispatch
					ArrayList<DataPort> ind_list = aadlColAPI.Thread_GetInDataPort(ttype);
					for(int i=0; i<ind_list.size(); i++){
						DataPort p1 = ind_list.get(i);
						if(aadlColAPI.Port_InputTime_IsDefault(timpl, p1)){
							SignalsDefinitionImpl eq1 = ASME2SSME_Def_FrozenTimeEvent(sub1_name, sub4_name, p1);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
						}
						else{
							Trace("Input_Time specified (not equal to default value), not yet implemented, use default value for the moment");
							SignalsDefinitionImpl eq1 = ASME2SSME_Def_FrozenTimeEvent(sub1_name, sub4_name, p1);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
							//add comments 
							ArrayList<String> comments1 = new ArrayList<String>();
							comments1.add("Input_Time not equal to default value (to be implemented), use default value for the moment");
							myLib.STree_addComment(eq1, comments1, false);
						}
					}
					
					ArrayList<EventPort> ine_list = aadlColAPI.Thread_GetInEventPort(ttype);
					for(int i1=0; i1<ine_list.size(); i1++){
						EventPort p2 = ine_list.get(i1);
						if(aadlColAPI.Port_InputTime_IsDefault(timpl, p2)){
							SignalsDefinitionImpl eq2 = ASME2SSME_Def_FrozenTimeEvent(sub1_name, sub4_name, p2);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq2);
						}
						else{
							Trace("Input_Time specified (not equal to default value), not yet implemented, use default value for the moment");
							SignalsDefinitionImpl eq2 = ASME2SSME_Def_FrozenTimeEvent(sub1_name, sub4_name, p2);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq2);
							//add comments 
							ArrayList<String> comments2 = new ArrayList<String>();
							comments2.add("Input_Time not equal to default value (to be implemented), use default value for the moment");
							myLib.STree_addComment(eq2, comments2, false);
						}
					}
					
					ArrayList<EventDataPort> ined_list = aadlColAPI.Thread_GetInEventDataPort(ttype);
					for(int i2=0; i2<ined_list.size(); i2++){
						EventDataPort p3 = ined_list.get(i2);
						if(aadlColAPI.Port_InputTime_IsDefault(timpl, p3)){
							SignalsDefinitionImpl eq3 = ASME2SSME_Def_FrozenTimeEvent(sub1_name, sub4_name, p3);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq3);
						}
						else{
							Trace("Input_Time specified (not equal to default value), not yet implemented, use default value for the moment");
							SignalsDefinitionImpl eq3 = ASME2SSME_Def_FrozenTimeEvent(sub1_name, sub4_name, p3);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq3);
							//add comments 
							ArrayList<String> comments3 = new ArrayList<String>();
							comments3.add("Input_Time not equal to default value (to be implemented), use default value for the moment");
							myLib.STree_addComment(eq3, comments3, false);
						}
					}
					
					//for each out port p2: defines the time event by default: xp_xt_time1.zz_Output_time_event := xp_xt_ctl2.Complete
					ArrayList<DataPort> outd_list = aadlColAPI.Thread_GetOutDataPort(ttype);
					for(int j=0; j<outd_list.size(); j++){
						DataPort p4 = outd_list.get(j);
						if(aadlColAPI.Port_OutputTime_IsDefault(timpl, p4)){
							SignalsDefinitionImpl eq4 = ASME2SSME_Def_OutputTimeEvent(sub1_name, sub4_name, p4);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq4);
						}
						else{
							Trace("Output_Time specified (not equal to default value), not yet implemented, use default value for the moment");
							SignalsDefinitionImpl eq4 = ASME2SSME_Def_OutputTimeEvent(sub1_name, sub4_name, p4);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq4);
							//add comments 
							ArrayList<String> comments4 = new ArrayList<String>();
							comments4.add("Output_Time not equal to default value (to be implemented), use default value for the moment");
							myLib.STree_addComment(eq4, comments4, false);
						}
					}
					
					ArrayList<EventPort> oute_list = aadlColAPI.Thread_GetOutEventPort(ttype);
					for(int j1=0; j1<oute_list.size(); j1++){
						EventPort p5 = oute_list.get(j1);
						if(aadlColAPI.Port_OutputTime_IsDefault(timpl, p5)){
							SignalsDefinitionImpl eq5 = ASME2SSME_Def_OutputTimeEvent(sub1_name, sub4_name, p5);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq5);
						}
						else{
							Trace("Output_Time specified (not equal to default value), not yet implemented, use default value for the moment");
							SignalsDefinitionImpl eq5 = ASME2SSME_Def_OutputTimeEvent(sub1_name, sub4_name, p5);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq5);
							//add comments 
							ArrayList<String> comments5 = new ArrayList<String>();
							comments5.add("Output_Time not equal to default value (to be implemented), use default value for the moment");
							myLib.STree_addComment(eq5, comments5, false);
						}
					}
					
					ArrayList<EventDataPort> outed_list = aadlColAPI.Thread_GetOutEventDataPort(ttype);
					for(int j2=0; j2<outed_list.size(); j2++){
						EventDataPort p6 = outed_list.get(j2);
						if(aadlColAPI.Port_OutputTime_IsDefault(timpl, p6)){
							SignalsDefinitionImpl eq6 = ASME2SSME_Def_OutputTimeEvent(sub1_name, sub4_name, p6);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq6);
						}
						else{
							Trace("Output_Time specified (not equal to default value), not yet implemented, use default value for the moment");
							SignalsDefinitionImpl eq6 = ASME2SSME_Def_OutputTimeEvent(sub1_name, sub4_name, p6);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq6);
							//add comments 
							ArrayList<String> comments6 = new ArrayList<String>();
							comments6.add("Output_Time not equal to default value (to be implemented), use default value for the moment");
							myLib.STree_addComment(eq6, comments6, false);
						}
					}
				}				
			}
		}				
		
		return;
	}
	
	
	
	/**
	 * Makes a equation that defines the frozen_time_event of a port p by default value: 
	 * xp_xt_time1.p_Frozen_time_event := xp_xt_ctl1.Dispatch
	 * 
	 * @param sub_process name of sub process
	 * @param sub_thread name of sub_thread
	 * @param p a port
	 * @return an equation
	 */
	public SignalsDefinitionImpl ASME2SSME_Def_FrozenTimeEvent(String sub_process, String sub_thread, Port p){
		
		Trace("\n ######## ASME2SSME_Simulate_TimeEvent: ASME2SSME_Def_FrozenTimeEvent : " + p.getName());
		String pname = p.getName();
		ExprSignal leftExp = myLib.STree_mkFieldAccess(sub_process+"_"+sub_thread+"_time1", pname+"_Frozen_time_event");
		ExprSignal rightExp = myLib.STree_mkFieldAccess(sub_process+"_"+sub_thread+"_ctl1", "Dispatch");		
		SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		
		//add comments 
		ArrayList<String> comments = new ArrayList<String>();
		String str = "default value for Frozen_time_event for port " + pname + " of thread " + sub_thread;
		comments.add(str);
		myLib.STree_addComment(eq, comments, false);
				
		return eq;
	}
	
	
	/**
	 * Makes a equation that defines the output_time_event of a port p by default value: 
	 * xp_xt_time1.zz_Output_time_event := xp_xt_ctl2.Complete
	 * 
	 * @param sub_process name of sub process
	 * @param sub_thread name of sub_thread
	 * @param p a port
	 * @return an equation
	 */
	public SignalsDefinitionImpl ASME2SSME_Def_OutputTimeEvent(String sub_process, String sub_thread, Port p){
		
		Trace("\n ######## ASME2SSME_Simulate_TimeEvent: ASME2SSME_Def_OutputTimeEvent : " + p.getName());
		String pname = p.getName();
		ExprSignal leftExp = myLib.STree_mkFieldAccess(sub_process+"_"+sub_thread+"_time1", pname+"_Output_time_event");
		ExprSignal rightExp = myLib.STree_mkFieldAccess(sub_process+"_"+sub_thread+"_ctl2", "Complete");		
		SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		
		//add comments 
		ArrayList<String> comments = new ArrayList<String>();
		String str = "default value for Output_time_event for port " + pname + " of thread " + sub_thread;
		comments.add(str);
		myLib.STree_addComment(eq, comments, false);
				
		return eq;
	}
    
}
