 
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import org.osate.aadl2.*;
import org.eclipse.pop.ssme.impl.*;

public class ASME2SSME_Translate_Bus extends ASME2SSME_Translate_Common {
	// public class ASME2SSME_Translate_Bus extends ASME2SSME_Translate_Property changed to ..
	private AADLCoLAPI aadlColAPI;
	private signalTreeAPI myLib;
	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_Bus() {
		aadlColAPI = new AADLCoLAPI();
		myLib= new signalTreeAPI();
	}

	/**
	 * A bus type is represented as a Signal process
	 * 
	 * @param bt a bus type
	 * @return
	 */
	public ProcessModelImpl ASME2SSME_BusType(BusType bt){

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_BusType : " + bt.getName());

		String sname = aadlColAPI.ComponentType_GetName(bt);			
		String pname = Name_of_Bus(sname);

		// bus process skeleton
		ProcessModelImpl busprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );

		//refer to the type features
		//ASME2SSME_BusType(bus_type);
		ASME2SSME_Bus_Features(bt, busprocess);

		//Modes=====================================================

		//Properties================================================
		//properties implemented in BusImpl_Property() sub process

		//body: external ========================================================
		DescriptionImpl  desc = (DescriptionImpl) busprocess.getDescription();	
		if (desc == null)
		{
			busprocess.setDescription(new DescriptionImpl());
			desc = (DescriptionImpl) busprocess.getDescription();
		}
		ExternalAttributeImpl externe = new ExternalAttributeImpl();
		desc.setBody(externe);

		return busprocess;
	}


	/**
	 * Translate bus features (bus access)
	 * 
	 * @param btype bus type
	 * @param vprocess a Signal process that represents the bus
	 */
	public void ASME2SSME_Bus_Features(BusType btype, ProcessModelImpl vprocess){
		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_Bus_Features: " + aadlColAPI.ComponentType_GetName(btype));

		ASME2SSME_Translate_Feature asme2ssme_feature = new ASME2SSME_Translate_Feature();
		ArrayList<BusAccess> list = aadlColAPI.Bus_GetBusAccess(btype);

		//bus access==============
		if(list.size()>0){
			for(int i=0; i<list.size(); i++)
			{
				BusAccess da1 = list.get(i);
				asme2ssme_feature.ASME2SSME_BusAccess(da1);			
			}
		}
	}


	/** 
	 * Returns a Signal process that represents a bus implementation component
	 * <p>
	 * This Signal process is named by a method Name_of_Bus(). 
	 * Gets its bus type, and translates it as the interface of this process. 
	 * Declares a Signal process for its behavior ASME2SSME_Bus_behavior(), and adds it to the body. 
	 * Declares a Signal process for its properties ASME2SSME_Bus_property(), and adds it to the body. 
	 * 
	 * @param b a bus implementation component
	 * @return a Signal process
	 */
	public ProcessModelImpl ASME2SSME_BusImpl(BusImplementation b){

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_BusImpl : " + b.getName());	    
		String sname = aadlColAPI.ComponentImpl_GetName(b);			
		String pname = Name_of_Bus(sname);

		// bus process skeleton
		ProcessModelImpl busprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );

		// bus process interface ================================
		BusType bus_type = (BusType) aadlColAPI.ComponentImpl_GetComponentType(b);
		//ASME2SSME_ComponentInterface(busprocess, bus_type);

		//refer to the type features
		//ASME2SSME_BusType(bus_type);
		ASME2SSME_Bus_Features(bus_type, busprocess);

		//bus behavior ===========================================
		ProcessModelImpl bus_behavior = ASME2SSME_Bus_behavior(pname);
		String bus_behavior_name = myLib.STree_GetStringName(myLib.Process_GetID(bus_behavior));//pname + "_behavior";
		//add to declaration
		myLib.STree_addProcessDeclaration(busprocess, bus_behavior);
		//add to body
		ProcessCallImpl bus_behavior_inst = myLib.STree_mkModelCall(bus_behavior_name);
		myLib.STree_addExpressionOfProcessToaModel(busprocess, bus_behavior_inst);

		//add comments for behavior process instance
		ArrayList<String> comments3 = new ArrayList<String>();
		comments3.add("bus behavior sub process");
		myLib.STree_addComment(bus_behavior_inst, comments3, false);

		//bus property ================================================
		ProcessModelImpl bus_property = ASME2SSME_Bus_property(b);
		String bus_property_name = myLib.STree_GetStringName(myLib.Process_GetID(bus_property));//pname + "_property";
		//add to declaration
		myLib.STree_addProcessDeclaration(busprocess, bus_property);
		//add to body
		ProcessCallImpl bus_property_inst = myLib.STree_mkModelCall(bus_property_name);
		myLib.STree_addExpressionOfProcessToaModel(busprocess, bus_property_inst);

		//add comments for property process instance
		ArrayList<String> comments4 = new ArrayList<String>();
		comments4.add("bus property sub process");
		myLib.STree_addComment(bus_property_inst, comments4, false);

		//Subcomponents: no subcomponents exist in version 1.5.8

		//Connections: no subcomponents exist in version 1.5.8

		//Modes 
		ASME2SSME_BusImpl_Modes(b);

		//Properties are implemented in BusImpl_Property() sub process

		return busprocess;
	}


	/**
	 * Returns a Signal process "xx_behavior" that represents the behavior of a bus implementation component
	 * <p>
	 * This Signal process is named as "xx_behavior", where "xx" is the name of the Signal process that represents this bus implementation. 
	 * An empty skeleton process is provided. 
	 * 
	 * @param p_name a String name of the Signal process that represents the bus implementation
	 * @return a ProcessModelImpl object (Signal process)
	 */
	public ProcessModelImpl ASME2SSME_Bus_behavior(String p_name){

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_Bus_behavior" + p_name); 
		String bname = p_name + "_behavior";
		return myLib.STree_mkSkeletonModel(myLib.mkIdentifier(bname) );
	}


	/**
	 * Returns a Signal process "xx_property" that represents the properties of a bus implementation component
	 * <p>
	 * This Signal process is named as "xx_property", where "xx" is the name of the Signal process that represents this bus implementation. 
	 * An empty skeleton process is provided.
	 * 
	 * @param p_name a String name of the Signal process that represents this bus implementation. 
	 * @return a ProcessModelImpl object (Signal process)
	 */
	public ProcessModelImpl ASME2SSME_Bus_property(BusImplementation b_impl){

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_Bus_property" + b_impl.getName()); 
		String process_name = Name_of_Bus(aadlColAPI.ComponentImpl_GetName(b_impl));
		return  ASME2SSME_Component_Properties_Call(b_impl, process_name);
	}


	/**
	 * Translate modes of a bus implementation component
	 * 
	 * @param bb
	 */
	public void ASME2SSME_BusImpl_Modes(BusImplementation bb){
		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_BusImpl_Modes" + bb.getName());
		/*	ASME2SSME_Translate_Common asme2ssme_common = new ASME2SSME_Translate_Common();
		asme2ssme_common.ASME2SSME_ComponentImpl_Modes(bb); */
		ASME2SSME_ComponentImpl_Modes(bb);
	}


}
