

package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Calendar;
import java.text.SimpleDateFormat;

import org.eclipse.emf.common.util.EList;
import org.osate.aadl2.*;
import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;
import org.eclipse.pop.ssme.polychrony.SignalOperators;



/*
 * This class provides some common methods that can be used in common by the translation of AADL components  
 */
public class ASME2SSME_Translate_Common extends ASME2SSME_Translate_Feature{
	
	private signalTreeAPI myLib;
	private AADLCoLAPI aadlColAPI;

	private ASME2SSME_Translate_Mode asme2ssme_mode1;
	
	private SignalsDeclarationImpl mkSignalDeclaration(String type, String name) {
		SignalsDeclarationImpl vDeclSignal;
		if(myLib.Is_PredefinedType(type)){
			PredefinedTypeImpl ptype = myLib.STree_mkPredefinedType(type);
			vDeclSignal = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name), ptype, null);
		}
		else {
			vDeclSignal = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name), myLib.mkIdentifier(type), null);
			//add comments for the new created bundle type
			ArrayList<String> comments = new ArrayList<String>();
			String str1 = Generate_Signal_comment(name);
			comments.add(str1);
			myLib.STree_addComment(vDeclSignal, comments, false);
		} 
		return vDeclSignal;
	}

	/** 
	 * @return "true" if sig_name is declared in the list of declarations lDecl, "false" otherwise.
	 */
	private boolean isInDeclarations(EList<TypesOrSignalsDeclaration> lDecl, String sig_name) {

		Iterator<TypesOrSignalsDeclaration> it3 = lDecl.iterator();
		while (it3.hasNext())
		{
			SignalsDeclarationImpl aoutput = (SignalsDeclarationImpl) it3.next(); 
			EList<SignalNaming> sig_naming_list2 = aoutput.getListSignal();
			for (int k =0; k< sig_naming_list2.size(); k++){
				SignalNaming sig_naming2 = sig_naming_list2.get(k);
				String name = sig_naming2.getIdentifier().getAtomString();
				if(name.equals(sig_name)) return true;
			}
		}	
		return false;
	}
	
	private boolean addDataPortToInterface(ProcessModelImpl vprocess, ArrayList<DataPort> list_ind_port, int where) {
		SignalsDeclarationImpl	vdecsig;
		
		if (list_ind_port.size() == 0 ) return false;
		Iterator<?> itd1 = list_ind_port.iterator();
		while (itd1.hasNext())
		{
			DataPort dataPort = (DataPort) itd1.next();
			vdecsig = ASME2SSME_DataPort(dataPort);
			myLib.STree_addDeclarationToInterface(vprocess, vdecsig, where);
		}
		return true;
	} 
 

	private boolean addEventPortToInterface(ProcessModelImpl vprocess,	ArrayList<EventPort> list_oute_port, int where) {
		SignalsDeclarationImpl	vdecsig;
		if (list_oute_port.size() ==0)  return false;
			Iterator<?> ite2 = list_oute_port.iterator();
			while (ite2.hasNext())
			{
				EventPort eventPort = (EventPort) ite2.next();
				vdecsig = ASME2SSME_EventPort(eventPort);
				myLib.STree_addDeclarationToInterface(vprocess, vdecsig, where);
				eventPort = null;
			}
			return true;	
		} 		
	 

	private boolean addEventDataPortToInterface(ProcessModelImpl vprocess, ArrayList<EventDataPort> list_ined_port, int where) {
		SignalsDeclarationImpl	vdecsig;

		if (list_ined_port.size() == 0)  return false;	
			Iterator<?> ited1 = list_ined_port.iterator();
			while (ited1.hasNext())
			{
				EventDataPort eventdataPort = (EventDataPort) ited1.next();
				vdecsig = ASME2SSME_EventDataPort(eventdataPort);	
				myLib.STree_addDeclarationToInterface(vprocess, vdecsig, where);
				eventdataPort = null;
			}
			return true;	
		}
	
	
	private void TranslateModeTransitions(ArrayList<ModeTransition> mt_list) {
		if(mt_list.size()>0) 
			for(int n=0; n<mt_list.size(); n++)
				asme2ssme_mode1.ASME2SSME_ModeTransition(mt_list.get(n));
	}


	private void TranslateModes(ArrayList<Mode> m_list) {
		if(m_list.size()>0) 
			for(int m=0; m<m_list.size(); m++)
				asme2ssme_mode1.ASME2SSME_Mode(m_list.get(m));
	}

	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_Common() {
		myLib= new signalTreeAPI();
		aadlColAPI = new AADLCoLAPI();
	    asme2ssme_mode1 = new ASME2SSME_Translate_Mode();
	}

	/**
	 * Returns the current date in the form of yyyy/MMM/dd HH:mm:ss
	 * @return current date
	 */
	public String GetDateNow() {
		Calendar currentDate = Calendar.getInstance();
		SimpleDateFormat formatter= new SimpleDateFormat("yyyy/MMM/dd HH:mm:ss");
		String dateNow = formatter.format(currentDate.getTime());
		return dateNow;
	}


	/** 
	 * Creates a Signal process interface from a component type, 
	 * and add the interface to the Signal process.
	 * Translates the ports and requires data access feature as Signal inputs/outputs 
	 * 
	 * @param vprocess a Signal process representing this component (implementation)
	 * @param st a component type component
	 */
	public void ASME2SSME_ComponentInterface(ProcessModelImpl vprocess, ComponentType st){
		Trace("\n ######## ASME2SSME_Translate_Common: ASME2SSME_ComponentInterface: "+ aadlColAPI.ComponentType_GetName(st));
		// Data port
		addDataPortToInterface(vprocess, aadlColAPI.Component_GetInDataPort(st), 2 );
		addDataPortToInterface(vprocess, aadlColAPI.Component_GetOutDataPort(st), 3 );
		// Event port	 	
		if (! addEventPortToInterface(vprocess, aadlColAPI.Component_GetInEventPort(st), 2 )) Trace(aadlColAPI.ComponentType_GetName(st) + " in event port null");
		addEventPortToInterface(vprocess, aadlColAPI.Component_GetOutEventPort(st), 3 );
		// Event data port.
		addEventDataPortToInterface(vprocess, aadlColAPI.Component_GetInEventDataPort(st), 2 );
		addEventDataPortToInterface(vprocess, aadlColAPI.Component_GetOutEventDataPort(st), 3 );
	}

	/** 
	 * Translates requires data access feature as outputs of vprocess's interface:
	 * FIFO_TIME xx;
	 * 
	 * @param vprocess a Signal process representing this component (implementation)
	 * @param st a component type component
	 */
	public void ASME2SSME_ComponentInterface_DataAccess(ProcessModelImpl vprocess, ComponentType st){		

		Trace("\n ######## ASME2SSME_Translate_Common: ASME2SSME_ComponentInterface_DataAccess: "+ aadlColAPI.ComponentType_GetName(st));

		ArrayList<DataAccess> list1 = aadlColAPI.Component_GetRequiresDataAccess(st);

		//requires data access
		if(list1!= null && list1.size() >0 ){

			for(int m =0; m< list1.size(); m++)
			{
				DataAccess da1 = (DataAccess) list1.get(m);
				//		    
				//		    //xx_readtime
				//		    SignalsDeclarationImpl vdecsig1 = ASME2SSME_RequiresDataAccess_ReadTime(da1);
				//			myLib.STree_addDeclarationToInterface(vprocess, vdecsig1, 3);
				//			
				//			//xx_writetime
				//			SignalsDeclarationImpl vdecsig2 = ASME2SSME_RequiresDataAccess_WriteTime(da1);
				//			myLib.STree_addDeclarationToInterface(vprocess, vdecsig2, 3);
				//			
				//			//xx_reset
				//			SignalsDeclarationImpl vdecsig3 = ASME2SSME_RequiresDataAccess_reset(da1);
				//			myLib.STree_addDeclarationToInterface(vprocess, vdecsig3, 3);

				//FIFO_TIME xx
				SignalsDeclarationImpl vdecsig4 = ASME2SSME_RequiresDataAccess_Bundle(da1);
				myLib.STree_addDeclarationToInterface(vprocess, vdecsig4, 3);
			}
		}
	}


	/** 
	 * Creates two local signals for each requires data access xx in vprocess (a thread): 
	 * shared integer xx_w, xx_r;
	 * 
	 * @param vprocess a Signal process representing a thread
	 * @param st a component type component
	 */
	public void ASME2SSME_DataAccess_local(ProcessModelImpl vprocess, ComponentType st){
		SharedVariablesDeclarationImpl sig;
		// Why integer type??

		Trace("\n ######## ASME2SSME_Translate_Common: ASME2SSME_DataAccess_local: "+ aadlColAPI.ComponentType_GetName(st));

		ArrayList<DataAccess> list1 = aadlColAPI.Component_GetRequiresDataAccess(st);
		if( list1 != null && list1.size() >0 ){
			for(int m =0; m< list1.size(); m++)
			{
				DataAccess da1 = (DataAccess) list1.get(m);
				String str1 = aadlColAPI.DataAccess_GetName(da1);

				IdentifierImpl xx_w = myLib.mkIdentifier(str1+"_w");
				sig = myLib.STree_mkSharedVariableDeclaration(xx_w, myLib.mkPredefinedType(SignalOperators.opinteger), null); 
				myLib.STree_addDeclarationToProcess(vprocess, sig);

				IdentifierImpl xx_r = myLib.mkIdentifier(str1+"_r");
				sig = myLib.STree_mkSharedVariableDeclaration(xx_r, myLib.mkPredefinedType(SignalOperators.opinteger), null); 
				myLib.STree_addDeclarationToProcess(vprocess, sig);
			}
		}
	}

	/** 
	 * Translates requires subprogram access feature as outputs of vprocess's interface:
	 * event xx;
	 * 
	 * @param vprocess a Signal process representing this component (implementation)
	 * @param st a component type component
	 */
	public void ASME2SSME_ComponentInterface_SubprogramAccesses(ProcessModelImpl vprocess, ComponentType st){		
		Trace("\n ######## ASME2SSME_Translate_Common: ASME2SSME_ComponentInterface_SubprogramAccess: "+ aadlColAPI.ComponentType_GetName(st));   
		//requires subprogram access
		ArrayList<SubprogramAccess> list1 = aadlColAPI.Component_GetRequiresSubprogramAccess(st);
		if(list1!= null && list1.size() >0 ){		
			for(int m =0; m< list1.size(); m++)
			{
				SubprogramAccess sp1 = (SubprogramAccess) list1.get(m);			
				//! event xx
				SignalsDeclarationImpl vdecsig3 = ASME2SSME_RequiresSubprogramAccess_call(sp1);
				myLib.STree_addDeclarationToInterface(vprocess, vdecsig3, 3);
			}
		}
	}


	/** 
	 * Translates call subprogram sp as outputs of vprocess's interface (is sp is not a subcomponent of itself). 
	 * Each called subprogram xx: subprogram sp
	 * is translated as output signal: event xx;
	 * 
	 * @param vprocess a Signal process representing component (implementation) t
	 * @param t a component implementation (thread or subprogram impl) that contains subprogram call sequences
	 */
	public void ASME2SSME_ComponentInterface_CallSubprograms(ProcessModelImpl vprocess, BehavioredImplementation t){		

		Trace("\n ######## ASME2SSME_Translate_Common: ASME2SSME_ComponentInterface_CallSubprogram: "+ aadlColAPI.ComponentImpl_GetName(t)); 
		ArrayList<SubprogramCallSequence> callseqs = aadlColAPI.BehavioredImpl_Get_CallSequences(t);

		for(int i =0; i< callseqs.size(); i++){
			SubprogramCallSequence callseq1 = callseqs.get(i);
			ArrayList<SubprogramCall> calls = aadlColAPI.SubprogramCallSequence_Get_SubprogramCalls(callseq1);

			for(int j=0; j< calls.size(); j++){
				SubprogramCall call1 = calls.get(j);
				CalledSubprogram called_sp = aadlColAPI.SubprogramCall_Get_CalledSubprogram(call1);

				//if called_sp is a sub component of t: ? event sp1;
				if(aadlColAPI.Is_CalledSubprogram_Sub_Of_ComponentImpl(called_sp, t)){

				}
				else{
					//else (the called subprogram is from external): ! event xx;
					SignalsDeclarationImpl vdecsig3 = ASME2SSME_SubprogramCall1(call1, t);
					myLib.STree_addDeclarationToInterface(vprocess, vdecsig3, 3);
				}
			}
		}
	}


	/** 
	 * For the sub subprograms of a thread (t):  
	 * For each sub subprogram x: subprogram sp, 
	 * add into the interface: ? sp_IN sp_x; ! sp_OUT sp_y; TID sp_id;
	 * 
	 * @param vprocess a Signal process representing component (implementation) t
	 * @param t a component implementation (thread or subprogram impl)
	 */
	public void ASME2SSME_ComponentInterface_Sub_Subprograms(ProcessModelImpl vprocess, BehavioredImplementation t){

		Trace("\n ######## ASME2SSME_Translate_Common: ASME2SSME_ComponentInterface_Sub_Subprograms: "+ aadlColAPI.ComponentImpl_GetName(t)); 
		ArrayList<SubprogramSubcomponent> sub_sps = new ArrayList<SubprogramSubcomponent>();

		if(t instanceof ThreadImplementation){
			sub_sps = aadlColAPI.Thread_GetSubcomponent_Subprograms((ThreadImplementation) t);
		}

		for(int i =0; i< sub_sps.size(); i++){
			SubprogramSubcomponent sub_sp = sub_sps.get(i);
			SubprogramClassifier classifier1 =  (SubprogramClassifier) aadlColAPI.Subprogramsubcomponent_GetClassifier(sub_sp);
			String sp_name = null; //name of subprogram type or implementation

			if (classifier1 instanceof SubprogramType)
			{
				sp_name = aadlColAPI.ComponentType_GetName((SubprogramType) classifier1);                 
			}
			else if (classifier1 instanceof SubprogramImplementation){
				sp_name = aadlColAPI.ComponentImpl_GetName((SubprogramImplementation) classifier1);
			}

			String s_name = sp_name.replace(".", "_");
			//	String sn_sp = getSname( s_name);
			//    	// ? event xx;
			//		SignalsDeclarationImpl vdecsig3 = ASME2SSME_BehavioredImpl_SubprogramSubcomponent1(sub_sp, t);
			//		myLib.STree_addDeclarationToInterface(vprocess, vdecsig3, 2);

			// ? sp_IN sp_x; 
			Identifier type1 = myLib.mkIdentifier(Name_of_Subprogram_Struct_In(s_name));
			SignalsDeclarationImpl sig1 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(s_name+"_x"), type1, null); 
			myLib.STree_addDeclarationToInterface(vprocess, sig1, 2);

			// ! sp_OUT sp_y; 
			Identifier type2 = myLib.mkIdentifier(Name_of_Subprogram_Struct_Out(s_name));
			SignalsDeclarationImpl sig2 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(s_name+"_y"), type2, null); 
			myLib.STree_addDeclarationToInterface(vprocess, sig2, 3);

			// ! TID sp_id; 
			Identifier type3 = myLib.mkIdentifier("TID");
			SignalsDeclarationImpl sig3 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(s_name+"_id"), type3, null); 
			myLib.STree_addDeclarationToInterface(vprocess, sig3, 3);
		}
	}


	/**
	 * Translates flow specifications. 
	 * A flow specification is implemented as a Signal process. 
	 * It is composed of the property calls.
	 * 
	 * @param t
	 */
	public ProcessModelImpl ASME2SSME_ComponentType_FlowSpecs(ComponentType t){

		Trace("\n ######## ASME2SSME_Translate_Common: ASME2SSME_ComponentType_FlowSpecs: " + t.getName());
		ArrayList<FlowSpecification> fs_list = aadlColAPI.Component_GetFlowSpecification(t);
		if (fs_list.size() == 0) return null;
		String pname = Name_of_Flow(t.getName());
		ProcessModelImpl vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );
		ASME2SSME_Translate_Flow asme2ssme_flow = new ASME2SSME_Translate_Flow();
		for(int i=0; i<fs_list.size(); i++){
			FlowSpecification fs = fs_list.get(i);
			asme2ssme_flow.ASME2SSME_FlowSpec(vprocess, fs);
		}
		return vprocess;
	}


	/**
	 * Translates modes and mode transitions.
	 * 
	 * @param c
	 */
	public void ASME2SSME_ComponentType_Modes(ComponentType c){
		// where is it translated in Signal ? ( LB)	
		Trace("######## ASME2SSME_Translate_Common: ASME2SSME_ComponentType_Modes: " + c.getName());
		TranslateModes(aadlColAPI.Component_GetModes(c));
		TranslateModeTransitions(aadlColAPI.Component_GetModeTransitions(c));
	}

	/**
	 * Translates modes
	 * 
	 * @param c
	 */
	public void ASME2SSME_ComponentImpl_Modes(ComponentImplementation c){
		// where is it translated in Signal ? ( LB)
		Trace("######## ASME2SSME_Translate_Common: ASME2SSME_ComponentImpl_Modes: " + c.getName());
		TranslateModes(aadlColAPI.ComponentImpl_GetModes(c));  //modes 
		TranslateModeTransitions(aadlColAPI.ComponentImpl_GetModeTransitions(c));		// mode transitions
	}

	/**
	 * Translates flow implementations. 
	 * A flow is implemented as a Signal process. 
	 * It is composed of property calls
	 * 
	 * @param c
	 */
	public ProcessModelImpl ASME2SSME_ComponentImpl_Flows(ComponentImplementation c){

		ArrayList<FlowImplementation> list = aadlColAPI.ComponentImpl_GetFlowImplementations(c);
		Trace("\n ######## ASME2SSME_Translate_Common: ASME2SSME_ComponentImpl_Flows (not yet implemented): " + c.getName());
		if (list.size() == 0) return null; 
		ProcessModelImpl vprocess;
		String pname = Name_of_Flow(c.getName().replace('.','_'));
		vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );
		ASME2SSME_Translate_Flow asme2ssme_flow = new ASME2SSME_Translate_Flow();

		for(int i=0; i< list.size();i++){
			FlowImplementation fi = list.get(i);
			String vstr = aadlColAPI.FlowImplementation_GetKind(fi);
			if (vstr.equals("source")) asme2ssme_flow.ASME2SSME_FlowSourceImpl(vprocess, fi);
			else if(vstr.equals("sink")) asme2ssme_flow.ASME2SSME_FlowSinkImpl(vprocess, fi);
			else if(vstr.equals("path")) asme2ssme_flow.ASME2SSME_FlowPathImpl(vprocess, fi);
			else Trace("flow implementation kind not yet implemented.");
		}
		return vprocess;		
	}

	/**
	 * Adds an input signal to an interface of a Singal process.
	 * <p>
	 * This input signal is given by a String name, and a String type. 
	 * Creates a signal declaration by a method STree_mkSignalDeclaration(), and adds it to the process's interface by STree_addDeclarationToInterface().
	 *  
	 * @param name a String name of the input signal
	 * @param type a String type name of the input signal
	 * @param vprocess a Signal process where this input signal is to be added into
	 */
	public void ASME2SSME_add_input_to_process_interface(String name, String type, ProcessModelImpl vprocess){

		Trace("######## ASME2SSME_Translate_Common: ASME2SSME_add_input_to_process_interface :" + name);
		SignalsDeclarationImpl input_signal = mkSignalDeclaration(type, name);
		myLib.STree_addDeclarationToInterface(vprocess, input_signal, 2);
	}


	/**
	 * Adds an output signal to an interface of a Singal process.
	 * <p>
	 * This output signal is given by a String name, and a String type. 
	 * Creates a signal declaration by a method STree_mkSignalDeclaration(), and adds it to the process's interface by STree_addDeclarationToInterface().
	 *  
	 * @param name a String name of the output signal
	 * @param type a String type name of the output signal
	 * @param vprocess a Signal process where this output signal is to be added into
	 */
	public void ASME2SSME_add_output_to_process_interface(String name, String type, ProcessModelImpl vprocess){

		Trace("######## ASME2SSME_Translate_Common: ASME2SSME_add_output_to_process_interface :" + name);
		SignalsDeclarationImpl output_signal = mkSignalDeclaration(type, name);
		myLib.STree_addDeclarationToInterface(vprocess, output_signal, 3);
	}

	/**
	 * Returns a string that comments a signal
	 * 
	 * @param signal_name a string of signal name
	 * @return a string description
	 */
	public String Generate_Signal_comment(String signal_name){

		//Trace("######## ASME2SSME_Translate_Common: Generate_Signal_comment ");
		String str = "signal ";
		if (signal_name.endsWith("_ctl1"))return "bundle CTL1 (Dispatch, Resume, Deadline) for thread " + signal_name.replace("_ctl1", "");
		if (signal_name.endsWith("_ctl2")) return "bundle CTL2 (Complete, Error) for thread " + signal_name.replace("_ctl2", "");
		if (signal_name.endsWith("_time1")) return "control signal for thread " + signal_name.replace("_time1", "");
		if (signal_name.equals("ctl1")) return "control signal of bundle CTL1 (Dispatch, Resume, Deadline)";	 
		if (signal_name.equals("ctl2")) return "control signal of bundle CTL2 (Complete, Error)" ;
		if (signal_name.equals("time1")) return "control signal of bundle type";
		if(signal_name.equals("x")) return "struct signal for in parameters";
		if(signal_name.equals("y")) return "struct signal for out parameters";
		return str;
	}


	/** 
	 * Makes a process call of a component Signal process, and adds it to the upper level's Signal process's body. 
	 * <p>
	 * For each formal declaration (parameter,input,output) of vprocess, create a local signal (vprocess_input) as an effective parameter of the call.
	 * 
	 * @param vprocess a Signal process that represents an upper level component that contains the component of vprocess
	 * @param pprocess a Signal process that represents an AADL (sub) component
	 * @param label a String label
	 */
	public void ASME2SSME_component_instance(ProcessModelImpl vprocess, ProcessModelImpl pprocess, String label){
		SignalElementImpl vexpr; 
		
		Trace("\n ######## ASME2SSME_Translate_Common: ASME2SSME_component_instance : " + myLib.Identifier_GetStr(pprocess.getIdentifier()));
		String instance_name = myLib.Identifier_GetStr(myLib.Process_GetID(pprocess));

		ArrayList<ExprParam> l_param = new ArrayList<ExprParam>();   //parameter (id) list for call the process pprocess
		ArrayList<ExprSignal> l_in = new ArrayList<ExprSignal>();    //input (id) list for call the process pprocess
		ArrayList<ExprSignal> l_out = new ArrayList<ExprSignal>();   //output (id) list for call the process pprocess
		ArrayList<Declaration> vdecl = new ArrayList<Declaration>(); //local lists

		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) pprocess.getInterfaceDefinition();
		if (inter != null){

			// EList<TypesOrSignalsDeclaration> l_parameters = inter.getListFormalParameter();        
			// Iterator<TypesOrSignalsDeclaration> it_p = l_parameters.iterator();
			//   while (it_p.hasNext())
			//	 {
			//  	 SignalsDeclarationImpl aparameter = (SignalsDeclarationImpl) it_p.next(); 	
			//  	 Type type1 = aparameter.getType();
			//    	 EList<SignalNaming> list1 = aparameter.getListSignal();
			//    	   for (int i = 0; i < list1.size(); i++) {  
			//    		   SignalNaming sig_name1 = list1.get(i);
			//    		   Identifier id_sig_name1 = sig_name1.getIdentifier();
			//    		   String name1 = label + '_' + id_sig_name1.getAtomString();
			//    		   SignalsDeclarationImpl sig2 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name1), type1, null);
			//    		   l_in.add(myLib.mkIdentifier(name1));
			//    		   vdecl.add(sig2);
			//    	   }	
			//	 }
			//   
			IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
			if (vioProfile != null) 
			{ 
				EList<TypesOrSignalsDeclaration> l_inputs = vioProfile.getInputs();
				component_instance_add_Declarations(label, l_inputs, l_in, vdecl);				
				EList<TypesOrSignalsDeclaration> l_outputs = vioProfile.getOutputs(); 
				component_instance_add_Declarations(label, l_outputs, l_out, vdecl);
			}
		}
	
		if (l_out.size()>0){
			SignalsDefinitionImpl  call_eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(instance_name), l_param, l_in, l_out);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, call_eq);
			vexpr = call_eq;			 
		}
		else{
			ProcessCallImpl pcall = myLib.STree_mkCompleteProcessCall(myLib.mkIdentifier(instance_name), l_param, l_in);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, pcall);
			vexpr = pcall;			 
		}
		//add comment to this instance
		String cstr = Generate_Process_Instance_comment(instance_name);
		ArrayList<String> comments = new ArrayList<String>();
		comments.add(cstr);
		myLib.STree_addComment(vexpr, comments, false);
		myLib.STree_addLocalDeclaration(vprocess, vdecl);
		}
	/**
	 * For each formal declaration  of vlist, create an new declaration in vdecl (local signal), and the identifier is added to vpeff.
	 *  Example: integer x in  vlist, it  declares "integer <label>_x" in vdecl, and adds <label>_x to vpeff
	 * @param label
	 * @param vlist
	 * @param l_in
	 * @param vdecl
	 */
	private void component_instance_add_Declarations(String label, EList<TypesOrSignalsDeclaration> vlist, 
			ArrayList<ExprSignal> vpeff, ArrayList<Declaration> vdecl) {

		SignalsDeclarationImpl sig;
		String name;
		PredefinedTypeImpl ptype1;
		IdentifierImpl type2;
		
		Iterator <TypesOrSignalsDeclaration> it_i = vlist.iterator();
		while (it_i.hasNext())
		{
			SignalsDeclarationImpl ainput = (SignalsDeclarationImpl) it_i.next(); 
			EList<SignalNaming> list2 = ainput.getListSignal();					 
			if (ainput.getType() instanceof PredefinedTypeImpl){
				for (int i = 0; i < list2.size(); i++) {
					name = label + '_' + list2.get(i).getIdentifier().getAtomString();		    		   
					ptype1 = myLib.STree_mkSamePredefinedType(ainput);
					sig = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(getSname(name) ), ptype1, null);
					vpeff.add(myLib.mkIdentifier(getSname(name)));
					vdecl.add(sig);
				}
			}
			else{
				Identifier type_id = (Identifier) ainput.getType();
				String type_name = type_id.getAtomString();
				type2 = myLib.mkIdentifier(type_name);				 
				for (int i = 0; i < list2.size(); i++) {
					name = label + '_' + list2.get(i).getIdentifier().getAtomString();
					sig = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(getSname(name) ), type2, null);
					vpeff.add(myLib.mkIdentifier(getSname(name)));
					vdecl.add(sig);
				}
			}		    	   
		}
	}
 
	

	/** 
	 * Adds a signal declaration to a ProcessType (which represents an AADL process type component) interface.
	 * <p>
	 * This singal is given by a String name, and a String type. 
	 * Creates a signal declaration by a method STree_mkSignalDeclaration(). 
	 * Find out the process type declaration from an upper level Signal process (mmodel) by STree_GetProcessTypeDeclarationFromAProcess(). 
	 * this ProcessType is named as TT_xx, where xx is the name of a ProcessModel (vprocess).
	 * Adds this signal declaration to the process type.
	 * 
	 * @param name a String name of this signal
	 * @param type a String type name of this signal
	 * @param vprocess a Signal process that represents an AADL process implementation component
	 * @param mmodel a Signal process that represents a system implementation component
	 * @param i an integer which 1 for parameter, 2 for input, and 3 for output
	 */
	public void ASME2SSME_add_Declaration_to_ProcessType_Interface(String name, String type, ProcessModelImpl vprocess, SignalElementImpl /*ProcessModelImpl*/ mmodel, int i){

		Trace("\n ######## ASME2SSME_Translate_Common: ASME2SSME_add_Declaration_to_ProcessType_Interface : " + name);

		SignalsDeclarationImpl signal1;
		if(myLib.Is_PredefinedType(type)){
			PredefinedTypeImpl ptype = myLib.STree_mkPredefinedType(type);
			signal1 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name), ptype, null);
		}
		else {
			signal1 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name), myLib.mkIdentifier(type), null);
		}

		String process_type_name = Name_of_Type_Process(vprocess.getIdentifier().getAtomString());
		//	ProcessTypeImpl process_type = myLib.STree_GetProcessTypeDeclarationFromAProcess(mmodel, process_type_name);
		ProcessTypeImpl process_type = myLib.STree_GetProcessTypeDeclaration(mmodel, process_type_name);
		//myLib.STree_addDeclarationToInterface(vprocess, input_signal, 2);

		//add comments 
		ArrayList<String> comments = new ArrayList<String>();
		String str1;
		String str2 = name;
		if(name.endsWith("_ctl1")){
			str1 = name.replace("_ctl1","");
			str2 = "bundle for Dispatch, Resume, Deadline time events of thread " + str1;
		}
		else if(name.endsWith("_ctl2")){
			str1 = name.replace("_ctl2","");
			str2 = "bundle for Complete, Error time events of thread " + str1;
		}
		else if(name.endsWith("_time1")){
			str1 = name.replace("_time1","");
			str2 = "bundle for inputs/outputs control time events of thread " + str1;
		}
		comments.add(str2);
		myLib.STree_addComment(signal1, comments, false);
		myLib.STree_addDeclarationToProcessType(process_type, signal1, i);
	}


	/** 
	 * Adds an interface to a process (vprocess), 
	 * where the interface is an identifier with a name (id_name)
	 * 	
	 * @param vprocess a Signal process which the interface is to be added
	 * @param id_name a string name of the interface identifier
	 */
	public void ASME2SSME_add_IDInterface_to_Process(ProcessModelImpl vprocess, String id_name){

		Trace("\n ######## ASME2SSME_Translate_Common: ASME2SSME_add_IDInterface_to_Process : " + id_name);
		InterfaceDefinition inter = myLib.STree_mkIDInterface(id_name);
		vprocess.setInterfaceDefinition(inter);
	}


	/**
	 * Returns a boolean value that indicates whether a signal (named sig_name) exists in the I/O interface of a process (vprocess)
	 * 
	 * @param vprocess a Signal process
	 * @param sig_name a string name of a signal
	 * @return a boolean value
	 */
	public boolean ASME2SSME_Process_Interface_Contain_Signal(ProcessModelImpl vprocess, String sig_name){
		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) vprocess.getInterfaceDefinition();
		if (inter == null) return false;
		//EList<TypesOrSignalsDeclaration> l_parameters = inter.getListFormalParameter();        
		// Iterator<TypesOrSignalsDeclaration> it_p = l_parameters.iterator();
		IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
		if (vioProfile == null) return false;
		if (isInDeclarations(vioProfile.getInputs(), sig_name)) return true;
		return isInDeclarations(vioProfile.getOutputs(), sig_name);
	}



	/**
	 * Translates a subprogram subcomponent (x: subprogram sp) of a thread (or subprogram) t 
	 * as an input event: event x, 
	 * 
	 * @param sub_sp a subprogram subcomponent
	 */
	public SignalsDeclarationImpl ASME2SSME_BehavioredImpl_SubprogramSubcomponent1(SubprogramSubcomponent sub_sp, BehavioredImplementation t){

		Trace("\n ######## ASME2SSME_Translate_Feature: ASME2SSME_SubprogramCall1: " + sub_sp.getName());
		return myLib.mkEventDecl(sub_sp.getName());
	}

}
