
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.osate.aadl2.*;
import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.Type;
import org.eclipse.pop.ssme.impl.*;
import org.eclipse.pop.ssme.polychrony.SignalOperators;

public class ASME2SSME_Translate_Data extends ASME2SSME_Translate_Common{

	private static final String SUFF_WRITEDATA = "_w";
	private static final String SUFF_READDATA = "_r";
	private static final String DATA_ID = "data";
	private static final String SUFF_CNT = "_cnt";
	private static final String SUFF_INIT = "_INIT";
	private static final String TREAD_ID = "TRead";
	private static final String TWRITE_ID = "TWrite";
	private static final String TRESET_ID = "TReset";

	private signalTreeAPI myLib;
	private AADLCoLAPI aadlColAPI;

	private void genDeclQueue(ProcessModelImpl vprocess, IdentifierImpl d_w, IdentifierImpl d_r, Type vtype,  Type vtype2) {
		// add local declarations: shared boolean d_w, boolean d_r	
		SharedVariablesDeclarationImpl sig1 = myLib.STree_mkSharedVariableDeclaration(d_w, vtype, null); 
		myLib.STree_addDeclarationToProcess(vprocess, sig1);
		SignalsDeclarationImpl sig2 = myLib.STree_mkSignalDeclaration(d_r, vtype2, null);
		myLib.STree_addDeclarationToProcess(vprocess, sig2);
	}

	private void genDeclQueue(ProcessModelImpl vprocess, IdentifierImpl d_w, IdentifierImpl d_r, int opType) {
		genDeclQueue(vprocess, d_w, d_r, myLib.mkPredefinedType( opType ), myLib.mkPredefinedType( opType ));
	}

	private ArrayList<ExprParam> addDataParameters(ExprParam vtype, ExprParam vinit) {
		ArrayList<ExprParam> lps = new ArrayList<ExprParam>(); 
		lps.add(vtype);
		lps.add(myLib.STree_mkIntegerConstant(Integer.toString(7)));
		lps.add(vinit);
		return lps;
	}

	private ArrayList<ExprParam> addDataParameters(  int op, ExprParam vinit) {
		return addDataParameters((PredefinedTypeImpl) myLib.mkPredefinedType(op), vinit);
	}

	
	/**
	 * @param list1: list of the connections of the compiled AADL model.
	 * @param d : a shared data of of the compiled AADL model.
	 * @return the list of the references to a shared data (d), the elements of the list are selected in the operand list (list1)
	 */
		private ArrayList<IdentifierImpl> mkListAccessToData( EList<AccessConnection> list1, DataSubcomponent d) {
			if(list1.size() == 0) return null;
			ArrayList<IdentifierImpl> listx = new ArrayList<IdentifierImpl>();
			for(int i =0; i < list1.size(); i++ ){
				AccessConnection c2 = (AccessConnection) list1.get(i);

				//if a data access connection
				if(aadlColAPI.AccessConnection_GetCategory(c2) == DATA_ID){
					//if a connection d -> t.x
					if (aadlColAPI.AccessConnection_GetSrc(c2) == d) {
						if(aadlColAPI.AccessConnection_GetDstContext(c2) instanceof ThreadSubcomponent){
							ThreadSubcomponent t1 = (ThreadSubcomponent) aadlColAPI.AccessConnection_GetDstContext(c2);
							String name1 = aadlColAPI.Subcomponent_GetName(t1);
							if(aadlColAPI.AccessConnection_GetDst(c2) instanceof DataAccess){		  									  							
								DataAccess d1 = (DataAccess) aadlColAPI.AccessConnection_GetDst(c2);
								String name2 = aadlColAPI.DataAccess_GetName(d1);
								String strx = Name_of_DataBundle(name1 + "_" + name2);
								String lstrx =  getSname(strx);
								IdentifierImpl idx = myLib.mkIdentifier(lstrx);
								listx.add(idx);									 
							}
						}
					}
				}
			}
			return listx;
		}

	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_Data() {
		myLib= new signalTreeAPI();
		aadlColAPI = new AADLCoLAPI();
	}

	/** 
	 * Returns a Signal type declaration that represents an AADL data type.
	 * <p>
	 * translate an AADL data type (data xxx) to a Signal type declaration:
	 *  Type xxx = external;
	 *  
	 * @param dtype a data type component
	 * @return a type declaration, or null (when the type of dtype is  a predefined one).
	 */
	public TypesDeclarationImpl ASME2SSME_DataType(DataType dtype){

		Trace("\n ====== ASME2SSME_Translate_Data: ASME2SSME_DataType: " + dtype.getName());

		String name = aadlColAPI.ComponentType_GetName(dtype);
		String sig_name = Name_of_Data(name);
		if (myLib.Is_PredefinedType(sig_name)) return null;
		IdentifierImpl id = myLib.mkIdentifier(sig_name);		 
		// External type
		TypeDescriptionImpl type1 = new TypeDescriptionImpl();
		type1.setIdentifier(id);
		ExternalAttributeImpl externe = new ExternalAttributeImpl();
		type1.setTypeDescriptionValue(externe);
		TypesDeclarationImpl type_decl = new TypesDeclarationImpl();
		type_decl.getTypeDefinition().add(type1);
		// Features 
		ASME2SSME_Data_Features(dtype);
		// Properties 
		ASME2SSME_Data_Properties(dtype);
		myLib.STree_addComment(type_decl, "AADL data (type) " + name,true); 
		return type_decl;		
	}

	/** 
	 * Returns a constant Signal that represents an initial value of AADL data type xx
	 * constant xx xx_INIT = external;
	 * <decl_constantes> :: = "constant" <type> <l_def_constantes>
	 * <l_def_constantes> ::= <def_constante>
	 * <def_constante> ::= <id> "=" <valeur_constante>
	 * <valeur_constante> ::= <indication_externe>
	 * <p>
	 * @param dtype a data type component
	 * @return a constant	
	 */
	public ConstantsDeclarationImpl ASME2SSME_DataType_Init(DataType dtype){

		Trace("\n ====== ASME2SSME_Translate_Data: ASME2SSME_DataType_Init" + dtype.getName());
		ConstantsDeclarationImpl constant_decl = new ConstantsDeclarationImpl();
		//<id>
		String tname = aadlColAPI.ComponentType_GetName(dtype);
		IdentifierImpl id = myLib.mkIdentifier(Name_of_Data(tname) + SUFF_INIT);
		//<indication_externe>
		ExternalAttributeImpl externe = new ExternalAttributeImpl();
		//<def_constante>
		ConstantDescriptionImpl constant_descr = new ConstantDescriptionImpl();
		constant_descr.setIdentifier(id);
		constant_descr.setConstantDescriptionValue(externe);
		//<type>		
		IdentifierImpl tid = myLib.mkIdentifier(Name_of_Data(tname));
		constant_decl.setType(tid);
		//<l_def_constantes>
		constant_decl.getListConstantDefinition().add(constant_descr);
		myLib.STree_addComment(constant_decl, "Init value for AADL data (type) " + tname, true);	 
		return constant_decl; 
	}


	/** 
	 * Returns a Signal type declaration that represents an AADL data implementation 
	 * (this data implementation is not a subcomponent of any other component. It is considered as a type)
	 * <p>
	 * If no subcomponent exists, then it (data implementation xxx.ii) is implemented as a simple Signal type definition (Type xxx_ii). 
	 * <p>
	 * If there exists subcomponents, for example, 
	 * a data implementation component xxx.ii, 
	 * contains two subcomponents: (x1: data yyy; x2: data zzz;),
	 * it is translated as a struct Signal type declaration: Type xxx_ii = struct(yyy x1; zzz x2;)
	 * 
	 * @param dimpl AADL data implementation component
	 * @return a Signal type declaration
	 */
	public TypesDeclarationImpl ASME2SSME_DataImpl(DataImplementation dimpl){

		Trace("\n ====== ASME2SSME_Translate_Data: ASME2SSME_DataImpl: " + dimpl.getName());
		String name = aadlColAPI.ComponentImpl_GetName(dimpl);		
		TypesDeclarationImpl type_decl = new TypesDeclarationImpl();
		// Subcomponents 
		// data subcomponents
		ASME2SSME_DataImpl_SubData(dimpl, type_decl);
		// subprogram subcomponents
		// not exist in version 1.5.8

		// Connections 
		ASME2SSME_DataImpl_Connections(dimpl);
		// Properties 
		ASME2SSME_DataImpl_Properties(dimpl);
		myLib.STree_addComment(type_decl , "AADL data (implementation) " + name, true);	
		return type_decl;
	}


	/** 
	 * Returns a constant Signal that represents an initial value of AADL data implementation xx
	 * constant xx xx_INIT = external;
	 * <decl_constantes> :: = "constant" <type> <l_def_constantes>
	 * <l_def_constantes> ::= <def_constante>
	 * <def_constante> ::= <id> "=" <valeur_constante>
	 * <valeur_constante> ::= <indication_externe>
	 * <p>
	 * @param dimpl a data implementation component
	 * @return a constant
	 */
	public ConstantsDeclarationImpl ASME2SSME_DataImpl_Init(DataImplementation dimpl){

		Trace("\n ====== ASME2SSME_Translate_Data: ASME2SSME_DataImpl_Init" + dimpl.getName());

		ConstantsDeclarationImpl constant_decl = new ConstantsDeclarationImpl();		
		//<id>
		String tname = aadlColAPI.ComponentImpl_GetName(dimpl);
		IdentifierImpl id = myLib.mkIdentifier(Name_of_Data(tname) + SUFF_INIT);		
		//<indication_externe>
		ExternalAttributeImpl externe = new ExternalAttributeImpl();		
		//<def_constante>
		ConstantDescriptionImpl constant_descr = new ConstantDescriptionImpl();
		constant_descr.setIdentifier(id);
		constant_descr.setConstantDescriptionValue(externe);		
		//<type>		
		IdentifierImpl tid = myLib.mkIdentifier(Name_of_Data(tname));
		constant_decl.setType(tid);		
		//<l_def_constantes>
		constant_decl.getListConstantDefinition().add(constant_descr);

		myLib.STree_addComment(constant_decl, "Init value for AADL data (implementation) " + tname, false);	
		return constant_decl; 
	}



	/**
	 * Translates data features
	 * 
	 * @param dt
	 */
	public void ASME2SSME_Data_Features(DataType dt){		
		Trace("\n ====== ASME2SSME_Translate_Data: ASME2SSME_Data_Features: " + dt.getName());
		if (dt.getOwnedFeatures() != null) Trace("\n data features not yet implemented: "+ dt.getName());
	}


	/**
	 * Translates data properties
	 * 
	 * @param dt
	 */
	public void ASME2SSME_Data_Properties(DataType dt){
		Trace("\n ====== ASME2SSME_Translate_Data: ASME2SSME_Data_Properties: " + dt.getName());
		ArrayList<PropertyAssociation> list = aadlColAPI.Component_GetPropertyAssociations(dt);		
		if(list.size()>0) Trace("\n data properties not yet implemented: "+ dt.getName());

	}


	/**
	 * Translates data subcomponents of a data implementation
	 * 
	 * @param dimpl
	 * @param type_decl
	 */
	public void ASME2SSME_DataImpl_SubData(DataImplementation dimpl, TypesDeclarationImpl type_decl){
		Type vtype;

		Trace("\n ====== ASME2SSME_Translate_Data: ASME2SSME_DataImpl_SubData: " + dimpl.getName());

		String name = aadlColAPI.ComponentImpl_GetName(dimpl);
		Identifier id = myLib.mkIdentifier(Name_of_Data(name));
		TypeDescriptionImpl typedescr = new TypeDescriptionImpl();	
		ExprSignal vinit = null;	

		ArrayList<DataSubcomponent> list_sub_data = aadlColAPI.Data_GetSubcomponentsData(dimpl);
		//if no data subcomponents
		if (list_sub_data.size() == 0){
			//TypeDescriptionValue description = null;

			//external type
			TypeDescriptionImpl type1 = new TypeDescriptionImpl();
			type1.setIdentifier(id);
			ExternalAttributeImpl externe = new ExternalAttributeImpl();
			type1.setTypeDescriptionValue(externe);			
			type_decl.getTypeDefinition().add(type1);			 
		}
		else {
			TupleTypeImpl description = new TupleTypeImpl();
			description.setTypeKind(TypeKind.STRUCT_LITERAL);
			//for each data subcomponent
			Iterator<DataSubcomponent> it = list_sub_data.iterator();
			while (it.hasNext())
			{
				DataSubcomponent data_sub = (DataSubcomponent) it.next();
				String data_sub_name = aadlColAPI.Subcomponent_GetName(data_sub);
				IdentifierImpl data_sub_id = myLib.mkIdentifier(data_sub_name);
				//get the corresponding classifier
				DataSubcomponentType data_classifier = aadlColAPI.DataSubcomponent_GetDataSubcomponentType(data_sub);
				String classifier_name = "integer"; //use "integer" as default data type

				if (data_classifier instanceof DataType) classifier_name = aadlColAPI.ComponentType_GetName((DataType) data_classifier);
				else if (data_classifier instanceof DataImplementation) classifier_name = aadlColAPI.ComponentImpl_GetName((DataImplementation) data_classifier);
				else {
					Trace("data_classifier = " + data_classifier.toString());
					Trace("Data Classifier type ERROR!! Check the aaxl file!!");
					Trace("use 'integer' as default data to continue the transformation");	

					java.lang.System.err.print("\n Data Classifier type ERROR!! ");
					throw new IllegalArgumentException("\n Data Classifier type ERROR!!  \n");
				}				

				if(myLib.Is_PredefinedType(classifier_name)) 
					vtype = myLib.STree_mkPredefinedType(classifier_name);
				else 
					vtype = myLib.mkIdentifier(classifier_name);
				SignalsDeclarationImpl decl = myLib.STree_mkSignalDeclaration(data_sub_id, vtype, null); 
				description.getListSignalsDeclaration().add(decl);
			}			
			typedescr = myLib.STree_mkDclType(id, description, vinit);						
			type_decl.getTypeDefinition().add(typedescr);
		}
	}

	/**
	 * Translates connections in a data implementation
	 * 
	 * @param dd
	 */
	public void ASME2SSME_DataImpl_Connections(DataImplementation dd){

		Trace("\n ====== ASME2SSME_Translate_Data: ASME2SSME_DataImpl_Connections: " + dd.getName());
		ArrayList<Connection> list= aadlColAPI.ComponentImpl_GetConnections(dd);
		if(list.size()>0){
			Trace("\n Connections of data implementation not yet implemented: "+ dd.getName());
		}
	}


	/**
	 * Transaltes properties of a data implementation
	 * 
	 * @param dd
	 */
	public void ASME2SSME_DataImpl_Properties(DataImplementation dd){

		Trace("\n ====== ASME2SSME_Translate_Data: ASME2SSME_DataImpl_Properties: " + dd.getName());
		ArrayList<PropertyAssociation> list1 = aadlColAPI.ComponentImpl_GetPropertyAssociations(dd);
		if(list1.size()>0){
			Trace("\n Properties of data implementation not yet implemented: "+ dd.getName());
		}
	}


	/**
	 * Translates a shared data: 
	 * a data subcomponent is considered as shared data. 
	 * In this case, we suppose that the data is type integer, size 7.
	 * Creates a process call instance in the body of vprocess:
	 * (d_r, d_cnt) := fifo{integer, 7, 0}(d_w, d_reset)
	 * d_w and d_reset are added in the local as shared signals, 
	 * d_r, d_cnt is added as local integer
	 * 
	 * @param d
	 * @param vprocess
	 */
	public void ASME2SSME_CreateSharedData(DataSubcomponent d, ProcessModelImpl vprocess){

		Trace("\n ====== ASME2SSME_Translate_Data: ASME2SSME_CreateSharedData: " + d.getName());

		ASME2SSME_Translate_Data asme2ssme_data = new ASME2SSME_Translate_Data();

		ArrayList<ExprParam> lps = new ArrayList<ExprParam>();  // parameter (id) list for call the process pprocess
		ArrayList<ExprSignal> lis = new ArrayList<ExprSignal>(); // input (id) list for call the process pprocess
		ArrayList<ExprSignal> los = new ArrayList<ExprSignal>(); // output (id) list for call the process pprocess

		ComponentClassifier data_classifier = aadlColAPI.Datasubcomponent_GetClassifier(d);	    
		//input1: d_w
		String str1 = d.getName() + SUFF_WRITEDATA;
		lis.add(myLib.mkIdentifier(str1));
		IdentifierImpl d_w = myLib.mkIdentifier(str1);

		//input2: d_reset
		String str0 = Name_of_DataReset(d.getName());
		lis.add(myLib.mkIdentifier(str0));
		IdentifierImpl d_reset = myLib.mkIdentifier(str0);

		//output1: d_r
		String str2 = d.getName() + SUFF_READDATA;
		los.add(myLib.mkIdentifier(str2));
		IdentifierImpl d_r = myLib.mkIdentifier(str2);

		//output2: d_cnt
		String str3 = d.getName() + SUFF_CNT;
		los.add(myLib.mkIdentifier(str3));

		if (data_classifier instanceof DataType) //data type
		{
			String name2 = aadlColAPI.ComponentType_GetName((DataType) data_classifier);      	 
			//if the data is not a predefined type
			if(asme2ssme_data.ASME2SSME_DataType((DataType) data_classifier) != null) {
				//parameter1:TT
				IdentifierImpl id3 = myLib.mkIdentifier(Name_of_Data(name2)+ SUFF_INIT);
				lps =  addDataParameters(myLib.mkIdentifier(Name_of_Data(name2)), id3);
				genDeclQueue(vprocess, d_w, d_r,  myLib.mkIdentifier(Name_of_Data(name2)),  myLib.mkIdentifier(Name_of_Data(name2)));   	            
			}
			else{
				String strx = Name_of_Data(name2);   	    	
				if(strx.equals("boolean")){
					int op = SignalOperators.oplogical;  	    		
					ConstantLogicalImpl cl1 = new ConstantLogicalImpl();
					cl1.setConstantLogicalKind(ConstantLogicalKind.FALSE_LITERAL);
					lps = addDataParameters(op, cl1);
					genDeclQueue(vprocess, d_w, d_r, op);
				}
				else if(strx.equals("integer")){
					int op = SignalOperators.opinteger;
					lps = addDataParameters( op, myLib.STree_mkIntegerConstant(Integer.toString(0)));
					genDeclQueue(vprocess, d_w, d_r, op);      	          
				}
				else if(strx.equals("float")){
					int op = SignalOperators.opdpreal;
					lps = addDataParameters(  op, myLib.STree_mkDoubleRealConstant("0.0"));
					genDeclQueue(vprocess, d_w, d_r, op);  
				}
				else{
					Trace("shared data (predefiend) type not yet complete implemented: " + strx);        		
				}
			}
		}
		else { // data implementation
			String name1 = aadlColAPI.ComponentImpl_GetName((DataImplementation) data_classifier); 	        
			IdentifierImpl id3 = myLib.mkIdentifier(Name_of_Data(name1)+ SUFF_INIT);
			lps =  addDataParameters(  myLib.mkIdentifier(Name_of_Data(name1)), id3);
			genDeclQueue(vprocess, d_w, d_r, myLib.mkIdentifier(Name_of_Data(name1)), myLib.mkIdentifier(Name_of_Data(name1)));  
		}	
		//add: integer d_cnt as local signal
		SignalsDeclarationImpl sig3 = myLib.mkIntegerDecl (str3 );
		myLib.STree_addDeclarationToProcess(vprocess, sig3);

		//add: shared event d_reset as a local signal
		SharedVariablesDeclarationImpl sig_reset = myLib.STree_mkSharedVariableDeclaration(d_reset, myLib.mkPredefinedType(SignalOperators.opevent), null);
		myLib.STree_addDeclarationToProcess(vprocess, sig_reset);

		//add fifo process call
		SignalsDefinitionImpl call1 = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("fifo_reset"), lps, lis, los);
		myLib.STree_addComment(call1, "(shared) data subcomponent: " + d.getName(), false);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, call1);
	}




	/**
	 * For a shared data: 
	 * give the clock of cnt: 
	 * xx_cnt ^= xx_w ^+ xx_r ^+ xx_reset
	 * 
	 * @param d
	 * @param vprocess
	 */
	public void ASME2SSME_SharedData_Cnt_Sync(DataSubcomponent d, ProcessModelImpl vprocess){

		Trace("\n ====== ASME2SSME_Translate_Data: ASME2SSME_SharedData_Cnt_Sync: " + d.getName());

		String str1 = aadlColAPI.Subcomponent_GetName(d);
		ExprSignal leftExp = myLib.mkIdentifier(str1+SUFF_CNT);

		// It should be: xx_cnt ^= xx_w ^+ xx_r ^+ xx_reset
		// For the moment, we make it as: xx_cnt ^= top
		//			//xx_w ^+ xx_r
		//			ClockUnionImpl rightExp1 = new ClockUnionImpl();
		//			rightExp1.setExprSignal1(myLib.mkIdentifier(str1+SUFF_WRITEDATA));
		//			rightExp1.setExprSignal2(myLib.mkIdentifier(str1+SUFF_READDATA));
		//			
		//			//xx_w ^+ xx_r ^+ xx_reset
		//			ClockUnionImpl rightExp = new ClockUnionImpl();
		//			rightExp.setExprSignal1(rightExp1);
		//			rightExp.setExprSignal2(myLib.mkIdentifier(str1+"_reset"));
		//			
		//			//xx_cnt ^= xx_w ^+ xx_r
		//		    ClockEqualityImpl clk_eq = new ClockEqualityImpl();
		//		    clk_eq.getExprSignal().add(leftExp);
		//		    clk_eq.getExprSignal().add(rightExp);

		// xx_cnt ^= xx_w ^+ xx_r
		ClockEqualityImpl clk_eq = new ClockEqualityImpl();
		clk_eq.getExprSignal().add(leftExp);
		clk_eq.getExprSignal().add(myLib.mkIdentifier(TOP_ID));

		//add comments 
		ArrayList<String> comments1 = new ArrayList<String>();
		String str2 = "CAUTION: Clock of count of data: " + d.getName();
		comments1.add(str2);
		comments1.add("This clock should be: xx_cnt ^= xx_w ^+ xx_r ^+ xx_reset");
		comments1.add("For compilation, it is made synchronous with top");
		myLib.STree_addComment(clk_eq, comments1, false);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq);
	}

	/**
	 * For a shared data xx: 
	 * give the clock of xx_r: 
	 * xx_r ^= l_1.TRead ^+ l_2.TRead ^+ ... 
	 * where each l_ is a local signal that represents a requires data access of a thread
	 * 
	 * @param d
	 * @param vprocess
	 */
	public void ASME2SSME_SharedData_Read_Sync(DataSubcomponent d, ProcessModelImpl vprocess, ProcessImplementation pimpl){

		Trace("\n ====== ASME2SSME_Translate_Data: ASME2SSME_SharedData_Read_Sync: " + d.getName());
		String str1 = aadlColAPI.Subcomponent_GetName(d);
		ExprSignal leftExp = myLib.mkIdentifier(str1 + SUFF_READDATA);

		if (aadlColAPI.ComponentImpl_GetAccessConnections(pimpl) != null){
			EList<AccessConnection> list1 = aadlColAPI.ComponentImpl_GetAccessConnections(pimpl);
			ArrayList<IdentifierImpl> listx = new ArrayList<IdentifierImpl>();

			if(list1.size()>0){
				for(int i =0; i < list1.size(); i++ ){

					AccessConnection c2 = (AccessConnection) list1.get(i);		  				
					//if data access connection
					if(aadlColAPI.AccessConnection_GetCategory(c2) == DATA_ID){

						//if a connection d -> t.x
						if(aadlColAPI.AccessConnection_GetSrc(c2) == d){
							//Trace("access connection source == " + d.getName());
							if(aadlColAPI.AccessConnection_GetDstContext(c2) instanceof ThreadSubcomponent){
								ThreadSubcomponent t1 = (ThreadSubcomponent) aadlColAPI.AccessConnection_GetDstContext(c2);
								String name1 = aadlColAPI.Subcomponent_GetName(t1);

								if(aadlColAPI.AccessConnection_GetDst(c2) instanceof DataAccess){		  									  							
									DataAccess d1 = (DataAccess) aadlColAPI.AccessConnection_GetDst(c2);
									String name2 = aadlColAPI.DataAccess_GetName(d1);
									String strx = Name_of_DataBundle(name1+"_"+name2);
									String lstrx = getSname(strx);
									IdentifierImpl idx = myLib.mkIdentifier(lstrx);
									listx.add(idx);
								}
							}
						}
					}
				}
			}

			if (listx.size()>0){
				if(listx.size()==1){
					//ExprSignal rightExp = listx.get(0);
					ExprSignal rightExp = myLib.STree_mkFieldAccess(listx.get(0).getAtomString(), TREAD_ID);
					ClockEqualityImpl clk_eq = myLib.STree_mkClockEquality(leftExp, rightExp);
					myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq);
				}
				else{
					ClockUnionImpl tmp = new ClockUnionImpl();
					ClockUnionImpl rightExp = new ClockUnionImpl();
					tmp.setExprSignal1(myLib.STree_mkFieldAccess(listx.get(0).getAtomString(), TREAD_ID));
					rightExp.setExprSignal1(myLib.STree_mkFieldAccess(listx.get(0).getAtomString(), TREAD_ID));
					Trace("listx.get(0) = " + listx.get(0).getAtomString());

					for(int m =1; m<listx.size(); m++){
						tmp.setExprSignal2(myLib.STree_mkFieldAccess(listx.get(m).getAtomString(), TREAD_ID));
						rightExp.setExprSignal2(myLib.STree_mkFieldAccess(listx.get(m).getAtomString(), TREAD_ID));
						tmp.setExprSignal1(rightExp);
					}
					//xx_r ^= ...
					ClockEqualityImpl clk_eq = myLib.STree_mkClockEquality(leftExp, rightExp);
					myLib.STree_addComment(clk_eq, "clock of read data: " + d.getName(), false);
					myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq);
				}
			}
		}
	}



	/**
	 * For a shared data xx: 
	 * give the write clock of xx_w: 
	 * xx_w ^= l.TWrite ^+ l.TWrite ^+ ...
	 * where each l_ is a local signal that represents a write clock
	 * 
	 * @param d a shared data
	 * @param vprocess
	 */
	public void ASME2SSME_SharedData_Write_Sync(DataSubcomponent d, ProcessModelImpl vprocess, ProcessImplementation pimpl){

		Trace("\n ====== ASME2SSME_Translate_Data: ASME2SSME_SharedData_Write_Sync: " + d.getName());
		String str1 = aadlColAPI.Subcomponent_GetName(d);
		ExprSignal leftExp = myLib.mkIdentifier(str1 + SUFF_WRITEDATA);

		if (aadlColAPI.ComponentImpl_GetAccessConnections(pimpl) != null) {
			ArrayList<IdentifierImpl> listx = null;
			listx = mkListAccessToData(aadlColAPI.ComponentImpl_GetAccessConnections(pimpl), d);
			if (listx == null) return;
			if (listx.size() == 0) return;
			if(listx.size() == 1) {		    			
				ExprSignal rightExp = myLib.STree_mkFieldAccess(listx.get(0).getAtomString(), TWRITE_ID);
				ClockEqualityImpl clk_eq = myLib.STree_mkClockEquality(leftExp, rightExp);
				myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq);		    		   
			}
			else {
				ClockUnionImpl tmp = new ClockUnionImpl();
				ClockUnionImpl rightExp = new ClockUnionImpl();
				tmp.setExprSignal1(myLib.STree_mkFieldAccess(listx.get(0).getAtomString(), TWRITE_ID));
				rightExp.setExprSignal1(myLib.STree_mkFieldAccess(listx.get(0).getAtomString(), TWRITE_ID));
				for(int m =1; m<listx.size(); m++){
					tmp.setExprSignal2(myLib.STree_mkFieldAccess(listx.get(m).getAtomString(), TWRITE_ID));
					rightExp.setExprSignal2(myLib.STree_mkFieldAccess(listx.get(m).getAtomString(), TWRITE_ID));
					tmp.setExprSignal1(rightExp);
				}
				//xx_w ^= ...
				ClockEqualityImpl clk_eq = myLib.STree_mkClockEquality(leftExp, rightExp);
				myLib.STree_addComment(clk_eq, "clock of write data: " + d.getName(), false);	
				myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq);		    		   
			}
		}
	}

	/**
	 * For a shared data xx: 
	 * give the reset clock of xx_reset: 
	 * xx_reset ^= l.TReset ^+ l.TReset ^+ ... 
	 * where each l_ is a local signal that represents a reset clock
	 * 
	 * @param d
	 * @param vprocess
	 */
	public void ASME2SSME_SharedData_Reset_Sync(DataSubcomponent d, ProcessModelImpl vprocess, ProcessImplementation pimpl){

		Trace("\n ====== ASME2SSME_Translate_Data: ASME2SSME_SharedData_Reset_Sync: " + d.getName());
		String str1 = aadlColAPI.Subcomponent_GetName(d);
		ExprSignal leftExp = myLib.mkIdentifier(Name_of_DataReset(str1));

		if (aadlColAPI.ComponentImpl_GetAccessConnections(pimpl) == null) return; 

		ArrayList<IdentifierImpl> listx = null;
		listx = mkListAccessToData (aadlColAPI.ComponentImpl_GetAccessConnections(pimpl), d );

		if (listx == null) return;
		if (listx.size() == 0) return;

		ClockEqualityImpl clk_eq = new ClockEqualityImpl();

		if(listx.size()==1){		    			
			ExprSignal rightExp = myLib.STree_mkFieldAccess(listx.get(0).getAtomString(), TRESET_ID);		    		    
			clk_eq.getExprSignal().add(leftExp);
			clk_eq.getExprSignal().add(rightExp);		    			
		}
		else{
			ClockUnionImpl tmp = new ClockUnionImpl();
			ClockUnionImpl rightExp = new ClockUnionImpl();
			tmp.setExprSignal1(myLib.STree_mkFieldAccess(listx.get(0).getAtomString(), TRESET_ID));
			rightExp.setExprSignal1(myLib.STree_mkFieldAccess(listx.get(0).getAtomString(), TRESET_ID));

			for(int m =1; m<listx.size(); m++){
				tmp.setExprSignal2(myLib.STree_mkFieldAccess(listx.get(m).getAtomString(), TRESET_ID));
				rightExp.setExprSignal2(myLib.STree_mkFieldAccess(listx.get(m).getAtomString(), TRESET_ID));
				tmp.setExprSignal1(rightExp);
			}
			//xx_reset ^= ...
			clk_eq.getExprSignal().add(leftExp);
			clk_eq.getExprSignal().add(rightExp);
		}		    		 
		myLib.STree_addComment(clk_eq, "clock of reset data: " + d.getName(), false);	
		myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq);
	}
}



