 
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import org.osate.aadl2.*;
import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;


public class ASME2SSME_Translate_EventDataPort extends ASME2SSME_Translate_Port{


	private AADLCoLAPI aadlColAPI;
	private signalTreeAPI myLib;
	
	
	/**
	 * Translates some properties which are fixed for an input event data port
	 * ( Input_Time, Overflow_Handling_Protocol, Queue_Size, Queue_Processing_Protocol, Dequeued_Items, Dequeue_Protocol)
	 * @param timpl
	 * @param p
	 * @param t_property
	 */
	private void InEventDataPort_Properties(ThreadImplementation timpl, EventDataPort p, ProcessModelImpl t_property){

		Trace("\n ######## ASME2SSME_Translate_EventDataPort: InEventDataPort_Properties "+ p.getName());
			
		myLib.STree_addExpressionOfProcessToaModel(t_property, ASME2SSME_InputTime(timpl, p));
		myLib.STree_addExpressionOfProcessToaModel(t_property, ASME2SSME_Overflow_Handling_Protocol(timpl, p));
		myLib.STree_addExpressionOfProcessToaModel(t_property, ASME2SSME_Queue_Size(timpl, p) );
		myLib.STree_addExpressionOfProcessToaModel(t_property, ASME2SSME_Queue_Processing_Protocol(timpl, p) );
		myLib.STree_addExpressionOfProcessToaModel(t_property, ASME2SSME_Dequeued_Items(timpl, p) );
		myLib.STree_addExpressionOfProcessToaModel(t_property, ASME2SSME_Dequeue_Protocol(timpl, p) );
	}
 
	/**
	 * Translates some properties which are fixed for an output event data port
	 * (Output_Time, Queue_Size, Queue_Processing_Protocol)
	 * @param timpl
	 * @param p
	 * @param t_property
	 */
	private void OutEventDataPort_Properties(ThreadImplementation timpl, EventDataPort p, ProcessModelImpl t_property){

		Trace("\n ######## ASME2SSME_Translate_EventDataPort: OutEventDataPort_Properties "+ p.getName());
		myLib.STree_addExpressionOfProcessToaModel(t_property, ASME2SSME_OutputTime(timpl, p));
		myLib.STree_addExpressionOfProcessToaModel(t_property, ASME2SSME_Queue_Size(timpl, p));
		myLib.STree_addExpressionOfProcessToaModel(t_property, ASME2SSME_Queue_Processing_Protocol(timpl, p));
	}

	
	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_EventDataPort() {
		aadlColAPI = new AADLCoLAPI();
		myLib= new signalTreeAPI();
	}

	/** 
	 * Returns a Signal process that represents an AADL in data port of a thread
	 * <p>
	 * This Signal process is xx_InDataPort, where xx is the name of the data port.  
	 * It is composed of two subprocesses: xx_InDataPort_Behavior() and xx_InDataPort_Property(). 
	 * xx_InDataPort_Behavior() calls the InDataPort_Behavior() process from the predefined library. 
	 * xx_InDataPort_Property() calls the processes that represent the properties.
	 * 
	 * @param timpl thread implementation component
	 * @param dataPort an AADL data port of thread timpl
	 * @return a Signal process
	 */	
	public ProcessModelImpl ASME2SSME_Thread_InEventDataPort(ThreadImplementation timpl, EventDataPort p1){		

		String name = aadlColAPI.Port_GetName(p1);
		String pname = Name_of_ThreadEventDataPort(name, "in");
		DataClassifier data_classifier = aadlColAPI.EventDataPort_GetClassifier(p1);
		String sig_data_type = Name_of_Reference_data_classifier(data_classifier);

		Trace("\n ######## ASME2SSME_Translate_EventDataPort: ASME2SSME_Thread_InEventDataPort " +  name);			
		ProcessModelImpl vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );

		//parameters: xx def_value; integer size; integer dequeue_number;
		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
		lparams.add(myLib.mkIdentifier(DEF_VALUE_ID));
		lparams.add(myLib.mkIdentifier(SIZE_ID));
		lparams.add(myLib.mkIdentifier(DEQUEUE_NUMBER_ID));
		myLib.STree_addDeclarationToInterface(vprocess, myLib.mkDecl(DEF_VALUE_ID, sig_data_type), 1);
		myLib.STree_addDeclarationToInterface(vprocess, myLib.mkIntegerDecl(SIZE_ID), 1);
		myLib.STree_addDeclarationToInterface(vprocess, myLib.mkIntegerDecl(DEQUEUE_NUMBER_ID), 1);
	 
		//inputs: event Frozen_time_event; xx write_flow; event Reference_time_event;event Initialize_event (--removed--);
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		lins.add(myLib.mkIdentifier(FROZEN_TIME_EVENT_ID));
		lins.add(myLib.mkIdentifier(WRITE_FLOW_ID));
		lins.add(myLib.mkIdentifier(REFERENCE_TIME_EVENT_ID));
		// add inputs to interface
		myLib.STree_addDeclarationToInterface(vprocess, myLib.mkEventDecl(FROZEN_TIME_EVENT_ID), 2);
		myLib.STree_addDeclarationToInterface(vprocess, myLib.mkDecl(WRITE_FLOW_ID, sig_data_type), 2);
		myLib.STree_addDeclarationToInterface(vprocess, myLib.mkEventDecl(REFERENCE_TIME_EVENT_ID), 2);
		// SignalsDeclarationImpl Initialize_event = myLib.mkEventDecl("Initialize_event");

		// outputs: xx read_flow;
		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
		louts.add(myLib.mkIdentifier(READ_FLOW_ID));
		
		myLib.STree_addDeclarationToInterface(vprocess, myLib.mkDecl(READ_FLOW_ID, sig_data_type), 3);

		// in event data port behavior---------------------------------------------//
		// process declaration
		ProcessModelImpl t_behavior = ASME2SSME_Thread_InEventDataPort_behavior(pname);
		String behavior_process_name = myLib.STree_GetStringName(t_behavior.getIdentifier()); 

		//add to declaration
		myLib.STree_addProcessDeclaration(vprocess, t_behavior);
		//add a behavior instance to body
		IdentifierImpl id = (IdentifierImpl) myLib.mkIdentifier(behavior_process_name);	
		ArrayList<ExprParam> lparams2 = new ArrayList<ExprParam>();
		lparams2.addAll(lparams);	
		StringValueImpl sv = myLib.STree_mkStringValue(name);
		lparams2.add(sv);

		ArrayList<ExprSignal> lins2 = new ArrayList<ExprSignal>();
		lins2.add(myLib.mkIdentifier(WRITE_FLOW_ID));
		lins2.add(myLib.mkIdentifier(FROZEN_TIME_EVENT_ID));
		
		SignalsDefinitionImpl eq_behavior_call = myLib.STree_mkCompleteProcessCallEquation(id, lparams2, lins2, louts);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, eq_behavior_call);
		myLib.STree_addComment(eq_behavior_call, "behavior sub process for in event data port "+ name, false);
		// in event data port PROPERTY
		// process declaration
		ProcessModelImpl t_property = ASME2SSME_Thread_Port_property(timpl, p1);
		//add fix number in event data port properties
		InEventDataPort_Properties(timpl, p1, t_property);
		String property_name = myLib.STree_GetStringName(t_property.getIdentifier());

		// add to local declaration
		myLib.STree_addProcessDeclaration(vprocess, t_property);
		//add an instance to body
		ArrayList<ExprParam> lparam3 = new ArrayList<ExprParam>();
		ArrayList<ExprSignal> lin3 = new ArrayList<ExprSignal>();
		lin3.add(myLib.mkIdentifier(FROZEN_TIME_EVENT_ID));
		lin3.add(myLib.mkIdentifier(REFERENCE_TIME_EVENT_ID));
		ProcessCallImpl property_inst = myLib.STree_mkCompleteProcessCall(myLib.mkIdentifier(property_name),lparam3, lin3);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, property_inst);
		myLib.STree_addComment(property_inst, "property sub process for in event data port ", false);

		TypesDeclarationImpl type_decl = myLib.STree_mkTypeDeclaration(myLib.mkIdentifier(MSG_TYPE_ID), myLib.mkIdentifier(sig_data_type));
		myLib.STree_addaDeclarationToProcess(vprocess, type_decl);
		Trace("\n ######################### end ASME2SSME_Thread_InEventDataPort " +  name + " ######################### \n");			
		return vprocess;
	}


	/**
	 * Returns a Signal process "xx_InEventDataPort_behavior", which is a subprocess of thread in event data port
	 * <p>
	 * This Signal process is named as xx_InEventDataPort_behavior, where xx is the event data port name. 
	 * The interface contains parameters: {type item_type;} 
	 *<p>
	 * inputs: ? event Frozen_time_event; item_type write_flow; event Reference_time_event; 
	 * outputs: ! item_type read_flow; 
	 * The body calls the InDataPort_behavior() process which is defined in our Signal library. 
	 * 
	 * @param t_name a String name of thread in data port Signal process
	 * @return a Signal process
	 */
	public ProcessModelImpl ASME2SSME_Thread_InEventDataPort_behavior(String t_name){

		// process xx_InEventDataPort_behavior =
		//	  { msg_type def_value; integer size; integer dequeue_number; string port_name;}
		//	  (? msg_type write_flow;  event Frozen_time_event; event Initialize_time;
		//	  ! msg_type read_flow;)
		//	  (|read_flow := InEventDataPort_Behavior{msg_type, def_value, size, dequeue_number, port_name}(write_flow,Frozen_time_event) 
		//	   |)

		Trace("\n ######## ASME2SSME_Translate_EventDataPort: ASME2SSME_Thread_InEventDataPort_behavior : " + t_name);

		String pname =  t_name + "_behavior";
		ProcessModelImpl t_behavior = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );

		//parameters: msg_type, default_value, size, dequeue_number, xx
		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
		lparams.add(myLib.mkIdentifier(MSG_TYPE_ID));
		lparams.add(myLib.mkIdentifier(DEF_VALUE_ID));
		lparams.add(myLib.mkIdentifier(SIZE_ID));
		lparams.add(myLib.mkIdentifier(DEQUEUE_NUMBER_ID));
		lparams.add(myLib.mkIdentifier(PORT_NAME_ID));
		
		myLib.STree_addDeclarationToInterface(t_behavior, myLib.mkDecl(DEF_VALUE_ID, MSG_TYPE_ID), 1);
		myLib.STree_addDeclarationToInterface(t_behavior, myLib.mkIntegerDecl(SIZE_ID), 1);
		myLib.STree_addDeclarationToInterface(t_behavior, myLib.mkIntegerDecl(DEQUEUE_NUMBER_ID), 1);
		myLib.STree_addDeclarationToInterface(t_behavior, myLib.mkStringDecl(PORT_NAME_ID), 1);

		//inputs:  msg_type write_flow;event Frozen_time_event;
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		lins.add(myLib.mkIdentifier(WRITE_FLOW_ID));
		lins.add(myLib.mkIdentifier(FROZEN_TIME_EVENT_ID));
		// add inputs to interface
		myLib.STree_addDeclarationToInterface(t_behavior, myLib.mkDecl(WRITE_FLOW_ID, MSG_TYPE_ID ), 2);
		myLib.STree_addDeclarationToInterface(t_behavior, myLib.mkEventDecl(FROZEN_TIME_EVENT_ID), 2);
		// event Initialize_time;
		// lins.add(myLib.mkIdentifier("Initialize_time"));
		// myLib.STree_addDeclarationToInterface(t_behavior, myLib.mkEventDecl("Initialize_time"), 2);

		//outputs: msg_type read_flow;
		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
		louts.add(myLib.mkIdentifier(READ_FLOW_ID));
		// add outputs to interface
		myLib.STree_addDeclarationToInterface(t_behavior, myLib.mkDecl( READ_FLOW_ID, MSG_TYPE_ID), 3);

		//body: call InEventDataPort_Behavior() process--------------------------//
		SignalsDefinitionImpl eq1 = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("InEventDataPort_Behavior"), lparams, lins, louts);
		myLib.STree_addExpressionOfProcessToaModel(t_behavior, eq1);
		return t_behavior;
	}


	/**
	 * Translates out event data port of a thread
	 * 
	 * @param timpl
	 * @param p1
	 * @return
	 */
	public ProcessModelImpl ASME2SSME_Thread_OutEventDataPort(ThreadImplementation timpl, EventDataPort p1){		

		Trace("\n ######## ASME2SSME_Translate_EventDataPort: ASME2SSME_Thread_OutEventDataPort " + p1.getName());
		String name = aadlColAPI.Port_GetName(p1);
		String pname = Name_of_ThreadEventDataPort(name, "out");
		DataClassifier data_classifier = (DataClassifier) aadlColAPI.EventDataPort_GetClassifier(p1);
		String sig_data_type = Name_of_Reference_data_classifier(data_classifier);

		Trace("=========ASME2SSME_Thread_OutEventDataPort : " + name);

		ProcessModelImpl vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );
		// parameters
		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
		lparams.add(myLib.mkIdentifier(DEF_VALUE_ID));
		lparams.add(myLib.mkIdentifier(SIZE_ID));
		// add parameters to interface
		myLib.STree_addDeclarationToInterface(vprocess, myLib.mkDecl(DEF_VALUE_ID, sig_data_type), 1);
		myLib.STree_addDeclarationToInterface(vprocess, myLib.mkIntegerDecl(SIZE_ID), 1);
	 
		//inputs: event Output_time_event; xx write_flow; event Reference_time_event;
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		lins.add(myLib.mkIdentifier(OUTPUT_TIME_EVENT_ID));
		lins.add(myLib.mkIdentifier(WRITE_FLOW_ID));
		lins.add(myLib.mkIdentifier(REFERENCE_TIME_EVENT_ID));
		
		myLib.STree_addDeclarationToInterface(vprocess, myLib.mkEventDecl(OUTPUT_TIME_EVENT_ID), 2);
		myLib.STree_addDeclarationToInterface(vprocess, myLib.mkDecl(WRITE_FLOW_ID, sig_data_type), 2);
		myLib.STree_addDeclarationToInterface(vprocess, myLib.mkEventDecl(REFERENCE_TIME_EVENT_ID), 2);
		// myLib.STree_addDeclarationToInterface(vprocess, myLib.mkEventDecl("Initialize_time"), 2);
		// lins.add(myLib.mkIdentifier("Initialize_time"));
		
		//outputs: xx sent_flow;
		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
		louts.add(myLib.mkIdentifier(SENT_FLOW_ID));
		myLib.STree_addDeclarationToInterface(vprocess, myLib.mkDecl(SENT_FLOW_ID, sig_data_type), 3);

		// out event data port behavior
		ProcessModelImpl t_behavior = ASME2SSME_Thread_OutEventDataPort_behavior(pname);
		String threadoutdataport_behavior_name = myLib.STree_GetStringName(t_behavior.getIdentifier());
		myLib.STree_addProcessDeclaration(vprocess, t_behavior);
		//add a behavior instance to body		
		IdentifierImpl id = (IdentifierImpl) myLib.mkIdentifier(threadoutdataport_behavior_name);

		ArrayList<ExprParam> lparams2 = new ArrayList<ExprParam>();
		lparams2.addAll(lparams);	
		StringValueImpl sv = myLib.STree_mkStringValue(name);
		lparams2.add(sv);

		ArrayList<ExprSignal> lins2 = new ArrayList<ExprSignal>();
		lins2.add(myLib.mkIdentifier(WRITE_FLOW_ID));
		lins2.add(myLib.mkIdentifier(OUTPUT_TIME_EVENT_ID));
		// lins2.add(myLib.mkIdentifier("Initialize_time"));
		SignalsDefinitionImpl eq_behavior_call = myLib.STree_mkCompleteProcessCallEquation(id, lparams2, lins2, louts);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, eq_behavior_call);
		myLib.STree_addComment(eq_behavior_call, "behavior sub process instance for out event data port "+ pname, false);

		//out event data port property
		ProcessModelImpl t_property = ASME2SSME_Thread_Port_property(timpl, p1);
		// add fix number out event data port properties
		OutEventDataPort_Properties(timpl, p1, t_property);

		String property_name = myLib.STree_GetStringName(t_property.getIdentifier());
		//add to declaration
		myLib.STree_addProcessDeclaration(vprocess, t_property);
		//add to body
		//ProcessCallImpl property_inst = myLib.STree_mkModelCall(thread_property_name);
		ArrayList<ExprParam> lparam3 = new ArrayList<ExprParam>();
		ArrayList<ExprSignal> lin3 = new ArrayList<ExprSignal>();
		lin3.add(myLib.mkIdentifier(OUTPUT_TIME_EVENT_ID));
		lin3.add(myLib.mkIdentifier(REFERENCE_TIME_EVENT_ID));
		ProcessCallImpl property_inst = myLib.STree_mkCompleteProcessCall(myLib.mkIdentifier(property_name),lparam3, lin3);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, property_inst);
		myLib.STree_addComment(property_inst, "property sub process instance of out event data port "+ pname, false);
		TypesDeclarationImpl type_decl = myLib.STree_mkTypeDeclaration(myLib.mkIdentifier(MSG_TYPE_ID), myLib.mkIdentifier(sig_data_type));
		myLib.STree_addaDeclarationToProcess(vprocess, type_decl);

		Trace("\n ######################### end ASME2SSME_Thread_OutEventDataPort " + p1.getName() + " ######################### \n");
		return vprocess;
	}


	/**
	 * Returns a Signal process "xx_OutEventDataPort_behavior", which is a subprocess of thread out event data port
	 * <p>
	 * This Signal process is named as xx_OutEventDataPort_behavior, where xx is the event data port name. 
	 * The interface contains parameters: {msg_type def_msg; integer size; string port_name;} 
	 *<p>
	 * inputs: ? msg_type write_flow; event Output_time_event; 
	 * outputs: ! msg_type sent_flow; 
	 * The body calls the OutEventDataPort_behavior() process which is defined in our Signal library. 
	 * 
	 * @param t_name a String name of thread in data port Signal process
	 * @return a Signal process
	 */
	public ProcessModelImpl ASME2SSME_Thread_OutEventDataPort_behavior(String t_name){

		//	 process xx_InEventDataPort_behavior =
		//	  {msg_type def_value; integer size; string port_name;}
		//	 (? msg_type write_flow; event Output_time_event; event Initialize_time;
		//	  ! msg_type sent_flow;)
		//	 (|sent_flow := OutEventDataPort_Behavior{msg_type, def_value, size, port_name}(write_flow,Output_time_event, Initialize_time) 
		//	  |)

		Trace("\n ######## ASME2SSME_Translate_EventDataPort: ASME2SSME_Thread_OutEventDataPort_behavior : " + t_name );

		String pname =  t_name + "_behavior";
		ProcessModelImpl t_behavior = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );

		//parameters------------------------------------------------------------------------//
		//  msg def_value; integer size;string port_name;
		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
		lparams.add(myLib.mkIdentifier(MSG_TYPE_ID));
		lparams.add(myLib.mkIdentifier(DEF_VALUE_ID));
		lparams.add(myLib.mkIdentifier(SIZE_ID));
		lparams.add(myLib.mkIdentifier(PORT_NAME_ID));
		myLib.STree_addDeclarationToInterface(t_behavior, myLib.mkDecl(DEF_VALUE_ID, MSG_TYPE_ID), 1);
		myLib.STree_addDeclarationToInterface(t_behavior,  myLib.mkIntegerDecl(SIZE_ID), 1);
		myLib.STree_addDeclarationToInterface(t_behavior, myLib.mkStringDecl(PORT_NAME_ID), 1);

		//inputs
		//msg_type write_flow; event Output_time_event; event Initialize_time (removed!);
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		lins.add(myLib.mkIdentifier(WRITE_FLOW_ID));
		lins.add(myLib.mkIdentifier(OUTPUT_TIME_EVENT_ID));
		// lins.add(myLib.mkIdentifier("Initialize_time"));
		myLib.STree_addDeclarationToInterface(t_behavior, myLib.mkDecl(WRITE_FLOW_ID, MSG_TYPE_ID), 2);
		myLib.STree_addDeclarationToInterface(t_behavior,  myLib.mkEventDecl(OUTPUT_TIME_EVENT_ID), 2);
		// myLib.STree_addDeclarationToInterface(t_behavior, myLib.mkEventDecl("Initialize_time"), 2);

		//outputs 
		// msg_type sent_flow;
		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
		louts.add(myLib.mkIdentifier(SENT_FLOW_ID));
		myLib.STree_addDeclarationToInterface(t_behavior, myLib.mkDecl(SENT_FLOW_ID, MSG_TYPE_ID), 3);

		SignalsDefinitionImpl eq1 = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("OutEventDataPort_Behavior"), lparams, lins, louts);
		myLib.STree_addExpressionOfProcessToaModel(t_behavior, eq1);

		return t_behavior;
	}




}
