
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import org.osate.aadl2.*;

import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;

public class ASME2SSME_Translate_EventPort extends ASME2SSME_Translate_Port{

	private signalTreeAPI myLib ;
	private AADLCoLAPI aadlColAPI  ;
	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_EventPort() {
		  myLib= new signalTreeAPI();
		  aadlColAPI = new AADLCoLAPI();
	}


	/** 
	 * Returns a Signal process that represents an AADL in event port of a thread
	 * <p>
	 * This Signal process is xx_InEventPort, where xx is the name of the event port.  
	 * It is composed of two subprocesses: xx_InEventPort_Behavior() and xx_InEventPort_Property(). 
	 * xx_InEventPort_Behavior() calls the InEventPort_Behavior() process from the predefined library. 
	 * xx_InEventPort_Property() calls the processes that represent the properties.
	 * 
	 * @param timpl thread implementation component
	 * @param p1 an AADL event port of thread timpl
	 * @return a Signal process
	 */	
	public ProcessModelImpl ASME2SSME_Thread_InEventPort(ThreadImplementation timpl, EventPort p1){		

		

		Trace("\n ######## ASME2SSME_Translate_EventPort: ASME2SSME_Thread_InEventPort " + p1.getName());

		String name = aadlColAPI.Port_GetName(p1);
		String pname = Name_of_ThreadEventPort(name, "in");

		//Trace("=========ASME2SSME_Thread_InEventPort : " + name);			
		ProcessModelImpl vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );

		//parameters integer size;	integer dequeue_number;	
		SignalsDeclarationImpl size = myLib.mkIntegerDecl(SIZE_ID);
		SignalsDeclarationImpl dequeue_number = myLib.mkIntegerDecl(DEQUEUE_NUMBER_ID);
		//add parameters to interface
		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
		lparams.add(myLib.mkIdentifier(SIZE_ID));
		myLib.STree_addDeclarationToInterface(vprocess, size, 1);
		lparams.add(myLib.mkIdentifier(DEQUEUE_NUMBER_ID));
		myLib.STree_addDeclarationToInterface(vprocess, dequeue_number, 1);

		// inputs: event Frozen_time_event;event write_flow;event Reference_time_event; (removed :event Initialize_time;)
		SignalsDeclarationImpl Frozen_time_event = myLib.mkEventDecl(FROZEN_TIME_EVENT_ID);
		SignalsDeclarationImpl write_flow = myLib.mkBooleanDecl(WRITE_FLOW_ID);
		SignalsDeclarationImpl Reference_time_event = myLib.mkEventDecl(REFERENCE_TIME_EVENT_ID);
		// SignalsDeclarationImpl Initialize_time = myLib.mkEventDecl("Initialize_time" );
		// add inputs to interface
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		lins.add(myLib.mkIdentifier(FROZEN_TIME_EVENT_ID));
		lins.add(myLib.mkIdentifier(WRITE_FLOW_ID));
		lins.add(myLib.mkIdentifier(REFERENCE_TIME_EVENT_ID));
		//lins.add(myLib.mkIdentifier("Initialize_time"));
		myLib.STree_addDeclarationToInterface(vprocess, Frozen_time_event, 2);
		myLib.STree_addDeclarationToInterface(vprocess, write_flow, 2);
		myLib.STree_addDeclarationToInterface(vprocess, Reference_time_event, 2);
		// myLib.STree_addDeclarationToInterface(vprocess, Initialize_time, 2);

		//outputs: event read_flow;
		SignalsDeclarationImpl read_flow = myLib.mkBooleanDecl(READ_FLOW_ID);
		// add outputs to interface
		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
		louts.add(myLib.mkIdentifier(READ_FLOW_ID));
		myLib.STree_addDeclarationToInterface(vprocess, read_flow, 3);

		//in event data port behavior 
		//process declaration
		ProcessModelImpl t_behavior = ASME2SSME_Thread_InEventPort_behavior(pname);
		String behavior_process_name = myLib.STree_GetStringName(t_behavior.getIdentifier());
		//Trace(threadindataport_behavior_name);

		//add to declaration
		myLib.STree_addProcessDeclaration(vprocess, t_behavior);

		//add a behavior instance to body
		IdentifierImpl id = (IdentifierImpl) myLib.mkIdentifier(behavior_process_name);	
		ArrayList<ExprParam> lparams2 = new ArrayList<ExprParam>();
		lparams2.addAll(lparams);	
		StringValueImpl sv = myLib.STree_mkStringValue(name);
		lparams2.add(sv);

		ArrayList<ExprSignal> lins2 = new ArrayList<ExprSignal>();
		lins2.add(myLib.mkIdentifier(WRITE_FLOW_ID));
		lins2.add(myLib.mkIdentifier(FROZEN_TIME_EVENT_ID));
		//lins2.add(myLib.mkIdentifier("Initialize_time"));
		SignalsDefinitionImpl eq_behavior_call = myLib.STree_mkCompleteProcessCallEquation(id, lparams2, lins2, louts);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, eq_behavior_call);
		myLib.STree_addComment(eq_behavior_call, "behavior sub process for in event port "+ name, false);

		//in event port property: process declaration

		//in event port property
		ProcessModelImpl t_property = ASME2SSME_Thread_Port_property(timpl, p1);
		//add fix number in data port properties
		ASME2SSME_Thread_EventPort_FixProperties(timpl, p1, t_property);

		String property_name = myLib.STree_GetStringName(t_property.getIdentifier());

		//add to local declaration
		myLib.STree_addProcessDeclaration(vprocess, t_property);
		//add an instance to body
		ArrayList<ExprParam> lparam3 = new ArrayList<ExprParam>();
		ArrayList<ExprSignal> lin3 = new ArrayList<ExprSignal>();
		lin3.add(myLib.mkIdentifier(FROZEN_TIME_EVENT_ID));
		lin3.add(myLib.mkIdentifier(REFERENCE_TIME_EVENT_ID));
		ProcessCallImpl property_inst = myLib.STree_mkCompleteProcessCall(myLib.mkIdentifier(property_name),lparam3, lin3);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, property_inst);
		myLib.STree_addComment(property_inst, "property sub process for in event port "+ name, false);

		//type msy_type = boolean;
		TypesDeclarationImpl type_decl = myLib.STree_mkTypeDeclaration(myLib.mkIdentifier("msg_type"), myLib.STree_mkPredefinedType("boolean"));
		myLib.STree_addaDeclarationToProcess(vprocess, type_decl);

		Trace("\n ######################### end ASME2SSME_Thread_InEventPort " + p1.getName() + " ######################### \n");
		return vprocess;
	}


	/**
	 * Returns a Signal process "xx_InEventPort_behavior", which is a subprocess of thread in event port
	 * <p>
	 * This Signal process is named as xx_InEventPort_behavior, where xx is the event port name. 
	 * The interface contains parameters: {integer size, dequeue_number; string port_name;} 
	 *<p>
	 * inputs: ? event write_flow; event Frozen_time_event; 
	 * outputs: ! event read_flow; 
	 * The body calls the InEventPort_Behavior() process which is defined in our Signal library. 
	 * 
	 * @param t_name a String name of thread in event port Signal process
	 * @return a Signal process
	 */
	public ProcessModelImpl ASME2SSME_Thread_InEventPort_behavior(String t_name){

		//		* process xx_InEventPort_behavior =
		//			 * {integer size;
		//           *  integer dequeue_number;
		//           *  string port_name;}
		//			 * (? event write_flow;
		//			 *    event Frozen_time_event;
		//           *    //event Initialize_time;
		//			 *  ! event read_flow;)
		//			 *  (|read_flow := InEventPort_Behavior{size, dequeue_number, port_name}(write_flow,Frozen_time_event, Initialize_time) 
		//		         |)

		Trace("\n ######## ASME2SSME_Translate_EventPort: ASME2SSME_Thread_InEventPort_behavior : " + t_name);

		String pname =  t_name + "_behavior";
		ProcessModelImpl t_behavior = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );
		//Trace(t_behavior.getIdentifier().getAtomString());

		//parameters: integer size;integer dequeue_number; string port_name;
		SignalsDeclarationImpl size = myLib.mkIntegerDecl(SIZE_ID);
		SignalsDeclarationImpl dequeue_number = myLib.mkIntegerDecl(DEQUEUE_NUMBER_ID);
		SignalsDeclarationImpl port_name = myLib.mkStringDecl(PORT_NAME_ID);

		//add parameters to interface
		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
		lparams.add(myLib.mkIdentifier(SIZE_ID));
		myLib.STree_addDeclarationToInterface(t_behavior, size, 1);
		lparams.add(myLib.mkIdentifier(DEQUEUE_NUMBER_ID));
		myLib.STree_addDeclarationToInterface(t_behavior, dequeue_number, 1);
		lparams.add(myLib.mkIdentifier(PORT_NAME_ID));
		myLib.STree_addDeclarationToInterface(t_behavior, port_name, 1);

		//inputs : event write_flow;event Frozen_time_event; (Removed :event Initialize_time;)
		SignalsDeclarationImpl write_flow = myLib.mkBooleanDecl(WRITE_FLOW_ID);
		SignalsDeclarationImpl Frozen_time_event = myLib.mkEventDecl(FROZEN_TIME_EVENT_ID);
		// SignalsDeclarationImpl Initialize_time = myLib.mkEventDecl("Initialize_time");

		// add inputs to interface
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		lins.add(myLib.mkIdentifier(WRITE_FLOW_ID));
		lins.add(myLib.mkIdentifier(FROZEN_TIME_EVENT_ID));
		//lins.add(myLib.mkIdentifier("Initialize_time"));
		myLib.STree_addDeclarationToInterface(t_behavior, write_flow, 2);
		myLib.STree_addDeclarationToInterface(t_behavior, Frozen_time_event, 2);
		// myLib.STree_addDeclarationToInterface(t_behavior, Initialize_time, 2);

		//outputs: event read_flow;
		SignalsDeclarationImpl read_flow = myLib.mkBooleanDecl(READ_FLOW_ID);
		// add outputs to interface
		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
		louts.add(myLib.mkIdentifier(READ_FLOW_ID));
		myLib.STree_addDeclarationToInterface(t_behavior, read_flow, 3);

		//body: call InEventDataPort_Behavior() 
		SignalsDefinitionImpl eq1 = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("InEventPort_Behavior"), lparams, lins, louts);
		myLib.STree_addExpressionOfProcessToaModel(t_behavior, eq1);

		//Trace(t_behavior.getIdentifier().getAtomString());
		return t_behavior;
	}


	/**
	 * Translates out event port of a thread
	 * 
	 * @param timpl
	 * @param p1
	 * @return
	 */
	public ProcessModelImpl ASME2SSME_Thread_OutEventPort(ThreadImplementation timpl, EventPort p1){		

		String name = aadlColAPI.Port_GetName(p1);
		String pname = Name_of_ThreadEventPort(name, "out");

		Trace("\n ######## ASME2SSME_Translate_EventPort: ASME2SSME_Thread_OutEventPort " + name);

		ProcessModelImpl vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );

		//parameters------------------------------------------------------------------------//
		// parameter1: integer size;
		SignalsDeclarationImpl size = myLib.mkIntegerDecl(SIZE_ID);
		//add parameters to interface
		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();				
		lparams.add(myLib.mkIdentifier(SIZE_ID));
		myLib.STree_addDeclarationToInterface(vprocess, size, 1);

		//inputs: event Output_time_event;event write_flow;event Reference_time_event; (removed:event Initialize_time;)
		SignalsDeclarationImpl Output_time_event = myLib.mkEventDecl(OUTPUT_TIME_EVENT_ID);
		SignalsDeclarationImpl write_flow = myLib.mkBooleanDecl( WRITE_FLOW_ID);
		SignalsDeclarationImpl Reference_time_event = myLib.mkEventDecl(REFERENCE_TIME_EVENT_ID);
		// SignalsDeclarationImpl Initialize_time = myLib.mkEventDecl("Initialize_time");
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		lins.add(myLib.mkIdentifier(OUTPUT_TIME_EVENT_ID));
		lins.add(myLib.mkIdentifier(WRITE_FLOW_ID));
		lins.add(myLib.mkIdentifier(REFERENCE_TIME_EVENT_ID));
		//		lins.add(myLib.mkIdentifier("Initialize_time"));
		myLib.STree_addDeclarationToInterface(vprocess, Output_time_event, 2);
		myLib.STree_addDeclarationToInterface(vprocess, write_flow, 2);
		myLib.STree_addDeclarationToInterface(vprocess, Reference_time_event, 2);
		//		myLib.STree_addDeclarationToInterface(vprocess, Initialize_time, 2);

		//outputs: event sent_flow;
		SignalsDeclarationImpl sent_flow = myLib.mkBooleanDecl(SENT_FLOW_ID);
		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
		louts.add(myLib.mkIdentifier(SENT_FLOW_ID));
		myLib.STree_addDeclarationToInterface(vprocess, sent_flow, 3);

		//out event port behavior
		ProcessModelImpl t_behavior = ASME2SSME_Thread_OutEventPort_behavior(pname);
		String threadoutdataport_behavior_name = myLib.STree_GetStringName(t_behavior.getIdentifier());//pname + "_behavior";
		myLib.STree_addProcessDeclaration(vprocess, t_behavior);

		//add a behavior instance to body		
		IdentifierImpl id = (IdentifierImpl) myLib.mkIdentifier(threadoutdataport_behavior_name);

		ArrayList<ExprParam> lparams2 = new ArrayList<ExprParam>();
		lparams2.addAll(lparams);	
		StringValueImpl sv = myLib.STree_mkStringValue(name);
		lparams2.add(sv);

		ArrayList<ExprSignal> lins2 = new ArrayList<ExprSignal>();
		lins2.add(myLib.mkIdentifier(WRITE_FLOW_ID));
		lins2.add(myLib.mkIdentifier(OUTPUT_TIME_EVENT_ID));
		//		lins2.add(myLib.mkIdentifier("Initialize_time"));
		SignalsDefinitionImpl eq_behavior_call = myLib.STree_mkCompleteProcessCallEquation(id, lparams2, lins2, louts);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, eq_behavior_call);
		myLib.STree_addComment(eq_behavior_call, "behavior sub process instance for out event port "+ pname, false);

		//out event data port property
		ProcessModelImpl t_property = ASME2SSME_Thread_Port_property(timpl,p1);
		//add fix number out data port properties
		ASME2SSME_Thread_EventPort_FixProperties(timpl, p1, t_property);

		String property_name = myLib.STree_GetStringName(t_property.getIdentifier());//pname + "_behavior";
		//add to declaration
		myLib.STree_addProcessDeclaration(vprocess, t_property);
		//add to body
		//ProcessCallImpl property_inst = myLib.STree_mkModelCall(thread_property_name);
		ArrayList<ExprParam> lparam3 = new ArrayList<ExprParam>();
		ArrayList<ExprSignal> lin3 = new ArrayList<ExprSignal>();
		lin3.add(myLib.mkIdentifier(OUTPUT_TIME_EVENT_ID));
		lin3.add(myLib.mkIdentifier(REFERENCE_TIME_EVENT_ID));
		ProcessCallImpl property_inst = myLib.STree_mkCompleteProcessCall(myLib.mkIdentifier(property_name),lparam3, lin3);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, property_inst);
		myLib.STree_addComment(property_inst, "property sub process instance of out event data port "+ pname, false);
		//type msy_type = boolean;
		TypesDeclarationImpl type_decl = myLib.STree_mkTypeDeclaration(myLib.mkIdentifier("msg_type"), myLib.STree_mkPredefinedType("boolean"));
		myLib.STree_addaDeclarationToProcess(vprocess, type_decl);

		Trace("\n ######################### end ASME2SSME_Thread_OutEventPort " + name + " ######################### \n");

		return vprocess;
	}


	/**
	 * Returns a Signal process "xx_OutEventPort_behavior", which is a subprocess of thread out event port
	 * <p>
	 * This Signal process is named as xx_OutEventPort_behavior, where xx is the event port name. 
	 * The interface contains parameters: {integer size; string port_name;} 
	 *<p>
	 * inputs: ? event write_flow; event Output_time_event; 
	 * outputs: ! event sent_flow; 
	 * The body calls the OutEventPort_Behavior() process which is defined in our Signal library. 
	 * 
	 * @param t_name a String name of thread out event port Signal process
	 * @return a Signal process
	 */
	public ProcessModelImpl ASME2SSME_Thread_OutEventPort_behavior(String t_name){

		//		* process xx_OutEventPort_behavior =
		//			 * {integer size;
		//           *  string port_name;}
		//			 * (? event write_flow;
		//			 *    event Output_time_event;
		//           *    //event Initialize_time;
		//			 *  ! event sent_flow;)
		//			 *  (|sent_flow := OutEventPort_Behavior{size, port_name}(write_flow,Output_time_event, Initialize_time) 
		//		         |)

		Trace("\n ######## ASME2SSME_Translate_EventPort: ASME2SSME_Thread_OutEventPort_behavior : " + t_name);

		String pname =  t_name + "_behavior";
		ProcessModelImpl t_behavior = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );
		//Trace(t_behavior.getIdentifier().getAtomString());

		//parameters------------------------------------------------------------------------//
		// parameter1: integer size;string port_name;
		SignalsDeclarationImpl size = myLib.mkIntegerDecl(SIZE_ID);
		SignalsDeclarationImpl port_name = myLib.mkStringDecl(PORT_NAME_ID);

		//add parameters to interface
		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
		lparams.add(myLib.mkIdentifier(SIZE_ID));
		myLib.STree_addDeclarationToInterface(t_behavior, size, 1);

		lparams.add(myLib.mkIdentifier(PORT_NAME_ID));
		myLib.STree_addDeclarationToInterface(t_behavior, port_name, 1);

		//inputs: event write_flow;event Output_time_event; (Removed : event Initialize_time;)
		SignalsDeclarationImpl write_flow = myLib.mkBooleanDecl(WRITE_FLOW_ID);
		SignalsDeclarationImpl Output_time_event = myLib.mkEventDecl(OUTPUT_TIME_EVENT_ID);
		// SignalsDeclarationImpl Initialize_time = myLib.mkEventDecl("Initialize_time"));

		// add inputs to interface
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		lins.add(myLib.mkIdentifier(WRITE_FLOW_ID));
		lins.add(myLib.mkIdentifier(OUTPUT_TIME_EVENT_ID));
		// lins.add(myLib.mkIdentifier("Initialize_time"));
		myLib.STree_addDeclarationToInterface(t_behavior, write_flow, 2);
		myLib.STree_addDeclarationToInterface(t_behavior, Output_time_event, 2);
		// myLib.STree_addDeclarationToInterface(t_behavior, Initialize_time, 2);

		//outputs: event sent_flow;
		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
		louts.add(myLib.mkIdentifier(SENT_FLOW_ID));
		myLib.STree_addDeclarationToInterface(t_behavior, myLib.mkBooleanDecl(SENT_FLOW_ID), 3);

		//body: call OutEventPort_Behavior() process--------------------------//
		SignalsDefinitionImpl eq1 = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("OutEventPort_Behavior"), lparams, lins, louts);
		myLib.STree_addExpressionOfProcessToaModel(t_behavior, eq1);

		//Trace(t_behavior.getIdentifier().getAtomString());
		return t_behavior;
	}


	/**
	 * Translates some properties which are fixed for event port
	 * 
	 * @param timpl
	 * @param p
	 * @param t_property
	 */
	public void ASME2SSME_Thread_EventPort_FixProperties(ThreadImplementation timpl, EventPort p, ProcessModelImpl t_property){

		Trace("\n ######## ASME2SSME_Translate_EventPort: ASME2SSME_Thread_EventPort_FixProperties: " + p.getName());

		if(aadlColAPI.isInputPort(p)  ){
			//Input_Time property
			ProcessCallImpl pcall1 = ASME2SSME_InputTime(timpl, p);
			myLib.STree_addExpressionOfProcessToaModel(t_property, pcall1);

			//Overflow_Handling_Protocol
			ProcessCallImpl pcall3 = ASME2SSME_Overflow_Handling_Protocol(timpl, p);
			myLib.STree_addExpressionOfProcessToaModel(t_property, pcall3);

			//Queue_Size
			ProcessCallImpl pcall4 = ASME2SSME_Queue_Size(timpl, p);
			myLib.STree_addExpressionOfProcessToaModel(t_property, pcall4);

			//Queue_Processing_Protocol			
			ProcessCallImpl pcall8 = ASME2SSME_Queue_Processing_Protocol(timpl, p);
			myLib.STree_addExpressionOfProcessToaModel(t_property, pcall8);

			//Dequeued_Items
			ProcessCallImpl pcall6 = ASME2SSME_Dequeued_Items(timpl, p);
			myLib.STree_addExpressionOfProcessToaModel(t_property, pcall6);

			//Dequeue_Protocol			
			ProcessCallImpl pcall7 = ASME2SSME_Dequeue_Protocol(timpl, p);
			myLib.STree_addExpressionOfProcessToaModel(t_property, pcall7);
		}
		else if(aadlColAPI.isOutputPort(p)){
			//Input_Time property
			ProcessCallImpl pcall2 = ASME2SSME_OutputTime(timpl, p);
			myLib.STree_addExpressionOfProcessToaModel(t_property, pcall2);

			//Queue_Size
			ProcessCallImpl pcall5 = ASME2SSME_Queue_Size(timpl, p);
			myLib.STree_addExpressionOfProcessToaModel(t_property, pcall5);

			//Queue_Processing_Protocol
			ProcessCallImpl pcall9 = ASME2SSME_Queue_Processing_Protocol(timpl, p);
			myLib.STree_addExpressionOfProcessToaModel(t_property, pcall9);		

			//Overflow_Handling_Protocol
			ProcessCallImpl pcall10 = ASME2SSME_Overflow_Handling_Protocol(timpl, p);
			myLib.STree_addExpressionOfProcessToaModel(t_property, pcall10);
		}
		else 
			Trace("Event port direction out of consideration");
	}
}
