 
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import org.osate.aadl2.*;
import org.eclipse.pop.ssme.Type;
import org.eclipse.pop.ssme.impl.*;

public class ASME2SSME_Translate_Feature extends ASME2SSME_Translate_Property{
	private static String SHAREDDATA_TIME = "SharedData_TIME";

	private signalTreeAPI myLib;
	private AADLCoLAPI aadlColAPI;
	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_Feature() {
		myLib= new signalTreeAPI();
		aadlColAPI = new AADLCoLAPI();
	}


	private SignalsDeclarationImpl mkSignalDeclaration(String name, DataClassifier data_classifier) {
		Type  ptype;
		String dname = Name_of_Reference_data_classifier(data_classifier);
		if(myLib.Is_PredefinedType(dname)) 
			ptype = myLib.STree_mkPredefinedType(dname);
		else 
			ptype = myLib.mkIdentifier(dname);  
		return myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name), ptype, null);
	}

	/**
	 * A port is represented as a signal
	 * 
	 * @param p a port
	 * @return
	 */
	public SignalsDeclarationImpl ASME2SSME_Port(Port p){

		Trace("\n ######## ASME2SSME_Translate_Feature: ASME2SSME_Port " + p.getName());
		SignalsDeclarationImpl vdecsig = new SignalsDeclarationImpl();
		if (p instanceof DataPort) return ASME2SSME_DataPort((DataPort) p);
		if (p instanceof EventPort) return ASME2SSME_EventPort((EventPort) p);
		if (p instanceof EventDataPort) return ASME2SSME_EventDataPort((EventDataPort) p);
		return vdecsig;
	}


	/** 
	 * Returns a Signal signal declaration that represents an AADL data port (data port of a process or system component, but not a thread)
	 * <p>
	 * The signal name is the same as the data port name, 
	 * and its type name is same with the data type
	 * 
	 * @param dataPort an AADL data port
	 * @return a signal declaration (typex namex;)
	 */	
	public SignalsDeclarationImpl ASME2SSME_DataPort(DataPort dataPort){		
		String name = aadlColAPI.Port_GetName(dataPort);
		Trace("\n ######## ASME2SSME_Translate_Feature: ASME2SSME_DataPort "  + name);
		return mkSignalDeclaration(name, aadlColAPI.DataPort_GetClassifier(dataPort));
	}


	/** 
	 * Returns a signal declaration that represents an AADL event port
	 *<p>
	 * The signal name is the same as the event port name, 
	 * and its type name is "event"
	 * 
	 * @param e an AADL event port
	 * @return a signal declaration (event namex;)
	 * */
	public SignalsDeclarationImpl ASME2SSME_EventPort(EventPort e){
		String name = aadlColAPI.Port_GetName(e);		
		Trace("\n ######## ASME2SSME_Translate_Feature: ASME2SSME_EventPort "  + name);	
		return myLib.mkBooleanDecl(name);	
	}


	/** 
	 * Returns a Signal signal declaration that represents an AADL event data port
	 * <p>
	 * The signal name is the same as the event data port name, 
	 * and its type name is same with the Signal name of its data type
	 * 
	 * @param p an AADL event data port
	 * @return a signal declaration (typex namex;)
	 */	
	public SignalsDeclarationImpl ASME2SSME_EventDataPort(EventDataPort p){
		String name = aadlColAPI.Port_GetName(p);
		Trace("\n ######## ASME2SSME_Translate_Feature: ASME2SSME_EventDataPort "  + name);	
		return mkSignalDeclaration(name, aadlColAPI.EventDataPort_GetClassifier(p));
	}


	/** 
	 * Returns a Signal signal declaration that represents an AADL paramter (feature)
	 * <p>
	 * The signal name is the same as the paramter, 
	 * and its type name is same with the data type
	 * 
	 * @param p an AADL paramter feature
	 * @return a signal declaration (typex namex;)
	 */	
	public SignalsDeclarationImpl ASME2SSME_Parameter(Parameter p){		
		String name = aadlColAPI.Parameter_GetName(p);
		Trace("\n ######## ASME2SSME_Translate_Feature: ASME2SSME_ParameterF : " + name);		
		return mkSignalDeclaration(name, aadlColAPI.Parameter_GetClassifier(p) );		 
	}

	/**
	 * Translates a bus access
	 * 
	 * @param ba a bus access feature
	 */
	public void ASME2SSME_BusAccess(BusAccess b1){
		Trace("\n ######## ASME2SSME_Translate_Feature: ASME2SSME_BusAccess: " + b1.getName());
		Trace("Bus access not yet implemented ");
	}


	/**
	 * Translates a data access
	 * 
	 * @param ba a data access feature
	 */
	public void ASME2SSME_DataAccess(DataAccess da1){

		Trace("\n ######## ASME2SSME_Translate_Feature: ASME2SSME_DataAccess: " + da1.getName());
		if(aadlColAPI.DataAccess_GetKind(da1)  == "provided"){

		}
		else if(aadlColAPI.DataAccess_GetKind(da1)  == "required"){

		}
	}



	/**
	 * Translates a requires data access feature x as an output bundle: bundle x;
	 * @param da1
	 */
	public SignalsDeclarationImpl ASME2SSME_RequiresDataAccess_Bundle(DataAccess da1){

		Trace("\n ######## ASME2SSME_Translate_Feature: ASME2SSME_RequiresDataAccess_Bundle: " + da1.getName());
		if(aadlColAPI.DataAccess_GetKind(da1)  != "requires"){
			Trace("not a requires data access feature");
			return null;
		}
		else{
			String name = da1.getName();
			String ename = Name_of_DataBundle(name);
			SignalsDeclarationImpl vdecsig = myLib.mkDecl(ename, SHAREDDATA_TIME);
			// myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(ename), myLib.mkIdentifier(SHAREDDATA_TIME), null);
			return vdecsig;
		}
	}


	/**
	 * Translates a requires data access feature x as an output event: event x_ReadTime;
	 * @param da1
	 */
	public SignalsDeclarationImpl ASME2SSME_RequiresDataAccess_ReadTime(DataAccess da1){

		Trace("\n ######## ASME2SSME_Translate_Feature: ASME2SSME_RequiresDataAccess: " + da1.getName());
		if(aadlColAPI.DataAccess_GetKind(da1) != "requires") {
			Trace("not a requires data access feature");
			return null;
		}
		else{
			String name = aadlColAPI.DataAccess_GetName(da1);
			return myLib.mkEventDecl(Name_of_DataReadTime(name));
		}
	}

	/**
	 * Translates a requires data access feature x as an output event: event x_WriteTime;
	 * @param da1
	 */
	public SignalsDeclarationImpl ASME2SSME_RequiresDataAccess_WriteTime(DataAccess da1){

		Trace("\n ######## ASME2SSME_Translate_Feature: ASME2SSME_RequiresDataAccess_WriteTime: " + da1.getName());
		if (aadlColAPI.DataAccess_GetKind(da1)  != "requires"){
			Trace("not a requires data access feature");
			return null;
		}
		else{
			String name = aadlColAPI.DataAccess_GetName(da1);
			return myLib.mkEventDecl(Name_of_DataReadTime(name));
		}
	}


	/**
	 * Translates a requires data access feature x as an output event: event x_reset;
	 * @param da1
	 */
	public SignalsDeclarationImpl ASME2SSME_RequiresDataAccess_reset(DataAccess da1){

		Trace("\n ######## ASME2SSME_Translate_Feature: ASME2SSME_RequiresDataAccess_reset: " + da1.getName());
		if(aadlColAPI.DataAccess_GetKind(da1)  != "requires"){
			Trace("not a requires data access feature");
			return null;
		}
		else{
			String name = aadlColAPI.DataAccess_GetName(da1);
			String ename = Name_of_DataReset(name);
			return myLib.mkEventDecl(ename);
		}
	}



	/**
	 * Translates a requires subprogram access feature x as an output event: event x;
	 * @param da1
	 */
	public SignalsDeclarationImpl ASME2SSME_RequiresSubprogramAccess_call(SubprogramAccess da1){

		Trace("\n ######## ASME2SSME_Translate_Feature: ASME2SSME_RequiresSubprogramAccess_call: " + da1.getName());
		if(aadlColAPI.SubprogramAccess_GetKind(da1) != "requires"){
			Trace("not a requires subprogram access feature");
			return null;
		}
		else{
			String name = aadlColAPI.SubprogramAccess_GetName(da1);
			String ename = Name_of_SubprogramCall(name);
			return  myLib.mkEventDecl(ename);
		}
	}


	/**
	 * Translates a subprogram call x as an  event: event x, 
	 * if the called subprogram sp is not a subcomponent of the thread t
	 * 
	 * @param call1 a subprogram call
	 */
	public SignalsDeclarationImpl ASME2SSME_SubprogramCall1(SubprogramCall call1, BehavioredImplementation t){

		Trace("\n ######## ASME2SSME_Translate_Feature: ASME2SSME_SubprogramCall1: " + call1.getName());
		CalledSubprogram called_sp = aadlColAPI.SubprogramCall_Get_CalledSubprogram(call1);
		SignalsDeclarationImpl vdecsig = new SignalsDeclarationImpl();
		// if called_sp is a sub component of t: ? event sp1;
		if(aadlColAPI.Is_CalledSubprogram_Sub_Of_ComponentImpl(called_sp, t)){
			Trace("the called subprogram is a sub component of t, it is translated in another method");
		}
		else{
			// x : subprogram sp;
			String name = call1.getName();
			vdecsig = myLib.mkEventDecl(name);
		}
		return vdecsig;
	}
}
