 
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;

import org.osate.aadl2.*;
import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;


public class ASME2SSME_Translate_FeatureGroup extends ASME2SSME_Translate_Feature{
	
	private signalTreeAPI myLib ;
	private AADLCoLAPI aadlColAPI  ;
	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_FeatureGroup() {
		myLib= new signalTreeAPI();
		aadlColAPI = new AADLCoLAPI();
	}
	
	/**
	 * Translates feature group (only the port features are considered). 
	 * The in ports are translated to a Signal bundle xx_BW
	 * 
	 * @param pg a port group
	 */
	public TypesDeclarationImpl  ASME2SSME_FeatureGroupType_BW(FeatureGroupType fgt){
		
		    Trace("\n == ASME2SSME_Trasnalte_FeatureGroup: ASME2SSME_FeatureGroupType_BW " + fgt.getName());
		    Trace("Only the port features are considered. The other features are not yet implemented.");
		   

		    TypesDeclarationImpl type_decl = new TypesDeclarationImpl();
		    TypeDescriptionImpl typedescr = new TypeDescriptionImpl();	
		    ExprSignal vinit = null;	

		    String pgt_name = fgt.getName();
		    String pgt_sig_name = Name_of_FeatureGroupType(pgt_name,  "BW");
		    Identifier id = myLib.mkIdentifier(pgt_sig_name);
		    
		    ArrayList<Port> list1 = aadlColAPI.FeatureGroupType_GetPorts(fgt);
		    FeatureGroupType inverse = aadlColAPI.FeatureGroupType_GetInverse(fgt);
		    
		    if (list1.size() == 0){		    
		       if (inverse == null) 
		    	    type_decl.getTypeDefinition().add(id);
		       else { //inverse of some other feature group: type xx_BW = inverse_WB
		    	   
		    	   if(ASME2SSME_FeatureGroupType_WB(inverse) == null) return null;
		    	   else{
		    		   String inverse_name = inverse.getName();
			    	   String inverse_sig_str = Name_of_FeatureGroupType(inverse_name, "WB");
		    		   Identifier inverse_id = myLib.mkIdentifier(inverse_sig_str);
					   typedescr = myLib.STree_mkDclType(id, inverse_id, vinit);
					   type_decl.getTypeDefinition().add(typedescr);
		    	   }
		       }		    				    
		    }
		    else {//features: in out ports
			    TupleTypeImpl description = new TupleTypeImpl();
			    description.setTypeKind(TypeKind.BUNDLE_LITERAL);						    		    	
		    	int cnt =0;
		    	for(int i =0; i< list1.size(); i++){		    		
			    		//in data port
			    		if(list1.get(i) instanceof DataPort){
						    DataPort p1 = (DataPort) list1.get(i);
						    if(aadlColAPI.isInputPort(p1)  ){
						    	SignalsDeclarationImpl decl1 = ASME2SSME_DataPort(p1);
							    description.getListSignalsDeclaration().add(decl1);
							    cnt++;
						    }						    
			    		}
			    		//in event port
			    		else if(list1.get(i) instanceof EventPort){
			    			EventPort p2 = (EventPort) list1.get(i);
						    if(aadlColAPI.isInputPort(p2)){
						    	SignalsDeclarationImpl decl2 = ASME2SSME_EventPort(p2);
							    description.getListSignalsDeclaration().add(decl2);
							    cnt++;
						    }	
			    		}
			    		//in event data port
			    		else if(list1.get(i) instanceof EventDataPort){
			    			EventDataPort p3 = (EventDataPort) list1.get(i);
						    if(aadlColAPI.isInputPort(p3)){
						    	SignalsDeclarationImpl decl3 = ASME2SSME_EventDataPort(p3);
							    description.getListSignalsDeclaration().add(decl3);
							    cnt++;
						    }	
			    		}
			    		else
			    			Trace("\n feature group of group not yet implemented");
		    	}  
			    	
		    	if(cnt == 0) return null;
		    	typedescr = myLib.STree_mkDclType(id, description, vinit);
		    	type_decl.getTypeDefinition().add(typedescr);
		    }

			myLib.STree_addComment(type_decl, "port group type for " + pgt_name, false);
		    return type_decl;
	}
	
	
	/**
	 * Translates port group.
	 * The out ports are translated to a Signal bundle xx_WB
	 * 
	 * @param pg a port group
	 */
	public TypesDeclarationImpl ASME2SSME_FeatureGroupType_WB(FeatureGroupType fgt){
	 
		    Trace("\n == ASME2SSME_Trasnalte_FeatureGroup: ASME2SSME_FeatureGroupType_WB " + fgt.getName());

		    TypesDeclarationImpl type_decl = new TypesDeclarationImpl();
		    TypeDescriptionImpl typedescr = new TypeDescriptionImpl();	
		    ExprSignal vinit = null;	

		    String pg_name = fgt.getName();
		    String pg_sig_name = Name_of_PortGroupType(pg_name, "WB");
		    Identifier id = myLib.mkIdentifier(pg_sig_name);
		    
		    ArrayList<Port> list1 = aadlColAPI.FeatureGroupType_GetPorts(fgt);
		    FeatureGroupType inverse = aadlColAPI.FeatureGroupType_GetInverse(fgt);
		    
		    if (list1.size() == 0 ){
		    
		       if (inverse == null) 
		    	   type_decl.getTypeDefinition().add(id);	
		       else{//inverse of some other feature group: type xx_WB = inverse_BW
		    	   if(ASME2SSME_FeatureGroupType_BW(inverse) == null) return null;
		    	   String inverse_name = inverse.getName();
		    	   Identifier inverse_id = myLib.mkIdentifier(Name_of_FeatureGroupType(inverse_name, "BW"));
		    	   typedescr = myLib.STree_mkDclType(id, inverse_id, vinit);
		    	   type_decl.getTypeDefinition().add(typedescr);
		       }		    
		    }
		    else {//features: in out ports
			    TupleTypeImpl description = new TupleTypeImpl();
			    description.setTypeKind(TypeKind.BUNDLE_LITERAL);
			    int cnt =0;
			    for(int i =0; i< list1.size(); i++){
			    		
			    		//out data port
			    		if(list1.get(i) instanceof DataPort){
						    DataPort p1 = (DataPort) list1.get(i);
						    if(aadlColAPI.isOutputPort(p1)){
						    	SignalsDeclarationImpl decl1 = ASME2SSME_DataPort(p1);
							    description.getListSignalsDeclaration().add(decl1);
							    cnt++;
						    }						    
			    		}
			    		//out event port
			    		else if(list1.get(i) instanceof EventPort){
			    			EventPort p2 = (EventPort) list1.get(i);
						    if(aadlColAPI.isOutputPort(p2)){
						    	SignalsDeclarationImpl decl2 = ASME2SSME_EventPort(p2);
							    description.getListSignalsDeclaration().add(decl2);
							    cnt++;
						    }	
			    		}
			    		//out event data port
			    		else if(list1.get(i) instanceof EventDataPort){
			    			EventDataPort p3 = (EventDataPort) list1.get(i);
						    if(aadlColAPI.isOutputPort(p3)){
						    	SignalsDeclarationImpl decl3 = ASME2SSME_EventDataPort(p3);
							    description.getListSignalsDeclaration().add(decl3);
							    cnt++;
						    }	
			    		}
			    		else 
			    			Trace("\n feature group of group not yet implemented");
			    	} 
			    	
			    	if(cnt ==0) return null;
			    	else{
			    		typedescr = myLib.STree_mkDclType(id, description, vinit);
					    type_decl.getTypeDefinition().add(typedescr);
			    	}		    		    
		    }
			myLib.STree_addComment(type_decl, "port group type for " + pg_name, false);
		    return type_decl;
	}
	
	
	
	/**
	 * Translates feature group. 
	 * A feature group is seperated into a pair of signals: xx_bw and xx_wb. 
	 * This method returns a signal declaration for xx_bw
	 * 
	 * @param pg a feature group
	 */
	public SignalsDeclarationImpl ASME2SSME_FeatureGroup_bw(FeatureGroup pg){
		return FeatureGroup(pg, "bw", "BW");
	}
	
	
	


	/**
	 * Translates feature group. 
	 * A feature group is seperated into two signals: an in bundle and an out bundle. 
	 * This method returns a signal declaration for the out bundle
	 * 
	 * @param pg a feature group
	 */
	public SignalsDeclarationImpl ASME2SSME_FeatureGroup_wb(FeatureGroup pg){
		return FeatureGroup(pg, "wb", "WB");
	}
	
private SignalsDeclarationImpl FeatureGroup(FeatureGroup pg, String vstr1, String vstr2) {
		
		String pg_name = pg.getName();
		Trace("\n == ASME2SSME_Trasnalte_FeatureGroup: ASME2SSME_FeatureGroup_"+ vstr1 + pg_name);
		FeatureGroupType pgt = pg.getFeatureGroupType();
		
		if(ASME2SSME_FeatureGroupType_BW(pgt) == null) return null;
		String pg_sig_name = Name_of_FeatureGroup(pg_name, vstr1);
		String pgt_sig_name = Name_of_FeatureGroupType(pgt.getName(), vstr2);
		Identifier type = myLib.mkIdentifier(pgt_sig_name);
		IdentifierImpl id = myLib.mkIdentifier(pg_sig_name);
		SignalsDeclarationImpl vdecsig = myLib.STree_mkSignalDeclaration(id, type, null);			
		return vdecsig;
		 
	}
	
}
