 
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import org.osate.aadl2.*;
import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;


public class ASME2SSME_Translate_FeatureGroupConnection extends ASME2SSME_Translate_Connection{
	private signalTreeAPI  myLib;
	private AADLCoLAPI aadlColAPI  ;

	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_FeatureGroupConnection() {
		myLib= new signalTreeAPI();
		aadlColAPI = new AADLCoLAPI();
	}


	/** 
	 * Returns a Signal signal definition (an equation) that represents a feature group connection. 
	 * A feature group connection is separated into two direction connections: 
	 * 1) from source feature (port) group's out ports to dst feature (port) group's in ports
	 * 2) from dst feature (port) group's out ports to source feature (port) group's in ports
	 * This method implements the first case: src_BUNDLEOut -> dst_BUNDLEIn
	 * <p>
	 * Depending on the source and dst context of the connection, different connection is provided. 
	 * 
	 * @param c1 a feature group connection
	 * @return a signal definition (an equation)
	 */
	public SignalsDefinitionImpl ASME2SSME_FeatureGroupConnection1(FeatureGroupConnection c1){

		Trace("\n ######## ASME2SSME_Translate_FeatureGroupConnection: ASME2SSME_FeatureGroupConnection1 "+ c1.getName());

		SignalsDefinitionImpl eq = new SignalsDefinitionImpl();

		FeatureGroup src = (FeatureGroup) aadlColAPI.Connection_GetSrc(c1);
		FeatureGroup dst = (FeatureGroup) aadlColAPI.Connection_GetDst(c1);

		FeatureGroupType fgt = src.getFeatureGroupType();
		ASME2SSME_Translate_FeatureGroup asme2ssme_fgt = new ASME2SSME_Translate_FeatureGroup();
		if(asme2ssme_fgt.ASME2SSME_FeatureGroupType_WB(fgt) == null){
			Trace("Feature group connection from wb bundle direction not exist ");
			return null;
		}
		else{

			Context src_context = aadlColAPI.Connection_GetSrcContext(c1);
			Context dst_context = aadlColAPI.Connection_GetDstContext(c1);
			String left_name;
			String right_name;

			//systemImpl.PortGroup -> ========================================
			if(src_context instanceof SystemImplementation){

				right_name = Name_of_FeatureGroup(src.getName(), "WB");
				ExprSignal right_expr = myLib.mkIdentifier(right_name);

				//systemImpl.PortGroup -> Subsystem.PortGroup | Subprocess.PortGroup | SubDevice.PortGroup
				if(dst_context instanceof SystemSubcomponent
						| dst_context instanceof ProcessSubcomponent
						| dst_context instanceof DeviceSubcomponent){

					if (dst_context instanceof SystemSubcomponent){
						SystemSubcomponent dst_sub = (SystemSubcomponent) dst_context;
						left_name =  dst_sub.getName() + "_" + Name_of_FeatureGroup(dst.getName(), "bw");
					}
					else if (dst_context instanceof ProcessSubcomponent){
						ProcessSubcomponent dst_sub = (ProcessSubcomponent) dst_context;
						left_name =  dst_sub.getName() + "_" + Name_of_FeatureGroup(dst.getName(), "bw");
					}
					else {
						DeviceSubcomponent dst_sub = (DeviceSubcomponent) dst_context;
						left_name =  dst_sub.getName() + "_" + Name_of_FeatureGroup(dst.getName(), "bw");
					}

					String sn1 = getSname(left_name);
					ExprSignal left_expr = myLib.mkIdentifier(sn1);
					eq = (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
				}
				else{
					Trace("Feature group connection from a system (some case) not yet implemented ");
					return null;
				}
			}

			//processImpl.PortGroup -> =======================================
			else if(src_context instanceof ProcessImplementation) {

				right_name = Name_of_FeatureGroup(src.getName(), "wb");
				ExprSignal right_expr = myLib.mkIdentifier(right_name);

				//processImpl.PortGroup ->Subthread.PortGroup
				if (dst_context instanceof ThreadSubcomponent){

					ThreadSubcomponent dst_sub = (ThreadSubcomponent) dst_context;
					left_name =  dst_sub.getName() + "_" + Name_of_FeatureGroup(dst.getName(), "bw");
					String sn1 = getSname(left_name);
					ExprSignal left_expr = myLib.mkIdentifier(sn1);
					eq = (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
				}
				else{
					Trace("Feature group connection from a process some case not yet implemented ");
					return null;
				}
			}

			//Subsystem.PortGroup | Subprocess.PortGroup | Subdevice.PortGroup -> ==========================================
			else if (src_context instanceof SystemSubcomponent
					| src_context instanceof ProcessSubcomponent
					| src_context instanceof DeviceSubcomponent){

				if (src_context instanceof SystemSubcomponent){
					SystemSubcomponent src_sub = (SystemSubcomponent) src_context;
					right_name =  src_sub.getName() + "_" + Name_of_FeatureGroup(src.getName(), "wb");
				}
				else if (src_context instanceof ProcessSubcomponent){
					ProcessSubcomponent src_sub = (ProcessSubcomponent) src_context;
					right_name =  src_sub.getName() + "_" + Name_of_FeatureGroup(src.getName(), "wb");
				}
				else {
					DeviceSubcomponent src_sub = (DeviceSubcomponent) src_context;
					right_name =  src_sub.getName() + "_" + Name_of_FeatureGroup(src.getName(), "wb");
				}

				String sn1 = getSname(right_name);
				ExprSignal right_expr = myLib.mkIdentifier(sn1);

				//Subsystem.PortGroup | ... -> system.PortGroup 
				if(dst_context instanceof SystemImplementation){

					left_name = Name_of_FeatureGroup(dst.getName(), "bw");
					ExprSignal left_expr = myLib.mkIdentifier(left_name);
					eq = (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
				}

				//Subsystem.PortGroup | ... -> Subsystem.PortGroup | Subprocess.PortGroup | Subdevice.PortGroup
				else if(dst_context instanceof SystemSubcomponent
						| dst_context instanceof ProcessSubcomponent
						| dst_context instanceof DeviceSubcomponent){

					if (dst_context instanceof SystemSubcomponent){
						SystemSubcomponent dst_sub = (SystemSubcomponent) dst_context;
						left_name =  dst_sub.getName() + "_" + Name_of_FeatureGroup(dst.getName(), "bw");
					}
					else if (dst_context instanceof ProcessSubcomponent){
						ProcessSubcomponent dst_sub = (ProcessSubcomponent) dst_context;
						left_name =  dst_sub.getName() + "_" + Name_of_FeatureGroup(dst.getName(), "bw");
					}
					else {
						DeviceSubcomponent dst_sub = (DeviceSubcomponent) dst_context;
						left_name =  dst_sub.getName() + "_" + Name_of_FeatureGroup(dst.getName(), "bw");
					}

					String sn2 = getSname(left_name);
					ExprSignal left_expr = myLib.mkIdentifier(sn2);
					eq = (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
				}

				else{
					Trace("feature group connection not yet implemented ");
					return null;
				}

			}

			else{
				Trace("Feature group connection not yet implemented ");
				return null;
			}
		}

		return eq;

	}


	/** 
	 * Returns a Signal signal definition (an equation) that represents a feature group connection. 
	 * A feature group connection is separated into two direction connections: 
	 * 1) from source feature (port) group's out ports to dst feature (port) group's in ports
	 * 2) from dst feature (port) group's out ports to source feature (port) group's in ports
	 * This method implements the second case: dst_BUNDLEOut -> src_BUNDLEIn
	 * <p>
	 * Depending on the source and dst context of the connection, different connection is provided. 
	 * 
	 * @param c1 a feature group connection
	 * @return a signal definition (an equation)
	 */
	public SignalsDefinitionImpl ASME2SSME_FeatureGroupConnection2(FeatureGroupConnection c1){

		Trace("\n ######## ASME2SSME_Translate_Connection: ASME2SSME_PortGroupConnection2 "+ c1.getName());

		SignalsDefinitionImpl eq = new SignalsDefinitionImpl();
		ASME2SSME_Translate_FeatureGroup asme2ssme_fgt = new ASME2SSME_Translate_FeatureGroup();

		FeatureGroup src = (FeatureGroup) aadlColAPI.Connection_GetSrc(c1);
		FeatureGroup dst = (FeatureGroup) aadlColAPI.Connection_GetDst(c1);

		FeatureGroupType fgt = src.getFeatureGroupType();
		if(asme2ssme_fgt.ASME2SSME_FeatureGroupType_BW(fgt) == null){
			Trace("Feature group connection from bw bundle direction not exist ");
			return null;
		}
		else{

			Context src_context = aadlColAPI.Connection_GetSrcContext(c1);
			Context dst_context = aadlColAPI.Connection_GetDstContext(c1);
			String left_name;
			String right_name;

			//systemImpl.PortGroup -> ========================================
			if(src_context instanceof SystemImplementation){

				left_name = Name_of_FeatureGroup(src.getName(), "bw");
				ExprSignal left_expr = myLib.mkIdentifier(left_name);

				//systemImpl.PortGroup -> Subsystem.PortGroup | Subprocess.PortGroup | SubDevice.PortGroup
				if(dst_context instanceof SystemSubcomponent
						| dst_context instanceof ProcessSubcomponent
						| dst_context instanceof DeviceSubcomponent){

					if (dst_context instanceof SystemSubcomponent){
						SystemSubcomponent dst_sub = (SystemSubcomponent) dst_context;
						right_name =  dst_sub.getName() + "_" + Name_of_FeatureGroup(dst.getName(), "wb");
					}
					else if (dst_context instanceof ProcessSubcomponent){
						ProcessSubcomponent dst_sub = (ProcessSubcomponent) dst_context;
						right_name =  dst_sub.getName() + "_" + Name_of_FeatureGroup(dst.getName(), "wb");
					}
					else {
						DeviceSubcomponent dst_sub = (DeviceSubcomponent) dst_context;
						right_name =  dst_sub.getName() + "_" + Name_of_FeatureGroup(dst.getName(), "wb");
					}

					String sn1 = getSname(right_name);
					ExprSignal right_expr = myLib.mkIdentifier(sn1);
					eq = (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
				}
				else{
					Trace("Feature group connection from a system (some case) not yet implemented ");
					return null;
				}
			}

			//processImpl.PortGroup -> =======================================
			else if(src_context instanceof ProcessImplementation) {

				left_name = Name_of_FeatureGroup(src.getName(), "bw");
				ExprSignal left_expr = myLib.mkIdentifier(left_name);

				//processImpl.PortGroup ->Subthread.PortGroup
				if (dst_context instanceof ThreadSubcomponent){

					ThreadSubcomponent dst_sub = (ThreadSubcomponent) dst_context;
					right_name =  dst_sub.getName() + "_" + Name_of_FeatureGroup(dst.getName(), "bw");
					String sn1 = getSname(right_name);
					ExprSignal right_expr = myLib.mkIdentifier(sn1);
					eq = (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
				}
				else{
					Trace("Feature group connection from a process some case not yet implemented ");
					return null;
				}
			}

			//Subsystem.PortGroup | Subprocess.PortGroup | Subdevice.PortGroup -> ==========================================
			else if (src_context instanceof SystemSubcomponent
					| src_context instanceof ProcessSubcomponent
					| src_context instanceof DeviceSubcomponent){

				if (src_context instanceof SystemSubcomponent){
					SystemSubcomponent src_sub = (SystemSubcomponent) src_context;
					left_name =  src_sub.getName() + "_" + Name_of_FeatureGroup(src.getName(), "bw");
				}
				else if (src_context instanceof ProcessSubcomponent){
					ProcessSubcomponent src_sub = (ProcessSubcomponent) src_context;
					left_name =  src_sub.getName() + "_" + Name_of_FeatureGroup(src.getName(), "bw");
				}
				else {
					DeviceSubcomponent src_sub = (DeviceSubcomponent) src_context;
					left_name =  src_sub.getName() + "_" + Name_of_FeatureGroup(src.getName(), "bw");
				}

				String sn1 = getSname(left_name);
				ExprSignal left_expr = myLib.mkIdentifier(sn1);

				//Subsystem.PortGroup | ... -> system.PortGroup 
				if(dst_context instanceof SystemImplementation){

					right_name = Name_of_FeatureGroup(dst.getName(), "wb");
					ExprSignal right_expr = myLib.mkIdentifier(right_name);
					eq = (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
				}

				//Subsystem.PortGroup | ... -> Subsystem.PortGroup | Subprocess.PortGroup | Subdevice.PortGroup
				else if(dst_context instanceof SystemSubcomponent
						| dst_context instanceof ProcessSubcomponent
						| dst_context instanceof DeviceSubcomponent){

					if (dst_context instanceof SystemSubcomponent){
						SystemSubcomponent dst_sub = (SystemSubcomponent) dst_context;
						right_name =  dst_sub.getName() + "_" + Name_of_FeatureGroup(dst.getName(), "wb");
					}
					else if (dst_context instanceof ProcessSubcomponent){
						ProcessSubcomponent dst_sub = (ProcessSubcomponent) dst_context;
						right_name =  dst_sub.getName() + "_" + Name_of_FeatureGroup(dst.getName(), "wb");
					}
					else {
						DeviceSubcomponent dst_sub = (DeviceSubcomponent) dst_context;
						right_name =  dst_sub.getName() + "_" + Name_of_FeatureGroup(dst.getName(), "wb");
					}
					String sn2 = getSname(right_name);
					ExprSignal right_expr = myLib.mkIdentifier(sn2);
					eq = (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
				}
				else{
					Trace("Feature group connection not yet implemented ");
					return null;
				}
			}
			else{
				Trace("Feature group connection not yet implemented ");
				return null;
			}
		}
		return eq;
	}
}
