
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;

import org.osate.aadl2.*;
import org.eclipse.pop.ssme.impl.*;

public class ASME2SSME_Translate_Flow extends ASME2SSME_Translate_Mode{
	private AADLCoLAPI aadlColAPI; 
	private signalTreeAPI myLib;
	
	/**
	 * 
	 * @param vprocess
	 * @param list1
	 * @param flow_name
	 */
	private void addPropertyPart(ProcessModelImpl vprocess, ArrayList<PropertyAssociation> list1, String flow_name) {
		ProcessCallImpl pcall;
		
		for(int i=0; i<list1.size(); i++)
		{
			PropertyAssociation pa = list1.get(i);
			String propertyDef_name = aadlColAPI.Property_Get_Name(pa);
			// Trace("propertyDef_name = " + propertyDef_name );
			
			if (propertyDef_name.equals("Latency")) 
				pcall = ASME2SSME_FlowLatency_Property_Call(flow_name, pa);
			else if (propertyDef_name.equals("Actual_Latency")) 
				pcall = ASME2SSME_ActualLatency_Property_Call(flow_name, pa);
			else 
				pcall = ASME2SSME_Property_Call_withName(pa);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, pcall); 
		}
	}

	private void addFlowProperty(ProcessModelImpl vprocess,	FlowImplementation f, String vselect) {
		String flow_name = aadlColAPI.FlowImplementation_GetName(f);
		Trace("\n ######## addFlowProperty: " + flow_name);
		if (aadlColAPI.FlowImplementation_GetKind(f) == vselect){
			ArrayList<PropertyAssociation> list1 = aadlColAPI.FlowImplementation_GetPropertyAssociation(f);
			addPropertyPart(vprocess, list1, flow_name);
		} 
		
	}
	
	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_Flow() {
		  myLib= new signalTreeAPI();
		  aadlColAPI = new AADLCoLAPI();
		}
	
	
	/**
	 * Translates flow specification. 
	 * Only the property part is translated: each property is translated as a property process call.
	 * Add a process instance xx_Flow_Property{flow_name, ...}() in the body of vprocess
	 * 
	 * @param fs a flow specification
	 */
	public void ASME2SSME_FlowSpec(ProcessModelImpl vprocess, FlowSpecification fs){
		
		Trace("\n ######## ASME2SSME_Translate_Flow: ASME2SSME_FlowSpec : " + fs.getName());
		//Trace("Flow specification not yet implemented: " + fs.getName());
		//ASME2SSME_Translate_Property asme2ssme_property = new ASME2SSME_Translate_Property();
		String flow_name = aadlColAPI.FlowSpec_GetName(fs);
		ArrayList<PropertyAssociation> list1 = aadlColAPI.FlowSpec_GetPropertyAssociation(fs);
		addPropertyPart(vprocess, list1, flow_name);
		
	}

	/**
	 * Translates end to end flow
	 * Only the property part is translated: each property is translated as a property process call.
	 * Add a process instance xx_Flow_Property{flow_name, ...}() in the body of vprocess
	 * 
	 * @param f
	 */
	public void ASME2SSME_EndToEndFlow(ProcessModelImpl vprocess, EndToEndFlow f){
		
		Trace("\n ######## ASME2SSME_Translate_Flow: ASME2SSME_EndToEndFlow : " + f.getName());
		String flow_name = aadlColAPI.EndToEndFlow_GetName(f);
		ArrayList<PropertyAssociation> list1 = aadlColAPI.EndToEndFlow_GetPropertyAssociation(f);
		addPropertyPart(vprocess, list1, flow_name);
	}
	
	
	
	
	/**
	 * Translates flow source implementation
	 * Only the property part is translated: each property is translated as a property process call.
	 * Add a process instance xx_Flow_Property{flow_name, ...}() in the body of vprocess
	 * 
	 * @param f
	 */
	public void ASME2SSME_FlowSourceImpl(ProcessModelImpl vprocess, FlowImplementation f){
		addFlowProperty(vprocess, f, "source");
	}
	
	/**
	 * Translates flow sink implementation
	 * 
	 * @param f
	 */
	public void ASME2SSME_FlowSinkImpl(ProcessModelImpl vprocess, FlowImplementation f){
		addFlowProperty(vprocess, f, "sink");
	}
	
	/**
	 * Translates flow path implementation
	 * 
	 * @param f
	 */
	public void ASME2SSME_FlowPathImpl(ProcessModelImpl vprocess, FlowImplementation f){
		addFlowProperty(vprocess, f, "path");
	}
}
