
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import org.osate.aadl2.*;
import org.eclipse.pop.ssme.impl.*;

public class ASME2SSME_Translate_Memory extends ASME2SSME_Translate_Feature{
	private AADLCoLAPI aadlColAPI  ;
	private signalTreeAPI myLib;

	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_Memory() {
		aadlColAPI = new AADLCoLAPI();
		myLib= new signalTreeAPI();
	}
	

/**
 * Translates AADL memory type as a Signal process
 * 
 * @param mt
 */
public ProcessModelImpl ASME2SSME_MemoryType(MemoryType mt){
	
	
	String sname = aadlColAPI.ComponentType_GetName(mt);
	
	Trace("\n ====== ASME2SSME_Translate_Memory: ASME2SSME_MemoryType: " + sname);
		
	ProcessModelImpl vprocess;
	vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(sname) );
	
	ArrayList<PropertyAssociation> list1 = aadlColAPI.Component_GetPropertyAssociations(mt);
	if(list1.size()>0){
		//memory property ================================================
		ProcessModelImpl mproperty = ASME2SSME_MemoryType_property(mt);
		String property_name = myLib.STree_GetStringName(mproperty.getIdentifier());//pname + "_property";
		//add to declaration
		myLib.STree_addProcessDeclaration(vprocess, mproperty);
		//add to body
		ProcessCallImpl property_inst = myLib.STree_mkModelCall(property_name);
	    myLib.STree_addExpressionOfProcessToaModel(vprocess, property_inst);
			    
		//add comments for property process instance
		ArrayList<String> comments4 = new ArrayList<String>();
		comments4.add("memory property sub process");
		myLib.STree_addComment(property_inst, comments4, false);
	}


	if((myLib.STree_IsProcessHasOutput(vprocess) == false) && (myLib.STree_IsProcessNull(vprocess))){
		myLib.STree_addNullProc(vprocess);
	}
	
	return vprocess;		
	}	
	

/**
 * Returns a Signal process "xx_property" that represents the properties of a memory type component
 * <p>
 * This Signal process is named as "xx_property", where "xx" is the name of the Signal process that represents this memory type. 
 * 
 * @param mi a  memory implementation. 
 * @return a ProcessModelImpl object (Signal process)
 */
public ProcessModelImpl ASME2SSME_MemoryType_property(MemoryType mt){

	Trace("\n ====== ASME2SSME_Translate_Memory: ASME2SSME_MemoryType_property" + mt.getName()); 

	
	
	String process_name = Name_of_Memory(aadlColAPI.ComponentType_GetName(mt));
	ProcessModelImpl p_property = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(process_name+"_property") );
	
	ArrayList<PropertyAssociation> list1 = aadlColAPI.Component_GetPropertyAssociations(mt);
	for(int i=0; i<list1.size(); i++)
	{
		PropertyAssociation pa = list1.get(i);
      ProcessCallImpl pcall1 =  ASME2SSME_Property_Call(pa);
      myLib.STree_addExpressionOfProcessToaModel(p_property, pcall1);   
	}
		
	if((myLib.STree_IsProcessHasOutput(p_property) == false) && (myLib.STree_IsProcessNull(p_property))){
		myLib.STree_addNullProc(p_property);
	}
	
	return p_property;
}


	
/**
 * Translates AADL memory implementation
 * 
 * @param mi
 */
public ProcessModelImpl ASME2SSME_MemoryImpl(MemoryImplementation mi){

	
	
	String sname = aadlColAPI.ComponentImpl_GetName(mi);
	
	Trace("\n ====== ASME2SSME_Translate_Memory: ASME2SSME_MemoryImpl: " + sname);
		
	ProcessModelImpl vprocess;
	vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(sname) );

	//memory behavior ===========================================
	ProcessModelImpl mbehavior = ASME2SSME_Memory_behavior(mi);
	String behavior_name = myLib.STree_GetStringName(mbehavior.getIdentifier());//pname + "_behavior";
	//add to declaration
	myLib.STree_addProcessDeclaration(vprocess, mbehavior);
	//add to body
	ProcessCallImpl behavior_inst = myLib.STree_mkModelCall(behavior_name);
    myLib.STree_addExpressionOfProcessToaModel(vprocess, behavior_inst);
    
    //add comments for behavior process instance
	ArrayList<String> comments3 = new ArrayList<String>();
	comments3.add("memory behavior sub process");
	myLib.STree_addComment(behavior_inst, comments3, false);
	
	//memory property ================================================
	ProcessModelImpl mproperty = ASME2SSME_Memory_property(mi);
	String property_name = myLib.STree_GetStringName(mproperty.getIdentifier());//pname + "_property";
	//add to declaration
	myLib.STree_addProcessDeclaration(vprocess, mproperty);
	//add to body
	ProcessCallImpl property_inst = myLib.STree_mkModelCall(property_name);
    myLib.STree_addExpressionOfProcessToaModel(vprocess, property_inst);
    
    //add comments for property process instance
	ArrayList<String> comments4 = new ArrayList<String>();
	comments4.add("memory property sub process");
	myLib.STree_addComment(property_inst, comments4, false);
	
	return vprocess;		
	}



/** 
 * Returns a "xx_Memory_behavior" process
 *
 * @param mi the memory implementation
 * @return a Signal process (ProcessModelImpl object) which represents the behavior part of the memory
 */
public ProcessModelImpl ASME2SSME_Memory_behavior(MemoryImplementation mi){
	
	
	String mname = Name_of_Memory(aadlColAPI.ComponentImpl_GetName(mi));
	Trace("\n == ASME2SSME_Translate_Memory: ASME2SSME_Memory_behavior: " + aadlColAPI.ComponentImpl_GetName(mi));
	
	String bname = mname + "_behavior";
	ProcessModelImpl mbehavior = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(bname) );
	
	if((myLib.STree_IsProcessHasOutput(mbehavior) == false) && (myLib.STree_IsProcessNull(mbehavior))){
		myLib.STree_addNullProc(mbehavior);
	}
	
	return mbehavior;
}


  /**
   * Returns a Signal process "xx_property" that represents the properties of a memory implementation component
   * <p>
   * This Signal process is named as "xx_property", where "xx" is the name of the Signal process that represents this memory implementation. 
   * An empty skeleton process is provided.
   * 
   * @param mi a  memory implementation. 
   * @return a ProcessModelImpl object (Signal process)
   */
public ProcessModelImpl ASME2SSME_Memory_property(MemoryImplementation mi){

	Trace("\n ====== ASME2SSME_Translate_Memory: ASME2SSME_Memory_property" + mi.getName()); 

	
	String process_name = Name_of_Memory(aadlColAPI.ComponentImpl_GetName(mi));
	ProcessModelImpl p_property = ASME2SSME_Component_Properties_Call(mi, process_name);
	
	
	if((myLib.STree_IsProcessHasOutput(p_property) == false) && (myLib.STree_IsProcessNull(p_property))){
		myLib.STree_addNullProc(p_property);
	}
	
	return p_property;
}



}
