
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;

/* This class is used to naming the AADL components (elements) in Signal
 * 
 */
public class ASME2SSME_Translate_Name {
	public static ArrayList<LocalName> NameList = new ArrayList<LocalName> ();
	private static int index = 0;

	public static final String TOP_ID  = "top";
	// AADL predefined Signals
	public static final String START_ID = "Start";
	public static final String RESUME_ID = "Resume";
	public static final String DISPATCH_ID = "Dispatch";
	public static final String COMPLETE_ID = "Complete";
	public static final String DEADLINE_ID = "Deadline";
	public static final String COMPLETION_ID = "Completion";// ?

	// Added Signals
	public static final String ALARM_ID = "Alarm";
	// Signals (bundle) added for defining the control.
	public static final String CTL1_ID = "ctl1";
	public static final String CTL2_ID = "ctl2";
	public static final String TIME1_ID = "time1";
	public static final String TIME2_ID = "time2";

	public static final String ERROR_ID = "Error";
	public static final String TREAD_ID = "TRead";
	public static final String TRESET_ID = "TReset";
	public static final String TWRITE_ID = "TWrite";


	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_Name() {
	}

	/**
	 * Prints the string str in the console
	 * 
	 * @param str  a string to be printed out
	 */
	public void Trace(String str){
		java.lang.System.out.println(str);
	}



	/**
	 * Returns a Signal String name of a system implementation
	 * <p>
	 * if impl_name = "ttt.iii", return a name = "ttt_iii_system"
	 * 
	 * @param impl_name a String name of the system implementation
	 * @return a String name
	 */
	public String Name_of_System(String impl_name){ 		
		return  impl_name.replace('.','_') + "_System";		 	
	}


	/**
	 * Returns a Signal String name of a process implementation
	 * <p>
	 * if impl_name = "ttt.iii", return a name = "ttt_iii_process"
	 * 
	 * @param impl_name a String name of the process implementation
	 * @return a String name 
	 */
	public String Name_of_Process(String impl_name){ 		
		return impl_name.replace('.','_') + "_Process";	
	}


	/**
	 * Returns a Signal String name of a device implementation
	 * <p>
	 * if impl_name = "ttt.iii", return a name = "ttt_iii_device"
	 * 
	 * @param impl_name a String name of the device implementation
	 * @return a String name 
	 */
	public String Name_of_Device(String impl_name){ 
		return impl_name.replace('.','_') + "_Device"; 
	}


	/**
	 * Returns a Signal String name of a processor implementation
	 * <p>
	 * if impl_name = "ttt.iii", return a name = "ttt_iii_processor"
	 * 
	 * @param impl_name a String name of the processor implementation
	 * @return a String name 
	 */
	public String Name_of_Processor(String impl_name){ 
		return impl_name.replace('.','_') + "_Processor";		 	
	}


	/**
	 * Returns a Signal String name of a bus implementation
	 * <p>
	 * if impl_name = "ttt.iii", return a name = "ttt_iii_bus"
	 * 
	 * @param impl_name a String name of the bus implementation
	 * @return a String name 
	 */
	public String Name_of_Bus(String impl_name){ 
		return impl_name.replace('.','_') + "_Bus";	
	}


	/**
	 * Returns a Signal String name of a thread implementation
	 * <p>
	 * if impl_name = "ttt.iii", return a name = "ttt_iii_thread"
	 * 
	 * @param impl_name a String name of the thread implementation
	 * @return a String name 
	 */
	public String Name_of_Thread(String impl_name){ 
		return impl_name.replace('.','_') + "_Thread"; 	
	}


	/**
	 * Returns a Signal String name of a data implementation
	 * <p>
	 * if impl_name = "ttt.iii", return a name = "ttt_iii_data"
	 * 
	 * @param impl_name a String name of the data implementation
	 * @return a String name 
	 */
	public String Name_of_Data(String impl_name){ 

		String name = impl_name.replace('.','_'); // + "_Data";
		// if (myLib.Is_PredefinedType(impl_name)) name = impl_name;	removed Lbesnard.
		return name;
	}

	/**
	 * Returns a Signal String name of a data implementation
	 * <p>
	 * if impl_name = "ttt.iii", return a name = "ttt_iii_data"
	 * 
	 * @param impl_name a String name of the data implementation
	 * @return a String name 
	 */
	public String Name_of_DataType(String tname){ 
		if (tname == null) return "integer";
		return tname;
	}

	/**
	 * Returns a Signal String name of a subprogram implementation
	 * <p>
	 * if impl_name = "ttt.iii", return a name = "ttt_iii_subprogram"
	 * 
	 * @param impl_name a String name of the subprogram implementation
	 * @return a String name 
	 */
	public String Name_of_Subprogram(String impl_name){ 
		return  impl_name.replace('.','_') + "_Subprogram";
	}


	/**
	 * Returns a Signal String name of a connection
	 * <p>
	 * if impl_name = "ttt.iii", return a name = "ttt_iii_Connection"
	 * 
	 * @param impl_name a String name of the connection
	 * @return a String name 
	 */
	public String Name_of_Connection(String cname){ 
		return  cname.replace('.','_') + "_Connection";	
	}


	/**
	 * Returns a Signal String name of a parameter connection
	 * <p>
	 * if impl_name = "ttt.iii", return a name = "ttt_iii_Paramete_Connection"
	 * 
	 * @param impl_name a String name of the connection
	 * @return a String name 
	 */
	public String Name_of_ParameterConnection(String cname){
		return cname.replace('.','_') + "_Parameter_Connection";
	}



	/** 
	 * Returns a String name of a thread data port
	 * <p>
	 * If it is an in data port, return a name "ttt_InDataPort", where ttt is the port name, 
	 * If it is an out data port, return a name "ttt_OutDataPort", 
	 * If it is an in out data port, return "not implemented"
	 * 
	 * @param pname a String name of the port
	 * @param inout a String that indicates the port direction: in, out or inout
	 * @return a Signal String name that represents a thread data port
	 */
	public String Name_of_ThreadDataPort(String pname, String inout){
		if (inout == "in") return pname + "_InDataPort";
		if (inout == "out") return pname + "_OutDataPort";
		return "DataPortNotImplemented";
	}


	/** 
	 * Returns a String name of a thread event  port
	 * <p>
	 * If it is an in event port, return a name "ttt_InEventPort", where ttt is the port name, 
	 * If it is an out event port, return a name "ttt_OutEventPort", 
	 * If it is an in out event port, return "not implemented"
	 * 
	 * @param pname a String name of the port
	 * @param inout a String that indicates the port direction: in, out or inout
	 * @return a Signal String name that represents a thread event port
	 */
	public String Name_of_ThreadEventPort(String pname, String inout) {  
		if (inout == "in")	return pname + "_InEventPort";
		if (inout == "out") return pname + "_OutEventPort";
		return "EventPortNotImplemented";
	}


	/** 
	 * Returns a String name of a thread event data port
	 * <p>
	 * If it is an in event data port, return a name "ttt_InEventDataPort", where ttt is the port name, 
	 * If it is an out event data port, return a name "ttt_OutEventDataPort", 
	 * If it is an in out event data port, return "not implemented"
	 * 
	 * @param pname a String name of the port
	 * @param inout a String that indicates the port direction: in, out or inout
	 * @return a Signal String name that represents a thread event data port
	 */
	public String Name_of_ThreadEventDataPort(String pname, String inout){
		if (inout == "in")  return pname + "_InEventDataPort";
		if (inout == "out") return  pname + "_OutEventDataPort";
		return "EventDataPortNotImplemented";
	}

	/** 
	 * Returns a String name of a local signal
	 * <p>
	 * if the original name is xx, return a name = "l_xx"
	 * 
	 * @param name a String name
	 * @return a String name
	 */
	public String Name_of_Local_Signal(String name){		 
		return "l_"+ name;		 	
	}


	/**
	 * Returns a Signal String name that represents the name of a process type
	 * <p>
	 * Return a string of "TT_xx", where xx is the original string
	 * 
	 * @param name a string name
	 * @return a string name
	 */
	public String Name_of_Type_Process(String name){
		return "TT_" + name;
	}


	/**
	 * Returns a Signal String name that represents the name of a bundle type: CTL1 = bundle (event Dispatch; event Resume; event Deadline;);
	 * <p>
	 * Return a string of "CTL1"
	 * 
	 * @return a string name of a bundle type
	 */
	public String Name_of_CTL1_type(){
		return "CTL1"; 
	}


	/**
	 * Returns a Signal String name that represents the name of a bundle type: CTL2 = bundle (event Complete; event Error;);
	 * <p>
	 * Return a string of "CTL1"
	 * 
	 * @return a string name of a bundle type
	 */
	public String Name_of_CTL2_type(){
		return "CTL2";
	}


	/**
	 * @return a Signal String name that represents the name of a bundle type that represents the thread input/output time events
	 */
	public String Name_of_ThreadTimeBundle(String name){		
		return name + "_TIME_EVENT"; 
	}


	/**
	 * @return a Signal String name that represents the name of a struct type that represents the in parameters.
	 */
	public String Name_of_Subprogram_Struct_In(String name) {
		return name + "_IN";
	}

	/**
	 * @return a Signal String name that represents the name of a structure type 
	 * that represents the out parameters and out event/event data ports.
	 */
	public String Name_of_Subprogram_Struct_Out(String name) {
		return name + "_OUT";
	}

	public String Name_of_Memory(String name){		
		return name.replace('.','_') + "_Memory";
	}


	/**
	 * @return a Signal string name that represents the name of a bundle type that represents a AADL port group. 
	 */
	public String Name_of_PortGroupType(String name, String in) {
		return name + "_" + in;
	}


	public String Name_of_PortGroup(String name, String in){
		return name + "_" + in;
	}


	/**
	 * @return a Signal string name that represents the name of a bundle type that represents a AADL feature group.
	 */
	public String Name_of_FeatureGroupType(String name, String in){
		return name + "_" + in;
	}


	/**
	 * @return the concatenation of the operands (separator _).
	 */
	public String Name_of_FeatureGroup(String name, String in){		
		return  name + "_" + in;	 
	}

	/**
	 * * @return the concatenation of the operand with "Flow" (separator _).
	 */
	public String Name_of_Flow(String name){		
		return name + "_Flow"; 
	}


	public String Name_of_TriggerDispatch_Event(String name){
		return  name + "_TE";

	}


	/**
	 * @return a string name that represents the bundle signal of (shared) requires data access (name)
	 */
	public String Name_of_DataBundle(String name){		
		return name;
	}

	/**
	 * @return  a string name that represents the read time of a data (name)	 
	 */
	public String Name_of_DataReadTime(String name) {
		return name + "_ReadTime";
	}

	/**
	 * @return a string name that represents the write time of a data (name)	 
	 */
	public String Name_of_DataWriteTime(String name){
		return  name + "_WriteTime";
	}


	/**
	 * @return  returns a string name that represents the reset time of a data (name)
	 */
	public String Name_of_DataReset(String name){
		return  name + "_reset";	 
	}


	/**
	 *  @return   a string name that represents the call time of a subprogram (name)
	 */
	public String Name_of_SubprogramCall(String name){	 
		return name;
	}

	/**
	 * Gets the short name (sname) of a given string name (str) from a list of LocalName
	 * 
	 * @param list a list of LocalName
	 * @param str a String of long name
	 * @return a String of short name
	 */
	private String Get_sname(ArrayList<LocalName> list, String str){
		if (list.size() >0)
			for(int i=0; i<list.size(); i++)	
			{
				LocalName ln = list.get(i);
				if (ln.lname.equals(str)) return ln.sname;
			}
		// if this long name str does not exist in the current list, add it to the list, and return the short name
		String sn = Set_sname();
		Add_name(list, str, sn);
		return sn;
	}


	/**
	 * Gets the long name (lname) of a given string name (str) from a list of LocalName
	 * 
	 * @param list a list of LocalName
	 * @param str a String of short name
	 * @return a String of long name
	 */
	private String Get_lname(ArrayList<LocalName> list, String str){
		for(int i=0; i<list.size(); i++)
		{
			LocalName ln = list.get(i);
			if (ln.sname.equals(str)) return ln.lname;
		}
		return str;
	}


	/**
	 * Set a short name as "L_i", where i is the current index
	 * @return a String name
	 */
	private String Set_sname(){
		String str = "L_" + Integer.toString(index);
		index = index +1;
		return str;
	}


	/**
	 * Adds a LocalName (ln, sn) to a list of LocalName (list)
	 * 
	 * @param list a list of LocalName
	 * @param ln a String of long name
	 * @param sn a String of short name
	 * 
	 * @return 
	 */
	private void Add_name(ArrayList<LocalName> list, String ln, String sn){
		LocalName name = new LocalName();
		name.lname = ln;
		name.sname = sn;
		list.add(name);
	}


	/**
	 * Returns a short Signal name that represents the top signal. 
	 * "lname" is the original top name: subsys_subprocessor_top, 
	 * get its short name from the name list: L_xx, 
	 * then set L_xx's short name as: top_xx
	 * 
	 * @param lname a string name that represents the original long name of a subprocessor's top
	 * @return
	 */
	public String Get_top_sname(String lname){
		String sname1 = Get_sname(NameList, lname);
		String top_sname = sname1.replaceFirst("L_", "top_");
		Add_name(NameList, sname1, top_sname);
		return top_sname;
	}


	/**
	 * Returns a string of comments that gives a brief description of a process instance
	 * 
	 * @param process_name the name of a process
	 * @return a string
	 */
	public String Generate_Process_Instance_comment(String process_name){
		String str = "process instance";
		if (process_name.endsWith("_System")){
			String str1 = process_name.replace("_System", "");
			return  "instance of AADL system " + str1.replace("_", ".");
		}
		if (process_name.endsWith("_Process")){
			String str1 = process_name.replace("_Process", "");
			return  "instance of AADL process " + str1.replace("_", ".");
		}
		if (process_name.endsWith("_Thread")){
			String str1 = process_name.replace("_Thread", "");
			return  "instance of AADL thread " + str1.replace("_", ".");
		}
		if (process_name.endsWith("_Data")){
			String str1 = process_name.replace("_Data", "");
			return  "instance of AADL data " + str1.replace("_", ".");
		}
		if (process_name.endsWith("_Processor")){
			String str1 = process_name.replace("_Processor", "");
			return  "instance of AADL processor " + str1.replace("_", ".");
		}
		if (process_name.endsWith("_Device")){
			String str1 = process_name.replace("_Device", "");
			return  "instance of AADL device " + str1.replace("_", ".");
		}
		if (process_name.endsWith("_Bus")){
			String str1 = process_name.replace("_Bus", "");
			return "instance of AADL bus " + str1.replace("_", ".");
		}
		return str;
	};

	public String getSname(String vname) {
		return Get_sname (NameList, vname);
	} 

}
