 
 
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.osate.aadl2.*;
import org.eclipse.pop.ssme.impl.*;
import org.osate.ba.aadlba.*;
import org.osate.ba.aadlba.impl.BehaviorAnnexImpl;
/* import fr.tpt.aadl.annex.behavior.aadlba.AssignmentAction;
import fr.tpt.aadl.annex.behavior.aadlba.BehaviorActionBlock;
import fr.tpt.aadl.annex.behavior.aadlba.BehaviorActions;
import fr.tpt.aadl.annex.behavior.aadlba.BehaviorCondition;
import fr.tpt.aadl.annex.behavior.aadlba.BehaviorState;
import fr.tpt.aadl.annex.behavior.aadlba.BehaviorTransition;
import fr.tpt.aadl.annex.behavior.aadlba.BehaviorVariable;
import fr.tpt.aadl.annex.behavior.aadlba.impl.BehaviorAnnexImpl;
import fr.tpt.aadl.annex.behavior.aadlba.impl.DispatchConditionImpl; */
import org.osate.ba.aadlba.impl.DispatchConditionImpl;

public class ASME2SSME_Translate_Package extends ASME2SSME_Translate_Name {

	private AADLCoLAPI aadlColAPI ;
	private signalTreeAPI myLib;
	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_Package() {
		aadlColAPI = new AADLCoLAPI();
		myLib= new signalTreeAPI();
	}


	/**
	 * an AADL package is translated as a Signal module, 
	 * the public and private section are translated separately. 
	 * (For the current version, we do not consider public or private)
	 * 
	 * @param ap
	 * @param vmodule
	 */
	public void ASME2SSME_AADLPackage(AadlPackage ap, ModuleImpl vmodule){

			
		String pkg_name = aadlColAPI.AadlPackage_GetName(ap);			
		Trace("\n == ASME2SSME_Translate_AADLPackage: ASME2SSME_AADLPackage : "+ pkg_name);

		//public package
		if(aadlColAPI.AadlPackage_GetPublicPkg(ap) != null){
			PublicPackageSection pub_pkg = aadlColAPI.AadlPackage_GetPublicPkg(ap);
			ASME2SSME_PublicPackageSection(pub_pkg, vmodule);
		}

		//private package
		if(aadlColAPI.AadlPackage_GetPrivatePkg(ap) != null){
			PrivatePackageSection private_pkg = aadlColAPI.AadlPackage_GetPrivatePkg(ap);
			ASME2SSME_PrivatePackageSection(private_pkg, vmodule);
		}

		// test Behaviour Annex
		ASME2SSME_Test_BA(ap);  			
		//data 
		ASME2SSME_Data(ap, vmodule); 		
		// feature group type in OSATE2  
		ASME2SSME_FeatureGroup(ap, vmodule);  		
		// subprogram  
		// ASME2SSME_Subprogram(ap, vmodule);  								
		//translate main system 		
		SystemImplementation main_sys = aadlColAPI.AadlPackage_GetMainSystem(ap);

		if(main_sys != null)
			ASME2SSME_MainSystem(main_sys, vmodule, ap);
		else { //no main system exists, then translates each element
			// Process  
			ASME2SSME_Process(ap, vmodule);			
			//Processor 
			ASME2SSME_Processor(ap, vmodule);			
			//Device 
			ASME2SSME_Device(ap, vmodule);  			
			//Bus 
			ASME2SSME_Bus(ap, vmodule); 			
			//Memory 
			ASME2SSME_Memory(ap, vmodule);
		}				
	}


	/**
	 * Test whether BA part is accessable
	 * 
	 * @param ap AADL package
	 */
	private void ASME2SSME_Test_BA(AadlPackage ap){

		
		String pkg_name = aadlColAPI.AadlPackage_GetName(ap);			
		Trace("\n == ASME2SSME_Translate_AADLPackage: ASME2SSME_Test_BA : "+ pkg_name);

		ArrayList<ThreadImplementation> timpl_list = aadlColAPI.AadlPackage_GetThreadImpl(ap);
		if(timpl_list.size()>0){
			for(int xx =0; xx<timpl_list.size();xx++){
				ThreadImplementation t = timpl_list.get(xx);

				/*********test BA access*************************/
				Trace("\n *********** ba test: ");
				EList<AnnexSubclause> annex_list = t.getOwnedAnnexSubclauses();
				if(annex_list != null){

					for(int ai =0; ai<annex_list.size(); ai++){
						AnnexSubclause annex1 = annex_list.get(ai);
						if(annex1 instanceof BehaviorAnnexImpl){
							BehaviorAnnexImpl o = (BehaviorAnnexImpl) annex1;
							Trace("\n **** BehaviorAnnexImpl: "  + o.toString());

							BehaviorAnnexImpl ba = (BehaviorAnnexImpl) o;

							/******************************/
							EList<BehaviorState> state_list= ba.getStates();
							if(state_list!= null){
								for(int j=0; j< state_list.size(); j++){
									BehaviorState state_i = state_list.get(j);
									Trace("\n ****** State : " + state_i.getName());
								}
							}

							/******************************/
							EList<BehaviorVariable> variable_list = ba.getVariables();
							if(variable_list!= null){
								for(int k=0; k< variable_list.size(); k++){
									BehaviorVariable variable_i = variable_list.get(k);
									Trace("\n ****** Variable : " + variable_i.getName());
								}
							}

							/******************************/
							EList<BehaviorTransition> trans_list = ba.getTransitions();
							if(trans_list != null){
								for(int m =0; m< trans_list.size(); m++){
									BehaviorTransition trans_i = trans_list.get(m);
									Trace("\n ****** BehaviorTransition : " + trans_i.getName() + " " + trans_i.toString());

									BehaviorState source = trans_i.getSourceState();
									Trace("\n ****** Source state : " + source.getName());

									BehaviorCondition cond = trans_i.getCondition();
									if (cond != null){
										Trace("\n ****** BehaviorCondition : " + cond.toString());

										if(cond instanceof DispatchConditionImpl){
											Trace("\n ****** DispatchCondition : ");
										}
									}

									BehaviorState dst = trans_i.getDestinationState();
									Trace("\n ****** Dest state : " + dst.getName());

									BehaviorActionBlock actionblock = trans_i.getActionBlock();
									if(actionblock != null){
										Trace("\n ****** Action Block : " + actionblock.toString());
										//AssignmentAction bas = (AssignmentAction) actionblock.getContent();
										BehaviorActions bas =  actionblock.getContent();
										Trace("\n ****** BehaviorActions : " + bas.toString());
										if(bas instanceof AssignmentAction){
											AssignmentAction aa = (AssignmentAction) bas;
											Trace("\n ****** AssignmentAction : " + aa.getTarget().toString());
										}
									}

								}
							}
							/*******************************/

						}//end if instanceof BA

					}//end for ai

				} /********** end test BA access**********************************/
			}
		}//end test ba


	}


	/**
	 * Translate data of aadl package ap
	 * 
	 * @param ap
	 * @param vmodule
	 */
	private void ASME2SSME_Data(AadlPackage ap, ModuleImpl vmodule){

		
		String pkg_name = aadlColAPI.AadlPackage_GetName(ap);				
		Trace("\n == ASME2SSME_Translate_AADLPackage: ASME2SSME_Data: "+ pkg_name);

		// Data types (type)
		ArrayList<DataType> listDataType = aadlColAPI.AadlPackage_GetdataType(ap);
		if(listDataType.size()>0){
			ASME2SSME_Translate_Data asme2ssme_data1 = new ASME2SSME_Translate_Data();
			Iterator<DataType> it_data = listDataType.iterator();
			while (it_data.hasNext())
			{
				DataType data1 = (DataType) it_data.next(); 
				if(asme2ssme_data1.ASME2SSME_DataType(data1) != null){
					//type xx
					TypesDeclarationImpl type_desc = asme2ssme_data1.ASME2SSME_DataType(data1);			
					myLib.STree_addaTypeDeclarationToModule(vmodule, type_desc);
					//constant xx xx_INIT =external;
					ConstantsDeclarationImpl constant_init1 = asme2ssme_data1.ASME2SSME_DataType_Init(data1);
					myLib.STree_addaConstantToModule(vmodule, constant_init1);
				}					
			}
		}

		// Data types (implementation)
		ArrayList<DataImplementation> listDataImpl = aadlColAPI.AadlPackage_GetdataImpl(ap);
		if(listDataImpl.size()>0){
			ASME2SSME_Translate_Data asme2ssme_data2 = new ASME2SSME_Translate_Data();
			Iterator<DataImplementation> it2 = listDataImpl.iterator();
			while (it2.hasNext())
			{
				DataImplementation data2 = (DataImplementation) it2.next();                
				//type xx
				TypesDeclarationImpl type_desc = asme2ssme_data2.ASME2SSME_DataImpl(data2);			
				myLib.STree_addaTypeDeclarationToModule(vmodule, type_desc);
				//constant xx xx_INIT =external;
				ConstantsDeclarationImpl constant_init2 = asme2ssme_data2.ASME2SSME_DataImpl_Init(data2);
				myLib.STree_addaConstantToModule(vmodule, constant_init2);
			}
		}
	}


	/**
	 * Translate featuregroups of aadl package ap
	 * 
	 * @param ap
	 * @param vmodule
	 */
	private void ASME2SSME_FeatureGroup(AadlPackage ap, ModuleImpl vmodule){
		
		String pkg_name = aadlColAPI.AadlPackage_GetName(ap);				
		Trace("\n == ASME2SSME_Translate_AADLPackage: ASME2SSME_FeatureGroup: "+ pkg_name);

		ArrayList<FeatureGroupType> listprt = aadlColAPI.AadlPackage_GetFeatureGroupType(ap);
		if(listprt.size()>0){
			ASME2SSME_Translate_FeatureGroup asme2ssme_pg = new ASME2SSME_Translate_FeatureGroup();

			for(int i1 =0; i1 < listprt.size(); i1 ++){
				FeatureGroupType pgt = listprt.get(i1);
				if(asme2ssme_pg.ASME2SSME_FeatureGroupType_BW(pgt) != null){
					TypesDeclarationImpl type_decl1 = asme2ssme_pg.ASME2SSME_FeatureGroupType_BW(pgt);
					myLib.STree_addaTypeDeclarationToModule(vmodule, type_decl1);
				}
				if(asme2ssme_pg.ASME2SSME_FeatureGroupType_WB(pgt) != null){
					TypesDeclarationImpl type_decl2 = asme2ssme_pg.ASME2SSME_FeatureGroupType_WB(pgt);		
					myLib.STree_addaTypeDeclarationToModule(vmodule, type_decl2);
				}		
			}			
		}
	}


	/**
	 * Translate subprograms of aadl package ap
	 * 
	 * @param ap
	 * @param vmodule
	 * 
	 */
	private void UNUSED_ASME2SSME_Subprogram(AadlPackage ap, ModuleImpl vmodule){
		String pkg_name = aadlColAPI.AadlPackage_GetName(ap);				
		Trace("\n == ASME2SSME_Translate_AADLPackage: ASME2SSME_Subprogram: "+ pkg_name);

		//subprogram implementation
		ArrayList<SubprogramImplementation> list_subp_impl = aadlColAPI.AadlPackage_GetSubprogramImpl(ap);
		ArrayList<SubprogramType> list_x = new ArrayList<SubprogramType>();
		if(list_subp_impl.size() >0){
			ASME2SSME_Translate_Subprogram asme2ssme_sp = new ASME2SSME_Translate_Subprogram();
			for(int j=0; j<list_subp_impl.size(); j++){
				SubprogramImplementation subp1 = list_subp_impl.get(j);
				SubprogramType subp1_t = (SubprogramType) aadlColAPI.ComponentImpl_GetComponentType(subp1);
				if(! list_x.contains(subp1_t)){
					list_x.add(subp1_t);
				}

				ProcessModelImpl sp_process1 = asme2ssme_sp.ASME2SSME_SubprogramImpl(subp1,vmodule);
				myLib.STree_addProcessDeclaration(vmodule, sp_process1);					
			}
		}
		//subprogram type (if no implementation exists)
		ArrayList<SubprogramType> list_subp_type = aadlColAPI.AadlPackage_GetSubprogramType(ap);
		list_subp_type.removeAll(list_x);
		if(list_subp_type.size()>0){
			ASME2SSME_Translate_Subprogram asme2ssme_sp = new ASME2SSME_Translate_Subprogram();
			for(int j1=0; j1<list_subp_type.size(); j1++){
				SubprogramType spt1 = list_subp_type.get(j1);
				ProcessModelImpl sp_process = asme2ssme_sp.ASME2SSME_SubprogramType(spt1, vmodule);
				myLib.STree_addProcessDeclaration(vmodule, sp_process);	
			}
		}
	}


	/**
	 * Translate processes of aadl package ap
	 * 
	 * @param ap
	 * @param vmodule
	 */
	private void ASME2SSME_Process(AadlPackage ap, ModuleImpl vmodule){

		String pkg_name = aadlColAPI.AadlPackage_GetName(ap);				
		Trace("\n == ASME2SSME_Translate_AADLPackage: ASME2SSME_Process: "+ pkg_name);

		ASME2SSME_Translate_Process asme2ssme_process = new ASME2SSME_Translate_Process();
		ArrayList<ProcessImplementation> pi_list = aadlColAPI.AadlPackage_GetProcessImpl(ap);			
		for(int m1 =0; m1 < pi_list.size(); m1++){
			ProcessImplementation pi1 = pi_list.get(m1);					
			org.osate.aadl2.ProcessType pt1 = aadlColAPI.Process_GetCompType(pi1);
			TypesDeclarationImpl type_decl = asme2ssme_process.ASME2SSME_ProcessType(pt1, pi1);
			myLib.STree_addaTypeDeclarationToModule(vmodule, type_decl);
			ProcessModelImpl lprocess4 = asme2ssme_process.ASME2SSME_ProcessImpl(pi1, vmodule, vmodule, ap);			
			myLib.STree_addProcessDeclaration(vmodule, lprocess4);	
		}
	}



	/**
	 * Translate processors of aadl package ap
	 * 
	 * @param ap
	 * @param vmodule
	 */
	private void ASME2SSME_Processor(AadlPackage ap, ModuleImpl vmodule){

		String pkg_name = aadlColAPI.AadlPackage_GetName(ap);				
		Trace("\n == ASME2SSME_Translate_AADLPackage: ASME2SSME_Processor: "+ pkg_name);

		ASME2SSME_Translate_Processor asme2ssme_processor = new ASME2SSME_Translate_Processor();
		ArrayList<ProcessorType> list1 = aadlColAPI.AadlPackage_GetProcessorType(ap);			
		for(int m2 =0; m2 < list1.size(); m2++){
			ProcessorType p1 = list1.get(m2);
			asme2ssme_processor.ASME2SSME_ProcessorType(p1);
		}
	}


	/**
	 * Translate devices of aadl package ap
	 * 
	 * @param ap
	 * @param vmodule
	 */
	private void ASME2SSME_Device(AadlPackage ap, ModuleImpl vmodule){

		String pkg_name = aadlColAPI.AadlPackage_GetName(ap);				
		Trace("\n == ASME2SSME_Translate_AADLPackage: ASME2SSME_Device: "+ pkg_name);

		ASME2SSME_Translate_Device asme2ssme_device = new ASME2SSME_Translate_Device();
		ArrayList<DeviceImplementation> list2 = aadlColAPI.AadlPackage_GetDeviceImpl(ap);			
		for(int m3 =0; m3 < list2.size(); m3++){
			DeviceImplementation p2 = list2.get(m3);
			ProcessModelImpl pm1 = asme2ssme_device.ASME2SSME_DeviceImpl(p2);		
			myLib.STree_addProcessDeclaration(vmodule, pm1);	
		}
	}


	/**
	 * Translate buses of aadl package ap
	 * 
	 * @param ap
	 * @param vmodule
	 */
	private void ASME2SSME_Bus(AadlPackage ap, ModuleImpl vmodule){
		String pkg_name = aadlColAPI.AadlPackage_GetName(ap);				
		Trace("\n == ASME2SSME_Translate_AADLPackage: ASME2SSME_Bus: "+ pkg_name);

		ASME2SSME_Translate_Bus asme2ssme_bus = new ASME2SSME_Translate_Bus();
		ArrayList<BusImplementation> list3 = aadlColAPI.AadlPackage_GetBusImpl(ap);			
		for(int m4 =0; m4 < list3.size(); m4++){
			BusImplementation p2 = list3.get(m4);
			ProcessModelImpl pm1 = asme2ssme_bus.ASME2SSME_BusImpl(p2);		
			myLib.STree_addProcessDeclaration(vmodule, pm1);	
		}
	}


	/**
	 * Translate memories of aadl package ap
	 * 
	 * @param ap
	 * @param vmodule
	 */
	private void ASME2SSME_Memory(AadlPackage ap, ModuleImpl vmodule){
		String pkg_name = aadlColAPI.AadlPackage_GetName(ap);				
		Trace("\n == ASME2SSME_Translate_AADLPackage: ASME2SSME_Memory: "+ pkg_name);

		ASME2SSME_Translate_Memory asme2ssme_memory = new ASME2SSME_Translate_Memory();
		ArrayList<MemoryImplementation> list4 = aadlColAPI.AadlPackage_GetMemoryImpl(ap);			
		for(int m5 =0; m5 < list4.size(); m5++){
			MemoryImplementation p5 = list4.get(m5);
			ProcessModelImpl pm1 = asme2ssme_memory.ASME2SSME_MemoryImpl(p5);	
			myLib.STree_addProcessDeclaration(vmodule, pm1);	
		}	


	}



	/**
	 * Translate a private package section
	 * 
	 * @param pkg
	 * @param vmodule
	 */
	private void ASME2SSME_PrivatePackageSection(PrivatePackageSection pkg, ModuleImpl vmodule){
		String pkg_name = aadlColAPI.Package_GetName(pkg);		
		Trace("\n == ASME2SSME_Translate_AADLPackage: ASME2SSME_PrivatePackageSection: "+ pkg_name);

		//ASME2SSME_PackageSection(pkg, vmodule);

	}


	/**
	 * Translate a public package section
	 * s
	 * @param pkg
	 * @param vmodule
	 */
	private void ASME2SSME_PublicPackageSection(PublicPackageSection pkg, ModuleImpl vmodule){
		String pkg_name = aadlColAPI.Package_GetName(pkg);
		Trace("\n == ASME2SSME_Translate_AADLPackage: ASME2SSME_PublicPackageSection: "+ pkg_name);

		//ASME2SSME_PackageSection(pkg, vmodule);

	}


	/**
	 * Translate a package section
	 * 
	 * @param pkg
	 * @param vmodule
	 */
	private void UNUSED_ASME2SSME_PackageSection(PackageSection pkg, ModuleImpl vmodule){
		String pkg_name = aadlColAPI.Package_GetName(pkg);
		Trace("\n == ASME2SSME_Translate_AADLPackage: ASME2SSME_PackageSection: "+ pkg_name);
		Trace("\n ************************* end ASME2SSME_PackageSection ************************* \n ");
	}



	/**
	 * returns a Signal process that represents a top level AADL system
	 * <p>
	 * gets this top level main system from an AADL specification, 
	 * translates it to a Signal process
	 * 	
	 * @param aadlRootModel an AADL specification
	 * @return a Signal process (ProcessModelImpl object)
	 */
	private void ASME2SSME_MainSystem(SystemImplementation main_sys, ModuleImpl vmodule, AadlPackage aadlpkg){

		Trace("\n **** ASME2SSME_Translate_AADLPackage: ASME2SSME_MainSystem " + main_sys.getName());

		ASME2SSME_Translate_System asme2ssme_system = new ASME2SSME_Translate_System();

		if(main_sys != null){

			ProcessModelImpl vprocess = asme2ssme_system.ASME2SSME_SystemImpl(main_sys, vmodule, aadlpkg);					
			myLib.STree_addProcessDeclaration(vmodule, vprocess);
		}
		Trace("\n ************************* end ASME2SSME_MainSystem ************************* \n ");
	}
}
