
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import org.eclipse.emf.common.util.EList;
import org.osate.aadl2.*;

import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;
import org.eclipse.pop.ssme.polychrony.SignalOperators;


public class ASME2SSME_Translate_ParameterConnection extends ASME2SSME_Translate_Connection{
	private signalTreeAPI myLib  ;
	private AADLCoLAPI aadlColAPI;
	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_ParameterConnection() {
		myLib= new signalTreeAPI();
		aadlColAPI = new AADLCoLAPI();
	}




	/** 
	 * Returns a Signal signal definition (an equation) that represents a parameter connection.
	 * 
	 * @param compImpl a component implementation component
	 * @param vprocess a Signal process that represents an AADL component (a thread impl) which contains this connection
	 * @param cc a parameter connection
	 */
	public SignalsDefinitionImpl ASME2SSME_ParameterConnection(ComponentImplementation compImpl, ProcessModelImpl vprocess, ParameterConnection cc){

		Trace("===== ASME2SSME_Translate_ParameterConnection: ASME2SSME_ParameterConnection === : " + cc.getName());

		SignalsDefinitionImpl eq = new SignalsDefinitionImpl();
		String connection_name = aadlColAPI.Connection_GetName(cc);
		//Trace(connection_name);
		String pc_process_name = Name_of_ParameterConnection(connection_name);

		Context src_context = aadlColAPI.Connection_GetSrcContext(cc);
		Context dst_context = aadlColAPI.Connection_GetDstContext(cc);
		ConnectionEnd src_end = aadlColAPI.Connection_GetSrc(cc);
		ConnectionEnd dst_end = aadlColAPI.Connection_GetDst(cc);
		String left_name;
		String right_name;

		ProcessModelImpl pc_def = ASME2SSME_ParameterConnection_Definition(compImpl, cc);
		myLib.STree_addProcessDeclaration(vprocess, pc_def);

		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();

		// thread ->
		if(src_context instanceof ThreadImplementation) {

			right_name = "l_"+ src_end.getName();
			ExprSignal right_expr = myLib.mkIdentifier(right_name);
			lins.add(right_expr);

			// thread -> subprogram subcomponent
			if (dst_context instanceof SubprogramSubcomponent){

				SubprogramSubcomponent dst_subp = (SubprogramSubcomponent) dst_context;
				left_name =  dst_subp.getName() + "_" + dst_end.getName();
				ExprSignal left_expr = myLib.mkIdentifier(left_name);	
				louts.add(left_expr);

				DataClassifier data_classifier = (DataClassifier) ((Parameter) dst_end).getClassifier();
				String str_type_name = Name_of_Reference_data_classifier(data_classifier);
				lparams.add(myLib.mkIdentifier(str_type_name));

				eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(pc_process_name), lparams, lins, louts);
				return eq;
			}
			else 
				Trace("ASME2SSME_ParameterConnection dst context not subprogram subcomponent, not implemented");
		}
		//subprogram subcomponent ->
		else if(src_context instanceof SubprogramSubcomponent){

			SubprogramSubcomponent src_subp = (SubprogramSubcomponent) src_context;
			right_name =  src_subp.getName() + "_" + src_end.getName();
			ExprSignal right_expr = myLib.mkIdentifier(right_name);
			lins.add(right_expr);

			DataClassifier data_classifier = (DataClassifier) ((Parameter)src_end).getClassifier();
			String str_type_name = Name_of_Reference_data_classifier(data_classifier);
			lparams.add(myLib.mkIdentifier(str_type_name));

			// subprogram subcomponent -> thread
			if (dst_context instanceof ThreadImplementation){

				left_name =  "l_"+ dst_end.getName();
				ExprSignal left_expr = myLib.mkIdentifier(left_name);				
				louts.add(left_expr);					    
				eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(pc_process_name), lparams, lins, louts);	
				return eq;
			}

			// subprogram subcomponent -> subprogram subcomponent
			else if (dst_context instanceof SubprogramSubcomponent){

				SubprogramSubcomponent dst_subp = (SubprogramSubcomponent) dst_context;
				left_name =  dst_subp.getName() + "_" + dst_end.getName();
				ExprSignal left_expr = myLib.mkIdentifier(left_name);			
				louts.add(left_expr);		    
				eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(pc_process_name), lparams, lins, louts); 
				return eq;
			}
			else 
				Trace("ASME2SSME_ParameterConnection src context subprogram subcomponent, not implemented");
		}	
		else 
			Trace("ASME2SSME_ParameterConnection cases not implemented");
		return null;
	}



	/** 
	 * Returns a Signal process that represents a data port connection between two processes that bound to different processors.
	 * <p>
	 * This Signal process is named as xx_DataPortConnection, where xx is the name of this data port connection. 
	 * this process is composed of two subprocesses: xx_DataPortConnection_Behavior(), xx_DataPortConnection_Property()
	 * xx_DataPortConnection_Behavior() calls the Connection() process from the library. 
	 * xx_DataPortConnection_Property() calls the properties, for example, 
	 * Latency_property{...}(). 
	 * All the parameters are translated from the Latency property
	 *  
	 *  @param compImpl a component implementation component
	 *  @param dconnection a data port connection
	 *  @return a Signal process call
	 */	
	public ProcessModelImpl ASME2SSME_ParameterConnection_Definition(ComponentImplementation compImpl, ParameterConnection cc){		

		//	   * xx_Parameter_Connection =
		//         * {type item_type;}	
		//		   * (? item_type write_data; ! item_type read_data;)
		//		   * (| read_data := write_data
		//		   *  | xx_Parameter_Connection_Property{}()
		//		   *  |)
		//		   *  where
		//		   *    process xx_Parameter_Connection_Property{}()
		//		   *  end;

		String connection_name = aadlColAPI.Connection_GetName(cc);
		String connection_process_name = Name_of_ParameterConnection(connection_name);

		Trace("========= ASME2SSME_Translate_ParameterConnection: ASME2SSME_ParameterConnection_Definition : " + connection_name);

		ProcessModelImpl vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(connection_process_name) );

		//parameters
		TypesDeclarationImpl item_type = new TypesDeclarationImpl();
		item_type.getTypeDefinition().add(myLib.mkIdentifier("item_type"));	
		myLib.STree_addDeclarationToInterface(vprocess, item_type, 1);

		//inputs	
		//item_type write_data;
		SignalsDeclarationImpl write_data = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("write_data"), myLib.mkIdentifier("item_type"), null);	

		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
		lparams.add(myLib.mkIdentifier("item_type"));

		// add inputs to interface
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		lins.add(myLib.mkIdentifier("write_data"));
		myLib.STree_addDeclarationToInterface(vprocess, write_data, 2);

		//outputs
		// item_type read_data;
		SignalsDeclarationImpl read_data = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("read_data"), myLib.mkIdentifier("item_type"), null);
		myLib.STree_addDeclarationToInterface(vprocess, read_data, 3);

		// add outputs to interface
		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
		louts.add(myLib.mkIdentifier("read_data"));

		//connection property
		ProcessModelImpl pc_property = ASME2SSME_ParameterConnection_property((ThreadImplementation) compImpl, connection_name);
		String pc_property_name = myLib.STree_GetStringName(pc_property.getIdentifier());//pname + "_behavior";
		//add to declaration
		myLib.STree_addProcessDeclaration(vprocess, pc_property);
		//add to body
		ProcessCallImpl pc_property_inst = myLib.STree_mkModelCall(pc_property_name);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, pc_property_inst);

		//read_data := write_data
		SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation( myLib.mkIdentifier("read_data"), myLib.mkIdentifier("write_data"));		
		myLib.STree_addExpressionOfProcessToaModel(vprocess, eq);

		return vprocess;
	}


	/**
	 * Returns a Signal process xx_Parameter_Connection_property that represents the properties of a parameter connection.
	 * <p>
	 * This process is named as xx_Parameter_Connection_property, where xxx is the connection name 
	 * Gets all the properties specified in this thread implementation and also its type component. 
	 * For each such property, ASME2SSME_Property_Call() method is used to make a process call.
	 * 
	 * @param thread_impl a thread implementation component
	 * @param pc_name a String name of parameter connection
	 * @return a ProcessModelImpl (Signal process)
	 */
	public ProcessModelImpl ASME2SSME_ParameterConnection_property(ThreadImplementation thread_impl, String pc_name){

		String pname = Name_of_ParameterConnection(pc_name) + "_property";
		Trace("========= ASME2SSME_Translate_ParameterConnection: ASME2SSME_ParameterConnection_property : " + pc_name);

		ProcessModelImpl p_property = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );

		ArrayList<PropertyAssociation> property_association_list1 = aadlColAPI.ComponentImpl_GetPropertyAssociations(thread_impl);
		ThreadType thread_type = (ThreadType) aadlColAPI.ComponentImpl_GetComponentType(thread_impl);
		ArrayList<PropertyAssociation> property_association_list2 = aadlColAPI.Component_GetPropertyAssociations(thread_type);
		property_association_list1.addAll(property_association_list2);

		if(property_association_list1.size()>0){
			for(int i=0; i<property_association_list1.size(); i++)
			{
				PropertyAssociation pa = property_association_list1.get(i);
				// String propertyDef_name = aadlColAPI.Property_Get_Name(pa);

				EList<ContainedNamedElement> list_applyto = pa.getAppliesTos();
				for (int j = 0; j < list_applyto.size(); j++) { 
					ContainedNamedElement applyto = list_applyto.get(j);
					EList<ContainmentPathElement> cpe = applyto.getContainmentPathElements();
					if(cpe != null){
						for (int k =0; k< cpe.size(); k++){
							ContainmentPathElement e1 = cpe.get(k);
							NamedElement e2 = e1.getNamedElement();
							String applyto_name = e2.getName();

							if (applyto_name.equals(pc_name)){
								ProcessCallImpl pcall1 = ASME2SSME_Property_Call(pa);
								myLib.STree_addExpressionOfProcessToaModel(p_property, pcall1);  
							}
						}
					}				
				}        
			}
		}

		return p_property;
	}


	/**
	 * for each parameter connnection (from thread to a called/required subprogram (case 1 or case2)): 
	 * c1: parameter a -> sp1.a;
	 * creates an equation: L_x.a ::= a when sp1, 
	 * and adds it to the body of thread behavior (vprocess), 
	 * where L is short for sp name, and sp1 is called name
	 * 
	 * @param vprocess a Signal process that represents the behavior process
	 * @param t a thread implementation
	 * 
	 * @return
	 */
	public void ASME2SSME_ParameterConnections_Thread_to_CalledSubprogram(ProcessModelImpl vprocess, ThreadImplementation t){

		Trace("\n ========== ASME2SSME_Translate_ParameterConnection: ASME2SSME_ParameterConnections_Thread_to_CalledSubprogram : " + t.getName());



		//get the subprogram calls of this thread
		ArrayList<SubprogramCall> list1 = new ArrayList<SubprogramCall>();
		ArrayList<SubprogramCallSequence> callseqs = aadlColAPI.BehavioredImpl_Get_CallSequences(t);    
		for(int i =0; i< callseqs.size(); i++){
			SubprogramCallSequence callseq1 = callseqs.get(i);
			ArrayList<SubprogramCall> calls = aadlColAPI.SubprogramCallSequence_Get_SubprogramCalls(callseq1);
			if(calls != null){
				list1.addAll(calls);
			}
		}

		//get parameter connections
		ArrayList<ParameterConnection> pcs = aadlColAPI.ComponentImpl_GetParameterConnections(t);

		for(int j=0; j< pcs.size(); j++){
			ParameterConnection pc1 = pcs.get(j);

			//if pc1 is connection from thread -> sp
			Context src_context = aadlColAPI.Connection_GetSrcContext(pc1);
			Context dst_context = aadlColAPI.Connection_GetDstContext(pc1);
			ConnectionEnd src_element = aadlColAPI.Connection_GetSrc(pc1);
			ConnectionEnd dst_element = aadlColAPI.Connection_GetDst(pc1);
			if((src_context == null) //&& (dst_context instanceof SubprogramCall)
					&& (src_element instanceof Port) && (dst_element instanceof Parameter)){

				Port p1 = (Port) src_element;
				Parameter p2 = (Parameter) dst_element;

				//if called subprogram
				if(dst_context instanceof SubprogramCall){
					//if sp is case1 or case 2
					CalledSubprogram csp1 = aadlColAPI.SubprogramCall_Get_CalledSubprogram((SubprogramCall) dst_context);
					if(!aadlColAPI.Is_CalledSubprogram_Sub_Of_ComponentImpl(csp1, t)){

						String sp1 = aadlColAPI.SubprogramCall_GetName((SubprogramCall) dst_context);
						String sp = aadlColAPI.CalledSubprogram_Get_Name(csp1);
						String sp2 = sp.replace(".", "_");
						String l_sp = getSname( sp2);

						//L_x.a
						ExprSignal leftExp = myLib.STree_mkFieldAccess(l_sp+"_x", aadlColAPI.Parameter_GetName(p2));

						//a when sp1
						WhenImpl rightExp = myLib.STree_mkWhen
								(myLib.mkIdentifier(aadlColAPI.Port_GetName(p1)), 
										myLib.mkIdentifier(sp1));

						//L_x.a ::= a when sp1
						SignalsPartialDefinitionImpl eq1 = (SignalsPartialDefinitionImpl) myLib.mkPartialEquation(leftExp, rightExp);

						//add to body
						myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
					}
				}
				//if required subprogram
				else if (dst_context instanceof SubprogramAccess){
					//if sp is case1 or case 2
					SubprogramClassifier csp1 = aadlColAPI.SubprogramAccess_GetClassifier((SubprogramAccess) dst_context);
					if(!aadlColAPI.Is_SubprogramClassifier_Sub_Of_ComponentImpl(csp1, t)){

						String sp1 = aadlColAPI.SubprogramAccess_GetName((SubprogramAccess) dst_context);
						String sp = aadlColAPI.Classifier_GetName(csp1);
						String sp2 = sp.replace(".", "_");
						String l_sp = getSname( sp2);

						//L_x.a
						ExprSignal leftExp = myLib.STree_mkFieldAccess(l_sp+"_x", aadlColAPI.Parameter_GetName(p2));

						//a when sp1
						WhenImpl rightExp = myLib.STree_mkWhen
								(myLib.mkIdentifier(aadlColAPI.Port_GetName(p1)), 
										myLib.mkIdentifier(sp1));

						//L_x.a ::= a when sp1
						SignalsPartialDefinitionImpl eq1 = (SignalsPartialDefinitionImpl) myLib.mkEquation(SignalOperators.opdefpart, leftExp, rightExp);

						//add to body
						myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
					}
				}

			}		
		}

		return;
	}


	/**
	 * for each parameter connnection (from a called subprogram to thread (case 1 or case2, sp is not t's subcomponent)): 
	 * c1: parameter sp1.a -> a;
	 * creates an equation: a := L_y.a when sp1, 
	 * and adds it to the body of thread behavior (vprocess), 
	 * where L is short for sp name, and sp1 is called name.
	 * Remove a from the interface (output) of behavior external process
	 * 
	 * @param vprocess a Signal process that represents the behavior process
	 * @param t a thread implementation
	 * 
	 * @return
	 */
	public void ASME2SSME_ParameterConnections_CalledSubprogram_to_Thread(ProcessModelImpl vprocess, ThreadImplementation t, ProcessModelImpl external_process){

		Trace("\n ========== ASME2SSME_Translate_ParameterConnection: ASME2SSME_ParameterConnections_CalledSubprogram_to_Thread : " + t.getName());

		//get the subprogram calls of this thread
		ArrayList<SubprogramCall> list1 = new ArrayList<SubprogramCall>();
		ArrayList<SubprogramCallSequence> callseqs = aadlColAPI.BehavioredImpl_Get_CallSequences(t);    
		for(int i =0; i< callseqs.size(); i++){
			SubprogramCallSequence callseq1 = callseqs.get(i);
			ArrayList<SubprogramCall> calls = aadlColAPI.SubprogramCallSequence_Get_SubprogramCalls(callseq1);
			if(calls != null){
				list1.addAll(calls);
			}
		}

		//get parameter connections
		ArrayList<ParameterConnection> pcs = aadlColAPI.ComponentImpl_GetParameterConnections(t);

		for(int j=0; j< pcs.size(); j++){
			ParameterConnection pc1 = pcs.get(j);

			//if pc1 is connection from sp -> thread
			Context src_context = aadlColAPI.Connection_GetSrcContext(pc1);
			Context dst_context = aadlColAPI.Connection_GetDstContext(pc1);
			ConnectionEnd src_element = aadlColAPI.Connection_GetSrc(pc1);
			ConnectionEnd dst_element = aadlColAPI.Connection_GetDst(pc1);
			if((dst_context == null ) && (dst_element instanceof Port)){

				Port p2 = (Port) dst_element;

				//if called subprogram
				if(src_context instanceof SubprogramCall){
					//if sp is case1 or case 2
					CalledSubprogram csp1 = aadlColAPI.SubprogramCall_Get_CalledSubprogram((SubprogramCall) src_context);
					if(!aadlColAPI.Is_CalledSubprogram_Sub_Of_ComponentImpl(csp1, t)){

						String sp1 = aadlColAPI.SubprogramCall_GetName((SubprogramCall) src_context);
						String sp = aadlColAPI.CalledSubprogram_Get_Name(csp1);
						String sp2 = sp.replace(".", "_");
						String l_sp = getSname( sp2);

						// a
						ExprSignal leftExp = myLib.mkIdentifier(aadlColAPI.Port_GetName(p2));

						//L_y.a when sp1
						String str1;
						if(src_element instanceof Parameter){
							str1 = aadlColAPI.Parameter_GetName((Parameter) src_element);
						}
						else{//src_element instanceof event/event data port
							str1 = aadlColAPI.Port_GetName((Port) src_element);
						}				
						ExprSignal exp = myLib.STree_mkFieldAccess(l_sp+"_y", str1);
						WhenImpl rightExp = myLib.STree_mkWhen(exp, myLib.mkIdentifier(sp1));

						//a := L_y.a when sp1, 
						SignalsDefinitionImpl eq1 = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);

						//add to body
						myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);

						//remove it from the interface of behavior_external
						myLib.STree_rmNamedDeclarationFromInterface(external_process, aadlColAPI.Port_GetName(p2), 3);
					}  	    		
				}
				else if(src_context instanceof SubprogramAccess){
					//if sp is case1 or case 2
					SubprogramClassifier csp1 = aadlColAPI.SubprogramAccess_GetClassifier((SubprogramAccess) src_context);
					if(!aadlColAPI.Is_SubprogramClassifier_Sub_Of_ComponentImpl(csp1, t)){

						String sp1 = aadlColAPI.SubprogramAccess_GetName((SubprogramAccess) dst_context);
						String sp = aadlColAPI.Classifier_GetName(csp1);
						String sp2 = sp.replace(".", "_");
						String l_sp = getSname( sp2);

						// a
						ExprSignal leftExp = myLib.mkIdentifier(aadlColAPI.Port_GetName(p2));

						//L_y.a when sp1
						String str1;
						if(src_element instanceof Parameter){
							str1 = aadlColAPI.Parameter_GetName((Parameter) src_element);
						}
						else{//src_element instanceof event/event data port
							str1 = aadlColAPI.Port_GetName((Port) src_element);
						}				
						ExprSignal exp = myLib.STree_mkFieldAccess(l_sp+"_y", str1);
						WhenImpl rightExp = myLib.STree_mkWhen(exp, myLib.mkIdentifier(sp1));

						//a := L_y.a when sp1, 
						SignalsDefinitionImpl eq1 = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);

						//add to body
						myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);

						//remove it from the interface of behavior_external
						myLib.STree_rmNamedDeclarationFromInterface(external_process, aadlColAPI.Port_GetName(p2), 3);
					}  	    		
				}			
			}		
		}

		return;
	}



}
