
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;

import org.eclipse.emf.common.util.EList;
import org.osate.aadl2.*;
import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;
import org.eclipse.pop.ssme.polychrony.SignalOperators;


public class ASME2SSME_Translate_Port extends ASME2SSME_Translate_Property{
	public static final String WRITE_FLOW_ID = "write_flow";
	public static final String ITEM_TYPE_ID = "item_type";
	public static final String READ_FLOW_ID = "read_flow";
	public static final String SENT_FLOW_ID = "sent_flow";
	public static final String DEF_VALUE_ID = "def_value";
	public static final String SIZE_ID = "size";
	public static final String DEQUEUE_NUMBER_ID = "dequeue_number" ;
	public static final String MSG_TYPE_ID = "msg_type";
	public static final String PORT_NAME_ID = "port_name";

	private AADLCoLAPI aadlColAPI;
	private signalTreeAPI myLib;
	
	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_Port() {
	  aadlColAPI = new AADLCoLAPI();
	  myLib= new signalTreeAPI();
	}	
	
	private String getThreadPortName(Port p){
		String n1 = aadlColAPI.Port_GetName(p);
		String dir = aadlColAPI.Port_GetDirection(p);
		if(p instanceof DataPort) return Name_of_ThreadDataPort(n1, dir);
		if(p instanceof EventPort) return Name_of_ThreadEventPort(n1, dir);
		return Name_of_ThreadEventDataPort(n1, dir);
	}
	
	/** 
	 * Returns a Signal process "xx_yyPort_property()" that represents the properties of a thread port.
	 * <p>
	 * This Signal process is named as xx_yyPort_property, 
	 * where xx is the name of this port, and yy is the in/out data or event or event data port type. 
	 * The interface contains two event input signals: Frozen_time_event and Reference_time_event. 
	 * The body is composed of the common properties that specified for this port,  (by a called method ASME2SSME_Port_properties()).
	 * The fixed properties (Input_Time ...) are implemented in the port FixProperties() method
	 * 
	 * @param timpl an AADL thread implementation component
	 * @param p a port of thread timpl
	 * @return a (ProcessModelImpl) Signal process (t_property)
	 */
	public ProcessModelImpl ASME2SSME_Thread_Port_property(ThreadImplementation timpl, Port p){
		
		Trace("\n ======== ASME2SSME_Translate_Port: ASME2SSME_Thread_Port_property: " + p.getName());
			
		String t_name = getThreadPortName(p);
		String pname = t_name + "_Property";
		ProcessModelImpl t_property = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );
		
		//inputs
		// event Frozen_time_event;
		SignalsDeclarationImpl Frozen_time_event = new SignalsDeclarationImpl();
		Frozen_time_event.setType(myLib.mkPredefinedType(SignalOperators.opevent));
		SignalNamingImpl sign2 = new SignalNamingImpl();
		if(aadlColAPI.isInputPort(p)){
			sign2.setIdentifier(myLib.mkIdentifier(FROZEN_TIME_EVENT_ID));
		}
		else{
			sign2.setIdentifier(myLib.mkIdentifier(OUTPUT_TIME_EVENT_ID));
		}
		
		Frozen_time_event.getListSignal().add(sign2);
		
		// event Reference_time_event;
		SignalsDeclarationImpl Reference_time_event = myLib.mkEventDecl("Reference_time_event" ); 
		
		// add inputs to interface
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		if(aadlColAPI.isInputPort(p)){
			lins.add(myLib.mkIdentifier(FROZEN_TIME_EVENT_ID));
		}
		else{
			lins.add(myLib.mkIdentifier(OUTPUT_TIME_EVENT_ID));
		}
			
		lins.add(myLib.mkIdentifier(REFERENCE_TIME_EVENT_ID));
		myLib.STree_addDeclarationToInterface(t_property, Frozen_time_event, 2);
		myLib.STree_addDeclarationToInterface(t_property, Reference_time_event, 2);

		//body: property calls---------------------------------------------------//
		// fix number properties (implemented in xxPort_FixProperties())
		//		//Input_Time property
		//		ASME2SSME_Translate_Property asme2ssme_property = new ASME2SSME_Translate_Property();
		//		ProcessCallImpl pcall1 = asme2ssme_property.ASME2SSME_InputTime(timpl, p);
		//		myLib.STree_addExpressionOfProcessToaModel(t_property, pcall1);

		//other properties
		ASME2SSME_Port_properties(timpl, p, t_property);

		return t_property;
	}
	
	
	/**
	 * Translate the properties of a thread port
	 * 
	 * @param timpl thread implementation
	 * @param p a port of timpl
	 * @param t_property a Signal process that represents the properties of port p
	 */
	public void ASME2SSME_Port_properties(ThreadImplementation timpl, Port p, ProcessModelImpl t_property){
		
		Trace("\n ======= ASME2SSME_Translate_Port: ASME2SSME_Port_properties: " + p.getName());
		
		
		
		ArrayList<PropertyAssociation> list3 = aadlColAPI.Port_Get_PropertyAssociation(p);
		if(list3.size()>0){
			for(int j=0; j<list3.size(); j++){
				PropertyAssociation pa1 = list3.get(j);
				if(ASME2SSME_Port_Common_Property(pa1) != null){
					ProcessCallImpl pcall1 = ASME2SSME_Port_Common_Property(pa1);
				    myLib.STree_addExpressionOfProcessToaModel(t_property, pcall1); 	
				}				
			}
		}
		
		ArrayList<PropertyAssociation> list1 = aadlColAPI.ComponentImpl_GetPropertyAssociations(timpl);
		ThreadType thread_type = (ThreadType) aadlColAPI.ComponentImpl_GetComponentType(timpl);
		ArrayList<PropertyAssociation> list2 = aadlColAPI.Component_GetPropertyAssociations(thread_type);
		list1.addAll(list2);
			
		for(int i=0; i<list1.size(); i++)
		{
			PropertyAssociation pa = list1.get(i);
			// String propertyDef_name = aadlColAPI.Property_Get_Name(pa);
			
			if(pa.getAppliesTos() != null){
				EList<ContainedNamedElement> list_applyto = pa.getAppliesTos();
				
				for (int j = 0; j < list_applyto.size(); j++) { 
					ContainedNamedElement c_namedelem = list_applyto.get(j);
					
					if(c_namedelem.getContainmentPathElements() != null){
						EList<ContainmentPathElement> list_cpe = c_namedelem.getContainmentPathElements();
						
						for(int k =0; k< list_cpe.size(); k++){
							ContainmentPathElement pe = list_cpe.get(k);
							String applyto_name = pe.getNamedElement().getName();
							Trace("apply to : " + applyto_name);
							
							if (applyto_name.equals(p.getName())){
								//if(propertyDef_name.equals("Time_Offset") | propertyDef_name.equals("Input_Time_Reference_Time") | propertyDef_name.equals("Input_Time") ){					
								
								if(ASME2SSME_Port_Common_Property(pa) != null){
									ProcessCallImpl pcall2 = ASME2SSME_Port_Common_Property(pa);
								    myLib.STree_addExpressionOfProcessToaModel(t_property, pcall2); 
								}										 
							}
						}
					}								
				}    
			}    
		}
	}
	
	
	/**
	 * Translates a property which is common for the port
	 * 
	 * @param pa
	 * @return pcall a process call
	 */
	public ProcessCallImpl ASME2SSME_Port_Common_Property(PropertyAssociation pa){

		String propertyDef_name = aadlColAPI.Property_Get_Name(pa);
		Trace("\n ======= ASME2SSME_Translate_Port: ASME2SSME_Port_Common_Property ");
		if (propertyDef_name.equals("Timing") | propertyDef_name.equals("Time_Offset")
				| propertyDef_name.equals("Input_Time_Reference_Time") | propertyDef_name.equals("Output_Time_Reference_Time")
				| propertyDef_name.equals("Input_Time") | propertyDef_name.equals("Output_Time")
				| propertyDef_name.equals("Queue_Size") | propertyDef_name.equals("Dequeue_Protocol")
				| propertyDef_name.equals("Dequeued_Items") | propertyDef_name.equals("Queue_Processing_Protocol")
				| propertyDef_name.equals("Overflow_Handling_Protocol")) 
		{ 

			Trace("property fix implemented in xx_FixProperties()");
			return null;
		}
		return ASME2SSME_Property_Call_withName(pa); 
	}

}
