 
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import java.util.Iterator;

import org.osate.aadl2.*;
import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;
 
public class ASME2SSME_Translate_PortConnection extends ASME2SSME_Translate_Connection{

	private AADLCoLAPI aadlColAPI;
	private signalTreeAPI myLib;
	ASME2SSME_Translate_Common asme2ssme_common;

    

    private SignalsDefinitionImpl LocDataPortConnection(Context dst_context, Port /* DataPort */ dst_port, ExprSignal right_expr)
	{
		String left_name;
		//system subcomponent -> system subcomponent | process subcomponent | device subcomponent
		if (dst_context instanceof SystemSubcomponent 
				| dst_context instanceof ProcessSubcomponent 
				| dst_context instanceof DeviceSubcomponent ){ 
			if (dst_context instanceof ProcessSubcomponent) 
				left_name =  aadlColAPI.Subcomponent_GetName((ProcessSubcomponent) dst_context);
			else if(dst_context instanceof DeviceSubcomponent)
				left_name =  aadlColAPI.Subcomponent_GetName((DeviceSubcomponent) dst_context); 
			else 
				left_name =  aadlColAPI.Subcomponent_GetName((SystemSubcomponent) dst_context);
			left_name = left_name  + "_"+ aadlColAPI.Port_GetName(dst_port);;
			ExprSignal left_expr = myLib.mkIdentifier(getSname( left_name));
			return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);	
		}
		return null;
	}
    
	private SignalsDefinitionImpl LocDataPortConnection(ComponentImplementation compImpl, Context dst_context, Port dst_port, String right_name) {
		
		ExprSignal right_expr = myLib.mkIdentifier(getSname(right_name));
		//sub device -> system
		if ((dst_context == null) && (compImpl instanceof SystemImplementation)){
			ExprSignal left_expr = myLib.mkIdentifier(aadlColAPI.Port_GetName(dst_port));				
			return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);			
		}
		return LocDataPortConnection(dst_context, dst_port, right_expr);
	}
    
	/**
	 * Returns a Signal signal definition (an equation) that represents a (event) port connection,  
	 * the source of which is a system component
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
    private SignalsDefinitionImpl ASME2SSME_EventPortConnection_S_System(ComponentImplementation compImpl,
    		Context src_context, Context dst_context, EventPort src_port, EventPort dst_port){

    	String right_name;
    	SignalsDefinitionImpl eq = new SignalsDefinitionImpl();

    	if ((src_context== null) && (compImpl instanceof SystemImplementation)){
    		right_name = aadlColAPI.Port_GetName(src_port);
    		ExprSignal right_expr = myLib.mkIdentifier(right_name);
    		eq = LocDataPortConnection(dst_context, dst_port, right_expr);
    		if (eq == null) Trace("Event port connection from system some case not yet implemented");
    		return eq;
    	}
    	Trace("Event port connection, not souce from system component");
    	return null; 
    }

    
    /** 
	 * Returns a Signal process that represents a (data) port connection between two processes that bound to different processors.
	 * <p>
	 * This Signal process is named as xx_DataPortConnection, where xx is the name of this data port connection. 
	 * this process is composed of two subprocesses: xx_DataPortConnection_Behavior(), xx_DataPortConnection_Property()
	 * xx_DataPortConnection_Behavior() calls the Connection() process from the library. 
	 * xx_DataPortConnection_Property() calls the properties, for example, 
	 * Latency_property{...}(). 
	 * All the parameters are translated from the Latency property
	 *  
	 *  @param compImpl a component implementation component
	 *  @param dconnection a data port connection
	 *  @return a Signal process call
	 */	
	private ProcessModelImpl ASME2SSME_DataPortConnection_Diff_Process(ComponentImplementation compImpl, PortConnection dconnection){		

		//		    xx_DataPortConnection =
		//			   {type item_type}
		//			  (? item_type write_data; event Latency_time_event; ! item_type read_data;)
		//			  (| read_data := xx_DataPortConnection_Behavior{item_type}(write_data, Latency_time_event)
		//			   | xx_DataPortConnection_Property{}(Latency_time_event, Reference_time_event)
		//			   | Reference_time_event := when ^write_data
		//			   |)
		//			  where
		//			     event Reference_time_event;
		//			     process xx_DataPortConnection_Behavior{}()
		//			     process xx_DataPortConnection_Property{}()
		//			  end;

		String connection_name = aadlColAPI.Connection_GetName(dconnection);
		String connection_process_name = Name_of_Connection(connection_name);
		Trace("========= ASME2SSME_Translate_PortConnection: ASME2SSME_DataPortConnection_Diff_Process : " + connection_name);
		ProcessModelImpl vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(connection_process_name) );
		//parameters
		TypesDeclarationImpl item_type = new TypesDeclarationImpl();
		item_type.getTypeDefinition().add(myLib.mkIdentifier("item_type"));	
		myLib.STree_addDeclarationToInterface(vprocess, item_type, 1);

		//inputs	
		//item_type write_data;
		SignalsDeclarationImpl write_data = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("write_data"), myLib.mkIdentifier("item_type"), null);	
		//event Latency_time_event;
		SignalsDeclarationImpl Latency_time_event = myLib.mkEventDecl("Latency_time_event");

		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
		lparams.add(myLib.mkIdentifier("item_type"));

		// add inputs to interface
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		lins.add(myLib.mkIdentifier("write_data"));
		lins.add(myLib.mkIdentifier("Latency_time_event"));
		myLib.STree_addDeclarationToInterface(vprocess, write_data, 2);
		myLib.STree_addDeclarationToInterface(vprocess, Latency_time_event, 2);

		//outputs
		// item_type read_data;
		SignalsDeclarationImpl read_data = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("read_data"), myLib.mkIdentifier("item_type"), null);
		myLib.STree_addDeclarationToInterface(vprocess, read_data, 3);

		// add outputs to interface
		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
		louts.add(myLib.mkIdentifier("read_data"));

		//connection behavior
		ProcessModelImpl connection_behavior = ASME2SSME_Connection_behavior(connection_name);
		String connection_behavior_name = myLib.STree_GetStringName(connection_behavior.getIdentifier()); 
		//add to declaration
		myLib.STree_addProcessDeclaration(vprocess, connection_behavior);
		//add to body
		IdentifierImpl id = (IdentifierImpl) myLib.mkIdentifier(connection_behavior_name);	
		SignalsDefinitionImpl eq_behavior_call = myLib.STree_mkCompleteProcessCallEquation(id, lparams, lins, louts);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, eq_behavior_call);

		//connection property
		ProcessModelImpl connection_property = ASME2SSME_Connection_property(compImpl, dconnection);
		String property_name = myLib.STree_GetStringName(connection_property.getIdentifier()); 
		//add to declaration
		myLib.STree_addProcessDeclaration(vprocess, connection_property);
		//add to body
		ArrayList<ExprParam> lparam3 = new ArrayList<ExprParam>();
		ArrayList<ExprSignal> lin3 = new ArrayList<ExprSignal>();
		lin3.add(myLib.mkIdentifier("Latency_time_event"));

		//create a local event Reference_time_event
		ArrayList<Declaration> vdecl = new ArrayList<Declaration>();
		SignalsDeclarationImpl Reference_time_event = myLib.mkEventDecl("Reference_time_event");
		vdecl.add(Reference_time_event);
		myLib.STree_addLocalDeclaration(vprocess, vdecl);

		// Reference_time_event := when ^write_data
		SignalsDefinitionImpl eq1 = ASME2SSME_Reference_time_event_Assignment();
		myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);

		lin3.add(myLib.mkIdentifier("Reference_time_event"));
		ProcessCallImpl property_inst = myLib.STree_mkCompleteProcessCall(myLib.mkIdentifier(property_name),lparam3, lin3);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, property_inst);

		return vprocess;
	}

	
	/**
	 * Returns a Signal signal definition (an equation) that represents a (event) port connection,  
	 * the source of which is a process
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_EventPortConnection_S_Process(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventPort src_port, EventPort dst_port){

		if((src_context==null) && (compImpl instanceof ProcessImplementation)) {
			String right_name = aadlColAPI.Port_GetName(src_port);
			ExprSignal right_expr = myLib.mkIdentifier(right_name);

			// process -> sub thread
			if (dst_context instanceof ThreadSubcomponent){
				String left_name = aadlColAPI.Subcomponent_GetName((ThreadSubcomponent) dst_context) + "_"+ aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(getSname( left_name));				
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);			
			}
			Trace("Event port connection from process some case not yet implemented");
			return null;
		}
		Trace("Event port connection, not souce from process");
		return null; 
	}


	/**
	 * Returns a Signal signal definition (an equation) that represents a (event) port connection,  
	 * the source of which is a thread subcomponent
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_EventPortConnection_S_SubThread(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventPort src_port, EventPort dst_port){

		String left_name, right_name;
		SignalsDefinitionImpl eq = new SignalsDefinitionImpl();
		if (src_context instanceof ThreadSubcomponent){
			ThreadSubcomponent sub = (ThreadSubcomponent) src_context;
			right_name = aadlColAPI.Subcomponent_GetName(sub) + "_"+ aadlColAPI.Port_GetName(src_port);
			String str1 = getSname( right_name);
			ExprSignal right_expr = myLib.mkIdentifier(str1);
			// sub thread -> sub thread
			if (dst_context instanceof ThreadSubcomponent){
				left_name =  aadlColAPI.Subcomponent_GetName((ThreadSubcomponent) dst_context) + "_" + aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(getSname(left_name));				
				return  (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);			
			}
			//sub thread -> process
			if ((dst_context == null) && (compImpl instanceof ProcessImplementation)){
				left_name =  aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(left_name);				
				eq = (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);	
				return eq;
			}
			Trace("ASME2SSME_EventPortConnection dst context not subprogram subcomponent, not implemented");
		}
		Trace("Event port connection, not souce from thread subcomponent");
		return null; 
	}


	/**
	 * Returns a Signal signal definition (an equation) that represents a (event) port connection,  
	 * the source of which is a thread component
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_EventPortConnection_S_Thread(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventPort src_port, EventPort dst_port){

		String left_name, right_name;
		SignalsDefinitionImpl eq = new SignalsDefinitionImpl();
		if ((src_context == null ) && ( compImpl instanceof ThreadImplementation)) {
			right_name = aadlColAPI.Port_GetName(src_port);
			ExprSignal right_expr = myLib.mkIdentifier(right_name);
			// thread -> subprogram subcomponent
			if (dst_context instanceof SubprogramSubcomponent){
				left_name =  aadlColAPI.Subcomponent_GetName( (SubprogramSubcomponent) dst_context) + "_" + aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(getSname( left_name));				
				eq = (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);			
				return eq;
			}
			Trace("ASME2SSME_EventPortConnection dst context not subprogram subcomponent, not implemented");
			return null;
		}
		Trace("Event port connection, not souce from thread component");
		return null; 
	}

	/**
	 * Returns a Signal signal definition (an equation) that represents a (data) port connection,  
	 * the source of which is a process component
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_DataPortConnection_S_Process(ComponentImplementation compImpl, 
			Context src_context, Context dst_context, DataPort src_port, DataPort dst_port){

		if ( (src_context==null) && (compImpl instanceof ProcessImplementation)) {	
			//process->thread subcomponent
			if (dst_context instanceof ThreadSubcomponent){

				ThreadSubcomponent dst_sub_thread = (ThreadSubcomponent) dst_context;
				String left_name =  aadlColAPI.Subcomponent_GetName(dst_sub_thread) + "_" + aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(getSname(left_name));
				ExprSignal right_expr = myLib.mkIdentifier(aadlColAPI.Port_GetName(src_port));
				SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
				return eq;
			}
			Trace("Data port connection from a process some case not yet implemented ");
			return null;
		}
		Trace("Data port connection, not source from a process component. ");
		return null;
	}


	/**
	 * Returns a Signal signal definition (an equation) that represents a (data) port connection,  
	 * the source of which is a thread subcomponent
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_DataPortConnection_S_SubThread(ComponentImplementation compImpl,
			Context src_context, Context dst_context, DataPort src_port, DataPort dst_port){

		String left_name;
		String right_name;
		//thread subcomponent 
		if(src_context instanceof ThreadSubcomponent) {
			//thread subcomponent ->thread subcomponent
			if (dst_context instanceof ThreadSubcomponent){ 

				ThreadSubcomponent dst_sub_thread = (ThreadSubcomponent) dst_context;
				left_name =  aadlColAPI.Subcomponent_GetName(dst_sub_thread) + "_" + aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(getSname(left_name));
				ThreadSubcomponent src_sub_thread = (ThreadSubcomponent) src_context;
				right_name =  aadlColAPI.Subcomponent_GetName(src_sub_thread) + "_" + aadlColAPI.Port_GetName(src_port);
				ExprSignal right_expr = myLib.mkIdentifier(getSname(right_name));
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
			}
			//thread subcomponent ->process		
			if ((dst_context==null) && (compImpl instanceof ProcessImplementation)){

				left_name =  aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(left_name);
				ThreadSubcomponent src_sub_thread = (ThreadSubcomponent) src_context;
				right_name =  aadlColAPI.Subcomponent_GetName(src_sub_thread) + "_" + aadlColAPI.Port_GetName(src_port);
				ExprSignal right_expr = myLib.mkIdentifier(getSname(right_name));
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
			}
			Trace("Data port connection from a thread subcomponent to xx not yet implemented ");
			return null;
		}
		Trace("Data port connection, not source from a thread subcomponent. ");
		return null;
	}

	
	/**
	 * Returns a Signal signal definition (an equation) that represents a (event) port connection,  
	 * the source of which is a system subcomponent
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_EventPortConnection_S_SubSystem(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventPort src_port, EventPort dst_port){

		String right_name;
		if (src_context instanceof SystemSubcomponent){
			right_name = aadlColAPI.Subcomponent_GetName((SystemSubcomponent) src_context) + "_" + aadlColAPI.Port_GetName(src_port);
			SignalsDefinitionImpl eq = LocDataPortConnection(compImpl, dst_context, dst_port, right_name );
    		if (eq == null) Trace("Event port connection from system subcomponent some case not yet implemented");
    		return eq;
		}
		Trace("Event port connection, not souce from system subcomponent");
		return null; 
	}

	/**
	 * Returns a Signal signal definition (an equation) that represents a (event) port connection,  
	 * the source of which is a device subcomponent
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_EventPortConnection_S_SubDevice(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventPort src_port, EventPort dst_port){

		String right_name;

		if (src_context instanceof DeviceSubcomponent){
			right_name = aadlColAPI.Subcomponent_GetName((DeviceSubcomponent)src_context) + "_" + aadlColAPI.Port_GetName(src_port);
			SignalsDefinitionImpl eq = LocDataPortConnection(compImpl, dst_context, dst_port, right_name );
			if (eq == null) Trace("Event port connection from device subcomponent some case not yet implemented");
			return eq;
		}
		Trace("Event port connection, not souce from device subcomponent");
		return null; 
	}

	
	/**
	 * Returns a Signal signal definition (an equation) that represents a (event) port connection,  
	 * the source of which is a process subcomponent
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_EventPortConnection_S_SubProcess(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventPort src_port, EventPort dst_port){

		String left_name, right_name;
		ExprSignal left_expr;

		if (src_context instanceof ProcessSubcomponent){
			right_name = aadlColAPI.Subcomponent_GetName((ProcessSubcomponent) src_context) + "_" + aadlColAPI.Port_GetName(src_port);
			//String str1 = getSname( "l_"+ right_name);
			String str1 = getSname( right_name);
			ExprSignal right_expr = myLib.mkIdentifier(str1);

			//sub process -> system
			if ((dst_context == null) && (compImpl instanceof SystemImplementation)){
				left_name =  aadlColAPI.Port_GetName(dst_port);
				left_expr = myLib.mkIdentifier(left_name);				
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);			
			}
			if (dst_context instanceof ProcessSubcomponent){
				ProcessSubcomponent sub1 = (ProcessSubcomponent) dst_context;
				left_name =  "l_"+ aadlColAPI.Subcomponent_GetName(sub1) + "_"+ aadlColAPI.Port_GetName(dst_port);
				right_expr = myLib.mkIdentifier(getSname( "l_"+right_name)  );
				left_expr = myLib.mkIdentifier(getSname(left_name));
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
			}
			if (dst_context instanceof DeviceSubcomponent 
					| dst_context instanceof SystemSubcomponent){
				if (dst_context instanceof DeviceSubcomponent){
					DeviceSubcomponent sub1 = (DeviceSubcomponent) dst_context;
					left_name =  aadlColAPI.Subcomponent_GetName(sub1) + "_"+ aadlColAPI.Port_GetName(dst_port);
					left_expr = myLib.mkIdentifier(getSname( "l_"+ left_name));				
				}
				else {  
					left_name =  aadlColAPI.Subcomponent_GetName((SystemSubcomponent) dst_context) + "_"+ aadlColAPI.Port_GetName(dst_port);
					left_expr = myLib.mkIdentifier( getSname( left_name));				
				}
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
			}
			Trace("Event port connection from process subcomponent some case not yet implemented");
			return null;
		}
		Trace("Event port connection, not souce from process subcomponent");
		return null; 
	}
	
	/**
	 * Returns a Signal signal definition (an equation) that represents a (event data) port connection,  
	 * the source of which is a system component
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_EventDataPortConnection_S_System(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventDataPort src_port, EventDataPort dst_port){

		String left_name, right_name;

		if((src_context == null) && (compImpl instanceof SystemImplementation)) {

			right_name = aadlColAPI.Port_GetName(src_port);
			ExprSignal right_expr = myLib.mkIdentifier(right_name);

			// system -> process subcomponent | device subcomponent | system subcomponent
			if (dst_context instanceof ProcessSubcomponent 
					| dst_context instanceof DeviceSubcomponent
					| dst_context instanceof SystemSubcomponent){

				if (dst_context instanceof ProcessSubcomponent) 
					left_name =  aadlColAPI.Subcomponent_GetName((ProcessSubcomponent) dst_context) ;
				else if (dst_context instanceof DeviceSubcomponent) 
					left_name =  aadlColAPI.Subcomponent_GetName((DeviceSubcomponent) dst_context);  
				else 
					left_name =  aadlColAPI.Subcomponent_GetName((SystemSubcomponent) dst_context);
				left_name =  left_name + "_"+ aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(getSname( left_name));				
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
			}			 
			Trace("\n Event data port connection from system some case not yet implemented");
			return null;
		}

		Trace("\n Event data port connection, not source from system component");
		return null;
	}


	/**
	 * Returns a Signal signal definition (an equation) that represents a (event data) port connection,  
	 * the source of which is a system subcomponent
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_EventDataPortConnection_S_SubSystem(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventDataPort src_port, EventDataPort dst_port){

		String left_name, right_name;
		if(src_context instanceof SystemSubcomponent) {
			SystemSubcomponent src_sub_sys = (SystemSubcomponent) src_context;
			right_name =  aadlColAPI.Subcomponent_GetName(src_sub_sys) + "_" + aadlColAPI.Port_GetName(src_port);
			ExprSignal right_expr = myLib.mkIdentifier(getSname( right_name)   );

			//sub system -> system
			if((dst_context == null) && (compImpl instanceof SystemImplementation)){
				left_name =  aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(left_name);				
				return  (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);	
			}
			//system subcomponent -> process subcomponent | device subcomponent | system subcomponent
			else if (dst_context instanceof ProcessSubcomponent 
					| dst_context instanceof DeviceSubcomponent
					| dst_context instanceof SystemSubcomponent){ 

				if(dst_context instanceof ProcessSubcomponent) 
					left_name =  aadlColAPI.Subcomponent_GetName((ProcessSubcomponent) dst_context);
				else if(dst_context instanceof DeviceSubcomponent) 
					left_name =  aadlColAPI.Subcomponent_GetName((DeviceSubcomponent) dst_context);
				else
					left_name =  aadlColAPI.Subcomponent_GetName((SystemSubcomponent) dst_context);
				left_name = left_name + "_"+ aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(getSname( left_name));		
				return  (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);		
			}
			Trace("\n Event data port connection from system subcomponent some case not yet implemented");
			return null;
		}
		Trace("\n Event data port connection, not source from system subcomponent");
		return null;
	}


	/**
	 * Returns a Signal signal definition (an equation) that represents a (event data) port connection,  
	 * the source of which is a device subcomponent
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_EventDataPortConnection_S_SubDevice(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventDataPort src_port, EventDataPort dst_port){


		String left_name, right_name;
		SignalsDefinitionImpl eq = new SignalsDefinitionImpl();

		if(src_context instanceof DeviceSubcomponent) {

			DeviceSubcomponent src_sub_device = (DeviceSubcomponent) src_context;
			right_name =  aadlColAPI.Subcomponent_GetName(src_sub_device) + "_" + aadlColAPI.Port_GetName(src_port);	    	
			String sn9 = getSname( right_name);
			ExprSignal right_expr = myLib.mkIdentifier( sn9/*right_name*/);

			//device subcomponent -> system
			if ((dst_context == null) && (compImpl instanceof SystemImplementation)){ 

				left_name =  aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(left_name);				
				eq = (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);		
				return eq;
			}
			//device subcomponent -> process subcomponent | device subcomponent | system subcomponent
			if (dst_context instanceof ProcessSubcomponent
					| dst_context instanceof DeviceSubcomponent
					| dst_context instanceof SystemSubcomponent){ 
				if (dst_context instanceof ProcessSubcomponent) 
					left_name =  aadlColAPI.Subcomponent_GetName((ProcessSubcomponent) dst_context);
				else if(dst_context instanceof DeviceSubcomponent) 
					left_name =  aadlColAPI.Subcomponent_GetName((DeviceSubcomponent) dst_context);
				else  
					left_name =  aadlColAPI.Subcomponent_GetName( (SystemSubcomponent) dst_context);
				left_name = left_name + "_"+ aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(getSname( left_name) );				
				return  (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);		
			}
			Trace("\n Event data port connection from device subcomponent some case not yet implemented");
			return null;
		}
		Trace("\n Event data port connection, not source from device subcomponent");
		return null;
	}


	/**
	 * Returns a Signal signal definition (an equation) that represents a (event data) port connection,  
	 * the source of which is a process subcomponent
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_EventDataPortConnection_S_SubProcess(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventDataPort src_port, EventDataPort dst_port){

		String left_name, right_name;
		if (src_context instanceof ProcessSubcomponent) {
			ProcessSubcomponent src_sub_process = (ProcessSubcomponent) src_context;
			right_name =  aadlColAPI.Subcomponent_GetName(src_sub_process) + "_" + aadlColAPI.Port_GetName(src_port);
			String str1 = getSname( "l_"+ right_name);
			ExprSignal right_expr = myLib.mkIdentifier(str1);

			//process subcomponent -> system
			if ((dst_context == null) && (compImpl instanceof SystemImplementation)){ 
				left_name =  aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(left_name);			
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);		
			}
			//process subcomponent -> process| subcomponent | device subcomponent | system subcomponent
			if (dst_context instanceof ProcessSubcomponent
					| dst_context instanceof DeviceSubcomponent 
					| dst_context instanceof SystemSubcomponent){ 

				if(dst_context instanceof ProcessSubcomponent) 
					left_name =  aadlColAPI.Subcomponent_GetName((ProcessSubcomponent) dst_context);
				else if(dst_context instanceof DeviceSubcomponent) 
					left_name =  aadlColAPI.Subcomponent_GetName((DeviceSubcomponent) dst_context);
				else
					left_name =  aadlColAPI.Subcomponent_GetName( (SystemSubcomponent) dst_context);
				left_name =  left_name + "_"+ aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier( getSname( "l_"+ left_name));			
				return  (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);		
			}
			Trace("\n Event data port connection from process subcomponent some case not yet implemented");
			return null;
		}
		Trace("\n Event data port connection, not source from process subcomponent");
		return null;
	}

	/**
	 * Returns a Signal signal definition (an equation) that represents a (event data) port connection,  
	 * the source of which is a thread subcomponent
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_EventDataPortConnection_S_SubThread(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventDataPort src_port, EventDataPort dst_port){

		String left_name, right_name;
		if(src_context instanceof ThreadSubcomponent) {

			ThreadSubcomponent src_sub_thread = (ThreadSubcomponent) src_context;
			right_name =  aadlColAPI.Subcomponent_GetName(src_sub_thread) + "_" + aadlColAPI.Port_GetName(src_port);	    	
			ExprSignal right_expr = myLib.mkIdentifier(getSname(right_name));

			//thread subcomponent ->thread subcomponent
			if (dst_context instanceof ThreadSubcomponent){ 
				ThreadSubcomponent dst_sub_thread = (ThreadSubcomponent) dst_context;
				left_name =  aadlColAPI.Subcomponent_GetName(dst_sub_thread) + "_" + aadlColAPI.Port_GetName(dst_port);		    	
				ExprSignal left_expr = myLib.mkIdentifier(getSname(left_name));							
				return  (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
			}
			//thread subcomponent ->process		
			if ((dst_context==null) && (compImpl instanceof ProcessImplementation)){
				left_name =  aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(left_name);			
				return   (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
			}
			Trace("\n Event data port connection from thread subcomponent some case not yet implemented");
			return null;
		}
		Trace("\n Event data port connection, not source from subthread");
		return null;
	}


	/**
	 * Returns a Signal signal definition (an equation) that represents a (event data) port connection,  
	 * the source of which is a thread component
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_EventDataPortConnection_S_Thread(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventDataPort src_port, EventDataPort dst_port){

		String left_name, right_name;

		if((src_context == null ) && ( compImpl instanceof ThreadImplementation)) {
			right_name = aadlColAPI.Port_GetName(src_port);
			ExprSignal right_expr = myLib.mkIdentifier(right_name);

			// thread -> subprogram subcomponent
			if (dst_context instanceof SubprogramSubcomponent){

				SubprogramSubcomponent dst_subp = (SubprogramSubcomponent) dst_context;
				left_name =  aadlColAPI.Subcomponent_GetName(dst_subp) + "_" + aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(getSname( left_name));				
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);			
			}
			Trace("ASME2SSME_EventPortConnection dst context not subprogram subcomponent, not implemented");
			return null;
		}
		Trace("\n Event data port connection, not source from thread component");
		return null;
	}


	/**
	 * Returns a Signal signal definition (an equation) that represents a (event data) port connection,  
	 * the source of which is a subprogram subcomponent
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_EventDataPortConnection_S_SubSubprogram(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventDataPort src_port, EventDataPort dst_port){

		String left_name, right_name;
		if(src_context instanceof SubprogramSubcomponent){

			SubprogramSubcomponent src_subp = (SubprogramSubcomponent) src_context;
			right_name =  aadlColAPI.Subcomponent_GetName(src_subp) + "_" + aadlColAPI.Port_GetName(src_port);
			String str2 = getSname( right_name);
			ExprSignal right_expr = myLib.mkIdentifier(str2);

			// subprogram subcomponent -> thread
			if ((dst_context == null ) && (compImpl instanceof ThreadImplementation)){

				left_name =  aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(left_name);				
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);			
			}

			// subprogram subcomponent -> subprogram subcomponen
			if (dst_context instanceof SubprogramSubcomponent){

				SubprogramSubcomponent dst_subp = (SubprogramSubcomponent) dst_context;
				left_name =  aadlColAPI.Subcomponent_GetName(dst_subp) + "_" + aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(getSname( left_name));			
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);		
			}
			Trace("ASME2SSME_EventPortConnection src context subprogram subcomponent, not implemented");
			return null;
		}
		Trace("\n Event data port connection, not source from subprogram subcomponent");
		return null;
	}

	/**
	 * Returns an equation: Reference_time_event := when ^write_data
	 * 
	 * @param
	 * @return a Signal equation
	 */
	private SignalsDefinitionImpl ASME2SSME_Reference_time_event_Assignment(){

		Trace("\n ================== ASME2SSME_Translate_PortConnection: ASME2SSME_Reference_time_event_Assignment ");
		ExprSignal leftExp = myLib.mkIdentifier("Reference_time_event");
		//^write_data
		ClockImpl exp_clock = myLib.STree_mkClock(myLib.mkIdentifier("write_data"));
		//when ^write_data
		UnaryWhenImpl rightExp = myLib.STree_mkUnaryWhen(exp_clock);

		//Reference_time_event := when ^write_data
		SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		return eq;
	}

	/** 
	 * Returns a Signal process "xx_Connection_behavior" that represents the behavior of a connection.
	 * <p>
	 * This Signal process is named as "xx_Connection_behavior", where xx is the name of this connection. 
	 * The interface contains parameters: type item_type, 
	 * inputs: item_type write_data,  event Latency_time_event, 
	 * and outputs: item_type read_data.
	 * The body calls a Connection() process which is predefined in the library.
	 * 
	 * @param c_name a String name of the connection
	 * @return a ProcessModelImpl (Signal process)
	 */
	private ProcessModelImpl ASME2SSME_Connection_behavior(String c_name){

		String bname = c_name + "_Connection_behavior";
		ProcessModelImpl c_behavior = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(bname) );	
		Trace("\n ================== start ASME2SSME_Connection_behavior: " + c_name);

		//add parameter type item_type to interface
		TypesDeclarationImpl item_type = new TypesDeclarationImpl();
		item_type.getTypeDefinition().add(myLib.mkIdentifier("item_type"));	
		myLib.STree_addDeclarationToInterface(c_behavior, item_type, 1);

		//add in item_type write_data to interface
		asme2ssme_common.ASME2SSME_add_input_to_process_interface("write_data", "item_type", c_behavior);

		//add in event Latency_time_event to interface
		asme2ssme_common.ASME2SSME_add_input_to_process_interface("Latency_time_event", "event", c_behavior);

		//add out item_type read_data to interface
		SignalsDeclarationImpl read_data = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("read_data"), myLib.mkIdentifier("item_type"), null);
		myLib.STree_addDeclarationToInterface(c_behavior, read_data, 3);

		//body
		ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
		lparams.add(myLib.mkIdentifier("item_type"));
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		lins.add(myLib.mkIdentifier("write_data"));
		lins.add(myLib.mkIdentifier("Latency_time_event"));
		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
		louts.add(myLib.mkIdentifier("read_data"));

		SignalsDefinitionImpl eq1 = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("Connection"), lparams, lins, louts);
		myLib.STree_addExpressionOfProcessToaModel(c_behavior, eq1);

		return c_behavior;
	}


	/** 
	 * Returns a Signal process "xx_Connection_property" that represents the properties of a (data) port connection
	 * <p>
	 * This Signal process is named as "xx_Connection_property", where xx is the name of this connection.
	 * The interface contains inputs: event Latency_time_event, event Reference_time_event. 
	 * The body calls the Latency property (ASME2SSME_Latency())
	 * 
	 * @param compImpl  a component implementation that contains this connection
	 * @param dconnecton a data port connection
	 * @return a ProcessModelImpl (Signal process)
	 */
	private ProcessModelImpl ASME2SSME_Connection_property(ComponentImplementation compImpl, PortConnection dconnection){

		String connection_name = aadlColAPI.Connection_GetName(dconnection);

		Trace("\n ================== ASME2SSME_Translate_PortConnection: ASME2SSME_Connection_property: " + connection_name);
		String connection_process_name = connection_name + "_Connection_property";
		ProcessModelImpl c_property = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(connection_process_name) );
		//inputs: event Latency_time_event;
		SignalsDeclarationImpl Latency_time_event = myLib.mkEventDecl("Latency_time_event"); 

		// event Reference_time_event;
		SignalsDeclarationImpl Reference_time_event = myLib.mkEventDecl("Reference_time_event");

		// add inputs to interface
		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
		lins.add(myLib.mkIdentifier("Latency_time_event"));
		lins.add(myLib.mkIdentifier("Reference_time_event"));
		myLib.STree_addDeclarationToInterface(c_property, Latency_time_event, 2);
		myLib.STree_addDeclarationToInterface(c_property, Reference_time_event, 2);
		//body

		ArrayList<PropertyAssociation> connection_property_list = aadlColAPI.Get_Properties_AppliesTo_Connection(compImpl, dconnection);
		ProcessCallImpl pcall1;
		
		if (connection_property_list.size()>0){
			Iterator<PropertyAssociation> it1 = connection_property_list.iterator();
			while (it1.hasNext())
			{
				PropertyAssociation pa = (PropertyAssociation) it1.next();
				String propertyDef_name = aadlColAPI.Property_Get_Name(pa);

				if (propertyDef_name.equals("Latency") ) 
					pcall1 = ASME2SSME_Latency_Property_Call(compImpl, dconnection);
				else  
					pcall1 = ASME2SSME_Property_Call(pa);
				myLib.STree_addExpressionOfProcessToaModel(c_property, pcall1);
			}
		}
		return c_property;
	}
	/**
	 * Returns a Signal signal definition (an equation) that represents a (event) port connection,  
	 * the source of which is a subprogram subcomponent
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_EventPortConnection_S_SubSubprogram(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventPort src_port, EventPort dst_port){

		String left_name, right_name;

		if(src_context instanceof SubprogramSubcomponent){
			SubprogramSubcomponent src_subp = (SubprogramSubcomponent) src_context;
			right_name =  aadlColAPI.Subcomponent_GetName(src_subp) + "_" + aadlColAPI.Port_GetName(src_port);
			ExprSignal right_expr = myLib.mkIdentifier(getSname( right_name));

			// subprogram subcomponent -> thread
			if ((dst_context == null ) && (compImpl instanceof ThreadImplementation)){

				left_name =  aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(left_name);				
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);			
			}
			// subprogram subcomponent -> subprogram subcomponen
			if (dst_context instanceof SubprogramSubcomponent){
				SubprogramSubcomponent dst_subp = (SubprogramSubcomponent) dst_context;
				left_name =  aadlColAPI.Subcomponent_GetName(dst_subp) + "_" + aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(getSname(left_name));			
				return  (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);		
			}
			Trace("ASME2SSME_EventPortConnection src context subprogram subcomponent, not implemented");
			return null;
		}
		Trace("Event port connection, not souce from subprogram subcomponent");
		return null; 
	}

	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_PortConnection() {
		aadlColAPI = new AADLCoLAPI();
		myLib= new signalTreeAPI();
		asme2ssme_common = new ASME2SSME_Translate_Common();	 
	}

	/** 
	 * Returns a signal definition (an equation) that represents a port connection
	 * <p>
	 * Depending on the port type of this connection, call the method:  
	 * ASME2SSME_DataPortConnection(), ...
	 * 
	 * @param compImpl a component implementation component
	 * @param vprocess a Signal process that represents an AADL component which contains this connection
	 * @param connection a port connection
	 */
	public SignalsDefinitionImpl ASME2SSME_PortConnection(ComponentImplementation compImpl, ProcessModelImpl vprocess, PortConnection connection){

		Trace("==================ASME2SSME_Translate_PortConnection : ASME2SSME_PortConnection : " + connection.getName());
		if (aadlColAPI.Is_DataPortConnection(connection)) return ASME2SSME_DataPortConnection(compImpl, vprocess,connection);
		if (aadlColAPI.Is_EventPortConnection(connection)) return ASME2SSME_EventPortConnection(compImpl, vprocess, connection);
		if (aadlColAPI.Is_EventDataPortConnection(connection)) return ASME2SSME_EventDataPortConnection(compImpl, vprocess,(PortConnection) connection);
		Trace("some port connection not yet implemented");
		return new SignalsDefinitionImpl();
	}

	/**
	 * Returns a Signal signal definition (an equation) that represents a (data) port connection,  
	 * the source of which is a system component
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_DataPortConnection_S_System(ComponentImplementation compImpl,
			Context src_context, Context dst_context, DataPort src_port, DataPort dst_port){

		SignalsDefinitionImpl eq = LocDataPortConnection(dst_context, dst_port, myLib.mkIdentifier(aadlColAPI.Port_GetName(src_port)));
		if (eq == null) Trace("data port connection from system (case not yet implemented)");
		return eq;
	}

	/**
	 * Returns a Signal signal definition (an equation) that represents a (data) port connection,  
	 * the source of which is a system subcomponent
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_DataPortConnection_S_SubSystem(ComponentImplementation compImpl,
			Context src_context, Context dst_context, DataPort src_port, DataPort dst_port){
		String left_name;
		String right_name;

		//system subcomponent 
		if (src_context instanceof SystemSubcomponent) {
			SystemSubcomponent src_sub_sys = (SystemSubcomponent) src_context;
			right_name =  aadlColAPI.Subcomponent_GetName(src_sub_sys) + "_" + aadlColAPI.Port_GetName(src_port);
			ExprSignal right_expr = myLib.mkIdentifier(getSname(right_name));

			//system subcomponent -> system
			if ((dst_context == null) && (compImpl instanceof SystemImplementation)){ 

				left_name =  aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(left_name);						
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
			}
			SignalsDefinitionImpl eq = LocDataPortConnection(dst_context, dst_port, right_expr);
			if (eq == null) Trace("Data port connection from a system subcomponent to xx not yet implemented ");
			return eq;
		}
		Trace("data port connection, not source from subsystem");
		return null;
	}

	/**
	 * Returns a Signal signal definition (an equation) that represents a (data) port connection,  
	 * the source of which is a device subcomponent
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_DataPortConnection_S_SubDevice(ComponentImplementation compImpl,
			Context src_context, Context dst_context, DataPort src_port, DataPort dst_port){

		String left_name;
		String right_name;
		SignalsDefinitionImpl eq = new SignalsDefinitionImpl();
		if(src_context instanceof DeviceSubcomponent) {
			DeviceSubcomponent src_sub_device = (DeviceSubcomponent) src_context;
			right_name =  aadlColAPI.Subcomponent_GetName(src_sub_device) + "_" + aadlColAPI.Port_GetName(src_port);
			ExprSignal right_expr = myLib.mkIdentifier(getSname( right_name));
			//device subcomponent -> system
			if ((dst_context ==null) && (compImpl instanceof SystemImplementation)){ 

				left_name =  aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(left_name);
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);		
			}
			eq = LocDataPortConnection(dst_context, dst_port, right_expr);
			if (eq == null) Trace("Data port connection from a device subcomponent to xx not yet implemented");
			return eq;
		}
		Trace("data port connection, not source from subdevice");
		return null;
	}

	/** 
	 * Returns a Signal signal definition (an equation) that represents a (data) port connection.
	 * <p>
	 * Depending on the source and dst context of the connection, different connection is provided. 
	 * In case of the source and dst context are two sub processes, and these two subprocesses bound to different processors: 
	 * creates a Signal process xx_Connection (ASME2SSME_DataPortConnection_Diff_Process()), 
	 * and adds the process declaration into the process (vprocess), 
	 * and makes a process call: p1_pa := xx_Connection(p2_pb,xx_Latency_time_event) in the body. 
	 * For the other cases, make an equation of the source and destination port in the body. 
	 * The local signal declarations are added. The sub component's name may be used as a prefix of these signals. 
	 * 
	 * @param compImpl a component implementation component
	 * @param vprocess a Signal process that represents an AADL component which contains this connection
	 * @param dconnection a data port connection
	 * @return a signal definition (an equation)
	 */
	public SignalsDefinitionImpl ASME2SSME_DataPortConnection(ComponentImplementation compImpl, ProcessModelImpl vprocess, PortConnection dconnection){

		String left_name;
		String right_name;

		String connection_name = aadlColAPI.Connection_GetName(dconnection);
		Trace("========= ASME2SSME_Translate_PortConnection: ASME2SSME_DataPortConnection : " + connection_name);
		if (!aadlColAPI.Is_DataPortConnection(dconnection)) {
			Trace("not data port connection");
			return null;
		}
		Context src_context = aadlColAPI.Connection_GetSrcContext(dconnection);
		Context dst_context = aadlColAPI.Connection_GetDstContext(dconnection);
		Port src_element = (Port) aadlColAPI.PortConnection_GetSrc(dconnection);
		Port dst_element = (Port) aadlColAPI.PortConnection_GetDst(dconnection);

		DataPort src_port = (DataPort) src_element;
		DataPort dst_port = (DataPort) dst_element;
		//1) process->  
		if((src_context==null) && (compImpl instanceof ProcessImplementation))  			
			return ASME2SSME_DataPortConnection_S_Process(compImpl, src_context, dst_context, src_port, dst_port);
		//2) thread subcomponent 
		if(src_context instanceof ThreadSubcomponent)  
			return ASME2SSME_DataPortConnection_S_SubThread(compImpl, src_context, dst_context, src_port, dst_port);
		//3) system  
		if((src_context==null) && (compImpl instanceof SystemImplementation))  
			return ASME2SSME_DataPortConnection_S_System(compImpl, src_context, dst_context, src_port, dst_port);
		//4) system subcomponent 
		if(src_context instanceof SystemSubcomponent) 
			return ASME2SSME_DataPortConnection_S_SubSystem(compImpl, src_context, dst_context, src_port, dst_port);
		//5) device subcomponent  
		if(src_context instanceof DeviceSubcomponent) 
			return ASME2SSME_DataPortConnection_S_SubDevice(compImpl, src_context, dst_context, src_port, dst_port);
		//6) process subcomponent 
		if(src_context instanceof ProcessSubcomponent) {

			ProcessSubcomponent src_sub_process = (ProcessSubcomponent) src_context;
			right_name =  aadlColAPI.Subcomponent_GetName(src_sub_process) + "_" + aadlColAPI.Port_GetName(src_port);
			ExprSignal right_expr = myLib.mkIdentifier(getSname(right_name));

			//process subcomponent -> system
			if ( (dst_context==null) && (compImpl instanceof SystemImplementation)/*dst_context instanceof SystemImplementation*/){ 
				left_name =  aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(left_name);			
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);		
			}
			//process subcomponent -> system subcomponent | device subcomponent
			if (dst_context instanceof SystemSubcomponent | dst_context instanceof DeviceSubcomponent) { 

				if (dst_context instanceof DeviceSubcomponent) 
					left_name =  aadlColAPI.Subcomponent_GetName((DeviceSubcomponent) dst_context);
				else  
					left_name =  aadlColAPI.Subcomponent_GetName((SystemSubcomponent) dst_context);
				left_name = left_name +  "_"+ aadlColAPI.Port_GetName(dst_port);
				ExprSignal left_expr = myLib.mkIdentifier(getSname( left_name));			
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);		
			}
			//process subcomponent ->process subcomponent
			if (dst_context instanceof ProcessSubcomponent){

				//the two sub-process bound to the same processor: l_d := l_s for a connection s-> d
				if (aadlColAPI.Process_Get_Binding_Processor((SystemImplementation) compImpl, (ProcessSubcomponent) src_context) == aadlColAPI.Process_Get_Binding_Processor((SystemImplementation) compImpl, (ProcessSubcomponent) dst_context)){
					ProcessSubcomponent dst_sub_process = (ProcessSubcomponent) dst_context;
					left_name =  "l_"+ aadlColAPI.Subcomponent_GetName(dst_sub_process) + "_" + aadlColAPI.Port_GetName(dst_port);
					right_expr = myLib.mkIdentifier(getSname( "l_"+right_name) );
					ExprSignal left_expr = myLib.mkIdentifier(getSname( left_name));
					return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
				}
				//the two sub-process bound to the different processors
				ProcessSubcomponent dst_sub_process = (ProcessSubcomponent) dst_context;
				ProcessModelImpl process_connection = ASME2SSME_DataPortConnection_Diff_Process(compImpl, dconnection);
				myLib.STree_addProcessDeclaration(vprocess, process_connection);

				String process_connection_name = myLib.STree_GetStringName(process_connection.getIdentifier());//pname + "_behavior";
				ArrayList<ExprParam> l_param = new ArrayList<ExprParam>();//parameter (id) list for call the process pprocess		    
				DataClassifier data_classifier = (DataClassifier) aadlColAPI.DataPort_GetClassifier(src_port);
				String str_type_name = Name_of_Reference_data_classifier(data_classifier);
				l_param.add(myLib.mkIdentifier(str_type_name));

				ArrayList<ExprSignal> l_in = new ArrayList<ExprSignal>();//input (id) list for call the process pprocess	    
				right_name = aadlColAPI.Subcomponent_GetName(src_sub_process) + "_" + aadlColAPI.Port_GetName(src_port);
				l_in.add(myLib.mkIdentifier(getSname( right_name) ));
				String latency_name = connection_name + "_Latency_time_event";
				l_in.add(myLib.mkIdentifier(latency_name));

				//create a local event xx_Latency_time_event
				ArrayList<Declaration> vdecl = new ArrayList<Declaration>();
				SignalsDeclarationImpl Latency_time_event = myLib.mkEventDecl(latency_name);
				vdecl.add(Latency_time_event);
				myLib.STree_addLocalDeclaration(vprocess, vdecl);

				ArrayList<ExprSignal> l_out = new ArrayList<ExprSignal>();
				left_name = aadlColAPI.Subcomponent_GetName(dst_sub_process) + "_" + aadlColAPI.Port_GetName(dst_port);
				l_out.add(myLib.mkIdentifier(getSname( right_name)));
				return myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(process_connection_name), l_param, l_in, l_out);
			}
			Trace("\n Data port connection from a process subcomponent to xx not yet implemented ");
			return null;
		}
		Trace("\n data port connection case not yet implemented");
		return null;
	}

	
	/** 
	 * Returns a Signal signal definition (an equation) that represents a (event) port connection.
	 * 
	 * @param compImpl a component implementation component that contains this event port connection (econnection)
	 * @param vprocess a Signal process that represents an AADL component which contains this connection
	 * @param econnection a event port connection
	 */
	//public SignalsDefinitionImpl ASME2SSME_EventPortConnection(EventConnection econnection){
	//		SignalsDefinitionImpl eq = new SignalsDefinitionImpl();
	//		return eq;
	//}
	public SignalsDefinitionImpl ASME2SSME_EventPortConnection(ComponentImplementation compImpl, ProcessModelImpl vprocess, PortConnection econnection){

		//SignalsDefinitionImpl eq = new SignalsDefinitionImpl();
		String connection_name = aadlColAPI.Connection_GetName(econnection);

		Trace("===== ASME2SSME_Translate_PortConnection: ASME2SSME_EventPortConnection === " + connection_name);
		if(! aadlColAPI.Is_EventPortConnection(econnection)){
			Trace("not event port connection");
			return null;
		}
		
		Context src_context = aadlColAPI.PortConnection_GetSrcContext(econnection);
		Context dst_context = aadlColAPI.PortConnection_GetDstContext(econnection);
		Port src_element = aadlColAPI.PortConnection_GetSrc(econnection);
		Port dst_element = aadlColAPI.PortConnection_GetDst(econnection);

		EventPort src_port = (EventPort) src_element;
		EventPort dst_port = (EventPort) dst_element;
		//1) process  
		//if the context is null, then the compImpl is considered as its context
		if((src_context==null) && (compImpl instanceof ProcessImplementation)) 
			return ASME2SSME_EventPortConnection_S_Process(compImpl, src_context, dst_context, src_port, dst_port);

		//2) thread subcomponent  
		if (src_context instanceof ThreadSubcomponent) 
			return ASME2SSME_EventPortConnection_S_SubThread(compImpl, src_context, dst_context, src_port, dst_port);


		//3) thread  
		if((src_context == null ) && ( compImpl instanceof ThreadImplementation))  
			return ASME2SSME_EventPortConnection_S_Thread(compImpl, src_context, dst_context, src_port, dst_port);


		//4) subprogram subcomponent 
		if(src_context instanceof SubprogramSubcomponent) 
			return ASME2SSME_EventPortConnection_S_SubSubprogram(compImpl, src_context, dst_context, src_port, dst_port);


		//5) system  
		if ((src_context== null) && (compImpl instanceof SystemImplementation))
			return ASME2SSME_EventPortConnection_S_System(compImpl, src_context, dst_context, src_port, dst_port);			

		//6) sub system  
		if (src_context instanceof SystemSubcomponent)
			return ASME2SSME_EventPortConnection_S_SubSystem(compImpl, src_context, dst_context, src_port, dst_port);	

		//7) sub device  
		if (src_context instanceof DeviceSubcomponent) 
			return ASME2SSME_EventPortConnection_S_SubDevice(compImpl, src_context, dst_context, src_port, dst_port);

		//8) sub process  
		if (src_context instanceof ProcessSubcomponent) 
			return ASME2SSME_EventPortConnection_S_SubProcess(compImpl, src_context, dst_context, src_port, dst_port);

		Trace("ASME2SSME_EventPortConnection cases not implemented");
		return null;
	}


	/**
	 * Returns a Signal signal definition (an equation) that represents a (event data) port connection,  
	 * the source of which is a process component
	 * <p>
	 * @param src_context
	 * @param dst_context
	 * @param src_port
	 * @param dst_port
	 * @return
	 */
	public SignalsDefinitionImpl ASME2SSME_EventDataPortConnection_S_Process(ComponentImplementation compImpl,
			Context src_context, Context dst_context, EventDataPort src_port, EventDataPort dst_port){

		String left_name, right_name;
		if ((src_context == null) && (compImpl instanceof ProcessImplementation)) {

			right_name = aadlColAPI.Port_GetName(src_port);
			ExprSignal right_expr = myLib.mkIdentifier(right_name);

			//process->thread subcomponent
			if (dst_context instanceof ThreadSubcomponent){

				ThreadSubcomponent dst_sub_thread = (ThreadSubcomponent) dst_context;
				left_name =  aadlColAPI.Subcomponent_GetName(dst_sub_thread) + "_" + aadlColAPI.Port_GetName(dst_port);		    	
				ExprSignal left_expr = myLib.mkIdentifier(getSname( left_name));				
				return (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
			}
			Trace("\n Event data port connection from process some case not yet implemented");
			return null;
		}
		Trace("\n Event data port connection, not source from process");
		return null;
	}


	
	

	/** 
	 * Returns a Signal signal definition (an equation) that represents a (event data) port connection.
	 * 
	 * @param compImpl a component implementation component
	 * @param vprocess a Signal process that represents an AADL component which contains this connection
	 * @param c1 a event data port connection
	 */
	//public SignalsDefinitionImpl ASME2SSME_EventDataPortConnection(EventDataConnection edconnection){
	//		SignalsDefinitionImpl eq = new SignalsDefinitionImpl();
	//		return eq;
	//}
	public SignalsDefinitionImpl ASME2SSME_EventDataPortConnection(ComponentImplementation compImpl, ProcessModelImpl vprocess, PortConnection c1){

		String connection_name = aadlColAPI.Connection_GetName(c1);

		Trace("===== ASME2SSME_Translate_PortConnection: ASME2SSME_EventDataPortConnection === " + connection_name);
		if(! aadlColAPI.Is_EventDataPortConnection(c1)){
			Trace("not event data port connection");
			return null;
		}

		Context src_context = aadlColAPI.PortConnection_GetSrcContext(c1);  
		Context dst_context = aadlColAPI.PortConnection_GetDstContext(c1);  
		Port src_element =  aadlColAPI.PortConnection_GetSrc(c1);  
		Port dst_element =  aadlColAPI.PortConnection_GetDst(c1);
		EventDataPort src_port = (EventDataPort) src_element;
		EventDataPort dst_port = (EventDataPort) dst_element;

		//1) process 
		if((src_context == null) && (compImpl instanceof ProcessImplementation))  
			return ASME2SSME_EventDataPortConnection_S_Process(compImpl, src_context, dst_context, src_port, dst_port);

		//2) thread subcomponent  
		else if(src_context instanceof ThreadSubcomponent)  
			return ASME2SSME_EventDataPortConnection_S_SubThread(compImpl, src_context, dst_context, src_port, dst_port);

		//3) thread  
		else if((src_context == null ) && ( compImpl instanceof ThreadImplementation)) 
			return ASME2SSME_EventDataPortConnection_S_Thread(compImpl, src_context, dst_context, src_port, dst_port);

		//4) subprogram subcomponent  
		else if(src_context instanceof SubprogramSubcomponent) 
			return ASME2SSME_EventDataPortConnection_S_SubSubprogram(compImpl, src_context, dst_context, src_port, dst_port);

		//5) system  
		else if((src_context == null) && (compImpl instanceof SystemImplementation))  
			return ASME2SSME_EventDataPortConnection_S_System(compImpl, src_context, dst_context, src_port, dst_port);

		//6) system subcomponent 
		else if(src_context instanceof SystemSubcomponent) 
			return ASME2SSME_EventDataPortConnection_S_SubSystem(compImpl, src_context, dst_context, src_port, dst_port);

		//7) device subcomponent  
		else if(src_context instanceof DeviceSubcomponent)  
			return ASME2SSME_EventDataPortConnection_S_SubDevice(compImpl, src_context, dst_context, src_port, dst_port);

		//8) process subcomponent 
		else if(src_context instanceof ProcessSubcomponent) 
			return ASME2SSME_EventDataPortConnection_S_SubProcess(compImpl, src_context, dst_context, src_port, dst_port);
		Trace("\n Event data port connection some case not yet implemented");
		return null;
	}



	
}
