
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.osate.aadl2.*;
import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;

public class ASME2SSME_Translate_Process extends ASME2SSME_Translate_Property{

	private AADLCoLAPI aadlColAPI ;
	private signalTreeAPI myLib ;
	private ASME2SSME_Translate_Common asme2ssme_common;
	
	 

	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_Process() {
		aadlColAPI = new AADLCoLAPI();
		myLib= new signalTreeAPI();
		asme2ssme_common = new ASME2SSME_Translate_Common();
	}


	/** 
	 * Returns a Signal process type declaration
	 * <p>
	 * This declaration is in the form of "type process TT_xx_process = (? ... ! ...)",
	 * where "xx" is the name of the process implementation component, 
	 * and the interface is made by the called method ASME2SSME_Type_Process_Interface()
	 * 
	 * @param ptype process type component
	 * @param pimpl process implementation component, whose type is ptype
	 * @return a Signal process type declaration
	 */
	public TypesDeclarationImpl ASME2SSME_ProcessType(org.osate.aadl2.ProcessType ptype, ProcessImplementation pimpl){

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_ProcessType: " + ptype.getName());

		TypesDeclarationImpl type_decl = new TypesDeclarationImpl();
		String impl_name = aadlColAPI.ComponentImpl_GetName(pimpl);
		String type_name = Name_of_Type_Process(Name_of_Process(impl_name));
		ProcessTypeImpl process_type = new ProcessTypeImpl();
		process_type.setIdentifier(myLib.mkIdentifier(type_name));
		process_type.setInterfaceDefinition(myLib.STree_mkSkeletonInterface());
		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) process_type.getInterfaceDefinition();		
		ASME2SSME_Type_Process_Interface(inter,ptype, pimpl);
		type_decl.getTypeDefinition().add(process_type);
		myLib.STree_addComment(type_decl, "process type for AADL process " + ptype.getName(), true);
		// Features 
		ASME2SSME_Process_Features(ptype, inter);
		// Flow specifications 
		ASME2SSME_Process_FlowSpecs(ptype);
		// Modes : no modes in version 1.5.8
		
		// Properties:  properties implemented in ProcessImpl_Property() sub process

		Trace("\n ======================== end ASME2SSME_ProcessType: " + ptype.getName() + " ========================= \n");
		return type_decl;
	}



	/**
	 * Translates process features
	 * <p>
	 * For each feature, use its corresponding translate method
	 * 
	 * @param syst a system type
	 */
	private void ASME2SSME_Process_Features(org.osate.aadl2.ProcessType ptype, ProcessModelInterfaceImpl inter){

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_Process_Features: " + ptype.getName());

		EList<Feature> pf = ptype.getAllFeatures();
		ASME2SSME_Translate_Feature asme2ssme_feature = new ASME2SSME_Translate_Feature();

		if (pf != null) {
			//port=============
			// data port: data port is implemented as a signal in the interface
			// EList<DataPort> dport_list = ptype.getOwnedDataPorts();

			//event port: event port is implemented as a signal in the interface
			//	 EList<EventPort> eport_list = ptype.getOwnedEventPorts();

			//event data port: event data port is implemented as a signal in the interface
			// EList<EventDataPort> edport_list = ptype.getOwnedEventDataPorts();
			 

			// feature group in OSATE2
			ArrayList<FeatureGroup> fgroup_list = aadlColAPI.Component_GetFeatureGroups(ptype);
			ASME2SSME_Translate_FeatureGroup asme2ssme_fg = new ASME2SSME_Translate_FeatureGroup();
			for(int i1=0; i1<fgroup_list.size(); i1++)
			{
				FeatureGroup pg = fgroup_list.get(i1);				
				if(asme2ssme_fg.ASME2SSME_FeatureGroup_bw(pg) != null){
					SignalsDeclarationImpl in1 = asme2ssme_fg.ASME2SSME_FeatureGroup_bw(pg);
					myLib.STree_addDeclarationToaInterface(inter,in1,2);
				}

				if(asme2ssme_fg.ASME2SSME_FeatureGroup_wb(pg) != null){
					SignalsDeclarationImpl out1 = asme2ssme_fg.ASME2SSME_FeatureGroup_wb(pg);
					myLib.STree_addDeclarationToaInterface(inter,out1,2);
				}
			} 
			//data access 
			EList<DataAccess> da_list = ptype.getOwnedDataAccesses();
			if(da_list.size()>0)
				for (int i4=0; i4<da_list.size(); i4++)
					asme2ssme_feature.ASME2SSME_DataAccess(da_list.get(i4));			

			//subprogram group: not exist in version 1.5.8
		}
	}

	/**
	 * Translates flow specifications of a process type
	 * 
	 * @param pt a process type component
	 */
	private void ASME2SSME_Process_FlowSpecs(org.osate.aadl2.ProcessType pt){

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_Process_Features: " + pt.getName());

		// ProcessModelImpl p1= asme2ssme_common.ASME2SSME_ComponentType_FlowSpecs(pt);
		// myLib.STree_addProcessDeclaration(vprocess, p1);
		// ArrayList<ExprParam> lparam = new ArrayList<ExprParam> ();
		// String pname = Name_of_Flow(pt.getName());
		// ProcessExpansionImpl call1 = myLib.STree_mkProcessCallWithParameters(myLib.mkIdentifier(pname), lparam);
		// myLib.STree_addExpressionOfProcessToaModel(vprocess, call1);
	}


	/** 
	 * Adds inputs/outputs to a process type interface.
	 * <p>
	 * The original inputs/outputs of the process is add in the method ASME2SSME_Type_Process_Interface_Original(). 
	 * The control signals for its sub threads, 
	 * i.e., xx_CTL1, xx_CTL2, xx_TIME1 will be added in the method ASME2SSME_Type_Process_Interface_Refine1(). 
	 * The events that indicate the arrival of event or event data is added as outputs (method ASME2SSME_Type_Process_Interface_Refine2()).
	 * 
	 * @param inter a Signal process model interface that represents an AADL process type
	 * @param st an AADL component (process) type 
	 */
	private void ASME2SSME_Type_Process_Interface(ProcessModelInterfaceImpl inter, ComponentType st, ProcessImplementation pimpl){		

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_Type_Process_Interface: " + aadlColAPI.ComponentType_GetName(st));

		ASME2SSME_Type_Process_Interface_Original(inter, st);
		ASME2SSME_Type_Process_Interface_Refine1(inter, st, pimpl);
		ASME2SSME_Type_Process_Interface_Refine2(inter, st,pimpl);

		Trace("\n ========================= end ASME2SSME_Type_Process_Interface: " + aadlColAPI.ComponentType_GetName(st) + " ========================= \n");
	}


	/** 
	 * Adds inputs/outputs (from the original process in/out port) to a process type interface. The signal TOP_ID is added as input.
	 * 
	 * @param inter a Signal process model interface that represents an AADL process type
	 * @param st an AADL component (process) type 
	 */
	private void ASME2SSME_Type_Process_Interface_Original(ProcessModelInterfaceImpl inter, ComponentType st){		

		ASME2SSME_Translate_Feature asme2ssme_feature = new ASME2SSME_Translate_Feature();

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_Type_Process_Interface_Original: " + aadlColAPI.ComponentType_GetName(st));
		SignalsDeclarationImpl vdecsig;      
		// in event: top
		myLib.STree_addDeclarationToaInterface(inter, myLib.mkEventDecl(TOP_ID), 2);

		ArrayList<DataPort> list_ind_port = aadlColAPI.Component_GetInDataPort(st);
		ArrayList<DataPort> list_outd_port = aadlColAPI.Component_GetOutDataPort(st);
		ArrayList<EventPort> list_ine_port = aadlColAPI.Component_GetInEventPort(st);
		ArrayList<EventPort> list_oute_port = aadlColAPI.Component_GetOutEventPort(st);
		ArrayList<EventDataPort> list_ined_port = aadlColAPI.Component_GetInEventDataPort(st);
		ArrayList<EventDataPort> list_outed_port = aadlColAPI.Component_GetOutEventDataPort(st);
		// In Data Port
		if (list_ind_port.size() >0 ){
			Iterator<?> itd1 = list_ind_port.iterator();
			while (itd1.hasNext())
			{
				DataPort dataPort = (DataPort) itd1.next();
				vdecsig = asme2ssme_feature.ASME2SSME_DataPort(dataPort);
				myLib.STree_addDeclarationToaInterface(inter,vdecsig,2);
			}
		}
		// Out Data Port
		if (list_outd_port.size() >0 ){
			Iterator<?> itd2 = list_outd_port.iterator(); 
			while(itd2.hasNext())
			{
				DataPort dataPort = (DataPort) itd2.next();
				vdecsig = asme2ssme_feature.ASME2SSME_DataPort(dataPort);
				myLib.STree_addDeclarationToaInterface(inter,vdecsig,3);
			}
		}
		// In Event Port
		if (list_ine_port.size() >0 ){
			Iterator<?> ite1 = list_ine_port.iterator();
			while (ite1.hasNext())
			{
				EventPort eventPort = (EventPort) ite1.next();
				vdecsig = asme2ssme_feature.ASME2SSME_EventPort(eventPort);	
				myLib.STree_addDeclarationToaInterface(inter,vdecsig,2);
			}
		}
		// Out Event Port
		if (list_oute_port.size() >0 ){
			Iterator<?> ite2 = list_oute_port.iterator();
			while (ite2.hasNext())
			{
				EventPort eventPort = (EventPort) ite2.next();
				vdecsig = asme2ssme_feature.ASME2SSME_EventPort(eventPort);
				myLib.STree_addDeclarationToaInterface(inter,vdecsig,3);
			}
		}
		// In  Event Data Port
		if (list_ined_port.size() >0 ){
			Iterator<?> ited1 = list_ined_port.iterator();
			while (ited1.hasNext())
			{
				EventDataPort eventdataPort = (EventDataPort) ited1.next();
				vdecsig = asme2ssme_feature.ASME2SSME_EventDataPort(eventdataPort);	
				myLib.STree_addDeclarationToaInterface(inter,vdecsig,2);
			}
		}
		// Out Event Data Port
		if (list_outed_port.size() >0 ){
			Iterator<?> ited2 = list_outed_port.iterator();
			while (ited2.hasNext())
			{
				EventDataPort eventdataPort = (EventDataPort) ited2.next();
				vdecsig = asme2ssme_feature.ASME2SSME_EventDataPort(eventdataPort);
				myLib.STree_addDeclarationToaInterface(inter,vdecsig,3);
			}
		}

		Trace("\n ========================= end ASME2SSME_Type_Process_Interface_Original: " + aadlColAPI.ComponentType_GetName(st) + " ========================= \n");
	}


	/** 
	 * Adds control signals of each sub thread into the interface of process, i.e., xx_ctl1, xx_ctl2, xx_time1, xx_Alarm.
	 * (ASME2SSME_add_Declaration_to_ProcessType_Interface() which is called in the method ASME2SSME_Thread_instance()).
	 * 
	 * @param inter a Signal process model interface that represents an AADL process type
	 * @param st an AADL component (process) type 
	 */
	private void ASME2SSME_Type_Process_Interface_Refine1(ProcessModelInterfaceImpl inter, ComponentType st, ComponentImplementation si){	

		Trace("\n ========== ASME2SSME_Translate_Process: ASME2SSME_Type_Process_Interface_Refine1 " + st.getName());

		ArrayList<ThreadSubcomponent> list1 = aadlColAPI.Process_GetSubcomponentsThread((ProcessImplementation) si);
		if(list1.size()>0){
			for(int i =0; i< list1.size(); i++){
				ThreadSubcomponent t1 = list1.get(i);
				ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(t1);
				ThreadType tt1= (ThreadType) aadlColAPI.ComponentImpl_GetComponentType(tt);
				
				String name1 = aadlColAPI.Subcomponent_GetName(t1) + "_" + CTL1_ID;
				SignalsDeclarationImpl sig1 = myLib.mkDecl(name1, Name_of_CTL1_type());
				myLib.STree_addDeclarationToaInterface(inter, sig1, 2);

				if(aadlColAPI.Thread_HasPorts(tt1)){
					String name2 = aadlColAPI.Subcomponent_GetName(t1) +  "_" + TIME1_ID;
					SignalsDeclarationImpl sig2 = myLib.mkDecl(name2, Name_of_ThreadTimeBundle(tt1.getName()));
					myLib.STree_addDeclarationToaInterface(inter, sig2, 2);
				}
				String name3 = aadlColAPI.Subcomponent_GetName(t1) + "_" + CTL2_ID;
				SignalsDeclarationImpl sig3 = myLib.mkDecl(name3, Name_of_CTL2_type());
				myLib.STree_addDeclarationToaInterface(inter, sig3, 3);

				String name4 = aadlColAPI.Subcomponent_GetName(t1) + "_" + ALARM_ID;
				SignalsDeclarationImpl sig4 = myLib.mkBooleanDecl(name4);
				myLib.STree_addDeclarationToaInterface(inter, sig4, 3);
			}
			 
		}
	}


	/** 
	 * Adds output event signals to the interface of process.  
	 * Each event indicates the arrival of a event or event data of a sub non periodic thread.
	 * event t_i_TE
	 * 
	 * @param inter a Signal process model interface that represents an AADL process type
	 * @param st an AADL component (process) type 
	 */
	private void ASME2SSME_Type_Process_Interface_Refine2(ProcessModelInterfaceImpl inter, ComponentType st, ComponentImplementation si){	

		Trace("\n ========== ASME2SSME_Translate_Process: ASME2SSME_Type_Process_Interface_Refine2 " + st.getName());

		ArrayList<ThreadSubcomponent> list1 = aadlColAPI.Process_GetSubcomponentsThread((ProcessImplementation) si);
		if (list1.size()>0){
			//for each sub thread:
			for(int i =0; i< list1.size(); i++){
				ThreadSubcomponent t1 = list1.get(i);
				ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(t1);
				ThreadType tt1= (ThreadType) aadlColAPI.ComponentImpl_GetComponentType(tt);

				String dispatch = aadlColAPI.Thread_Get_DispatchProtocol_str(tt);
				Trace(t1.getName() + " " + dispatch);
				//if the thread is not periodic, an event that indicates the arrival of in event or event data port is added to the outputs of process type
				if (dispatch!= null && dispatch.equals("Periodic") == false){

					ArrayList<EventPort> list2 = aadlColAPI.Component_GetInEventPort(tt1);
					ArrayList<EventDataPort> list3 = aadlColAPI.Component_GetInEventDataPort(tt1);

					//for each in event port i of sub thread t: add an event e_t_i as an output of process type
					if (list2.size()>0) {
						for(int j=0; j<list2.size(); j++){
							EventPort ep1 = list2.get(j);
							String name1 = Name_of_TriggerDispatch_Event(t1.getName() + "_" + ep1.getName());
							SignalsDeclarationImpl sig1 = myLib.mkEventDecl(name1);
							myLib.STree_addDeclarationToaInterface(inter, sig1, 3);
						}
					}
					else 
						Trace("no in event port of sub thread " + t1.getName());
					//for each in event data port i of sub thread t: add an event e_t_i as an output of process type
					if (list3.size()>0) {
						for(int k=0; k<list3.size(); k++){
							EventDataPort edp1 = list3.get(k);
							String name2 = Name_of_TriggerDispatch_Event(t1.getName() + "_" + edp1.getName());
							SignalsDeclarationImpl sig2 = myLib.mkEventDecl(name2);
							myLib.STree_addDeclarationToaInterface(inter, sig2, 3);
						}
					}
					else 
						Trace("no in event data port of sub thread " + t1.getName());
				}			    
			}
		}

	}


	/** 
	 * Returns a Signal process that represents an AADL process implementaton component
	 * <p>
	 * The interface is an identifier whose name is "TT_xx" (xx is the name of this Signal process). 
	 * The body is composed of xx_behavior(), xx_property() 
	 * and its subcomponents: threads, data and subprograms.
	 * 
	 * @param p process implementation component
	 * @param sys_process a Signal process that represents its upper level AADL system component
	 * @return a Signal process that represents an AADL process implementation
	 */
	public ProcessModelImpl ASME2SSME_ProcessImpl(ProcessImplementation p, 
			SignalElementImpl /*ProcessModelImpl*/ sys_process, ModuleImpl vmodule, AadlPackage aadlpkg){	

		ProcessModelImpl vprocess;
		String sname = aadlColAPI.ComponentImpl_GetName(p);

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_ProcessImpl: " + sname);

		String pname = Name_of_Process(sname);
		vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );
		//interface 
		asme2ssme_common.ASME2SSME_add_IDInterface_to_Process(vprocess, Name_of_Type_Process(pname));
		//process behavior 
		ProcessModelImpl p_behavior = ASME2SSME_process_behavior(pname);
		String process_behavior_name = myLib.STree_GetStringName(p_behavior.getIdentifier());
		myLib.STree_addProcessDeclaration(vprocess, p_behavior);
		ProcessCallImpl behavior_inst = myLib.STree_mkModelCall(process_behavior_name);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, behavior_inst);
		myLib.STree_addComment(behavior_inst, "process behavior sub process", false);
		// process property 
		ProcessModelImpl p_property = ASME2SSME_process_property(p);
		String process_property_name = myLib.STree_GetStringName(p_property.getIdentifier());
		myLib.STree_addProcessDeclaration(vprocess, p_property);
		ProcessCallImpl property_inst = myLib.STree_mkModelCall(process_property_name);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, property_inst);
		myLib.STree_addComment(property_inst, "process property sub process", false);
		//subcomponents 
		ArrayList<ThreadSubcomponent> list_thread_subcomponent = aadlColAPI.Process_GetSubcomponentsThread(p);
		ArrayList<DataSubcomponent> sub_data = aadlColAPI.Process_GetSubcomponentsData(p);
		ArrayList<SubprogramSubcomponent> sub_subprogram = aadlColAPI.Process_GetSubcomponents_Subprogram(p);
		ArrayList<ThreadGroupSubcomponent> list2 = aadlColAPI.Process_GetSubcomponentsThreadGroup(p);

		//thread subcomponent
		ASME2SSME_ProcessImpl_SubThreads(vprocess, list_thread_subcomponent, sys_process, vmodule);
		// DataSubcomponent
		ASME2SSME_ProcessImpl_SubData(vprocess, sub_data, p);

		//adds connections of thread local data variable with shared data
		//ASME2SSME_ProcessImpl_DataAccessConnection(p, vprocess);

		//Subprogram subcomponent
		ASME2SSME_ProcessImpl_SubSubprograms(vprocess, sub_subprogram, p, vmodule);
		//instantiate subprograms (not subcomponent of any process or thread) that is called by its subthreads
		ASME2SSME_ProcessImpl_Instantiate_Subprograms(vprocess, p, vmodule, aadlpkg);
		//give the clocks of shared subprograms that are subcomponents of its subthread
		ASME2SSME_ProcessImpl_SubThread_SubSubprograms_Clocks(vprocess, p, vmodule, aadlpkg);
		//subprogram group subcomponent:not exist in version 1.5.8
		//thread group subcomponent
		ASME2SSME_ProcessImpl_SubThreadGroups(vprocess, list2);

		//abstract subcomponent: not exist in version 1.5.8

		//connections 
		ASME2SSME_ProcessImpl_Connections(p, vprocess);		
		//Flows 
		ASME2SSME_ProcessImpl_Flows(vprocess, p);
		//Modes 
		ASME2SSME_ProcessImpl_Modes(p);

		//Properties: properties implemented in ProcessImpl_Property() sub process	

		//Definition of out events (which indicate the arrival of in event/event data port of a non-periodic thread)
		ASME2SSME_ProcessImpl_OutEvent_Definition(vprocess, p);

		Trace("\n ========================= end ASME2SSME_ProcessImpl: " + sname + " ========================= \n");
		return vprocess;
	}


	/** 
	 * Returns a Signal process "xx_process_behavior()" that represents the behavior of an AADL process
	 * 
	 * @param p_name string name of the AADL process
	 * @return a ProcessModelImpl (p_behavior)
	 */
	private ProcessModelImpl ASME2SSME_process_behavior(String p_name){

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_process_behavior: " + p_name);
		String bname = p_name + "_behavior";
		ProcessModelImpl p_behavior = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(bname) );

		if (myLib.STree_IsProcessNull(p_behavior) && (! myLib.STree_IsProcessHasOutput(p_behavior)))
			myLib.STree_addNullProc(p_behavior);
		return p_behavior;
	}


	/** 
	 * Returns a Signal process "xx_process_property" that represents the property of an AADL process
	 * 
	 * @param p_name string name of the AADL process
	 * @return a ProcessModelImpl (p_property)
	 */
	private ProcessModelImpl ASME2SSME_process_property(ProcessImplementation p){

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_process_property: " + p.getName());

		String sname = aadlColAPI.ComponentImpl_GetName(p);
		String p_name = Name_of_Process(sname);
		String pname = p_name + "_property";
		ProcessModelImpl p_property = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );

		ArrayList<PropertyAssociation> property_association_list1 = aadlColAPI.ComponentImpl_GetPropertyAssociations(p);
		org.osate.aadl2.ProcessType p_type = (org.osate.aadl2.ProcessType) aadlColAPI.ComponentImpl_GetComponentType(p);
		ArrayList<PropertyAssociation> property_association_list2 = aadlColAPI.Component_GetPropertyAssociations(p_type);
		property_association_list1.addAll(property_association_list2);
		if(property_association_list1.size()>0){

			for(int i=0; i<property_association_list1.size(); i++)
			{
				PropertyAssociation pa = property_association_list1.get(i);
				ProcessCallImpl pcall1 =  ASME2SSME_Property_Call(pa);
				myLib.STree_addExpressionOfProcessToaModel(p_property, pcall1);   
			}
		}
		if( ! myLib.STree_IsProcessHasOutput(p_property) && (myLib.STree_IsProcessNull(p_property))) 
			myLib.STree_addNullProc(p_property);
		return p_property;
	}

   /**
     * Creates a thread instance, and adds it into a process (vprocess) body.
     * <p>
     * For each formal declaration (parameter,input,output) of the thread declaration (pprocess),
     * create a local signal (tt_ss) as an effective parameter of the call,
     * where tt is a string name which equals to the name of this sub thread,
     * and ss is the name of the original signal of the thread declaration.
     * <p>
     * For the formal signals (xx) ended with Error, Complete,
     * creates an output (tt_Error, tt_Complete).
     * For the formal signals (yy) ended with _Frozen_time_event, _Output_time_event, Dispatch, Resume, Deadline,
     * creates an input (tt_yy).
     * <p>
     * Creates a thread instance, and adds it into a process (vprocess) body.
     *
     * @param vprocess a Signal process that represents an AADL process.
     * @param pprocess a Signal process that represents a thread component
     * @param sys_process a Signal process that represents a system implementation
     * @param label a String label that represents the name of this sub thread instance
     */
    private void ASME2SSME_Thread_instance(ProcessModelImpl vprocess, ProcessModelImpl pprocess,
                                          SignalElementImpl /*ProcessModelImpl*/ sys_process, String label, ThreadImplementation timpl) {

        Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_Thread_instance " + timpl.getName());

        String instance_name = pprocess.getIdentifier().getAtomString();

        ArrayList<ExprParam> l_param = new ArrayList<ExprParam>();//parameter (id) list for call the process pprocess
        ArrayList<ExprSignal> l_in = new ArrayList<ExprSignal>();//input (id) list for call the process pprocess
        ArrayList<ExprSignal> l_out = new ArrayList<ExprSignal>();//output (id) list for call the process pprocess
        ArrayList<Declaration> vdecl = new ArrayList<Declaration>();//local lists

        ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) pprocess.getInterfaceDefinition();
        if (inter != null) {

            IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
            if (vioProfile != null)
            {
                EList<TypesOrSignalsDeclaration> l_inputs = vioProfile.getInputs();
                Iterator<TypesOrSignalsDeclaration> it_i = l_inputs.iterator();
                while (it_i.hasNext())
                {
                    SignalsDeclarationImpl ainput = (SignalsDeclarationImpl) it_i.next();
                    EList<SignalNaming> list2 = ainput.getListSignal();

                    if (ainput.getType() instanceof PredefinedTypeImpl) {
                        PredefinedTypeImpl type_2 = (PredefinedTypeImpl) ainput.getType();
                        Trace(type_2.toString());

                        for (int i = 0; i < list2.size(); i++) {
                            SignalNaming sig_name2 = list2.get(i);
                            Identifier id_sig_name2 = sig_name2.getIdentifier();
                            String str_sig_name2 = id_sig_name2.getAtomString();
                            String name2 = label + '_' + str_sig_name2;
                            String lname1 = getSname(name2);
                            
                            if (str_sig_name2.endsWith("_Frozen_time_event") | str_sig_name2.endsWith("_Output_time_event")) 
                                l_in.add(myLib.mkIdentifier(name2));
                            else if (str_sig_name2.equals(TOP_ID))
                                l_in.add(myLib.mkIdentifier(TOP_ID));
                            else {
                                PredefinedTypeImpl ptype1 = myLib.STree_mkSamePredefinedType(ainput);
                                SignalsDeclarationImpl sig2 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(lname1), ptype1, null);
                                l_in.add(myLib.mkIdentifier(lname1 /*name2*/));
                                vdecl.add(sig2);
                            }
                        }
                    }
                    else {
                        Identifier type_id = (Identifier) ainput.getType();
                        String type_name = type_id.getAtomString();
                        IdentifierImpl type2 = myLib.mkIdentifier(type_name);
                        Trace(type2.getAtomString());

                        for (int i = 0; i < list2.size(); i++) {
                            SignalNaming sig_name2 = list2.get(i);
                            Identifier id_sig_name2 = sig_name2.getIdentifier();
                            String name2 = label + '_' + id_sig_name2.getAtomString();

                            if (name2.endsWith("_ctl1"))  
                                l_in.add(myLib.mkIdentifier(name2));
                            else if (name2.endsWith(TIME1_ID))
                                l_in.add(myLib.mkIdentifier(name2));
                            else {
                                String lname2 = getSname(name2);
                                SignalsDeclarationImpl sig2 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(lname2), type2, null);
                                l_in.add(myLib.mkIdentifier(lname2 /*name2*/));
                                vdecl.add(sig2);
                            }

                        }
                    }
                }

                EList<TypesOrSignalsDeclaration> l_outputs = vioProfile.getOutputs();
                Iterator<TypesOrSignalsDeclaration> it_o = l_outputs.iterator();
                while (it_o.hasNext())
                {
                    SignalsDeclarationImpl aoutput = (SignalsDeclarationImpl) it_o.next();
                    EList<SignalNaming> list3 = aoutput.getListSignal();

                    if (aoutput.getType() instanceof PredefinedTypeImpl) {
                        for (int i = 0; i < list3.size(); i++) {
                            SignalNaming sig_name3 = list3.get(i);
                            Identifier id_sig_name3 = sig_name3.getIdentifier();
                            String str_sig_name3 = id_sig_name3.getAtomString();
                            //Trace(str_sig_name3);
                            String name3 = label + '_' + str_sig_name3;
                            String lname3 = getSname(name3);
                            
                            if(str_sig_name3.endsWith("Alarm")) 
                                l_out.add(myLib.mkIdentifier(name3));
                            else {
                            	PredefinedTypeImpl ptype3 = myLib.STree_mkSamePredefinedType(aoutput);
                                SignalsDeclarationImpl sig3 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(lname3), ptype3, null);
                                l_out.add(myLib.mkIdentifier(lname3 ));
                                vdecl.add(sig3);
                            }
                        }
                    }
                    else {
                        Identifier type_id = (Identifier) aoutput.getType();
                        String type_name = type_id.getAtomString();
                        IdentifierImpl type3 = myLib.mkIdentifier(type_name);
                        //Trace(type3.getAtomString());

                        for (int i = 0; i < list3.size(); i++) {
                            SignalNaming sig_name3 = list3.get(i);
                            Identifier id_sig_name3 = sig_name3.getIdentifier();
                            String name3 = label + '_' + id_sig_name3.getAtomString();
                            String lname4 = getSname(name3);
                            
                            if (name3.endsWith("_ctl2")) 
                                l_out.add(myLib.mkIdentifier(name3));
                            else {
                                SignalsDeclarationImpl sig3 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(lname4), type3, null);
                                l_out.add(myLib.mkIdentifier(lname4));
                                vdecl.add(sig3);
                            }

                        }
                    }
                }
            }
        }

        if (l_out.size()>0) {
            SignalsDefinitionImpl call_eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(instance_name), l_param, l_in, l_out);
            myLib.STree_addExpressionOfProcessToaModel(vprocess, call_eq);
            myLib.STree_addComment(call_eq, "instance of AADL thread " + label, false);
        }
        else {
            ProcessCallImpl pcall = myLib.STree_mkCompleteProcessCall(myLib.mkIdentifier(instance_name), l_param, l_in);
            myLib.STree_addExpressionOfProcessToaModel(vprocess, pcall);
            myLib.STree_addComment(pcall, "instance of AADL thread " + label, false);
        }
        myLib.STree_addLocalDeclaration(vprocess, vdecl);
        Trace("\n ========================= end ASME2SSME_Thread_instance " + timpl.getName() + " ========================= \n");
    }

	/** Translates the sub threads of a process implementation.
	 * <p>
	 * For each thread subcomponent, get its classifier (a thread implementation). 
	 * If this classifier has already been implemented, get its Signal process declaration (STree_GetProcessDeclarationFromProcess()), 
	 * otherwise, create a Signal process (ASME2SSME_ThreadImpl()). 
	 * Add an instance of this Signal process in the body of process_process (ASME2SSME_Thread_instance()).
	 * 
	 * @param process_process a Signal process that represents a process implementation component
	 * @param subthread_list a list of thread subcomponents of a process 
	 * @param sys_process a Signal process that represents a system implementation
	 */
	private void ASME2SSME_ProcessImpl_SubThreads(ProcessModelImpl process_process, ArrayList<ThreadSubcomponent> subthread_list, 
			SignalElementImpl /*ProcessModelImpl*/ sys_process, ModuleImpl vmodule){


		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_ProcessImpl_SubThreads ");		
		ArrayList<ThreadClassifier> l_thread_classifier = new ArrayList<ThreadClassifier>();
		Iterator<?> it_1 = subthread_list.iterator();
		while (it_1.hasNext())
		{
			ThreadSubcomponent tsub = (ThreadSubcomponent) it_1.next();
			String t_sub_name = aadlColAPI.Subcomponent_GetName(tsub);

			// ThreadImpl
			ThreadImplementation t = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(tsub);
			String thread_impl_name = aadlColAPI.ComponentImpl_GetName(t);
			String thread_impl_process_name = Name_of_Thread(thread_impl_name);			       
			ProcessModelImpl lprocess6;
			if (l_thread_classifier.contains(t)){
				Trace("thread classifier process has already been created in Signal");
				lprocess6 = myLib.STree_GetProcessDeclarationFromProcess(process_process, thread_impl_process_name);
			}
			else {					 
				// ThreadImpl declaration
				lprocess6 = ASME2SSME_ProcessImpl_Thread(t, vmodule);
				myLib.STree_addProcessDeclaration(process_process, lprocess6);
				l_thread_classifier.add(t);
			}
			// Adding a reference to the model in sys_process
			ASME2SSME_Thread_instance(process_process, lprocess6, sys_process, t_sub_name, t);
		}
	}


	/** Translates the sub data of a process implementation.
	 * <p>
	 * For each data subcomponent, get its classifier (a data type or implementation). 
	 * If this classifier has not been implemented, 
	 * create a Signal type declaration for this data (ASME2SSME_DataType() or ASME2SSME_DataImpl()), 
	 * and add it to the declaration of process_process. 
	 * Adds shared data in the body: ASME2SSME_CreateSharedData(),
	 * and specify the clock of read and count of this shared data: 
	 * ASME2SSME_SharedData_Cnt_Sync(), ASME2SSME_SharedData_Read_Sync()
	 * 
	 * @param process_process a Signal process that represents a process implementation component
	 * @param subdata_list a list of data subcomponents of a system 
	 */
	private void ASME2SSME_ProcessImpl_SubData(ProcessModelImpl process_process, ArrayList<DataSubcomponent> subdata_list, ProcessImplementation p){


		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_ProcessImpl_SubData ");

		ArrayList<DataClassifier> l_data_classifier = new ArrayList<DataClassifier>();
		if(subdata_list.size()==0){
			Trace("ASME2SSME_ProcessImpl_SubData() size =0");
		}
		Iterator<?> it2 = subdata_list.iterator();
		while (it2.hasNext())
		{
			DataSubcomponent d1 = (DataSubcomponent) it2.next();
			Trace("data subcomponent: " + d1.getName() + ", classifier: " + d1.getClassifier() );

			DataClassifier data_classifier =  (DataClassifier) aadlColAPI.Datasubcomponent_GetClassifier(d1);

			if (l_data_classifier.contains(data_classifier)){
				Trace("Data classifier has already been created in Signal!!");
			}
			else {
				TypesDeclarationImpl type_decl = new TypesDeclarationImpl();
				ASME2SSME_Translate_Data asme2ssme_data = new ASME2SSME_Translate_Data();

				if (data_classifier instanceof DataType)
				{
					if(asme2ssme_data.ASME2SSME_DataType((DataType) data_classifier) != null){
						type_decl = asme2ssme_data.ASME2SSME_DataType((DataType) data_classifier);
						myLib.STree_addaDeclarationToProcess(process_process, type_decl);
						l_data_classifier.add(data_classifier);
					}

				}
				else if (data_classifier instanceof DataImplementation){
					type_decl = asme2ssme_data.ASME2SSME_DataImpl((DataImplementation) data_classifier);
					myLib.STree_addaDeclarationToProcess(process_process, type_decl);
					l_data_classifier.add(data_classifier);
				}
				else{
					java.lang.System.err.print("\n data classifier illegal ");
					throw new IllegalArgumentException("data classifier illegal!!  \n");
				}

				//shared data: create fifo_reset{} process call, and add local shared signals
				asme2ssme_data.ASME2SSME_CreateSharedData(d1, process_process);   

				//specifies the clock for each cnt of a shared data: cnt ^= x_w ^+ x_r
				asme2ssme_data.ASME2SSME_SharedData_Cnt_Sync(d1, process_process);  

				//specifies the clock for each _r of a shared data: xx_r ^= l_1 ^+ l_2 ^+ ...
				asme2ssme_data.ASME2SSME_SharedData_Read_Sync(d1, process_process, p);   

				//specifies the clock for each _w of a shared data: xx_w ^= l_3 ^+ l_4 ^+ ...
				asme2ssme_data.ASME2SSME_SharedData_Write_Sync(d1, process_process, p);  

				//specifies the clock for each _reset of a shared data: xx_reset ^= l_5 ^+ l_6 ^+ ...
				asme2ssme_data.ASME2SSME_SharedData_Reset_Sync(d1, process_process, p);  
			}
		}

	}


	/** Translates the sub subprograms of a process implementation. case 1
	 * <p>
	 * For each subprogram subcomponent, get its classifier (a subprogram type or implementation). 
	 * If this classifier has not been implemented, 
	 * create a Signal process for this subprogram (ASME2SSME_SubprogramType() or ASME2SSME_SubprogramImpl()), 
	 * and add it to the declaration of process_process. 
	 * Adds a process instance in the body: ASME2SSME_Create_SharedSubprogram(),
	 * and specify the call clock of this subprogram: 
	 * ASME2SSME_SharedSubprogram_Call_Sync_x(), ASME2SSME_SharedSubprogram_Sync_y()
	 * 
	 * @param process_process a Signal process that represents a process implementation component
	 * @param subprogram_list a list of subprogram subcomponents of a system 
	 * @param p a process implementation
	 */
	private void ASME2SSME_ProcessImpl_SubSubprograms(ProcessModelImpl process_process, 
			ArrayList<SubprogramSubcomponent> subprogram_list, ProcessImplementation p, ModuleImpl vmodule){


		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_ProcessImpl_SubSubprogram ");

		ArrayList<SubprogramClassifier> l_classifier = new ArrayList<SubprogramClassifier>();
		if(subprogram_list.size()==0){
			Trace("ASME2SSME_ProcessImpl_SubSubprogram() size =0");
		}
		Iterator<?> it2 = subprogram_list.iterator();
		while (it2.hasNext())
		{
			SubprogramSubcomponent d1 = (SubprogramSubcomponent) it2.next();
			Trace("Subprogram subcomponent: " + d1.getName() + ", classifier: " + d1.getClassifier() );

			SubprogramClassifier classifier1 =  (SubprogramClassifier) aadlColAPI.Subprogramsubcomponent_GetClassifier(d1);

			if (l_classifier.contains(classifier1)){
				Trace("Subprogram classifier has already been created!!");
			}
			else {
				ProcessModelImpl sp_process = new ProcessModelImpl();
				ASME2SSME_Translate_Subprogram asme2ssme_sp = new ASME2SSME_Translate_Subprogram();
				String sp_name = null; //name of subprogram type or implementation

				if (classifier1 instanceof SubprogramType)
				{
					sp_name = aadlColAPI.ComponentType_GetName((SubprogramType) classifier1);
					sp_process = asme2ssme_sp.ASME2SSME_SubprogramType((SubprogramType) classifier1, vmodule);
					myLib.STree_addaDeclarationToProcess(process_process, sp_process);
					l_classifier.add(classifier1);                      
				}
				else if (classifier1 instanceof SubprogramImplementation){
					sp_name = aadlColAPI.ComponentImpl_GetName((SubprogramImplementation) classifier1);
					sp_process = asme2ssme_sp.ASME2SSME_SubprogramImpl((SubprogramImplementation) classifier1, vmodule);
					myLib.STree_addaDeclarationToProcess(process_process, sp_process);
					l_classifier.add(classifier1);
				}
				else{
					java.lang.System.err.print("\n subprogram classifier illegal ");
					throw new IllegalArgumentException("subprogram classifier illegal!!  \n");
				}

				//create a instance of subprogram, and add local shared signals
				asme2ssme_sp.ASME2SSME_Create_SharedSubprogram(sp_name, d1.getName(), process_process, sp_process);   

				//specifies the clock of calling subprogram: l_x ^= t1_call ^+ t2_call
				asme2ssme_sp.ASME2SSME_SharedSubprogram_Call_Sync_x(sp_name, process_process, p);   

				//specifies the clock of calling subprogram: l_y ^= l_id ^= l_x
				asme2ssme_sp.ASME2SSME_SharedSubprogram_Sync_y(sp_name, process_process, p); 
			}
		}

	}


	/** Instantiate the subprograms that are called/required by the sub threads of the process implementation, 
	 * if they are not sub components of any other processes or threads (case 2)
	 * <p>
	 * For each sub thread t: 
	 * { for each called/required subprogram sp (a subprogram type or implementation):
	 *      { if(sp is not a sub component of any process 
	 *           && sp is not a sub component of any thread 
	 *           && sp has not been instantiated): {
	 *               1) creates a Signal process for this subprogram (ASME2SSME_SubprogramType() or ASME2SSME_SubprogramImpl()), 
	 *               2) add it to the declaration of process_process. 
	 *               3) add sp into the exist list
	 *               4) Adds a process instance in the body: ASME2SSME_Create_CalledSubprogram(),
	 *               5) specify the call clock of this subprogram: ASME2SSME_CalledSubprogram_Call_Sync_x(), ASME2SSME_CalledSubprogram_Sync_y()
	 *               }
	 *      }}   
	 * 
	 * @param process_process a Signal process that represents a process implementation component
	 * @param p a process implementation
	 */
	private void ASME2SSME_ProcessImpl_Instantiate_Subprograms(ProcessModelImpl process_process, 
			ProcessImplementation p, ModuleImpl vmodule, AadlPackage aadlpkg){

		boolean b;
		ProcessModelImpl sp_process=null;
		String sp_name;
		
		ASME2SSME_Translate_Subprogram asme2ssme_sp = new ASME2SSME_Translate_Subprogram();

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_ProcessImpl_Instantiate_Subprograms ");

		ArrayList<SubprogramSubcomponent> sub_subprograms = aadlColAPI.Process_GetSubcomponents_Subprogram(p);
		// Subprograms
		ArrayList sp_exist = new ArrayList();		
		for(int k=0; k< sub_subprograms.size(); k++){
			SubprogramSubcomponent sub1 = sub_subprograms.get(k);
			SubprogramClassifier spc1 = aadlColAPI.Subprogramsubcomponent_GetClassifier(sub1);
			if(spc1 instanceof SubprogramType) 
				sp_exist.add((SubprogramType) spc1);
			else if(spc1 instanceof SubprogramImplementation) 
				sp_exist.add((SubprogramImplementation) spc1);
		}

		ArrayList<ThreadSubcomponent> subthreads = aadlColAPI.Process_GetSubcomponentsThread(p);
		for(int i =0; i< subthreads.size(); i++){//for each sub thread
			ThreadSubcomponent subthread1 = subthreads.get(i);
			ThreadImplementation ti = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(subthread1);
			b = false;
			//called subprograms------------------------------------------------------
			ArrayList<CalledSubprogram> list1 = aadlColAPI.Thread_Get_CalledSubprograms(ti);			
			for(int j=0; j< list1.size(); j++){//for each called subprogram
				CalledSubprogram called_sp = list1.get(j);

				if (called_sp instanceof SubprogramType){ //subprogram type
					SubprogramType called_spt = (SubprogramType) called_sp;
					sp_name = aadlColAPI.ComponentType_GetName(called_spt);  
					if((! aadlColAPI.Is_SubprogramType_Sub_Of_Any_Process(called_spt, aadlpkg)) //if spt is not a sub component of any process
							&& (! aadlColAPI.Is_SubprogramType_Sub_Of_Any_Thread(called_spt, aadlpkg)) // if spt not sub of any thread,
							&& (! sp_exist.contains(called_spt))){  // if spt not exist in sp_exist list

						//create process, add declaration
						sp_process = asme2ssme_sp.ASME2SSME_SubprogramType(called_spt, vmodule);
						myLib.STree_addaDeclarationToProcess(process_process, sp_process);	 
						//add to exist list
						sp_exist.add(called_spt);        
						b = true;
					}
				}
				else {
					//subprogram implementation
					SubprogramImplementation called_spi = (SubprogramImplementation) called_sp;
					sp_name = aadlColAPI.ComponentImpl_GetName(called_spi);
					if((!aadlColAPI.Is_SubprogramImpl_Sub_Of_Any_Process(called_spi, aadlpkg)) //if spt is not a sub component of any process
							&& (!aadlColAPI.Is_SubprogramImpl_Sub_Of_Any_Thread(called_spi, aadlpkg)) // if sp not sub of any thread,
							&& (! sp_exist.contains(called_spi))){  // if spt not exist in sp_exist list

						//create process
						sp_process = asme2ssme_sp.ASME2SSME_SubprogramImpl(called_spi, vmodule);
						//add declaration
						myLib.STree_addaDeclarationToProcess(process_process, sp_process);	 
						//add to exist list
						sp_exist.add(called_spi);        
						b = true;
					}
				}
				if (b) {
					// //create a instance of subprogram, and add local shared signals
					// asme2ssme_sp.ASME2SSME_Create_CalledSubprogram(sp_name, process_process, sp_process);   
					// //specifies the clock of calling subprogram: xx_call := t1_call ^+ t2_call
					// asme2ssme_sp.ASME2SSME_CalledSubprogram_Call_Sync_x(sp_name, process_process, p);  
					// //specifies the clock of calling subprogram: l_y ^= l_id ^= l_x
					// asme2ssme_sp.ASME2SSME_CalledSubprogram_Sync_y(sp_name, process_process, p); 

					//create a instance of subprogram, and add local shared signals
					asme2ssme_sp.ASME2SSME_Create_SharedSubprogram(sp_name, null, process_process, sp_process);   
					//specifies the clock of calling subprogram: l_x ^= t1_call ^+ t2_call
					asme2ssme_sp.ASME2SSME_SharedSubprogram_Call_Sync_x(sp_name, process_process, p);   
					//specifies the clock of calling subprogram: l_y ^= l_id ^= l_x
					asme2ssme_sp.ASME2SSME_SharedSubprogram_Sync_y(sp_name, process_process, p); 
				}

			}

			//requires subprograms------------------------------------------------------
			ThreadType tt = aadlColAPI.Thread_GetCompType(ti);
			ArrayList<SubprogramAccess>  list2 = aadlColAPI.Thread_GetRequiresSubprogramAccess(tt);	
			for(int k=0; k< list2.size(); k++) { 
				//for each required subprogram
				SubprogramAccess spa1 = list2.get(k);
				SubprogramClassifier spc = aadlColAPI.SubprogramAccess_GetClassifier(spa1);
				b = false;
				if (spc instanceof SubprogramType){ //subprogram type
					SubprogramType spc1 = (SubprogramType) spc;
					sp_name = aadlColAPI.ComponentType_GetName(spc1);  

					if((!aadlColAPI.Is_SubprogramType_Sub_Of_Any_Process(spc1,aadlpkg)) //if spt is not a sub component of any process
							&& (!aadlColAPI.Is_SubprogramType_Sub_Of_Any_Thread(spc1, aadlpkg)) // if sp not sub of any thread,
							&& (! sp_exist.contains(spc1))){  // if spt not exist in sp_exist list

						//create process and add declaration
						sp_process = asme2ssme_sp.ASME2SSME_SubprogramType(spc1, vmodule);
						myLib.STree_addaDeclarationToProcess(process_process, sp_process);	 
						sp_exist.add(spc1);        
						b = true;		 
					}
				}
				else { //subprogram implementation
					SubprogramImplementation spc2 = (SubprogramImplementation) spc;
					sp_name = aadlColAPI.ComponentImpl_GetName(spc2);

					if((!aadlColAPI.Is_SubprogramImpl_Sub_Of_Any_Process(spc2,aadlpkg)) //if spt is not a sub component of any process
							&& (!aadlColAPI.Is_SubprogramImpl_Sub_Of_Any_Thread(spc2, aadlpkg)) // if sp not sub of any thread,
							&& (! sp_exist.contains(spc2))){  // if spt not exist in sp_exist list

						// create process and add declaration
						sp_process = asme2ssme_sp.ASME2SSME_SubprogramImpl(spc2, vmodule);
						myLib.STree_addaDeclarationToProcess(process_process, sp_process);	 
						sp_exist.add(spc2);        
						b = true;
					}
				}
				if (b)
				{
					// //create a instance of subprogram, and add local shared signals
					// asme2ssme_sp.ASME2SSME_Create_CalledSubprogram(sp_name2, process_process, sp_process2);   
					// //specifies the clock of calling subprogram: xx_call := t1_call ^+ t2_call
					// asme2ssme_sp.ASME2SSME_CalledSubprogram_Call_Sync_x(sp_name2, process_process, p);  
					// //specifies the clock of calling subprogram: l_y ^= l_id ^= l_x
					// asme2ssme_sp.ASME2SSME_CalledSubprogram_Sync_y(sp_name2, process_process, p); 

					//create a instance of subprogram, and add local shared signals
					asme2ssme_sp.ASME2SSME_Create_SharedSubprogram(sp_name, null, process_process, sp_process);   
					//specifies the clock of calling subprogram: l_x ^= t1_call ^+ t2_call
					asme2ssme_sp.ASME2SSME_SharedSubprogram_Call_Sync_x(sp_name, process_process, p);   
					//specifies the clock of calling subprogram: l_y ^= l_id ^= l_x
					asme2ssme_sp.ASME2SSME_SharedSubprogram_Sync_y(sp_name, process_process, p); 
				}

			}
		} 
		} 


	/** give the clocks of subprograms that are called/required by the sub threads of the process implementation, 
	 * if they are sub components of sub threads (case 3)
	 * <p>
	 * For each sub thread t1: 
	 * { for each sub subcomponent sp1: subprogram sp: {
	 *          specify the call clock of this subprogram: 
	 *          l_x ^= e1 ^+ ...  (ASME2SSME_SubThread_SubSubprogram_Call_Sync_x())
	 *          l_y ^= l_id ^= l_x (ASME2SSME_SubThread_SubSubprogram_Sync_y())
	 *          //L1 := l_x (ASME2SSME_SubThread_SubSubprogram_Def_localx())
	 *          //l_y := L2 (ASME2SSME_SubThread_SubSubprogram_Def_localy())
	 *          //l_id := L3 (ASME2SSME_SubThread_SubSubprogram_Def_localid())
	 *          where "l" is short for sp,
	 *                l_x is local shared for sp_x
	 *                l_y, l_id are local for sp_y and sp_id respectively
	 *                L1,L2,L3 are local short for t1_sp_x, t1_sp_y, t1_sp_id
	 *      }} 
	 * 
	 * @param process_process a Signal process that represents a process implementation component
	 * @param p a process implementation
	 * @param vmodule the top Signal module that represents the package
	 * @param aadlpkg the root aadl package
	 * 
	 * @return
	 */
	private void ASME2SSME_ProcessImpl_SubThread_SubSubprograms_Clocks(ProcessModelImpl process_process, 
			ProcessImplementation p, ModuleImpl vmodule, AadlPackage aadlpkg){

		ASME2SSME_Translate_Subprogram asme2ssme_sp = new ASME2SSME_Translate_Subprogram();

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_ProcessImpl_SubThread_SubSubprograms_Clocks ");	

		ArrayList<ThreadSubcomponent> subthreads = aadlColAPI.Process_GetSubcomponentsThread(p);
		for(int i =0; i< subthreads.size(); i++){//for each sub thread
			ThreadSubcomponent subthread1 = subthreads.get(i);
			ThreadImplementation ti = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(subthread1);

			//sub subprograms------------------------------------------------------
			ArrayList<SubprogramSubcomponent> list1 = aadlColAPI.Thread_GetSubcomponent_Subprograms(ti);			
			for(int j=0; j< list1.size(); j++){//for each sub-subprogram
				SubprogramSubcomponent sub_sp = list1.get(j);
				SubprogramClassifier classifier1 =  (SubprogramClassifier) aadlColAPI.Subprogramsubcomponent_GetClassifier(sub_sp);
				String sp_name = aadlColAPI.Classifier_GetName(classifier1);
				// specifies the clock of calling subprogram: l_x ^= t1_call ^+ t2_call
				asme2ssme_sp.ASME2SSME_SubThread_SubSubprogram_Call_Sync_x(sp_name, process_process, p);   
				// specifies the clock of calling subprogram: l_y ^= l_id ^= l_x
				asme2ssme_sp.ASME2SSME_SubThread_SubSubprogram_Sync_y(sp_name, process_process, p); 
				// //L1 := l_x
				//	 asme2ssme_sp.ASME2SSME_SubThread_SubSubprogram_Def_localx(sp_name, process_process, p, subthread1);
				// //l_y := L2
				// asme2ssme_sp.ASME2SSME_SubThread_SubSubprogram_Def_localy(sp_name, process_process, p, subthread1);
				//	  //l_z := L3
				// asme2ssme_sp.ASME2SSME_SubThread_SubSubprogram_Def_localid(sp_name, process_process, p, subthread1);
			}
		} 
	}



	/**
	 * Translates thread group subcomponents of a AADL process
	 * 
	 * @param process_process
	 * @param list
	 * @param sys_process
	 * @param vmodule
	 */
	private void ASME2SSME_ProcessImpl_SubThreadGroups(ProcessModelImpl process_process, ArrayList<ThreadGroupSubcomponent> list){

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_ProcessImpl_SubThreadGroups ");
		if(list.size()>0) {
			ASME2SSME_Translate_ThreadGroup asme2ssme_tg = new ASME2SSME_Translate_ThreadGroup();
			for(int i = 0; i < list.size(); i++){
				ThreadGroupSubcomponent t1 = list.get(i);
				ThreadGroupClassifier tgc = (ThreadGroupClassifier) aadlColAPI.ThreadGroupsubcomponent_GetClassifier(t1);
				if(tgc instanceof ThreadGroupType) 
					asme2ssme_tg.ASME2SSME_ThreadGroupType((ThreadGroupType) tgc);
				else 
					asme2ssme_tg.ASME2SSME_ThreadGroupImpl((ThreadGroupImplementation) tgc);
			}
		}
	}



	/**
	 * Translate the connections of a process implementation component
	 * 
	 * @param pimpl
	 * @param vprocess
	 */
	private void ASME2SSME_ProcessImpl_Connections(ProcessImplementation pimpl, ProcessModelImpl vprocess){

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_ProcessImpl_Connections: " + pimpl.getName());
		ArrayList<Connection> connections = aadlColAPI.ComponentImpl_GetConnections(pimpl);
		if (connections != null){
			ASME2SSME_Translate_Connection asme2ssme_connection = new ASME2SSME_Translate_Connection();
			ASME2SSME_Translate_PortConnection asme2ssme_pconnection = new ASME2SSME_Translate_PortConnection();
			// DataConnections 
			ArrayList<PortConnection> listDataConnections = aadlColAPI.ComponentImpl_GetDataPortConnections(pimpl);
			if(listDataConnections.size()>0){
				for(int i=0; i<listDataConnections.size(); i++)
				{
					PortConnection dataConnection = listDataConnections.get(i);
					if(asme2ssme_pconnection.ASME2SSME_DataPortConnection(pimpl, vprocess, dataConnection) != null){
						SignalsDefinitionImpl eq1 = asme2ssme_pconnection.ASME2SSME_DataPortConnection(pimpl, vprocess, dataConnection);
						myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
					}
				}
			}

			// EventConnections 
			ArrayList<PortConnection> listEventConnections = aadlColAPI.ComponentImpl_GetEventPortConnections(pimpl);
			if(listEventConnections.size()>0){
				for(int j=0; j<listEventConnections.size(); j++)
				{
					PortConnection c1 = listEventConnections.get(j);
					if(asme2ssme_pconnection.ASME2SSME_EventPortConnection(pimpl, vprocess, c1) != null) {
						SignalsDefinitionImpl eq1 = asme2ssme_pconnection.ASME2SSME_EventPortConnection(pimpl, vprocess, c1);
						myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
					}	
				}
			}

			// EventDataConnections 
			ArrayList<PortConnection> listEventDataConnections = aadlColAPI.ComponentImpl_GetEventDataPortConnections(pimpl);
			if(listEventDataConnections.size()>0){
				for(int k=0; k<listEventDataConnections.size(); k++)
				{
					PortConnection c2 = listEventDataConnections.get(k);
					if(asme2ssme_pconnection.ASME2SSME_EventDataPortConnection(pimpl, vprocess, c2) !=null){
						SignalsDefinitionImpl eq2 = asme2ssme_pconnection.ASME2SSME_EventDataPortConnection(pimpl, vprocess, c2);
						myLib.STree_addExpressionOfProcessToaModel(vprocess, eq2);
					}			    
				}
			}

			// access connections
			if(aadlColAPI.ComponentImpl_GetAccessConnections(pimpl) != null) {  		
				//bus access connection
				ArrayList<AccessConnection> list1 = aadlColAPI.ComponentImpl_GetBusAccessConnections(pimpl);
				if(list1.size()>0)
					for(int i1 =0; i1 < list1.size(); i1++ ) 
						asme2ssme_connection.ASME2SSME_BusAccessConnection(list1.get(i1));

				//data access connection
				ArrayList<AccessConnection> list2 = aadlColAPI.ComponentImpl_GetDataAccessConnections(pimpl);
				if(list2.size()>0) 
					for(int i2 =0; i2 < list2.size(); i2++ ) 
						asme2ssme_connection.ASME2SSME_DataAccessConnection(list2.get(i2));
				
				//subprogram access connection

				//subprogram group access connection
			}

			//feature group connections
			ArrayList<FeatureGroupConnection> list3 = aadlColAPI.ComponentImpl_GetFeatureGroupConnections(pimpl);
			ASME2SSME_Translate_FeatureGroupConnection asme2ssme_fgc = new ASME2SSME_Translate_FeatureGroupConnection();
			for(int i3= 0; i3< list3.size(); i3++){
				FeatureGroupConnection c3 = list3.get(i3);
				if( asme2ssme_fgc.ASME2SSME_FeatureGroupConnection1(c3) != null) {
					SignalsDefinitionImpl eq3 = asme2ssme_fgc.ASME2SSME_FeatureGroupConnection1(c3);
					myLib.STree_addExpressionOfProcessToaModel(vprocess, eq3);
				}
				if( asme2ssme_fgc.ASME2SSME_FeatureGroupConnection2(c3) != null) {
					SignalsDefinitionImpl eq4 = asme2ssme_fgc.ASME2SSME_FeatureGroupConnection2(c3);
					myLib.STree_addExpressionOfProcessToaModel(vprocess, eq4);
				}
			}
		}
	}



	/**
	 * Translates flows of a process implementation
	 * 
	 * @param p a process implementation component
	 */
	private void ASME2SSME_ProcessImpl_Flows(ProcessModelImpl vprocess,ProcessImplementation p){

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_ProcessImpl_Flows: "+ p.getName());
		asme2ssme_common.ASME2SSME_ComponentImpl_Flows(p);
		if(asme2ssme_common.ASME2SSME_ComponentImpl_Flows(p) != null) {
			ProcessModelImpl p1= asme2ssme_common.ASME2SSME_ComponentImpl_Flows(p);
			myLib.STree_addProcessDeclaration(vprocess, p1);
			ArrayList<ExprParam> lparam = new ArrayList<ExprParam> ();
			String pname = Name_of_Flow(p.getName().replace('.','_'));
			ProcessExpansionImpl call1 = myLib.STree_mkProcessCallWithParameters(myLib.mkIdentifier(pname), lparam);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, call1);
		}
	}


	/**
	 * Translate modes of a process implementation component
	 * 
	 * @param pimpl
	 */
	private void ASME2SSME_ProcessImpl_Modes(ProcessImplementation pimpl){
		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_ProcessImpl_Modes: "+ pimpl.getName());
		asme2ssme_common.ASME2SSME_ComponentImpl_Modes(pimpl);
	}


	/**
	 * Add definitions of out events 
	 * (which indicate the arrival of in event/event data port of a non-periodic thread) 
	 * to the body of process (vprocess): 
	 * t_i_TE := when xxx
	 * 
	 * @param vprocess A Signal process that represents an AADL process implementation
	 * @param pimpl A process implementation
	 */
	private void ASME2SSME_ProcessImpl_OutEvent_Definition(ProcessModelImpl vprocess, ProcessImplementation pimpl){

		Trace("\n ========== ASME2SSME_Translate_Process: ASME2SSME_ProcessImpl_OutEvent_Definition " + pimpl.getName());

		ArrayList<ThreadSubcomponent> list1 = aadlColAPI.Process_GetSubcomponentsThread((ProcessImplementation) pimpl);
		if(list1.size()>0){
			//for each sub thread:
			for(int i =0; i< list1.size(); i++){
				ThreadSubcomponent t1 = list1.get(i);
				ThreadImplementation tt = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(t1);
				ThreadType tt1= (ThreadType) aadlColAPI.ComponentImpl_GetComponentType(tt);

				String dispatch = aadlColAPI.Thread_Get_DispatchProtocol_str(tt);
				Trace(t1.getName() + " " + dispatch);
				//if the thread is not periodic, adds definition of t_i_TE in the body
				if(dispatch.equals("Periodic") == false){

					ArrayList<EventPort> list2 = aadlColAPI.Component_GetInEventPort(tt1);
					ArrayList<EventDataPort> list3 = aadlColAPI.Component_GetInEventDataPort(tt1);

					//for each in event port i of sub thread t: add  a definition t_i_TE := when xx (where xx is a local signal that represents t_i) in the body
					if(list2.size()>0){
						for(int j=0; j<list2.size(); j++){
							EventPort ep1 = list2.get(j);
							String left_name =  Name_of_TriggerDispatch_Event(t1.getName() + "_" + ep1.getName());
							ExprSignal left_expr = myLib.mkIdentifier(left_name);
							String right_name = getSname(t1.getName() + "_" + ep1.getName());
							//ExprSignal right_expr = myLib.mkIdentifier(right_name);
							UnaryWhenImpl right_expr = myLib.STree_mkUnaryWhen(myLib.mkIdentifier(right_name));
							SignalsDefinitionImpl eq1 = (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
						}
					}
					else{
						Trace("no in event port of sub thread " + t1.getName());
					}

					//for each in event data port i of sub thread t: add  a definition t_i_TE := when ^xx (where xx is a local signal that represents t_i) in the body
					if(list3.size()>0){
						for(int k=0; k<list3.size(); k++){
							EventDataPort edp1 = list3.get(k);
							String left_name =  Name_of_TriggerDispatch_Event(t1.getName() + "_" + edp1.getName());
							ExprSignal left_expr = myLib.mkIdentifier(left_name);
							String right_name = getSname(t1.getName() + "_" + edp1.getName());
							ExprSignal exp1 = myLib.mkIdentifier(right_name);
							ClockImpl clk = new ClockImpl();
							clk.setExprSignal(exp1);
							//when ^xx
							UnaryWhenImpl right_expr = myLib.STree_mkUnaryWhen(clk);
							SignalsDefinitionImpl eq1 = (SignalsDefinitionImpl) myLib.mkEquation( left_expr, right_expr);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
						}
					}
					else{
						Trace("no in event data port of sub thread " + t1.getName());
					}
				}			    
			}
		}

	}



	/**
	 * for each data access from a data subcomponent dd to a thread requires data access x, 
	 * in the body of thread, adds connection of local variable with the shared data: 
	 * dd_w ::= x_w, x_r := dd_r
	 * 
	 * @param pimpl
	 * @param vprocess
	 */
	private void ASME2SSME_ProcessImpl_DataAccessConnection(ProcessImplementation pimpl, ProcessModelImpl vprocess){
		// --NOT USED--
		Trace("\n ========== ASME2SSME_Translate_Process: ASME2SSME_ProcessImpl_DataAccessConnection " + pimpl.getName());
		EList<AccessConnection> connections = aadlColAPI.ComponentImpl_GetAccessConnections(pimpl);

		if (connections != null) {
			ArrayList<AccessConnection> list = aadlColAPI.ComponentImpl_GetDataAccessConnections(pimpl);
			if(list.size()>0) {
				//for each data access connection
				for(int m =0; m < list.size(); m++ ){
					AccessConnection c2 = list.get(m);

					//for each data access: sub data component -> 
					if(aadlColAPI.AccessConnection_GetSrc(c2) instanceof DataSubcomponent){
						DataSubcomponent dd = (DataSubcomponent) aadlColAPI.AccessConnection_GetSrc(c2);

						if(aadlColAPI.AccessConnection_GetSrcContext(c2) instanceof ThreadSubcomponent){ 

							//for each data access: sub data component -> sub thread

							ThreadSubcomponent tt= (ThreadSubcomponent) aadlColAPI.AccessConnection_GetSrcContext(c2);
							ThreadImplementation timpl = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(tt);
							String name1 = aadlColAPI.ComponentImpl_GetName(timpl);
							String t_name = Name_of_Thread(name1);
							ProcessModelImpl tprocess = myLib.STree_GetProcessDeclarationFromProcess(vprocess, t_name);

							//for each data access: sub data component -> sub thread.requires data access
							if(aadlColAPI.AccessConnection_GetDst(c2) instanceof DataAccess){

								DataAccess xx = (DataAccess) aadlColAPI.AccessConnection_GetDst(c2);

								//creates an equation: dd_w ::= xx_w
								ExprSignal leftExp0 = myLib.mkIdentifier(dd.getName()+"_w");
								ExprSignal rightExp0 = myLib.mkIdentifier(xx.getName()+"_w");
								SignalsPartialDefinitionImpl eq0 = (SignalsPartialDefinitionImpl) myLib.mkPartialEquation(leftExp0, rightExp0);

								//creates an equation: xx_r := dd_r
								ExprSignal leftExp1 = myLib.mkIdentifier(xx.getName()+"_r");
								ExprSignal rightExp1 = myLib.mkIdentifier(dd.getName()+"_r");
								SignalsDefinitionImpl eq1 = (SignalsDefinitionImpl) myLib.mkEquation( leftExp1, rightExp1);

								//adds in the thread body					
								myLib.STree_addExpressionOfProcessToaModel(tprocess, eq0);
								myLib.STree_addExpressionOfProcessToaModel(tprocess, eq1);
							}						
						}
					}
				}
			}
		}

	}




	/**
	 * Adds an instance of Signal process that represents an AADL process 
	 * into the body of a Signal process that represents a processor.
	 * <p>
	 * Gets the model type id from formal process (fprocess), 
	 * and finds out its process type declaration from the upper system level process (sysimpl_process) by a method STree_GetProcessTypeDeclarationFromAProcess(). 
	 * Makes a process call of this process type. 
	 * The input/output control signals (xx_ctl1, xx_ctl2, xx_time1, xx_Alarm, xx_TE) are taken as effective parameters of the call, 
	 * for the  other input/output signals, a local signal l_xx is used as a an effective parameter. 
	 * (in the processor Signal process body, 
	 * an equation will be add if the input (output) is passed from the processor interface: 
	 * l_xx := xx (for an input) or xx := l_xx (for an output), in process ASME2SSME_Processor_InOut_Info() ) 
	 * This process call is added into the body of the processor Signal process (vprocess).
	 * Local signal declarations are added.
	 * 
	 * @param fprocess a Signal formal process which is a parameter of the interface of vprocess
	 * @param vprocess a Signal process that represents a processor component
	 * @param process_sub a process subcomponent
	 * @param sysimpl_process a Signal process that represents a system implementation component
	 */
	public void ASME2SSME_ProcessType_Instance(FormalProcessImpl fprocess, ProcessModelImpl vprocess, ProcessSubcomponent process_sub,  ProcessModelImpl sysimpl_process){

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_ProcessType_Instance: "+ process_sub.getName());

		String label = aadlColAPI.Subcomponent_GetName(process_sub);
		Identifier model_type_id = fprocess.getModelType();
		Identifier model_name_id = fprocess.getModelName();
		String model_name = myLib.Identifier_GetStr(model_name_id); //instance name
		String process_type_name = myLib.Identifier_GetStr(model_type_id);
		ProcessTypeImpl process_type = myLib.STree_GetProcessTypeDeclarationFromAProcess(sysimpl_process, process_type_name);

		ArrayList<ExprParam> l_param = new ArrayList<ExprParam>();//parameter (id) list for call the process pprocess
		ArrayList<ExprSignal> l_in = new ArrayList<ExprSignal>();//input (id) list for call the process pprocess
		ArrayList<ExprSignal> l_out = new ArrayList<ExprSignal>();//output (id) list for call the process pprocess
		ArrayList<Declaration> vdecl = new ArrayList<Declaration>();//local lists

		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) process_type.getInterfaceDefinition();
		if (inter != null){

			// EList<TypesOrSignalsDeclaration> l_parameters = inter.getListFormalParameter();        
			IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
			if (vioProfile != null) 
			{ 
				EList<TypesOrSignalsDeclaration> l_inputs = vioProfile.getInputs(); 
				Iterator<TypesOrSignalsDeclaration> it_i = l_inputs.iterator();
				while (it_i.hasNext())
				{
					SignalsDeclarationImpl ainput = (SignalsDeclarationImpl) it_i.next(); 
					EList<SignalNaming> list2 = ainput.getListSignal();

					if (ainput.getType() instanceof PredefinedTypeImpl){
						//Trace("ainput.getType() instanceof PredefinedTypeImpl");
						// PredefinedTypeImpl type_2 = (PredefinedTypeImpl) ainput.getType();

						for (int i = 0; i < list2.size(); i++) {  
							SignalNaming sig_name2 = list2.get(i);
							Identifier id_sig_name2 = sig_name2.getIdentifier();
							String name2 = label + '_' + id_sig_name2.getAtomString();			    		   
							String lname2 = getSname("l_"+name2);

							//local name as l_xx			    	
							// SignalsDeclarationImpl sig2 = new SignalsDeclarationImpl();
							//  if(((PredefinedTypeImpl)ainput.getType()).getType() == SignalIntrinsicType.BOOLEAN_LITERAL){
							//  sig2 = myLib.mkBooleanDecl (lname2);  else  sig2 =myLib.mkEventDecl(lname2)
							PredefinedTypeImpl ptype2 = myLib.STree_mkSamePredefinedType(ainput);
							SignalsDeclarationImpl sig2 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(lname2),ptype2, null);
							if (name2.endsWith(DISPATCH_ID) | name2.endsWith(RESUME_ID) | name2.endsWith(DEADLINE_ID) 
									| name2.endsWith("_Frozen_time_event") | name2.endsWith("_Output_time_event")) {
								l_in.add(myLib.mkIdentifier(name2));
							}
							else if (id_sig_name2.getAtomString().equals(TOP_ID)) 
								l_in.add(myLib.mkIdentifier(TOP_ID));
							else {
								l_in.add(myLib.mkIdentifier(lname2));
								// if (ASME2SSME_Process_Interface_Contain_Signal(vprocess, name2) == false){
								vdecl.add(sig2);
								//}			    			   
							}			    				    		   
						}
					}
					else{
						Identifier type_id = (Identifier) ainput.getType();
						String type_name = type_id.getAtomString();
						IdentifierImpl type2 = myLib.mkIdentifier(type_name);

						for (int i = 0; i < list2.size(); i++) {  
							SignalNaming sig_name2 = list2.get(i);
							Identifier id_sig_name2 = sig_name2.getIdentifier();
							String name2 = label + '_' + id_sig_name2.getAtomString();
							if (name2.endsWith(CTL1_ID) | name2.endsWith(TIME1_ID)) 
								l_in.add(myLib.mkIdentifier(name2));
							else {
								//local name as l_xx
								String lname2 = getSname("l_"+name2);
								SignalsDeclarationImpl sig2 = myLib.mkDecl( lname2, type2); 
								l_in.add(myLib.mkIdentifier(lname2 ));
								vdecl.add(sig2);
							}
						}
					}		    	   
				}

				EList<TypesOrSignalsDeclaration> l_outputs = vioProfile.getOutputs(); 
				Iterator<TypesOrSignalsDeclaration> it_o = l_outputs.iterator();
				while (it_o.hasNext())
				{
					SignalsDeclarationImpl aoutput = (SignalsDeclarationImpl) it_o.next(); 
					EList<SignalNaming> list3 = aoutput.getListSignal();

					if (aoutput.getType() instanceof PredefinedTypeImpl){
						// Trace("aoutput.getType() instanceof PredefinedTypeImpl");
						// PredefinedTypeImpl type_3 = (PredefinedTypeImpl) aoutput.getType();
						for (int i = 0; i < list3.size(); i++) {  
							SignalNaming sig_name3 = list3.get(i);
							Identifier id_sig_name3 = sig_name3.getIdentifier();
							String name3 = label + '_' + id_sig_name3.getAtomString();
							//local name as l_xx
							String lname3 = getSname("l_"+name3);			    		  
							PredefinedTypeImpl ptype3 = myLib.STree_mkSamePredefinedType(aoutput);
							SignalsDeclarationImpl sig3 = myLib.mkDecl(lname3, ptype3);			

							if (name3.endsWith("_Complete") | name3.endsWith("_Error") 
									| name3.endsWith("_Alarm") | id_sig_name3.getAtomString().endsWith("_TE")){
								l_out.add(myLib.mkIdentifier(name3));
							}
							else {
								l_out.add(myLib.mkIdentifier(lname3));
								vdecl.add(sig3);
							}
						}
					}
					else{
						Identifier type_id = (Identifier) aoutput.getType();
						String type_name = type_id.getAtomString();
						IdentifierImpl type3 = myLib.mkIdentifier(type_name);
						Trace(type3.getAtomString());

						for (int i = 0; i < list3.size(); i++) {  
							SignalNaming sig_name3 = list3.get(i);
							Identifier id_sig_name3 = sig_name3.getIdentifier();
							String name3 = label + '_' + id_sig_name3.getAtomString();
							//local name as l_xx		    		   		    		   
							if (name3.endsWith("ctl2")) 
								l_out.add(myLib.mkIdentifier(name3));
							else {
								String lname3 = getSname("l_"+name3);
								SignalsDeclarationImpl sig3 = myLib.mkDecl( lname3 , type3 );
								l_out.add(myLib.mkIdentifier(lname3));
								vdecl.add(sig3);
							}
						}
					}
				}
			}
		}

		if (l_out.size()>0){
			SignalsDefinitionImpl call_eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(model_name), l_param, l_in, l_out);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, call_eq);
			myLib.STree_addComment(call_eq, "process instance of AADL process " + model_name, false);
		}
		else{
			ProcessCallImpl pcall = myLib.STree_mkCompleteProcessCall(myLib.mkIdentifier(model_name), l_param, l_in);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, pcall);
			myLib.STree_addComment(pcall, "process instance of AADL process " + model_name, false);
		}
		myLib.STree_addLocalDeclaration(vprocess, vdecl);
		Trace("\n ========================= end ASME2SSME_ProcessType_Instance: "+ process_sub.getName() + " ========================= \n");
	}


	/**
	 * A thread t is represented as a Singla process depending on its type
	 * 
	 * @param t
	 * @param vmodule
	 * @return
	 */
	public ProcessModelImpl ASME2SSME_ProcessImpl_Thread(ThreadImplementation t, ModuleImpl vmodule){	

		ProcessModelImpl vprocess = new ProcessModelImpl();

		Trace("\n ====== ASME2SSME_Translate_Process: ASME2SSME_ProcessImpl_Thread: "+ t.getName());

		if (aadlColAPI.Thread_Get_DispatchProtocol_str(t) == null) {
			java.lang.System.err.println("\n Thread " + t.getName() + " Dispatch_Protocol property not specified !! ");
			throw new IllegalArgumentException(t.getName() + " Dispatch_Protocol property not specified !!  \n");
		}
		String dispatchp = aadlColAPI.Thread_Get_DispatchProtocol_str(t);
		if(dispatchp.equals("Periodic")){
			ASME2SSME_Translate_PeriodicThread asme2ssme_pthread = new ASME2SSME_Translate_PeriodicThread();
			vprocess = asme2ssme_pthread.ASME2SSME_ThreadImpl(t, vmodule);
		}
		else if(dispatchp.equals("Aperiodic")){
			ASME2SSME_Translate_AperiodicThread asme2ssme_athread = new ASME2SSME_Translate_AperiodicThread();
			vprocess = asme2ssme_athread.ASME2SSME_ThreadImpl(t, vmodule);
		}
		else if(dispatchp.equals("Sporadic")){
			ASME2SSME_Translate_SporadicThread asme2ssme_sthread = new ASME2SSME_Translate_SporadicThread();
			vprocess = asme2ssme_sthread.ASME2SSME_ThreadImpl(t, vmodule);
		}
		else if(dispatchp.equals("Timed")){
			ASME2SSME_Translate_TimedThread asme2ssme_tthread = new ASME2SSME_Translate_TimedThread();
			vprocess = asme2ssme_tthread.ASME2SSME_ThreadImpl(t, vmodule);
		}
		else if(dispatchp.equals("Hybrid")){
			ASME2SSME_Translate_HybridThread asme2ssme_hthread = new ASME2SSME_Translate_HybridThread();
			vprocess = asme2ssme_hthread.ASME2SSME_ThreadImpl(t, vmodule);
		}
		else if(dispatchp.equals("Background")){//Background
			ASME2SSME_Translate_BackgroundThread asme2ssme_bthread = new ASME2SSME_Translate_BackgroundThread();
			vprocess = asme2ssme_bthread.ASME2SSME_ThreadImpl(t, vmodule);
		}
		else{
			java.lang.System.err.println("\n Thread " + t.getName() + " Dispatch_Protocol property illegal !! ");
			throw new IllegalArgumentException(t.getName() + " Dispatch_Protocol property illegal !! \n");
		}
		return vprocess;
	}

}
