 
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.osate.aadl2.*;
import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;
 
public class ASME2SSME_Translate_Processor extends ASME2SSME_Translate_Property{	
	private signalTreeAPI myLib;
	private AADLCoLAPI aadlColAPI;
	private ASME2SSME_Translate_Common asme2ssme_common;
	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_Processor() {
		  myLib= new signalTreeAPI();
		  aadlColAPI = new AADLCoLAPI();
		  asme2ssme_common = new ASME2SSME_Translate_Common();
	}
	
    /**
     * Translates AADL processor type
     * 
     * @param pt
     */
    public void ASME2SSME_ProcessorType(ProcessorType pt){
		    Trace("Processor type not yet implemented");
    }
	
	
    /** 
     * Returns a Signal process that represents a processor implementation component. 
     * <p>
     * This Signal process is named as "xx_processor", where "xx" is the name of the processor subcomponent. 
     * This is implemented by the Name_of_Processor() method. 
     * From the processor implementation, gets its processor type, 
     * and translates it as the interface of this Signal process (ASME2SSME_ComponentInterface() method). 
     * Two methods (ASME2SSME_Processor_Interface() and ASME2SSME_Processor_Interface_Refine()) are used to refine this interface. 
     * <p>
     * The processes bound to this processor are represented as process instances (ASME2SSME_Processor_ProcessType_instances()). 
     * <p>
     * Processor behavior and property sub processes are declared and added as process calls in the body 
     * (ASME2SSME_Processor_behavior(), ASME2SSME_Processor_Behavior_Instance(), ASME2SSME_Processor_property()). 
     * <p>
     * The connections between these processes are added by ASME2SSME_DataPortConnection() method.
     * 
     * @param p a processor implementation component
     * @param subprocessor a processor subcomponent whose classifier is p
     * @param sysimpl a system implementation component which is the upper level of this processor subcomponent
     * @param sysimpl_process a Signal process which represents this system implementation component
     * @return a ProcessModelImpl (Signal process)	
     */
  public ProcessModelImpl ASME2SSME_ProcessorImpl(ProcessorImplementation p, ProcessorSubcomponent subprocessor, 
		                                        SystemImplementation sysimpl, ProcessModelImpl sysimpl_process){

      Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_ProcessorImpl : " + p.getName());
      
      String subprocessor_name = aadlColAPI.Subcomponent_GetName(subprocessor);
      String pname = Name_of_Processor(subprocessor_name);

      // processor process skeleton
      ProcessModelImpl pprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );

      // processor process interface ===================================
      ProcessorType processor_type = (ProcessorType) aadlColAPI.ComponentImpl_GetComponentType(p);
      
      //refer to ProcessorType
      ASME2SSME_ProcessorType(processor_type);
     
      asme2ssme_common.ASME2SSME_ComponentInterface(pprocess, processor_type);   
      ASME2SSME_Processor_Interface(pprocess, subprocessor, sysimpl, sysimpl_process);
      //interface refinement
      ASME2SSME_Processor_Interface_Refine(pprocess, subprocessor, sysimpl, sysimpl_process);
      
      //processes instance
      ASME2SSME_Processor_ProcessType_instances(pprocess, sysimpl_process, subprocessor, sysimpl);
       
      //processor behavior ====================================================
  	ProcessModelImpl processor_behavior = ASME2SSME_Processor_Behavior(pprocess, subprocessor, sysimpl, sysimpl_process);
  
  	//add to declaration
  	myLib.STree_addProcessDeclaration(pprocess, processor_behavior);
  	//add a processor_behavior instance (process call) to the body  	 
  	ASME2SSME_Processor_Behavior_Instance(pprocess, processor_behavior);
  	
  	//processor property =========================================================
  	ProcessModelImpl processor_property = ASME2SSME_Processor_property(subprocessor);
  	String processor_property_name = myLib.STree_GetStringName(myLib.Process_GetID(processor_property));//pname + "_property";
  	//add to declaration
  	myLib.STree_addProcessDeclaration(pprocess, processor_property);
  	//add to body
  	ArrayList<ExprParam> lparam = new ArrayList<ExprParam> ();
  	ProcessExpansionImpl processor_property_inst = myLib.STree_mkProcessCallWithParameters(myLib.mkIdentifier(processor_property_name), lparam);
      myLib.STree_addExpressionOfProcessToaModel(pprocess, processor_property_inst);
      
      //add comments for property sub process
  	ArrayList<String> comments = new ArrayList<String>();
  	comments.add("processor property sub process");
  	myLib.STree_addComment(processor_property_inst, comments, false);
  	
  	//connections between processes
      // data port connection========================================================================
      ArrayList<PortConnection> list_dataconnection = aadlColAPI.Get_ProcessDataConnection_SameProcessor(sysimpl, subprocessor);
  	
      if(list_dataconnection.size()>0){
      	ASME2SSME_Translate_PortConnection asme2ssme_pconnection = new ASME2SSME_Translate_PortConnection();
      	
      	for(int m1=0; m1<list_dataconnection.size(); m1++)
      	{
      		PortConnection dataConnection = list_dataconnection.get(m1);				
      		SignalsDefinitionImpl eq1 = asme2ssme_pconnection.ASME2SSME_DataPortConnection(sysimpl,pprocess, dataConnection);
      		myLib.STree_addExpressionOfProcessToaModel(pprocess, eq1);
      	}
      }
      
   // event port connection========================================================================
      ArrayList<PortConnection> elist1 = aadlColAPI.Get_ProcessEventConnection_SameProcessor(sysimpl, subprocessor);
  	
      if(elist1.size()>0){
      	ASME2SSME_Translate_PortConnection asme2ssme_pconnection = new ASME2SSME_Translate_PortConnection();
      	
      	for(int n1=0; n1<elist1.size(); n1++)
      	{
      		PortConnection ec1 = elist1.get(n1);				
      		SignalsDefinitionImpl eq2 = asme2ssme_pconnection.ASME2SSME_EventPortConnection(sysimpl,pprocess, ec1);
      		myLib.STree_addExpressionOfProcessToaModel(pprocess, eq2);
      	}
      }
      
   // event data port connection========================================================================
      ArrayList<PortConnection> edlist1 = aadlColAPI.Get_ProcessEventDataConnection_SameProcessor(sysimpl, subprocessor);
  	
      if(edlist1.size()>0){
      	ASME2SSME_Translate_PortConnection asme2ssme_pconnection = new ASME2SSME_Translate_PortConnection();
      	
      	for(int k1=0; k1<edlist1.size(); k1++)
      	{
      		PortConnection ec1 = edlist1.get(k1);				
      		SignalsDefinitionImpl eq3 = asme2ssme_pconnection.ASME2SSME_EventDataPortConnection(sysimpl,pprocess, ec1);
      		myLib.STree_addExpressionOfProcessToaModel(pprocess, eq3);
      	}
      }
          
  	//explicitly defines the input/output: xx := l_xx
  	ASME2SSME_Processor_InOut_Info(pprocess);
  	
  	//Subcomponents========================================================
  	ASME2SSME_ProcessorImpl_Subs(p, pprocess);
  	
  	//Connections===========================================================
  	//connections not exist in version 1.5.8
  	
  	//Flows================================================================
  	ASME2SSME_ProcessorImpl_Flows(pprocess, p);
  	
  	//Modes================================================================
  	ASME2SSME_ProcessorImpl_Modes(p);
  	
  	//Properties==========================================================
  	//properties implemented in ProcessorImpl_Property() sub process
  	
  	
  	 Trace("\n ========================= end ASME2SSME_ProcessorImpl : " + p.getName() + " ========================= \n");
  	 
      return pprocess;
  	}


  /** 
   * Adds inputs/outputs to the interface of a processor.
   * <p>
   * Gets the processes that bound to this processor subcomponent (subprocessor) by method Processor_Get_Binding_Process(). 
   * For each process p1, this interface should not include the inputs: 
   * top, p1_xx, where xx is an output of p1, and it ends with "_ctl2", "_Alarm", "_TE"
   * and the outputs: 
   * p1_yy, where yy is an input of p1, and it ends with "_ctl1", "_time1"
   * 
   * @param vprocess a Signal process that represents this processor component
   * @param subprocessor a processor subcomponent
   * @param sysimpl a system implementation component which is the uppler level component that contains this processor subcomponent (subprocessor)
   * @param sysimpl_process a Signal process that represents this system implementation (sysimpl)
   */
public void ASME2SSME_Processor_Interface(ProcessModelImpl vprocess, ProcessorSubcomponent subprocessor, SystemImplementation sysimpl, ProcessModelImpl sysimpl_process){		
			
    Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_Interface: "+ subprocessor.getName());
    	
     //add input: event top;
     asme2ssme_common.ASME2SSME_add_input_to_process_interface("top", "event", vprocess); 
     
    ArrayList<ProcessSubcomponent> list_process_subcomponent = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subprocessor);
    
    Iterator<?> it1 = list_process_subcomponent.iterator();
    while (it1.hasNext()){
    	ProcessSubcomponent process_subcomponent = (ProcessSubcomponent) it1.next();
    	String process_subcomponent_name = aadlColAPI.Subcomponent_GetName(process_subcomponent);
    	ProcessImplementation process_impl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(process_subcomponent);
    	String process_impl_name = aadlColAPI.ComponentImpl_GetName(process_impl);
    	String process_impl_process_name = Name_of_Process(process_impl_name);
    	Trace(process_impl_process_name);
    	//ProcessModelImpl aprocess = myLib.STree_GetProcessDeclarationFromProcess(sysimpl_process, process_impl_process_name);
    	String process_type_name = Name_of_Type_Process(process_impl_process_name);
    	ProcessTypeImpl process_type = myLib.STree_GetProcessTypeDeclarationFromAProcess(sysimpl_process, process_type_name);
    	
    	//add the formal process as parameter
    	FormalProcessImpl formal_process = new FormalProcessImpl();
    	formal_process.setModelType(myLib.mkIdentifier(process_type_name));
    	formal_process.setModelName(myLib.mkIdentifier(process_subcomponent_name));
    	myLib.STree_addDeclarationToInterface(vprocess, formal_process, 1); 
    	
        ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) process_type.getInterfaceDefinition();
        if (inter != null){            
             IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
    	     if (vioProfile != null) 
    	     { 
    		       EList<TypesOrSignalsDeclaration> l_inputs = vioProfile.getInputs(); 
    		       Iterator<TypesOrSignalsDeclaration> it_i = l_inputs.iterator();
    		       while (it_i.hasNext())
    		     	 {
    		    	   SignalsDeclarationImpl ainput = (SignalsDeclarationImpl) it_i.next(); 
    		    	   EList<SignalNaming> list2 = ainput.getListSignal();
    		    	   //Trace(ainput.toString());
    		    	   
    		    	   if (ainput.getType() instanceof PredefinedTypeImpl){
    		    		  // Trace("ainput.getType() instanceof PredefinedTypeImpl");
    		    		   // PredefinedTypeImpl type_2 = (PredefinedTypeImpl) ainput.getType();
    		    		   
    			    	   for (int i = 0; i < list2.size(); i++) {  
    			    		   SignalNaming sig_name2 = list2.get(i);
    			    		   Identifier id_sig_name2 = sig_name2.getIdentifier();
    			    		   String str_sig_name2 = id_sig_name2.getAtomString();
    			    		   String name2 = process_subcomponent_name + '_' + str_sig_name2;
    			    		   
    			    		   if (!(str_sig_name2.endsWith("_Dispatch") | str_sig_name2.endsWith("_Resume") 
    			    				   | str_sig_name2.endsWith("_Deadline") | str_sig_name2.endsWith("_Frozen_time_event") 
    			    				   | str_sig_name2.endsWith("_Output_time_event") 
    			    				   | str_sig_name2.endsWith("_ctl1") | str_sig_name2.endsWith("_ctl2") 
    			    				   | str_sig_name2.endsWith("_time1") | str_sig_name2.equals("top")
    			    				   | str_sig_name2.endsWith("_Alarm") | str_sig_name2.endsWith("_TE") )){                                 			    			   
    			    			   //asme2ssme_common.ASME2SSME_add_input_to_process_interface(name2, "event", vprocess); 
    			    			   
//    				    		   if(((PredefinedTypeImpl)ainput.getType()).getType() == SignalIntrinsicType.BOOLEAN_LITERAL){
//    				    			   asme2ssme_common.ASME2SSME_add_input_to_process_interface(name2, "boolean", vprocess);
//    				    		   }
//    				    		   else{
//    				    			   asme2ssme_common.ASME2SSME_add_input_to_process_interface(name2, "event", vprocess);
//    				    		   }
    				    		   
    				    		   PredefinedTypeImpl ptype1 = myLib.STree_mkSamePredefinedType(ainput);
    				    		   Trace("ptype1.getType().getName() = " + ptype1.getType().getName());
    				    		   asme2ssme_common.ASME2SSME_add_input_to_process_interface(name2, ptype1.getType().getName(), vprocess);
    			    		   }  			    		   
    			    	   }
    		    	   }
    		    	   else {
    		    		   Identifier type_id = (Identifier) ainput.getType();
    		    	       String type_name = type_id.getAtomString();
    		    	       for (int j = 0; j < list2.size(); j++) {  
    			    		   SignalNaming sig_name2 = list2.get(j);
    			    		   Identifier id_sig_name2 = sig_name2.getIdentifier();
    			    		   String name2 = process_subcomponent_name + '_' + id_sig_name2.getAtomString();
    			    		   //ASME2SSME_add_input_to_process_interface(name2, type_name, vprocess);  
    			    		   if (!(name2.endsWith("_ctl1") | name2.endsWith("_time1"))){                                 			    			   
    			    			   asme2ssme_common.ASME2SSME_add_input_to_process_interface(name2, type_name, vprocess);  
    			    		   } 
    			    	   }
    		    	   }
    		     	 }
    		       
    	           EList<TypesOrSignalsDeclaration> l_outputs = vioProfile.getOutputs(); 
    	           Iterator<TypesOrSignalsDeclaration> it_o = l_outputs.iterator();
    		       while (it_o.hasNext())
    		     	 {
    		    	   SignalsDeclarationImpl aoutput = (SignalsDeclarationImpl) it_o.next(); 
    		    	   EList<SignalNaming> list3 = aoutput.getListSignal();
    		    	   
    		    	   if (aoutput.getType() instanceof PredefinedTypeImpl){
    		    		  // Trace("aoutput.getType() instanceof PredefinedTypeImpl");
    		    		   // PredefinedTypeImpl type_3 = (PredefinedTypeImpl) aoutput.getType();
    		    		   
    		    		   for (int i = 0; i < list3.size(); i++) {  
    			    		   SignalNaming sig_name3 = list3.get(i);
    			    		   Identifier id_sig_name3 = sig_name3.getIdentifier();
    			    		   String str_sig_name3 = id_sig_name3.getAtomString();
    			    		   Trace(str_sig_name3);
    			    		   String name3 = process_subcomponent_name + '_' + str_sig_name3;
    			    		   
    			    		   if (!(str_sig_name3.endsWith("_Complete") | str_sig_name3.endsWith("_Error")
    			    				   | str_sig_name3.endsWith("_Alarm") | str_sig_name3.endsWith("_TE"))){
    			    			   //asme2ssme_common.ASME2SSME_add_output_to_process_interface(name3, "event", vprocess); 
//    			    			   if(((PredefinedTypeImpl)aoutput.getType()).getType() == SignalIntrinsicType.BOOLEAN_LITERAL){
//    			    				   asme2ssme_common.ASME2SSME_add_output_to_process_interface(name3, "boolean", vprocess);
//    				    		   }
//    				    		   else{
//    				    			   asme2ssme_common.ASME2SSME_add_output_to_process_interface(name3, "event", vprocess);
//    				    		   }
    			    			   
    			    			   PredefinedTypeImpl ptype2 = myLib.STree_mkSamePredefinedType(aoutput);
    				    		   Trace("ptype2.getType().getName() = " + ptype2.getType().getName());
    				    		   asme2ssme_common.ASME2SSME_add_output_to_process_interface(name3, ptype2.getType().getName(), vprocess);
    			    		   }    			    		 
    			    	   }
    		    	   }
    		    	   else {
    		    		   Identifier type_id = (Identifier) aoutput.getType();
    		    	       String type_name = type_id.getAtomString();
    		    	       for (int j = 0; j < list3.size(); j++) {  
    			    		   SignalNaming sig_name3 = list3.get(j);
    			    		   Identifier id_sig_name3 = sig_name3.getIdentifier();
    			    		   String name3 = process_subcomponent_name + '_' + id_sig_name3.getAtomString();
    			    		   //ASME2SSME_add_output_to_process_interface(name3, type_name, vprocess);  
    			    		   if (!(name3.endsWith("_ctl2") )){
    			    			   asme2ssme_common.ASME2SSME_add_output_to_process_interface(name3, type_name, vprocess); 
    			    		   } 
    			    	   }
    		    	   }
    		     	 }
             }
        }
    }
	
    Trace("\n ========================= end ASME2SSME_Processor_Interface: "+ subprocessor.getName() + " ========================= \n");
}

  
/** 
 * Refines the processor interface, and removes some internal signals from the interface. 
 * <p>
 * Gets the processes that bound to this processor subcomponent (Processor_Get_Binding_Process()), 
 * and gets the connections between these processes. 
 * If a process connection connects two processes that bound to a same processor,
 * then the input/output of the processes are removed from the interface of the processor, 
 * they are represented as local signals.
 * 
 * @param vprocess a Signal process that represents this processor subcomponent
 * @param subprocessor a processor subcomponent
 * @param sysimpl a system implementation component which is the uppler level component that contains this processor subcomponent (subprocessor)
 * @param sysimpl_process a Signal process that represents this system implementation (sysimpl)
 */
public void ASME2SSME_Processor_Interface_Refine(ProcessModelImpl vprocess, ProcessorSubcomponent subprocessor, 
		                                                SystemImplementation sysimpl, ProcessModelImpl sysimpl_process){		
			
    Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_Interface_Refine: "+ subprocessor.getName());
  
    ArrayList<ProcessSubcomponent> list_process_subcomponent = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subprocessor);  
    //port connections
	EList<PortConnection> list1 = aadlColAPI.ComponentImpl_GetPortConnections(sysimpl);
    if( list1!= null){
  	       	
    	// DataConnections
  	    ArrayList<PortConnection> listDataConnections = aadlColAPI.ComponentImpl_GetDataPortConnections(sysimpl);
  	    for(int i = 0; i< listDataConnections.size(); i++)
  	    {
  	    	PortConnection dataConnection = (PortConnection) listDataConnections.get(i);
  		    if ((aadlColAPI.PortConnection_GetSrcContext(dataConnection) instanceof ProcessSubcomponent) 
  		    		&& (aadlColAPI.PortConnection_GetDstContext(dataConnection) instanceof ProcessSubcomponent)){ //sub-process -> sub-process
  			   
  		    	ProcessSubcomponent sub_process1 = (ProcessSubcomponent) aadlColAPI.PortConnection_GetSrcContext(dataConnection);
  			    DataPort src_port = (DataPort) aadlColAPI.PortConnection_GetSrc(dataConnection);
  			    ProcessSubcomponent sub_process2 = (ProcessSubcomponent) aadlColAPI.PortConnection_GetDstContext(dataConnection);
  			    DataPort dst_port = (DataPort) aadlColAPI.PortConnection_GetDst(dataConnection);
  			    Trace(sub_process1.getName() + ' ' + src_port.getName() + ' ' + sub_process2.getName() + ' ' + dst_port.getName() );
  			    
  			    if (list_process_subcomponent.contains(sub_process1) && list_process_subcomponent.contains(sub_process2)){
  				    myLib.STree_rmNamedDeclarationFromInterface(vprocess, aadlColAPI.Subcomponent_GetName(sub_process2)+'_'+ aadlColAPI.Port_GetName(dst_port), 2);
  				    if(!aadlColAPI.Exist_Connection_SubProcess_to_System(sysimpl, sub_process1, src_port)){ //output of sub process1 is not connected to upper level system
			        	myLib.STree_rmNamedDeclarationFromInterface(vprocess, aadlColAPI.Subcomponent_GetName(sub_process1)+'_'+ aadlColAPI.Port_GetName(src_port), 3);
			        }
  			    }
  		    }
  	    }
  	    
  	    // 	EventConnections
  	    ArrayList<PortConnection> list2 = aadlColAPI.ComponentImpl_GetEventPortConnections(sysimpl);
  	    for(int j = 0; j< list2.size(); j++)
  	    {
  	    	PortConnection c2 = (PortConnection) list2.get(j);
  		    if ((aadlColAPI.PortConnection_GetSrcContext(c2) instanceof ProcessSubcomponent) 
  		    		&& (aadlColAPI.PortConnection_GetDstContext(c2) instanceof ProcessSubcomponent)){ //sub-process -> sub-process
  			   
  		    	ProcessSubcomponent sub_process1 = (ProcessSubcomponent) aadlColAPI.PortConnection_GetSrcContext(c2);
  			    EventPort src_port = (EventPort) aadlColAPI.PortConnection_GetSrc(c2);
  			    ProcessSubcomponent sub_process2 = (ProcessSubcomponent) aadlColAPI.PortConnection_GetDstContext(c2);
  			    EventPort dst_port = (EventPort) aadlColAPI.PortConnection_GetDst(c2);
  			    Trace(sub_process1.getName() + ' ' + src_port.getName() + ' ' + sub_process2.getName() + ' ' + dst_port.getName() );
  			    
  			    if (list_process_subcomponent.contains(sub_process1) && list_process_subcomponent.contains(sub_process2)){
  				    myLib.STree_rmNamedDeclarationFromInterface(vprocess, aadlColAPI.Subcomponent_GetName(sub_process2)+'_'+ aadlColAPI.Port_GetName(dst_port), 2);
  				    if(!aadlColAPI.Exist_Connection_SubProcess_to_System(sysimpl, sub_process1, src_port)){ //output of sub process1 is not connected to upper level system
			        	myLib.STree_rmNamedDeclarationFromInterface(vprocess, aadlColAPI.Subcomponent_GetName(sub_process1)+'_'+ aadlColAPI.Port_GetName(src_port), 3);
			        }
  			    }
  		    }
  	    }
  	    
  	// EventDataConnections
  	    ArrayList<PortConnection> list3 = aadlColAPI.ComponentImpl_GetEventDataPortConnections(sysimpl);
  	    for(int k = 0; k< list3.size(); k++)
  	    {
  	    	PortConnection c3 = (PortConnection) list3.get(k);
  		    if ((aadlColAPI.PortConnection_GetSrcContext(c3) instanceof ProcessSubcomponent) 
  		    		&& (aadlColAPI.PortConnection_GetDstContext(c3) instanceof ProcessSubcomponent)){ //sub-process -> sub-process
  			   
  		    	ProcessSubcomponent sub_process1 = (ProcessSubcomponent) aadlColAPI.PortConnection_GetSrcContext(c3);
  			    EventDataPort src_port = (EventDataPort) aadlColAPI.PortConnection_GetSrc(c3);
  			    ProcessSubcomponent sub_process2 = (ProcessSubcomponent) aadlColAPI.PortConnection_GetDstContext(c3);
  			    EventDataPort dst_port = (EventDataPort) aadlColAPI.PortConnection_GetDst(c3);
  			    Trace(sub_process1.getName() + ' ' + src_port.getName() + ' ' + sub_process2.getName() + ' ' + dst_port.getName() );
  			    
  			    if (list_process_subcomponent.contains(sub_process1) && list_process_subcomponent.contains(sub_process2)){
  				    myLib.STree_rmNamedDeclarationFromInterface(vprocess, aadlColAPI.Subcomponent_GetName(sub_process2)+'_'+ aadlColAPI.Port_GetName(dst_port), 2);
  				    if(!aadlColAPI.Exist_Connection_SubProcess_to_System(sysimpl, sub_process1, src_port)){ //output of sub process1 is not connected to upper level system
  			        	myLib.STree_rmNamedDeclarationFromInterface(vprocess, aadlColAPI.Subcomponent_GetName(sub_process1)+'_'+ aadlColAPI.Port_GetName(src_port), 3);
  			        }
  			    }
  		    }
  	    }
  	      	    
      }
}


/** 
 * Adds process calls of the process types (which represents AADL process components) to the body of a Signal process that represents a processor component.  
 * <p>
 * Gets the processes that bound to this subprocessor (Processor_Get_Binding_Process()). 
 * For each process, find out its corresponding translated process type, 
 * make a process call and add it in the processor's Signal process body (ASME2SSME_ProcessType_Instance()).
 * 
 * @param vprocess a Signal process that represents this processor subcomponent
 * @param sysimpl_process a Signal process that represents the upper level system implementation component
 * @param subprocessor a sub processor component
 * @param sysimpl an upper level system implementation component that contains this processor subcomponent
 */
public void ASME2SSME_Processor_ProcessType_instances(ProcessModelImpl vprocess, ProcessModelImpl sysimpl_process, 
		                                              ProcessorSubcomponent subprocessor, SystemImplementation sysimpl){
	
	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_ProcessType_instances : " + subprocessor.getName());
	ASME2SSME_Translate_Process asme2ssme_process = new ASME2SSME_Translate_Process();
	
	ArrayList<ProcessSubcomponent> list_process_subcomponent = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subprocessor);
  
  Iterator<?> it1 = list_process_subcomponent.iterator();
  while (it1.hasNext()){
  	ProcessSubcomponent process_subcomponent = (ProcessSubcomponent) it1.next();
  	ProcessImplementation process_impl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(process_subcomponent);
  	String process_impl_name = aadlColAPI.ComponentImpl_GetName(process_impl);
  	String process_impl_process_name = Name_of_Process(process_impl_name);   	
  	String type_name = Name_of_Type_Process(process_impl_process_name);	
  	
  	ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) vprocess.getInterfaceDefinition();
	    EList<TypesOrSignalsDeclaration> param_list = inter.getListFormalParameter();	
	    for(int i = 0; i< param_list.size(); i++){
		    if(param_list.get(i) instanceof FormalProcessImpl){
			     FormalProcessImpl formal_process = (FormalProcessImpl) param_list.get(i);
			     String formal_process_model_type = formal_process.getModelType().getAtomString();
			     if (formal_process_model_type.equals(type_name)){
			    	 asme2ssme_process.ASME2SSME_ProcessType_Instance(formal_process, vprocess, process_subcomponent, sysimpl_process);
			     }
		   }	    
	    }
  }
  
  Trace("\n ========================= end ASME2SSME_Processor_ProcessType_instances ========================= \n");
}



/**
 * Translates subcomponents of a processor implementation
 * 
 * @param p
 */
public void ASME2SSME_ProcessorImpl_Subs(ProcessorImplementation p, ProcessModelImpl vprocess){
	
	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_ProcessorImpl_Subs: " + p.getName());
	
    ArrayList<Subcomponent> psub = aadlColAPI.Processor_GetSubcomponents(p);
	if(psub.size() >0){
		
		//MemorySubcomponent
		ArrayList<MemorySubcomponent> list1 = aadlColAPI.Processor_GetSubMemorys(p);		
		ASME2SSME_ProcessorImpl_SubMemorys(p, vprocess, list1);
		
		//BusSubcomponent
		ArrayList<BusSubcomponent> list2 = aadlColAPI.Processor_GetSubBus(p);		
		ASME2SSME_ProcessorImpl_SubBuses(p, vprocess, list2);
	}
}


/**
 * Translates the sub memorys of a processor implementation.
 * <p>
 * For each memory subcomponent, get its classifier (a memory type or implementation). 
 * If this classifier has already been implemented, get its Signal process declaration, 
 * otherwise, create a Signal process (ASME2SSME_MemoryImpl()). 
 * Add an instance of this Signal process in the body of sys_process.
 * 
 * @param vprocess
 * @param list1
 */
public void ASME2SSME_ProcessorImpl_SubMemorys(ProcessorImplementation p, ProcessModelImpl vprocess, ArrayList<MemorySubcomponent> list1){
	
	Trace("\n == ASME2SSME_Translate_Processor: ASME2SSME_ProcessorImpl_SubMemorys : " + p.getName() );

	ArrayList<MemoryClassifier> l_m_classifier = new ArrayList<MemoryClassifier>();
	if(list1.size()>0){	
		Iterator<?> it = list1.iterator();
		while (it.hasNext())
		{
			MemorySubcomponent ms1 = (MemorySubcomponent) it.next();
			String subm_name = aadlColAPI.Subcomponent_GetName(ms1); 
			MemoryClassifier mc = (MemoryClassifier) aadlColAPI.Memorysubcomponent_GetClassifier(ms1);
			ProcessModelImpl aprocess;
			
			if(mc instanceof MemoryType){
				MemoryType m_type = (MemoryType) mc;
				String mt_name = aadlColAPI.ComponentType_GetName(m_type);
				String mt_process_name = Name_of_Memory(mt_name);
				if (l_m_classifier.contains(m_type) ){ 
					Trace("memory classifier (type) has already been created ");
					aprocess = myLib.STree_GetProcessDeclarationFromProcess(vprocess, mt_process_name);
				}
				else{//this memeory type has been created in Signal
					ASME2SSME_Translate_Memory asme2ssme_memory = new ASME2SSME_Translate_Memory();
					aprocess = asme2ssme_memory.ASME2SSME_MemoryType(m_type);
					myLib.STree_addProcessDeclaration(vprocess, aprocess);
				}
			}
			else{//MemoryImpl
				MemoryImplementation m_imp = (MemoryImplementation) mc;
				String mi_name = aadlColAPI.ComponentImpl_GetName(m_imp);
				String mi_process_name = Name_of_Memory(mi_name);
				if (l_m_classifier.contains(m_imp) ){ 
					Trace("memory classifier (implementation) has already been created ");
					aprocess = myLib.STree_GetProcessDeclarationFromProcess(vprocess, mi_process_name);
				}
				else{//this memeory type has been created in Signal
					ASME2SSME_Translate_Memory asme2ssme_memory = new ASME2SSME_Translate_Memory();
					aprocess = asme2ssme_memory.ASME2SSME_MemoryImpl(m_imp);
					myLib.STree_addProcessDeclaration(vprocess, aprocess);
				}
			}
			
			asme2ssme_common.ASME2SSME_component_instance(vprocess, aprocess, subm_name);
		}
	}
}


/** Translates the sub buses of a processor implementation.
 * <p>
 * For each bus subcomponent, get its classifier (a device type or implementation). 
 * If this classifier has already been implemented, get its Signal process declaration, 
 * otherwise, create a Signal process (ASME2SSME_BusImpl()). 
 * Add an instance of this Signal process in the body of sys_process.
 * 
 * @param sys_process a Signal process that represents a system implementation component
 * @param subbus_list a list of bus subcomponents of a system 
 */
public void ASME2SSME_ProcessorImpl_SubBuses(ProcessorImplementation p, ProcessModelImpl vprocess, ArrayList<BusSubcomponent> subbus_list){
	
	Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemImpl_SubBuses : " +p.getName() );

	ArrayList<BusClassifier> l_bus_classifier = new ArrayList<BusClassifier>();
	Iterator<?> it5 = subbus_list.iterator();
	while (it5.hasNext())
	{
		BusSubcomponent b1 = (BusSubcomponent) it5.next();
		String subbus_name = aadlColAPI.Subcomponent_GetName(b1);
		//suppose the bus classifier is always a bus implementation 
		BusImplementation bus_impl = (BusImplementation) aadlColAPI.Bussubcomponent_GetClassifier(b1);
		String bus_impl_name = aadlColAPI.ComponentImpl_GetName(bus_impl);
		String bus_impl_process_name = Name_of_Bus(bus_impl_name);
		
		ProcessModelImpl aprocess;
		if (l_bus_classifier.contains(bus_impl) ){ 
				Trace("bus classifier has already been created ");
				aprocess = myLib.STree_GetProcessDeclarationFromProcess(vprocess, bus_impl_process_name);
		}
		else { //bus classifier has not been created in Signal					    
			   // processImpl declaration
			    ASME2SSME_Translate_Bus asme2ssme_bus = new ASME2SSME_Translate_Bus();
			    if (bus_impl instanceof BusType){
			    	aprocess = new ProcessModelImpl();
			    	Trace("bus_impl instanceof BusType not implemented ");
			    }
			    else if (bus_impl instanceof BusImplementation){
				        aprocess = asme2ssme_bus.ASME2SSME_BusImpl((BusImplementation) bus_impl);
			         }
			         else{
				          aprocess = new ProcessModelImpl();
				          java.lang.System.err.print("\n bus classifier illegal ");
						  throw new IllegalArgumentException("bus classifier illegal !!  \n");
			         }
			    myLib.STree_addProcessDeclaration(vprocess, aprocess);
			    l_bus_classifier.add(bus_impl);
		      } 
		asme2ssme_common.ASME2SSME_component_instance(vprocess, aprocess, subbus_name);
	}
}


/**
 * Translates flows of a processor implementation
 * 
 * @param p a processor implementation component
 */
public void ASME2SSME_ProcessorImpl_Flows(ProcessModelImpl vprocess, ProcessorImplementation p){
	
	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_ProcessorImpl_Flows: " + p.getName());
	
	asme2ssme_common.ASME2SSME_ComponentImpl_Flows(p);
	
	if(asme2ssme_common.ASME2SSME_ComponentImpl_Flows(p) != null){
		ProcessModelImpl p1= asme2ssme_common.ASME2SSME_ComponentImpl_Flows(p);
		
		myLib.STree_addProcessDeclaration(vprocess, p1);
		ArrayList<ExprParam> lparam = new ArrayList<ExprParam> ();
		String pname = Name_of_Flow(p.getName().replace('.','_'));
		ProcessExpansionImpl call1 = myLib.STree_mkProcessCallWithParameters(myLib.mkIdentifier(pname), lparam);
        myLib.STree_addExpressionOfProcessToaModel(vprocess, call1);
	}
}


/**
 * Translate modes of a processor implementation component
 * 
 * @param p
 */
public void ASME2SSME_ProcessorImpl_Modes(ProcessorImplementation p){
	
	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_ProcessorImpl_Modes: " + p.getName());
	asme2ssme_common.ASME2SSME_ComponentImpl_Modes(p);
}

  
/**
 * Makes a link between the input/output signals to the local signals.
 * For example, an input xx defined in the interface, then an equation is added in the process body:
 * l_xx := xx ( xx:= l_xx for an output). 
 * The local signals are created and used in the process instances of the AADL process.
 * The signals that coming from external system is not included in this action, i.e., event top
 * 
 * @param vprocess a Signal process that represents an AADL processor
 */
public void ASME2SSME_Processor_InOut_Info(ProcessModelImpl vprocess){
	
	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_InOut_Info ");
	
	ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) vprocess.getInterfaceDefinition();
	IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
	
	if (vioProfile == null) 
	{ 
		vioProfile = new IOProfileImpl();
		inter.setIoProfile(vioProfile);
	}
	
	EList<TypesOrSignalsDeclaration> l_in = vioProfile.getInputs();
	EList<TypesOrSignalsDeclaration> l_out = vioProfile.getOutputs();

	//inputs ==================================
	Iterator<TypesOrSignalsDeclaration> it_i = l_in.iterator();
    while (it_i.hasNext())
  	 {
 	   SignalsDeclarationImpl ainput = (SignalsDeclarationImpl) it_i.next(); 
 	   EList<SignalNaming> list1 = ainput.getListSignal();
 	   
 	   for (int i = 0; i < list1.size(); i++) { 
 		   SignalNaming sig_name1 = list1.get(i);
		   Identifier id_sig_name1 = sig_name1.getIdentifier();
		   String right_name1 =  id_sig_name1.getAtomString();
		   if(right_name1.equals("top") == false){
			   ExprSignal right_expr1 = myLib.mkIdentifier(right_name1);
	  		   String left_name1 =  "l_"+right_name1;
			   ExprSignal left_expr1 = myLib.mkIdentifier(getSname( left_name1));
			   ExprProcess eq1 = (SignalsDefinitionImpl) myLib.mkEquation(left_expr1, right_expr1);
			   myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1); 
		   }
 	  }	   
  	 }
    
    //outputs ============================================
    Iterator<TypesOrSignalsDeclaration> it_o = l_out.iterator();
    while (it_o.hasNext())
  	 {
 	   SignalsDeclarationImpl aoutput = (SignalsDeclarationImpl) it_o.next(); 
 	   EList<SignalNaming> list2 = aoutput.getListSignal();
 	   
 	   for (int i = 0; i < list2.size(); i++) { 
 		   SignalNaming sig_name2 = list2.get(i);
		   Identifier id_sig_name2 = sig_name2.getIdentifier();
		   String left_name2 =  id_sig_name2.getAtomString();
  		   ExprSignal left_expr2 = myLib.mkIdentifier(left_name2);
  		   String right_name2 =  "l_"+left_name2;
		   ExprSignal right_expr2 = myLib.mkIdentifier(getSname( right_name2) /*right_name2*/);
		   ExprProcess eq2 = (SignalsDefinitionImpl) myLib.mkEquation( left_expr2, right_expr2);
		   myLib.STree_addExpressionOfProcessToaModel(vprocess, eq2); 
 	  }	   
  	 }
}
  
  
/**
 * Returns a Signal process "xx_processor_behavior" that represents the behavior of a processor subcomponenet
 * <p>
 * This Signal process is named as xx_processor_behavior, where xx is this processor subcomponent's name. 
 * An empty process skeleton model is provided. 
 * The interface is implemented by ASME2SSME_Processor_Behavior_Interface() method. 
 * 
 * @param vprocess a Signal process that represents this processor subcomponent
 * @param subprocessor a processor subcomponent
 * @param sysimpl a system implementation component which contains this subprocessor
 * @param sysimpl_process a Signal process that represents this sysimpl system implementation
 * @return a ProcessModelImpl (Signal process)
 */
public ProcessModelImpl ASME2SSME_Processor_Behavior(ProcessModelImpl vprocess, ProcessorSubcomponent subprocessor, 
		                                          SystemImplementation sysimpl, ProcessModelImpl sysimpl_process){
	
	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_Behavior " + subprocessor.getName());
	
	String subprocessor_name = aadlColAPI.Subcomponent_GetName(subprocessor);
	String pname = Name_of_Processor(subprocessor_name);
	String bname = pname + "_Behavior";
	ProcessModelImpl p_behavior = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(bname) );
	ASME2SSME_Processor_Behavior_Interface(p_behavior, subprocessor, sysimpl,sysimpl_process);
	boolean b= false;
	
	if (b)
	{
		//OLD OLD  
		//version 1 -- simulate dispatch, resume, deadline, initialize, timeevent
		//Simulate Dispatch
		ASME2SSME_Processor_SimDispatch(subprocessor, p_behavior, sysimpl);
		//Simulate Resume
		ASME2SSME_Processor_SimResume(subprocessor, p_behavior, sysimpl);	
		//Simulate Deadline
		ASME2SSME_Processor_SimDeadline(subprocessor, p_behavior, sysimpl);	
		//Simulate Initialize
		ASME2SSME_Processor_SimInitialize(subprocessor, p_behavior, sysimpl);	 
		//Simulate Time_Event
		ASME2SSME_Processor_SimTimeEvent(subprocessor, p_behavior, sysimpl);
	}
	else
	{
		// version 2 -- connect to scheduling generator.
		ASME2SSME_Scheduling scheduling1 = new ASME2SSME_Scheduling();
		scheduling1.ASME2SSME_Processor_Scheduling(subprocessor, p_behavior, sysimpl);
	}
	return p_behavior;
}


/** 
 * Adds inputs/outputs to the interface of a Signal process that represents a processor behavior
 * <p>
 * Gets the processes that bound to this processor subcomponent (Processor_Get_Binding_Process()), 
 * and gets the connections between these processes. 
 * For each such process p1, the interface should includes the inputs: 
 * p1_xx, where xx is an output of p1, and it ends with "_ctl2", "_Alarm", "_TE", 
 * and the outputs: 
 * p1_yy, where yy is an input of p1, and it ends with "_ctl1", "_time1"
 * 
 * @param vprocess a Signal process that represents this processor subcomponent
 * @param subprocessor a processor subcomponent
 * @param sysimpl a system implementation component which is the uppler level component that contains this processor subcomponent (subprocessor)
 * @param sysimpl_process a Signal process that represents this system implementation (sysimpl)
 */
public void ASME2SSME_Processor_Behavior_Interface(ProcessModelImpl vprocess, ProcessorSubcomponent subprocessor, 
		                                       SystemImplementation sysimpl, ProcessModelImpl sysimpl_process){		
		
//	(? event top;
//     CTL2 xx_ctl2, ... ;
//     boolean xx_Alarm, ...;
//	   event xx_yy_TE, ...;
//	 ! CTL1 xx_ctl1; 
//	   xx_TIME_EVENT xx_time1;
//	   ...)
  Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_Behavior_Interface: "+ subprocessor.getName());
  
  // add input signal: event top;
  asme2ssme_common.ASME2SSME_add_input_to_process_interface("top", "event", vprocess);	
  
  ArrayList<ProcessSubcomponent> list_process_subcomponent = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subprocessor);
  
  Iterator<?> it1 = list_process_subcomponent.iterator();
  while (it1.hasNext()){
  	ProcessSubcomponent process_subcomponent = (ProcessSubcomponent) it1.next();
  	String process_subcomponent_name = aadlColAPI.Subcomponent_GetName(process_subcomponent);
  	ProcessImplementation process_impl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(process_subcomponent);
  	String process_impl_name = aadlColAPI.ComponentImpl_GetName(process_impl);
  	String process_impl_process_name = Name_of_Process(process_impl_name);
  	Trace(process_impl_process_name);
  	ProcessTypeImpl process_type = myLib.STree_GetProcessTypeDeclarationFromAProcess(sysimpl_process, Name_of_Type_Process(process_impl_process_name));
  	   	
      ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) process_type.getInterfaceDefinition();
      if (inter != null){            
           IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
  	     if (vioProfile != null) 
  	     { 
  		       EList<TypesOrSignalsDeclaration> l_inputs = vioProfile.getInputs(); 
  		       Iterator<TypesOrSignalsDeclaration> it_i = l_inputs.iterator();
  		       while (it_i.hasNext())
  		     	 {
  		    	   SignalsDeclarationImpl ainput = (SignalsDeclarationImpl) it_i.next(); 
  		    	   EList<SignalNaming> list2 = ainput.getListSignal();
  		    	   //Trace(ainput.toString());
  		    	   
  		    	   if (ainput.getType() instanceof PredefinedTypeImpl){
  		    		  // Trace("ainput.getType() instanceof PredefinedTypeImpl");
  		    		  // PredefinedTypeImpl type_2 = (PredefinedTypeImpl) ainput.getType();
  		    		   
  			    	   for (int i = 0; i < list2.size(); i++) {  
  			    		   SignalNaming sig_name2 = list2.get(i);
  			    		   Identifier id_sig_name2 = sig_name2.getIdentifier();
  			    		   String str_sig_name2 = id_sig_name2.getAtomString();
  			    		   String name2 = process_subcomponent_name + '_' + str_sig_name2;
  			    		   if (str_sig_name2.endsWith("_Dispatch") | str_sig_name2.endsWith("_Resume") 
  			    				   | str_sig_name2.endsWith("_Deadline") 
  			    				   | str_sig_name2.endsWith("_Frozen_time_event") 
  			    				   | str_sig_name2.endsWith("_Output_time_event")){                                 			    			   
  			    			   asme2ssme_common.ASME2SSME_add_output_to_process_interface(name2, "event", vprocess);  
  			    		   } 
  			    	   }
  		    	   }
  		       else {
  		    	   Identifier type_id = (Identifier) ainput.getType();
		    	       String type_name = type_id.getAtomString();
		    	       for (int j = 0; j < list2.size(); j++) {  
			    		   SignalNaming sig_name2 = list2.get(j);
			    		   Identifier id_sig_name2 = sig_name2.getIdentifier();
			    		   String name2 = process_subcomponent_name + '_' + id_sig_name2.getAtomString();
			    		   if (name2.endsWith("_ctl1")){
			    			   asme2ssme_common.ASME2SSME_add_output_to_process_interface(name2, type_name, vprocess);                      			    			   
			    		   }
			    		   else  if( name2.endsWith("_time1")){      			    		  
			    			   asme2ssme_common.ASME2SSME_add_output_to_process_interface(name2, type_name, vprocess); 
			    		   } 
			    	   }
		    	   }
  		    }
  		       
  	           EList<TypesOrSignalsDeclaration> l_outputs = vioProfile.getOutputs(); 
  	           Iterator<TypesOrSignalsDeclaration> it_o = l_outputs.iterator();
  		       while (it_o.hasNext())
  		     	 {
  		    	   SignalsDeclarationImpl aoutput = (SignalsDeclarationImpl) it_o.next(); 
  		    	   EList<SignalNaming> list3 = aoutput.getListSignal();
  		    	   
  		    	   if (aoutput.getType() instanceof PredefinedTypeImpl){
  		    		   // Trace("aoutput.getType() instanceof PredefinedTypeImpl");
  		    		   // PredefinedTypeImpl type_3 = (PredefinedTypeImpl) aoutput.getType();
  		    		   
  		    		   for (int i = 0; i < list3.size(); i++) {  
  			    		   SignalNaming sig_name3 = list3.get(i);
  			    		   Identifier id_sig_name3 = sig_name3.getIdentifier();
  			    		   String str_sig_name3 = id_sig_name3.getAtomString();
  			    		   Trace(str_sig_name3);
  			    		   String name3 = process_subcomponent_name + '_' + str_sig_name3;
  			    		   if (str_sig_name3.endsWith("_Complete") | str_sig_name3.endsWith("_Error") ){
  			    			   asme2ssme_common.ASME2SSME_add_input_to_process_interface(name3, "event", vprocess); 
  			    		   }  
  			    		   else if (name3.endsWith("_Alarm") ){
			    			   asme2ssme_common.ASME2SSME_add_input_to_process_interface(name3, "boolean", vprocess);
			    		   }
  			    		   else if (str_sig_name3.endsWith("_TE") ){
			    			   asme2ssme_common.ASME2SSME_add_input_to_process_interface(name3, "event", vprocess);
			    		   }
  			    		   
  			    	   }
  		    	   }
  		    	   else {
      		    	   Identifier type_id = (Identifier) aoutput.getType();
  		    	       String type_name = type_id.getAtomString();
  		    	       for (int j = 0; j < list3.size(); j++) {  
  			    		   SignalNaming sig_name3 = list3.get(j);
  			    		   Identifier id_sig_name3 = sig_name3.getIdentifier();
  			    		   String name3 = process_subcomponent_name + '_' + id_sig_name3.getAtomString();

  			    		   if (name3.endsWith("_ctl2")  ){
  			    			   asme2ssme_common.ASME2SSME_add_input_to_process_interface(name3, type_name, vprocess);
  			    		   }
  			    	   }
  		    	   }
  		    	   
  		     	 }
           }
      }
  }
  		   
    Trace("\n ========================= end ASME2SSME_Processor_Behavior_Interface: "+ subprocessor.getName() + " ========================= \n");
}



/**
 * Adds an instance of a processor behavior Signal process (behavior_process) into a processor process (vprocess).
 * <p>
 * Each formal declaration (parameter,input,output) of behavior_process is taken as an effective parameter of the call, 
 * except for the input event top (this event coming from the processor Signal process). 
 * They are omitted as implicited parameters (in order to simplify the code).
 * Local signal declarations are added in the (vprocess) body.
 * 
 * @param vprocess a Signal process that represents a processor subcomponent
 * @param behavior_process a Signal process that represents the behavior part of a processor subcomponent
 */
public void ASME2SSME_Processor_Behavior_Instance(ProcessModelImpl vprocess, ProcessModelImpl behavior_process){
	
	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_Behavior_Instance");
	String instance_name = behavior_process.getIdentifier().getAtomString();
	
	// ArrayList<ExprParam> l_param = new ArrayList<ExprParam>();//parameter (id) list for call the process pprocess
  ArrayList<ExprSignal> l_in = new ArrayList<ExprSignal>();//input (id) list for call the process pprocess
  ArrayList<ExprSignal> l_out = new ArrayList<ExprSignal>();//output (id) list for call the process pprocess
	ArrayList<Declaration> vdecl = new ArrayList<Declaration>();//local signals list
	
  ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) behavior_process.getInterfaceDefinition();
  if (inter != null){
  	
	 // EList<TypesOrSignalsDeclaration> l_parameters = inter.getListFormalParameter();        
	 // Iterator<TypesOrSignalsDeclaration> it_p = l_parameters.iterator();        
       IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
	     if (vioProfile != null) 
	     { 
		       EList<TypesOrSignalsDeclaration> l_inputs = vioProfile.getInputs(); 
		       Iterator<TypesOrSignalsDeclaration> it_i = l_inputs.iterator();
		       while (it_i.hasNext())
		     	 {
		    	   SignalsDeclarationImpl ainput = (SignalsDeclarationImpl) it_i.next(); 
		    	   EList<SignalNaming> list2 = ainput.getListSignal();
		    	   
		    	   if (ainput.getType() instanceof PredefinedTypeImpl){
		    		  // PredefinedTypeImpl type_2 = (PredefinedTypeImpl) ainput.getType();		    		   
			    	   for (int i = 0; i < list2.size(); i++) {  
			    		   SignalNaming sig_name2 = list2.get(i);
			    		   Identifier id_sig_name2 = sig_name2.getIdentifier();
			    		   String name2 = id_sig_name2.getAtomString();
			    		   
                           if(name2.endsWith("_Alarm")){
                        	   SignalsDeclarationImpl sig2 = myLib.mkBooleanDecl(name2);
				    		   l_in.add(myLib.mkIdentifier(name2));
				    		   vdecl.add(sig2);
			    		   }
                           else if(name2.equals("top")){
                        	   
                           }
                           else if(name2.endsWith("_TE")){
                        	   SignalsDeclarationImpl sig2 = myLib.mkEventDecl(name2);
				    		   l_in.add(myLib.mkIdentifier(name2));
				    		   vdecl.add(sig2);
			    		   }
                           else{
                        	   SignalsDeclarationImpl sig2 = myLib.mkDecl(name2, myLib.STree_mkSamePredefinedType(ainput));
				    		   l_in.add(myLib.mkIdentifier(name2));
				    		   vdecl.add(sig2);
			    		   }
                           
//                           if(! name2.equals("top")){
//                        	   SignalsDeclarationImpl sig2 = myLib.mkEventDecl (name2);
//				    		   l_in.add(myLib.mkIdentifier(name2));
//				    		   vdecl.add(sig2);
//			    		   }	
			    		   
			    	   }
		    	   }
		    	   else{
		    		   Identifier type_id = (Identifier) ainput.getType();
		    	       String type_name = type_id.getAtomString();
		    	       IdentifierImpl type2 = myLib.mkIdentifier(type_name);
		    	   
		    	       for (int i = 0; i < list2.size(); i++) {  
		    		       SignalNaming sig_name2 = list2.get(i);
		    		       Identifier id_sig_name2 = sig_name2.getIdentifier();
		    		       String name2 = id_sig_name2.getAtomString();
		    		       SignalsDeclarationImpl sig2 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(name2), type2, null);
		    		       l_in.add(myLib.mkIdentifier(name2));
		    		       vdecl.add(sig2);
		    	       }
		    	       }		    	   
		     	   }
		       
	           EList<TypesOrSignalsDeclaration> l_outputs = vioProfile.getOutputs(); 
	           Iterator<TypesOrSignalsDeclaration> it_o = l_outputs.iterator();
		       while (it_o.hasNext())
		     	 {
		    	   SignalsDeclarationImpl aoutput = (SignalsDeclarationImpl) it_o.next(); 
		    	   EList<SignalNaming> list3 = aoutput.getListSignal();
		    	   
		    	   if (aoutput.getType() instanceof PredefinedTypeImpl){
		    		  // PredefinedTypeImpl type_3 = (PredefinedTypeImpl) aoutput.getType();
		    		   for (int i = 0; i < list3.size(); i++) {  
			    		   SignalNaming sig_name3 = list3.get(i);
			    		   Identifier id_sig_name3 = sig_name3.getIdentifier();
			    		   String name3 = id_sig_name3.getAtomString();
			    		   SignalsDeclarationImpl sig3 = myLib.mkDecl(name3, myLib.STree_mkSamePredefinedType(aoutput));
			    		   l_out.add(myLib.mkIdentifier(name3));
			    		   vdecl.add(sig3);
			    	   }
		    	   }
		    	   else{
		    		   Identifier type_id = (Identifier) aoutput.getType();
		    	       String type_name = type_id.getAtomString();		     
		    	       IdentifierImpl type3 = myLib.mkIdentifier(type_name);    	   
		    	   
		    	       for (int i = 0; i < list3.size(); i++) {  
		    		       SignalNaming sig_name3 = list3.get(i);
		    		       Identifier id_sig_name3 = sig_name3.getIdentifier();
		    		       String name3 = id_sig_name3.getAtomString();
		    		       SignalsDeclarationImpl sig3 = myLib.mkDecl( name3, type3);
		    		       l_out.add(myLib.mkIdentifier(name3));
		    		       vdecl.add(sig3);
		    	       }
		    	     }
		     	 }
       }
  }
 
  // a complete version for instance with all the parameters (same name as in the process definition)
//  if (l_out.size()>0){
//      SignalsDefinitionImpl call_eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(instance_name), l_param, l_in, l_out);
//      myLib.STree_addExpressionOfProcessToaModel(vprocess, call_eq);
//  }
//  else{
//  	ProcessCallImpl pcall = myLib.STree_mkCompleteProcessCall(myLib.mkIdentifier(instance_name), l_param, l_in);
//      myLib.STree_addExpressionOfProcessToaModel(vprocess, pcall);
//  }
  
  // to simplify the generated code, the effective parameters are omitted
  //ProcessCallImpl pcall = myLib.STree_mkModelCall(instance_name);
  ArrayList<ExprParam> lparam = new ArrayList<ExprParam> ();
  ProcessExpansionImpl pcall = myLib.STree_mkProcessCallWithParameters(myLib.mkIdentifier(instance_name), lparam);
  myLib.STree_addExpressionOfProcessToaModel(vprocess, pcall);
  
  //add comments for processor behavior process instance
	ArrayList<String> comments3 = new ArrayList<String>();
	comments3.add("processor behavior sub process");
	myLib.STree_addComment(pcall, comments3, false);
  
  myLib.STree_addLocalDeclaration(vprocess, vdecl);
  Trace("\n ========================= end ASME2SSME_Processor_Behavior_Instance ========================= \n");
}


/** 
 * Returns a Signal process "xx_property" that represents the properties of a processor component.
 * <p>
 * This Signal process is named as "xx_property", where "xx" is a name of a Signal process which represents this processor subcomponent. 
 * An empty skeleton process model is provided
 * 
 *  @param p_name a String name of a Signal process which represents this processor subcomponent
 * @return a ProcessModelImpl (Signal process)
 */
public ProcessModelImpl ASME2SSME_Processor_property(ProcessorSubcomponent subprocessor){
		
	String subprocessor_name = aadlColAPI.Subcomponent_GetName(subprocessor);
	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_property: " + subprocessor_name );
	
    String p_name = Name_of_Processor(subprocessor_name);
	String pname = p_name + "_property";
	ProcessModelImpl p_property = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );
	
	ProcessorImplementation p_impl = (ProcessorImplementation) aadlColAPI.Processorsubcomponent_GetClassifier(subprocessor);
	ArrayList<PropertyAssociation> property_association_list1 = aadlColAPI.ComponentImpl_GetPropertyAssociations(p_impl);
	ProcessorType p_type = (ProcessorType) aadlColAPI.ComponentImpl_GetComponentType(p_impl);
	ArrayList<PropertyAssociation> property_association_list2 = aadlColAPI.Component_GetPropertyAssociations(p_type);
	property_association_list1.addAll(property_association_list2);
	if(property_association_list1.size()>0){
		for(int i=0; i<property_association_list1.size(); i++)
		{
			PropertyAssociation pa = property_association_list1.get(i);
		    ProcessCallImpl pcall1 =  ASME2SSME_Property_Call(pa);
	        myLib.STree_addExpressionOfProcessToaModel(p_property, pcall1);   
		}
	}
	
	if(myLib.STree_IsProcessNull(p_property)){
		myLib.STree_addNullProc(p_property);
	}
	
	return p_property;
}


        /**
         * Simulate dispatch: make process calls to simulate dispatch signals, 
         * and add it to the body of processor behavior
         * 
         * @param subp
         * @param p_behavior
         * @param sysimpl
         */
        public void ASME2SSME_Processor_SimDispatch(ProcessorSubcomponent subp, ProcessModelImpl p_behavior, SystemImplementation sysimpl){
        	
        	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_SimDispatch : " + subp.getName());
        	
        	ASME2SSME_Processor_SimDispatch_Periodic(subp, p_behavior, sysimpl);
        	ASME2SSME_Processor_SimDispatch_Aperiodic(subp, p_behavior, sysimpl);
        	ASME2SSME_Processor_SimDispatch_Sporadic(subp, p_behavior, sysimpl);
        	ASME2SSME_Processor_SimDispatch_Timed(subp, p_behavior, sysimpl);
        	ASME2SSME_Processor_SimDispatch_Hybrid(subp, p_behavior, sysimpl);
        	ASME2SSME_Processor_SimDispatch_Background(subp, p_behavior, sysimpl);
        }

        
        
        /**
         * Makes a Sim_PeriodicDispatch Signal process, and adds it to the declaration part of processor behavior process, 
         * and make an instance
         * (xp_xt_ctl1.Dispatch, ... ) := SimDispatch_Periodic{p1, ... }(top) 
         * where: p1, ... are the period values coming from the original thread periods (value, unit), 
         * xp is a sub process name, xt is a sub thread name of xp
         * 
         * @param subp
         * @param p_behavior
         */
        public void ASME2SSME_Processor_SimDispatch_Periodic(ProcessorSubcomponent subp, ProcessModelImpl p_behavior, SystemImplementation sysimpl){
        	
        	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_SimDispatch_Periodic : " + subp.getName());
        	ASME2SSME_Simulate_Dispatch asme2ssme_simdisp = new ASME2SSME_Simulate_Dispatch();
        	
        	ArrayList<IntegerLiteral> plist = new ArrayList<IntegerLiteral>(); // thread period list
        	ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
        	lins.add(myLib.mkIdentifier("top"));    	
        	ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
        	
        	//get the processes which are executed on this processor
        	ArrayList<ProcessSubcomponent> list_process_subcomponent = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subp);
        	Iterator<?> it1 = list_process_subcomponent.iterator();
        	while (it1.hasNext()){
        	  	ProcessSubcomponent subpr = (ProcessSubcomponent) it1.next(); //sub process
        	  	String subpr_name = aadlColAPI.Subcomponent_GetName(subpr); //sub process name
        	  	ProcessImplementation pr_impl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(subpr); // process impl
        	  	
        	  	ArrayList<ThreadSubcomponent> list_subt = aadlColAPI.Process_GetSubcomponentsThread(pr_impl);
        	  	for(int i =0; i< list_subt.size(); i++){
        	  		ThreadSubcomponent subt = list_subt.get(i); // sub thread
        	  		String subt_name = aadlColAPI.Subcomponent_GetName(subt); // sub thread name
        	  		ThreadImplementation t_impl = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier(subt);
        	  		
        	  		//Simulate dispatch only for the periodic threads
        	  		String dispatchp = aadlColAPI.Thread_Get_DispatchProtocol_str(t_impl);      	  		
        	  		if(dispatchp.equals("Periodic")){
        	  			IntegerLiteral period = aadlColAPI.Thread_GetPeriod(t_impl);
            	  		Trace(" \n ==== Thread " + subt_name +" period : " + aadlColAPI.IntegerLiteral_Get_Value(period) + aadlColAPI.IntegerLiteral_Get_Unit(period));
            	  		plist.add(period);
            	  		
            	  		String out_name = subpr_name + "_" + subt_name + "_ctl1"; // xx_ctl1
            	  		FieldAccessImpl out1 = myLib.STree_mkFieldAccess(out_name, "Dispatch"); //xx_ctl1.Dispatch
            	  		louts.add(out1);
        	  		}
        	  	}  
        	  }
        	     	
        	Integer n = plist.size();
        	
        	if(n>0){
        		//make a Signal process Sim_Dispatch
        		ProcessModelImpl vprocess = asme2ssme_simdisp.ASME2SSME_mkSimDispatch_Periodic(n);
            	myLib.STree_addProcessDeclaration(p_behavior, vprocess);
            	      	
            	//make an instance
            	SignalsDefinitionImpl eq = asme2ssme_simdisp.ASME2SSME_Call_SimDispatch_Periodic(plist, lins, louts);
            	myLib.STree_addExpressionOfProcessToaModel(p_behavior, eq);   
        	}
        }
        
        
        /**
         * Makes a Sim_Dispatch_Aeriodic Signal process, and adds it to the declaration part of processor behavior process, 
         * and make an instance: (xp_xt_ctl1.Dispatch, ... ) := Sim_Dispatch_Aeriodic{}(top, ...)
         * 
         * @param subp
         * @param p_behavior
         */
        public void ASME2SSME_Processor_SimDispatch_Aperiodic(ProcessorSubcomponent subp, ProcessModelImpl p_behavior, SystemImplementation sysimpl){
        	
        	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_SimDispatch_Aperiodic : " + subp.getName());
        	ASME2SSME_Simulate_Dispatch asme2ssme_simdisp = new ASME2SSME_Simulate_Dispatch();
        	
        	if( asme2ssme_simdisp.ASME2SSME_mkSimDispatch_Aperiodic(subp, p_behavior, sysimpl) != null){
        		//make a Signal process Sim_Dispatch_Aeriodic
        		ProcessModelImpl vprocess = asme2ssme_simdisp.ASME2SSME_mkSimDispatch_Aperiodic(subp, p_behavior, sysimpl);
            	myLib.STree_addProcessDeclaration(p_behavior, vprocess);
            	      	
            	//make an instance            	
            	ArrayList<ExprParam> lps = new ArrayList<ExprParam>();
        		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>(); 
        		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
        		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) vprocess.getInterfaceDefinition();
        		IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
        		EList<TypesOrSignalsDeclaration> list1 = vioProfile.getInputs();
        		for(int m =0; m< list1.size(); m++)
		     	 {
		    	   SignalsDeclarationImpl a1 = (SignalsDeclarationImpl) list1.get(m); 
		    	   EList<SignalNaming> list2 = a1.getListSignal();
		    	   for (int n = 0; n < list2.size(); n++) {  
			    		   SignalNaming sig_name2 = list2.get(n);
			    		   Identifier id_1 = sig_name2.getIdentifier();
			    		   IdentifierImpl id_2 = myLib.mkIdentifier(id_1.getAtomString());
			    		   lins.add(id_2);
		    	   }
		     	 }
        		EList<TypesOrSignalsDeclaration> list3 = vioProfile.getOutputs();
        		for(int i =0; i< list3.size(); i++)
		     	 {
		    	   SignalsDeclarationImpl a1 = (SignalsDeclarationImpl) list3.get(i); 
		    	   EList<SignalNaming> list4 = a1.getListSignal();
		    	   for (int j = 0; j < list4.size(); j++) {  
			    		   SignalNaming sig_name2 = list4.get(j);
			    		   Identifier id_1 = sig_name2.getIdentifier();
			    		   String out_name = id_1.getAtomString().replace("_dispatch", "_ctl1") ; // xx_ctl1
	            	  	   FieldAccessImpl out1 = myLib.STree_mkFieldAccess(out_name, "Dispatch"); //xx_ctl1.Dispatch
			    		   louts.add(out1);
		    	   }
		     	 }
        		
            	SignalsDefinitionImpl eq = asme2ssme_simdisp.ASME2SSME_Call_SimDispatch_Aperiodic(lps, lins, louts);           	  
            	myLib.STree_addExpressionOfProcessToaModel(p_behavior, eq);   
        	}
        }
        
        
        /**
         * Makes a Sim_Dispatch_Sporadic Signal process, and adds it to the declaration part of processor behavior process, 
         * and make an instance: (xp_xt_ctl1.Dispatch, ... ) := Sim_Dispatch_Sporadic{}(top, ...)
         * 
         * @param subp
         * @param p_behavior
         */
        public void ASME2SSME_Processor_SimDispatch_Sporadic(ProcessorSubcomponent subp, ProcessModelImpl p_behavior, SystemImplementation sysimpl){
        	
        	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_SimDispatch_Sporadic : " + subp.getName());
        	ASME2SSME_Simulate_Dispatch asme2ssme_simdisp = new ASME2SSME_Simulate_Dispatch();
        	
        	if( asme2ssme_simdisp.ASME2SSME_mkSimDispatch_Sporadic(subp, p_behavior, sysimpl) != null){
        		//make a Signal process Sim_Dispatch_Aeriodic
        		ProcessModelImpl vprocess = asme2ssme_simdisp.ASME2SSME_mkSimDispatch_Sporadic(subp, p_behavior, sysimpl);
            	myLib.STree_addProcessDeclaration(p_behavior, vprocess);
            	      	
            	//make an instance
            	ArrayList<ExprParam> lps = new ArrayList<ExprParam>();
        		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>(); 
        		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
        		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) vprocess.getInterfaceDefinition();
        		IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
        		EList<TypesOrSignalsDeclaration> list1 = vioProfile.getInputs();
        		for(int m =0; m< list1.size(); m++)
		     	 {
		    	   SignalsDeclarationImpl a1 = (SignalsDeclarationImpl) list1.get(m); 
		    	   EList<SignalNaming> list2 = a1.getListSignal();
		    	   for (int n = 0; n < list2.size(); n++) {  
			    		   SignalNaming sig_name2 = list2.get(n);
			    		   Identifier id_1 = sig_name2.getIdentifier();
			    		   IdentifierImpl id_2 = myLib.mkIdentifier(id_1.getAtomString());
			    		   lins.add(id_2);
		    	   }
		     	 }
        		EList<TypesOrSignalsDeclaration> list3 = vioProfile.getOutputs();
        		for(int i =0; i< list3.size(); i++)
		     	 {
		    	   SignalsDeclarationImpl a1 = (SignalsDeclarationImpl) list3.get(i); 
		    	   EList<SignalNaming> list4 = a1.getListSignal();
		    	   for (int j = 0; j < list4.size(); j++) {  
			    		   SignalNaming sig_name2 = list4.get(j);
			    		   Identifier id_1 = sig_name2.getIdentifier();
			    		   String out_name = id_1.getAtomString().replace("_dispatch", "_ctl1") ; // xx_ctl1
	            	  	   FieldAccessImpl out1 = myLib.STree_mkFieldAccess(out_name, "Dispatch"); //xx_ctl1.Dispatch
			    		   louts.add(out1);
		    	   }
		     	 }
        		
            	SignalsDefinitionImpl eq = asme2ssme_simdisp.ASME2SSME_Call_SimDispatch_Sporadic(lps, lins, louts);           	  
            	myLib.STree_addExpressionOfProcessToaModel(p_behavior, eq);   
        	}
        }
        
        
        
        /**
         * Makes a Sim_Dispatch_Timed Signal process, and adds it to the declaration part of processor behavior process, 
         * and make an instance: (xp_xt_ctl1.Dispatch, ... ) := Sim_Dispatch_Timed{}(top, ...)
         * 
         * @param subp
         * @param p_behavior
         */
        public void ASME2SSME_Processor_SimDispatch_Timed(ProcessorSubcomponent subp, ProcessModelImpl p_behavior, SystemImplementation sysimpl){
        	
        	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_SimDispatch_Timed : " + subp.getName());
        	ASME2SSME_Simulate_Dispatch asme2ssme_simdisp = new ASME2SSME_Simulate_Dispatch();
        	
        	if( asme2ssme_simdisp.ASME2SSME_mkSimDispatch_Timed(subp, p_behavior, sysimpl) != null){
        		//make a Signal process Sim_Dispatch_Aeriodic
        		ProcessModelImpl vprocess = asme2ssme_simdisp.ASME2SSME_mkSimDispatch_Timed(subp, p_behavior, sysimpl);
            	myLib.STree_addProcessDeclaration(p_behavior, vprocess);
            	      	
            	//make an instance
            	ArrayList<ExprParam> lps = new ArrayList<ExprParam>();
        		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>(); 
        		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
        		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) vprocess.getInterfaceDefinition();
        		IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
        		EList<TypesOrSignalsDeclaration> list1 = vioProfile.getInputs();
        		for(int m =0; m< list1.size(); m++)
		     	 {
		    	   SignalsDeclarationImpl a1 = (SignalsDeclarationImpl) list1.get(m); 
		    	   EList<SignalNaming> list2 = a1.getListSignal();
		    	   for (int n = 0; n < list2.size(); n++) {  
			    		   SignalNaming sig_name2 = list2.get(n);
			    		   Identifier id_1 = sig_name2.getIdentifier();
			    		   IdentifierImpl id_2 = myLib.mkIdentifier(id_1.getAtomString());
			    		   lins.add(id_2);
		    	   }
		     	 }
        		EList<TypesOrSignalsDeclaration> list3 = vioProfile.getOutputs();
        		for(int i =0; i< list3.size(); i++)
		     	 {
		    	   SignalsDeclarationImpl a1 = (SignalsDeclarationImpl) list3.get(i); 
		    	   EList<SignalNaming> list4 = a1.getListSignal();
		    	   for (int j = 0; j < list4.size(); j++) {  
			    		   SignalNaming sig_name2 = list4.get(j);
			    		   Identifier id_1 = sig_name2.getIdentifier();
			    		   String out_name = id_1.getAtomString().replace("_dispatch", "_ctl1") ; // xx_ctl1
	            	  	   FieldAccessImpl out1 = myLib.STree_mkFieldAccess(out_name, "Dispatch"); //xx_ctl1.Dispatch
			    		   louts.add(out1);
		    	   }
		     	 }
        		
            	SignalsDefinitionImpl eq = asme2ssme_simdisp.ASME2SSME_Call_SimDispatch_Timed(lps, lins, louts);           	  
            	myLib.STree_addExpressionOfProcessToaModel(p_behavior, eq);   
        	}
        }
        
        
        /**
         * Makes a Sim_Dispatch_Hybrid Signal process, and adds it to the declaration part of processor behavior process, 
         * and make an instance: (xp_xt_ctl1.Dispatch, ... ) := Sim_Dispatch_Hybrid{}(top, ...)
         * 
         * @param subp
         * @param p_behavior
         */
        public void ASME2SSME_Processor_SimDispatch_Hybrid(ProcessorSubcomponent subp, ProcessModelImpl p_behavior, SystemImplementation sysimpl){
        	
        	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_SimDispatch_Hybrid : " + subp.getName());
        	ASME2SSME_Simulate_Dispatch asme2ssme_simdisp = new ASME2SSME_Simulate_Dispatch();
        	
        	if( asme2ssme_simdisp.ASME2SSME_mkSimDispatch_Hybrid(subp, p_behavior, sysimpl) != null){
        		//make a Signal process Sim_Dispatch_Aeriodic
        		ProcessModelImpl vprocess = asme2ssme_simdisp.ASME2SSME_mkSimDispatch_Hybrid(subp, p_behavior, sysimpl);
            	myLib.STree_addProcessDeclaration(p_behavior, vprocess);
            	      	
            	//make an instance
            	ArrayList<ExprParam> lps = new ArrayList<ExprParam>();
        		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>(); 
        		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
        		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) vprocess.getInterfaceDefinition();
        		IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
        		EList<TypesOrSignalsDeclaration> list1 = vioProfile.getInputs();
        		for(int m =0; m< list1.size(); m++)
		     	 {
		    	   SignalsDeclarationImpl a1 = (SignalsDeclarationImpl) list1.get(m); 
		    	   EList<SignalNaming> list2 = a1.getListSignal();
		    	   for (int n = 0; n < list2.size(); n++) {  
			    		   SignalNaming sig_name2 = list2.get(n);
			    		   Identifier id_1 = sig_name2.getIdentifier();
			    		   IdentifierImpl id_2 = myLib.mkIdentifier(id_1.getAtomString());
			    		   lins.add(id_2);
		    	   }
		     	 }
        		EList<TypesOrSignalsDeclaration> list3 = vioProfile.getOutputs();
        		for(int i =0; i< list3.size(); i++)
		     	 {
		    	   SignalsDeclarationImpl a1 = (SignalsDeclarationImpl) list3.get(i); 
		    	   EList<SignalNaming> list4 = a1.getListSignal();
		    	   for (int j = 0; j < list4.size(); j++) {  
			    		   SignalNaming sig_name2 = list4.get(j);
			    		   Identifier id_1 = sig_name2.getIdentifier();
			    		   String out_name = id_1.getAtomString().replace("_dispatch", "_ctl1") ; // xx_ctl1
	            	  	   FieldAccessImpl out1 = myLib.STree_mkFieldAccess(out_name, "Dispatch"); //xx_ctl1.Dispatch
			    		   louts.add(out1);
		    	   }
		     	 }
        		
            	SignalsDefinitionImpl eq = asme2ssme_simdisp.ASME2SSME_Call_SimDispatch_Hybrid(lps, lins, louts);           	  
            	myLib.STree_addExpressionOfProcessToaModel(p_behavior, eq);   
        	}
        }
        
        
        
        /**
         * Makes a Sim_Dispatch_Background Signal process, and adds it to the declaration part of processor behavior process, 
         * and make an instance: (xp_xt_ctl1.Dispatch, ... ) := Sim_Dispatch_Background{}(top, ...)
         * 
         * @param subp
         * @param p_behavior
         */
        public void ASME2SSME_Processor_SimDispatch_Background(ProcessorSubcomponent subp, ProcessModelImpl p_behavior, SystemImplementation sysimpl){
        	
        	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_SimDispatch_Background : " + subp.getName());
        	ASME2SSME_Simulate_Dispatch asme2ssme_simdisp = new ASME2SSME_Simulate_Dispatch();
        	
        	if( asme2ssme_simdisp.ASME2SSME_mkSimDispatch_Background(subp, p_behavior, sysimpl) != null){
        		//make a Signal process Sim_Dispatch_Aeriodic
        		ProcessModelImpl vprocess = asme2ssme_simdisp.ASME2SSME_mkSimDispatch_Background(subp, p_behavior, sysimpl);
            	myLib.STree_addProcessDeclaration(p_behavior, vprocess);
            	      	
            	//make an instance
            	ArrayList<ExprParam> lps = new ArrayList<ExprParam>();
        		ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>(); 
        		ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
        		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) vprocess.getInterfaceDefinition();
        		IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
        		EList<TypesOrSignalsDeclaration> list1 = vioProfile.getInputs();
        		for(int m =0; m< list1.size(); m++)
		     	 {
		    	   SignalsDeclarationImpl a1 = (SignalsDeclarationImpl) list1.get(m); 
		    	   EList<SignalNaming> list2 = a1.getListSignal();
		    	   for (int n = 0; n < list2.size(); n++) {  
			    		   SignalNaming sig_name2 = list2.get(n);
			    		   Identifier id_1 = sig_name2.getIdentifier();
			    		   IdentifierImpl id_2 = myLib.mkIdentifier(id_1.getAtomString());
			    		   lins.add(id_2);
		    	   }
		     	 }
        		EList<TypesOrSignalsDeclaration> list3 = vioProfile.getOutputs();
        		for(int i =0; i< list3.size(); i++)
		     	 {
		    	   SignalsDeclarationImpl a1 = (SignalsDeclarationImpl) list3.get(i); 
		    	   EList<SignalNaming> list4 = a1.getListSignal();
		    	   for (int j = 0; j < list4.size(); j++) {  
			    		   SignalNaming sig_name2 = list4.get(j);
			    		   Identifier id_1 = sig_name2.getIdentifier();
			    		   String out_name = id_1.getAtomString().replace("_dispatch", "_ctl1") ; // xx_ctl1
	            	  	   FieldAccessImpl out1 = myLib.STree_mkFieldAccess(out_name, "Dispatch"); //xx_ctl1.Dispatch
			    		   louts.add(out1);
		    	   }
		     	 }
        		
            	SignalsDefinitionImpl eq = asme2ssme_simdisp.ASME2SSME_Call_SimDispatch_Background(lps, lins, louts);           	  
            	myLib.STree_addExpressionOfProcessToaModel(p_behavior, eq);   
        	}
        }
        
        
        
        /**
         * Makes a Sim_Deadline Signal process, and adds it to the declaration part of processor behavior process, 
         * and make an instance of Sim_Deadline
         * (xp_xt_ctl1.Deadline, ... ) := Sim_Deadline(top, xp_xt_ctl1.Dispatch, ...) 
         * where: xp is a sub process name, xt is a sub thread name of xp
         * 
         * @param subp
         * @param p_behavior
         */
        public void ASME2SSME_Processor_SimDeadline(ProcessorSubcomponent subp, ProcessModelImpl p_behavior, SystemImplementation sysimpl){
        	
        	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_SimDeadline : " + subp.getName());
        	ASME2SSME_Simulate_Deadline asme2ssme_sim_deadline = new ASME2SSME_Simulate_Deadline();
        	
        	ArrayList<ExprParam> lparams = new ArrayList<ExprParam>(); 
        	ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
        	lins.add(myLib.mkIdentifier("top"));    	
        	ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
        	
        	//get the processes which are executed on this processor
        	ArrayList<ProcessSubcomponent> list1 = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subp);
        	Iterator<?> it1 = list1.iterator();
        	while (it1.hasNext()){
        	  	ProcessSubcomponent subpr = (ProcessSubcomponent) it1.next(); //sub process
        	  	String subpr_name = aadlColAPI.Subcomponent_GetName(subpr); //sub process name
        	  	ProcessImplementation pr_impl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(subpr); // process impl
        	  	
        	  	ArrayList<ThreadSubcomponent> list_subt = aadlColAPI.Process_GetSubcomponentsThread(pr_impl);
        	  	for(int i =0; i< list_subt.size(); i++){
        	  		ThreadSubcomponent subt = list_subt.get(i); // sub thread
        	  		String subt_name = aadlColAPI.Subcomponent_GetName(subt); // sub thread name
        	  		String out_name = subpr_name + "_" + subt_name + "_ctl1";  
            	  	FieldAccessImpl in1 = myLib.STree_mkFieldAccess(out_name, "Dispatch");  
            	  	lins.add(in1);
            	  	
            	  	FieldAccessImpl out1 = myLib.STree_mkFieldAccess(out_name, "Deadline");  
            	  	louts.add(out1);
        	  	}  
        	  }
        	     	
        	Integer n = lins.size();
        	
        	if(n>0){
        		//make a Signal process Sim_Deadline
        		ProcessModelImpl vprocess = asme2ssme_sim_deadline.ASME2SSME_mkSimDeadline(subp, list1);
            	myLib.STree_addProcessDeclaration(p_behavior, vprocess);
            	      	
            	//make an instance
            	SignalsDefinitionImpl eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("Sim_Deadline"), lparams, lins, louts);
            	myLib.STree_addExpressionOfProcessToaModel(p_behavior, eq);   
        	}
        }
        
        
        /**
         * Makes a Sim_Resume Signal process, and adds it to the declaration part of processor behavior process, 
         * and make an instance of Sim_Resume
         * (xp_xt_ctl1.Resume, ... ) := Sim_Resume(top, xp_xt_ctl1.Dispatch, ...) 
         * where: xp is a sub process name, xt is a sub thread name of xp
         * 
         * @param subp
         * @param p_behavior
         */
        public void ASME2SSME_Processor_SimResume(ProcessorSubcomponent subp, ProcessModelImpl p_behavior, SystemImplementation sysimpl){
        	
        	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_SimResume : " + subp.getName());
        	ASME2SSME_Simulate_Resume asme2ssme_sim_resume = new ASME2SSME_Simulate_Resume();
        	
        	ArrayList<ExprParam> lparams = new ArrayList<ExprParam>(); 
        	ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
        	lins.add(myLib.mkIdentifier("top"));    	
        	ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
        	
        	//get the processes which are executed on this processor
        	ArrayList<ProcessSubcomponent> list1 = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subp);
        	Iterator<?> it1 = list1.iterator();
        	while (it1.hasNext()){
        	  	ProcessSubcomponent subpr = (ProcessSubcomponent) it1.next(); //sub process
        	  	String subpr_name = aadlColAPI.Subcomponent_GetName(subpr); //sub process name
        	  	ProcessImplementation pr_impl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(subpr); // process impl
        	  	
        	  	ArrayList<ThreadSubcomponent> list_subt = aadlColAPI.Process_GetSubcomponentsThread(pr_impl);
        	  	for(int i =0; i< list_subt.size(); i++){
        	  		ThreadSubcomponent subt = list_subt.get(i); // sub thread
        	  		String subt_name = aadlColAPI.Subcomponent_GetName(subt); // sub thread name
        	  		String out_name = subpr_name + "_" + subt_name + "_ctl1"; // xx_ctl1
            	  	FieldAccessImpl in1 = myLib.STree_mkFieldAccess(out_name, "Dispatch"); //xx_ctl1.Dispatch
            	  	lins.add(in1);
            	  	
            	  	FieldAccessImpl out1 = myLib.STree_mkFieldAccess(out_name, "Resume"); //xx_ctl1.Deadline
            	  	louts.add(out1);
        	  	}  
        	  }
        	     	
        	Integer n = lins.size();
        	
        	if(n>0){
        		//make a Signal process Sim_Deadline
        		ProcessModelImpl vprocess = asme2ssme_sim_resume.ASME2SSME_mkSimResume(subp, list1);
            	myLib.STree_addProcessDeclaration(p_behavior, vprocess);
            	      	
            	//make an instance
            	SignalsDefinitionImpl eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("Sim_Resume"), lparams, lins, louts);
            	myLib.STree_addExpressionOfProcessToaModel(p_behavior, eq);   
        	}
        }
        
        
      
        /**
         * Makes a Sim_Initialize Signal process, and adds it to the declaration part of processor behavior process, 
         * and make an instance of Sim_Deadline
         * (xp_xt_ctl1.Initialize, ... ) := Sim_Initialize(top, xp_xt_ctl1.Dispatch, ...) 
         * where: xp is a sub process name, xt is a sub thread name of xp
         * 
         * @param subp
         * @param p_behavior
         */
        public void ASME2SSME_Processor_SimInitialize(ProcessorSubcomponent subp, ProcessModelImpl p_behavior, SystemImplementation sysimpl){
        	
        	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_SimInitialize : " + subp.getName());
        	ASME2SSME_Simulate_Initialize asme2ssme_sim_init = new ASME2SSME_Simulate_Initialize();
        	
        	ArrayList<ExprParam> lparams = new ArrayList<ExprParam>(); 
        	ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
        	lins.add(myLib.mkIdentifier("top"));    	
        	ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
        	
        	//get the processes which are executed on this processor
        	ArrayList<ProcessSubcomponent> list1 = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subp);
        	Iterator<?> it1 = list1.iterator();
        	while (it1.hasNext()){
        	  	ProcessSubcomponent subpr = (ProcessSubcomponent) it1.next(); //sub process
        	  	String subpr_name = aadlColAPI.Subcomponent_GetName(subpr); //sub process name
        	  	ProcessImplementation pr_impl = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(subpr); // process impl
        	  	
        	  	ArrayList<ThreadSubcomponent> list_subt = aadlColAPI.Process_GetSubcomponentsThread(pr_impl);
        	  	for(int i =0; i< list_subt.size(); i++){
        	  		ThreadSubcomponent subt = list_subt.get(i); // sub thread
        	  		String subt_name = aadlColAPI.Subcomponent_GetName(subt); // sub thread name
        	  		String out_name = subpr_name + "_" + subt_name + "_ctl1"; // xx_ctl1
            	  	FieldAccessImpl in1 = myLib.STree_mkFieldAccess(out_name, "Dispatch"); //xx_ctl1.Dispatch
            	  	lins.add(in1);
            	  	
            	  	FieldAccessImpl out1 = myLib.STree_mkFieldAccess(out_name, "Initialize"); //xx_ctl1.Initialize
            	  	louts.add(out1);
        	  	}  
        	  }
        	     	
        	Integer n = lins.size();
        	
        	if(n>0){
        		//make a Signal process Sim_Initialize
        		ProcessModelImpl vprocess = asme2ssme_sim_init.ASME2SSME_mkSimInitialize(subp, list1);
            	myLib.STree_addProcessDeclaration(p_behavior, vprocess);
            	      	
            	//make an instance
            	SignalsDefinitionImpl eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier("Sim_Initialize"), lparams, lins, louts);
            	myLib.STree_addExpressionOfProcessToaModel(p_behavior, eq);   
        	}
        }
        
        
        
        /**
         * Makes a Sim_TimeEvent Signal process, and adds it to the declaration part of processor behavior process, 
         * and make an instance of Sim_TimeEvent: Sim_TimeEvent{}, 
         * suppose that the formal parameters and the effective parameters have the same name (to simplifier the code) 
         * 
         * @param subp
         * @param p_behavior
         */
        public void ASME2SSME_Processor_SimTimeEvent(ProcessorSubcomponent subp, ProcessModelImpl p_behavior, SystemImplementation sysimpl){
        	
        	Trace("\n ====== ASME2SSME_Translate_Processor: ASME2SSME_Processor_SimTimeEvent : " + subp.getName());
        	ASME2SSME_Simulate_TimeEvent asme2ssme_sim_timeevent = new ASME2SSME_Simulate_TimeEvent();
        	
        	//get the processes which are executed on this processor
        	ArrayList<ProcessSubcomponent> list1 = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subp);
        	
        	//make a Signal process Sim_TimeEvent
    		ProcessModelImpl vprocess = asme2ssme_sim_timeevent.ASME2SSME_mkSimTimeEvent(subp, list1);
        	myLib.STree_addProcessDeclaration(p_behavior, vprocess);
        	      	
        	//make an instance
        	ArrayList<ExprParam> lparams = new ArrayList<ExprParam>(); 
          	ProcessExpansionImpl inst = myLib.STree_mkProcessCallWithParameters(myLib.mkIdentifier("Sim_TimeEvent"), lparams);
            myLib.STree_addExpressionOfProcessToaModel(p_behavior, inst);
        }        
}
