 

package fr.inria.tea.aadl2ssme.transformation.popup.actions;


import java.util.ArrayList;

import org.osate.aadl2.*;
import org.eclipse.pop.ssme.Identifier;
import org.eclipse.pop.ssme.impl.ListModuleImpl;
import org.eclipse.pop.ssme.impl.ModuleImpl;

public class ASME2SSME_Translate_Spec extends ASME2SSME_Translate_Common
{
	private signalTreeAPI myLib;
	private	AADLCoLAPI aadlColAPI;

	public ASME2SSME_Translate_Spec() {
		myLib= new signalTreeAPI();
		aadlColAPI = new AADLCoLAPI();
	}

	/**
	 * Returns a ListModuleImpl object (a module) that contains all the components specified in an AADL specification
	 * 
     @param aadlRootModel an AADL specification 
	 *
     @return the ListModuleImpl (lmodule) corresponding to the AADL specification
	 */
	public ListModuleImpl ASME2SSME_AadlSpec_package(AadlPackage aadlRootModel){

		String spec_name = aadlColAPI.AadlPackage_GetName(aadlRootModel);			
		Trace("\n **** ASME2SSME_Translate_AADLSpec: ASME2SSME_AadlSpec_package " + spec_name);
		
		ListModuleImpl lmodule = mkSSMERootObject(spec_name);
		ModuleImpl vmodule= mkSSMEModule(spec_name);
		myLib.STree_addModuleToRootModule(lmodule, vmodule);	
		ASME2SSME_Translate_Package asme2ssme_aadlpackage = new ASME2SSME_Translate_Package();
		asme2ssme_aadlpackage.ASME2SSME_AADLPackage(aadlRootModel, vmodule);
		
		Trace("\n ************************* end ASME2SSME_AadlSpec_package ************************* \n ");
		return lmodule;					
	}

	/**
	 * Returns a ListModuleImpl object (a module) that contains all the components specified in an AADL specification
     @param aadlRootModel an AADL specification 
     @return the ListModuleImpl (lmodule) corresponding to the AADL specification
	 */
	public ListModuleImpl ASME2SSME_AadlSpec_PropertySet(PropertySet aadlRootModel){
		String spec_name = aadlColAPI.PropertySet_GetName(aadlRootModel);			
		Trace("\n **** ASME2SSME_Translate_AADLSpec: ASME2SSME_AadlSpec_package " + spec_name);

		ListModuleImpl lmodule = mkSSMERootObject(spec_name);
		ModuleImpl vmodule= mkSSMEModule(spec_name);
		myLib.STree_addModuleToRootModule(lmodule, vmodule);
		ASME2SSME_Translate_PropertySet asme2ssme_propertyset = new ASME2SSME_Translate_PropertySet();
		asme2ssme_propertyset.ASME2SSME_PropertySet(aadlRootModel, vmodule);

		Trace("\n ************************* end ASME2SSME_AadlSpec_package ************************* \n ");
		return lmodule;					
	}

	/**
	 * @return a skeleton of a module named spec_name but with "use ASME2SSME_LIB" in the local declaration.
	 * ASME2SSME_LIB is a predefined Signal library provided with Polychrony/POP tool.
	 * @param spec_name
	 * @return
	 */
	private ModuleImpl mkSSMEModule(String spec_name) {
		ModuleImpl vmodule = myLib.STree_mkModule(myLib.mkIdentifier(spec_name));
		myLib.STree_adduseModuleToAModule(vmodule, (Identifier) myLib.mkIdentifier("ASME2SSME_LIB"));
		return vmodule;
	}

	/**
	 * @return the SSME object associated with a aadl root object named spec_name.
	 * This object will contains all the components specified in an AADL specification
	 */
	private ListModuleImpl mkSSMERootObject(String spec_name) {
		ListModuleImpl lmodule = myLib.STree_mkSkeletonModule();
		addHeaderComment(lmodule, spec_name); 
		return lmodule;
	}

	/**
	 * Add the header comment to the SSME list of modules (lmodule); the comment is parametized by spec_name.
	 */
	private void addHeaderComment(ListModuleImpl lmodule, String spec_name) {
		ArrayList<String> comments = new ArrayList<String>();
		comments.add(" This file has been generated by ASME2SSME translator version " + ASME2SSME.ASME2SSME_Version);
		comments.add(" Copyright : IRISA/INRIA Rennes - developed by ESPRESSO/TEA team.");
		comments.add(" Source file: " + spec_name +".aadl2");
		comments.add(" Date " + GetDateNow() + " ");
		myLib.STree_addComment(lmodule, comments, true); 
	}

}
