
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;

import org.osate.aadl2.*;
import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.Type;
import org.eclipse.pop.ssme.impl.*;

public class ASME2SSME_Translate_Subprogram extends ASME2SSME_Translate_Property
{
	private AADLCoLAPI aadlColAPI;
	private signalTreeAPI myLib;
	private ASME2SSME_Translate_Common asme2ssme_common;

	/*
	 * 	  Class constructor
	 */
	public ASME2SSME_Translate_Subprogram ()
	{
		aadlColAPI = new AADLCoLAPI ();
		myLib = new signalTreeAPI ();
		asme2ssme_common = new ASME2SSME_Translate_Common ();
	}

	/**
	 * Returns a Signal process that represents a subprogram type component
	 * <p>
	 * An in event "call" is added to give the clock when the subprogram is
	 * called. Gets the out event or event data ports of this subprogram type,
	 * and translates them as the output of the interface Gets the in/out
	 * parameters, and translates them as the input/output
	 *
	 * @param subp_type
	 *            a subprogram type component
	 * @return a Signal process
	 */
	public ProcessModelImpl ASME2SSME_SubprogramType (SubprogramType subp_type, ModuleImpl vmodule)
	{

		Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_SubprogramType : " + subp_type.getName ());
		String sname = aadlColAPI.ComponentType_GetName (subp_type);
		String pname = Name_of_Subprogram (sname);

		// process skeleton
		ProcessModelImpl dprocess = myLib.STree_mkSkeletonModel (myLib.mkIdentifier (pname) );

		// process interface
		ASME2SSME_Subprogram_Interface (dprocess, subp_type, vmodule);

		// add clock for requires data access
		asme2ssme_common.ASME2SSME_ComponentInterface_DataAccess (dprocess, subp_type);
		// add clock for requires subprogram access: ! event x;
		asme2ssme_common.ASME2SSME_ComponentInterface_SubprogramAccesses (dprocess, subp_type);

		// Features
		ASME2SSME_SubprogramType_Features (subp_type, dprocess);

		// Flow specifications
		ASME2SSME_SubprogramType_FlowSpecs (dprocess, subp_type);

		// Modes
		ASME2SSME_SubprogramType_Modes (dprocess, subp_type);

		// Properties
		ASME2SSME_SubprogramType_Properties (dprocess, subp_type);

		if (myLib.STree_IsProcessNull (dprocess) && !myLib.STree_IsProcessHasOutput (dprocess))
			myLib.STree_addNullProc (dprocess);
		else
		{
			// external process
			DescriptionImpl desc = (DescriptionImpl) dprocess.getDescription ();
			if (desc == null)
			{
				dprocess.setDescription (new DescriptionImpl ());
				desc = (DescriptionImpl) dprocess.getDescription ();
			}
			ExternalAttributeImpl externe = new ExternalAttributeImpl ();
			desc.setBody (externe);
		}
		return dprocess;
	}

	/**
	 * Returns a Signal process that represents a device implementation
	 * component.
	 * <p>
	 * This Signal process is named by a method Name_of_Subprogram(). An empty
	 * skeleton process is provided.
	 *
	 * @param d
	 *            a subprogram implementation component
	 * @return a Signal process
	 */
	public ProcessModelImpl ASME2SSME_SubprogramImpl (SubprogramImplementation subp_impl, ModuleImpl vmodule)
	{

		Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_SubprogramImpl : " + subp_impl.getName ());

		String sname = aadlColAPI.ComponentImpl_GetName (subp_impl);
		String pname = Name_of_Subprogram (sname);

		// subprogram skeleton
		ProcessModelImpl subp_process = myLib.STree_mkSkeletonModel (myLib.mkIdentifier (pname) );

		// subprogram process interface
		SubprogramType subp_type = (SubprogramType) aadlColAPI.ComponentImpl_GetComponentType (subp_impl);
		ASME2SSME_Subprogram_Interface (subp_process, subp_type, vmodule);

		// interface for requires subprograms, calls subprograms
		// add clock for requires data access
		asme2ssme_common.ASME2SSME_ComponentInterface_DataAccess (subp_process, subp_type);
		// add clock for requires subprogram access: ! event x;
		asme2ssme_common.ASME2SSME_ComponentInterface_SubprogramAccesses (subp_process, subp_type);
		// add clock for calls subprogram: ! event x;
		asme2ssme_common.ASME2SSME_ComponentInterface_CallSubprograms (subp_process, subp_impl);

		// Features
		ASME2SSME_SubprogramType_Features (subp_type, subp_process);

		// subprogram behavior 
		ProcessModelImpl subp_behavior = ASME2SSME_Subprogram_behavior (subp_impl, vmodule);
		String subp_behavior_name = myLib.STree_GetStringName (subp_behavior.getIdentifier ());
		// add to declaration
		myLib.STree_addProcessDeclaration (subp_process, subp_behavior);
		// add to body
		// ProcessCallImpl subp_behavior_inst =
		// myLib.STree_mkModelCall(subp_behavior_name);
		ArrayList < ExprParam > lparam = new ArrayList < ExprParam > ();
		ProcessExpansionImpl subp_behavior_inst = myLib.STree_mkProcessCallWithParameters (myLib.mkIdentifier (subp_behavior_name), lparam);
		myLib.STree_addExpressionOfProcessToaModel (subp_process, subp_behavior_inst);

		// subprogram property 
		ProcessModelImpl subp_property = ASME2SSME_Subprogram_property (subp_impl);
		String subp_property_name = myLib.STree_GetStringName (subp_property.getIdentifier ());
		// add to declaration
		myLib.STree_addProcessDeclaration (subp_process, subp_property);
		// add to body
		// ProcessCallImpl subp_property_inst =
		// myLib.STree_mkModelCall(subp_property_name);
		ArrayList < ExprParam > lparam2 = new ArrayList < ExprParam > ();
		ProcessExpansionImpl subp_property_inst = myLib.STree_mkProcessCallWithParameters (myLib.mkIdentifier (subp_property_name), lparam2);
		myLib.STree_addExpressionOfProcessToaModel (subp_process, subp_property_inst);

		// Subcomponents 
		ASME2SSME_SubprogramImpl_SubData (subp_process, subp_impl);

		// subprogram calls (call sequences) 
		ASME2SSME_SubprogramImpl_CallSequences (subp_impl);

		// connections 
		ASME2SSME_SubprogramImpl_Connections (subp_impl);

		// flows 
		ASME2SSME_SubprogramImpl_Flows (subp_process, subp_impl);

		// modes 
		ASME2SSME_SubprogramImpl_Modes (subp_impl);

		// properties : subprogram impl properties are implemented in the SubprogramImpl_Property() sub process

		Trace ("Subprogram implementation not yet implemented");
		return subp_process;
	}

	/**
	 * Creates a Signal process interface from a subprogram type, and add the
	 * interface to the Signal process
	 * <p>
	 * Only the in parameters, out parameters and out event (event data) ports
	 * are translated in the interface
	 * <p>
	 * The in parameters are translated as an in struct: ? sp_IN x; sp_IN is
	 * added in the type declaration of the module. The out parameters and out
	 * event (event data) ports are translated as an out struct: ! sp_OUT y; An
	 * out TID id is added to identifier the result is to be sent to which
	 * thread.
	 *
	 * @param vprocess
	 *            a Signal process representing this subprogram (type or
	 *            implementation component)
	 * @param subp_type
	 *            a subprogram type component
	 * @param vmodule
	 *            a Signal module that the structs will be added into
	 *
	 * @return
	 */
	private void ASME2SSME_Subprogram_Interface (ProcessModelImpl vprocess, SubprogramType subp_type, ModuleImpl vmodule)
	{

		String sp_name = aadlColAPI.ComponentType_GetName (subp_type);
		// ASME2SSME_Translate_Feature asme2ssme_feature = new ASME2SSME_Translate_Feature ();

		Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_Subprogram_Interface " + sp_name);
		// SignalsDeclarationImpl vdecsig;
		// in event call
		// org.eclipse.pop.ssme.Type id2 = myLib.mkPredefinedType(SignalOperators.opevent);
		// SignalsDeclarationImpl call = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier("spcall"), id2, null);
		// myLib.STree_addDeclarationToInterface(vprocess, call, 2);

		ArrayList < EventPort > list_oute_port = aadlColAPI.Component_GetOutEventPort (subp_type);
		ArrayList < EventDataPort > list_outed_port = aadlColAPI.Component_GetOutEventDataPort (subp_type);
		ArrayList < Parameter > list_in_parameters = aadlColAPI.Subprogram_Get_InParameter (subp_type);
		ArrayList < Parameter > list_out_parameters = aadlColAPI.Subprogram_Get_OutParameter (subp_type);

		// ? sp_IN x;
		// create a struct type for the in parameters, and adds to interface:
		// xx_IN x
		TypesDeclarationImpl struct1 = ASME2SSME_mkSubprogram_StructIn (subp_type, list_in_parameters);
		// add comments
		ArrayList < String > comments = new ArrayList < String > ();
		comments.add ("struct for the in parameters of subprogram " + sp_name);
		myLib.STree_addComment (struct1, comments, true);
		// add the type in the module
		myLib.STree_addaTypeDeclarationToModule (vmodule, struct1);
		asme2ssme_common.ASME2SSME_add_input_to_process_interface ("x", Name_of_Subprogram_Struct_In (sp_name), vprocess);

		// ! sp_OUT y;
		// create a struct type for the out parameters and out event/event data
		// ports, and adds to interface: xx_OUT y
		TypesDeclarationImpl struct2 = ASME2SSME_mkSubprogram_StructOut (subp_type, list_out_parameters, list_oute_port, list_outed_port);
		// add comments
		ArrayList < String > comments2 = new ArrayList < String > ();
		String str2 = "struct for the out parameters and out event (event data) ports of subprogram " + sp_name;
		comments2.add (str2);
		myLib.STree_addComment (struct2, comments2, true);
		// add the type in the module
		myLib.STree_addaTypeDeclarationToModule (vmodule, struct2);
		asme2ssme_common.ASME2SSME_add_output_to_process_interface ("y", Name_of_Subprogram_Struct_Out (sp_name), vprocess);

		// ! TID id;
		SignalsDeclarationImpl sp_id = myLib.STree_mkSignalDeclaration (myLib.mkIdentifier ("id"), myLib.mkIdentifier ("TID"), null);
		myLib.STree_addDeclarationToInterface (vprocess, sp_id, 3);

		// //feature: out event port
		// if (list_oute_port.size() >0 ){
		// for(int i=0; i< list_oute_port.size(); i++){
		// EventPort eventPort = list_oute_port.get(i);
		// vdecsig = asme2ssme_feature.ASME2SSME_EventPort(eventPort);
		// myLib.STree_addDeclarationToInterface(vprocess, vdecsig, 3);
		// }
		// }
		//
		// //feature: out event data port
		// if (list_outed_port.size() >0 ){
		// for(int j=0; j< list_outed_port.size(); j++){
		// EventDataPort eventdataPort = list_outed_port.get(j);
		// vdecsig = asme2ssme_feature.ASME2SSME_EventDataPort(eventdataPort);
		// myLib.STree_addDeclarationToInterface(vprocess, vdecsig, 3);
		// eventdataPort = null;
		// }
		// }
		//
		// //feature: in parameter
		// if (list_in_parameters.size() >0 ){
		// for(int k1 =0; k1< list_in_parameters.size(); k1++){
		// Parameter p1 = list_in_parameters.get(k1);
		// vdecsig = asme2ssme_feature.ASME2SSME_Parameter(p1);
		// myLib.STree_addDeclarationToInterface(vprocess, vdecsig, 2);
		// }
		// }
		//
		// //feature: out parameter
		// if (list_out_parameters.size() >0 ){
		// for(int k2 =0; k2<list_out_parameters.size(); k2++){
		// Parameter p2 = list_out_parameters.get(k2);
		// vdecsig = asme2ssme_feature.ASME2SSME_Parameter(p2);
		// myLib.STree_addDeclarationToInterface(vprocess, vdecsig, 3);
		// }
		// }

		// //feature: feature group
		// EList<FeatureGroup> fgroup_list = subp_type.getOwnedFeatureGroups();
		// if(fgroup_list != null){
		// ASME2SSME_Translate_FeatureGroup asme2ssme_fg = new
		// ASME2SSME_Translate_FeatureGroup();
		// for(int m=0; m<fgroup_list.size(); m++)
		// {
		// FeatureGroup fg = fgroup_list.get(m);
		// if(asme2ssme_fg.ASME2SSME_FeatureGroup_bw(fg) != null){
		// SignalsDeclarationImpl in1 =
		// asme2ssme_fg.ASME2SSME_FeatureGroup_bw(fg);
		// myLib.STree_addDeclarationToInterface(vprocess, in1, 2);
		// }
		//
		// if(asme2ssme_fg.ASME2SSME_FeatureGroup_wb(fg) != null){
		// SignalsDeclarationImpl out1 =
		// asme2ssme_fg.ASME2SSME_FeatureGroup_wb(fg);
		// myLib.STree_addDeclarationToInterface(vprocess, out1, 3);
		// }
		// }
		// }
		//
		// //feature: requires data access
		// //data access==============
		// EList<DataAccess> da_list = subp_type.getOwnedDataAccesses();
		// if(da_list.size()>0){
		// for(int i4=0; i4<da_list.size(); i4++)
		// {
		// DataAccess da1 = da_list.get(i4);
		// asme2ssme_feature.ASME2SSME_DataAccess(da1);
		// }
		// }

		// feature: require subprogram access

		// feature: requires subprogram group access: not implemented as interface
	}

	/**
	 * Returns a Signal process "xx_Subprogram_behavior" that represents the
	 * behavior of a subprogram component
	 * <p>
	 * This process name is "xxx_behavior", where xxx is this device's
	 * corresponding Signal process name. In this moment, only a skeleton
	 * process model is provided. The interface and body are empry.
	 *
	 * @param subp_impl
	 *            a subprogram implementation component
	 * @return a ProcessModelImpl object (a Signal process)
	 */
	private ProcessModelImpl ASME2SSME_Subprogram_behavior (SubprogramImplementation subp_impl, ModuleImpl vmodule)
	{

		Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_Subprogram_behavior : " + subp_impl.getName ());

		String sname = aadlColAPI.ComponentImpl_GetName (subp_impl);
		String bname = Name_of_Subprogram (sname) + "_Behavior";
		SubprogramType subp_type = (SubprogramType) aadlColAPI.ComponentImpl_GetComponentType (subp_impl);
		ProcessModelImpl subp_behavior = myLib.STree_mkSkeletonModel (myLib.mkIdentifier (bname) );

		// interface
		ASME2SSME_Subprogram_Interface (subp_behavior, subp_type, vmodule);
		// add clock for requires data access
		asme2ssme_common.ASME2SSME_ComponentInterface_DataAccess (subp_behavior, subp_type);
		// add clock for requires subprogram access: ! event x;
		asme2ssme_common.ASME2SSME_ComponentInterface_SubprogramAccesses (subp_behavior, subp_type);
		// add clock for calls subprogram: ! event x;
		asme2ssme_common.ASME2SSME_ComponentInterface_CallSubprograms (subp_behavior, subp_impl);

		// if(myLib.STree_IsProcessNull(subp_behavior) && (! myLib.STree_IsProcessHasOutput(subp_behavior)))
		// myLib.STree_addNullProc(subp_behavior);

		// external process
		DescriptionImpl desc = (DescriptionImpl) subp_behavior.getDescription ();
		if (desc == null)
		{
			subp_behavior.setDescription (new DescriptionImpl ());
			desc = (DescriptionImpl) subp_behavior.getDescription ();
		}
		ExternalAttributeImpl externe = new ExternalAttributeImpl ();
		desc.setBody (externe);

		return subp_behavior;
	}

	/**
	 * Returns a Signal process xx_Subprogram_property that represents the
	 * properties of a subprogram component.
	 * <p>
	 * This process is named as xxx_property, where xxx is the name of this
	 * subprogram component's corresponding Signal process. Gets all the
	 * properties specified in this subprogram implementation and also its type
	 * component. For each such property, ASME2SSME_Property_Call() method is
	 * used to make a process call.
	 *
	 * @param subp_impl
	 *            a subprogram implementation component
	 * @return a ProcessModelImpl (Signal process)
	 */
	private ProcessModelImpl ASME2SSME_Subprogram_property (SubprogramImplementation subp_impl)
	{

		Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_Subprogram_property : " + subp_impl.getName ());

		String sname = aadlColAPI.ComponentImpl_GetName (subp_impl);
		String pname = Name_of_Subprogram (sname) + "_Property";
		ProcessModelImpl p_property = myLib.STree_mkSkeletonModel (myLib.mkIdentifier (pname) );

		ArrayList < PropertyAssociation > property_association_list1 = aadlColAPI.ComponentImpl_GetPropertyAssociations (subp_impl);
		SubprogramType subp_type = (SubprogramType) aadlColAPI.ComponentImpl_GetComponentType (subp_impl);
		ArrayList < PropertyAssociation > property_association_list2 = aadlColAPI.Component_GetPropertyAssociations (subp_type);
		property_association_list1.addAll (property_association_list2);

		for (int i = 0; i < property_association_list1.size (); i++)
		{
			PropertyAssociation pa = property_association_list1.get (i);
			ProcessCallImpl pcall1 = ASME2SSME_Property_Call (pa);
			myLib.STree_addExpressionOfProcessToaModel (p_property, pcall1);
		}
		if (myLib.STree_IsProcessNull (p_property) && ! myLib.STree_IsProcessHasOutput (p_property))
			myLib.STree_addNullProc (p_property);
		return p_property;
	}

	/**
	 * Translates subprogram features
	 *
	 * @param sp
	 */
	private void ASME2SSME_SubprogramType_Features (SubprogramType sp, ProcessModelImpl dprocess)
	{
		Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_Subprogram_Features: " + sp.getName ());
		// ASME2SSME_Translate_Feature asme2ssme_feature = new ASME2SSME_Translate_Feature ();
		if (sp.getAllFeatures () != null)
		{

			// parameter 
			// out event port
			// out event data port
			// feature group
			// requires data access
			// implemented as a signal in the interface
			// requires subprogram access =================
			// requires subprogram group access====================
		}
	}

	/**
	 * Translates flow specifications of a subprogram type
	 *
	 * @param tt
	 *            a subprogram type component
	 */
	private void ASME2SSME_SubprogramType_FlowSpecs (ProcessModelImpl vprocess, SubprogramType tt)
	{
		ArrayList < FlowSpecification > fs_list = aadlColAPI.Component_GetFlowSpecification (tt);
		if (fs_list != null)
			if (fs_list.size () > 0)
			{
				Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_SubprogramType_FlowSpecs: " + tt.getName ());

				ProcessModelImpl p1 = asme2ssme_common.ASME2SSME_ComponentType_FlowSpecs (tt);
				myLib.STree_addProcessDeclaration (vprocess, p1);
				ArrayList < ExprParam > lparam = new ArrayList < ExprParam > ();
				String pname = Name_of_Flow (tt.getName ());
				ProcessExpansionImpl call1 = myLib.STree_mkProcessCallWithParameters (myLib.mkIdentifier (pname), lparam);
				myLib.STree_addExpressionOfProcessToaModel (vprocess, call1);
			}
	}

	/**
	 * Translates modes of a subprogram type
	 *
	 * @param tt
	 *            a subprogram type component
	 */
	private void ASME2SSME_SubprogramType_Modes (ProcessModelImpl vprocess, SubprogramType tt)
	{
		Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_SubprogramType_Modes: " + tt.getName ());
		asme2ssme_common.ASME2SSME_ComponentType_Modes (tt);
	}

	/**
	 * Translates properties of a subprogram type
	 *
	 * @param tt
	 *            a subprogram type component
	 */
	private void ASME2SSME_SubprogramType_Properties (ProcessModelImpl vprocess, SubprogramType tt)
	{
		ArrayList < PropertyAssociation > list1 = aadlColAPI.Component_GetPropertyAssociations (tt);
		if (list1 != null)
			if (list1.size () > 0)
			{
				Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_SubprogramType_Properties: " + tt.getName ());
				for (int i = 0; i < list1.size (); i++)
				{
					PropertyAssociation pa = list1.get (i);
					ProcessCallImpl pcall1 = ASME2SSME_Property_Call (pa);
					myLib.STree_addExpressionOfProcessToaModel (vprocess, pcall1);
				}
			}
	}

	/**
	 * Translates data subcomponents of a subprogram
	 *
	 * @param vprocess
	 *            a Signal process that represents subprogram implementation
	 *            component
	 * @param sp
	 */
	private void ASME2SSME_SubprogramImpl_SubData (ProcessModelImpl vprocess, SubprogramImplementation sp)
	{
		Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_SubprogramImpl_SubData " + sp.getName ());
		if (sp.getOwnedDataSubcomponents () != null)
			Trace ("data subcomponent of a subprogram has not been implemented in this version of OSATE ");
	}

	/**
	 * Translates call sequences of subprogram
	 *
	 * @param sp
	 */
	private void ASME2SSME_SubprogramImpl_CallSequences (SubprogramImplementation sp)
	{
		Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_SubprogramImpl_CallSequences");
		if (sp.getOwnedSubprogramCallSequences () != null)
			Trace ("Subprogram Impl call sequences not yet implemented");
	}

	/**
	 * Translates connections of a subprogram
	 *
	 * @param sp
	 */
	private void ASME2SSME_SubprogramImpl_Connections (SubprogramImplementation sp)
	{
		Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_SubprogramImpl_Connections");
		if (sp.getOwnedConnections () != null)
			Trace ("Subprogram Impl connections not yet implemented");
	}

	/**
	 * Translates flows of a subprogram implementation
	 *
	 * @param sp  a subprogram implementation component
	 */
	private void ASME2SSME_SubprogramImpl_Flows (ProcessModelImpl vprocess, SubprogramImplementation sp)
	{
		ArrayList < FlowImplementation > list = aadlColAPI.ComponentImpl_GetFlowImplementations (sp);
		if (list != null)
			if (list.size () > 0)
			{
				Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_SubprogramImpl_Flows " + sp.getName ());

				ProcessModelImpl p1 = asme2ssme_common.ASME2SSME_ComponentImpl_Flows (sp);
				myLib.STree_addProcessDeclaration (vprocess, p1);
				ArrayList < ExprParam > lparam = new ArrayList < ExprParam > ();
				String pname = Name_of_Flow (sp.getName ().replace ('.', '_'));
				ProcessExpansionImpl call1 = myLib.STree_mkProcessCallWithParameters (myLib.mkIdentifier (pname), lparam);
				myLib.STree_addExpressionOfProcessToaModel (vprocess, call1);
			}
	}

	/**
	 * Translate modes of a subprogram implementation component
	 *
	 * @param sp
	 */
	private void ASME2SSME_SubprogramImpl_Modes (SubprogramImplementation sp)
	{
		Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_SubprogramImpl_Modes");
		asme2ssme_common.ASME2SSME_ComponentImpl_Modes (sp);
	}

	/**
	 * Translates a shared subprogram as a shared Signal process. Creates a
	 * process call instance in the body of vprocess: (l_y, l_id) :=
	 * xx_Subprogram(l_x) where "l" is short for xx, l_x is shared
	 *
	 * @param sp_name
	 *            AADL subprogram (type or implementation) (original) name
	 * @param label
	 *            subprogram subcomponent name
	 * @param vprocess
	 *            the upper level process
	 * @param sp_process
	 *            the Signal process that represents the subprogram
	 */
	public void ASME2SSME_Create_SharedSubprogram (String sp_name, String label, ProcessModelImpl vprocess, ProcessModelImpl sp_process)
	{

		String s_name = sp_name.replace (".", "_");	// replace "." with "_"
		Identifier id = myLib.Process_GetID (sp_process);

		Trace ("\n ====== ASME2SSME_Translate_Subprogram: ASME2SSME_Create_SharedSubprogram: " + sp_name);

		ArrayList < ExprParam > lps = new ArrayList < ExprParam > ();	 
		// list for call the process sp_process
		ArrayList < ExprSignal > lis = new ArrayList < ExprSignal > ();	 
		// list for call the process sp_process
		ArrayList < ExprSignal > los = new ArrayList < ExprSignal > ();	 
		// list for call  the process  sp_process
		ArrayList < Declaration > vdecl = new ArrayList < Declaration > ();	 
		// lists to be added to vprocess

		// ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl)
		// sp_process.getInterfaceDefinition();
		// if (inter != null){
		//
		// IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
		// if (vioProfile != null)
		// {
		// EList l_inputs = vioProfile.getInputs();
		// Iterator it_i = l_inputs.iterator();
		// while (it_i.hasNext())
		// {
		// SignalsDeclarationImpl ainput = (SignalsDeclarationImpl) it_i.next();
		// EList<SignalNaming> list2 = ainput.getListSignal();
		//
		// if (ainput.getType() instanceof PredefinedTypeImpl){
		// PredefinedTypeImpl type_2 = (PredefinedTypeImpl) ainput.getType();
		// for (int i = 0; i < list2.size(); i++) {
		// SignalNaming sig_name2 = list2.get(i);
		// Identifier id_sig_name2 = sig_name2.getIdentifier();
		// String str_sig_name2 = id_sig_name2.getAtomString();
		// String name2 = s_name + '_' + str_sig_name2;
		// String lname1 = getSname(
		// name2);
		// //Trace("input predefined lname1 = " + lname1);
		// IdentifierImpl id1 = myLib.mkIdentifier(lname1);
		// //IdentifierImpl id1 = myLib.mkIdentifier(name2);
		// if (str_sig_name2.equals("spcall")){
		// PredefinedTypeImpl event_type =
		// myLib.STree_mkPredefinedType("event");
		// SignalsDeclarationImpl sig1 =
		// myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(lname1),
		// event_type, null);
		// myLib.STree_addDeclarationToProcess(vprocess, sig1);
		// vdecl.add(sig1);
		// }
		// else{
		// PredefinedTypeImpl ptype1 = myLib.STree_mkSamePredefinedType(ainput);
		// SharedVariablesDeclarationImpl sig2 =
		// myLib.STree_mkSharedVariableDeclaration(myLib.mkIdentifier(lname1),
		// ptype1, null);
		// myLib.STree_addDeclarationToProcess(vprocess, sig2);
		// vdecl.add(sig2);
		// }
		// lis.add(id1);
		// }
		// }
		// else{ //input not predefined type
		// Identifier type_id = (Identifier) ainput.getType();
		// String type_name = type_id.getAtomString();
		// IdentifierImpl type2 = myLib.mkIdentifier(type_name);
		//
		// for (int i = 0; i < list2.size(); i++) {
		// SignalNaming sig_name2 = list2.get(i);
		// Identifier id_sig_name2 = sig_name2.getIdentifier();
		// String name2 = s_name + '_' + id_sig_name2.getAtomString();
		// String lname2 = getSname(
		// name2);
		// IdentifierImpl id2 = myLib.mkIdentifier(lname2);
		// //IdentifierImpl id2 = myLib.mkIdentifier(name2);
		// SharedVariablesDeclarationImpl sig3 =
		// myLib.STree_mkSharedVariableDeclaration(myLib.mkIdentifier(lname2),
		// type2, null);
		// lis.add(id2);
		// vdecl.add(sig3);
		// }
		// }
		// }
		//
		// //outputs
		// EList<TypesOrSignalsDeclaration> l_outputs = vioProfile.getOutputs();
		// Iterator<TypesOrSignalsDeclaration> it_o = l_outputs.iterator();
		// while (it_o.hasNext())
		// {
		// SignalsDeclarationImpl aoutput = (SignalsDeclarationImpl)
		// it_o.next();
		// EList<SignalNaming> list3 = aoutput.getListSignal();
		//
		// if (aoutput.getType() instanceof PredefinedTypeImpl){
		// PredefinedTypeImpl type_3 = (PredefinedTypeImpl) aoutput.getType();
		// for (int i = 0; i < list3.size(); i++) {
		// SignalNaming sig_name3 = list3.get(i);
		// Identifier id_sig_name3 = sig_name3.getIdentifier();
		// String str_sig_name3 = id_sig_name3.getAtomString();
		// String name3 = s_name + '_' + str_sig_name3;
		// String lname3 = getSname(
		// name3);
		// IdentifierImpl id3 = myLib.mkIdentifier(lname3);
		// //IdentifierImpl id3 = myLib.mkIdentifier(name3);
		// PredefinedTypeImpl ptype3 =
		// myLib.STree_mkSamePredefinedType(aoutput);
		// SharedVariablesDeclarationImpl sig3 =
		// myLib.STree_mkSharedVariableDeclaration(myLib.mkIdentifier(lname3),
		// ptype3, null);
		// los.add(id3);
		// vdecl.add(sig3);
		// }
		// }
		// else{//output not predefined type
		// Identifier type_id = (Identifier) aoutput.getType();
		// String type_name = type_id.getAtomString();
		// IdentifierImpl type3 = myLib.mkIdentifier(type_name);
		//
		// for (int i = 0; i < list3.size(); i++) {
		// SignalNaming sig_name3 = list3.get(i);
		// Identifier id_sig_name3 = sig_name3.getIdentifier();
		// String name3 = s_name + '_' + id_sig_name3.getAtomString();
		// String lname4 = getSname(
		// name3);
		// IdentifierImpl id4 = myLib.mkIdentifier(lname4);
		// //IdentifierImpl id4 = myLib.mkIdentifier(name3);
		// SharedVariablesDeclarationImpl sig3 =
		// myLib.STree_mkSharedVariableDeclaration(myLib.mkIdentifier(lname4),
		// type3, null);
		// los.add(id4);
		// vdecl.add(sig3);
		// }
		// }
		// }
		// }
		// }

		// in: shared sp_IN l_x;
		String sn_sp = getSname( s_name);
		Identifier type1 = myLib.mkIdentifier (Name_of_Subprogram_Struct_In (s_name));
		IdentifierImpl x1 = myLib.mkIdentifier (sn_sp + "_x");
		SharedVariablesDeclarationImpl sig1 = myLib.STree_mkSharedVariableDeclaration (myLib.mkIdentifier (sn_sp + "_x"), type1, null);
		lis.add (x1);
		vdecl.add (sig1);

		// out: sp_OUT l_y;
		// Identifier type2 = myLib.mkIdentifier (Name_of_Subprogram_Struct_Out (s_name));
		IdentifierImpl y1 = myLib.mkIdentifier (sn_sp + "_y");
		SignalsDeclarationImpl sig2 = myLib.mkDecl(sn_sp + "_y", Name_of_Subprogram_Struct_Out (s_name)); 
		// myLib.STree_mkSignalDeclaration (myLib.mkIdentifier (sn_sp + "_y"), type2, null);
		los.add (y1);
		vdecl.add (sig2);

		// out: TID l_id;
		// Identifier type3 = myLib.mkIdentifier ("TID");
		IdentifierImpl id1 = myLib.mkIdentifier (sn_sp + "_id");
		SignalsDeclarationImpl sig3 = myLib.mkDecl(sn_sp + "_id", "TID"); // 
		// myLib.STree_mkSignalDeclaration (myLib.mkIdentifier (sn_sp + "_id"), type3, null);
		los.add (id1);
		vdecl.add (sig3);

		// add comments for the instance
		ArrayList < String > comments1 = new ArrayList < String > ();
		String str1 = "instance of AADL subprogram " + sp_name + " (" + sn_sp + ")";
		comments1.add (str1);

		if (los.size () > 0)
		{
			SignalsDefinitionImpl call_eq = myLib.STree_mkCompleteProcessCallEquation (myLib.mkIdentifier (id.getAtomString ()), lps, lis, los);
			if (label != null)
			{
				ArrayList < Identifier > ids = new ArrayList < Identifier > ();
				ids.add (myLib.mkIdentifier (label));
				myLib.STree_addLabelsInModel (ids, vprocess);
				LabelledProcessImpl lp = myLib.STree_mkLabelledProcessCall (label, call_eq);
				myLib.STree_addExpressionOfProcessToaModel (vprocess, lp);
				myLib.STree_addComment (lp, comments1, false);
			}
			else
			{
				myLib.STree_addExpressionOfProcessToaModel (vprocess, call_eq);
				myLib.STree_addComment (call_eq, comments1, false);
			}
		}
		else
		{
			ProcessCallImpl pcall = myLib.STree_mkCompleteProcessCall (myLib.mkIdentifier (id.getAtomString ()), lps, lis);
			if (label != null)
			{
				ArrayList < Identifier > ids = new ArrayList < Identifier > ();
				ids.add (myLib.mkIdentifier (label));
				myLib.STree_addLabelsInModel (ids, vprocess);
				LabelledProcessImpl lp = myLib.STree_mkLabelledProcessCall (label, pcall);
				myLib.STree_addExpressionOfProcessToaModel (vprocess, lp);
				myLib.STree_addComment (lp, comments1, false);
			}
			else
			{
				myLib.STree_addExpressionOfProcessToaModel (vprocess, pcall);
				myLib.STree_addComment (pcall, comments1, false);
			}
		}
		myLib.STree_addLocalDeclaration (vprocess, vdecl);
	}

	// /**
	// * For a shared subprogram xx:
	// * give the clock of xx_spcall:
	// * l_x := t1 ^+ t2 ^+ ...
	// * where "l" is a local short name for the subprogram xx,
	// * l_x is a local signal that represents in struct x,
	// * each ti is a local signal that represents a thread's requires
	// subprogram access
	// *
	// * @param sp_name a string name of AADL subprogram type of implementation
	// * @param vprocess Signal process that represents AADL process
	// implementation
	// * @pimpl a process implementation
	// */
	// private void ASME2SSME_SharedSubprogram_Call_Def(String sp_name,
	// ProcessModelImpl vprocess, ProcessImplementation pimpl){
	//
	// Trace("\n ====== ASME2SSME_Translate_Subprogram: ASME2SSME_SharedSubprogram_Call_Def: "
	// + sp_name);
	//
	// ASME2SSME_Translate_Common asme2ssme_common = new
	// ASME2SSME_Translate_Common();
	//
	// String s_name = sp_name.replace(".", "_"); //replace "." with "_"
	// String str1 = getSname(
	// s_name+"_spcall");
	// ExprSignal leftExp = myLib.mkIdentifier(str1);
	//
	// //gets the threads that requires access or calls this subprogram
	// ArrayList<ThreadSubcomponent> threads1 =
	// aadlColAPI.Process_Get_Threads_Requires_Subprogram_x(pimpl, sp_name);
	//
	// if (threads1.size()>0){
	// //ClockEqualityImpl clk_eq = new ClockEqualityImpl();
	// //SignalsDefinitionImpl clk_eq = new SignalsDefinitionImpl();
	// SignalsPartialDefinitionImpl clk_eq = new SignalsPartialDefinitionImpl();
	//
	// if(threads1.size()==1){
	// ThreadSubcomponent ts1 = threads1.get(0);
	// String label1 = aadlColAPI.Subcomponent_GetName(ts1);//sub thread name
	// ThreadImplementation t1 = (ThreadImplementation)
	// aadlColAPI.Threadsubcomponent_GetClassifier(ts1);
	// ThreadType tt1 = (ThreadType)
	// aadlColAPI.ComponentImpl_GetComponentType(t1);
	// SubprogramAccess spa1 =
	// aadlColAPI.ThreadType_Get_SubprogramAccess_x(tt1,sp_name);
	// String name1 = aadlColAPI.SubprogramAccess_GetName(spa1);//requires
	// subprogram access name
	// String sname1 = getSname(
	// label1+"_"+name1);//local short name
	// IdentifierImpl rightExp = myLib.mkIdentifier(sname1);
	// //xx :=
	// clk_eq = (SignalsPartialDefinitionImpl)
	// myLib.mkEquation(SignalOperators.opdefpart, leftExp, rightExp);
	// }
	// else{
	// ClockUnionImpl tmp = new ClockUnionImpl();
	// ClockUnionImpl rightExp = new ClockUnionImpl();
	//
	// ThreadSubcomponent ts1 = threads1.get(0);
	// String label1 = aadlColAPI.Subcomponent_GetName(ts1);//sub thread name
	// ThreadImplementation t1 = (ThreadImplementation)
	// aadlColAPI.Threadsubcomponent_GetClassifier(ts1);
	// ThreadType tt1 = (ThreadType)
	// aadlColAPI.ComponentImpl_GetComponentType(t1);
	// SubprogramAccess spa2 =
	// aadlColAPI.ThreadType_Get_SubprogramAccess_x(tt1,sp_name);
	// String name2 = aadlColAPI.SubprogramAccess_GetName(spa2);
	// String sname2 = getSname(
	// label1+"_"+name2);//local short name
	// tmp.setExprSignal1(myLib.mkIdentifier(sname2));
	// rightExp.setExprSignal1(myLib.mkIdentifier(sname2));
	//
	// for(int m =1; m<threads1.size(); m++){
	//
	// ThreadSubcomponent ts2 = threads1.get(m);
	// String label2 = aadlColAPI.Subcomponent_GetName(ts2);//sub thread name
	// ThreadImplementation t2 = (ThreadImplementation)
	// aadlColAPI.Threadsubcomponent_GetClassifier(ts2);
	// ThreadType tt2 = (ThreadType)
	// aadlColAPI.ComponentImpl_GetComponentType(t2);
	// SubprogramAccess spa3 =
	// aadlColAPI.ThreadType_Get_SubprogramAccess_x(tt2,sp_name);
	// String name3 = aadlColAPI.SubprogramAccess_GetName(spa3);
	// String sname3 = getSname(
	// label2+"_"+name3);//local short name
	// tmp.setExprSignal2(myLib.mkIdentifier(sname3));
	// rightExp.setExprSignal2(myLib.mkIdentifier(sname3));
	// tmp.setExprSignal1(rightExp);
	// }
	//
	// //xx := ...
	// clk_eq = (SignalsPartialDefinitionImpl)
	// myLib.mkEquation(SignalOperators.opdefpart, leftExp,
	// rightExp);//myLib.STree_mkClockEquality(leftExp, rightExp);
	// }
	//
	// //add comments
	// ArrayList<String> comments1 = new ArrayList<String>();
	// String str = "clock of call subprogram : " + sp_name;
	// comments1.add(str);
	// myLib.STree_addComment(clk_eq, comments1, false);
	//
	// myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq);
	// return;
	// }
	// // }
	//
	// return;
	// }

	private String mkIdForSynchroThreadImplementation(ThreadSubcomponent ts1, String sp_name)
	{
		String name1;
		String label1;

		label1 	= aadlColAPI.Subcomponent_GetName (ts1); 
		ThreadImplementation t1 = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier (ts1);
		ThreadType tt1 = (ThreadType) aadlColAPI.ComponentImpl_GetComponentType (t1);
		SubprogramCall spcall1 = aadlColAPI.Thread_Get_SubprogramCall_CalledSubprogram_x (t1, sp_name);

		if (spcall1 != null) 
			name1 = aadlColAPI.SubprogramCall_GetName (spcall1); 
		else
		{
			SubprogramAccess spa1 = aadlColAPI.ThreadType_Get_SubprogramAccess_x (tt1, sp_name);
			name1 = aadlColAPI.SubprogramAccess_GetName (spa1);  
		}
		return getSname( label1 + "_" + name1); // local

	}

	/**
	 * For a shared subprogram xx, give the clock of in puts: l_x ^= t1 ^+ t2 ^+
	 * ... where "l" is a local short name for the subprogram xx, l_x is a local
	 * signal that represents in struct x, each ti is a local signal that
	 * represents a thread's call subprogram or requires subprogram access
	 *
	 * @param sp_name
	 *            a string name of AADL subprogram type of implementation
	 * @param vprocess
	 *            Signal process that represents AADL process implementation
	 * @pimpl a process implementation
	 */
	public void ASME2SSME_SharedSubprogram_Call_Sync_x (String sp_name, ProcessModelImpl vprocess, ProcessImplementation pimpl)
	{
		Trace ("\n ====== ASME2SSME_Translate_Subprogram: ASME2SSME_SharedSubprogram_Call_Sync_x: " + sp_name);

		String s_name = sp_name.replace (".", "_");	
		String str1 = getSname(s_name);
		ExprSignal leftExp = myLib.mkIdentifier (str1 + "_x");	// l_x

		// gets the threads that requires access or calls this subprogram
		ArrayList < ThreadSubcomponent > threads1 = aadlColAPI.Process_Get_Threads_Requires_Subprogram_x (pimpl, sp_name);
		ArrayList < ThreadSubcomponent > threads2 = aadlColAPI.Process_Get_Threads_Calls_Subprogram_x (pimpl, sp_name);
		if (threads2.size () > 0) threads1.addAll (threads2);
		if (threads1.size () > 0)
		{
			ClockEqualityImpl clk_eq = new ClockEqualityImpl ();

			if (threads1.size () == 1)
			{
				ThreadSubcomponent ts1 = threads1.get (0);
				IdentifierImpl rightExp = myLib.mkIdentifier (mkIdForSynchroThreadImplementation(ts1, sp_name)); // short name
				clk_eq = myLib.STree_mkClockEquality (leftExp, rightExp);	// l_x ^=
			}
			else
			{
				ClockUnionImpl tmp = new ClockUnionImpl ();
				ClockUnionImpl rightExp = new ClockUnionImpl ();
				ThreadSubcomponent ts1 = threads1.get (0);
				String sname2 =  mkIdForSynchroThreadImplementation(ts1, sp_name);
				tmp.setExprSignal1 (myLib.mkIdentifier (sname2));
				rightExp.setExprSignal1 (myLib.mkIdentifier (sname2));

				for (int m = 1; m < threads1.size (); m++)
				{
					ThreadSubcomponent ts2 = threads1.get (m);
					String sname3 =  mkIdForSynchroThreadImplementation(ts2, sp_name);
					tmp.setExprSignal2 (myLib.mkIdentifier (sname3));
					rightExp.setExprSignal2 (myLib.mkIdentifier (sname3));
					tmp.setExprSignal1 (rightExp);
				}
				clk_eq = myLib.STree_mkClockEquality (leftExp, rightExp); // l_x ^= ...
			}
			myLib.STree_addComment (clk_eq, "clock of call subprogram : " + sp_name , false);
			myLib.STree_addExpressionOfProcessToaModel (vprocess, clk_eq);
		}
	}

	/**
	 * For a shared subprogram xx, give the clock of out puts: l_y ^= l_id ^=
	 * l_x where "l" is a local short name for the subprogram xx,
	 *
	 * @param sp_name
	 *            a string name of AADL subprogram type of implementation
	 * @param vprocess
	 *            Signal process that represents AADL process implementation
	 * @pimpl a process implementation
	 */
	public void ASME2SSME_SharedSubprogram_Sync_y (String sp_name, ProcessModelImpl vprocess, ProcessImplementation pimpl)
	{

		Trace ("\n ====== ASME2SSME_Translate_Subprogram: ASME2SSME_SharedSubprogram_Sync_y: " + sp_name);

		String s_name = sp_name.replace (".", "_");	 
		String str1 = getSname(s_name);

		ExprSignal exp1 = myLib.mkIdentifier (str1 + "_y");	 // l_y
		ExprSignal exp2 = myLib.mkIdentifier (str1 + "_id"); // l_id
		ExprSignal exp3 = myLib.mkIdentifier (str1 + "_x");  // l_x
		ArrayList < ExprSignal > exps = new ArrayList < ExprSignal > ();
		exps.add (exp1);
		exps.add (exp2);
		exps.add (exp3);

		ClockEqualityImpl clk_eq = myLib.STree_mkClockEqualitys (exps);
		myLib.STree_addComment (clk_eq, "output clocks of subprogram : " + sp_name, false);
		myLib.STree_addExpressionOfProcessToaModel (vprocess, clk_eq);
	}

	/**
	 * Translates a called subprogram: a called subprogram is instantiated as
	 * shared process. Creates a process call instance in the body of vprocess:
	 * (l_y, l_id) := xx_Subprogram(l_x) where "l" is local short for xx, l_x is
	 * shared
	 *
	 * @param sp_name
	 *            AADL subprogram (type or implementation) name
	 * @param vprocess
	 *            the upper level process
	 * @param sp_process
	 *            the Signal process that represents the subprogram
	 */
	private void UNUSED_ASME2SSME_Create_CalledSubprogram (String sp_name, ProcessModelImpl vprocess, ProcessModelImpl sp_process)
	{

		String s_name = sp_name.replace (".", "_");
		Identifier id = myLib.Process_GetID (sp_process);

		Trace ("\n ====== ASME2SSME_Translate_Subprogram: ASME2SSME_Create_CalledSubprogram: " + sp_name);

		ArrayList < ExprParam > lps = new ArrayList < ExprParam > ();	 
		ArrayList < ExprSignal > lis = new ArrayList < ExprSignal > ();	 
		ArrayList < ExprSignal > los = new ArrayList < ExprSignal > ();	 
		ArrayList < Declaration > vdecl = new ArrayList < Declaration > ();	

		// ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl)
		// sp_process.getInterfaceDefinition();
		// if (inter != null){
		//
		// IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
		// if (vioProfile != null)
		// {
		// EList l_inputs = vioProfile.getInputs();
		// Iterator it_i = l_inputs.iterator();
		// while (it_i.hasNext())
		// {
		// SignalsDeclarationImpl ainput = (SignalsDeclarationImpl) it_i.next();
		// EList<SignalNaming> list2 = ainput.getListSignal();
		//
		// if (ainput.getType() instanceof PredefinedTypeImpl){
		// PredefinedTypeImpl type_2 = (PredefinedTypeImpl) ainput.getType();
		// for (int i = 0; i < list2.size(); i++) {
		// SignalNaming sig_name2 = list2.get(i);
		// Identifier id_sig_name2 = sig_name2.getIdentifier();
		// String str_sig_name2 = id_sig_name2.getAtomString();
		// String name2 = s_name + '_' + str_sig_name2;
		// String lname1 = getSname(
		// name2);
		// //Trace("input predefined lname1 = " + lname1);
		// IdentifierImpl id1 = myLib.mkIdentifier(lname1);
		// //IdentifierImpl id1 = myLib.mkIdentifier(name2);
		// if (str_sig_name2.equals("spcall")){
		// PredefinedTypeImpl event_type =
		// myLib.STree_mkPredefinedType("event");
		// SignalsDeclarationImpl sig1 =
		// myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(lname1),
		// event_type, null);
		// myLib.STree_addDeclarationToProcess(vprocess, sig1);
		// vdecl.add(sig1);
		// }
		// else{
		// PredefinedTypeImpl ptype1 = myLib.STree_mkSamePredefinedType(ainput);
		// SharedVariablesDeclarationImpl sig2 =
		// myLib.STree_mkSharedVariableDeclaration(myLib.mkIdentifier(lname1),
		// ptype1, null);
		// myLib.STree_addDeclarationToProcess(vprocess, sig2);
		// vdecl.add(sig2);
		// }
		// lis.add(id1);
		// }
		// }
		// else{ //input not predefined type
		// Identifier type_id = (Identifier) ainput.getType();
		// String type_name = type_id.getAtomString();
		// IdentifierImpl type2 = myLib.mkIdentifier(type_name);
		//
		// for (int i = 0; i < list2.size(); i++) {
		// SignalNaming sig_name2 = list2.get(i);
		// Identifier id_sig_name2 = sig_name2.getIdentifier();
		// String name2 = s_name + '_' + id_sig_name2.getAtomString();
		// String lname2 = getSname(
		// name2);
		// IdentifierImpl id2 = myLib.mkIdentifier(lname2);
		// //IdentifierImpl id2 = myLib.mkIdentifier(name2);
		// SharedVariablesDeclarationImpl sig3 =
		// myLib.STree_mkSharedVariableDeclaration(myLib.mkIdentifier(lname2),
		// type2, null);
		// lis.add(id2);
		// vdecl.add(sig3);
		// }
		// }
		// }
		//
		// //outputs
		// EList<TypesOrSignalsDeclaration> l_outputs = vioProfile.getOutputs();
		// Iterator<TypesOrSignalsDeclaration> it_o = l_outputs.iterator();
		// while (it_o.hasNext())
		// {
		// SignalsDeclarationImpl aoutput = (SignalsDeclarationImpl)
		// it_o.next();
		// EList<SignalNaming> list3 = aoutput.getListSignal();
		//
		// if (aoutput.getType() instanceof PredefinedTypeImpl){
		// PredefinedTypeImpl type_3 = (PredefinedTypeImpl) aoutput.getType();
		// for (int i = 0; i < list3.size(); i++) {
		// SignalNaming sig_name3 = list3.get(i);
		// Identifier id_sig_name3 = sig_name3.getIdentifier();
		// String str_sig_name3 = id_sig_name3.getAtomString();
		// String name3 = s_name + '_' + str_sig_name3;
		// String lname3 = getSname(
		// name3);
		// IdentifierImpl id3 = myLib.mkIdentifier(lname3);
		// //IdentifierImpl id3 = myLib.mkIdentifier(name3);
		// PredefinedTypeImpl ptype3 =
		// myLib.STree_mkSamePredefinedType(aoutput);
		// SharedVariablesDeclarationImpl sig3 =
		// myLib.STree_mkSharedVariableDeclaration(myLib.mkIdentifier(lname3),
		// ptype3, null);
		// los.add(id3);
		// vdecl.add(sig3);
		// }
		// }
		// else{//output not predefined type
		// Identifier type_id = (Identifier) aoutput.getType();
		// String type_name = type_id.getAtomString();
		// IdentifierImpl type3 = myLib.mkIdentifier(type_name);
		//
		// for (int i = 0; i < list3.size(); i++) {
		// SignalNaming sig_name3 = list3.get(i);
		// Identifier id_sig_name3 = sig_name3.getIdentifier();
		// String name3 = s_name + '_' + id_sig_name3.getAtomString();
		// String lname4 = getSname(
		// name3);
		// IdentifierImpl id4 = myLib.mkIdentifier(lname4);
		// //IdentifierImpl id4 = myLib.mkIdentifier(name3);
		// SharedVariablesDeclarationImpl sig3 =
		// myLib.STree_mkSharedVariableDeclaration(myLib.mkIdentifier(lname4),
		// type3, null);
		// los.add(id4);
		// vdecl.add(sig3);
		// }
		// }
		// }
		// }
		// }
		//
		// //add comments for the instance
		// ArrayList<String> comments1 = new ArrayList<String>();
		// String str1 = "instance of AADL subprogram " + sp_name;
		// comments1.add(str1);
		//
		// if (los.size()>0){
		// SignalsDefinitionImpl call_eq =
		// myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(id.getAtomString()),
		// lps, lis, los);
		// myLib.STree_addExpressionOfProcessToaModel(vprocess, call_eq);
		// myLib.STree_addComment(call_eq, comments1, false);
		// }
		// else{
		// ProcessCallImpl pcall =
		// myLib.STree_mkCompleteProcessCall(myLib.mkIdentifier(id.getAtomString()),
		// lps, lis);
		// myLib.STree_addExpressionOfProcessToaModel(vprocess, pcall);
		// myLib.STree_addComment(pcall, comments1, false);
		// }
		//
		// myLib.STree_addLocalDeclaration(vprocess, vdecl);
		// return;

		// in: shared sp_IN l_x;
		String sn_sp = getSname(s_name);
		Identifier type1 = myLib.mkIdentifier (Name_of_Subprogram_Struct_In (s_name));
		IdentifierImpl x1 = myLib.mkIdentifier (sn_sp + "_x");
		SharedVariablesDeclarationImpl sig1 = myLib.STree_mkSharedVariableDeclaration (myLib.mkIdentifier (sn_sp + "_x"), type1, null);
		lis.add (x1);
		vdecl.add (sig1);

		// out: sp_OUT l_y;
		IdentifierImpl y1 = myLib.mkIdentifier (sn_sp + "_y");
		SignalsDeclarationImpl sig2 = myLib.mkDecl(sn_sp + "_y", Name_of_Subprogram_Struct_Out (s_name) ); 
		los.add (y1);
		vdecl.add (sig2);

		// out: TID l_id;
		IdentifierImpl id1 = myLib.mkIdentifier (sn_sp + "_id");
		SignalsDeclarationImpl sig3 = myLib.mkDecl( sn_sp + "_id", "TID"); 
		los.add (id1);
		vdecl.add (sig3);

		// add comments for the instance
		ArrayList < String > comments1 = new ArrayList < String > ();
		String str1 = "instance of AADL subprogram " + sp_name + " (" + sn_sp + ")";
		comments1.add (str1);
		if (los.size () > 0)
		{
			SignalsDefinitionImpl call_eq = myLib.STree_mkCompleteProcessCallEquation (myLib.mkIdentifier (id.getAtomString ()), lps, lis, los);
			myLib.STree_addExpressionOfProcessToaModel (vprocess, call_eq);
			myLib.STree_addComment (call_eq, comments1, false);
		}
		else
		{
			ProcessCallImpl pcall = myLib.STree_mkCompleteProcessCall (myLib.mkIdentifier (id.getAtomString ()), lps, lis);
			myLib.STree_addExpressionOfProcessToaModel (vprocess, pcall);
			myLib.STree_addComment (pcall, comments1, false);
		}
		myLib.STree_addLocalDeclaration (vprocess, vdecl);
	}



	private String mkIdForSynchro(ThreadSubcomponent ts1, String sp_name) {
		String name1;
		String label1 = aadlColAPI.Subcomponent_GetName (ts1); 
		ThreadImplementation t1 = (ThreadImplementation) aadlColAPI.Threadsubcomponent_GetClassifier (ts1);
		ThreadType tt1 = (ThreadType) aadlColAPI.ComponentImpl_GetComponentType (t1);

		SubprogramCall spcall1 = aadlColAPI.Thread_Get_SubprogramCall_CalledSubprogram_x (t1, sp_name);
		if (spcall1 != null)
			name1 = aadlColAPI.SubprogramCall_GetName (spcall1);	// requires subprogram access  name
		else
		{
			SubprogramAccess spa1 = aadlColAPI.ThreadType_Get_SubprogramAccess_x (tt1, sp_name);
			name1 = aadlColAPI.SubprogramAccess_GetName (spa1);			 
		} 
		return  getSname( label1 + "_" + name1); 
	}

	/**
	 * For a subprogram (type or implementatin) named xx (xx is a sub subprogram
	 * of a sub thread t in process pimpl): give the clock of in puts: l_x ^= t1
	 * ^+ t2 ^+ ... where "l_x" is a local shared signal that represents sp_x
	 * each ti is a local signal that represents a thread's call subprogram or
	 * requires subprogram access
	 *
	 * @param sp_name
	 *            a string name of AADL subprogram type of implementation
	 * @param vprocess
	 *            Signal process that represents AADL process implementation
	 * @pimpl a process implementation
	 */
	public void ASME2SSME_SubThread_SubSubprogram_Call_Sync_x (String sp_name, ProcessModelImpl vprocess, ProcessImplementation pimpl)
	{

		Trace ("\n ====== ASME2SSME_Translate_Subprogram: ASME2SSME_SubThread_SubSubprogram_Call_Sync_x: " + sp_name);

		ArrayList < Declaration > vdecl = new ArrayList < Declaration > ();	 

		String s_name = sp_name.replace (".", "_");	 
		String str1 = getSname(s_name);
		ExprSignal leftExp = myLib.mkIdentifier (str1 + "_x");	 

		Identifier type1 = myLib.mkIdentifier (Name_of_Subprogram_Struct_In (s_name));
		IdentifierImpl x1 = myLib.mkIdentifier (str1 + "_x");
		SharedVariablesDeclarationImpl sig1 = myLib.STree_mkSharedVariableDeclaration (x1, type1, null);
		vdecl.add (sig1);

		// gets the threads that requires access or calls this subprogram
		ArrayList < ThreadSubcomponent > threads1 = aadlColAPI.Process_Get_Threads_Requires_Subprogram_x (pimpl, sp_name);
		ArrayList < ThreadSubcomponent > threads2 = aadlColAPI.Process_Get_Threads_Calls_Subprogram_x (pimpl, sp_name);
		if (threads2.size () > 0) threads1.addAll (threads2);

		if (threads1.size () > 0)
		{
			ClockEqualityImpl clk_eq = new ClockEqualityImpl ();

			if (threads1.size () == 1)
			{
				ThreadSubcomponent ts1 = threads1.get (0);
				String sname1 = mkIdForSynchro(ts1, sp_name);
				clk_eq = myLib.STree_mkClockEquality (leftExp, myLib.mkIdentifier (sname1)); // l_x ^=
			}
			else
			{
				ClockUnionImpl tmp = new ClockUnionImpl ();
				ClockUnionImpl rightExp = new ClockUnionImpl ();
				ThreadSubcomponent ts1 = threads1.get (0);
				String  sname2 = mkIdForSynchro(  ts1,  sp_name);
				tmp.setExprSignal1 (myLib.mkIdentifier (sname2));
				rightExp.setExprSignal1 (myLib.mkIdentifier (sname2));

				for (int m = 1; m < threads1.size (); m++)
				{
					ThreadSubcomponent ts2 = threads1.get (m);
					// String label2 = aadlColAPI.Subcomponent_GetName (ts2); Trace ("sub thread " + label2);
					String sname3 = mkIdForSynchro(  ts2,  sp_name);
					tmp.setExprSignal2 (myLib.mkIdentifier (sname3));
					rightExp.setExprSignal2 (myLib.mkIdentifier (sname3));
					tmp.setExprSignal1 (rightExp);
				}
				// l_x ^= ...
				clk_eq = myLib.STree_mkClockEquality (leftExp, rightExp);
			}
			myLib.STree_addComment (clk_eq, "clock of call subprogram : " + sp_name, false);
			myLib.STree_addExpressionOfProcessToaModel (vprocess, clk_eq);
		}
		myLib.STree_addLocalDeclaration (vprocess, vdecl);
	}



	/**
	 * For a subprogram (type or implementatin) named xx (xx is a sub subprogram
	 * of a sub thread t in process pimpl): give the clock of out puts: l_y ^=
	 * l_id ^= l_x where "l" is a local short name for the subprogram xx,
	 *
	 * @param sp_name
	 *            a string name of AADL subprogram type of implementation
	 * @param vprocess
	 *            Signal process that represents AADL process implementation
	 * @pimpl a process implementation
	 */
	public void ASME2SSME_SubThread_SubSubprogram_Sync_y (String sp_name, ProcessModelImpl vprocess, ProcessImplementation pimpl)
	{

		Trace ("\n ====== ASME2SSME_Translate_Subprogram: ASME2SSME_CalledSubprogram_Sync_y: " + sp_name);

		ArrayList < Declaration > vdecl = new ArrayList < Declaration > ();	// local
		// lists to
		// be added
		// to
		// vprocess

		String s_name = sp_name.replace (".", "_");	
		String str1 = getSname(s_name);

		ExprSignal exp1 = myLib.mkIdentifier (str1 + "_y");	 // l_y
		ExprSignal exp2 = myLib.mkIdentifier (str1 + "_id"); // l_id
		ExprSignal exp3 = myLib.mkIdentifier (str1 + "_x");	 // l_x
		ArrayList < ExprSignal > exps = new ArrayList < ExprSignal > ();
		exps.add (exp1);
		exps.add (exp2);
		exps.add (exp3);

		ClockEqualityImpl clk_eq = myLib.STree_mkClockEqualitys (exps);
		myLib.STree_addComment (clk_eq, "output clocks of subprogram : " + sp_name, false);

		myLib.STree_addExpressionOfProcessToaModel (vprocess, clk_eq);
		// add l_y, l_id as local signals
		SignalsDeclarationImpl sig2 = myLib.mkDecl( str1 + "_y", Name_of_Subprogram_Struct_Out (s_name)); 
		vdecl.add (sig2);
		SignalsDeclarationImpl sig3 = myLib.mkDecl(str1 + "_id", "TID"); 
		vdecl.add (sig3);

		myLib.STree_addLocalDeclaration (vprocess, vdecl);
	}

	/**
	 * For a subprogram (type or implementatin) named xx (xx is a sub subprogram
	 * of a sub thread t in process pimpl): give the connections of x L1 := l_x
	 * where "l" is a local short name for the subprogram xx, L1 is local short
	 * for subt_sp_x
	 *
	 * @param sp_name
	 *            a string name of AADL subprogram type of implementation
	 * @param vprocess
	 *            Signal process that represents AADL process implementation
	 * @pimpl a process implementation
	 */
	private void UNUSED_ASME2SSME_SubThread_SubSubprogram_Def_localx (String sp_name, ProcessModelImpl vprocess, ProcessImplementation pimpl, ThreadSubcomponent subt)
	{

		Trace ("\n ====== ASME2SSME_Translate_Subprogram: ASME2SSME_SubThread_SubSubprogram_Def_localx: " + sp_name);

		// ArrayList < Declaration > vdecl = new ArrayList < Declaration > ();	 
		String s_name = sp_name.replace (".", "_");	 
		String str1 = getSname(s_name);
		String str2 = aadlColAPI.Subcomponent_GetName (subt);
		String str3 = getSname(str2 + "_" + s_name + "_x");

		ExprSignal left_exp = myLib.mkIdentifier (str3);	// L1
		ExprSignal right_exp = myLib.mkIdentifier (str1 + "_x");	// l_x
		SignalsDefinitionImpl clk_eq = (SignalsDefinitionImpl) myLib.mkEquation(left_exp, right_exp);
		myLib.STree_addComment (clk_eq, "input connection for subprogram : " + sp_name, false);
		myLib.STree_addExpressionOfProcessToaModel (vprocess, clk_eq);
	}

	/**
	 * For a subprogram (type or implementatin) named xx (xx is a sub subprogram
	 * of a sub thread t in process pimpl): give the connections of y l_y := L2
	 * where "l" is a local short name for the subprogram xx, L2 is local short
	 * for subt_sp_y
	 *
	 * @param sp_name
	 *            a string name of AADL subprogram type of implementation
	 * @param vprocess
	 *            Signal process that represents AADL process implementation
	 * @pimpl a process implementation
	 */
	private void UNUSED_ASME2SSME_SubThread_SubSubprogram_Def_localy (String sp_name, ProcessModelImpl vprocess, ProcessImplementation pimpl, ThreadSubcomponent subt)
	{

		Trace ("\n ====== ASME2SSME_Translate_Subprogram: ASME2SSME_SubThread_SubSubprogram_Def_localy: " + sp_name);
		// ArrayList < Declaration > vdecl = new ArrayList < Declaration > (); 
		String s_name = sp_name.replace (".", "_");	// replace "." with "_"
		String str1 = getSname(s_name);
		String str2 = aadlColAPI.Subcomponent_GetName (subt);
		String str3 = getSname(str2 + "_" + s_name + "_y");

		ExprSignal right_exp = myLib.mkIdentifier (str3);	// L2
		ExprSignal left_exp = myLib.mkIdentifier (str1 + "_y");	// l_y

		SignalsDefinitionImpl clk_eq = (SignalsDefinitionImpl) myLib.mkEquation (left_exp, right_exp);
		myLib.STree_addComment (clk_eq, "input connection for subprogram : " + sp_name, false);
		myLib.STree_addExpressionOfProcessToaModel (vprocess, clk_eq);
	}

	/**
	 * For a subprogram (type or implementatin) named xx (xx is a sub subprogram
	 * of a sub thread t in process pimpl): give the connections of id l_id :=
	 * L3 where "l" is a local short name for the subprogram xx, L3 is local
	 * short for subt_sp_id
	 *
	 * @param sp_name
	 *            a string name of AADL subprogram type of implementation
	 * @param vprocess
	 *            Signal process that represents AADL process implementation
	 * @pimpl a process implementation
	 */
	private void UNUSED_ASME2SSME_SubThread_SubSubprogram_Def_localid (String sp_name, ProcessModelImpl vprocess, ProcessImplementation pimpl, ThreadSubcomponent subt)
	{

		Trace ("\n ====== ASME2SSME_Translate_Subprogram: ASME2SSME_SubThread_SubSubprogram_Def_localid: " + sp_name);

		// ArrayList < Declaration > vdecl = new ArrayList < Declaration > ();	 
		String s_name = sp_name.replace (".", "_");
		String str1 = getSname(s_name);
		String str2 = aadlColAPI.Subcomponent_GetName (subt);
		String str3 = getSname(str2 + "_" + s_name + "_id");

		ExprSignal right_exp = myLib.mkIdentifier (str3);	// L3
		ExprSignal left_exp = myLib.mkIdentifier (str1 + "_id");	// l_id

		SignalsDefinitionImpl clk_eq = (SignalsDefinitionImpl) myLib.mkEquation(left_exp, right_exp);
		myLib.STree_addComment (clk_eq, "input connection for subprogram : " + sp_name, false);
		myLib.STree_addExpressionOfProcessToaModel (vprocess, clk_eq);
	}

	private Type getType_data_classifier(DataClassifier data_classifier) {
		String dname = Name_of_Reference_data_classifier (data_classifier);
		if (myLib.Is_PredefinedType (dname)) return myLib.STree_mkPredefinedType (dname);
		return myLib.mkIdentifier (dname);
	}

	/**
	 * Returns a Signal type declaration (a struct) that represents the in
	 * parameters of a subprogram (spt)
	 *
	 * @param spt
	 *            a subprogram type
	 * @param list_in_parameters
	 *            a list of in parameters
	 *
	 * @return a (struct) type declaration
	 */
	private TypesDeclarationImpl ASME2SSME_mkSubprogram_StructIn (SubprogramType spt, ArrayList < Parameter > list_in_parameters)
	{
		Type vtype;

		Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_mkSubprogram_StructIn: " + spt.getName ());
		TypeDescriptionImpl typedescr = new TypeDescriptionImpl ();
		ExprSignal vinit = null;
		String tname = aadlColAPI.ComponentType_GetName (spt);
		Identifier id = myLib.mkIdentifier (Name_of_Subprogram_Struct_In (tname));
		if (list_in_parameters == null)
		{
			// external type
			ExternalAttributeImpl externe = new ExternalAttributeImpl ();
			typedescr.setTypeDescriptionValue (externe);
			typedescr.setIdentifier (id);
		}
		else
		{
			TupleTypeImpl description = new TupleTypeImpl ();
			description.setTypeKind (TypeKind.STRUCT_LITERAL);

			for (int i = 0; i < list_in_parameters.size (); i++)
			{
				Parameter p1 = list_in_parameters.get (i);
				String pname1 = aadlColAPI.Parameter_GetName (p1);
				IdentifierImpl pid1 = myLib.mkIdentifier (pname1);

				DataClassifier data_classifier = aadlColAPI.Parameter_GetClassifier (p1);
				vtype = getType_data_classifier(data_classifier);

				SignalsDeclarationImpl decl1 = myLib.STree_mkSignalDeclaration (pid1, vtype, null);
				description.getListSignalsDeclaration ().add (decl1);
			}
			typedescr = myLib.STree_mkDclType (id, description, vinit);
		}
		TypesDeclarationImpl type_decl = new TypesDeclarationImpl ();
		type_decl.getTypeDefinition ().add (typedescr);
		return type_decl;
	}



	/**
	 * Returns a Signal type declaration (a struct) that represents the out
	 * parameters and out event/event data ports of a subprogram (spt)
	 *
	 * @param spt
	 *            a subprogram type
	 * @param list_in_parameters
	 *            a list of in parameters
	 *
	 * @return a (struct) type declaration
	 */
	private TypesDeclarationImpl ASME2SSME_mkSubprogram_StructOut (SubprogramType spt, ArrayList < Parameter > list_out_parameters,
			ArrayList < EventPort > list_oute_port, ArrayList < EventDataPort > list_outed_port)
	{


		Trace ("\n ========== ASME2SSME_Translate_Subprogram: ASME2SSME_mkSubprogram_StructOut: " + spt.getName ());

		TypeDescriptionImpl typedescr = new TypeDescriptionImpl ();
		ExprSignal vinit = null;

		String tname = aadlColAPI.ComponentType_GetName (spt);
		Identifier id = myLib.mkIdentifier (Name_of_Subprogram_Struct_Out (tname));	// xx_OUT

		if ((list_out_parameters.size () == 0) && (list_oute_port.size () == 0) && (list_outed_port.size () == 0))
		{
			Trace ("no output for subprogram " + spt.getName ());
			// external type
			ExternalAttributeImpl externe = new ExternalAttributeImpl ();
			typedescr.setTypeDescriptionValue (externe);
			typedescr.setIdentifier (id);
		}
		else
		{
			TupleTypeImpl description = new TupleTypeImpl ();
			description.setTypeKind (TypeKind.STRUCT_LITERAL);

			// out parameters
			for (int i = 0; i < list_out_parameters.size (); i++)
			{
				Parameter p1 = list_out_parameters.get (i);
				String pname1 = aadlColAPI.Parameter_GetName (p1);
				IdentifierImpl pid1 = myLib.mkIdentifier (pname1);

				DataClassifier data_classifier = aadlColAPI.Parameter_GetClassifier (p1);
				Type vtype = getType_data_classifier(data_classifier);
				SignalsDeclarationImpl decl1 = myLib.STree_mkSignalDeclaration (pid1, vtype, null);
				description.getListSignalsDeclaration ().add (decl1);
			}

			// out event ports
			for (int j = 0; j < list_oute_port.size (); j++)
			{
				EventPort p2 = list_oute_port.get (j);
				String pname2 = aadlColAPI.Port_GetName (p2);
				SignalsDeclarationImpl decl2 = myLib.mkBooleanDecl( pname2); 
				description.getListSignalsDeclaration ().add (decl2);
			}

			// out event data ports
			for (int k = 0; k < list_outed_port.size (); k++)
			{
				EventDataPort p3 = list_outed_port.get (k);
				String pname3 = aadlColAPI.Port_GetName (p3);
				IdentifierImpl pid3 = myLib.mkIdentifier (pname3);

				DataClassifier data_classifier3 = aadlColAPI.EventDataPort_GetClassifier (p3);
				Type vtype = getType_data_classifier(data_classifier3);  
				SignalsDeclarationImpl decl1 = myLib.STree_mkSignalDeclaration (pid3, vtype, null);
				description.getListSignalsDeclaration ().add (decl1);
			}
			typedescr = myLib.STree_mkDclType (id, description, vinit);
		}

		TypesDeclarationImpl type_decl = new TypesDeclarationImpl ();
		type_decl.getTypeDefinition ().add (typedescr);

		return type_decl;
	}

}
