 
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.osate.aadl2.*;
import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;
 
public class ASME2SSME_Translate_System extends ASME2SSME_Translate_Property{	
	private AADLCoLAPI aadlColAPI;
	private signalTreeAPI myLib ;
	private ASME2SSME_Translate_Common asme2ssme_common;
	
	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_System() {
		aadlColAPI = new AADLCoLAPI();
		myLib= new signalTreeAPI();
		asme2ssme_common = new ASME2SSME_Translate_Common();
	}

	/**
	 * translates a system implementation (s) to a Signal process. 
	 * <p>
	 * the interface is translated form its system type.  
	 * each subcomponent classifier (type or implementation) is translated by the corresponding rule, 
	 * and this subcomponent is an instance in the Signal process body (except for the process subcomponents)
	 * 
	 * @param s a system implementation component
	 * @return a Signal process (ProcessModelImpl object) (vprocess)	
	 */
	public ProcessModelImpl ASME2SSME_SystemImpl(SystemImplementation s, ModuleImpl vmodule, AadlPackage aadlpkg){

		ProcessModelImpl vprocess;
		String sname = aadlColAPI.ComponentImpl_GetName(s);

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemImpl: " + sname);
		String pname = Name_of_System(sname);
		vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );

		// get the component type
		SystemType st = (SystemType) aadlColAPI.ComponentImpl_GetComponentType(s);

		//interface 
		ASME2SSME_SystemImpl_Interface(s, vprocess);

		//features (from system type) 
		ASME2SSME_System_Features(st, vprocess);

		//system behavior 
		ProcessModelImpl sys_behavior = ASME2SSME_System_behavior(pname);
		String sys_behavior_name = myLib.STree_GetStringName(sys_behavior.getIdentifier());//pname + "_behavior";
		//add to declaration
		myLib.STree_addProcessDeclaration(vprocess, sys_behavior);
		//add to body
		ProcessCallImpl behavior_inst = myLib.STree_mkModelCall(sys_behavior_name);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, behavior_inst);
		//add comments 
		ArrayList<String> comments1 = new ArrayList<String>();
		comments1.add("system behavior sub process");
		myLib.STree_addComment(behavior_inst, comments1, false);

		//system property 
		ProcessModelImpl sys_property = ASME2SSME_System_property(s,pname);
		String sys_property_name = myLib.STree_GetStringName(sys_property.getIdentifier());//pname + "_property";
		//add to declaration
		myLib.STree_addProcessDeclaration(vprocess, sys_property);
		//add to body
		ProcessCallImpl property_inst = myLib.STree_mkModelCall(sys_property_name);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, property_inst);
		//add comments 
		ArrayList<String> comments2 = new ArrayList<String>();
		comments2.add("system property sub process");
		myLib.STree_addComment(property_inst, comments2, false);

		//Subcomponents 
		ArrayList<SystemSubcomponent> sub_syss = aadlColAPI.System_GetSubcomponentsSystem(s);
		ArrayList<ProcessSubcomponent> sub_processes = aadlColAPI.System_GetSubcomponentsProcess(s);
		ArrayList<DataSubcomponent> sub_data = aadlColAPI.System_GetSubcomponentsData(s);
		ArrayList<ProcessorSubcomponent> sub_processors = aadlColAPI.System_GetSubcomponentsProcessor(s);
		ArrayList<DeviceSubcomponent> sub_devices = aadlColAPI.System_GetSubcomponentsDevice(s);
		ArrayList<BusSubcomponent> sub_buses = aadlColAPI.System_GetSubcomponentsBus(s);
		ArrayList<MemorySubcomponent> sub_memorys = aadlColAPI.System_GetSubcomponentsMemory(s);

		// SystemSubcomponent
		ASME2SSME_SystemImpl_SubSystems(vprocess, sub_syss, vmodule, aadlpkg);

		//process subcomponent
		ASME2SSME_SystemImpl_SubProcesses(vprocess, sub_processes, vmodule, aadlpkg);

		// DataSubcomponent
		ASME2SSME_SystemImpl_SubData(vprocess, sub_data);

		// ProcessorSubcomponent
		ASME2SSME_SystemImpl_SubProcessors(vprocess, sub_processors, s);

		// DeviceSubcomponent
		ASME2SSME_SystemImpl_SubDevices(vprocess, sub_devices);

		// BusSubcomponent
		ASME2SSME_SystemImpl_SubBuses(vprocess, sub_buses);

		//MemorySubcomponent
		ASME2SSME_SystemImpl_SubMemorys(vprocess, sub_memorys);

		//Subprogram subcomponent not exist in version 1.5.8

		//subprogram group subcomponent not exist in version 1.5.8

		//connections 	
		ASME2SSME_SystemImpl_Connections(s, vprocess);

		//Flows
		ASME2SSME_SystemImpl_Flows(vprocess, s);

		//Modes 
		ASME2SSME_SystemImpl_Modes(s);

		//Properties : properties implemented in the SystemImpl_Property() subprocess

		Trace("\n ======= end ASME2SSME_SystemImpl: " + sname + " ======= \n");				
		return vprocess;
	}

	/** 
	 * Returns a Signal process that represents an AADL system type
	 *<p>
	 * the ASME2SSME_ComponentInterface() method is called, 
	 * and the in/out ports of the type features are transformed as the input/outputs of the interface
	 * 
	 * @param systemType an AADL system type component
	 * @return a Signal process
	 */
	private ProcessModelImpl ASME2SSME_SystemType(SystemType systemType){

		String sname = aadlColAPI.ComponentType_GetName(systemType);
		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemType: " + sname);

		ProcessModelImpl vprocess;
		vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(sname) );
		asme2ssme_common.ASME2SSME_ComponentInterface(vprocess, systemType);
		//features 
		ASME2SSME_System_Features(systemType, vprocess);
		//flow specifications
		ASME2SSME_System_FlowSpecs(vprocess, systemType);

		//modes : no modes in version 1.5.8

		//properties : properties implemented in the SystemImpl_Property() sub process

		if (! myLib.STree_IsProcessHasOutput(vprocess) && (myLib.STree_IsProcessNull(vprocess))) 
			myLib.STree_addNullProc(vprocess);
		Trace("\n ======= end ASME2SSME_SystemType: " + sname + " ======= \n");
		return vprocess;		
	}


	/**
	 * Translates system features.
	 * Remember that the ports are implemented as a signal in the interface.
	 * <p>
	 * For each feature, use its corresponding translate method
	 * 
	 * @param syst a system type
	 */
	private void ASME2SSME_System_Features(SystemType syst, ProcessModelImpl vprocess){

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_System_Features: " + syst.getName());
		ASME2SSME_Translate_Feature asme2ssme_feature = new ASME2SSME_Translate_Feature();

		// ports : implemented as a signal in the interface 
		
		// feature group in OSATE2 
		ArrayList<FeatureGroup> fgroup_list = aadlColAPI.Component_GetFeatureGroups(syst);
		ASME2SSME_Translate_FeatureGroup asme2ssme_fg = new ASME2SSME_Translate_FeatureGroup();
		for(int i1=0; i1<fgroup_list.size(); i1++)
		{
			FeatureGroup pg = fgroup_list.get(i1);

			if(asme2ssme_fg.ASME2SSME_FeatureGroup_bw(pg) != null){
				SignalsDeclarationImpl in1 = asme2ssme_fg.ASME2SSME_FeatureGroup_bw(pg);
				myLib.STree_addDeclarationToInterface(vprocess, in1, 2);
			}

			if(asme2ssme_fg.ASME2SSME_FeatureGroup_wb(pg) != null){
				SignalsDeclarationImpl out1 = asme2ssme_fg.ASME2SSME_FeatureGroup_wb(pg);
				myLib.STree_addDeclarationToInterface(vprocess, out1, 3);
			}
		}

		// subprogram group : not exist in version 1.5.8

		// bus access 
		EList<BusAccess> ba_list = syst.getOwnedBusAccesses();
		if(ba_list.size()>0){
			for(int i3=0; i3<ba_list.size(); i3++)
			{
				BusAccess ba1 = ba_list.get(i3);
				asme2ssme_feature.ASME2SSME_BusAccess(ba1);			
			}
		}
		// data access 
		EList<DataAccess> da_list = syst.getOwnedDataAccesses();
		if(da_list.size()>0){
			for(int i4=0; i4<da_list.size(); i4++)
			{
				DataAccess da1 = da_list.get(i4);
				asme2ssme_feature.ASME2SSME_DataAccess(da1);			
			}
		}
	}


	/**
	 * Translates flow specifications of a system type
	 * 
	 * @param syst a system type component
	 */
	private void ASME2SSME_System_FlowSpecs(ProcessModelImpl vprocess, SystemType syst){

		Trace("\n == ASME2SSME_System_FlowSpecs: ASME2SSME_System_FlowSpecifications: " + syst.getName());

		if(asme2ssme_common.ASME2SSME_ComponentType_FlowSpecs(syst) != null){
			ProcessModelImpl p1= asme2ssme_common.ASME2SSME_ComponentType_FlowSpecs(syst);

			myLib.STree_addProcessDeclaration(vprocess, p1);
			ArrayList<ExprParam> lparam = new ArrayList<ExprParam> ();
			String pname = Name_of_Flow(syst.getName());
			ProcessExpansionImpl call1 = myLib.STree_mkProcessCallWithParameters(myLib.mkIdentifier(pname), lparam);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, call1);
		}
	}




	/** 
	 * Returns a "xx_System_behavior" process
	 *
	 * @param sys_name the name of the system process
	 * @return a Signal process (ProcessModelImpl object) which represents the behavior part of the system
	 */
	private ProcessModelImpl ASME2SSME_System_behavior(String sys_name){

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_System_behavior: " + sys_name);

		String bname = sys_name + "_behavior";
		ProcessModelImpl sys_behavior = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(bname) );
		if ((! myLib.STree_IsProcessHasOutput(sys_behavior)) && (myLib.STree_IsProcessNull(sys_behavior)))
			myLib.STree_addNullProc(sys_behavior);
		return sys_behavior;
	}


	/** 
	 * Returns a "xx_System_property" process 
	 * <p>
	 * this Signal process represents the property part of a system implementation. 
	 * Each property specified in this system implementation is represented as a process call
	 * 
	 * @param sys_impl a system implementation
	 * @param sys_name the name of this system implementation Signal process
	 * @return a Signal process
	 */
	private ProcessModelImpl ASME2SSME_System_property(SystemImplementation sys_impl, String sys_name){

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_System_property: " + sys_name);

		String pname = sys_name + "_property";
		ProcessModelImpl sys_property = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );

		ArrayList<PropertyAssociation> list1 = aadlColAPI.Component_GetPropertyAssociations(aadlColAPI.ComponentImpl_GetComponentType(sys_impl));
		ArrayList<PropertyAssociation> property_association_list = aadlColAPI.ComponentImpl_GetPropertyAssociations(sys_impl);
		if(list1.size()>0) property_association_list.addAll(list1);

		for(int i=0; i<property_association_list.size(); i++)
		{
			PropertyAssociation pa = property_association_list.get(i);
			String propertyDef_name = aadlColAPI.Property_Get_Name(pa);

			if (propertyDef_name.equals("Latency")) 
				Trace("property for process connection is implemented in the connection_property");
			else 
			{
				ProcessCallImpl pcall1 = ASME2SSME_SystemImpl_Property(pa);
				myLib.STree_addExpressionOfProcessToaModel(sys_property, pcall1); 
			}   
		}
		if(myLib.STree_IsProcessNull(sys_property)) myLib.STree_addNullProc(sys_property);		
		return sys_property;
	}


	private void ASME2SSME_SystemImpl_Interface(SystemImplementation s, ProcessModelImpl vprocess){

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemImpl_Interface: " + s.getName());

		SystemType st = (SystemType) aadlColAPI.ComponentImpl_GetComponentType(s);
		asme2ssme_common.ASME2SSME_ComponentInterface(vprocess, st);

		//for each sub processor xx, add an input event xx_top in the interface
		ArrayList<ProcessorSubcomponent> list_subp = aadlColAPI.System_GetSubcomponentsProcessor(s);
		for(int i=0; i<list_subp.size(); i++ ){
			ProcessorSubcomponent subp = list_subp.get(i);
			String subp_name = aadlColAPI.Subcomponent_GetName(subp);
			asme2ssme_common.ASME2SSME_add_input_to_process_interface(subp_name +"_top", "event", vprocess); 
		}

		//for each sub system yy, for each its sub processor zz, add an input event top_n (which represents yy_zz_top) in the interface
		ArrayList<SystemSubcomponent> list2 = aadlColAPI.System_GetSubcomponentsSystem(s);
		for(int j=0; j<list2.size(); j++ ){
			SystemSubcomponent sub1 = list2.get(j);
			String name1 = aadlColAPI.Subcomponent_GetName(sub1);
			SystemClassifier sc = aadlColAPI.Systemsubcomponent_GetClassifier(sub1);
			//for each sub system (if it is SystemImplementation)
			if(sc instanceof SystemImplementation){
				SystemImplementation sc1 = (SystemImplementation) sc;
				ArrayList<ProcessorSubcomponent> list3 = aadlColAPI.System_GetSubcomponentsProcessor(sc1);
				//for each sub processor
				for(int k=0; k<list3.size(); k++ ){
					ProcessorSubcomponent sub3 = list3.get(k);
					String name3 = aadlColAPI.Subcomponent_GetName(sub3);
					String top_sname = asme2ssme_common.Get_top_sname(name1 + "_" + name3 +"_top");
					asme2ssme_common.ASME2SSME_add_input_to_process_interface(top_sname, "event", vprocess); 
				}
			}
		}
	}



	/** Translates the sub systems of a system implementation.
	 * <p>
	 * For each system subcomponent, get its classifier (a system type or implementation). 
	 * If this classifier has already been implemented, get its Signal process declaration, 
	 * otherwise, create a Signal process (ASME2SSME_SystemType() or ASME2SSME_SystemImpl()). 
	 * Add an instance of this Signal process in the body of sys_process.
	 * 
	 * @param sys_process a Signal process that represents a system implementation component
	 * @param subsys_list a list of system subcomponents of a system 
	 */
	private void ASME2SSME_SystemImpl_SubSystems(ProcessModelImpl sys_process, ArrayList<SystemSubcomponent> subsys_list, 
			ModuleImpl vmodule, AadlPackage aadlpkg){

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemImpl_SubSystems " );

		ArrayList<SystemClassifier> l_classifier = new ArrayList<SystemClassifier>();
		if(subsys_list.size()>0){

			Iterator<?> it1 = subsys_list.iterator();
			while (it1.hasNext())
			{
				SystemSubcomponent s1 = (SystemSubcomponent) it1.next();
				String subsys_name = aadlColAPI.Subcomponent_GetName(s1);	
				ProcessModelImpl aprocess;
				//SystemClassifier sys_classifier = aadlColAPI.Systemsubcomponent_GetClassifier(s1);

				if(aadlColAPI.Systemsubcomponent_GetClassifier(s1) instanceof SystemType){
					SystemType sys_classifier = (SystemType) aadlColAPI.Systemsubcomponent_GetClassifier(s1);
					String system_impl_name = aadlColAPI.ComponentType_GetName(sys_classifier);
					String system_impl_process_name = Name_of_System(system_impl_name);

					if (l_classifier.contains(sys_classifier) ){ 
						Trace("system classifier has already been created ");
						aprocess = myLib.STree_GetProcessDeclarationFromProcess(sys_process, system_impl_process_name);
					}
					else { //system classifier has not been created in Signal

						aprocess = ASME2SSME_SystemType((SystemType) sys_classifier);
						myLib.STree_addProcessDeclaration(sys_process, aprocess);
						l_classifier.add(sys_classifier);
					} 
				}
				else{
					SystemImplementation sys_classifier = (SystemImplementation) aadlColAPI.Systemsubcomponent_GetClassifier(s1);
					String system_impl_name = aadlColAPI.ComponentImpl_GetName(sys_classifier);
					String system_impl_process_name = Name_of_System(system_impl_name);

					if (l_classifier.contains(sys_classifier) ){ 
						Trace("system classifier has already been created ");
						aprocess = myLib.STree_GetProcessDeclarationFromProcess(sys_process, system_impl_process_name);
					}
					else { //system classifier has not been created in Signal

						// processImpl declaration
						if (sys_classifier instanceof SystemType){
							aprocess = ASME2SSME_SystemType((SystemType) sys_classifier);
						}
						else if (sys_classifier instanceof SystemImplementation){
							aprocess = ASME2SSME_SystemImpl((SystemImplementation) sys_classifier, vmodule, aadlpkg);
						}
						else{
							aprocess = new ProcessModelImpl();
							java.lang.System.err.print("\n system classifier illegal!! ");
							throw new IllegalArgumentException(" system classifier illegal!!  \n");
						}
						myLib.STree_addProcessDeclaration(sys_process, aprocess);
						l_classifier.add(sys_classifier);
					} 
				}

				//add a system subcomponent instance (process call) to the process
				asme2ssme_common.ASME2SSME_component_instance(sys_process, aprocess, subsys_name);

				//add equation for the top
				ASME2SSME_SystemImpl_Add_Top_Equation(sys_process, s1);
			}
		}
	}


	/** Translates the sub processes of a system implementation.
	 * <p>
	 * For each process subcomponent, get its classifier (a process implementation). 
	 * If this classifier has already been implemented, get its Signal process declaration, 
	 * otherwise, create a Signal process type for this process component's type (ASME2SSME_ProcessType()), 
	 * and a Signal process (ASME2SSME_ProcessImpl()) for this process component's implementaton, 
	 * and add them to the declaration of sys_process. 
	 * 
	 * @param sys_process a Signal process that represents a system implementation component
	 * @param subprocess_list a list of process subcomponents of a system 
	 */
	private void ASME2SSME_SystemImpl_SubProcesses(ProcessModelImpl sys_process, ArrayList<ProcessSubcomponent> subprocess_list, 
			ModuleImpl vmodule, AadlPackage aadlpkg){

		ASME2SSME_Translate_Process asme2ssme_process = new ASME2SSME_Translate_Process();

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemImpl_SubProcesses " );

		ArrayList<ProcessClassifier> l_process_classifier = new ArrayList<ProcessClassifier>();
		Iterator<?> it2 = subprocess_list.iterator();
		while (it2.hasNext())
		{
			ProcessSubcomponent p1 = (ProcessSubcomponent) it2.next();
			// String p_sub_name = aadlColAPI.Subcomponent_GetName(p1);

			// ProcessImpl
			ProcessImplementation p = (ProcessImplementation) aadlColAPI.Processsubcomponent_GetClassifier(p1);
			String process_impl_name = aadlColAPI.ComponentImpl_GetName(p);
			String process_impl_process_name = Name_of_Process(process_impl_name);

			ProcessModelImpl lprocess4;
			if (l_process_classifier.contains(p)){
				Trace("process classifier has already been created in Signal!!");
				lprocess4 = myLib.STree_GetProcessDeclarationFromProcess(sys_process, process_impl_process_name);
			}
			else {

				//create a Signal type process for this AADL process
				org.osate.aadl2.ProcessType ptype = (org.osate.aadl2.ProcessType) aadlColAPI.ComponentImpl_GetComponentType(p);
				TypesDeclarationImpl type_decl = asme2ssme_process.ASME2SSME_ProcessType(ptype, p);
				myLib.STree_addTypeDeclarationToProcess(sys_process, (TypesDeclaration) type_decl);

				// processImpl declaration
				if(p instanceof org.osate.aadl2.ProcessType) 
					lprocess4 = new ProcessModelImpl(); 
				else  
					lprocess4 = asme2ssme_process.ASME2SSME_ProcessImpl(p, sys_process, vmodule, aadlpkg);			
				myLib.STree_addProcessDeclaration(sys_process, lprocess4);					   
				l_process_classifier.add(p);
			}
			//ASME2SSME_component_instance(sys_process, lprocess4, p_sub_name);
		}	
	}


	/** Translates the sub data of a system implementation.
	 * <p>
	 * For each data subcomponent, get its classifier (a data type or implementation). 
	 * If this classifier has not been implemented, 
	 * create a Signal type declaration for this data (ASME2SSME_DataType() or ASME2SSME_DataImpl()), 
	 * and add it to the declaration of sys_process. 
	 * 
	 * @param sys_process a Signal process that represents a system implementation component
	 * @param subdata_list a list of data subcomponents of a system 
	 */
	private void ASME2SSME_SystemImpl_SubData(ProcessModelImpl sys_process, ArrayList<DataSubcomponent> subdata_list){

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemImpl_SubData " );

		ArrayList<DataClassifier> l_data_classifier = new ArrayList<DataClassifier>();
		Iterator<DataSubcomponent> it_data = subdata_list.iterator();
		while (it_data.hasNext())
		{
			DataSubcomponent data1 = (DataSubcomponent) it_data.next();                           
			DataClassifier data_classifier = (DataClassifier) aadlColAPI.Datasubcomponent_GetClassifier(data1);

			if (l_data_classifier.contains(data_classifier)){
				Trace("data classifier has already been created in Signal!!");
			}
			else {
				TypesDeclarationImpl type_decl = new TypesDeclarationImpl();
				ASME2SSME_Translate_Data asme2ssme_data = new ASME2SSME_Translate_Data();
				if (data_classifier instanceof DataType)
				{
					if (asme2ssme_data.ASME2SSME_DataType((DataType) data_classifier) != null){
						type_decl = asme2ssme_data.ASME2SSME_DataType((DataType) data_classifier);
						myLib.STree_addaDeclarationToProcess(sys_process, type_decl);
						l_data_classifier.add(data_classifier);
					}
					else ; // ?? LB
				}
				else {
					type_decl = asme2ssme_data.ASME2SSME_DataImpl((DataImplementation) data_classifier);
					myLib.STree_addaDeclarationToProcess(sys_process, type_decl);
					l_data_classifier.add(data_classifier);
				}		     
			}
		}
	}


	/** Translates the sub processors of a system implementation.
	 * <p>
	 * For each processor subcomponent, get its classifier (a processor implementation). 
	 * Create a Signal process type for this processor (ASME2SSME_ProcessorImpl()), 
	 * and add it to the declaration of sys_process. 
	 * Create an instance in the body of sys_process
	 * 
	 * @param sys_process a Signal process that represents a system implementation component
	 * @param subprocessor_list a list of processor subcomponents of a system 
	 * @param sys_impl a system implementation component
	 */
	private void ASME2SSME_SystemImpl_SubProcessors(ProcessModelImpl sys_process, 
			ArrayList<ProcessorSubcomponent> subprocessor_list, SystemImplementation sys_impl){

		ASME2SSME_Translate_Processor asme2ssme_processor = new ASME2SSME_Translate_Processor();

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemImpl_SubProcessors " );

		ArrayList<ProcessorClassifier> l_processor_classifier = new ArrayList<ProcessorClassifier>();
		Iterator<?> it3 = subprocessor_list.iterator();
		while (it3.hasNext())
		{
			ProcessorSubcomponent pr1 = (ProcessorSubcomponent) it3.next();
			//suppose the processor classifier is always a processor implementation 
			ProcessorClassifier processor_impl = aadlColAPI.Processorsubcomponent_GetClassifier(pr1);
			// String processor_impl_name;
			// if (processor_impl instanceof ProcessorType)
			// processor_impl_name = aadlColAPI.ComponentType_GetName((ProcessorType) processor_impl);
			// else processor_impl_name = aadlColAPI.ComponentImpl_GetName((ProcessorImplementation)processor_impl);

			ProcessModelImpl aprocess;			    
			// processImpl declaration
			if (processor_impl instanceof ProcessorType){
				aprocess = new ProcessModelImpl();
				Trace("processor_impl instanceof ProcessorType not implemented ");
			}
			else if (processor_impl instanceof ProcessorImplementation){
				aprocess = asme2ssme_processor.ASME2SSME_ProcessorImpl((ProcessorImplementation)processor_impl, pr1, sys_impl, sys_process);
			}
			else{
				aprocess = new ProcessModelImpl();
				java.lang.System.err.print("\n processor classifier illegal !! ");
				throw new IllegalArgumentException(" processor classifier illegal !!  \n");
			}
			myLib.STree_addProcessDeclaration(sys_process, aprocess);
			l_processor_classifier.add(processor_impl);
			ASME2SSME_Processor_instance(sys_process, aprocess, sys_impl, pr1);
		}
	}


	/** Translates the sub devices of a system implementation.
	 * <p>
	 * For each device subcomponent, get its classifier (a device type or implementation). 
	 * If this classifier has already been implemented, get its Signal process declaration, 
	 * otherwise, create a Signal process (ASME2SSME_DeviceType() or ASME2SSME_DeviceImpl()). 
	 * Add an instance of this Signal process in the body of sys_process.
	 * 
	 * @param sys_process a Signal process that represents a system implementation component
	 * @param subdevice_list a list of device subcomponents of a system 
	 */
	private void ASME2SSME_SystemImpl_SubDevices(ProcessModelImpl sys_process, ArrayList<DeviceSubcomponent> subdevice_list){

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemImpl_SubDevices " );

		if(subdevice_list.size()>0){

			ArrayList<DeviceClassifier> l_device_classifier = new ArrayList<DeviceClassifier>();
			Iterator<?> it4 = subdevice_list.iterator();
			while (it4.hasNext())
			{
				DeviceSubcomponent d1 = (DeviceSubcomponent) it4.next();
				String subdevice_name = aadlColAPI.Subcomponent_GetName(d1);				   
				DeviceImplementation device_classifier = (DeviceImplementation) aadlColAPI.Devicesubcomponent_GetClassifier(d1);
				String device_impl_name = aadlColAPI.ComponentImpl_GetName(device_classifier);
				String device_impl_process_name = Name_of_Device(device_impl_name);

				ProcessModelImpl aprocess;
				if (l_device_classifier.contains(device_classifier) ){ 
					Trace("device classifier has already been created ");
					aprocess = myLib.STree_GetProcessDeclarationFromProcess(sys_process, device_impl_process_name);
				}
				else { //device classifier has not been created in Signal processImpl declaration
					ASME2SSME_Translate_Device asme2ssme_device = new ASME2SSME_Translate_Device(); 
					if (device_classifier instanceof DeviceType){
						aprocess = asme2ssme_device.ASME2SSME_DeviceType((DeviceType) device_classifier);
					}
					else if (device_classifier instanceof DeviceImplementation){
						aprocess = asme2ssme_device.ASME2SSME_DeviceImpl((DeviceImplementation) device_classifier);
					}
					else{
						aprocess = new ProcessModelImpl();
						java.lang.System.err.print("\n device classifier illegal !! ");
						throw new IllegalArgumentException(" device classifier illegal !!  \n");
					}
					myLib.STree_addProcessDeclaration(sys_process, aprocess);
					l_device_classifier.add(device_classifier);
				} 
				asme2ssme_common.ASME2SSME_component_instance(sys_process, aprocess, subdevice_name);
			}
		}
	}


	/** Translates the sub buses of a system implementation.
	 * <p>
	 * For each bus subcomponent, get its classifier (a device type or implementation). 
	 * If this classifier has already been implemented, get its Signal process declaration, 
	 * otherwise, create a Signal process (ASME2SSME_BusImpl()). 
	 * Add an instance of this Signal process in the body of sys_process.
	 * 
	 * @param sys_process a Signal process that represents a system implementation component
	 * @param subbus_list a list of bus subcomponents of a system 
	 */
	private void ASME2SSME_SystemImpl_SubBuses(ProcessModelImpl sys_process, ArrayList<BusSubcomponent> subbus_list){

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemImpl_SubBuses " );

		ArrayList<BusClassifier> l_bus_classifier = new ArrayList<BusClassifier>();
		Iterator<?> it5 = subbus_list.iterator();
		while (it5.hasNext())
		{
			BusSubcomponent b1 = (BusSubcomponent) it5.next();
			String subbus_name = aadlColAPI.Subcomponent_GetName(b1);
			//suppose the bus classifier is always a bus implementation 
			BusClassifier bus_cl = aadlColAPI.Bussubcomponent_GetClassifier(b1);
			String bus_name;
			if(bus_cl instanceof BusType) 
				bus_name = aadlColAPI.ComponentType_GetName((BusType) bus_cl);
			else
				bus_name = aadlColAPI.ComponentImpl_GetName((BusImplementation) bus_cl);
			String bus_process_name = Name_of_Bus(bus_name);

			ProcessModelImpl aprocess;
			if (l_bus_classifier.contains(bus_cl) ){ 
				Trace("bus classifier has already been created ");				
				aprocess = myLib.STree_GetProcessDeclarationFromProcess(sys_process, bus_process_name);
			}
			else { //bus classifier has not been created in Signal					    
				// processImpl declaration
				ASME2SSME_Translate_Bus asme2ssme_bus = new ASME2SSME_Translate_Bus();
				if (bus_cl instanceof BusType){
					aprocess = asme2ssme_bus.ASME2SSME_BusType((BusType) bus_cl);
				}
				else if (bus_cl instanceof BusImplementation){
					aprocess = asme2ssme_bus.ASME2SSME_BusImpl((BusImplementation) bus_cl);
				}
				else{
					aprocess = new ProcessModelImpl();
					java.lang.System.err.print("\n bus classifier illegal !! ");
					throw new IllegalArgumentException(" bus classifier illegal !!  \n");
				}
				myLib.STree_addProcessDeclaration(sys_process, aprocess);
				l_bus_classifier.add(bus_cl);
			} 
			asme2ssme_common.ASME2SSME_component_instance(sys_process, aprocess, subbus_name);
		}
	}


	/**
	 * Translates the sub memorys of a system implementation.
	 * <p>
	 * For each memory subcomponent, get its classifier (a memory type or implementation). 
	 * If this classifier has already been implemented, get its Signal process declaration, 
	 * otherwise, create a Signal process (ASME2SSME_MemoryImpl()). 
	 * Add an instance of this Signal process in the body of sys_process.
	 * 
	 * @param sys_process
	 * @param list1
	 */
	private void ASME2SSME_SystemImpl_SubMemorys(ProcessModelImpl sys_process, ArrayList<MemorySubcomponent> list1){

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemImpl_SubMemorys " );

		ArrayList<MemoryClassifier> l_m_classifier = new ArrayList<MemoryClassifier>();
		if(list1.size()>0){

			Iterator<?> it = list1.iterator();
			while (it.hasNext())
			{
				MemorySubcomponent ms1 = (MemorySubcomponent) it.next();
				String subm_name = aadlColAPI.Subcomponent_GetName(ms1); 
				MemoryClassifier mc = (MemoryClassifier) aadlColAPI.Memorysubcomponent_GetClassifier(ms1);
				ProcessModelImpl aprocess;

				if(mc instanceof MemoryType){
					MemoryType m_type = (MemoryType) mc;
					String mt_name = aadlColAPI.ComponentType_GetName(m_type);
					String mt_process_name = Name_of_Memory(mt_name);
					if (l_m_classifier.contains(m_type) ){ 
						Trace("memory classifier (type) has already been created ");
						aprocess = myLib.STree_GetProcessDeclarationFromProcess(sys_process, mt_process_name);
					}
					else{//this memeory type has been created in Signal
						ASME2SSME_Translate_Memory asme2ssme_memory = new ASME2SSME_Translate_Memory();
						aprocess = asme2ssme_memory.ASME2SSME_MemoryType(m_type);
						myLib.STree_addProcessDeclaration(sys_process, aprocess);
					}
				}
				else{ //MemoryImpl
					MemoryImplementation m_imp = (MemoryImplementation) mc;
					String mi_name = aadlColAPI.ComponentImpl_GetName(m_imp);
					String mi_process_name = Name_of_Memory(mi_name);
					if (l_m_classifier.contains(m_imp) ){ 
						Trace("memory classifier (implementation) has already been created ");
						aprocess = myLib.STree_GetProcessDeclarationFromProcess(sys_process, mi_process_name);
					}
					else{ //this memeory type has been created in Signal
						ASME2SSME_Translate_Memory asme2ssme_memory = new ASME2SSME_Translate_Memory();
						aprocess = asme2ssme_memory.ASME2SSME_MemoryImpl(m_imp);
						myLib.STree_addProcessDeclaration(sys_process, aprocess);
					}
				}

				asme2ssme_common.ASME2SSME_component_instance(sys_process, aprocess, subm_name);
			}
		}
	}


	/**
	 * Translate the connections in a system implementation component
	 * @param s a system implementation component
	 * @param vprocess a Signal process that represents this AADL system implementation component
	 */
	private void ASME2SSME_SystemImpl_Connections(SystemImplementation s, ProcessModelImpl vprocess){

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemImpl_Connections: " + s.getName());

		ArrayList<ProcessorSubcomponent> sub_processors = aadlColAPI.System_GetSubcomponentsProcessor(s);

		if(aadlColAPI.ComponentImpl_GetConnections(s) !=null){

			ASME2SSME_Translate_Connection asme2ssme_connection = new ASME2SSME_Translate_Connection();

			//port connection
			if(aadlColAPI.ComponentImpl_GetPortConnections(s) != null) {

				// DataPortConnections 
				ArrayList<PortConnection> listDataConnections = aadlColAPI.ComponentImpl_GetDataPortConnections(s);
				for(int k =0; k< sub_processors.size(); k++){
					ProcessorSubcomponent a_sub_processor = sub_processors.get(k);
					ArrayList<PortConnection> list1 = aadlColAPI.Get_ProcessDataConnection_SameProcessor(s, a_sub_processor);
					listDataConnections.removeAll(list1);
				}
				if(listDataConnections.size()>0){

					ASME2SSME_Translate_PortConnection asme2ssme_pconnection = new ASME2SSME_Translate_PortConnection();
					Iterator<?> itListDataConnections = listDataConnections.iterator();
					while (itListDataConnections.hasNext())
					{
						PortConnection dataConnection = (PortConnection) itListDataConnections.next();
						// read_DataConnection(dataConnection);	
						if(asme2ssme_pconnection.ASME2SSME_DataPortConnection(s,vprocess, dataConnection) != null){
							SignalsDefinitionImpl eq1 = asme2ssme_pconnection.ASME2SSME_DataPortConnection(s,vprocess, dataConnection);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
						}  			
					}
				}

				// EventPortConnections 
				ArrayList<PortConnection> listEventConnections = aadlColAPI.ComponentImpl_GetEventPortConnections(s);
				for(int k =0; k< sub_processors.size(); k++){
					ProcessorSubcomponent a_sub_processor = sub_processors.get(k);
					ArrayList<PortConnection> list2 = aadlColAPI.Get_ProcessEventConnection_SameProcessor(s, a_sub_processor);
					listEventConnections.removeAll(list2);
				}
				if(listEventConnections.size()>0){

					Trace("event port connections > 0");
					ASME2SSME_Translate_PortConnection asme2ssme_pconnection2 = new ASME2SSME_Translate_PortConnection();
					Iterator<?> itListEventConnections = listEventConnections.iterator();
					while (itListEventConnections.hasNext())
					{
						PortConnection eventConnection = (PortConnection) itListEventConnections.next();
						if(asme2ssme_pconnection2.ASME2SSME_EventPortConnection(s,vprocess,eventConnection) != null){
							SignalsDefinitionImpl eq2 = asme2ssme_pconnection2.ASME2SSME_EventPortConnection(s,vprocess,eventConnection);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq2);
						}
					}
				}

				// EventDataPortConnections
				ArrayList<PortConnection> listEventDataConnections = aadlColAPI.ComponentImpl_GetEventDataPortConnections(s);
				for(int k =0; k< sub_processors.size(); k++){
					ProcessorSubcomponent a_sub_processor = (ProcessorSubcomponent) sub_processors.get(k);
					ArrayList<PortConnection> list3 = aadlColAPI.Get_ProcessEventDataConnection_SameProcessor(s, a_sub_processor);
					listEventDataConnections.removeAll(list3);
				}
				if(listEventDataConnections.size()>0){

					ASME2SSME_Translate_PortConnection asme2ssme_pconnection3 = new ASME2SSME_Translate_PortConnection();
					Iterator<?> itListEventDataConnections = listEventDataConnections.iterator();
					while (itListEventDataConnections.hasNext())
					{
						PortConnection eventdata_Connection = (PortConnection) itListEventDataConnections.next();
						if(asme2ssme_pconnection3.ASME2SSME_EventDataPortConnection(s,vprocess,eventdata_Connection) != null){
							SignalsDefinitionImpl eq3 = asme2ssme_pconnection3.ASME2SSME_EventDataPortConnection(s,vprocess,eventdata_Connection);
							myLib.STree_addExpressionOfProcessToaModel(vprocess, eq3);
						}
					}
				}
			}

			//access connections
			if(s.getOwnedAccessConnections() != null){
				EList<AccessConnection> list1 = s.getOwnedAccessConnections();

				if(list1.size()>0){
					for(int i1 =0; i1 < list1.size(); i1++ ){
						AccessConnection c1 = list1.get(i1);

						//bus access connection
						if(c1.getAccessCategory().toString() == "bus"){
							asme2ssme_connection.ASME2SSME_BusAccessConnection(c1);
						}
						//data access connection
						else if(c1.getAccessCategory().toString() == "data"){
							asme2ssme_connection.ASME2SSME_DataAccessConnection(c1);
						}
					}
				}   		
			}

			//feature group connections
			ArrayList<FeatureGroupConnection> list3 = aadlColAPI.ComponentImpl_GetFeatureGroupConnections(s);
			ASME2SSME_Translate_FeatureGroupConnection asme2ssme_fgc = new ASME2SSME_Translate_FeatureGroupConnection();
			for(int i3= 0; i3< list3.size(); i3++){
				FeatureGroupConnection c3 = list3.get(i3);
				if( asme2ssme_fgc.ASME2SSME_FeatureGroupConnection1(c3) != null){
					SignalsDefinitionImpl eq3 = asme2ssme_fgc.ASME2SSME_FeatureGroupConnection1(c3);
					myLib.STree_addExpressionOfProcessToaModel(vprocess, eq3);
				}
				if( asme2ssme_fgc.ASME2SSME_FeatureGroupConnection2(c3) != null){
					SignalsDefinitionImpl eq4 = asme2ssme_fgc.ASME2SSME_FeatureGroupConnection2(c3);
					myLib.STree_addExpressionOfProcessToaModel(vprocess, eq4);
				}
			}

		}
	}


	/**
	 * Translates flows of a system implementation
	 * 
	 * @param s a system implementation component
	 */
	private void ASME2SSME_SystemImpl_Flows(ProcessModelImpl vprocess, SystemImplementation s){

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemImpl_Flows: " + s.getName());

		if(asme2ssme_common.ASME2SSME_ComponentImpl_Flows(s) != null){
			ProcessModelImpl p1= asme2ssme_common.ASME2SSME_ComponentImpl_Flows(s);

			myLib.STree_addProcessDeclaration(vprocess, p1);
			ArrayList<ExprParam> lparam = new ArrayList<ExprParam> ();
			String pname = Name_of_Flow(s.getName().replace('.','_'));
			ProcessExpansionImpl call1 = myLib.STree_mkProcessCallWithParameters(myLib.mkIdentifier(pname), lparam);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, call1);
		}
	}


	/**
	 * Translate modes of a system implementation component
	 * @param s
	 */
	private void ASME2SSME_SystemImpl_Modes(SystemImplementation s){

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemImpl_Modes: " + s.getName());

		ASME2SSME_Translate_Common asme2ssme_common = new ASME2SSME_Translate_Common();
		asme2ssme_common.ASME2SSME_ComponentImpl_Modes(s);
	}


	/** 
	 * Returns a Signal process call that represents a system's property
	 * <p>
	 * if "Actual_Processor_Binding" property, use ASME2SSME_Actual_Processor_Binding_Property()
	 * if "Actual_Connection_Binding" property, use ASME2SSME_Actual_Connection_Binding_property()
	 * else use ASME2SSME_Property_Call() for other properties
	 * 
	 * @param pa a property association defined for a system implementation
	 * @return a Signal process call
	 */
	private ProcessCallImpl ASME2SSME_SystemImpl_Property(PropertyAssociation pa){

		Property propertyDef = pa.getProperty();
		String propertyDef_name = propertyDef.getName();
		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_SystemImpl_Property: " + propertyDef_name);
		ProcessCallImpl pcall = ASME2SSME_Property_Call(pa);
		return pcall;
	}


	/** 
	 * Adds a process call of a Signal process that represetns a processor (pprocess) subcomponent into a system (Signal) process (vprocess)
	 * <p>
	 * Each formal declaration (parameter,input,output) of pprocess acts as the effective parameter of the process call. 
	 * Local signal declaratons are added.
	 * 
	 * @param vprocess a Signal process that represents a system implementation component
	 * @param pprocess a Signal process that represents a processor subcomponent
	 * @param sysimpl a system implementation component
	 * @param subprocessor a processor subcomponent
	 */
	private void ASME2SSME_Processor_instance(ProcessModelImpl vprocess, ProcessModelImpl pprocess, 
			SystemImplementation sysimpl, ProcessorSubcomponent subprocessor){

		Trace("\n == ASME2SSME_Translate_System: ASME2SSME_Processor_instance: " + subprocessor.getName());

		String instance_name = pprocess.getIdentifier().getAtomString();
		ArrayList<ProcessSubcomponent> list_process_subcomponent = aadlColAPI.Processor_Get_Binding_Process(sysimpl, subprocessor);

		ArrayList<ExprParam> l_param = new ArrayList<ExprParam>();//parameter (id) list for call the process pprocess
		ArrayList<ExprSignal> l_in = new ArrayList<ExprSignal>();//input (id) list for call the process pprocess
		ArrayList<ExprSignal> l_out = new ArrayList<ExprSignal>();//output (id) list for call the process pprocess
		ArrayList<Declaration> vdecl = new ArrayList<Declaration>();//local lists

		// xx_top
		l_in.add(myLib.mkIdentifier(subprocessor.getName() +"_top"));

		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) pprocess.getInterfaceDefinition();
		if (inter != null){

			// EList<TypesOrSignalsDeclaration> parameter_list = inter.getListFormalParameter();                       
			IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();

			for (int j =0; j<list_process_subcomponent.size(); j++)
			{
				ProcessSubcomponent process_sub = list_process_subcomponent.get(j);
				String param_name1 = Name_of_Process(process_sub.getClassifier().getName());
				l_param.add(myLib.mkIdentifier(param_name1));        	 
			}

			if (vioProfile != null) 
			{ 
				//inputs ========
				EList<TypesOrSignalsDeclaration> l_inputs = vioProfile.getInputs(); 
				Iterator<TypesOrSignalsDeclaration> it_i = l_inputs.iterator();
				while (it_i.hasNext())
				{
					SignalsDeclarationImpl ainput = (SignalsDeclarationImpl) it_i.next(); 
					EList<SignalNaming> list2 = ainput.getListSignal();
					Trace(ainput.toString());

					if (ainput.getType() instanceof PredefinedTypeImpl){
						// Trace("ainput.getType() instanceof PredefinedTypeImpl");
						// PredefinedTypeImpl type_2 = (PredefinedTypeImpl) ainput.getType();

						for (int i = 0; i < list2.size(); i++) {  
							SignalNaming sig_name2 = list2.get(i);
							Identifier id_sig_name2 = sig_name2.getIdentifier();
							String name2 = id_sig_name2.getAtomString();	    		         

							PredefinedTypeImpl ptype = myLib.STree_mkSamePredefinedType(ainput);
							SignalsDeclarationImpl sig2 = myLib.mkDecl( getSname(name2), ptype); 

							if (! name2.equals("top")) {
								l_in.add(myLib.mkIdentifier(getSname( name2)));
								if (name2.endsWith("_Complete") | name2.endsWith("_Error") ){ 			 
								}
								else 			    		   
									vdecl.add(sig2);
							}
						}
					}
					else{
						Identifier type_id = (Identifier) ainput.getType();
						String type_name = type_id.getAtomString();	    	   
						IdentifierImpl type2 = myLib.mkIdentifier(type_name);

						for (int i = 0; i < list2.size(); i++) {  
							SignalNaming sig_name2 = list2.get(i);
							Identifier id_sig_name2 = sig_name2.getIdentifier();
							String name2 = id_sig_name2.getAtomString();   		   
							String sn = getSname( name2);
							SignalsDeclarationImpl sig2 = myLib.mkDecl(sn, type2); 
							if (! name2.endsWith("top") ) 
							{			    		   
								l_in.add(myLib.mkIdentifier(sn ));
								vdecl.add(sig2);
							} 		              
						}
					}		    	   
				}

				//outputs
				EList<TypesOrSignalsDeclaration> l_outputs = vioProfile.getOutputs(); 
				Iterator<TypesOrSignalsDeclaration> it_o = l_outputs.iterator();
				while (it_o.hasNext())
				{
					SignalsDeclarationImpl aoutput = (SignalsDeclarationImpl) it_o.next(); 
					EList<SignalNaming> list3 = aoutput.getListSignal();

					if (aoutput.getType() instanceof PredefinedTypeImpl){
						// Trace("aoutput.getType() instanceof PredefinedTypeImpl");
						for (int i = 0; i < list3.size(); i++) {  
							SignalNaming sig_name3 = list3.get(i);
							Identifier id_sig_name3 = sig_name3.getIdentifier();
							String name3 = id_sig_name3.getAtomString();	    		   
							String sn3 = getSname( name3);
							PredefinedTypeImpl ptype3 = myLib.STree_mkSamePredefinedType(aoutput);
							SignalsDeclarationImpl sig3 = myLib.mkDecl(sn3, ptype3); 
							l_out.add(myLib.mkIdentifier(sn3));
							if (! (name3.endsWith("Dispatch") | name3.endsWith("Resume") | name3.endsWith("Deadline")
									| name3.endsWith("_Frozen_time_event") | name3.endsWith("_Output_time_event")))	    		   
								vdecl.add(sig3);
						}
					}
					else{
						Identifier type_id = (Identifier) aoutput.getType();
						String type_name = type_id.getAtomString();		    	   
						IdentifierImpl type3 = myLib.mkIdentifier(type_name);
						Trace(type3.getAtomString());

						for (int i = 0; i < list3.size(); i++) {  
							SignalNaming sig_name3 = list3.get(i);
							Identifier id_sig_name3 = sig_name3.getIdentifier();
							String name3 = id_sig_name3.getAtomString();  		       
							String sn3 = getSname( name3);
							SignalsDeclarationImpl sig3 = myLib.mkDecl(sn3, type3); 
							l_out.add(myLib.mkIdentifier(sn3));
							vdecl.add(sig3);
						}
					}
				}
			}
		}

		if (l_out.size()>0){
			SignalsDefinitionImpl call_eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(instance_name), l_param, l_in, l_out);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, call_eq);
			myLib.STree_addComment(call_eq, asme2ssme_common.Generate_Process_Instance_comment(instance_name), false);
		}
		else{
			ProcessCallImpl pcall = myLib.STree_mkCompleteProcessCall(myLib.mkIdentifier(instance_name), l_param, l_in);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, pcall);
			myLib.STree_addComment(pcall, asme2ssme_common.Generate_Process_Instance_comment(instance_name), false);
		}
		myLib.STree_addLocalDeclaration(vprocess, vdecl);
		Trace("\n ======= end ASME2SSME_Processor_instance: " + subprocessor.getName() + " ======= \n");
	}



	/**
	 * add top equations:
	 * if subsystem (subsys) refers to a system implementation, and it contains processor subcomponents, 
	 * then for each sub processor p, add an equation in the vprocess body: l_xx := top_yy (which represents subsys_p_top), 
	 * where l_xx is a local signal generated during the creation of subsystem instance
	 * 
	 * @param vprocess a Signal process that represents the upper AADL system
	 * @param subsys a subsystem
	 */
	private void ASME2SSME_SystemImpl_Add_Top_Equation(ProcessModelImpl vprocess, SystemSubcomponent subsys){

		Trace("\n ######## ASME2SSME_Translate_System: ASME2SSME_SystemImpl_Add_Top_Equation : " + subsys.getName());

		//for a sub system subsys, for each its sub processor zz, add an input event yy_zz_top in the interface
		SystemClassifier sc = aadlColAPI.Systemsubcomponent_GetClassifier(subsys);

		if (sc instanceof SystemImplementation){
			SystemImplementation sc1 = (SystemImplementation) sc;
			ArrayList<ProcessorSubcomponent> list3 = aadlColAPI.System_GetSubcomponentsProcessor(sc1);
			//for each sub processor
			for(int k=0; k<list3.size(); k++ ){
				ProcessorSubcomponent sub3 = list3.get(k);
				//L_xx := top_xx 
				String top_lname = aadlColAPI.Subcomponent_GetName(subsys) + "_" + aadlColAPI.Subcomponent_GetName(sub3) +"_top";
				String sn = getSname( top_lname);
				//String top_sname = getSname( sn);
				String top_sname = asme2ssme_common.Get_top_sname(top_lname);

				IdentifierImpl leftExp = myLib.mkIdentifier(sn);
				IdentifierImpl rightExp = myLib.mkIdentifier(top_sname);
				SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation(leftExp, rightExp);
				myLib.STree_addExpressionOfProcessToaModel(vprocess, eq);
			}
		}
	}
}
