 
package fr.inria.tea.aadl2ssme.transformation.popup.actions;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.osate.aadl2.*;
import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;
 
abstract public class ASME2SSME_Translate_Thread extends ASME2SSME_Translate_Property {
	public static final String NOT_YET_IMPLEMENTED = "<<< NOT YET IMPLEMENTED >>>";
	
	// Signal labels used for the translation of the Threads (behavior).
	public static final String LABEL1 = "e1";
	public static final String LABEL2 = "e2";

	// Signal types
	public static final String SIGNAL_EVENT_TYPE = "event";
	public static final String SIGNAL_BOOLEAN_TYPE = "boolean";

	protected signalTreeAPI myLib;
	protected AADLCoLAPI aadlColAPI;
	protected ASME2SSME_Translate_Common asme2ssme_common;

	abstract String get_Thread_type();
	abstract void ASME2SSME_Dispatch_Constraint(ThreadImplementation timpl, ProcessModelImpl t_property);
	abstract void ASME2SSME_FixProperties(ThreadImplementation timpl, ProcessModelImpl t_property);
	
	/**
	 * Class constructor
	 */
	public ASME2SSME_Translate_Thread() 
	{
		myLib= new signalTreeAPI();
		aadlColAPI = new AADLCoLAPI();
		asme2ssme_common = new ASME2SSME_Translate_Common();
	}	

	/** 
	 * Returns a Signal process that represents an AADL thread implementation component
	 * <p>
	 * The interface is generated by a method ASME2SSME_ComponentInterface(), where its type component is passed as a parameter. 
	 * Beside the original in/out port of the corresponding thread type, 
	 * additional signals are added: ctl1 (Dispatch, Resume and Deadline) are added as input signals, 
	 * and ctl2 (Complete and Error), Alarm as output signals. 
	 * <p>
	 * The data access (ASME2SSME_ComponentInterface_DataAccess()), 
	 * subprogram access (ASME2SSME_ComponentInterface_SubprogramAccesses()) features are translated and added into the interface. 
	 * <p>
	 * Call external subprograms are required in ASME2SSME_ComponentInterface_CallSubprograms(), 
	 * sub-subprograms are provided through ASME2SSME_ComponentInterface_Sub_Subprograms()
	 * <p>
	 * The body is composed of t_thread_behavior(), t_thread_property(), and the subprocesses of ports and subcomponents.
	 * Each in data port xx is declared as a Signal process xx_InDataPort(), and called in the body. 
	 * Each out data port xx is declared as a Signal process xx_OutDataPort(), and called in the body. 
	 * The subprogram and data subcomponents are declared as local Signal processes, and the instances are composed in the body.  
	 * 
	 * @param t thread implementation component
	 * @return a Signal process
	 */	
	public ProcessModelImpl ASME2SSME_ThreadImpl(ThreadImplementation t, ModuleImpl vmodule){	

		Trace ("\n ========== " + get_Thread_type() + ": ASME2SSME_ThreadImpl: " + t.getName ());

		ProcessModelImpl vprocess;

		String sname = aadlColAPI.ComponentImpl_GetName(t);			
		String pname = Name_of_Thread(sname);
		vprocess = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );

		// get the component type
		ThreadType pt = (ThreadType) aadlColAPI.ComponentImpl_GetComponentType(t);

		//elements defined in the thread type
		//Features
		ASME2SSME_Thread_Features(pt, vprocess);

		//Flow specifications============================
		ASME2SSME_Thread_FlowSpecs(vprocess, pt);

		//Modes  no modes in version 1.5.8

		//Properties implemented in ProcessImpl_Property() sub process

		//create interface
		asme2ssme_common.ASME2SSME_ComponentInterface(vprocess, pt);
		//add clock for requires data access
		asme2ssme_common.ASME2SSME_ComponentInterface_DataAccess(vprocess, pt);
		//add clock for requires subprogram access: ! event x;
		asme2ssme_common.ASME2SSME_ComponentInterface_SubprogramAccesses(vprocess, pt);
		//add clock for calls subprogram: ! event x;
		asme2ssme_common.ASME2SSME_ComponentInterface_CallSubprograms(vprocess, t);
		//	//add clock for subprogram subcomponents: ? event x;
		//	asme2ssme_common.ASME2SSME_ComponentInterface_Sub_Subprograms(vprocess, t);

		asme2ssme_common.ASME2SSME_add_input_to_process_interface(CTL1_ID, Name_of_CTL1_type(), vprocess);	
		asme2ssme_common.ASME2SSME_add_output_to_process_interface(CTL2_ID, Name_of_CTL2_type(), vprocess);
		asme2ssme_common.ASME2SSME_add_output_to_process_interface(ALARM_ID, SIGNAL_BOOLEAN_TYPE, vprocess);

		ArrayList<Declaration> vdecl = new ArrayList<Declaration>(); //local signal list       
		ArrayList<ExprParam> vdecl_behavior_param = new ArrayList<ExprParam>();//parameter list for call thread_behavior
		ArrayList<ExprSignal> vdecl_behavior_in = new ArrayList<ExprSignal>();//input list for call thread_behavior
		ArrayList<ExprSignal> vdecl_behavior_out = new ArrayList<ExprSignal>();//output list for call thread_behavior     
		ArrayList<ExprParam> vdecl_property_param = new ArrayList<ExprParam>();//parameter list for call thread_property
		ArrayList<ExprSignal> vdecl_property_in = new ArrayList<ExprSignal>();//input list for call thread_property
		ArrayList<ExprSignal> vdecl_property_out = new ArrayList<ExprSignal>();//output list for call thread_property

		//port===================
		ArrayList<DataPort> list_ind_port = aadlColAPI.Component_GetInDataPort(pt);	
		ArrayList<DataPort> list_outd_port = aadlColAPI.Component_GetOutDataPort(pt);
		ArrayList<EventPort> list_ine_port = aadlColAPI.Component_GetInEventPort(pt);
		ArrayList<EventPort> list_oute_port = aadlColAPI.Component_GetOutEventPort(pt);
		ArrayList<EventDataPort> list_ined_port = aadlColAPI.Component_GetInEventDataPort(pt);
		ArrayList<EventDataPort> list_outed_port = aadlColAPI.Component_GetOutEventDataPort(pt);
		ASME2SSME_ThreadImpl_Ports(vprocess, t, vdecl, vdecl_behavior_param, vdecl_behavior_in, 
				vdecl_behavior_out, vdecl_property_param, vdecl_property_in, vdecl_property_out);

		//requires data access
		ArrayList<DataAccess> list = aadlColAPI.Component_GetRequiresDataAccess(pt);
		//add output effective parameters of behavior process call
		ASME2SSME_ThreadImpl_RequiresDataAccesses(vprocess, list, t, vdecl, 
				vdecl_behavior_param, vdecl_behavior_in, vdecl_behavior_out, 
				vdecl_property_param, vdecl_property_in, vdecl_property_out);

		//requires subprogram access
		ArrayList<SubprogramAccess> list_sp1 = aadlColAPI.Component_GetRequiresSubprogramAccess(pt);
		//add output effective parameters of behavior process call
		ASME2SSME_ThreadImpl_RequiresSubprogramAccesses(vprocess, list_sp1, t, vdecl, 
				vdecl_behavior_param, vdecl_behavior_in, vdecl_behavior_out, 
				vdecl_property_param, vdecl_property_in, vdecl_property_out);

		//subprogram calls
		ArrayList<SubprogramCallSequence> callseqs = aadlColAPI.BehavioredImpl_Get_CallSequences(t);    
		for(int i =0; i< callseqs.size(); i++){
			SubprogramCallSequence callseq1 = callseqs.get(i);
			ArrayList<SubprogramCall> calls = aadlColAPI.SubprogramCallSequence_Get_SubprogramCalls(callseq1);   	    	
			//add output effective parameters of behavior process call
			ASME2SSME_ThreadImpl_SubprogramCalls(vprocess, calls, t, vdecl, 
					vdecl_behavior_param, vdecl_behavior_in, vdecl_behavior_out, 
					vdecl_property_param, vdecl_property_in, vdecl_property_out);
		} 

		//create a bundle type for the Frozen_time_event and Output_time_event time signals: xx_TIME_EVENT time1
		if(aadlColAPI.Thread_HasPorts(pt)){
			TypesDeclarationImpl bundle_type = ASME2SSME_mkThreadTimeBundle(t, list_ind_port, 
					list_outd_port, list_ine_port, list_oute_port, list_ined_port, list_outed_port);   	
			myLib.STree_addComment(bundle_type, "bundle for the input/output time events of thread " + pt.getName(), true);   	
			//add the type in the module
			myLib.STree_addaTypeDeclarationToModule(vmodule, bundle_type);
			asme2ssme_common.ASME2SSME_add_input_to_process_interface( TIME1_ID, Name_of_ThreadTimeBundle(aadlColAPI.ComponentType_GetName(pt)), vprocess);
		}

		asme2ssme_common.ASME2SSME_add_input_to_process_interface(TOP_ID, SIGNAL_EVENT_TYPE, vprocess);

		//subcomponents 
		//data subcomponent
		ASME2SSME_ThreadImpl_SubData(t, vprocess);

		//subprogram subcomponent
		ASME2SSME_ThreadImpl_SubSubprograms(t, vprocess, vmodule);

		//subprogram group subcomponent : subprogram group not exist in V1.5.8

		//subprogram calls
		ASME2SSME_ThreadImpl_CallSequences(t);

		//connections 
		//the connections between thread t and subprogram sp (sp is not a subcomponent of t) are implemented in behavior body
		ASME2SSME_ThreadImpl_Connections(t, vprocess);

		//flows 
		ASME2SSME_ThreadImpl_Flows(vprocess, t);

		//modes
		ASME2SSME_ThreadImpl_Modes(t);

		//properties: implemented in the ThreadImpl_Property() sub process

		// add local declaration event Start to list vdecl
		SignalsDeclarationImpl vdec_sig_Frozen_time_event = myLib.mkEventDecl(START_ID );
		vdecl.add(vdec_sig_Frozen_time_event);

		// add local declarations to the Signal process
		myLib.STree_addLocalDeclaration(vprocess, vdecl);

		//thread behavior 
		ProcessModelImpl t_behavior = ASME2SSME_thread_behavior(t, pname);
		String thread_behavior_name = myLib.STree_GetStringName(myLib.Process_GetID (t_behavior)); 
		//add to declaration
		myLib.STree_addProcessDeclaration(vprocess, t_behavior);
		//add to body
		vdecl_behavior_in.add(myLib.STree_mkFieldAccess(CTL1_ID, DISPATCH_ID));
		vdecl_behavior_in.add(myLib.mkIdentifier(START_ID));
		vdecl_behavior_in.add(myLib.STree_mkFieldAccess(CTL1_ID, RESUME_ID));
		vdecl_behavior_out.add(myLib.mkIdentifier(CTL2_ID));
		SignalsDefinitionImpl behavior_inst = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(thread_behavior_name), 
				vdecl_behavior_param, vdecl_behavior_in, vdecl_behavior_out);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, behavior_inst);
		myLib.STree_addComment(behavior_inst, "thread behavior sub process", false);

		//thread property 
		ProcessModelImpl t_property = ASME2SSME_thread_property(t, pname);
		String thread_property_name = myLib.STree_GetStringName(myLib.Process_GetID (t_property));
		//add to declaration
		myLib.STree_addProcessDeclaration(vprocess, t_property);
		//add to body	
		vdecl_property_in.add(myLib.mkIdentifier(CTL1_ID));
		vdecl_property_in.add(myLib.mkIdentifier(TOP_ID));
		vdecl_property_out.add(myLib.mkIdentifier(START_ID));
		vdecl_property_out.add(myLib.mkIdentifier(ALARM_ID));
		SignalsDefinitionImpl property_inst = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(thread_property_name), 
				vdecl_property_param, vdecl_property_in,vdecl_property_out);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, property_inst);
		myLib.STree_addComment(property_inst, "thread property sub process", false);

		//add local shared signals for each requires data access: shared xx_w, xx_r
		//asme2ssme_common.ASME2SSME_DataAccess_local(vprocess, pt);

		Trace("\n ========================= end ASME2SSME_ThreadImpl: " + sname + " ======================= \n");
		return vprocess;
	}

	/**
	 * Returns a Signal process "xx_behavior", which is a subprocess of thread
	 * <p>
	 * The interface is created by a called method ASME2SSME_ComponentInterface().
	 * Besides the original in/out port of the corresponding thread type,
	 * three input signals: Dispatch, Start, Resume,
	 * and two output signals: Complete and Error are added in the interface.
	 * The data access and subprogram access/subprogram call signals are added in the interface.
	 *
	 * process xx_behavior =
	 * (? ... event Dispatch, Start, Resume;
	 *  ! ... xx_CTL2 ctl2;)
	 *  (| e1 :: (...) := xx_external(...)
	 *   | e1 ^= when Start
	 *   | e2 :: ctl2.Complete := when Start
	 *   | e1 --> e2
	 *   | ctl2.Error := ^0
	 *   | out ^= when Start
	 *   | dd_ReadTime := when Start
	 *   | dd_WriteTime := when Start
	 *   | dd_reset := when Start
	 *   | sp := ^0
	 *   | L_x.a ::= a when sp
	 *   | ...
	 *   |)
	 * where
	 *     process xx_external = (? ... ! ...) external;
	 *     label e1, e2;
	 * end;
	 *
	 * @param t thread implementation component
	 * @param t_name string name of the Signal process for the thread implementation
	 * @return a ProcessModelImpl (t_behavior)
	 */
	public ProcessModelImpl ASME2SSME_thread_behavior(ThreadImplementation t, String t_name) {

		Trace ("\n ==========" +  get_Thread_type() + ": ASME2SSME_thread_behavior: " + t.getName ());

		String bname = t_name + "_behavior";
		ProcessModelImpl t_behavior = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(bname) );
		ASME2SSME_Translate_ParameterConnection asme2ssme_pc = new ASME2SSME_Translate_ParameterConnection();

		// get the component type
		ThreadType pt = (ThreadType) aadlColAPI.ComponentImpl_GetComponentType(t);

		//interface, inputs and outputs are the same as xx_thread
		asme2ssme_common.ASME2SSME_ComponentInterface(t_behavior, pt);
		asme2ssme_common.ASME2SSME_ComponentInterface_DataAccess(t_behavior, pt);
		//add clock for requires subprogram access
		asme2ssme_common.ASME2SSME_ComponentInterface_SubprogramAccesses(t_behavior, pt);
		//add clock for calls subprogram
		asme2ssme_common.ASME2SSME_ComponentInterface_CallSubprograms(t_behavior, t);

		//add in event Dispatch to interface
		asme2ssme_common.ASME2SSME_add_input_to_process_interface(DISPATCH_ID, SIGNAL_EVENT_TYPE, t_behavior);

		//add in event Start to interface
		asme2ssme_common.ASME2SSME_add_input_to_process_interface(START_ID, SIGNAL_EVENT_TYPE, t_behavior);

		//add in event Resume to interface
		asme2ssme_common.ASME2SSME_add_input_to_process_interface(RESUME_ID, SIGNAL_EVENT_TYPE, t_behavior);

		//add out bundle CTL2 ctl2 to interface
		asme2ssme_common.ASME2SSME_add_output_to_process_interface(CTL2_ID, Name_of_CTL2_type(), t_behavior);

		//external process
		ProcessModelImpl external_process = ASME2SSME_thread_external(t, t_name);
		myLib.STree_addProcessDeclaration(t_behavior, external_process);

		//call external process
		ArrayList<ExprParam> lparam = new ArrayList<ExprParam> ();
		Identifier external_id = myLib.Process_GetID(external_process);
		ProcessExpansionImpl pcall = myLib.STree_mkProcessCallWithParameters(myLib.mkIdentifier(external_id.getAtomString()), lparam);
		myLib.STree_addComment(pcall, "external (behavior automaton) process call", false);

		LabelledProcessImpl labelled_expr = myLib.STree_mkLabelledProcessCall(LABEL1, pcall);
		myLib.STree_addExpressionOfProcessToaModel(t_behavior, labelled_expr);

		// e1 ^= when Start
		ClockEqualityImpl clk_eq = ASME2SSME_Execute_Sync();
		myLib.STree_addExpressionOfProcessToaModel(t_behavior, clk_eq);

		//e2 :: ctl2.Complete := when Start
		SignalsDefinitionImpl compl_eq = ASME2SSME_Complete_Def();
		LabelledProcessImpl labelled_expr2 = myLib.STree_mkLabelledProcessCall(LABEL2,compl_eq);
		myLib.STree_addExpressionOfProcessToaModel(t_behavior, labelled_expr2);
		myLib.STree_addExpressionOfProcessToaModel (t_behavior, ASME2SSME_mk_Synchro(LABEL2, START_ID));  //added LBesnard
		//e1 --> e2
		SignalsImpl sigs = new SignalsImpl();
		ListExprSignalImpl list_expr = new ListExprSignalImpl();
		list_expr.getExprSignal().add(myLib.mkIdentifier(LABEL1));
		sigs.setListExprSignal(list_expr);

		ArrayList<Signals> list2 = new ArrayList<Signals>();
		SignalsImpl sigs2 = new SignalsImpl();
		ListExprSignalImpl list_expr2 = new ListExprSignalImpl();
		list_expr2.getExprSignal().add(myLib.mkIdentifier(LABEL2));
		sigs2.setListExprSignal(list_expr2);
		list2.add(sigs2);

		UnconditionedDependences dep = myLib.STree_mkDependency(sigs, list2);
		myLib.STree_addExpressionOfProcessToaModel(t_behavior, dep);

		//ctl2.Error := ^0
		SignalsDefinitionImpl err_eq = ASME2SSME_Error_Def();
		myLib.STree_addExpressionOfProcessToaModel(t_behavior, err_eq);

		LabelsImpl labels = new LabelsImpl();
		IdentifierImpl label1 = myLib.mkIdentifier(LABEL1);
		labels.getListIdentifier().add(label1);
		IdentifierImpl label2 = myLib.mkIdentifier(LABEL2);
		labels.getListIdentifier().add(label2);
		myLib.STree_addDeclarationToProcess(t_behavior, labels);

		//out ^= when Start
		ASME2SSME_Outputs_Sync(t_behavior, pt);
		//dd_ReadTime := when Start
		ASME2SSME_RequiresDataAccesses_ReadTime_Def(t_behavior, pt);
		//dd_WriteTime := when Start
		ASME2SSME_RequiresDataAccesses_WriteTime_Def(t_behavior, pt);
		//dd_reset := ^0
		ASME2SSME_RequiresDataAccesses_Reset_Def(t_behavior, pt);
		//requires subprograms: sp1 := when Start
		ASME2SSME_RequiresSubprogramAccesses_Clock_Def(t_behavior, pt);
		//calls subprograms: sp2 := when Start
		ASME2SSME_SubprogramCalls_Clock_Def(t_behavior, t);
		//parameter connections (thread -> sp1.a): L_x.a ::= a when sp1
		asme2ssme_pc.ASME2SSME_ParameterConnections_Thread_to_CalledSubprogram(t_behavior, t);
		//parameter connections (sp2.b -> thread.b'): b' := L_y.b when sp2,
		//and remove b' from behavior_external interface
		asme2ssme_pc.ASME2SSME_ParameterConnections_CalledSubprogram_to_Thread(t_behavior, t, external_process);

		return t_behavior;
	}

	/**
	 * Returns a Signal process "xx_thread_property" that represents the properties of a thread
	 * 
	 * process xx_Thread_property =(? CTL1 ctl1; event top; ! event Start; boolean Alarm;)
	 * (| Start := when((ctl1.Resume from(ctl1.Dispatch))=1)
	 *  | "Specific_constraints..."
	 *  | xx..._property{}() 
	 *  |)
	 *  *<p>
	 * All the properties defined in this thread are represented as process
	 * calls: ccc_property{}(), where ccc is the name of the property, and the
	 * property values are fetched and passed as process call parameters.
	 * The "Specific constraints" part must be generated in the implementation of the abstract method ASME2SSME_Dispatch_Constraint
	 * @param timpl
	 *            a thread implementation component
	 * @param t_name
	 *            a Signal process name of this thread implementation
	 * @return a Signal process
	 */
	public ProcessModelImpl ASME2SSME_thread_property(ThreadImplementation timpl, String t_name)
	{
		Trace("\n ========== " + get_Thread_type() + ": ASME2SSME_thread_property: " + timpl.getName());
		String pname = t_name + "_property";
	 
		ProcessModelImpl t_property = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(pname) );
		// add in bundle CTL1 ctl1 to interface
		asme2ssme_common.ASME2SSME_add_input_to_process_interface(CTL1_ID, Name_of_CTL1_type(), t_property);
		asme2ssme_common.ASME2SSME_add_input_to_process_interface(TOP_ID, SIGNAL_EVENT_TYPE, t_property);
		// add out event Start, Alarm to interface
		asme2ssme_common.ASME2SSME_add_output_to_process_interface(START_ID, SIGNAL_EVENT_TYPE, t_property);
		asme2ssme_common.ASME2SSME_add_output_to_process_interface(ALARM_ID, SIGNAL_BOOLEAN_TYPE, t_property);
		// Start := when((Resume from Dispatch) = 1)
		SignalsDefinitionImpl start_def = ASME2SSME_Start_Assignment();
		myLib.STree_addExpressionOfProcessToaModel(t_property, start_def);

		// Dispatch constraint
		ASME2SSME_Dispatch_Constraint(timpl, t_property);
		// fix properties
		ASME2SSME_FixProperties(timpl, t_property);
		// process calls of common thread properties
		ASME2SSME_Thread_Properties(timpl, t_property);
		return t_property;
	}

	/**
	 * Translates AADL thread type
	 *
	 * @param t
	 */
	private ProcessModelImpl UNUSED_ASME2SSME_ThreadType(ThreadType t) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadType " + t.getName());
		ProcessModelImpl vprocess = new ProcessModelImpl();
		//Features
		ASME2SSME_Thread_Features(t, vprocess);
		//Flow specifications
		ASME2SSME_Thread_FlowSpecs(vprocess, t);
		//Modes: no modes in version 1.5.8
		//properties are implemented in ProcessImpl_Property() sub process
		return vprocess;
	}


	/**
	 * Translates thread features
	 *
	 * @param t
	 */
	private void ASME2SSME_Thread_Features(ThreadType t, ProcessModelImpl vprocess) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_Thread_Features " + t.getName());

		ASME2SSME_Translate_Feature asme2ssme_feature = new ASME2SSME_Translate_Feature();

		if (t.getOwnedFeatures() != null) {
			// ports are implemented as a signal in the interface
			
			//port group 
			EList<FeatureGroup> pgroup_list = t.getOwnedFeatureGroups();
			if(pgroup_list.size()>0) {

				ASME2SSME_Translate_PortGroup asme2ssme_pg = new ASME2SSME_Translate_PortGroup();
				//				Trace("\n ===== port groups not yet implemented for system: " + syst.getName());
				for(int i1=0; i1<pgroup_list.size(); i1++)
				{
					FeatureGroup pg = pgroup_list.get(i1);

					if(asme2ssme_pg.ASME2SSME_PortGroup_bw(pg) != null) {
						SignalsDeclarationImpl in1 = asme2ssme_pg.ASME2SSME_PortGroup_bw(pg);
						myLib.STree_addDeclarationToInterface(vprocess, in1, 2);
					}

					if(asme2ssme_pg.ASME2SSME_PortGroup_wb(pg) != null) {
						SignalsDeclarationImpl out1 = asme2ssme_pg.ASME2SSME_PortGroup_wb(pg);
						myLib.STree_addDeclarationToInterface(vprocess, out1, 3);
					}
				}

			}
			//data access 
			EList<DataAccess> da_list = t.getOwnedDataAccesses();
			if(da_list.size()>0) {
				for(int i4=0; i4<da_list.size(); i4++)
				{
					DataAccess da1 = da_list.get(i4);
					asme2ssme_feature.ASME2SSME_DataAccess(da1);
				}
			}
			//subprogram access are added in the interface: asme2ssme_common.ASME2SSME_ComponentInterface_SubprogramAccess

			//subprogram group : not exist in version 1.5.8
		}
	}


	/**
	 * Translates flow specifications of a thread type
	 *
	 * @param tt a thread type component
	 */
	private void ASME2SSME_Thread_FlowSpecs(ProcessModelImpl vprocess,ThreadType tt) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_Thread_FlowSpecs " + tt.getName());

		if(asme2ssme_common.ASME2SSME_ComponentType_FlowSpecs(tt) != null) {
			ProcessModelImpl p1= asme2ssme_common.ASME2SSME_ComponentType_FlowSpecs(tt);

			myLib.STree_addProcessDeclaration(vprocess, p1);
			ArrayList<ExprParam> lparam = new ArrayList<ExprParam> ();
			String pname = Name_of_Flow(tt.getName());
			ProcessExpansionImpl call1 = myLib.STree_mkProcessCallWithParameters(myLib.mkIdentifier(pname), lparam);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, call1);
		}
	}


	/**
	 * Translates thread properties
	 *
	 * @param t
	 * @param t_property
	 */
	private void ASME2SSME_Thread_Properties(ThreadImplementation t, ProcessModelImpl t_property) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_Thread_Properties: " + t.getName());

		ArrayList<PropertyAssociation> list1 = aadlColAPI.ComponentImpl_GetPropertyAssociations(t);
		ArrayList<PropertyAssociation> list2 = aadlColAPI.Component_GetPropertyAssociations(t.getType());

		ArrayList<PropertyAssociation> list_p = new ArrayList<PropertyAssociation>();

		Iterator<?> it1 = list1.iterator();
		while (it1.hasNext())
		{
			PropertyAssociation s1 = (PropertyAssociation) it1.next();
			list_p.add(s1);
		}

		Iterator<?> it2 = list2.iterator();
		while (it2.hasNext())
		{
			PropertyAssociation s2 = (PropertyAssociation) it2.next();
			list_p.add(s2);
		}

		Iterator<PropertyAssociation> it = list_p.iterator();
		while (it.hasNext())
		{
			PropertyAssociation pa = (PropertyAssociation) it.next();
			String propertyDef_name = aadlColAPI.Property_Get_Name(pa);

			if (propertyDef_name.equals("Dispatch_Protocol")  | propertyDef_name.equals("Period")	)
			{
				Trace("property is implemented as fixed thread property");
			}
			else if (propertyDef_name.equals("Time_Offset")
					| propertyDef_name.equals("Input_Time_Reference_Time") | propertyDef_name.equals("Output_Time_Reference_Time")
					| propertyDef_name.equals("Queue_Size") | propertyDef_name.equals("Dequeue_Protocol")
					| propertyDef_name.equals("Dequeued_Items") | propertyDef_name.equals("Queue_Processing_Protocol")	)
			{
				Trace("property for port is implemented in the port_property");
			}
			else {
				ProcessCallImpl pcall1 = ASME2SSME_Thread_Property(pa);
				myLib.STree_addExpressionOfProcessToaModel(t_property, pcall1);
			}
		}
	}




	/**
	 * Returns a Signal process call that represents a property of a thread
	 * <p>
	 * if it is a "Period" property, use ASME2SSME_Period_Property().
	 * if it is a "Dispatch_Protocol" property, use ASME2SSME_Dispatch_Protocol_Property().
	 * if it is a DEADLINE_ID property, use ASME2SSME_Deadline_Property().
	 * else use ASME2SSME_Property_Call() method for other properties.
	 *
	 * @param pa a property association defined for a thread implementation
	 * @return a ProcessCallImpl (pcall)
	 */
	private ProcessCallImpl ASME2SSME_Thread_Property(PropertyAssociation pa) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_Thread_Property ");
		return ASME2SSME_Property_Call(pa);
	}




	/**
	 * Translates data subcomponents of thread implementation:
	 * for each data subcomponent dd, whose classifier is xx:
	 * adds a type declaration and a local signal in the local declaration:
	 * type xx = external;
	 * shared xx dd;
	 *
	 * @param t
	 * @param tprocess
	 */
	private void ASME2SSME_ThreadImpl_SubData(ThreadImplementation t, ProcessModelImpl tprocess) {


		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_SubData");

		ArrayList<DataSubcomponent> sub_data = aadlColAPI.Thread_GetSubcomponentsData(t);

		// DataSubcomponent
		ArrayList<DataClassifier> l_data_classifier = new ArrayList<DataClassifier>();
		if(sub_data.size()>0) {
			Iterator<?> it2 = sub_data.iterator();
			while (it2.hasNext())
			{
				DataSubcomponent d1 = (DataSubcomponent) it2.next();
				DataClassifier data_classifier = (DataClassifier) aadlColAPI.Datasubcomponent_GetClassifier(d1);

				Trace("data subcomponent: " + d1.getName());
				Trace("classifier: " + data_classifier);

				if (l_data_classifier.contains(data_classifier)) {
					Trace("Data classifier has already been created in Signal!!");
				}
				else {
					TypesDeclarationImpl type_decl = new TypesDeclarationImpl();
					ASME2SSME_Translate_Data asme2ssme_data = new ASME2SSME_Translate_Data();
					IdentifierImpl dd_id = myLib.mkIdentifier(d1.getName());

					if (data_classifier instanceof DataType)
					{
						String name2 = aadlColAPI.ComponentType_GetName((DataType) data_classifier);
						if(asme2ssme_data.ASME2SSME_DataType((DataType) data_classifier) != null) {
							type_decl = asme2ssme_data.ASME2SSME_DataType((DataType) data_classifier);
							myLib.STree_addaDeclarationToProcess(tprocess, type_decl);
							l_data_classifier.add(data_classifier);

							// add local declarations: shared xx dd;
							IdentifierImpl type2 = myLib.mkIdentifier(Name_of_Data(name2));
							SharedVariablesDeclarationImpl sig2 = myLib.STree_mkSharedVariableDeclaration(dd_id, type2, null);
							myLib.STree_addDeclarationToProcess(tprocess, sig2);
						}
						else { //DataType is a predefined type: integer, boolean, float
							String strx = Name_of_Data(name2);
							Trace("name of data type in Name_of_Data() is " + strx);
							if(myLib.Is_PredefinedType(strx)) {
								PredefinedTypeImpl ptype1 = myLib.STree_mkPredefinedType(strx);
								SharedVariablesDeclarationImpl sig2 = myLib.STree_mkSharedVariableDeclaration(dd_id, ptype1, null);
								myLib.STree_addDeclarationToProcess(tprocess, sig2);
							}
							else {
								Trace("data classifier not recongnized!!");
							}
						}

					}
					else {
						type_decl = asme2ssme_data.ASME2SSME_DataImpl((DataImplementation) data_classifier);
						myLib.STree_addaDeclarationToProcess(tprocess, type_decl);
						l_data_classifier.add(data_classifier);

						// add local declarations: shared xx dd;
						String name1 = aadlColAPI.ComponentImpl_GetName((DataImplementation) data_classifier);
						IdentifierImpl type1 = myLib.mkIdentifier(Name_of_Data(name1));
						SharedVariablesDeclarationImpl sig1 = myLib.STree_mkSharedVariableDeclaration(dd_id, type1, null);
						myLib.STree_addDeclarationToProcess(tprocess, sig1);
					}
				}
			}
		}
	}




	/**
	 * Translates subprogram subcomponents of a thread implementation:
	 * <p>
	 * for each sub subprogram sp1: subprogram sp:
	 * a process declaration is created, and added,
	 * an instance is added into the body: sp1 :: (l_y, l_id) := sp{}(l_x)
	 *
	 * @param t
	 * @param tprocess
	 */
	private void ASME2SSME_ThreadImpl_SubSubprograms(ThreadImplementation t,	ProcessModelImpl tprocess, ModuleImpl vmodule) 
	{
		ASME2SSME_Translate_Subprogram asme2ssme_subprogram = new ASME2SSME_Translate_Subprogram();

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_SubSubprograms");

		ArrayList<SubprogramSubcomponent> sub_subprograms = aadlColAPI.Thread_GetSubcomponent_Subprograms(t);
		ArrayList<SubprogramClassifier> sp_exist = new ArrayList<SubprogramClassifier>();

		for(int i=0; i< sub_subprograms.size(); i++) {
			SubprogramSubcomponent t1 = sub_subprograms.get(i);
			String sub_sp_name = aadlColAPI.Subcomponent_GetName(t1);
			SubprogramClassifier subp_classifier = aadlColAPI.Subprogramsubcomponent_GetClassifier(t1);
			ProcessModelImpl subp_process = new ProcessModelImpl();
			String s_name = null;

			if (subp_classifier instanceof SubprogramImplementation) {
				String s_name1 = aadlColAPI.ComponentImpl_GetName((SubprogramImplementation) subp_classifier);
				s_name = s_name1.replace(".", "_"); 
			}
			else  
				s_name = aadlColAPI.ComponentType_GetName((SubprogramType) subp_classifier);

			String sn_sp = getSname(s_name); 

			// create a Signal process
			if (sp_exist.contains(subp_classifier)) {
				Trace("subprogram classifier has already been created in Signal!!");
				subp_process = myLib.STree_GetProcessDeclarationFromProcess(tprocess, Name_of_Subprogram(subp_classifier.getName()));
			}
			else {
				if (subp_classifier instanceof SubprogramImplementation)  
					subp_process = asme2ssme_subprogram.ASME2SSME_SubprogramImpl((SubprogramImplementation) subp_classifier, vmodule);
				else 
					subp_process = asme2ssme_subprogram.ASME2SSME_SubprogramType((SubprogramType) subp_classifier, vmodule);

				myLib.STree_addProcessDeclaration(tprocess, subp_process);
				sp_exist.add(subp_classifier);
			}

			// asme2ssme_common.ASME2SSME_component_instance(tprocess, subp_process, t1.getName());
			//make an instance-----------------------------------------------------------------------------
			ArrayList<ExprParam> lps = new ArrayList<ExprParam>();//parameter (id) list for call the process sp_process
			ArrayList<ExprSignal> lis = new ArrayList<ExprSignal>();//input (id) list for call the process sp_process
			ArrayList<ExprSignal> los = new ArrayList<ExprSignal>();//output (id) list for call the process sp_process

			//in: shared sp_IN sp_x;
			//Identifier type1 = myLib.mkIdentifier(Name_of_Subprogram_Struct_In(s_name));
			IdentifierImpl x1 = myLib.mkIdentifier(sn_sp+"_x"); 
			//SignalsDeclarationImpl sig1 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(s_name+"_x"), type1, null);
			lis.add(x1);

			//out: sp_OUT sp_y;
			//Identifier type2 = myLib.mkIdentifier(Name_of_Subprogram_Struct_Out(s_name));
			IdentifierImpl y1 = myLib.mkIdentifier(sn_sp+"_y");  
			//SignalsDeclarationImpl sig2 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(s_name+"_y"), type2, null);
			los.add(y1);

			//out: TID sp_id;
			//Identifier type3 = myLib.mkIdentifier("TID");
			IdentifierImpl id1 = myLib.mkIdentifier(sn_sp+"_id");  
			//SignalsDeclarationImpl sig3 = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(s_name+"_id"), type3, null);
			los.add(id1);

			String str1 = "instance of AADL subprogram " + sub_sp_name + " (" + s_name + ")";
			Identifier id = myLib.Process_GetID(subp_process);
			SignalsDefinitionImpl call_eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(id.getAtomString()), lps, lis, los);
			if(sub_sp_name != null) {
				ArrayList<Identifier> ids = new ArrayList<Identifier>();
				ids.add(myLib.mkIdentifier(sub_sp_name));
				myLib.STree_addLabelsInModel(ids, tprocess);
				LabelledProcessImpl lp = myLib.STree_mkLabelledProcessCall(sub_sp_name, call_eq);
				myLib.STree_addExpressionOfProcessToaModel(tprocess, lp);
				myLib.STree_addComment(lp, str1, false);
			}
			else {
				myLib.STree_addExpressionOfProcessToaModel(tprocess, call_eq);
				myLib.STree_addComment(call_eq, str1, false);
			}
		}
	}


	/**
	 * Translates call sequences of thread implementation
	 *
	 * @param t
	 */
	private void ASME2SSME_ThreadImpl_CallSequences(ThreadImplementation t) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_CallSequences: " + t.getName());
		if(t.getOwnedSubprogramCallSequences() != null) {
			Trace("Thread Impl call sequences not implemented");
			EList<SubprogramCallSequence> list = t.getOwnedSubprogramCallSequences();
			if(list.size()>0) {
 
			}
		}
	}



	/**
	 * Translates connections in a thread implementation
	 *
	 * @param t
	 * @param vprocess
	 */
	private void ASME2SSME_ThreadImpl_Connections(ThreadImplementation t, ProcessModelImpl vprocess) {


		if (aadlColAPI.ComponentImpl_GetConnections(t) != null) {

			Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_Connections " + t.getName());

			//    	ASME2SSME_Translate_Connection asme2ssme_connection = new ASME2SSME_Translate_Connection();
			//    	ASME2SSME_Translate_PortConnection asme2ssme_pconnection = new ASME2SSME_Translate_PortConnection();
			//    	ASME2SSME_Translate_ParameterConnection asme2ssme_pc = new ASME2SSME_Translate_ParameterConnection();
			//    	ASME2SSME_Translate_FeatureGroupConnection asme2ssme_fgc = new ASME2SSME_Translate_FeatureGroupConnection();
			//
			//    	// EventConnections========================
			//	    ArrayList<PortConnection> listEventConnections = aadlColAPI.ComponentImpl_GetEventPortConnections(t);
			//	    for(int i=0; i<listEventConnections.size(); i++)
			//	    {
			//	    	Trace("event connection in thread");
			//	    	PortConnection ec1 = listEventConnections.get(i);
			//		    SignalsDefinitionImpl eq1 = asme2ssme_pconnection.ASME2SSME_EventPortConnection(t, vprocess, ec1);
			//		    myLib.STree_addExpressionOfProcessToaModel(vprocess, eq1);
			//	    }
			//
			//	    //event data port connection=============================
			//	    ArrayList<PortConnection> listEventDataConnections = aadlColAPI.ComponentImpl_GetEventDataPortConnections(t);
			//	    for(int j=0; j<listEventDataConnections.size(); j++)
			//	    {
			//	    	PortConnection edc1 = listEventDataConnections.get(j);
			//	    	SignalsDefinitionImpl eq2 = asme2ssme_pconnection.ASME2SSME_EventDataPortConnection(t, vprocess, edc1);
			//		    myLib.STree_addExpressionOfProcessToaModel(vprocess, eq2);
			//	    }
			//
			//	    //parameter connection==================================
			//	    ArrayList<ParameterConnection> listParameterConnections = aadlColAPI.ComponentImpl_GetParameterConnections(t);
			//	    for(int k =0; k< listParameterConnections.size(); k++){
			//	    	//Trace("parameter connection in thread");
			//	    	ParameterConnection edc1 = listParameterConnections.get(k);
			//	    	if(asme2ssme_pc.ASME2SSME_ParameterConnection(t, vprocess, edc1) != null){
			//	    		SignalsDefinitionImpl eq3 = asme2ssme_pc.ASME2SSME_ParameterConnection(t, vprocess, edc1);
			//			    myLib.STree_addExpressionOfProcessToaModel(vprocess, eq3);
			//	    	}
			//	    }
			//
			//	  //access connections
			//    	if(t.getOwnedAccessConnections() != null){
			//    		EList<AccessConnection> list1 = t.getOwnedAccessConnections();
			//	  		if(list1.size()>0){
			//	  			for(int i1 =0; i1 < list1.size(); i1++ ){
			//	  				AccessConnection c1 = list1.get(i1);
			//
			//	  				//bus access connection
			//	  				if(aadlColAPI.AccessConnection_GetCategory(c1) == "bus"){
			//	  					asme2ssme_connection.ASME2SSME_BusAccessConnection(c1);
			//	  				}
			//	  				//data access connection
			//	  				else if(aadlColAPI.AccessConnection_GetCategory(c1) == "data"){
			//	  					asme2ssme_connection.ASME2SSME_DataAccessConnection(c1);
			//	  				}
			//	  			}
			//	  		}
			//    	}
			//
			//	  	//feature group connections
			//    	ArrayList<FeatureGroupConnection> list3 = aadlColAPI.ComponentImpl_GetFeatureGroupConnections(t);
			//	  	for(int i3 =0; i3 < list3.size(); i3++ ){
			//	  			FeatureGroupConnection c3 = list3.get(i3);
			//	  			if( asme2ssme_fgc.ASME2SSME_FeatureGroupConnection1(c3) != null){
			//            		SignalsDefinitionImpl eq3 = asme2ssme_fgc.ASME2SSME_FeatureGroupConnection1(c3);
			//            		myLib.STree_addExpressionOfProcessToaModel(vprocess, eq3);
			//            	}
			//            	if( asme2ssme_fgc.ASME2SSME_FeatureGroupConnection2(c3) != null){
			//            		SignalsDefinitionImpl eq4 = asme2ssme_fgc.ASME2SSME_FeatureGroupConnection2(c3);
			//            		myLib.STree_addExpressionOfProcessToaModel(vprocess, eq4);
			//            	}
			//	  			}

		}
	}



	/**
	 * Translates flows of a thread implementation
	 *
	 * @param t a thread implementation component
	 */
	private void ASME2SSME_ThreadImpl_Flows(ProcessModelImpl vprocess, ThreadImplementation t) {
		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_Flows: " + t.getName());
	 
		asme2ssme_common.ASME2SSME_ComponentImpl_Flows(t);
		if(asme2ssme_common.ASME2SSME_ComponentImpl_Flows(t) != null) {
			ProcessModelImpl p1= asme2ssme_common.ASME2SSME_ComponentImpl_Flows(t);
			myLib.STree_addProcessDeclaration(vprocess, p1);
			ArrayList<ExprParam> lparam = new ArrayList<ExprParam> ();
			String pname = Name_of_Flow(t.getName().replace('.','_'));
			ProcessExpansionImpl call1 = myLib.STree_mkProcessCallWithParameters(myLib.mkIdentifier(pname), lparam);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, call1);
		}

	}
	/**
	 * Translate modes of a thread implementation component
	 *
	 * @param timpl
	 */
	private void ASME2SSME_ThreadImpl_Modes(ThreadImplementation timpl) {
		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_Modes: " + timpl.getName());
		asme2ssme_common.ASME2SSME_ComponentImpl_Modes(timpl);
	}

	private void addSignalDeclarationPort(TupleTypeImpl description, Port event_port1, String vsuff) {
		String pname3 = aadlColAPI.Port_GetName(event_port1);
		SignalsDeclarationImpl decl3 = myLib.mkEventDecl(pname3 + vsuff);
		description.getListSignalsDeclaration().add(decl3);
		
	}
	/**
	 * Returns a Signal type declaration (a bundle) that represents the time control signals (Frozen_time_event and Output_time_event) of a thread
	 *
	 * @param thread_impl a thread implementation
	 * @param list_ind_port a list of in data ports
	 * @param list_outd_port a list of out data ports
	 * @param list_ine_port a list of in event ports
	 * @param list_oute_port a list of out event ports
	 * @param list_ined_port a list of in event data ports
	 * @param list_outed_port a list of out event data ports
	 * @return a (bundle) type declaration
	 */
	private TypesDeclarationImpl ASME2SSME_mkThreadTimeBundle(ThreadImplementation thread_impl,
			ArrayList<DataPort> list_ind_port, ArrayList<DataPort> list_outd_port,
			ArrayList<EventPort> list_ine_port, ArrayList<EventPort> list_oute_port,
			ArrayList<EventDataPort> list_ined_port, ArrayList<EventDataPort> list_outed_port) 
	{
		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_mkThreadTimeBundle: " + thread_impl.getName());

		TypeDescriptionImpl typedescr = new TypeDescriptionImpl();
		ExprSignal vinit = null;

		ThreadType thread_type = aadlColAPI.Thread_GetCompType(thread_impl);
		String tname = aadlColAPI.ComponentType_GetName(thread_type);
		Identifier id = myLib.mkIdentifier(Name_of_ThreadTimeBundle(tname));

		if (list_ind_port == null && list_outd_port == null && list_ine_port == null && list_oute_port == null &&
				list_ined_port == null && list_outed_port == null ) {
			TypeDescriptionValue description = null;
			typedescr = myLib.STree_mkDclType(id, description, vinit);
		}
		else {
			TupleTypeImpl description = new TupleTypeImpl();
			description.setTypeKind(TypeKind.BUNDLE_LITERAL);

			Iterator<DataPort> it1 = list_ind_port.iterator();
			while (it1.hasNext())
			{
				DataPort data_port1 = (DataPort) it1.next();
				addSignalDeclarationPort(description, data_port1, "_Frozen_time_event");
			}

			Iterator<DataPort> it2 = list_outd_port.iterator();
			while (it2.hasNext())
			{
				DataPort data_port2 = (DataPort) it2.next();
				addSignalDeclarationPort(description, data_port2, "_Output_time_event" );
			}

			Iterator<EventPort> it3 = list_ine_port.iterator();
			while (it3.hasNext())
			{
				EventPort event_port1 = (EventPort) it3.next();
				addSignalDeclarationPort(description, event_port1, "_Frozen_time_event");
				
			}
			Iterator<EventPort> it4 = list_oute_port.iterator();
			while (it4.hasNext())
			{
				EventPort event_port2 = (EventPort) it4.next();
				addSignalDeclarationPort(description, event_port2, "_Output_time_event" );
			}
			Iterator<EventDataPort> it5 = list_ined_port.iterator();
			while (it5.hasNext())
			{
				EventDataPort port5 = (EventDataPort) it5.next();
				addSignalDeclarationPort(description, port5, "_Frozen_time_event");
			}

			Iterator<EventDataPort> it6 = list_outed_port.iterator();
			while (it6.hasNext())
			{
				EventDataPort port6 = (EventDataPort) it6.next();
				addSignalDeclarationPort(description, port6, "_Output_time_event");
			}
			typedescr = myLib.STree_mkDclType(id, description, vinit);
		}
		TypesDeclarationImpl type_decl = new TypesDeclarationImpl();
		type_decl.getTypeDefinition().add(typedescr);
		return type_decl;
	}

	
	/** Translates the in data ports of a thread implementation.
	 * <p>
	 * For each in data port, create a Signal process (ASME2SSME_Thread_InDataPort()).
	 * Add this declaration to thread_process's declaration part, and add a process call in the body.
	 * Add an event signal xx_Frozen_time_event to the interface of thread_process.
	 * Local signal declarations are added.
	 * The clock of local signal l_xx is synchronized with Start: l_xx ^= Start (ASME2SSME_InDataPort_Sync())
	 *
	 * @param thread_process a Signal process that represents a thread implementation component
	 * @param list_ind_port a list of in data ports of a thread
	 * @param thread_impl a thread implementation component
	 * @param vdecl a list of local declaration
	 * @param vdecl_behavior_param
	 * @param vdecl_behavior_in
	 * @param vdecl_behavior_out
	 * @param vdecl_property_param
	 * @param vdecl_property_in
	 * @param vdecl_property_out
	 */
	private void ASME2SSME_ThreadImpl_InDataPorts(ProcessModelImpl thread_process, ArrayList<DataPort> list_ind_port,
			ThreadImplementation thread_impl, ArrayList<Declaration> vdecl,
			ArrayList<ExprParam> vdecl_behavior_param, ArrayList<ExprSignal> vdecl_behavior_in,
			ArrayList<ExprSignal> vdecl_behavior_out, ArrayList<ExprParam> vdecl_property_param,
			ArrayList<ExprSignal> vdecl_property_in, ArrayList<ExprSignal> vdecl_property_out) {


		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_InDataPorts " + thread_impl.getName());

		if(list_ind_port.size()>0) {
			ASME2SSME_Translate_DataPort asme2ssme_dataport = new ASME2SSME_Translate_DataPort();
			Iterator<?> itd1 = list_ind_port.iterator();
			while (itd1.hasNext())
			{
				DataPort dataPort = (DataPort) itd1.next();
				//translate as a Signal process
				ProcessModelImpl indataport_process = asme2ssme_dataport.ASME2SSME_Thread_InDataPort(thread_impl, dataPort);
				//add to declaration
				myLib.STree_addProcessDeclaration(thread_process, indataport_process);

				//create a process call, and add it to the body
				Identifier id1 = indataport_process.getIdentifier();
				String id_name = myLib.STree_GetStringName(id1);
				IdentifierImpl id2 = myLib.mkIdentifier(id_name);
				String pn = dataPort.getName();

				ArrayList<ExprParam> lparam3 = new ArrayList<ExprParam>();

				DataClassifier data_classifier = (DataClassifier) dataPort.getClassifier();
				String vdec_sig_in_type_name = Name_of_Reference_data_classifier(data_classifier);
				if(myLib.Is_PredefinedType(vdec_sig_in_type_name)) {
					PredefinedTypeImpl pt1 = myLib.STree_mkPredefinedType(vdec_sig_in_type_name);
					lparam3.add(pt1);
				}
				else {
					Identifier vdec_sig_in_type = myLib.mkIdentifier(vdec_sig_in_type_name);
					//myLib.mkPredefinedType(SignalOperators.opinteger)
					lparam3.add(vdec_sig_in_type);
					//lparam3.add(myLib.mkIdentifier(vdec_sig_in_type_name+"_INIT"));
				}

				ArrayList<ExprSignal> lin3 = new ArrayList<ExprSignal>();
				String str_sig_Frozen_time_event = pn + "_Frozen_time_event";
				
				lin3.add(myLib.STree_mkFieldAccess(TIME1_ID, str_sig_Frozen_time_event));
				lin3.add(myLib.mkIdentifier(pn));
				String ref_time = aadlColAPI.Port_Get_InputTime_Time1_str(thread_impl, dataPort);
				if (ref_time.equals(DISPATCH_ID)) {
					lin3.add(myLib.STree_mkFieldAccess(CTL1_ID, DISPATCH_ID));
				}
				else if (ref_time.equals(START_ID)) {
					lin3.add(myLib.mkIdentifier(START_ID));
				}
				else if (ref_time.equals( COMPLETION_ID)) {
					lin3.add(myLib.STree_mkFieldAccess(CTL2_ID, COMPLETE_ID));
				}
				else {
					lin3.add(myLib.mkIdentifier(ref_time));
				}

				ArrayList<ExprSignal> lout3 = new ArrayList<ExprSignal>();
				lout3.add(myLib.mkIdentifier(Name_of_Local_Signal(pn)));

				// add local declaration l_xx to list vdecl
				SignalsDeclarationImpl vdec_sig_in = new SignalsDeclarationImpl();
				if(myLib.Is_PredefinedType(vdec_sig_in_type_name)) {
					PredefinedTypeImpl pt2 = myLib.STree_mkPredefinedType(vdec_sig_in_type_name);
					vdec_sig_in = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(Name_of_Local_Signal(pn)), pt2, null);
				}
				else {
					Identifier vdec_sig_in_type2 = myLib.mkIdentifier(vdec_sig_in_type_name);
					vdec_sig_in = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(Name_of_Local_Signal(pn)), vdec_sig_in_type2, null);
				}
				vdecl.add(vdec_sig_in);
				vdecl_behavior_in.add(myLib.mkIdentifier(Name_of_Local_Signal(pn)));

				//process call
				SignalsDefinitionImpl lproc2 = myLib.STree_mkCompleteProcessCallEquation(id2, lparam3, lin3, lout3);
				myLib.STree_addExpressionOfProcessToaModel(thread_process, lproc2);
				myLib.STree_addComment(lproc2, " process instance of in data port " + pn, false);

				//add an equation for clock synchronization in the body: l_xx ^= Start
				ExprSignal leftExp = myLib.mkIdentifier(Name_of_Local_Signal(pn));
				ExprSignal rightExp = myLib.mkIdentifier(START_ID);
				ASME2SSME_InDataPort_Sync(leftExp, rightExp, thread_process);
			}
		}
	}


	/** Translates the out data ports of a thread implementation.
	 * <p>
	 * For each out data port, create a Signal process (ASME2SSME_Thread_OutDataPort()).
	 * Add this declaration to thread_process's declaration part, and add a process call in the body.
	 * Add an event signal xx_Output_time_event to the interface of thread_process.
	 * Local signal declarations are added.
	 *
	 * @param thread_process a Signal process that represents a thread implementation component
	 * @param list_outd_port a list of out data ports of a thread
	 * @param thread_impl a thread implementation component
	 * @param vdecl a list of local declaration
	 * @param vdecl_behavior_param
	 * @param vdecl_behavior_in
	 * @param vdecl_behavior_out
	 * @param vdecl_property_param
	 * @param vdecl_property_in
	 * @param vdecl_property_out
	 */
	private void ASME2SSME_ThreadImpl_OutDataPorts(ProcessModelImpl thread_process, ArrayList<DataPort> list_outd_port,
			ThreadImplementation thread_impl, ArrayList<Declaration> vdecl,
			ArrayList<ExprParam> vdecl_behavior_param, ArrayList<ExprSignal> vdecl_behavior_in,
			ArrayList<ExprSignal> vdecl_behavior_out, ArrayList<ExprParam> vdecl_property_param,
			ArrayList<ExprSignal> vdecl_property_in, ArrayList<ExprSignal> vdecl_property_out) {


		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_OutDataPorts " + thread_impl.getName());

		if(list_outd_port.size()>0) {
			ASME2SSME_Translate_DataPort asme2ssme_dataport = new ASME2SSME_Translate_DataPort();

			Iterator<?> itd2 = list_outd_port.iterator();
			while (itd2.hasNext())
			{
				DataPort dataPort2 = (DataPort) itd2.next();
				//translate as a Signal process
				ProcessModelImpl outdataport_process = asme2ssme_dataport.ASME2SSME_Thread_OutDataPort(thread_impl, dataPort2);
				//add to declaration
				myLib.STree_addProcessDeclaration(thread_process, outdataport_process);
				//create a process call, and add it to the body
				Identifier p_id = outdataport_process.getIdentifier();
				String str_p_id = myLib.STree_GetStringName(p_id);
				IdentifierImpl p_id_impl = myLib.mkIdentifier(str_p_id);
				String str_p_id_impl = dataPort2.getName();

				ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();

				DataClassifier data_classifier = (DataClassifier) dataPort2.getClassifier();
				String vdec_sig_in_type_name = Name_of_Reference_data_classifier(data_classifier);
				if(myLib.Is_PredefinedType(vdec_sig_in_type_name)) {
					PredefinedTypeImpl pt1 = myLib.STree_mkPredefinedType(vdec_sig_in_type_name);
					lparams.add(pt1);
				}
				else {
					Identifier vdec_sig_in_type = myLib.mkIdentifier(vdec_sig_in_type_name);//myLib.mkPredefinedType(SignalOperators.opinteger)
					lparams.add(vdec_sig_in_type);
				}

				ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();
				String str_sig_Output_time_event = str_p_id_impl + "_Output_time_event";
				lins.add(myLib.STree_mkFieldAccess( TIME1_ID, str_sig_Output_time_event));
				lins.add(myLib.mkIdentifier(Name_of_Local_Signal(str_p_id_impl)));
				// add local declaration l_xx to list vdecl
				SignalsDeclarationImpl vdec_sig_in = new SignalsDeclarationImpl();
				if(myLib.Is_PredefinedType(vdec_sig_in_type_name)) {
					PredefinedTypeImpl pt2 = myLib.STree_mkPredefinedType(vdec_sig_in_type_name);
					vdec_sig_in = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(Name_of_Local_Signal(str_p_id_impl)), pt2, null);
				}
				else {
					Identifier vdec_sig_in_type2 = myLib.mkIdentifier(vdec_sig_in_type_name);
					vdec_sig_in = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(Name_of_Local_Signal(str_p_id_impl)), vdec_sig_in_type2, null);
				}
				vdecl.add(vdec_sig_in);
				vdecl_behavior_out.add(myLib.mkIdentifier(Name_of_Local_Signal(str_p_id_impl)));

				String ref_time = aadlColAPI.Port_Get_OutputTime_Time1_str(thread_impl, dataPort2);
				if (ref_time.equals(START_ID)) {
					lins.add(myLib.mkIdentifier(START_ID));
				}
				else if (ref_time.equals( COMPLETION_ID)) {
					Trace("ref_time = Completion");
					lins.add(myLib.STree_mkFieldAccess( CTL2_ID, COMPLETE_ID));
				}
				else if (ref_time.equals(DEADLINE_ID)) {
					lins.add(myLib.STree_mkFieldAccess(CTL1_ID, DEADLINE_ID));
				}
				else {
					Trace("ref_time = " + ref_time);
					lins.add(myLib.mkIdentifier(ref_time));
				}

				ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
				louts.add(myLib.mkIdentifier(str_p_id_impl));

				SignalsDefinitionImpl lproc3 = myLib.STree_mkCompleteProcessCallEquation(p_id_impl, lparams, lins, louts);
				myLib.STree_addExpressionOfProcessToaModel(thread_process, lproc3);
				myLib.STree_addComment(lproc3, " process instance of out data port " + str_p_id_impl, false);
			}
		}
	}


	/** Translates the ports of a thread implementation t.
	 * <p>
	 * Translate each in/out event/data/event data port.
	 *
	 * @param vprocess a Signal process that represents a thread implementation component
	 * @param t a thread implementation component
	 * @param vdecl a list of local declaration
	 * @param vdecl_behavior_param
	 * @param vdecl_behavior_in
	 * @param vdecl_behavior_out
	 * @param vdecl_property_param
	 * @param vdecl_property_in
	 * @param vdecl_property_out
	 */
	private void ASME2SSME_ThreadImpl_Ports(ProcessModelImpl vprocess, ThreadImplementation t, ArrayList<Declaration> vdecl,
			ArrayList<ExprParam> vdecl_behavior_param, ArrayList<ExprSignal> vdecl_behavior_in,
			ArrayList<ExprSignal> vdecl_behavior_out, ArrayList<ExprParam> vdecl_property_param,
			ArrayList<ExprSignal> vdecl_property_in, ArrayList<ExprSignal> vdecl_property_out) {

		ThreadType pt = (ThreadType) aadlColAPI.ComponentImpl_GetComponentType(t);

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_Ports " + t.getName());

		//In data port
		ArrayList<DataPort> list_ind_port = aadlColAPI.Component_GetInDataPort(pt);
		ASME2SSME_ThreadImpl_InDataPorts(vprocess, list_ind_port, t, vdecl,
				vdecl_behavior_param, vdecl_behavior_in, vdecl_behavior_out,
				vdecl_property_param, vdecl_property_in, vdecl_property_out);

		//Out data port
		ArrayList<DataPort> list_outd_port = aadlColAPI.Component_GetOutDataPort(pt);
		ASME2SSME_ThreadImpl_OutDataPorts(vprocess, list_outd_port, t, vdecl,
				vdecl_behavior_param, vdecl_behavior_in, vdecl_behavior_out,
				vdecl_property_param, vdecl_property_in, vdecl_property_out);

		//in event port
		ArrayList<EventPort> list_ine_port = aadlColAPI.Component_GetInEventPort(pt);
		ASME2SSME_ThreadImpl_InEventPorts(vprocess, list_ine_port, t, vdecl,
				vdecl_behavior_param, vdecl_behavior_in, vdecl_behavior_out,
				vdecl_property_param, vdecl_property_in, vdecl_property_out);

		//out event port
		ArrayList<EventPort> list_oute_port = aadlColAPI.Component_GetOutEventPort(pt);
		ASME2SSME_ThreadImpl_OutEventPorts(vprocess, list_oute_port, t, vdecl,
				vdecl_behavior_param, vdecl_behavior_in, vdecl_behavior_out,
				vdecl_property_param, vdecl_property_in, vdecl_property_out);

		//in event data port
		ArrayList<EventDataPort> list_ined_port = aadlColAPI.Component_GetInEventDataPort(pt);
		ASME2SSME_ThreadImpl_InEventDataPorts(vprocess, list_ined_port, t, vdecl,
				vdecl_behavior_param, vdecl_behavior_in, vdecl_behavior_out,
				vdecl_property_param, vdecl_property_in, vdecl_property_out);

		//out event data port
		ArrayList<EventDataPort> list_outed_port = aadlColAPI.Component_GetOutEventDataPort(pt);
		ASME2SSME_ThreadImpl_OutEventDataPorts(vprocess, list_outed_port, t, vdecl,
				vdecl_behavior_param, vdecl_behavior_in, vdecl_behavior_out,
				vdecl_property_param, vdecl_property_in, vdecl_property_out);
	}


	/** Translates the in event ports of a thread implementation.
	 * <p>
	 * For each in event port, create a Signal process (ASME2SSME_Thread_InEventPort()).
	 * Add this declaration to thread_process's declaration part, and add a process call in the body.
	 * Local signal declarations are added.
	 * The clock of local signal l_xx is synchronized with Start: l_xx ^= Start (ASME2SSME_InEventPort_Sync())
	 *
	 * @param thread_process a Signal process that represents a thread implementation component
	 * @param list_ine_port a list of in event ports of a thread
	 * @param thread_impl a thread implementation component
	 * @param vdecl a list of local declaration
	 * @param vdecl_behavior_param
	 * @param vdecl_behavior_in
	 * @param vdecl_behavior_out
	 * @param vdecl_property_param
	 * @param vdecl_property_in
	 * @param vdecl_property_out
	 */
	private void ASME2SSME_ThreadImpl_InEventPorts(ProcessModelImpl thread_process, ArrayList<EventPort> list_ine_port,
			ThreadImplementation thread_impl, ArrayList<Declaration> vdecl,
			ArrayList<ExprParam> vdecl_behavior_param, ArrayList<ExprSignal> vdecl_behavior_in,
			ArrayList<ExprSignal> vdecl_behavior_out, ArrayList<ExprParam> vdecl_property_param,
			ArrayList<ExprSignal> vdecl_property_in, ArrayList<ExprSignal> vdecl_property_out) {


		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_InEventPorts " + thread_impl.getName());

		if(list_ine_port.size()>0) {
			ASME2SSME_Translate_EventPort asme2ssme_eventport = new ASME2SSME_Translate_EventPort();

			Iterator<?> ite1 = list_ine_port.iterator();
			while (ite1.hasNext())
			{
				EventPort p1 = (EventPort) ite1.next();
				//translate as a Signal process
				ProcessModelImpl p1_process = asme2ssme_eventport.ASME2SSME_Thread_InEventPort(thread_impl, p1);
				//add to declaration
				myLib.STree_addProcessDeclaration(thread_process, p1_process);

				//create a process call, and add it to the body
				//process call
				Identifier id1 = p1_process.getIdentifier();
				String id_name = myLib.STree_GetStringName(id1);
				IdentifierImpl id2 = myLib.mkIdentifier(id_name);
				String pn = p1.getName();

				//parameters--------------------------------------------------------------//
				ArrayList<ExprParam> lparam3 = new ArrayList<ExprParam>();
				//parameter1:queue_size
				String queue_size = aadlColAPI.Port_Get_QueueSize_str(thread_impl, p1);
				ConstantIntegerImpl ci1 = new ConstantIntegerImpl();
				ci1.setAtomString(queue_size);		//default value of queue size is 1
				lparam3.add(ci1);

				//parameter2: dequeue_number
				String dequeue_number = aadlColAPI.Port_Get_Dequeue_Number(thread_impl, p1);
				ConstantIntegerImpl ci2 = new ConstantIntegerImpl();
				ci2.setAtomString(dequeue_number);
				lparam3.add(ci2);

				//inputs----------------------------------------------------------------//
				ArrayList<ExprSignal> lin3 = new ArrayList<ExprSignal>();
				String str_sig_Frozen_time_event = pn + "_Frozen_time_event";
				//input1: time1.xx_Frozen_time_event
				lin3.add(myLib.STree_mkFieldAccess( TIME1_ID, str_sig_Frozen_time_event));
				//input2: xx
				lin3.add(myLib.mkIdentifier(pn));
				//input3: Reference_time_event
				String ref_time = aadlColAPI.Port_Get_InputTime_Time1_str(thread_impl, p1);
				if (ref_time.equals(DISPATCH_ID)) {
					lin3.add(myLib.STree_mkFieldAccess(CTL1_ID, DISPATCH_ID));
				}
				else if (ref_time.equals(START_ID)) {
					lin3.add(myLib.mkIdentifier(START_ID));
				}
				else if (ref_time.equals( COMPLETION_ID)) {
					lin3.add(myLib.STree_mkFieldAccess( CTL2_ID, COMPLETE_ID));
				}
				else {
					lin3.add(myLib.mkIdentifier(ref_time));
				}
				//input4: ctl1.Initialize
				//				lin3.add(myLib.STree_mkFieldAccess(CTL1_ID, "Initialize"));

				//outputs------------------------------------------------------//
				ArrayList<ExprSignal> lout3 = new ArrayList<ExprSignal>();
				//output1: l_xx
				lout3.add(myLib.mkIdentifier(Name_of_Local_Signal(pn)));

				// add local declaration l_xx to list vdecl
				SignalsDeclarationImpl vdec_sig_in = myLib.mkBooleanDecl(Name_of_Local_Signal(pn));
				vdecl.add(vdec_sig_in);
				vdecl_behavior_in.add(myLib.mkIdentifier(Name_of_Local_Signal(pn)));

				//process call----------------------------------------------------------//
				SignalsDefinitionImpl lproc2 = myLib.STree_mkCompleteProcessCallEquation(id2, lparam3, lin3, lout3);
				myLib.STree_addExpressionOfProcessToaModel(thread_process, lproc2);
				myLib.STree_addComment(lproc2, " process instance of in event port " + pn, false);

				//add an equation for clock synchronization in the body: l_xx ^= Start
				ExprSignal leftExp = myLib.mkIdentifier(Name_of_Local_Signal(pn));
				ExprSignal rightExp = myLib.mkIdentifier(START_ID);
				ASME2SSME_InEventPort_Sync(leftExp, rightExp, thread_process);
			}
		}
	}


	/**
	 * Translates out event data ports of thread
	 *
	 * @param thread_process
	 * @param list_outed_port
	 * @param thread_impl
	 * @param vdecl
	 * @param vdecl_behavior_param
	 * @param vdecl_behavior_in
	 * @param vdecl_behavior_out
	 * @param vdecl_property_param
	 * @param vdecl_property_in
	 * @param vdecl_property_out
	 */
	private void ASME2SSME_ThreadImpl_OutEventDataPorts(ProcessModelImpl thread_process, ArrayList<EventDataPort> list_outed_port,
			ThreadImplementation thread_impl, ArrayList<Declaration> vdecl,
			ArrayList<ExprParam> vdecl_behavior_param, ArrayList<ExprSignal> vdecl_behavior_in,
			ArrayList<ExprSignal> vdecl_behavior_out, ArrayList<ExprParam> vdecl_property_param,
			ArrayList<ExprSignal> vdecl_property_in, ArrayList<ExprSignal> vdecl_property_out) {


		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_OutEventDataPorts " + thread_impl.getName());

		if(list_outed_port.size()>0) {
			ASME2SSME_Translate_EventDataPort asme2ssme_edp = new ASME2SSME_Translate_EventDataPort();
			Iterator<?> ited2 = list_outed_port.iterator();
			while (ited2.hasNext())
			{
				EventDataPort p1 = (EventDataPort) ited2.next();

				//translate as a Signal process
				ProcessModelImpl p1_process = asme2ssme_edp.ASME2SSME_Thread_OutEventDataPort(thread_impl, p1);

				//add to declaration
				myLib.STree_addProcessDeclaration(thread_process, p1_process);

				//create a process call, and add it to the body
				Identifier p_id = p1_process.getIdentifier();
				String str_p_id = myLib.STree_GetStringName(p_id);
				IdentifierImpl p_id_impl = myLib.mkIdentifier(str_p_id);
				String str_p_id_impl = p1.getName();

				DataClassifier data_classifier = (DataClassifier) p1.getClassifier();
				String vdec_sig_in_type_name = Name_of_Reference_data_classifier(data_classifier);
				// Identifier vdec_sig_in_type = myLib.mkIdentifier(vdec_sig_in_type_name);//myLib.mkPredefinedType(SignalOperators.opinteger)

				//parameters----------------------------------------------------------------//
				ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
				//parameter1: def_value (init_value)
				lparams.add(myLib.mkIdentifier(vdec_sig_in_type_name+"_INIT"));

				//parameter2:queue_size
				String queue_size = aadlColAPI.Port_Get_QueueSize_str(thread_impl, p1);
				ConstantIntegerImpl ci1 = new ConstantIntegerImpl();
				ci1.setAtomString(queue_size);
				lparams.add(ci1);

				//inputs----------------------------------------------------------------------//
				ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();

				//input1: time1.xx_Output_time_event
				String str_sig_Output_time_event = str_p_id_impl + "_Output_time_event";
				lins.add(myLib.STree_mkFieldAccess( TIME1_ID, str_sig_Output_time_event));

				//input2: l_xx
				lins.add(myLib.mkIdentifier(Name_of_Local_Signal(str_p_id_impl)));

				//input3: Reference_time
				String ref_time = aadlColAPI.Port_Get_OutputTime_Time1_str(thread_impl, p1);
				if (ref_time.equals(START_ID)) {
					lins.add(myLib.mkIdentifier(START_ID));
				}
				else if (ref_time.equals( COMPLETION_ID)) {
					Trace("ref_time = Completion");
					lins.add(myLib.STree_mkFieldAccess( CTL2_ID, COMPLETE_ID));
				}
				else if (ref_time.equals(DEADLINE_ID)) {
					lins.add(myLib.STree_mkFieldAccess(CTL1_ID, DEADLINE_ID));
				}
				else {
					Trace("ref_time = " + ref_time);
					lins.add(myLib.mkIdentifier(ref_time));
				}

				// input4: ctl1.Initialize
				// lins.add(myLib.STree_mkFieldAccess(CTL1_ID, "Initialize"));

				//outputs-------------------------------------------//
				ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
				louts.add(myLib.mkIdentifier(str_p_id_impl));

				SignalsDefinitionImpl lproc3 = myLib.STree_mkCompleteProcessCallEquation(p_id_impl, lparams, lins, louts);
				myLib.STree_addExpressionOfProcessToaModel(thread_process, lproc3);

				// add local declaration l_xx to list vdecl
				Identifier vdec_sig_in_type2 = myLib.mkIdentifier(vdec_sig_in_type_name);
				SignalsDeclarationImpl vdec_sig_in = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(Name_of_Local_Signal(str_p_id_impl)), vdec_sig_in_type2, null);
				vdecl.add(vdec_sig_in);
				vdecl_behavior_out.add(myLib.mkIdentifier(Name_of_Local_Signal(str_p_id_impl)));
				myLib.STree_addComment(lproc3, " process instance of out event data port " + str_p_id_impl, false);
			}
		}
	}


	/** Translates the in event data ports of a thread implementation.
	 * <p>
	 * For each in event data port, create a Signal process (ASME2SSME_Thread_InEventDataPort()).
	 * Add this declaration to thread_process's declaration part, and add a process call in the body.
	 * Add an event signal xx_Frozen_time_event to the interface of thread_process.
	 * Local signal declarations are added.
	 * The clock of local signal l_xx is synchronized with Start: l_xx ^= Start (ASME2SSME_InEventDataPort_Sync())
	 *
	 * @param thread_process a Signal process that represents a thread implementation component
	 * @param list_ind_port a list of in data ports of a thread
	 * @param thread_impl a thread implementation component
	 * @param vdecl a list of local declaration
	 * @param vdecl_behavior_param
	 * @param vdecl_behavior_in
	 * @param vdecl_behavior_out
	 * @param vdecl_property_param
	 * @param vdecl_property_in
	 * @param vdecl_property_out
	 */
	private void ASME2SSME_ThreadImpl_InEventDataPorts(ProcessModelImpl thread_process, ArrayList<EventDataPort> list_ined_port,
			ThreadImplementation thread_impl, ArrayList<Declaration> vdecl,
			ArrayList<ExprParam> vdecl_behavior_param, ArrayList<ExprSignal> vdecl_behavior_in,
			ArrayList<ExprSignal> vdecl_behavior_out, ArrayList<ExprParam> vdecl_property_param,
			ArrayList<ExprSignal> vdecl_property_in, ArrayList<ExprSignal> vdecl_property_out) {


		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_InEventDataPorts " + thread_impl.getName());

		if(list_ined_port.size()>0) {
			ASME2SSME_Translate_EventDataPort asme2ssme_edp = new ASME2SSME_Translate_EventDataPort();
			Iterator<?> ited1 = list_ined_port.iterator();
			while (ited1.hasNext())
			{
				EventDataPort p1 = (EventDataPort) ited1.next();
				//translate as a Signal process
				ProcessModelImpl p1_process = asme2ssme_edp.ASME2SSME_Thread_InEventDataPort(thread_impl, p1);
				//add to declaration
				myLib.STree_addProcessDeclaration(thread_process, p1_process);

				//create a process call, and add it to the body
				//process call
				Identifier id1 = p1_process.getIdentifier();
				String id_name = myLib.STree_GetStringName(id1);
				IdentifierImpl id2 = myLib.mkIdentifier(id_name);
				String pn = p1.getName();

				DataClassifier data_classifier = (DataClassifier) p1.getClassifier();
				String sig_type_name = Name_of_Reference_data_classifier(data_classifier);
				// Identifier vdec_sig_in_type = myLib.mkIdentifier(sig_type_name);//myLib.mkPredefinedType(SignalOperators.opinteger)

				//parameters--------------------------------------------------------------//
				//parameter1: def_value (init_value)
				ArrayList<ExprParam> lparam3 = new ArrayList<ExprParam>();
				lparam3.add(myLib.mkIdentifier(sig_type_name+"_INIT"));

				//parameter2:queue_size
				String queue_size = aadlColAPI.Port_Get_QueueSize_str(thread_impl, p1);
				ConstantIntegerImpl ci1 = new ConstantIntegerImpl();
				ci1.setAtomString(queue_size);		//default value of queue size is 1
				lparam3.add(ci1);

				//parameter3: dequeue_number
				String dequeue_number = aadlColAPI.Port_Get_Dequeue_Number(thread_impl, p1);
				ConstantIntegerImpl ci2 = new ConstantIntegerImpl();
				ci2.setAtomString(dequeue_number);
				lparam3.add(ci2);

				//inputs----------------------------------------------------------------//
				ArrayList<ExprSignal> lin3 = new ArrayList<ExprSignal>();
				String str_sig_Frozen_time_event = pn + "_Frozen_time_event";
				//input1: time1.xx_Frozen_time_event
				lin3.add(myLib.STree_mkFieldAccess( TIME1_ID, str_sig_Frozen_time_event));
				//input2: xx
				lin3.add(myLib.mkIdentifier(pn));
				//input3: Reference_time_event
				String ref_time = aadlColAPI.Port_Get_InputTime_Time1_str(thread_impl, p1);
				if (ref_time.equals(DISPATCH_ID)) {
					lin3.add(myLib.STree_mkFieldAccess(CTL1_ID, DISPATCH_ID));
				}
				else if (ref_time.equals(START_ID)) {
					lin3.add(myLib.mkIdentifier(START_ID));
				}
				else if (ref_time.equals( COMPLETION_ID)) {
					lin3.add(myLib.STree_mkFieldAccess( CTL2_ID, COMPLETE_ID));
				}
				else {
					lin3.add(myLib.mkIdentifier(ref_time));
				}
				// input4: ctl1.Initialize
				// lin3.add(myLib.STree_mkFieldAccess(CTL1_ID, "Initialize"));

				//outputs------------------------------------------------------//
				ArrayList<ExprSignal> lout3 = new ArrayList<ExprSignal>();
				//output1: l_xx
				lout3.add(myLib.mkIdentifier(Name_of_Local_Signal(pn)));

				// add local declaration l_xx to list vdecl
				Identifier vdec_sig_in_type2 = myLib.mkIdentifier(sig_type_name);
				SignalsDeclarationImpl vdec_sig_in = myLib.STree_mkSignalDeclaration(myLib.mkIdentifier(Name_of_Local_Signal(pn)), vdec_sig_in_type2, null);
				vdecl.add(vdec_sig_in);
				vdecl_behavior_in.add(myLib.mkIdentifier(Name_of_Local_Signal(pn)));

				//process call----------------------------------------------------------//
				SignalsDefinitionImpl lproc2 = myLib.STree_mkCompleteProcessCallEquation(id2, lparam3, lin3, lout3);
				myLib.STree_addExpressionOfProcessToaModel(thread_process, lproc2);
				myLib.STree_addComment(lproc2, " process instance of in event data port " + pn, false);

				//add an equation for clock synchronization in the body: l_xx ^= Start
				ExprSignal leftExp = myLib.mkIdentifier(Name_of_Local_Signal(pn));
				ExprSignal rightExp = myLib.mkIdentifier(START_ID);
				ASME2SSME_InEventDataPort_Sync(leftExp, rightExp, thread_process);
			}
		}
	}


	/**
	 * Translates out event ports of thread
	 *
	 * @param thread_process
	 * @param list_oute_port
	 * @param thread_impl
	 * @param vdecl
	 * @param vdecl_behavior_param
	 * @param vdecl_behavior_in
	 * @param vdecl_behavior_out
	 * @param vdecl_property_param
	 * @param vdecl_property_in
	 * @param vdecl_property_out
	 */
	private void ASME2SSME_ThreadImpl_OutEventPorts(ProcessModelImpl thread_process, ArrayList<EventPort> list_oute_port,
			ThreadImplementation thread_impl, ArrayList<Declaration> vdecl,
			ArrayList<ExprParam> vdecl_behavior_param, ArrayList<ExprSignal> vdecl_behavior_in,
			ArrayList<ExprSignal> vdecl_behavior_out, ArrayList<ExprParam> vdecl_property_param,
			ArrayList<ExprSignal> vdecl_property_in, ArrayList<ExprSignal> vdecl_property_out) {


		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_OutEventPorts " + thread_impl.getName());

		if (list_oute_port.size()>0) {
			ASME2SSME_Translate_EventPort asme2ssme_eventport = new ASME2SSME_Translate_EventPort();
			Iterator<?> ite2 = list_oute_port.iterator();
			while (ite2.hasNext())
			{
				EventPort p1 =  (EventPort) ite2.next();

				//translate as a Signal process
				ProcessModelImpl p1_process = asme2ssme_eventport.ASME2SSME_Thread_OutEventPort(thread_impl, p1);

				//add to declaration
				myLib.STree_addProcessDeclaration(thread_process, p1_process);

				//create a process call, and add it to the body
				Identifier p_id = p1_process.getIdentifier();
				String str_p_id = myLib.STree_GetStringName(p_id);
				IdentifierImpl p_id_impl = myLib.mkIdentifier(str_p_id);
				String str_p_id_impl = p1.getName();

				//parameters----------------------------------------------------------------//
				ArrayList<ExprParam> lparams = new ArrayList<ExprParam>();
				//parameter1:queue_size
				String queue_size = aadlColAPI.Port_Get_QueueSize_str(thread_impl, p1);
				ConstantIntegerImpl ci1 = new ConstantIntegerImpl();
				ci1.setAtomString(queue_size);
				lparams.add(ci1);

				//inputs----------------------------------------------------------------------//
				ArrayList<ExprSignal> lins = new ArrayList<ExprSignal>();

				//input1: time1.xx_Output_time_event
				String str_sig_Output_time_event = str_p_id_impl + "_Output_time_event";
				lins.add(myLib.STree_mkFieldAccess( TIME1_ID, str_sig_Output_time_event));

				//input2: l_xx
				lins.add(myLib.mkIdentifier(Name_of_Local_Signal(str_p_id_impl)));

				//input3: Reference_time
				String ref_time = aadlColAPI.Port_Get_OutputTime_Time1_str(thread_impl, p1);
				if (ref_time.equals(START_ID)) {
					lins.add(myLib.mkIdentifier(START_ID));
				}
				else if (ref_time.equals( COMPLETION_ID)) {
					Trace("ref_time = Completion");
					lins.add(myLib.STree_mkFieldAccess( CTL2_ID, COMPLETE_ID));
				}
				else if (ref_time.equals(DEADLINE_ID)) {
					lins.add(myLib.STree_mkFieldAccess(CTL1_ID, DEADLINE_ID));
				}
				else {
					Trace("ref_time = " + ref_time);
					lins.add(myLib.mkIdentifier(ref_time));
				}

				// input4: ctl1.Initialize
				// lins.add(myLib.STree_mkFieldAccess(CTL1_ID, "Initialize"));

				//outputs-------------------------------------------//
				ArrayList<ExprSignal> louts = new ArrayList<ExprSignal>();
				louts.add(myLib.mkIdentifier(str_p_id_impl));

				SignalsDefinitionImpl lproc3 = myLib.STree_mkCompleteProcessCallEquation(p_id_impl, lparams, lins, louts);
				myLib.STree_addExpressionOfProcessToaModel(thread_process, lproc3);

				// add local declaration l_xx to list vdecl
				SignalsDeclarationImpl vdec_sig_in = myLib.mkBooleanDecl(Name_of_Local_Signal(str_p_id_impl));
				vdecl.add(vdec_sig_in);
				vdecl_behavior_out.add(myLib.mkIdentifier(Name_of_Local_Signal(str_p_id_impl)));
				myLib.STree_addComment(lproc3, " process instance of out event port " + str_p_id_impl, false);
			}
		}
	}



	/**
	 * Translates requires data accesses of thread:
	 * for each requires data access,
	 * add FIFO_TIME x to vdecl_behavior_out (for behavior subprocess call)
	 *
	 * @param thread_process
	 * @param list
	 * @param thread_impl
	 * @param vdecl
	 * @param vdecl_behavior_param
	 * @param vdecl_behavior_in
	 * @param vdecl_behavior_out
	 * @param vdecl_property_param
	 * @param vdecl_property_in
	 * @param vdecl_property_out
	 */
	private void ASME2SSME_ThreadImpl_RequiresDataAccesses(ProcessModelImpl thread_process, ArrayList<DataAccess> list,
			ThreadImplementation thread_impl, ArrayList<Declaration> vdecl,
			ArrayList<ExprParam> vdecl_behavior_param, ArrayList<ExprSignal> vdecl_behavior_in,
			ArrayList<ExprSignal> vdecl_behavior_out, ArrayList<ExprParam> vdecl_property_param,
			ArrayList<ExprSignal> vdecl_property_in, ArrayList<ExprSignal> vdecl_property_out) {


		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_RequiresDataAccesses " + thread_impl.getName());

		if (list.size()>0) {
			for(int m =0; m< list.size(); m++)
			{
				DataAccess p1 =  (DataAccess) list.get(m);
				String name = p1.getName();
				String fname = Name_of_DataBundle(name);
				vdecl_behavior_out.add(myLib.mkIdentifier(fname));
			}
		}
	}



	/**
	 * Translates requires subprogram accesses of thread:
	 * for each requires subprogram access,
	 * add event x to vdecl_behavior_out (for behavior subprocess call)
	 *
	 * @param thread_process
	 * @param list
	 * @param thread_impl
	 * @param vdecl
	 * @param vdecl_behavior_param
	 * @param vdecl_behavior_in
	 * @param vdecl_behavior_out
	 * @param vdecl_property_param
	 * @param vdecl_property_in
	 * @param vdecl_property_out
	 */
	private void ASME2SSME_ThreadImpl_RequiresSubprogramAccesses(ProcessModelImpl thread_process, ArrayList<SubprogramAccess> list,
			ThreadImplementation thread_impl, ArrayList<Declaration> vdecl,
			ArrayList<ExprParam> vdecl_behavior_param, ArrayList<ExprSignal> vdecl_behavior_in,
			ArrayList<ExprSignal> vdecl_behavior_out, ArrayList<ExprParam> vdecl_property_param,
			ArrayList<ExprSignal> vdecl_property_in, ArrayList<ExprSignal> vdecl_property_out) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_RequiresSubprogramAccesses " + thread_impl.getName());

		if (list.size()>0) {

			for(int m =0; m< list.size(); m++)
			{
				SubprogramAccess p1 =  (SubprogramAccess) list.get(m);
				String name = p1.getName();
				String ename = Name_of_SubprogramCall(name);
				vdecl_behavior_out.add(myLib.mkIdentifier(ename));
			}
		}
	}


	/**
	 * Translates subprogram calls of thread:
	 * for each subprogram call,
	 * add event x to vdecl_behavior_out (for behavior subprocess call)
	 *
	 * @param thread_process
	 * @param list
	 * @param thread_impl
	 * @param vdecl
	 * @param vdecl_behavior_param
	 * @param vdecl_behavior_in
	 * @param vdecl_behavior_out
	 * @param vdecl_property_param
	 * @param vdecl_property_in
	 * @param vdecl_property_out
	 */
	private void ASME2SSME_ThreadImpl_SubprogramCalls(ProcessModelImpl thread_process, ArrayList<SubprogramCall> list,
			ThreadImplementation thread_impl, ArrayList<Declaration> vdecl,
			ArrayList<ExprParam> vdecl_behavior_param, ArrayList<ExprSignal> vdecl_behavior_in,
			ArrayList<ExprSignal> vdecl_behavior_out, ArrayList<ExprParam> vdecl_property_param,
			ArrayList<ExprSignal> vdecl_property_in, ArrayList<ExprSignal> vdecl_property_out) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_ThreadImpl_SubprogramCalls " + thread_impl.getName());

		if (list.size()>0) {

			for(int m =0; m< list.size(); m++)
			{
				SubprogramCall p1 =  (SubprogramCall) list.get(m);
				String name = p1.getName();
				String ename = Name_of_SubprogramCall(name);
				vdecl_behavior_out.add(myLib.mkIdentifier(ename));
			}
		}
	}



	/**
	 * Returns an equation: Start := when ((Resume from Dispatch)=1)
	 *
	 * @param
	 * @return a Signal equation
	 */
	private SignalsDefinitionImpl ASME2SSME_Start_Assignment() {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_Start_Assignment ");

		ExprSignal leftExp = myLib.mkIdentifier(START_ID);
		//Resume from Dispatch
		FromImpl exp1 = myLib.STree_mkFrom(myLib.STree_mkFieldAccess(CTL1_ID, RESUME_ID), myLib.STree_mkFieldAccess(CTL1_ID, DISPATCH_ID));
		ConstantIntegerImpl exp2 = new ConstantIntegerImpl();
		exp2.setAtomString("1");
		//(Resume from Dispatch)=1
		EqualImpl equal = myLib.STree_mkEqual(exp1, exp2);
		//when ((Resume from Dispatch)=1)
		UnaryWhenImpl rightExp = myLib.STree_mkUnaryWhen(equal);
		//Start := when ((Resume from Dispatch)=1)
		SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		return eq;
	}


	/**
	 * Returns an ClockEquality: e1 ^= when Start
	 */
	private ClockEqualityImpl ASME2SSME_Execute_Sync() {
		return ASME2SSME_mk_Synchro(LABEL1, START_ID);
	}

	/**
	 * Returns an ClockEquality: id ^= when c
	 */
	private ClockEqualityImpl ASME2SSME_mk_Synchro(String id, String c ) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_mk_Sync");

		ExprSignal leftExp = myLib.mkIdentifier(id);
		//when Start
		UnaryWhenImpl rightExp = myLib.STree_mkUnaryWhen(myLib.mkIdentifier(c));
		//e1 ^= when Start
		ClockEqualityImpl clk_eq = new ClockEqualityImpl();
		clk_eq.getExprSignal().add(leftExp);
		clk_eq.getExprSignal().add(rightExp);
		return clk_eq;
	}


	/**
	 * Returns an ClockEquality that represents the clock of an output: o1 ^= when Start
	 */
	private ClockEqualityImpl ASME2SSME_Output_Sync(String out_name) {

		return ASME2SSME_mk_Synchro(out_name, START_ID);
	}



	/**
	 * Returns an equation that represents the read clock of a data access:
	 * x_ReadTime := when Start
	 * @param
	 * @return a Signal equation
	 */
	private SignalsDefinitionImpl ASME2SSME_RequiresDataAccess_ReadTime_Def(DataAccess dd) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_RequiresDataAccess_ReadTime_Def ");

		//xx.TRead
		String str1 = Name_of_DataBundle(dd.getName());
		ExprSignal leftExp = myLib.STree_mkFieldAccess(str1, TREAD_ID);
		//when Start
		UnaryWhenImpl rightExp = myLib.STree_mkUnaryWhen(myLib.mkIdentifier(START_ID));
		//xx_ReadTime := when Start
		SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		return eq;
	}



	/**
	 * Returns an equation that represents the write clock of a data access:
	 * x_WriteTime := when Start
	 * @param
	 * @return a Signal equation
	 */
	private SignalsDefinitionImpl ASME2SSME_RequiresDataAccess_WriteTime_Def(DataAccess dd) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_RequiresDataAccess_WriteTime_Def ");
		//xx.TWrite
		String str1 = Name_of_DataBundle(dd.getName());
		ExprSignal leftExp = myLib.STree_mkFieldAccess(str1, TWRITE_ID);
		//when Start
		UnaryWhenImpl rightExp = myLib.STree_mkUnaryWhen(myLib.mkIdentifier(START_ID));
		//xx_ReadTime := when Start
		SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		return eq;
	}



	/**
	 * Returns an equation that represents the reset clock of a data access:
	 * x_reset := ^0
	 * @param
	 * @return a Signal equation
	 */
	private SignalsDefinitionImpl ASME2SSME_RequiresDataAccess_Reset_Def(DataAccess dd) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_RequiresDataAccess_Reset_Def ");

		//xx.TReset
		String str1 = Name_of_DataBundle(dd.getName());
		ExprSignal leftExp = myLib.STree_mkFieldAccess(str1, TRESET_ID);
		//^0
		NullClockImpl null_clk = new NullClockImpl();
		//xx_reset := ^0
		SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, null_clk);
		return eq;
	}


	/**
	 * Returns an equation that represents the clock of a requires subprogram access:
	 * x := when Start
	 * @param
	 * @return a Signal equation
	 */
	private SignalsDefinitionImpl ASME2SSME_RequiresSubprogramAccess_Clock_Def(SubprogramAccess dd) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_RequiresSubprogramAccess_Clock_Def ");
		String str1 = Name_of_SubprogramCall(dd.getName());
		ExprSignal leftExp = myLib.mkIdentifier(str1);
		//when Start
		UnaryWhenImpl rightExp = myLib.STree_mkUnaryWhen(myLib.mkIdentifier(START_ID));
		//x := when Start
		SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		return eq;
	}


	/**
	 * Returns an equation that represents the clock of a subprogram call:
	 * x := when Start
	 * @param
	 * @return a Signal equation
	 */
	private SignalsDefinitionImpl ASME2SSME_SubprogramCall_Clock_Def(SubprogramCall dd) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_SubprogramCall_Clock_Def ");

		String str1 = Name_of_SubprogramCall(dd.getName());
		ExprSignal leftExp = myLib.mkIdentifier(str1);

		//	//^0
		//    NullClockImpl null_clk = new NullClockImpl();
		//when Start
		UnaryWhenImpl rightExp = myLib.STree_mkUnaryWhen(myLib.mkIdentifier(START_ID));

		//x := ^0
		SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		return eq;
	}



	/**
	 * for each output of thread, creates an equation: o ^= when Start,
	 * and adds it to the body of thread behavior (vprocess)
	 */
	private void ASME2SSME_Outputs_Sync(ProcessModelImpl vprocess, ThreadType pt) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_Outputs_Sync : " + pt.getName());


		ArrayList<DataPort> list_outd_port = aadlColAPI.Component_GetOutDataPort(pt);
		ArrayList<EventPort> list_oute_port = aadlColAPI.Component_GetOutEventPort(pt);
		ArrayList<EventDataPort> list_outed_port = aadlColAPI.Component_GetOutEventDataPort(pt);

		for (int i=0; i< list_outd_port.size(); i++) {
			DataPort x = list_outd_port.get(i);
			ClockEqualityImpl clk_eq1 = ASME2SSME_Output_Sync(x.getName());
			myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq1);
		}

		for (int j=0; j< list_oute_port.size(); j++) {
			EventPort y = list_oute_port.get(j);
			ClockEqualityImpl clk_eq2 = ASME2SSME_Output_Sync(y.getName());
			myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq2);
		}

		for (int k=0; k< list_outed_port.size(); k++) {
			EventDataPort z = list_outed_port.get(k);
			ClockEqualityImpl clk_eq3 = ASME2SSME_Output_Sync(z.getName());
			myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq3);
		}
	}



	/**
	 * for each requires data access x of thread, creates an equation: x.TRead := when Start,
	 * and adds it to the body of thread behavior (vprocess)
	 */
	private void ASME2SSME_RequiresDataAccesses_ReadTime_Def(ProcessModelImpl vprocess, ThreadType pt) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_RequiresDataAccesses_ReadTime_Def : " + pt.getName());


		ArrayList<DataAccess> list1 = aadlColAPI.Component_GetRequiresDataAccess(pt);

		for (int i=0; i< list1.size(); i++) {
			DataAccess x = list1.get(i);
			SignalsDefinitionImpl clk_eq1 = ASME2SSME_RequiresDataAccess_ReadTime_Def(x);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq1);
		}
	}



	/**
	 * for each requires data access x of thread, creates an equation: x.TWrite := when Start,
	 * and adds it to the body of thread behavior (vprocess)
	 */
	private void ASME2SSME_RequiresDataAccesses_WriteTime_Def(ProcessModelImpl vprocess, ThreadType pt) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_RequiresDataAccesses_WriteTime_Def : " + pt.getName());

		ArrayList<DataAccess> list1 = aadlColAPI.Component_GetRequiresDataAccess(pt);

		for (int i=0; i< list1.size(); i++) {
			DataAccess x = list1.get(i);
			SignalsDefinitionImpl clk_eq1 = ASME2SSME_RequiresDataAccess_WriteTime_Def(x);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq1);
		}


	}



	/**
	 * for each requires data access x of thread, creates an equation: x.TReset := ^0,
	 * and adds it to the body of thread behavior (vprocess)
	 */
	private void ASME2SSME_RequiresDataAccesses_Reset_Def(ProcessModelImpl vprocess, ThreadType pt) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_RequiresDataAccesses_Reset_Def : " + pt.getName());
		ArrayList<DataAccess> list1 = aadlColAPI.Component_GetRequiresDataAccess(pt);

		for (int i=0; i< list1.size(); i++) {
			DataAccess x = list1.get(i);
			SignalsDefinitionImpl clk_eq1 = ASME2SSME_RequiresDataAccess_Reset_Def(x);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq1);
		}
	}


	/**
	 * for each requires subprogram access x of thread, creates an equation: x := ^0,
	 * and adds it to the body of thread behavior (vprocess)
	 *
	 * @param vprocess a Signal process that represents the behavior part of a component
	 * @param pr a thread type
	 * @return
	 */
	private void ASME2SSME_RequiresSubprogramAccesses_Clock_Def(ProcessModelImpl vprocess, ThreadType pt) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_RequiresSubprogramAccesses_Def : " + pt.getName());
		ArrayList<SubprogramAccess> list1 = aadlColAPI.Component_GetRequiresSubprogramAccess(pt);
		for (int i=0; i< list1.size(); i++) {
			SubprogramAccess x = list1.get(i);
			SignalsDefinitionImpl clk_eq1 = ASME2SSME_RequiresSubprogramAccess_Clock_Def(x);
			myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq1);
		}
	}


	/**
	 * for each subprogram call x of thread, creates an equation: x := when Start,
	 * and adds it to the body of thread behavior (vprocess)
	 */
	private void ASME2SSME_SubprogramCalls_Clock_Def(ProcessModelImpl vprocess, ThreadImplementation t) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_SubprogramCalls_Clock_Def : " + t.getName());
		ArrayList<SubprogramCallSequence> callseqs = aadlColAPI.BehavioredImpl_Get_CallSequences(t);
		for(int i =0; i< callseqs.size(); i++) {
			SubprogramCallSequence callseq1 = callseqs.get(i);
			ArrayList<SubprogramCall> calls = aadlColAPI.SubprogramCallSequence_Get_SubprogramCalls(callseq1);

			for(int j=0; j< calls.size(); j++) {
				SubprogramCall call1 = calls.get(j);
				CalledSubprogram called_sp1 = aadlColAPI.SubprogramCall_Get_CalledSubprogram(call1);

				//if called_sp is a sub component of t: ? event sp1;
				if(aadlColAPI.Is_CalledSubprogram_Sub_Of_ComponentImpl(called_sp1, t)) {

				}
				else {
					//else (the called subprogram is from external): x := when Start
					SignalsDefinitionImpl clk_eq1 = ASME2SSME_SubprogramCall_Clock_Def(call1);
					myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq1);
				}
			}
		}
	}


	/**
	 * Returns an equation: ctl2.Complete := when Start
	 *
	 * @param
	 * @return a Signal equation
	 */
	private SignalsDefinitionImpl ASME2SSME_Complete_Def() {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_Complete_Def : ");
		ExprSignal leftExp = myLib.STree_mkFieldAccess( CTL2_ID, COMPLETE_ID);
		UnaryWhenImpl rightExp = myLib.STree_mkUnaryWhen(myLib.mkIdentifier(START_ID));
		//ctl2.Complete := when Start
		SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, rightExp);
		return eq;
	}


	/**
	 * Returns an equation: ctl2.Error := ^0
	 * @return
	 */
	private SignalsDefinitionImpl ASME2SSME_Error_Def() {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_Error_Def ");
		ExprSignal leftExp = myLib.STree_mkFieldAccess( CTL2_ID, ERROR_ID);
		//^0
		NullClockImpl null_clk = new NullClockImpl();
		//ctl2.Error := ^0
		SignalsDefinitionImpl eq = (SignalsDefinitionImpl) myLib.mkEquation( leftExp, null_clk);
		return eq;
	}


	/**
	 * Creates an ClockEquality: leftExp ^= rightExp,
	 * and adds it to the body of process (vprocess)
	 */
	private void ASME2SSME_InDataPort_Sync(ExprSignal leftExp, ExprSignal rightExp, ProcessModelImpl vprocess) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_InDataPort_Sync ");
		//eleftExp ^= rightExp
		ClockEqualityImpl clk_eq = new ClockEqualityImpl();
		clk_eq.getExprSignal().add(leftExp);
		clk_eq.getExprSignal().add(rightExp);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq);
	}


	/**
	 * Creates an ClockEquality: leftExp ^= rightExp,
	 * and adds it to the body of process (vprocess)
	 */
	private void ASME2SSME_InEventPort_Sync(ExprSignal leftExp, ExprSignal rightExp, ProcessModelImpl vprocess) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_InEventPort_Sync ");
		//eleftExp ^= rightExp
		ClockEqualityImpl clk_eq = new ClockEqualityImpl();
		clk_eq.getExprSignal().add(leftExp);
		clk_eq.getExprSignal().add(rightExp);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq);

	}


	/**
	 * Creates an ClockEquality: leftExp ^= rightExp,
	 * and adds it to the body of process (vprocess)
	 */
	private void ASME2SSME_InEventDataPort_Sync(ExprSignal leftExp, ExprSignal rightExp, ProcessModelImpl vprocess) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_InEventPort_Sync ");

		//eleftExp ^= rightExp
		ClockEqualityImpl clk_eq = new ClockEqualityImpl();
		clk_eq.getExprSignal().add(leftExp);
		clk_eq.getExprSignal().add(rightExp);
		myLib.STree_addExpressionOfProcessToaModel(vprocess, clk_eq);
	}


	/**
	 * Creates an external process
	 *
	 * process xx_external = (? ... ! ...) external;
	 *
	 * @param t
	 * @param t_name
	 * @return
	 */
	private ProcessModelImpl ASME2SSME_thread_external(ThreadImplementation t, String t_name) {

		Trace("\n ========== ASME2SSME_Translate_Thread: ASME2SSME_thread_external: " + t.getName());

		String name = t_name + "_external";
		ProcessModelImpl t_external = myLib.STree_mkSkeletonModel(myLib.mkIdentifier(name) );

		// get the component type
		ThreadType pt = (ThreadType) aadlColAPI.ComponentImpl_GetComponentType(t);

		//interface, inputs and outputs are the same as xx_thread
 
		asme2ssme_common.ASME2SSME_ComponentInterface(t_external, pt);
		//	//add clock for requires data access
		//	asme2ssme_common.ASME2SSME_ComponentInterface_DataAccess(t_external, pt);
		//	//add clock for requires subprogram access
		//	asme2ssme_common.ASME2SSME_ComponentInterface_SubprogramAccesses(t_external, pt);
		//	//add clock for calls subprogram
		//	asme2ssme_common.ASME2SSME_ComponentInterface_CallSubprograms(t_external, t);

		//external process
		DescriptionImpl  desc = (DescriptionImpl) t_external.getDescription();
		if (desc == null) {
			t_external.setDescription(new DescriptionImpl());
			desc = (DescriptionImpl) t_external.getDescription();
		}
		ExternalAttributeImpl externe = new ExternalAttributeImpl();
		desc.setBody(externe);
		return t_external;
	}

	/**
	 * It generates the dispatch protocol (XXX in {Aperiodic, Background, Hybrid, Periodic, Sporadic, Timed) 
	 * as Dispatch_Protocol_property{Supported_Dispatch_Protocols#XXX}(), Supported_Dispatch_Protocols is an enumerated Signal type predefined in a library.
	 * @param timpl
	 * @param t_property
	 */
	protected void genDispatchProtocol(ThreadImplementation timpl, ProcessModelImpl t_property)
	{
		Trace("\n ========== " + get_Thread_type() + ": genDispatchProtocol: " + timpl.getName());
		ArrayList<ExprParam> lparam = new  ArrayList<ExprParam>();
		ArrayList<ExprSignal> lin = new ArrayList<ExprSignal>();
		String dispatchp = aadlColAPI.Thread_Get_DispatchProtocol_str(timpl);
		ConstantEnumImpl dispatch_protocol = myLib.STree_mkEnumConstant(SUPPORTED_DISPATCH_PROTOCOLS ,dispatchp);
		lparam.add(dispatch_protocol);
		ProcessCallImpl pcall1 =  myLib.STree_mkCompleteProcessCall(myLib.mkIdentifier(DISPATCH_PROTOCOL_PROPERTY),lparam, lin);
		myLib.STree_addExpressionOfProcessToaModel(t_property, pcall1);
	}

	/**
	 * It generates in the Signal tree (t_property) the period value f a thread (timpl). It must be called for {Hybrid, Periodic, Sporadic, Timed} thread.
	 * as Period_property{Time_Units#tu, vperiod}() where tu denotes the time unit of the period, and vperiod denotes the value of the period.
	 * @param timpl 
	 * @param t_property
	 */
	protected void genPeriod(ThreadImplementation timpl, ProcessModelImpl t_property)
	{
		IntegerLiteral period = aadlColAPI.Thread_GetPeriod (timpl);
		if (period != null) 
		{
 
			ArrayList < ExprParam > lparam2 = new ArrayList < ExprParam > ();
			ArrayList < ExprSignal > lin2 = new ArrayList < ExprSignal > ();

			String str_period_unit = period.getUnit().getName ();
			ConstantEnumImpl period_unit = asme2ssme_common.ASME2SSME_Time_Unit_ConstantEnum (str_period_unit);
			lparam2.add (period_unit);

			String str_period_value = String.valueOf (period.getValue ());
			ConstantRealImpl period_value = new ConstantRealImpl ();
			period_value.setAtomString (str_period_value);
			lparam2.add (period_value);

			ProcessCallImpl pcall2 = myLib.STree_mkCompleteProcessCall (myLib.mkIdentifier (PERIOD_PROPERTY), lparam2, lin2);
			myLib.STree_addExpressionOfProcessToaModel (t_property, pcall2);
		}
	}

	protected void genAlarm(ThreadImplementation timpl, ProcessModelImpl t_property)
	{
		Trace ("\n ========== "+ get_Thread_type() + "thread : genAlarm: " + timpl.getName ());
		
		String vstr = get_Thread_type() +"Dispatch_constraint";
		IntegerLiteral period = aadlColAPI.Thread_GetPeriod(timpl);
		//(period != null)(assumed)
		ArrayList<ExprParam> lparam = new ArrayList<ExprParam>();
		ArrayList<ExprSignal> lin = new ArrayList<ExprSignal>();
		ArrayList<ExprSignal> lout = new ArrayList<ExprSignal>();
		String period_value = aadlColAPI.IntegerLiteral_Get_Value(period);
		ConstantIntegerImpl ci1 = myLib.STree_mkIntegerConstant(period_value);
		lparam.add(ci1);
		lin.add(myLib.mkIdentifier(TOP_ID));
		lin.add(myLib.STree_mkFieldAccess(CTL1_ID, DISPATCH_ID));
		lout.add(myLib.mkIdentifier(ALARM_ID));
		SignalsDefinitionImpl eq = myLib.STree_mkCompleteProcessCallEquation(myLib.mkIdentifier(vstr), lparam, lin, lout);
		myLib.STree_addExpressionOfProcessToaModel(t_property, eq);
	}

}
