 
package fr.inria.tea.aadl2ssme.transformation.popup.actions;


import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.pop.ssme.*;
import org.eclipse.pop.ssme.impl.*;
//import org.eclipse.pop.ssme.polychrony.CyclicDefinitionException;
import org.eclipse.pop.ssme.polychrony.SignalOperators;
import org.eclipse.emf.common.util.EList;


 /**
  * API to access/build SSME elements.
  */
public class signalTreeAPI {


	/**
	 * Class constructor
	 */
	public signalTreeAPI(){

	}


	/**
	 * Print out a string 
	 * @param str
	 */
	protected void Trace(String str)
	{
		//  System.out.println(str);
	}


	/**
	 * Returns a identifier from a string (str)
	 * 
	 * @param str a string
	 * @return a identifier
	 */
	public IdentifierImpl mkIdentifier(String str)
	{
		IdentifierImpl name = new IdentifierImpl();
		name.setAtomString(str);
		return name;
	}	


	/**
	 * Returns a string that represents an identifier
	 * 
	 * @param id an identifier
	 * @return a string
	 */
	public String Identifier_GetStr(Identifier id)
	{
		return id.getAtomString();
	}	


	/**
	 * Returns the identifier of a process p
	 * 
	 * @param p a process
	 * @return identifier
	 */
	public Identifier Process_GetID(ProcessModel p){
		return p.getIdentifier();
	}


	/**
	 * Returns whether str is a predefined type (in string)
	 * @param str
	 * @return
	 */
	public boolean Is_PredefinedType(String str){

		if(str == null) return false;
		return (str.equals("integer") | str.equals("event") | str.equals("boolean")
				| str.equals("short") | str.equals("long") | str.equals("real")
				| str.equals("dreal") | str.equals("string") | str.equals("float"));
	}	

	/**
	 * Returns a Signal predefined type is str is a key word of predefined type in Signal
	 * 
	 * @param str
	 * @return
	 */
	public PredefinedTypeImpl STree_mkPredefinedType(String str){

		PredefinedTypeImpl  vtype = new PredefinedTypeImpl();
		SignalIntrinsicType t;

		if(Is_PredefinedType(str)){
			if(str.equals("integer")){
				t = SignalIntrinsicType.INTEGER_LITERAL;
			}
			else if(str.equals("event")){
				t = SignalIntrinsicType.EVENT_LITERAL;
			}
			else if(str.equals("boolean")){
				t = SignalIntrinsicType.BOOLEAN_LITERAL;
			}
			else if(str.equals("short")){
				t = SignalIntrinsicType.SHORT_LITERAL;
			}
			else if(str.equals("long")){
				t = SignalIntrinsicType.LONG_LITERAL;
			}
			else if(str.equals("real")){
				t = SignalIntrinsicType.REAL_LITERAL;
			}
			else if(str.equals("dreal")){
				t = SignalIntrinsicType.DREAL_LITERAL;
			}
			else if(str.equals("string")){
				t = SignalIntrinsicType.STRING_LITERAL;
			}
			else if(str.equals("float")){
				t = SignalIntrinsicType.DREAL_LITERAL;
			}
			else{
				t = SignalIntrinsicType.UNDEFINED_LITERAL; 
			}	
			vtype.setType(t);
			return vtype;
		}
		return null;
	}


	/**
	 * Creates a predefined type that is the same as the signal s's type
	 * @param s
	 * @return
	 */
	public PredefinedTypeImpl STree_mkSamePredefinedType(SignalsDeclarationImpl s){
		if (s.getType() instanceof PredefinedTypeImpl){
			PredefinedTypeImpl vtype = new PredefinedTypeImpl();
			vtype.setType(((PredefinedTypeImpl)s.getType()).getType());
			return vtype;
		}
		return null;
	}


	/**
	 * Returns a predefined type based on the integer value
	 * 
	 * @param signalTypeOperator an integer
	 * @return a predefined type
	 */
	public Type mkPredefinedType(int signalTypeOperator) 
	{
		// opevent; oplogical  opshort opinteger oplong  opreal opdpreal  opcomplex opdcomplex opchar opstring
		SignalIntrinsicType t;
		PredefinedTypeImpl  vtype = new PredefinedTypeImpl();
		switch (signalTypeOperator)
		{
		case SignalOperators.opinteger:
			t = SignalIntrinsicType.INTEGER_LITERAL;
			break;
		case SignalOperators.opevent: 
			t = SignalIntrinsicType.EVENT_LITERAL; 
			break;
		case SignalOperators.oplogical: 
			t = SignalIntrinsicType.BOOLEAN_LITERAL; 
			break;
		case SignalOperators.opstring: 
			t = SignalIntrinsicType.STRING_LITERAL; 
			break;
		case SignalOperators.opdpreal: 
			t = SignalIntrinsicType.DREAL_LITERAL; 
			break;
			// TO DO for the other types.
		default:
			t = SignalIntrinsicType.UNDEFINED_LITERAL; 
		} 	
		vtype.setType(t);
		return vtype;
	}


	/**
	 * returns a type declaration: type type_id = value;
	 * 
	 * @param type_id identifier of type
	 * @param value
	 * @return
	 */
	public TypesDeclarationImpl STree_mkTypeDeclaration(Identifier type_id, TypeDescriptionValue value){

		TypesDeclarationImpl type_decl = new TypesDeclarationImpl();
		TypeDescriptionImpl type1 = new TypeDescriptionImpl();
		type1.setIdentifier(type_id);
		type1.setTypeDescriptionValue(value);
		type_decl.getTypeDefinition().add(type1);

		return type_decl;
	}

	/**
	 * Set identifier of a process (vprocess)'s
	 * 
	 * @param vprocess a process
	 * @param id an identifier
	 */
	public void STree_setNameModel(ProcessModelImpl vprocess, Identifier id) 
	{
		vprocess.setIdentifier(id);
	}


	/**
	 * Returns a skeleton of a process
	 * 
	 * @param id an identifier of this process
	 * @param op an integer 
	 * @return a process
	 */
	public ProcessModelImpl STree_mkSkeletonModel(Identifier id)
	{
		ProcessModelImpl vprocess = new ProcessModelImpl();
		//identifier
		vprocess.setIdentifier(id);
		//interfaceDefinition
		ProcessModelInterfaceImpl inter = STree_mkSkeletonInterface();
		vprocess.setInterfaceDefinition(inter);
		//description
		vprocess.setDescription(new DescriptionImpl());
		return vprocess;
	}


	/**
	/**
	 * Returns a skeleton of a process if op=opprocess, else null
	 * 
	 * @param id an identifier of this process
	 * @param op an integer 
	 * @return a process
	 */
	public ProcessModelImpl STree_mkSkeletonModel(Identifier id, int op)
	{
		if (op == SignalOperators.opprocess) return STree_mkSkeletonModel(id);
		// to be continued
		return null;
	}


	/**
	 * Returns a skeleton of listmodule
	 * 
	 * @return a ListModuleImpl object
	 */
	public ListModuleImpl STree_mkSkeletonModule()
	{
		ListModuleImpl lmodule = new ListModuleImpl();
		// to be continued
		return lmodule;
	}


	/**
	 * Add a module (amodule) to the root ListModule (lmodule)
	 * 
	 * @param lmodule a root ListModuleImpl object
	 * @param amodule a module
	 */
	public void STree_addModuleToRootModule(ListModuleImpl lmodule, ModuleImpl amodule) 
	{
		lmodule.getListModule().add(amodule);	
	}


	/**
	 * Returns a signal declaration (in the form of: vtype id init vinit)
	 * 
	 * @param id the name of this signal
	 * @param vtype the type
	 * @param vinit an init value
	 * @return a signal declaration
	 */
	public SignalsDeclarationImpl STree_mkSignalDeclaration(IdentifierImpl id, Type vtype, Object vinit) 
	{	
		SignalsDeclarationImpl vdecl = new SignalsDeclarationImpl();
		vdecl.setType(vtype);
		// id init vinit
		SignalNamingImpl x = new SignalNamingImpl();
		x.setIdentifier(id);
		if (vinit != null) x.setInitValue((ExprSignal) vinit);
		EList<SignalNaming> l = vdecl.getListSignal();
		l.add(x);
		return vdecl;
	}



	/**
	 * Returns a signal declaration (in the form of: vtype id init vinit)
	 * 
	 * @param id the name of this signal
	 * @param vtype the type
	 * @param vinit an init value
	 * @return a signal declaration
	 */
	public SharedVariablesDeclarationImpl STree_mkSharedVariableDeclaration(IdentifierImpl id, Type vtype, Object vinit) 
	{	
		SharedVariablesDeclarationImpl vdecl = new SharedVariablesDeclarationImpl();
		vdecl.setType(vtype);
		// id init vinit
		SignalNamingImpl x = new SignalNamingImpl();
		x.setIdentifier(id);
		if (vinit != null) x.setInitValue((ExprSignal) vinit);
		EList<SignalNaming> l = vdecl.getListSignal();
		l.add(x);
		return vdecl;
	}


	/**
	 * Add a signal to a process (vprocess)'s interface
	 * <p>
	 * i= 1 for parameters, 2 for inputs, 3 for outputs
	 * 
	 * @param vprocess a process
	 * @param vdecsig a signal or type declaration
	 * @param i an integer
	 */
	public void STree_addDeclarationToInterface(ProcessModelImpl vprocess, TypesOrSignalsDeclaration vdecsig, int i) 
	{	
		EList<TypesOrSignalsDeclaration> l;
		ProcessModelInterfaceImpl  inter = (ProcessModelInterfaceImpl) vprocess.getInterfaceDefinition();
		if (inter == null) {
			vprocess.setInterfaceDefinition( new ProcessModelInterfaceImpl());
			inter = (ProcessModelInterfaceImpl) vprocess.getInterfaceDefinition();
		}
		if (i==1)  
			inter.getListFormalParameter().add(vdecsig);
		else 
		{ 
			IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
			if (vioProfile == null) 
			{ 
				vioProfile = new IOProfileImpl();
				inter.setIoProfile(vioProfile);
			}
			if (i==2)  
			{ 
				l = vioProfile.getInputs();
				l.add(vdecsig);		  
			}
			else if (i==3)
			{ 
				l = vioProfile.getOutputs();
				l.add(vdecsig);
			}
		}		 
	}


	/**
	 * Remove a signal (named dec_name) from the interface of a process (vprocess)
	 * <p>
	 * i= 1 for parameters, 2 for inputs, 3 for outputs
	 * 
	 * @param vprocess a process
	 * @param dec_name the name of this signal
	 * @param i an integer
	 */
	public void STree_rmNamedDeclarationFromInterface(ProcessModelImpl vprocess, String dec_name, int i) 
	{	
		Trace("======== STree_rmNamedDeclarationFromInterface : " + dec_name);
		EList<TypesOrSignalsDeclaration> l;
		ProcessModelInterfaceImpl  inter = (ProcessModelInterfaceImpl) vprocess.getInterfaceDefinition();
		if (inter == null)
		{
			vprocess.setInterfaceDefinition( new ProcessModelInterfaceImpl());
			inter = (ProcessModelInterfaceImpl) vprocess.getInterfaceDefinition();
		}
		if (i==1)  /* parameter */
		{
			//inter.getListFormalParameter().remove(vdecsig);
			Trace("remove named declaration from interface has not been implmented yet !!!");
		}
		else 
		{ 
			IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
			if (vioProfile == null) 
			{ 
				//Trace("IOProfile null");
				vioProfile = new IOProfileImpl();
				inter.setIoProfile(vioProfile);
			}
			if (i==2)  /* input */
			{ 
				Trace("---------------removing input from interface");
				l = vioProfile.getInputs();

				Outer1: 
					for (int k = 0; k< l.size(); k++){
						SignalsDeclarationImpl in_sig = (SignalsDeclarationImpl) l.get(k);
						EList<SignalNaming> sig_naming_list = in_sig.getListSignal();
						for (int j =0; j< sig_naming_list.size(); j++){
							SignalNaming sig_naming = sig_naming_list.get(j);
							String name = sig_naming.getIdentifier().getAtomString();
							Trace(name);
							if(name.equals(dec_name)){
								if (sig_naming_list.size()==1){
									Trace("removing one signal declaration");
									l.remove(in_sig);
								}
								else{
									sig_naming_list.remove(sig_naming);
									Trace("removing one signal naming");
								}
								break Outer1;
							}
						}
					}	  
			}
			else if (i==3)  /* output */
			{ 
				Trace("----------------removing output from interface");
				l = vioProfile.getOutputs();
				Outer2: 
					for (int m = 0; m< l.size(); m++){
						SignalsDeclarationImpl out_sig = (SignalsDeclarationImpl) l.get(m);
						EList<SignalNaming> sig_naming_list = out_sig.getListSignal();
						for (int n =0; n< sig_naming_list.size(); n++){
							SignalNaming sig_naming = sig_naming_list.get(n);
							String name = sig_naming.getIdentifier().getAtomString();
							if(name.equals(dec_name)){
								if (sig_naming_list.size()==1){
									l.remove(out_sig);
								}
								else{
									sig_naming_list.remove(sig_naming);
								}
								break Outer2;
							}
						}
					}	  
			}
		}		 
	}


	/**
	 * Adds a signal (vdecsig) to an interface (inter)
	 * <p>
	 * i= 1 for parameters, 2 for inputs, 3 for outputs
	 * 
	 * @param inter an interface
	 * @param vdecsig a signal
	 * @param i an integer
	 */
	public void STree_addDeclarationToaInterface(ProcessModelInterfaceImpl inter, TypesOrSignalsDeclaration vdecsig, int i) 
	{	
		EList<TypesOrSignalsDeclaration> l;

		Trace("====== STree_addDeclarationToaInterface =========");
		if (i==1)  /* parameter */
		{
			inter.getListFormalParameter().add(vdecsig);
		}
		else 
		{ 
			IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
			if (vioProfile == null) 
			{ 
				vioProfile = new IOProfileImpl();
				inter.setIoProfile(vioProfile);
			}
			if (i==2)  /* input */
			{ 
				l = vioProfile.getInputs();
				l.add(vdecsig);		  
			}
			else if (i==3)  /* output */
			{ 
				l = vioProfile.getOutputs();
				l.add(vdecsig);
			}
		}		 
	}


	/**
	 * Add a declaration (vdecsig) to the interface of a ProcessType (process_type)
	 * 
	 * @param process_type a process type
	 * @param vdecsig a signal declaration
	 * @param i an integer (i=1 for parameter, i=2 for input, i=3 for output)
	 */
	public void STree_addDeclarationToProcessType(ProcessTypeImpl process_type, TypesOrSignalsDeclaration vdecsig, int i) 
	{		
		Trace("=== STree_addDeclarationToProcessType");

		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) process_type.getInterfaceDefinition();	
		if(inter == null){
			process_type.setInterfaceDefinition(new ProcessModelInterfaceImpl());
			inter = (ProcessModelInterfaceImpl) process_type.getInterfaceDefinition();
		}

		STree_addDeclarationToaInterface(inter, vdecsig, i);
	}


	/**
	 * Add a signal definition to a process (vprocess)
	 * @param vprocess
	 * @param vdef
	 */
	public void STree_addDefinition(ProcessModelImpl vprocess, SignalsDefinitionImpl vdef) 
	{
		Description vdescr = (DescriptionImpl) vprocess.getDescription();
		if (vdescr == null) 
		{
			//Trace("\n Description null ...");
			vdescr  = new DescriptionImpl();	
		}
		vprocess.setDescription(vdescr);

	}


	/**
	 * Adds a process declaration (vtreeproapp)  to a model (vtreeproc) as a local declaration
	 * 
	 * @param vtreeproc a process declaration where vtreeproapp is to be added into
	 * @param vtreeproapp a process 
	 */
	public void STree_addProcessDeclarationToProcess(ProcessModelImpl vtreeproc, ProcessModelImpl vtreeproapp)
	{
		DescriptionImpl  desc = (DescriptionImpl) vtreeproc.getDescription();

		if (desc == null)
		{
			//Trace("Description null");
			vtreeproc.setDescription(new DescriptionImpl());
			desc = (DescriptionImpl) vtreeproc.getDescription();
		}

		RestrictionProcessImpl body = (RestrictionProcessImpl) desc.getBody();

		if (body == null)
		{
			//Trace("body null");
			RestrictionProcessImpl rprocess = new RestrictionProcessImpl();
			desc.setBody(rprocess);
			body = (RestrictionProcessImpl) desc.getBody();
		}

		EList<Declaration> ldecl = body.getListDeclarations();
		ldecl.add(vtreeproapp);
	}


	/**
	 * Adds a process declaration (vtreeproapp)  to a module (vtreeproc)
	 * 
	 * @param vtreeproc a module where vtreeproapp is to be added into
	 * @param vtreeproapp a process
	 */
	public void STree_addProcessDeclarationToModule(ModuleImpl vtreeproc, ProcessModelImpl vtreeproapp)
	{
		ModuleDescriptionImpl  desc = (ModuleDescriptionImpl) vtreeproc.getModuleDescription();

		if (desc == null)
		{
			vtreeproc.setModuleDescription(new ModuleDescriptionImpl());
			desc = (ModuleDescriptionImpl) vtreeproc.getModuleDescription();
		}

		EList<Declaration> ldecl = desc.getListDeclaration();
		ldecl.add(vtreeproapp);
	}


	/**
	 * Adds a process declaration (vtreeproapp) to a process or module (vtreeproc)
	 * 
	 * @param vtreeproc a process or module that vtreeproapp is to be added into
	 * @param vtreeproapp a process
	 */
	public void STree_addProcessDeclaration(SignalElementImpl vtreeproc, ProcessModelImpl vtreeproapp)
	{
		if (vtreeproc instanceof ProcessModelImpl){
			ProcessModelImpl aprocess = (ProcessModelImpl) vtreeproc;
			STree_addProcessDeclarationToProcess(aprocess, vtreeproapp);
			//Trace("STree_addProcessDeclaration: instanceof ProcessModelImpl ---");
		}
		else if (vtreeproc instanceof ModuleImpl){
			ModuleImpl amodule = (ModuleImpl) vtreeproc;
			STree_addProcessDeclarationToModule(amodule, vtreeproapp);
		}
		else {
			Trace("STree_addProcessDeclaration: type unknown ---");
		} 
	}


	public void STree_addDeclarationToProcess(ProcessModelImpl vtreeproc, Declaration decl)
	{
		if (decl instanceof TypesDeclaration){
			TypesDeclaration type_decl = (TypesDeclaration) decl;
			STree_addTypeDeclarationToProcess(vtreeproc, type_decl);
			return;
		}
		else{
			DescriptionImpl  desc = (DescriptionImpl) vtreeproc.getDescription();  		
			if (desc == null)
			{
				vtreeproc.setDescription(new DescriptionImpl());
				desc = (DescriptionImpl) vtreeproc.getDescription();
			}

			RestrictionProcessImpl body = (RestrictionProcessImpl) desc.getBody();  	 
			if (body == null)
			{
				RestrictionProcessImpl rprocess = new RestrictionProcessImpl();
				desc.setBody(rprocess);
				body = (RestrictionProcessImpl) desc.getBody();
			}

			EList<Declaration> ldecl = body.getListDeclarations();
			STree_addDeclarationToList(ldecl, decl);
		}
	}


	/**
	 * Adds a type declaration (type_decl)  to a process model (vtreeproc) as a local declaration
	 * 
	 * @param vtreeproc a process model
	 * @param type_decl a type declaration
	 */
	public void STree_addTypeDeclarationToProcess(ProcessModelImpl vtreeproc, TypesDeclaration type_decl)
	{
		DescriptionImpl  desc = (DescriptionImpl) vtreeproc.getDescription();

		if (desc == null)
		{
			//Trace("Description null");
			vtreeproc.setDescription(new DescriptionImpl());
			desc = (DescriptionImpl) vtreeproc.getDescription();
		}

		RestrictionProcessImpl body = (RestrictionProcessImpl) desc.getBody();

		if (body == null)
		{
			//Trace("body null");
			RestrictionProcessImpl rprocess = new RestrictionProcessImpl();
			desc.setBody(rprocess);
			body = (RestrictionProcessImpl) desc.getBody();
		}

		EList<Declaration> ldecl = body.getListDeclarations();
		//ldecl.add(vtreeproapp);
		STree_addDeclarationToList(ldecl, type_decl);
	}


	/**
	 * Adds a an empty interface for a process (p_proc)
	 * 
	 * @param p_proc a process model
	 * @return this new process model
	 */
	public ProcessModelImpl STree_addEmptyInterface(ProcessModelImpl p_proc)
	{
		p_proc.setInterfaceDefinition( new ProcessModelInterfaceImpl());
		return p_proc;
	}


	/**
	 * Makes an empty body tree (empty list of pragma, empty body)
	 * \bsg <description> ::= <o_l_pragmas> <o_corps>. \esg
	 * 
	 * @return a body description object
	 */
	public DescriptionImpl STree_mkEmptyBody()
	{
		DescriptionImpl  desc = new DescriptionImpl();
		PragmaImpl p = new PragmaImpl();
		desc.getListPragma().add(p);
		desc.getBody();

		return desc;
	}


	/**
	 * Returns an empty confined process.
	 * \bsg <processus_confine> ::= <primaire_processus> <bloc_declarations> ;\esg
	 * 
	 * @return an empty restriction (confined) process
	 */
	public RestrictionProcessImpl STree_mkEmptyConfinedExpr()
	{
		RestrictionProcessImpl rprocess = new RestrictionProcessImpl();
		NullProcImpl exprp = new NullProcImpl();
		rprocess.setExprProcess(exprp);
		//Declaration d = new Declaration();
		//rprocess.getListDeclarations().add(d);
		return rprocess;
	}


	/**
	 * Makes a skeleton empty interface
	 * <p>
	 * The skeleton of an interface with: empty lists for parameters, inputs, outputs, 
	 * and empty tree for the specification. 
	 * \bsg <interface> ::= <o_suite_parametres_formels> <entrees_sorties> <graphe_externe>.\esg
	 * 
	 * @return an empty process interface
	 */
	public ProcessModelInterfaceImpl STree_mkSkeletonInterface()
	{
		ProcessModelInterfaceImpl inter = new ProcessModelInterfaceImpl();

		//parameter
		//	SignalsDeclarationImpl pp = new SignalsDeclarationImpl();
		//	inter.getListFormalParameter().add(pp);
		inter.getListFormalParameter();

		//IOPrifile
		IOProfileImpl vioProfile = new IOProfileImpl(); 
		inter.setIoProfile(vioProfile);

		//input
		//	SignalsDeclarationImpl ii = new SignalsDeclarationImpl();
		//	vioProfile.getInputs().add(ii);		
		vioProfile.getInputs();

		//output
		//	SignalsDeclarationImpl oo = new SignalsDeclarationImpl();
		//	vioProfile.getOutputs().add(oo);
		vioProfile.getOutputs();

		//ExternalGraph
		ExternalGraphImpl exg = new ExternalGraphImpl();
		inter.setExternalGraph(exg);

		return inter;
	}


	/**
	 * Makes an interface, which is an identifier, whose name is id_name
	 * 
	 * @param id_name a string name of this identifer
	 * @return an interface
	 */
	public InterfaceDefinition STree_mkIDInterface(String id_name)
	{
		IdentifierImpl id = mkIdentifier(id_name);
		InterfaceDefinition inter = (InterfaceDefinition) id;	 
		return inter;
	}


	/**
	 * Makes a model of a process with its name (p_nom), its interface (p_inte) and its description (expression + local declarations) (p_desc)
	 * 
	 * @param p_nom an identifier of this process's name
	 * @param p_inte an interface
	 * @param p_desc a body description
	 * @return a process model
	 */
	public ProcessModelImpl STree_mkProcess(Identifier p_nom, ProcessModelInterfaceImpl p_inte, Description p_desc)
	{
		ProcessModelImpl vprocess = STree_mkSkeletonModel(p_nom );
		vprocess.setInterfaceDefinition(p_inte);
		vprocess.setDescription(p_desc);

		return vprocess;
	}


	/** 
	 * Makes an interface with the list of parameters (param), inputs (ins), outputs (outs) and external graph (spec)
	 * <p>
	 * \bsg  <interface> ::= <o_suite_parametres_formels> <entrees_sorties> <graphe_externe>. \esg
	 * 
	 * @param param a list of parameters
	 * @param ins a list of input signals
	 * @param outs a list of output signals
	 * @param spec specification
	 * @return an interface
	 */
	public ProcessModelInterfaceImpl STree_mkInterface(ArrayList<TypesOrSignalsDeclaration> param, ArrayList<TypesOrSignalsDeclaration> ins, ArrayList<TypesOrSignalsDeclaration> outs, ExternalGraphImpl spec)
	{
		ProcessModelInterfaceImpl inter = new ProcessModelInterfaceImpl();

		//parameter
		Iterator<TypesOrSignalsDeclaration> it1 = param.iterator();
		while (it1.hasNext())
		{
			TypesOrSignalsDeclaration param1 = (TypesOrSignalsDeclaration) it1.next(); 
			inter.getListFormalParameter().add(param1);
		}

		//IOPrifile
		IOProfileImpl vioProfile = new IOProfileImpl(); 
		inter.setIoProfile(vioProfile);

		//input
		Iterator<TypesOrSignalsDeclaration> it2 = ins.iterator();
		while (it2.hasNext())
		{
			TypesOrSignalsDeclaration in1 = (TypesOrSignalsDeclaration) it2.next(); 
			vioProfile.getInputs().add(in1);	
		}

		//output
		Iterator<TypesOrSignalsDeclaration> it3 = outs.iterator();
		while (it3.hasNext())
		{
			TypesOrSignalsDeclaration out1 = (TypesOrSignalsDeclaration) it3.next(); 
			vioProfile.getOutputs().add(out1);	
		}

		//ExternalGraph
		inter.setExternalGraph(spec);

		return inter;
	}


	/** 
	 * Makes a process call of an identifier (id)
	 * 
	 * @param id an identifier
	 * @return a  process call
	 */
	public ProcessCallImpl STree_mkReferenceToModel(Identifier id)
	{
		ProcessCallImpl pc = new ProcessCallImpl();
		pc.setReferenceProcess(id);
		return pc;
		//return nmakebinary(opinst, idcopy(id), STree_mkNullaryTree(opnil));
	}


	/**
	 * Makes a process call of a string name
	 * <p>
	 * The calling process expression to name without parameters.
	 * 
	 * @param name a string name of the called process
	 * @return a process call
	 */
	public ProcessCallImpl STree_mkModelCall(String name)
	{
		ProcessCallImpl pc = new ProcessCallImpl();
		pc.setReferenceProcess(mkIdentifier(name));
		return pc; 
	}


	/**
	 * Makes a complete process call of an identifier (pname) with the effective parameters and inputs
	 * 
	 * @param pname an identifier 
	 * @param lparam a list of parameters
	 * @param lin a list of input signals
	 * @return a process call: pname{lparam}(lin)
	 */ 
	public ProcessCallImpl STree_mkCompleteProcessCall(IdentifierImpl pname, ArrayList<ExprParam> lparam, ArrayList<ExprSignal> lin)
	{

		ProcessExpansionImpl refproc = new ProcessExpansionImpl();
		refproc.setIdentifier(pname); 

		//parameters
		Iterator<ExprParam> it1 = lparam.iterator();
		while (it1.hasNext())
		{
			ExprParam param1 = it1.next();
			refproc.getListExprParam().add(param1); //parameters
		}

		ProcessCallImpl pcall = new ProcessCallImpl();	
		pcall.setReferenceProcess(refproc);
		//lin
		if(lin.size() > 0){
			ListExprSignalImpl listexprin = new ListExprSignalImpl();  
			Iterator<ExprSignal> it2 = lin.iterator();
			while (it2.hasNext())
			{
				ExprSignal in1 = it2.next(); 	
				listexprin.getExprSignal().add(in1);	
			}
			pcall.setListExprSignal(listexprin);
		}

		return pcall;
	}


	/**
	 * Makes a process call of identifier (pname) with a list of effective parameters
	 * 
	 * @param pname an identifier
	 * @param lparam a list of parameters
	 * @return a process call: pname{lparam}
	 */ 
	public ProcessExpansionImpl STree_mkProcessCallWithParameters(Identifier pname, ArrayList<ExprParam> lparam)
	{

		ProcessExpansionImpl refproc = new ProcessExpansionImpl();
		refproc.setIdentifier(pname);//id

		Iterator<ExprParam> it1 = lparam.iterator();
		while (it1.hasNext())
		{
			ExprParam param1 = it1.next();
			refproc.getListExprParam().add(param1); //parameters
		}

		//    ProcessCallImpl pcall = new ProcessCallImpl();	
		//	pcall.setReferenceProcess(refproc);
		//	
		//    return pcall;

		return refproc;
	}


	/**
	 * Makes a complete process call of a process model (process)
	 * <p>
	 * The names of the formal declaration (parameters, intputs, outputs) are taken as effective names for the call.
	 * 
	 * @param process a process model
	 * @return a process call
	 */ 
	public ProcessCallImpl STree_mkCompleteModelCall(ProcessModelImpl process)
	{
		ProcessCallImpl pcall = new ProcessCallImpl();	
		ProcessExpansionImpl refproc = new ProcessExpansionImpl();
		refproc.setIdentifier(process.getIdentifier());
		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) process.getInterfaceDefinition();
		if (inter == null){
			process.setInterfaceDefinition(new ProcessModelInterfaceImpl());
			inter = (ProcessModelInterfaceImpl) process.getInterfaceDefinition();
		}
		// EList<TypesOrSignalsDeclaration> l_parameters = inter.getListFormalParameter();
		// refproc.getListExprParam().addAll(l_parameters); //parameters
		pcall.setReferenceProcess(refproc);

		IOProfileImpl vioProfile = (IOProfileImpl) inter.getIoProfile();
		if (vioProfile == null) 
		{ 
			inter.setIoProfile(new IOProfileImpl());
			vioProfile = (IOProfileImpl) inter.getIoProfile();
		}
		// EList<TypesOrSignalsDeclaration> l_inputs = vioProfile.getInputs(); 
		//pcall.getListExprSignal().addAll(l_inputs); //inputs

		// EList<TypesOrSignalsDeclaration> l_outputs = vioProfile.getOutputs(); 
		// ExprSignal leftExp; //outputs

		//params = mkIdListFromDeclarationList(STree_getParameters(process), oplesign);
		//if (!nisniltree(params))  pcall = nmakebinary(opinst, pcall, params);
		//inputs = mkIdListFromDeclarationList(STree_getInputs(process), oplesign ); 
		//pcall = nmakebinary(opappe, pcall, inputs );
		//outputs = mkIdListFromDeclarationList(STree_getOutputs(process), opleflot);
		//if (!nisniltree(outputs)) pcall = nmakebinary(opdefsig, outputs, pcall);

		//ExprProcess eq = mkEquation(SignalOperators.opdefsig, leftExp, pcall);
		return pcall;
	}


	/**
	 * Makes an equation of a complete process call
	 * <p>
	 * A process instance of identifier (pname). 
	 * In the form of (lout) := pname{lparam}(lin). 
	 * The name of the called process, a list of parameters, inputs and outputs are given. 
	 * If one output, use ExprSignal, otherwise, use ListFlows instead
	 * 
	 * @param pname an identifier of the called process
	 * @param lparam a list of parameters
	 * @param lin a list of input signals
	 * @param lout a list of output signals
	 * @return an equation
	 */ 
	public SignalsDefinitionImpl STree_mkCompleteProcessCallEquation(IdentifierImpl pname, ArrayList<ExprParam> lparam, ArrayList<ExprSignal> lin, ArrayList<ExprSignal> lout)
	{

		ProcessExpansionImpl refproc = new ProcessExpansionImpl();
		refproc.setIdentifier(pname);//id

		Iterator<ExprParam> it1 = lparam.iterator();
		while (it1.hasNext())
		{
			ExprParam param1 = it1.next();
			refproc.getListExprParam().add(param1); //parameters
		}

		ProcessCallImpl pcall = new ProcessCallImpl();	
		pcall.setReferenceProcess(refproc);

		ListExprSignalImpl listexprin = new ListExprSignalImpl(); //lin
		Iterator<ExprSignal> it2 = lin.iterator();
		while (it2.hasNext())
		{
			ExprSignal in1 = it2.next(); 	
			listexprin.getExprSignal().add(in1);	
		}
		if (listexprin.getExprSignal().size()>0){
			pcall.setListExprSignal(listexprin);
		}

		SignalsDefinitionImpl pcalleq = new SignalsDefinitionImpl();


		if (lout.size() == 1){ //if only one output
			ExprSignal expr_out = lout.get(0);
			pcalleq = (SignalsDefinitionImpl) mkEquation( expr_out, pcall);
		}
		else { // more than one output
			ListFlowsImpl expr_out = new ListFlowsImpl(); //lout
			Iterator<ExprSignal> it3 = lout.iterator();
			while (it3.hasNext())
			{
				ExprSignal out1 = it3.next(); 
				expr_out.getExprSignal().add(out1);
			}
			pcalleq = (SignalsDefinitionImpl) mkEquation(expr_out, pcall);
		}

		return pcalleq;
	}


	/**
	 * Adds (tail) a process expression (vexprpro) to a process model (vtreeproc).
	 * 
	 * @param vtreeproc a process model
	 * @param vexprpro an expression
	 */
	public void STree_addExpressionOfProcessToaModel(ProcessModelImpl vtreeproc, ExprProcess vexprpro)
	{

		DescriptionImpl desc = (DescriptionImpl) vtreeproc.getDescription();

		if (desc == null)
		{ vtreeproc.setDescription(new DescriptionImpl()); 
		desc = (DescriptionImpl) vtreeproc.getDescription();
		}

		RestrictionProcessImpl body = (RestrictionProcessImpl) desc.getBody();

		if (body == null)
		{
			RestrictionProcessImpl rp = new RestrictionProcessImpl();
			desc.setBody(rp);
			body = (RestrictionProcessImpl) desc.getBody();
		}

		CompositionImpl comp = (CompositionImpl) body.getExprProcess();
		if (comp == null)
		{
			CompositionImpl cc = new CompositionImpl();
			body.setExprProcess(cc);
			comp = (CompositionImpl) body.getExprProcess();
		}

		EList<ExprProcess> list_expr = comp.getListExprProcess();
		list_expr.add(vexprpro);

		//	DescriptionImpl desc = (DescriptionImpl) vtreeproc.getDescription();
		//	
		//	if (desc == null)
		//	{ vtreeproc.setDescription(new DescriptionImpl()); 
		//	  desc = (DescriptionImpl) vtreeproc.getDescription();
		//	}
		//	
		//	RestrictionProcessImpl body = (RestrictionProcessImpl) desc.getBody();
		//	 
		//	if (body == null)
		//		{
		//			RestrictionProcessImpl rp = new RestrictionProcessImpl();
		//			desc.setBody(rp);
		//			body = (RestrictionProcessImpl) desc.getBody();
		//		}
		//	 
		//	body.setExprProcess(vexprpro);
	}


	/**
	 * Adds (head) a process expression (vexprpro) to a process model (vtreeproc).
	 * 
	 * @param vtreeproc a process model
	 * @param vexprpro an expression
	 */
	public void STree_addPreExpressionOfProcessToaModel(ProcessModelImpl vtreeproc, ExprProcess vexprpro)
	{

		DescriptionImpl desc = (DescriptionImpl) vtreeproc.getDescription();

		if (desc == null)
		{ vtreeproc.setDescription(new DescriptionImpl()); 
		desc = (DescriptionImpl) vtreeproc.getDescription();
		}

		RestrictionProcessImpl body = (RestrictionProcessImpl) desc.getBody();

		if (body == null)
		{
			RestrictionProcessImpl rp = new RestrictionProcessImpl();
			desc.setBody(rp);
			body = (RestrictionProcessImpl) desc.getBody();
		}

		CompositionImpl comp = (CompositionImpl) body.getExprProcess();
		if (comp == null)
		{
			CompositionImpl cc = new CompositionImpl();
			body.setExprProcess(cc);
			comp = (CompositionImpl) body.getExprProcess();
		}

		EList<ExprProcess> list_expr = comp.getListExprProcess();
		list_expr.add(0,vexprpro);
	}


	/**
	 * Adds an element (vexpapp) at the tail of a composition list (vlfeed).
	 * @param vlfeed a compoeition object, created when empty.
	 * @param vexpapp (in) element to add (assumed: process expression).
	 */
	public void STree_addExprToComposition_Post(CompositionImpl vlfeed, ExprProcess vexpapp)
	{

		if (vexpapp == null) return;
		if (vlfeed == null){ 

		}
		vlfeed.getListExprProcess().add(vexpapp);
	}


	/**
	 * Adds an element (vexpapp) at the head of a composition list (vlfeed).
	 * @param vlfeed (in/out) feed list, created when empty.
	 * @param vexpapp (in) element to add (assumed : process expression).
	 */
	public void STree_addExprToComposition_Pre(CompositionImpl vlfeed, ExprProcess vexpapp)
	{
		if (vexpapp == null) return;

		vlfeed.getListExprProcess().add(0,vexpapp);
	}


	/** 
	 * Makes an abtract tree representing a Signal confined process
	 * <p>
	 * \bsg (| expr |) where <decl> end;  \esg
	 * 
	 * @param expr an expression
	 * @param decl a list of declaration
	 * @return a restriction process
	 */
	public RestrictionProcessImpl STree_mkConfinedExpression(ExprProcess expr, EList<Declaration> decl)
	{
		RestrictionProcessImpl rp = new RestrictionProcessImpl();
		rp.setExprProcess(expr);
		EList<Declaration> list_decl = rp.getListDeclarations();
		list_decl.addAll(decl);

		return rp;
	}


	/**
	 * Sets a description (desc) to an assumed model (process).
	 * 
	 * @param process a process model
	 * @param desc a body description
	 */
	public void STree_setDescription(ProcessModelImpl process, DescriptionImpl  desc)
	{
		process.setDescription(desc);
	}


	/*
It adds (tail) a process expression (vexprpro) to the Composition expression of a process model (vtreeproc).
	 */
	//public void STree_addExprToaModel(ProcessModelImpl vtreeproc, ExprProcess vexprpro)
	//{
	//	DescriptionImpl desc = (DescriptionImpl) vtree(EList<PragmaImpl>)proc.getDescription();
	//	
	//	if (desc == null)
	//	{ vtreeproc.setDescription(new DescriptionImpl()); 
	//	  desc = (DescriptionImpl) vtreeproc.getDescription();
	//	}
	//	
	//	RestrictionProcessImpl body = (RestrictionProcessImpl) desc.getBody();
	//	 
	//	if (body == null)
	//		{
	//			RestrictionProcessImpl rp = new RestrictionProcessImpl();
	//			desc.setBody(rp);
	//			body = (RestrictionProcessImpl) desc.getBody();
	//		}
	//	 
	//	CompositionImpl comp = (CompositionImpl) body.getExprProcess();
	//	if (comp == null)
	//	{
	//		CompositionImpl cc = new CompositionImpl();
	//		body.setExprProcess(cc);
	//		comp = (CompositionImpl) body.getExprProcess();
	//	}
	//	
	//	EList<ExprProcess> list_expr = comp.getListExprProcess();
	//	list_expr.add(vexprpro);
	//}


	/**
	 * Inserts a list of declarations (l_decl) to the local declarations of a model (p).
	 * 
	 * @param p a process model
	 * @param l_decl a list of declarations
	 */
	public void STree_addDeclarations(ProcessModelImpl p, ArrayList<Declaration> l_decl)
	{
		DescriptionImpl desc = (DescriptionImpl) p.getDescription();

		if (desc == null)
		{ p.setDescription(new DescriptionImpl()); 
		desc = (DescriptionImpl) p.getDescription();
		}

		RestrictionProcessImpl body = (RestrictionProcessImpl) desc.getBody();

		if (body == null)
		{
			RestrictionProcessImpl rp = new RestrictionProcessImpl();
			desc.setBody(rp);
			body = (RestrictionProcessImpl) desc.getBody();
		}

		EList<Declaration> list_decl = body.getListDeclarations();

		Iterator<?> it = l_decl.iterator();
		while (it.hasNext())
		{
			Declaration decl = (Declaration) it.next();
			STree_addDeclarationToList(list_decl, decl);
		}
	}


	/**
	 * Inserts a declaration (decl) to the local declarations of a model (p).
	 * 
	 * @param p a process model
	 * @param decl a declaration
	 */
	public void STree_addaDeclarationToProcess(ProcessModelImpl p, Declaration decl)
	{
		DescriptionImpl desc = (DescriptionImpl) p.getDescription();

		if (desc == null)
		{ p.setDescription(new DescriptionImpl()); 
		desc = (DescriptionImpl) p.getDescription();
		}

		RestrictionProcessImpl body = (RestrictionProcessImpl) desc.getBody();

		if (body == null)
		{
			RestrictionProcessImpl rp = new RestrictionProcessImpl();
			desc.setBody(rp);
			body = (RestrictionProcessImpl) desc.getBody();
		}

		EList<Declaration> list_decl = body.getListDeclarations();
		STree_addDeclarationToList(list_decl, decl);

	}


	/**
	 * Adds a declaration (vdecl) to a list of declarations (ldecl).
	 * <p>
	 * For typed identifier, it searches the existency of such type for adding. 
	 * Otherwise, it adds at the head of the list.
	 * 
	 * @param ldecl a list of declarations
	 * @param vdecl a declaration
	 * @return the new list of declarations.
	 */
	public EList<Declaration> STree_addDeclarationToList(EList<Declaration> ldecl, Declaration vdecl)
	{
		if (ldecl.size()>0){//if ldecl list is not empty

			if (vdecl instanceof TypesDeclarationImpl){ // type x;
				if (ldecl.size() == 1){
					if (ldecl.get(0) instanceof TypesDeclarationImpl){
						ldecl.add(vdecl);
					}
					else if (ldecl.get(0) instanceof ModulesImportImpl){
						ldecl.add(vdecl);
					}
					else if (ldecl.get(0) instanceof ProcessModelImpl){
						ldecl.add(0,vdecl);
					}
					else{
						ldecl.add(0,vdecl);
					}
				}
				else {
					boolean found1 = false;
					for (int i = 1; i< ldecl.size(); i++){
						if (ldecl.get(i-1) instanceof TypesDeclarationImpl){
							if( !(ldecl.get(i) instanceof TypesDeclarationImpl)){
								ldecl.add(i,vdecl);
								found1 = true;
								break;
							}	
							else if(i == (ldecl.size()-1)){
								ldecl.add(vdecl);
								found1 = true;
								break;
							}
						}
					}
					//		    	for(int i = ldecl.size()-1; i > 1; i--){
						//		    		if (ldecl.get(i) instanceof TypesDeclarationImpl){
					//		    			ldecl.add(i+1, vdecl);
					//		    			found1 = true;
					//		    			break;
					//		    		}
					//		    	}
					if (found1 == false){

						if(ldecl.get(ldecl.size()-1) instanceof TypesDeclarationImpl){
							ldecl.add(vdecl);
						}
						else{
							ldecl.add(0,vdecl);
						}		    		
					}
				}	    	
			}

			else if (vdecl instanceof SignalsDeclarationImpl){ // typeA x;

				EList<SignalNaming> list_sig_naming = ((SignalsDeclarationImpl) vdecl).getListSignal();
				boolean found = false;
				for (int j = 0; j< ldecl.size(); j++){
					if (ldecl.get(j) instanceof SignalsDeclarationImpl){		    		
						SignalsDeclarationImpl sig_decl = (SignalsDeclarationImpl) ldecl.get(j);
						if (sig_decl.getType() == ((SignalsDeclarationImpl) vdecl).getType() ){
							sig_decl.getListSignal().addAll(list_sig_naming);
							found = true;
							break;
						}

					}	
				}
				if (found == false){
					ldecl.add(0, vdecl);
				}
			}
			else{// other declaration
				ldecl.add(0, vdecl);
			}
		}
		else{//if ldecl list is empty, add vdecl as the first one
			ldecl.add(0, vdecl);
		}
		return ldecl;
	}


	/**
	 * Adds a declaration (vdecl) to the end of the list of declarations (ldecl).
	 * 
	 * @param ldecl a list of declarations
	 * @param vdecl a declaration
	 * @return the new list of declarations.
	 */
	public EList<Declaration> STree_addEndDeclarationToList(EList<Declaration> ldecl, Declaration vdecl)
	{
		ldecl.add(vdecl);
		return ldecl;
	}


	/**
	 * Adds a list of declarations(vdecl) to an assumed confined expression (p_conf).
	 * 
	 * @param p_conf a restriction process 
	 * @param vdecl a list of declarations
	 */
	public void STree_addDeclarationToConfinedExpr(RestrictionProcess p_conf, ArrayList<Declaration> vdecl)
	{
		EList<Declaration> list_decl = p_conf.getListDeclarations();

		Iterator<?> it = vdecl.iterator();
		while (it.hasNext())
		{
			Declaration decl = (Declaration) it.next();
			STree_addDeclarationToList(list_decl, decl);
		}
	}


	/**
	 * Declares, in process p, the list of identifiers (lids) as labels.
	 * <p>
	 * It assumes that p is the Signal abstract tree representation of a process.
	 * 
	 * @param lids a list of identifiers
	 * @param p a process
	 */
	public void STree_addLabelsInModel(ArrayList<Identifier> lids, ProcessModelImpl p)
	{

		if (lids != null)
		{
			DescriptionImpl descr = (DescriptionImpl) p.getDescription();
			RestrictionProcessImpl rp = (RestrictionProcessImpl) descr.getBody();
			EList<Declaration> list_decl = rp.getListDeclarations();

			LabelsImpl ll = new LabelsImpl();
			EList<Identifier> e_id = ll.getListIdentifier();

			Iterator<?> it = lids.iterator();
			while (it.hasNext())
			{
				Identifier decl = (Identifier) it.next();	   
				e_id.add(decl);
			}
			STree_addDeclarationToList(list_decl, ll);
		}
	}


	/** 
	 * Adds a pragma (apragma) to a process (aProcess). 
	 *
	 * @param aProcess a process model
	 * @param apragma a pragma
	 */
	public void STree_addPragma(ProcessModelImpl aProcess, PragmaImpl apragma)
	{
		DescriptionImpl descr = (DescriptionImpl) aProcess.getDescription();
		descr.getListPragma().add(apragma);
	}



	/**
	 * Returns whether a process contains output signals
	 * 
	 * @param p a process model
	 * @return
	 */
	public boolean STree_IsProcessHasOutput(ProcessModelImpl p){

		if(p.getInterfaceDefinition() != null){
			ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) p.getInterfaceDefinition();

			if(inter.getIoProfile() != null){
				IOProfile iop = inter.getIoProfile();

				if(iop.getOutputs() != null){
					EList<TypesOrSignalsDeclaration> list = iop.getOutputs();
					if(list.size()>0){
						return true;
					}
				}
			}
		}

		return false;
	}


	/**
	 * Returns whether a process body is null or not
	 * @param p a process model
	 * @return
	 */
	public boolean STree_IsProcessNull(ProcessModelImpl p){

		DescriptionImpl desc = (DescriptionImpl) p.getDescription();
		if (desc == null){
			return true;
		}
		if (desc.getBody() == null){
			return true;
		}

		return false;
	}


	/**
	 * Adds a null proc to a process model (p).
	 * 
	 * @param p a process model
	 * @return 
	 */
	public void STree_addNullProc(ProcessModelImpl p)
	{
		DescriptionImpl desc = (DescriptionImpl) p.getDescription();

		if (desc == null)
		{ 
			p.setDescription(new DescriptionImpl()); 
			desc = (DescriptionImpl) p.getDescription();
		}

		if (desc.getBody() == null)
		{
			NullProcImpl np = new NullProcImpl();	 	
			desc.setBody(np);
		}
	}


	/*
It adds a specification object (vtreeSpecif) to a process model (process).
@param process (in) : process model (assumed)
@param vtreeSpecif (in) : specification expression(assumed).
	 */
	//public void STree_addSpecification(ntrees process, ntrees vtreeSpecif)
	//{
	//ntrees vinterface, vSpec, vt;
	//
	//if (nisniltree(vtreeSpecif)) return;    
	//vinterface = nchild(2, process);
	//vSpec = nchild(3, vinterface); /* external graph */
	//if (vSpec == NILTREE) 
	//    {
	//	vt = NILTREE;
	//	STree_addExprToComposition_Post(&vt, vtreeSpecif); 
	//	vSpec = nmakebinary(opspec, STree_mkAtomNilTree(), vt);
	//	nchangchild( vinterface, vSpec, 3);
	//    }
	//else
	//    {
	//	vt =  nchild(2, vSpec);
	//	STree_addExprToComposition_Post(&vt, vtreeSpecif); 
	//    }
	//}


	/**
	 * Makes a constant declaration  with p_typ as type and p_def a list of definitions of constants.
	 * <p>
	 * \bsg <decl_constantes> ::= "constant" <type> <l_def_constantes>. \esg
	 * 
	 * @param p_def a list of constant definitions
	 * @param p_typ a type
	 * @return a constan declaration
	 */
	public ConstantsDeclarationImpl STree_mkConstantDeclaration(EList<ConstantDefinition> p_def, Type p_typ)
	{
		ConstantsDeclarationImpl c_decl = new ConstantsDeclarationImpl();
		c_decl.setType(p_typ);
		c_decl.getListConstantDefinition().addAll(p_def);
		return c_decl; 
	}


	/**
	 * Makes an abstract tree denoting an equation.
	 * 
	 * @param op (in) Signal operator ( :=, ::= code).
	 * @param leftExp (in) left operand of the equation.
	 * @param rightExp (in) right operand of the equation.
	 * @return an equation (:=) or a partial equation (::=)
	 */
	public ExprProcess  mkEquation(int op, ExprSignal leftExp, ExprSignal rightExp)
	{
		if (op == SignalOperators.opdefsig)
		{
			SignalsDefinitionImpl sig_def = new SignalsDefinitionImpl();
			sig_def.setExprSignalLeft(leftExp);
			sig_def.setExprSignalRight(rightExp);
			return sig_def;
		}
		else if (op == SignalOperators.opdefpart)
		{
			SignalsPartialDefinitionImpl egal_def = new SignalsPartialDefinitionImpl();
			egal_def.setExprSignalLeft(leftExp);
			egal_def.setExprSignalRight(rightExp);
			return egal_def;
		}
		return null;
	}


	/**
	 * Makes an abstract tree denoting an default expression: exp1 default exp2.
	 * 
	 * @param exp1
	 * @param exp2
	 * @return a default expression
	 */
	public DefaultImpl STree_mkDefault(ExprSignal exp1, ExprSignal exp2)
	{
		DefaultImpl def1 = new DefaultImpl();
		def1.setExprSignal1(exp1);
		def1.setExprSignal2(exp2);

		return def1;
	}



	/**
	 * sigs --> list
	 * 
	 * @param sigs
	 * @param list
	 * @return
	 */
	public UnconditionedDependences STree_mkDependency(Signals sigs, ArrayList<Signals> list)
	{
		UnconditionedDependencesImpl dep = new UnconditionedDependencesImpl();
		dep.setSignals(sigs);

		for(int i =0; i<list.size(); i++){
			Signals s1 = list.get(i);
			dep.getDependenceElementsList().add(s1);
		}

		return dep;
	}


	/**
	 * Makes an abstract tree denoting an UnaryWhenImpl: when exp
	 * 
	 * @param exp an expression
	 * @return a when expression 
	 */
	public UnaryWhenImpl STree_mkUnaryWhen(ExprSignal exp)
	{
		UnaryWhenImpl unary_when = new UnaryWhenImpl();
		unary_when.setExprSignal(exp);
		return unary_when;
	}


	/**
	 * Makes a when expression: exp1 when exp2
	 * 
	 * @param exp1
	 * @param exp2
	 * @return
	 */
	public WhenImpl STree_mkWhen(ExprSignal exp1, ExprSignal exp2){

		WhenImpl whenx = new WhenImpl();
		whenx.setExprSignal1(exp1);
		whenx.setExprSignal2(exp2);

		return whenx;
	}


	/**
	 * Makes an abstract tree denoting an EqualImpl: exp1 = exp2
	 * 
	 * @param exp1 left expression
	 * @param exp2 right expression
	 * @return an equal expression 
	 */
	public EqualImpl STree_mkEqual(ExprSignal exp1, ExprSignal exp2)
	{
		EqualImpl equal = new EqualImpl();
		equal.setExprSignal1(exp1);
		equal.setExprSignal2(exp2);
		return equal;
	}


	/**
	 * Makes an abstract tree denoting an FromImpl: exp1 from exp2
	 * 
	 * @param exp1 
	 * @param exp2
	 * @return a from expression 
	 */
	public FromImpl STree_mkFrom(ExprSignal exp1, ExprSignal exp2)
	{
		FromImpl from = new FromImpl();
		from.setExprSignal1(exp1);
		from.setExprSignal2(exp2);
		return from;
	}


	/**
	 * Makes an abstract tree denoting an ClockUnionImpl: exp1 ^+ exp2
	 * 
	 * @param exp1 
	 * @param exp2
	 * @return a union expression 
	 */
	public ClockUnionImpl STree_mkClockUnion(ExprSignal exp1, ExprSignal exp2)
	{
		ClockUnionImpl union1 = new ClockUnionImpl();
		union1.setExprSignal1(exp1);
		union1.setExprSignal2(exp2);
		return union1;
	}


	/**
	 * Makes an abstract tree denoting an ClockEqualityImpl: exp1 ^= exp2
	 * 
	 * @param exp1 
	 * @param exp2
	 * @return a clock equality expression 
	 */
	public ClockEqualityImpl STree_mkClockEquality(ExprSignal exp1, ExprSignal exp2)
	{
		ClockEqualityImpl ceq = new ClockEqualityImpl();
		ceq.getExprSignal().add(exp1);
		ceq.getExprSignal().add(exp2);
		return ceq;
	}


	/**
	 * Makes an abstract tree denoting an ClockEqualityImpl: exp1 ^= exp2 ^= ...
	 * 
	 * @param exp1 
	 * @param exp2
	 * @return a clock equality expression 
	 */
	public ClockEqualityImpl STree_mkClockEqualitys(ArrayList<ExprSignal> exps)
	{
		ClockEqualityImpl ceq = new ClockEqualityImpl();
		for(int i=0; i< exps.size(); i++){
			ExprSignal exp1 = exps.get(i);
			ceq.getExprSignal().add(exp1);
		}
		return ceq;
	}


	/**
	 * Makes an abstract tree denoting a ClockImpl (the clock of the expression): ^ exp
	 * 
	 * @param exp an expression
	 * @return a clock expression
	 */
	public ClockImpl STree_mkClock(ExprSignal exp)
	{
		ClockImpl exp_clock = new ClockImpl();
		exp_clock.setExprSignal(exp);
		return exp_clock;
	}


	/**
	 * Adds a module ID (mid) to the end of a ModulesImportImpl (usemodule): use ..., mid;
	 * 
	 * @param usemodule a ModulesImportImpl
	 * @param mid a module identifier
	 */
	public void STree_addModuleIDToAModulesImport(ModulesImportImpl usemodule, Identifier mid){
		EList <Identifier> list_module = usemodule.getListModule();
		Trace("STree_addModuleIDToAModulesImport");
		list_module.add(mid);
		for (int i = 0; i< list_module.size(); i++ ) {
			Identifier id = list_module.get(i);
			Trace(id.getAtomString());
		}
		return;
	}


	/**
	 * Adds a module ID (mid) to a ModuleImpl (amodule)
	 * <p>
	 * Adds mid to the last of the ModulesImport list:
	 * use xx, ...;
	 * use yy, ..., mid;
	 * 
	 * @param amodule a module
	 * @param mid an identifier of module to be used
	 */
	public void STree_adduseModuleToAModule(ModuleImpl amodule, Identifier mid){
		ModuleDescriptionImpl module_descr = (ModuleDescriptionImpl) amodule.getModuleDescription();
		if (module_descr == null){
			module_descr = new ModuleDescriptionImpl();
			amodule.setModuleDescription(module_descr);
		}

		EList<Declaration> list_declaration = module_descr.getListDeclaration();
		if (list_declaration.size() == 0){
			Trace("STree_adduseModuleToAModule : list_declaration.size() == 0");
			ModulesImportImpl usemodule1 = new ModulesImportImpl();
			STree_addModuleIDToAModulesImport(usemodule1, mid);
			list_declaration.add(0,usemodule1);
			return;
		}
		else{
			for(int i = list_declaration.size()-1; i>=0; i--){
				if (list_declaration.get(i) instanceof ModulesImportImpl){
					Trace("STree_adduseModuleToAModule : find the last ModulesImportImpl declaration");
					ModulesImportImpl usemodule2 = (ModulesImportImpl) list_declaration.get(i);
					STree_addModuleIDToAModulesImport(usemodule2, mid);
					//list_declaration.add(0,usemodule2);
					return;
				}
			}
			//if no ModulesImportImpl declared in the list, then add one
			Trace("STree_adduseModuleToAModule : no ModulesImportImpl declared");
			ModulesImportImpl usemodule3 = new ModulesImportImpl();
			STree_addModuleIDToAModulesImport(usemodule3, mid);
			list_declaration.add(0, usemodule3);
			return;
		}

	}


	/**
	 * Adds a use module ID (mid) to a ProcessModelImpl (aprocess) in position pre if bpre is true, else adds it to the last of the ModulesImport list
	 * <p>
	 * Adds mid to the last of the ModulesImport list:
	 * use xx, ...;
	 * use yy, ..., mid;
	 * 
	 * @param aprocess a process
	 * @param mid an identifier of module to be used
	 */
	public void STree_adduseModuleToAProcess(ProcessModelImpl aprocess, Identifier mid, boolean bpre){
		DescriptionImpl descr = (DescriptionImpl) aprocess.getDescription();
		if (descr == null){
			descr = new DescriptionImpl();
			aprocess.setDescription(descr);
		}

		RestrictionProcessImpl body = (RestrictionProcessImpl) descr.getBody();
		EList<Declaration> list_declaration = body.getListDeclarations();
		if (list_declaration.size() == 0){
			Trace("STree_adduseModuleToAProcess : list_declaration.size() == 0");
			ModulesImportImpl usemodule1 = new ModulesImportImpl();
			STree_addModuleIDToAModulesImport(usemodule1, mid);
			list_declaration.add(0,usemodule1);
			return;
		}
		else{

			if (bpre)
			{
				for(int i = 0; i<list_declaration.size()-1; i++){
					if (list_declaration.get(i) instanceof ModulesImportImpl){
						Trace("STree_adduseModuleToAProcess : find the first ModulesImportImpl declaration");
						ModulesImportImpl usemodule4 = new ModulesImportImpl();
						STree_addModuleIDToAModulesImport(usemodule4, mid);
						list_declaration.add(i,usemodule4);
						//list_declaration.add(0,usemodule2);
						return;
					}
				}
			}
			else{
				for(int i = list_declaration.size()-1; i>=0; i--){
					if (list_declaration.get(i) instanceof ModulesImportImpl){
						Trace("STree_adduseModuleToAProcess : find the last ModulesImportImpl declaration");
						ModulesImportImpl usemodule2 = (ModulesImportImpl) list_declaration.get(i);
						STree_addModuleIDToAModulesImport(usemodule2, mid);
						//list_declaration.add(0,usemodule2);
						return;
					}
				}
			}

			//if no ModulesImportImpl declared in the list, then add one
			Trace("STree_adduseModuleToAProcess : no ModulesImportImpl declared");
			ModulesImportImpl usemodule3 = new ModulesImportImpl();
			STree_addModuleIDToAModulesImport(usemodule3, mid);
			list_declaration.add(0, usemodule3);
			return;
		}

	}


	/**
	 * Makes an empty tree local declarations of a model (p_proc).
	 * 
	 * @param p_proc a process model
	 * @return a list of declarations 
	 */
	public EList<Declaration> STree_mkLocalDeclaration(ProcessModelImpl p_proc)
	{
		DescriptionImpl  desc = (DescriptionImpl) p_proc.getDescription();	 
		RestrictionProcessImpl body = (RestrictionProcessImpl) desc.getBody();
		return body.getListDeclarations();

	}


	/**
	 * Makes an empty module with an identifier (name).
	 * 
	 * @param name an identifier
	 * @return a module
	 */
	public ModuleImpl STree_mkModule(Identifier name)
	{
		ModuleImpl m = new ModuleImpl();
		m.setIdentifier(name);
		return m;
	}


	/** 
	 * Make a pragma with id as \<identificateur\>, obj as \<o_l_objets_pragma\> and stmt as \<enonce_pragma\>
	 * <p>
	 * \bsg  <pragma> ::=<identificateur> <o_l_objets_pragma> <enonce_pragma>\esg
	 * 
	 * @param id an identifier
	 * @param obj a list of identifier
	 * @param stmt
	 * @return a new pragma
	 */
	public PragmaImpl STree_mkPragmaIdent(Identifier id, EList<Identifier> obj, StringValue stmt)
	{
		PragmaImpl pragma = new PragmaImpl();
		pragma.setIdentifier(id);
		pragma.getListPragmaObject().addAll(obj);
		pragma.setPragmaStatement(stmt);
		return pragma;
	}


	/** 
	 * Make a pragma with kw as the name of \<identificateur\>, obj as \<o_l_objets_pragma\> and stmt as \<enonce_pragma\>.
	 * <p>
	 * \bsg  <pragma> ::=<identificateur> <o_l_objets_pragma> <enonce_pragma>\esg
	 * 
	 * @param kw a string name of the identifier
	 * @param obj a list of identifiers
	 * @param stmt
	 * @return a new pragma 
	 */
	public PragmaImpl STree_mkPragma(String kw, EList<Identifier> obj, StringValue stmt)
	{
		Identifier id = mkIdentifier(kw);
		return STree_mkPragmaIdent(id, obj,stmt);
	}


	/**
	 * Adds an empty pragmas area for a model (process).
	 * 
	 * @param process a process model
	 * @return the created tree (empty list of pragmas).
	 */
	public EList<Pragma> STree_mkPragmasList(ProcessModelImpl process)
	{
		DescriptionImpl  desc = (DescriptionImpl) process.getDescription();	 
		EList<Pragma> list_pragma = desc.getListPragma();

		return list_pragma;
	}


	/*
@return the Signal skeleton tree of an external process.
@param nameprocess (in) the name of the model.
@param vlang (in) tree denoting language implementation of the external process.
@param ope (in) a Signal operator assumed to be in {opprocess, opprocessp, opfunc, opfuncp, opact}
@see {STree_mkSkeleton}
	 */
	//public ntrees STree_mkSkeletonExternProcess(ptident nameprocess, ntrees vlang, noperators ope)
	//{
	//ntrees vtree2, vtree3;
	//
	//vtree2 = nmaketernary(opinte, NILTREE, nmakebinary(opentsor, NILTREE, NILTREE), NILTREE);
	//if (nisniltree(vlang)) vlang = STree_mkEmptyTree(); else  vlang= STree_copy(vlang, false);
	//vtree3 = nmakebinary(opdesc, NILTREE, vlang);
	////  if (aProcess) ope = opprocess; else ope = opfunc;
	//return  nmaketernary(ope, identtotree(nameprocess), vtree2, vtree3);
	//}


	/**
	 * Sets the "external graph" of a model (proc) by eg.
	 * 
	 * @param proc a process model
	 * @param eg
	 */
	public void STree_setExternalGraph(ProcessModelImpl proc, ExternalGraphImpl eg)
	{
		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) proc.getInterfaceDefinition();
		inter.setExternalGraph(eg);
	} 


	/**
	 * Sets (replaces?) local declarations (ndecls) to  a model (p).
	 * 
	 * @param p a process model
	 * @param ndecls a new list of local declarations
	 * @return the previous local declarations of p.
	 */
	public ArrayList<Declaration> STree_setLocalDeclarations(ProcessModelImpl p, EList<Declaration> ndecls)
	{
		DescriptionImpl descr = (DescriptionImpl) p.getDescription();
		RestrictionProcessImpl body = (RestrictionProcessImpl) descr.getBody();
		EList<Declaration> ld = body.getListDeclarations();
		ArrayList<Declaration> ald = new ArrayList<Declaration> ();

		Iterator<?> it = ld.iterator();
		while (it.hasNext())
		{
			Declaration s = (Declaration) it.next();
			ald.add(s);
		}

		ld.clear();
		ld.addAll(ndecls);

		return ald;
	}


	/**
	 * Replaces the parameters of a model (proc) by param.
	 * 
	 * @param proc a process model
	 * @param param a list of parameters
	 */
	public void STree_setParameters(ProcessModelImpl proc, EList<TypesOrSignalsDeclaration> param)
	{
		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) proc.getInterfaceDefinition();
		EList<TypesOrSignalsDeclaration> lparam = inter.getListFormalParameter();

		lparam.clear();
		lparam.addAll(param);
	}


	/**
	 * Assigns (replaces) the pragmas (pragmas) to a model (proc).
	 * 
	 * @param proc a process model
	 * @param pragmas a list of pragmas
	 */
	public void STree_setPragmas(ProcessModelImpl proc, EList<Pragma> pragmas)
	{
		DescriptionImpl descr = (DescriptionImpl) proc.getDescription();
		EList<Pragma> lpragma = descr.getListPragma();

		lpragma.clear();
		lpragma.addAll(pragmas);
	}


	/**
	 * Adds an empty "local declaration (where)" area in second child of the p_dec tree (Description).
	 * 
	 * @param p_desc a body description
	 * @return the created restriction process tree.
	 */
	public RestrictionProcessImpl STree_addLocalBlock(DescriptionImpl p_desc)
	{
		RestrictionProcessImpl p = new RestrictionProcessImpl();
		p_desc.setBody(p);
		RestrictionProcessImpl p_conf = (RestrictionProcessImpl) p_desc.getBody();
		return p_conf;
	}


	/**
	 * Adds the declaration (vdecl) as local to a model or a module (aProcess) or a confined process.
	 * 
	 * @param aProcess a process or a module
	 * @param vdecl a list of local declarations
	 */
	public void STree_addLocalDeclaration(SignalElementImpl aProcess, ArrayList<Declaration> vdecl)
	{
		if (aProcess instanceof RestrictionProcessImpl){
			RestrictionProcessImpl rp = (RestrictionProcessImpl) aProcess;
			STree_addDeclarationToConfinedExpr(rp, vdecl);
		}

		else if (aProcess instanceof ProcessModelImpl){
			ProcessModelImpl aprocess = (ProcessModelImpl) aProcess;
			STree_addDeclarations(aprocess, vdecl);
		}

		else if (aProcess instanceof ModuleImpl){
			ModuleImpl amodule = (ModuleImpl) aProcess;
			ModuleDescriptionImpl mdescr = (ModuleDescriptionImpl) amodule.getModuleDescription();
			EList<Declaration> list_decl = mdescr.getListDeclaration();

			Iterator<?> it = vdecl.iterator();
			while (it.hasNext())
			{
				Declaration decl = (Declaration) it.next();
				STree_addDeclarationToList(list_decl, decl);
			}
		}
		else {
			Trace("add local declaration to unknown signal element!!");
		}
	}


	/**
	 * Adds a specification object (vtreeSpecif) to a process model (process).
	 * 
	 * @param process (in) : process model (assumed)
	 * @param vtreeSpecif (in) : specification expression(assumed).
	 */
	public void STree_addSpecification(ProcessModelImpl process, ExprProcess vtreeSpecif)
	{
		if (vtreeSpecif == null) return;

		ProcessModelInterfaceImpl inter = (ProcessModelInterfaceImpl) process.getInterfaceDefinition();
		if (inter == null){
			process.setInterfaceDefinition(new ProcessModelInterfaceImpl());
			inter = (ProcessModelInterfaceImpl) process.getInterfaceDefinition();
		}

		ExternalGraphImpl eg = (ExternalGraphImpl) inter.getExternalGraph();
		if (eg == null){
			inter.setExternalGraph(new ExternalGraphImpl());
			eg = (ExternalGraphImpl) inter.getExternalGraph();
		}

		GraphSpecificationImpl gs = (GraphSpecificationImpl) eg.getGraphSpecification();
		if (gs == null){
			eg.setGraphSpecification(new GraphSpecificationImpl());
			gs = (GraphSpecificationImpl) eg.getGraphSpecification();
		}

		CompositionImpl comp = (CompositionImpl) gs.getExprProcess();
		if (comp == null){
			gs.setExprProcess(new CompositionImpl());
			comp = (CompositionImpl) gs.getExprProcess();
		}

		STree_addExprToComposition_Post(comp, vtreeSpecif);
	}


	/**
	 * Makes a list of synchronizations initialized with e (i.e. e ^= ).
	 * <p>
	 * It must be completed to have a syntactic correct tree.
	 * 
	 * @param e an expression that this synchronization expresstion is initialized with
	 * @return a list of expressions
	 */
	public ArrayList<ExprSignal> STree_mkSynchroIni(ExprSignal e)
	{
		ArrayList<ExprSignal> alist = new ArrayList<ExprSignal>();
		alist.add(e);
		return alist;
	}


	/**
	 * Adds an expression (e) to a list of synchronizations (l).
	 * 
	 * @param l a list of expression that they are synchronized
	 * @param e a new expression to be synchronized with the list
	 * @return the new list.
	 */
	@SuppressWarnings("null")
	public EList<ExprSignal> STree_addSynchro(EList<ExprSignal> l, ExprSignal e)
	{
		if (l == null) {
			l.add(e);
			return l;
		}
		l.add(e);
		return l;

	}


	/** 
	 * Adds a type declaration to a list of type definitions
	 * 
	 * @param vtypes a list of type definitions
	 * @param vtype a type declaration
	 * @return the list of type declaration vtypes + {vtype}.
	 */
	public EList<TypeDefinition> STree_addTypeToTypes(EList<TypeDefinition> vtypes, TypeDefinition vtype)
	{
		// if (vtypes ==null){ vtypes.add(vtype);  return vtypes; } else
		{
			vtypes.add(vtype);
			return vtypes;
		}
	}


	/**
	 * Removes the parameters tree of a model (p_proc).
	 * 
	 * @param p_proc a process model
	 */
	public void STree_deleteParameters(ProcessModelImpl p_proc)
	{
		ProcessModelInterfaceImpl inter  = (ProcessModelInterfaceImpl) p_proc.getInterfaceDefinition();
		if (inter == null) return;
		EList<TypesOrSignalsDeclaration> lparams = inter.getListFormalParameter();
		lparams.clear();
	}


	/**
	 * Makes a type declaration with name as \<identificateur\>, description as \<description_type\> and vinit as \<expr_signal\>.
	 * <p>
	 * \bsg  <def_type> ::= <identificateur> "=" <description_type> "init" <expr_signal>. \esg
	 * 
	 * @param name an identifier
	 * @param description type
	 * @param vinit an expression that represents the initial value
	 * @return a type declaration 
	 */
	public TypeDescriptionImpl STree_mkDclType(Identifier name, TypeDescriptionValue description, ExprSignal vinit)
	{ 
		TypeDescriptionImpl typedescr = new TypeDescriptionImpl();
		typedescr.setIdentifier(name);
		if (description != null) {
			typedescr.setTypeDescriptionValue(description);
		}
		if (vinit != null) {
			typedescr.setInitValue(vinit);
		}
		return typedescr;
	}


	/**
	 * Inserts a type declaration (adecl) to a module (m).
	 * 
	 * @param p a module
	 * @param adecl a type declaration
	 */
	public void STree_addaTypeDeclarationToModule(ModuleImpl p, TypesDeclarationImpl adecl)
	{
		ModuleDescriptionImpl module_descr = (ModuleDescriptionImpl) p.getModuleDescription();

		if (module_descr == null){
			p.setModuleDescription(new ModuleDescriptionImpl());
			module_descr = (ModuleDescriptionImpl) p.getModuleDescription();
		}

		//module_descr.getListDeclaration().add(0,adecl);
		STree_addDeclarationToList(module_descr.getListDeclaration(), adecl);
		return;
	}



	/**
	 * Inserts a type declaration (adecl) to a module (m).
	 * 
	 * @param p a module
	 * @param adecl a type declaration
	 */
	public void STree_addaConstantToModule(ModuleImpl p, ConstantsDeclarationImpl adecl)
	{
		ModuleDescriptionImpl module_descr = (ModuleDescriptionImpl) p.getModuleDescription();

		if (module_descr == null){
			p.setModuleDescription(new ModuleDescriptionImpl());
			module_descr = (ModuleDescriptionImpl) p.getModuleDescription();
		}

		module_descr.getListDeclaration().add(adecl);

		return;
	}


	/**
	 * Makes a labeled process call of a process (pname)
	 * 
	 * @param lable a string label
	 * @param pname a string name of the called process
	 * @return a labelled process
	 */
	public LabelledProcessImpl STree_mkLabelledModelCall(String label, String pname)
	{
		LabelledProcessImpl lprocess = new LabelledProcessImpl();
		lprocess.setLabel(mkIdentifier(label));	
		ProcessCallImpl process_call = STree_mkModelCall(pname);
		lprocess.setExprProcess(process_call);
		return lprocess; 
	}



	/**
	 * Makes a labeled process call of a process
	 * 
	 * @param lable a string label
	 * @param exp a process call expression
	 * @return a labelled process
	 */
	public LabelledProcessImpl STree_mkLabelledProcessCall(String label, ExprProcess exp)
	{
		LabelledProcessImpl lprocess = new LabelledProcessImpl();
		lprocess.setLabel(mkIdentifier(label));	
		lprocess.setExprProcess(exp);
		return lprocess; 
	}


	/**
	 * Gets the string name of an identifier (e)
	 * 
	 * @param e an identifier
	 * @return a string name
	 */
	public String STree_GetStringName(Identifier e)
	{
		return e.getAtomString();
	}


	/**
	 * Gets a process declaration whose name is aname, that is defined in the vprocess
	 * 
	 * @param vprocess a process model where the aname process is declared
	 * @param aname a string name of the required process
	 * @return a process model
	 */
	public ProcessModelImpl STree_GetProcessDeclarationFromProcess(ProcessModelImpl vprocess, String aname){
		Trace("====== start STree_GetProcessDeclarationFromProcess");

		ProcessModelImpl pprocess = new ProcessModelImpl();

		DescriptionImpl descr = (DescriptionImpl) vprocess.getDescription();
		RestrictionProcessImpl body = (RestrictionProcessImpl) descr.getBody();
		EList<Declaration> l_declaration = body.getListDeclarations();

		for (int i = 0; i < l_declaration.size(); i++) {  
			if (l_declaration.get(i) instanceof ProcessModelImpl) {
				ProcessModelImpl aprocess = (ProcessModelImpl) l_declaration.get(i);
				Identifier id = aprocess.getIdentifier();
				String sub_process_name = id.getAtomString();
				if (sub_process_name.equals(aname)) return aprocess;
			}
		}
		return pprocess;
	}


	/**
	 * Gets a ProcessTypeImpl (process_type) whose name is aname, defined in the ProcessModel (vprocess).
	 * 
	 * @param vprocess a process model where this required process type is declared
	 * @param aname a string name of this required process type
	 * @return a process type declaration
	 */
	public ProcessTypeImpl STree_GetProcessTypeDeclarationFromAProcess(ProcessModelImpl vprocess, String aname){
		Trace("====== start STree_GetProcessTypeDeclarationFromAProcess");
		Trace("== get ProcessTypeDeclaration : " + aname + " from ProcessModel : " + vprocess.getIdentifier().getAtomString());

		ProcessTypeImpl process_type = new ProcessTypeImpl();

		DescriptionImpl descr = (DescriptionImpl) vprocess.getDescription();
		RestrictionProcessImpl body = (RestrictionProcessImpl) descr.getBody();
		EList<Declaration> l_declaration = body.getListDeclarations();

		for (int i = 0; i < l_declaration.size(); i++) {  
			if (l_declaration.get(i) instanceof TypesDeclarationImpl) {
				TypesDeclarationImpl type_decl = (TypesDeclarationImpl) l_declaration.get(i);
				EList<TypeDefinition> list_type_def = type_decl.getTypeDefinition();
				for (int j=0; j<list_type_def.size(); j++){
					if (list_type_def.get(j) instanceof ProcessTypeImpl){
						ProcessTypeImpl a_process_type = (ProcessTypeImpl) list_type_def.get(j) ;
						Identifier id = a_process_type.getIdentifier();
						String process_type_name = id.getAtomString();
						Trace("process type name = " + process_type_name);
						if (process_type_name.equals(aname)){
							process_type = a_process_type;
							return process_type;
						}
					}

				}
			}
		}
		return process_type;
	}


	public ProcessTypeImpl STree_GetProcessTypeDeclaration(SignalElementImpl vtreeproc, String aname)
	{
		ProcessTypeImpl process_type = new ProcessTypeImpl();

		if (vtreeproc instanceof ProcessModelImpl){
			ProcessModelImpl aprocess = (ProcessModelImpl) vtreeproc;
			return STree_GetProcessTypeDeclarationFromAProcess(aprocess, aname);
		}
		if (vtreeproc instanceof ModuleImpl){
			ModuleImpl amodule = (ModuleImpl) vtreeproc;
			return STree_GetProcessTypeDeclarationFromAModule(amodule, aname);
		}
		Trace("STree_addProcessDeclaration: type unknown ---");
		return process_type;
	}


	public ProcessTypeImpl STree_GetProcessTypeDeclarationFromAModule(ModuleImpl vmodule, String aname){
		Trace("====== start STree_GetProcessTypeDeclarationFromAModule");
		Trace("== get ProcessTypeDeclaration : " + aname + " from ProcessModel : " + vmodule.getIdentifier().getAtomString());

		ProcessTypeImpl process_type = new ProcessTypeImpl();

		ModuleDescription descr = (ModuleDescription) vmodule.getModuleDescription();

		if(descr!= null){
			EList<Declaration> decl_list = descr.getListDeclaration();

			for (int i = 0; i < decl_list.size(); i++) {  
				if (decl_list.get(i) instanceof TypesDeclarationImpl) {
					TypesDeclarationImpl type_decl = (TypesDeclarationImpl) decl_list.get(i);

					EList<TypeDefinition> list_type_def = type_decl.getTypeDefinition();
					for (int j=0; j<list_type_def.size(); j++){
						if (list_type_def.get(j) instanceof ProcessTypeImpl){
							ProcessTypeImpl a_process_type = (ProcessTypeImpl) list_type_def.get(j) ;
							Identifier id = a_process_type.getIdentifier();
							String process_type_name = id.getAtomString();
							Trace("process type name = " + process_type_name);
							if (process_type_name.equals(aname)) return a_process_type;
						}			
					}
				}
			}
		}

		return process_type;
	}


	public TypeDescriptionImpl STree_GetTypeDescriptionFromAModule(ModuleImpl vmodule, String aname){

		Trace("\n == signalTreeAPI: STree_GetTypeDescriptionFromAModule " + aname + " from " + vmodule.getIdentifier().getAtomString());

		ModuleDescription descr = (ModuleDescription) vmodule.getModuleDescription();

		if(descr!= null){
			EList<Declaration> decl_list = descr.getListDeclaration();

			for (int i = 0; i < decl_list.size(); i++) {  
				if (decl_list.get(i) instanceof TypesDeclarationImpl) {
					TypesDeclarationImpl type1 = (TypesDeclarationImpl) decl_list.get(i);				
					EList<TypeDefinition> list_type_def = type1.getTypeDefinition();
					for (int j=0; j<list_type_def.size(); j++){
						if (list_type_def.get(j) instanceof TypeDescriptionImpl){
							TypeDescriptionImpl type_desc = (TypeDescriptionImpl) list_type_def.get(j) ;
							Identifier id = type_desc.getIdentifier();
							String type_desc_name = id.getAtomString();
							if (type_desc_name.equals(aname)) return type_desc;
						}				
					}
				}
			}
		}
		return null;
	}


	/**
	 * Makess a constant enumeration (constant_enum): EnumType_name#constantvalue
	 * 
	 * @param EnumType_name a string name of the enumeration type
	 * @param constantvalue a string of the constant value
	 * @return a constant enumeration
	 */
	public ConstantEnumImpl STree_mkEnumConstant(String EnumType_name, String constantvalue){
		ConstantEnumImpl constant_enum = new ConstantEnumImpl();
		constant_enum.setTypeIdentifier(mkIdentifier(EnumType_name));
		constant_enum.setConstantEnumValue(mkIdentifier(constantvalue));
		return constant_enum;
	}


	/**
	 * Makes a constant string (constant_str) with string value of constantvalue
	 * 
	 * @param constantvalue a string of the constant string value
	 * @return a constant string
	 */
	public ConstantStringImpl STree_mkStringConstant(String constantvalue){
		ConstantStringImpl constant_str = new ConstantStringImpl();
		constant_str.setAtomString(constantvalue);
		return constant_str;
	}


	/**
	 * Makes a StringValue with string value of constantvalue
	 * 
	 * @param constantvalue a string of the constant string value
	 * 
	 * @return a StringValueImpl which represents a constant string signal 
	 */
	public StringValueImpl STree_mkStringValue(String constantvalue){

		StringValueImpl  v1 = new StringValueImpl();
		ConstantStringImpl cst_str1 = STree_mkStringConstant(constantvalue);
		v1.getListConstantString().add(cst_str1);
		return v1;
	}


	/**
	 * Makess a constant real (cr) with (string) value of constantvalue
	 * 
	 * @param constantvalue a string of the constant real value, for example, "2.0"
	 * 
	 * @return a constant real
	 */
	public ConstantRealImpl STree_mkRealConstant(String constantvalue){
		ConstantRealImpl cr = new ConstantRealImpl();
		cr.setAtomString(constantvalue);		
		return cr;
	}


	/**
	 * Makess a constant double real (cr) with (string) value of constantvalue
	 * 
	 * @param constantvalue a string of the constant real value, for example, "2.0"
	 * 
	 * @return a constant double real
	 */
	public ConstantDoubleRealImpl STree_mkDoubleRealConstant(String constantvalue){
		ConstantDoubleRealImpl cr = new ConstantDoubleRealImpl();
		cr.setAtomString(constantvalue);		
		return cr;
	}



	/**
	 * Makess a constant Integer (ci) with (string) value of constantvalue
	 * 
	 * @param constantvalue a string of the constant integer value, for example, "2"
	 * 
	 * @return a constant integer
	 */
	public ConstantIntegerImpl STree_mkIntegerConstant(String constantvalue){
		ConstantIntegerImpl ci = new ConstantIntegerImpl();
		ci.setAtomString(constantvalue);	
		return ci;
	}


	/**
	 * Makes a field access (field_access): expr_name.id_name
	 * 
	 * @param expr_name a string name of the expression type
	 * @param id_name a string name of the field
	 * @return a field access
	 */
	public FieldAccessImpl STree_mkFieldAccess(String expr_name, String id_name){
		FieldAccessImpl field_access = new FieldAccessImpl();
		field_access.setExprSignal(mkIdentifier(expr_name));
		field_access.setIdentifier(mkIdentifier(id_name));
		return field_access;
	}


	/**
	 * Adds comments (str_list) to a Signal element (sig_element) in prefix position if bpre, postfix position otherwise
	 * 
	 * @param sig_element
	 * @param str_list a list of string comment
	 * @param bpre 
	 */
	public void STree_addComment(SignalElementImpl sig_element, ArrayList<String> str_list, boolean bpre){
		for (int i = 0; i < str_list.size(); i++) {  
			String str = (String) str_list.get(i);
			CommentImpl comment1 = new CommentImpl();
			comment1.setAtomString(str);
			if (bpre) sig_element.getCommentPre().add(comment1); else sig_element.getCommentPost().add(comment1);
		}
	}

	/**
	 * Add a comment (vcomment) to a Signal tree (vt), in prefix position if b, else in postfix position.
	 */
	public void STree_addComment(SignalElementImpl vt, String vcomment, boolean b) 
	{ 
		ArrayList<String> comments = new ArrayList<String>();			 
		comments.add(vcomment);
		STree_addComment(vt, comments, b);		
	}

	/**
	 * @return the Signal declaration of a signal named id of type vtype.
	 */
	public SignalsDeclarationImpl mkDecl(String id, Type vtype) {
		return STree_mkSignalDeclaration(mkIdentifier(id), vtype, null);
	}
	/**
	 * @return the Signal declaration of an event signal named id .
	 */
	public SignalsDeclarationImpl mkEventDecl(String id) {
		return STree_mkSignalDeclaration(mkIdentifier(id), mkPredefinedType(SignalOperators.opevent), null);
	}
	/**
	 * @return the Signal declaration of an integer signal named id .
	 */
	public SignalsDeclarationImpl mkIntegerDecl(String id) {
		return STree_mkSignalDeclaration(mkIdentifier(id), mkPredefinedType(SignalOperators.opinteger), null);
	}
	
	/**
	 * @return the Signal declaration of a string signal named id.
	 */
	public SignalsDeclarationImpl mkStringDecl(String id) {
		return STree_mkSignalDeclaration(mkIdentifier(id),  mkPredefinedType(SignalOperators.opstring), null);
	}
	/**
	 * @return the Signal declaration of a signal named id with typeId as type.
	 */
	public SignalsDeclarationImpl mkDecl(String id, String typeId ) {
		return STree_mkSignalDeclaration(mkIdentifier(id), mkIdentifier(typeId), null);
	}
	/**
	 * @return the Signal declaration of a boolean signal named id  .
	 */
	public SignalsDeclarationImpl mkBooleanDecl(String name )
	{
		return STree_mkSignalDeclaration(mkIdentifier(name), mkPredefinedType(SignalOperators.oplogical), null);
	}
	
	/**
	 * @return the Signal equation left_expr := right_expr.
	 */
	public SignalsDefinitionImpl  mkEquation( ExprSignal left_expr, ExprSignal right_expr) {
		return  (SignalsDefinitionImpl) mkEquation(SignalOperators.opdefsig, left_expr, right_expr);
	}
	
	/**
	 * @return the Signal partial equation left_expr ::= right_expr.
	 */
	public SignalsPartialDefinitionImpl  mkPartialEquation( ExprSignal left_expr, ExprSignal right_expr) {
		return  (SignalsPartialDefinitionImpl)  mkEquation(SignalOperators.opdefpart, left_expr, right_expr);
	}
}
