 
package fr.inria.tea.aadl2ssme.transformation.popup.actions;


import java.util.ArrayList;
import java.util.Iterator;

import org.osate.aadl2.*;
import org.eclipse.emf.common.util.EList;
import org.osate.aadl2.AadlPackage;
import org.osate.aadl2.Element;
import org.osate.aadl2.impl.AbstractTypeImpl;



public class AADLCoLAPI
{
	// private final static AADLCoLAPI	INSTANCE		= new AADLCoLAPI();

	/**
	 * Class constructor
	 */
	public AADLCoLAPI() {
	}


	/**
	 * Prints the string str in the console
	 *  
	 * @param s a string to be printed out   
	 */
	public void Trace(String s) {
		// java.lang.System.out.println(s);
	}


	/**
	 * Returns the string name of a aadl package (ap)
	 * 
	 * @param ap a aadl package
	 * 
	 * @return string name
	 */
	public String AadlPackage_GetName(AadlPackage ap){
		return ap.getName();
	}


	/**
	 * Returns the public package section of a aadl package (ap)
	 * 
	 * @param ap a aadl package
	 * 
	 * @return public package
	 */
	public PublicPackageSection AadlPackage_GetPublicPkg(AadlPackage ap){
		return ap.getPublicSection();
	}


	/**
	 * Returns the private package section of aadl a package (ap)
	 * 
	 * @param ap a aadl package
	 * 
	 * @return private package
	 */
	public PrivatePackageSection AadlPackage_GetPrivatePkg(AadlPackage ap){
		return ap.getPrivateSection();
	}


	/**
	 * Get a list of all DataType objects of AadlPackage (ap)
	 * 
	 * @param ap
	 * @return an ArrayList of DataType objects
	 */
	public ArrayList<DataType> AadlPackage_GetdataType(AadlPackage ap){

		ArrayList<DataType> alist = new ArrayList<DataType>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<DataType> list1 = Package_GetdataType(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<DataType> list2 = Package_GetdataType(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}
		}

		return alist;
	}


	/**
	 * Get a list of all DataImplementation objects of AadlPackage (ap)
	 * 
	 * @param ap
	 * @return an ArrayList of DataImplementation objects
	 */
	public ArrayList<DataImplementation> AadlPackage_GetdataImpl(AadlPackage ap){

		ArrayList<DataImplementation> alist = new ArrayList<DataImplementation>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<DataImplementation> list1 = Package_GetdataImpl(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<DataImplementation> list2 = Package_GetdataImpl(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}	
		}

		return alist;
	}


	/**
	 * Get a list of all SystemType objects of AadlPackage (ap)
	 * 
	 * @param ap 
	 * @return an ArrayList of SystemType objects
	 */
	public ArrayList<SystemType> AadlPackage_GetsystemType(AadlPackage ap){

		ArrayList<SystemType> alist = new ArrayList<SystemType>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<SystemType> list1 = Package_GetsystemType(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<SystemType> list2 = Package_GetsystemType(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}
		}

		return alist;
	}


	/**
	 * get list of all SystemImplementation objects of a AadlPackage (ap)
	 * 
	 * @param ap an AADL AadlPackage object
	 * @return An ArrayList of SystemImplementation objects
	 */
	public ArrayList<SystemImplementation> AadlPackage_GetsystemImpl(AadlPackage ap){

		ArrayList<SystemImplementation> alist = new ArrayList<SystemImplementation>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<SystemImplementation> list1 = Package_GetsystemImpl(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<SystemImplementation> list2 = Package_GetsystemImpl(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}	
		}

		return alist;
	}


	/**
	 * Get a list of all ProcessType objects of AadlPackage (ap)
	 * 
	 * @param ap 
	 * @return an ArrayList of ProcessType objects
	 */
	public ArrayList<ProcessType> AadlPackage_GetProcessType(AadlPackage ap){
		ArrayList<ProcessType> alist = new ArrayList<ProcessType>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<ProcessType> list1 = Package_GetProcessType(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<ProcessType> list2 = Package_GetProcessType(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}
		}


		return alist;
	}


	/**
	 * Get list of all ProcessImplementation objects of a AadlPackage (aspec)
	 * 
	 * @param aspec an AADL AadlSpec object
	 * @return An ArrayList of ProcessImplementation objects
	 */
	public ArrayList<ProcessImplementation> AadlPackage_GetProcessImpl(AadlPackage ap){

		ArrayList<ProcessImplementation> alist = new ArrayList<ProcessImplementation>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<ProcessImplementation> list1 = Package_GetProcessImpl(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<ProcessImplementation> list2 = Package_GetProcessImpl(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}	
		}

		return alist;
	}


	/**
	 * Get a list of all ThreadType objects of AadlPackage (ap)
	 * 
	 * @param ap 
	 * @return an ArrayList of ThreadType objects
	 */
	public ArrayList<ThreadType> AadlPackage_GetThreadType(AadlPackage ap){
		ArrayList<ThreadType> alist = new ArrayList<ThreadType>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<ThreadType> list1 = Package_GetThreadType(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<ThreadType> list2 = Package_GetThreadType(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}
		}

		return alist;
	}


	/**
	 * Get list of all ThreadImplementation objects of a AadlPackage (aspec)
	 * 
	 * @param aspec an AADL AadlSpec object
	 * @return An ArrayList of ThreadImplementation objects
	 */
	public ArrayList<ThreadImplementation> AadlPackage_GetThreadImpl(AadlPackage ap){

		ArrayList<ThreadImplementation> alist = new ArrayList<ThreadImplementation>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<ThreadImplementation> list1 = Package_GetThreadImpl(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<ThreadImplementation> list2 = Package_GetThreadImpl(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}	
		}

		return alist;
	}


	/**
	 * Get a list of all ProcessorType objects of AadlPackage (ap)
	 * 
	 * @param ap 
	 * @return an ArrayList of ProcessorType objects
	 */
	public ArrayList<ProcessorType> AadlPackage_GetProcessorType(AadlPackage ap){
		ArrayList<ProcessorType> alist = new ArrayList<ProcessorType>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<ProcessorType> list1 = Package_GetProcessorType(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<ProcessorType> list2 = Package_GetProcessorType(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}
		}

		return alist;
	}


	/**
	 * Get list of all ProcessorImplementation objects of a AadlPackage (aspec)
	 * 
	 * @param aspec an AADL AadlSpec object
	 * @return An ArrayList of ProcessorImplementation objects
	 */
	public ArrayList<ProcessorImplementation> AadlPackage_GetProcessorImpl(AadlPackage ap){

		ArrayList<ProcessorImplementation> alist = new ArrayList<ProcessorImplementation>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<ProcessorImplementation> list1 = Package_GetProcessorImpl(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<ProcessorImplementation> list2 = Package_GetProcessorImpl(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}	
		}

		return alist;
	}



	/**
	 * Get a list of all DeviceType objects of AadlPackage (ap)
	 * 
	 * @param ap 
	 * @return an ArrayList of DeviceType objects
	 */
	public ArrayList<DeviceType> AadlPackage_GetDeviceType(AadlPackage ap){
		ArrayList<DeviceType> alist = new ArrayList<DeviceType>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<DeviceType> list1 = Package_GetDeviceType(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<DeviceType> list2 = Package_GetDeviceType(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}
		}

		return alist;
	}


	/**
	 * Get list of all DeviceImplementation objects of a AadlPackage (aspec)
	 * 
	 * @param aspec an AADL AadlSpec object
	 * @return An ArrayList of DeviceImplementation objects
	 */
	public ArrayList<DeviceImplementation> AadlPackage_GetDeviceImpl(AadlPackage ap){

		ArrayList<DeviceImplementation> alist = new ArrayList<DeviceImplementation>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<DeviceImplementation> list1 = Package_GetDeviceImpl(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<DeviceImplementation> list2 = Package_GetDeviceImpl(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}	
		}

		return alist;
	}


	/**
	 * Get a list of all BusType objects of AadlPackage (ap)
	 * 
	 * @param ap 
	 * @return an ArrayList of BusType objects
	 */
	public ArrayList<BusType> AadlPackage_GetBusType(AadlPackage ap){
		ArrayList<BusType> alist = new ArrayList<BusType>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<BusType> list1 = Package_GetBusType(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<BusType> list2 = Package_GetBusType(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}
		}

		return alist;
	}


	/**
	 * Get list of all BusImplementation objects of a AadlPackage (aspec)
	 * 
	 * @param aspec an AADL AadlSpec object
	 * @return An ArrayList of BusImplementation objects
	 */
	public ArrayList<BusImplementation> AadlPackage_GetBusImpl(AadlPackage ap){

		ArrayList<BusImplementation> alist = new ArrayList<BusImplementation>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<BusImplementation> list1 = Package_GetBusImpl(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<BusImplementation> list2 = Package_GetBusImpl(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}	
		}

		return alist;
	}


	/**
	 * Get a list of all MemoryType objects of AadlPackage (ap)
	 * 
	 * @param ap 
	 * @return an ArrayList of MemoryType objects
	 */
	public ArrayList<MemoryType> AadlPackage_GetMemoryType(AadlPackage ap){
		ArrayList<MemoryType> alist = new ArrayList<MemoryType>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<MemoryType> list1 = Package_GetMemoryType(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<MemoryType> list2 = Package_GetMemoryType(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}
		}

		return alist;
	}


	/**
	 * Get list of all MemoryImplementation objects of a AadlPackage (aspec)
	 * 
	 * @param aspec an AADL AadlSpec object
	 * @return An ArrayList of MemoryImplementation objects
	 */
	public ArrayList<MemoryImplementation> AadlPackage_GetMemoryImpl(AadlPackage ap){

		ArrayList<MemoryImplementation> alist = new ArrayList<MemoryImplementation>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<MemoryImplementation> list1 = Package_GetMemoryImpl(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<MemoryImplementation> list2 = Package_GetMemoryImpl(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}	
		}

		return alist;
	}


	/**
	 * Get a list of all SubprogramType objects of AadlPackage (ap)
	 * 
	 * @param ap 
	 * @return an ArrayList of SubprogramType objects
	 */
	public ArrayList<SubprogramType> AadlPackage_GetSubprogramType(AadlPackage ap){
		ArrayList<SubprogramType> alist = new ArrayList<SubprogramType>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<SubprogramType> list1 = Package_GetSubprogramType(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<SubprogramType> list2 = Package_GetSubprogramType(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}
		}

		return alist;
	}


	/**
	 * Get list of all SubprogramImplementation objects of a AadlPackage (aspec)
	 * 
	 * @param aspec an AADL AadlSpec object
	 * @return An ArrayList of SubprogramImplementation objects
	 */
	public ArrayList<SubprogramImplementation> AadlPackage_GetSubprogramImpl(AadlPackage ap){

		ArrayList<SubprogramImplementation> alist = new ArrayList<SubprogramImplementation>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<SubprogramImplementation> list1 = Package_GetSubprogramImpl(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<SubprogramImplementation> list2 = Package_GetSubprogramImpl(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}	
		}

		return alist;
	}


	/**
	 * Get list of all FeatureGroupType objects of a AadlPackage (aspec)
	 * 
	 * @param aspec an AADL AadlSpec object
	 * @return An ArrayList of FeatureGroupType objects
	 */
	public ArrayList<FeatureGroupType> AadlPackage_GetFeatureGroupType(AadlPackage ap){

		ArrayList<FeatureGroupType> alist = new ArrayList<FeatureGroupType>();

		if(ap.getPublicSection() != null){
			PublicPackageSection pub_pkg = ap.getPublicSection();
			ArrayList<FeatureGroupType> list1 = Package_GetFeatureGroupType(pub_pkg);
			if(list1.size()>0){
				alist.addAll(list1);
			}
		}

		if(ap.getPrivateSection() != null){
			PrivatePackageSection private_pkg = ap.getPrivateSection();
			ArrayList<FeatureGroupType> list2 = Package_GetFeatureGroupType(private_pkg);
			if(list2.size()>0){
				alist.addAll(list2);
			}	
		}

		return alist;
	}


	/**
	 * Get the main system (implementation) of a AadlPackage (ap)
	 * <p>
	 * Assume that a main system is a system implementation which is not a subsystem of any other systems
	 * 
	 * @param ap an AADL AadlPackage object
	 * @return a system implementation
	 */
	public SystemImplementation AadlPackage_GetMainSystem(AadlPackage ap){

		//get all the systemImplementation of the package
		ArrayList<SystemImplementation> l_sys = AadlPackage_GetsystemImpl(ap);

		//for each systemImpl s, if it is not a subsystem of any other system, then it is selected as the main system
		Iterator<?> it = l_sys.iterator();
		while (it.hasNext())
		{
			SystemImplementation s = (SystemImplementation) it.next();
			if (!IsSubsystem(s,l_sys)){
				Trace("Main system is " + s.getName());
				return s;
			}
		}

		Trace("Not find a main system");
		return null;
	}



	/**
	 * Returns a string name of a package section (pkg)
	 * 
	 * @param pkg a (public or private) package section
	 * 
	 * @return package name
	 */
	public String Package_GetName(PackageSection pkg){
		return pkg.getName();
	}


	/**
	 * Get a list of all DataType objects from a (public or private) package section (pkg)
	 * 
	 * @param pkg
	 * @return an ArrayList of DataType objects
	 */
	public ArrayList<DataType> Package_GetdataType(PackageSection pkg){

		EList<Element> list1 = pkg.getOwnedElements();
		ArrayList<DataType> alist = new ArrayList<DataType>();

		if(list1 != null){
			for(int i=0; i< list1.size(); i++){
				Element e1= list1.get(i);
				if(e1 instanceof DataType){
					alist.add((DataType) e1);
				}	
			}
		}

		return alist;
	}


	/**
	 * Get list of all DataImplementation objects of a package (pkg)
	 * 
	 * @param pkg 
	 * @return An ArrayList of DataImpl objects
	 */
	public ArrayList<DataImplementation> Package_GetdataImpl(PackageSection pkg){
		EList<Element> list1 = pkg.getOwnedElements();
		ArrayList<DataImplementation> alist = new ArrayList<DataImplementation>();

		if(list1 != null){
			for(int i=0; i< list1.size(); i++){
				Element e1= list1.get(i);
				if(e1 instanceof DataImplementation){
					alist.add((DataImplementation) e1);
				}	
			}
		}
		return alist;
	}


	/**
	 * Get a list of all SystemType objects of a package (pkg)
	 * 
	 * @param pkg 
	 * @return an ArrayList of SystemType objects
	 */
	public ArrayList<SystemType> Package_GetsystemType(PackageSection pkg){

		EList<Element> list1 = pkg.getOwnedElements();
		ArrayList<SystemType> alist = new ArrayList<SystemType>();

		if(list1 != null)
		{
			for(int i =0; i< list1.size(); i++){
				if(list1.get(i) instanceof SystemType){
					SystemType s = (SystemType) list1.get(i);
					alist.add(s);
				}	
			}    	   		
		}
		return alist;
	}


	/**
	 * get list of all SystemImplementation objects of a Package (pkg)
	 * 
	 * @param pkg an AADL PackageSection object
	 * @return An ArrayList of SystemImpl objects
	 */
	public ArrayList<SystemImplementation> Package_GetsystemImpl(PackageSection pkg){

		ArrayList<SystemImplementation> alist = new ArrayList<SystemImplementation>();

		if(pkg.getOwnedElements() != null){
			EList<Element> list1 = pkg.getOwnedElements();

			if(list1 != null)
			{
				for(int i =0; i< list1.size(); i++){
					if(list1.get(i) instanceof SystemImplementation){
						SystemImplementation s = (SystemImplementation) list1.get(i);
						alist.add(s);
					}	
				}  	   		
			}
		}

		return alist;
	}


	/**
	 * Get a list of all ProcessType objects of a package (pkg)
	 * 
	 * @param pkg 
	 * @return an ArrayList of ProcessType objects
	 */
	public ArrayList<ProcessType> Package_GetProcessType(PackageSection pkg){

		EList<Element> list1 = pkg.getOwnedElements();
		ArrayList<ProcessType> alist = new ArrayList<ProcessType>();

		if(list1 != null)
		{
			for(int i =0; i< list1.size(); i++){
				if(list1.get(i) instanceof ProcessType){
					ProcessType s = (ProcessType) list1.get(i);
					alist.add(s);
				}	
			}    	   		
		}
		return alist;
	}


	/**
	 * get list of all ProcessImplementation objects of a Package (pkg)
	 * 
	 * @param pkg an AADL PackageSection object
	 * @return An ArrayList of ProcessImplementation objects
	 */
	public ArrayList<ProcessImplementation> Package_GetProcessImpl(PackageSection pkg){

		ArrayList<ProcessImplementation> alist = new ArrayList<ProcessImplementation>();

		if(pkg.getOwnedElements() != null){
			EList<Element> list1 = pkg.getOwnedElements();

			if(list1 != null)
			{
				for(int i =0; i< list1.size(); i++){
					if(list1.get(i) instanceof ProcessImplementation){
						ProcessImplementation s = (ProcessImplementation) list1.get(i);
						alist.add(s);
					}	
				}    	   		
			}
		}

		return alist;
	}


	/**
	 * Get a list of all ThreadType objects of a package (pkg)
	 * 
	 * @param pkg 
	 * @return an ArrayList of ThreadType objects
	 */
	public ArrayList<ThreadType> Package_GetThreadType(PackageSection pkg){

		EList<Element> list1 = pkg.getOwnedElements();
		ArrayList<ThreadType> alist = new ArrayList<ThreadType>();

		if(list1 != null)
		{
			for(int i =0; i< list1.size(); i++){
				if(list1.get(i) instanceof ThreadType){
					ThreadType s = (ThreadType) list1.get(i);
					alist.add(s);
				}	
			}    	   		
		}
		return alist;
	}


	/**
	 * get list of all ThreadImplementation objects of a Package (pkg)
	 * 
	 * @param pkg an AADL PackageSection object
	 * @return An ArrayList of ThreadImplementation objects
	 */
	public ArrayList<ThreadImplementation> Package_GetThreadImpl(PackageSection pkg){

		ArrayList<ThreadImplementation> alist = new ArrayList<ThreadImplementation>();

		if(pkg.getOwnedElements() != null){
			EList<Element> list1 = pkg.getOwnedElements();

			if(list1 != null)
			{
				for(int i =0; i< list1.size(); i++){
					if(list1.get(i) instanceof ThreadImplementation){
						ThreadImplementation s = (ThreadImplementation) list1.get(i);
						alist.add(s);
					}	
				}    	   		
			}
		}

		return alist;
	}


	/**
	 * Get a list of all ProcessorType objects of a package (pkg)
	 * 
	 * @param pkg 
	 * @return an ArrayList of ProcessorType objects
	 */
	public ArrayList<ProcessorType> Package_GetProcessorType(PackageSection pkg){

		EList<Element> list1 = pkg.getOwnedElements();
		ArrayList<ProcessorType> alist = new ArrayList<ProcessorType>();

		if(list1 != null)
		{
			for(int i =0; i< list1.size(); i++){
				if(list1.get(i) instanceof ProcessorType){
					ProcessorType s = (ProcessorType) list1.get(i);
					alist.add(s);
				}	
			}    	   		
		}

		return alist;
	}


	/**
	 * get list of all ProcessorImplementation objects of a Package (pkg)
	 * 
	 * @param pkg an AADL PackageSection object
	 * @return An ArrayList of ProcessorImplementation objects
	 */
	public ArrayList<ProcessorImplementation> Package_GetProcessorImpl(PackageSection pkg){

		ArrayList<ProcessorImplementation> alist = new ArrayList<ProcessorImplementation>();

		if(pkg.getOwnedElements() != null){
			EList<Element> list1 = pkg.getOwnedElements();

			if(list1 != null)
			{
				for(int i =0; i< list1.size(); i++){
					if(list1.get(i) instanceof ProcessorImplementation){
						ProcessorImplementation s = (ProcessorImplementation) list1.get(i);
						alist.add(s);
					}	
				}   	   		
			}
		}

		return alist;
	}


	/**
	 * Get a list of all DeviceType objects of a package (pkg)
	 * 
	 * @param pkg 
	 * @return an ArrayList of DeviceType objects
	 */
	public ArrayList<DeviceType> Package_GetDeviceType(PackageSection pkg){

		EList<Element> list1 = pkg.getOwnedElements();
		ArrayList<DeviceType> alist = new ArrayList<DeviceType>();

		if(list1 != null)
		{
			for(int i =0; i< list1.size(); i++){
				if(list1.get(i) instanceof DeviceType){
					DeviceType s = (DeviceType) list1.get(i);
					alist.add(s);
				}	
			}    	   		
		}
		return alist;
	}


	/**
	 * get list of all DeviceImplementation objects of a Package (pkg)
	 * 
	 * @param pkg an AADL PackageSection object
	 * @return An ArrayList of DeviceImplementation objects
	 */
	public ArrayList<DeviceImplementation> Package_GetDeviceImpl(PackageSection pkg){

		ArrayList<DeviceImplementation> alist = new ArrayList<DeviceImplementation>();

		if(pkg.getOwnedElements() != null){
			EList<Element> list1 = pkg.getOwnedElements();

			if(list1 != null)
			{
				for(int i =0; i< list1.size(); i++){
					if(list1.get(i) instanceof DeviceImplementation){
						DeviceImplementation s = (DeviceImplementation) list1.get(i);
						alist.add(s);
					}	
				}    	   		
			}
		}

		return alist;
	}


	/**
	 * Get a list of all BusType objects of a package (pkg)
	 * 
	 * @param pkg 
	 * @return an ArrayList of BusType objects
	 */
	public ArrayList<BusType> Package_GetBusType(PackageSection pkg){

		EList<Element> list1 = pkg.getOwnedElements();
		ArrayList<BusType> alist = new ArrayList<BusType>();

		if(list1 != null)
		{
			for(int i =0; i< list1.size(); i++){
				if(list1.get(i) instanceof BusType){
					BusType s = (BusType) list1.get(i);
					alist.add(s);
				}	
			}    	   		
		}
		return alist;
	}


	/**
	 * get list of all BusImplementation objects of a Package (pkg)
	 * 
	 * @param pkg an AADL PackageSection object
	 * @return An ArrayList of BusImplementation objects
	 */
	public ArrayList<BusImplementation> Package_GetBusImpl(PackageSection pkg){

		ArrayList<BusImplementation> alist = new ArrayList<BusImplementation>();

		if(pkg.getOwnedElements() != null){
			EList<Element> list1 = pkg.getOwnedElements();

			if(list1 != null)
			{
				for(int i =0; i< list1.size(); i++){
					if(list1.get(i) instanceof BusImplementation){
						BusImplementation s = (BusImplementation) list1.get(i);
						alist.add(s);
					}	
				}   	   		
			}
		}

		return alist;
	}


	/**
	 * Get a list of all MemoryType objects of a package (pkg)
	 * 
	 * @param pkg 
	 * @return an ArrayList of MemoryType objects
	 */
	public ArrayList<MemoryType> Package_GetMemoryType(PackageSection pkg){

		EList<Element> list1 = pkg.getOwnedElements();
		ArrayList<MemoryType> alist = new ArrayList<MemoryType>();

		if(list1 != null)
		{
			for(int i =0; i< list1.size(); i++){
				if(list1.get(i) instanceof MemoryType){
					MemoryType s = (MemoryType) list1.get(i);
					alist.add(s);
				}	
			}    	   		
		}
		return alist;
	}


	/**
	 * get list of all MemoryImplementation objects of a Package (pkg)
	 * 
	 * @param pkg an AADL PackageSection object
	 * @return An ArrayList of MemoryImplementation objects
	 */
	public ArrayList<MemoryImplementation> Package_GetMemoryImpl(PackageSection pkg){

		ArrayList<MemoryImplementation> alist = new ArrayList<MemoryImplementation>();

		if(pkg.getOwnedElements() != null){
			EList<Element> list1 = pkg.getOwnedElements();

			if(list1 != null)
			{
				for(int i =0; i< list1.size(); i++){
					if(list1.get(i) instanceof MemoryImplementation){
						MemoryImplementation s = (MemoryImplementation) list1.get(i);
						alist.add(s);
					}	
				}  	   		
			}
		}

		return alist;
	}


	/**
	 * Get a list of all SubprogramType objects of a package (pkg)
	 * 
	 * @param pkg 
	 * @return an ArrayList of SubprogramType objects
	 */
	public ArrayList<SubprogramType> Package_GetSubprogramType(PackageSection pkg){

		EList<Element> list1 = pkg.getOwnedElements();
		ArrayList<SubprogramType> alist = new ArrayList<SubprogramType>();

		if(list1 != null)
		{
			for(int i =0; i< list1.size(); i++){
				if(list1.get(i) instanceof SubprogramType){
					SubprogramType s = (SubprogramType) list1.get(i);
					alist.add(s);
				}	
			}    	   		
		}
		return alist;
	}


	/**
	 * get list of all SubprogramImplementation objects of a Package (pkg)
	 * 
	 * @param pkg an AADL PackageSection object
	 * @return An ArrayList of SubprogramImplementation objects
	 */
	public ArrayList<SubprogramImplementation> Package_GetSubprogramImpl(PackageSection pkg){

		ArrayList<SubprogramImplementation> alist = new ArrayList<SubprogramImplementation>();

		if(pkg.getOwnedElements() != null){
			EList<Element> list1 = pkg.getOwnedElements();

			if(list1 != null)
			{
				for(int i =0; i< list1.size(); i++){
					if(list1.get(i) instanceof SubprogramImplementation){
						SubprogramImplementation s = (SubprogramImplementation) list1.get(i);
						alist.add(s);
					}	
				}  	   		
			}
		}

		return alist;
	}



	/**
	 * get list of all FeatureGroupType objects of a Package (pkg)
	 * 
	 * @param pkg an AADL PackageSection object
	 * @return An ArrayList of FeatureGroupType objects
	 */
	public ArrayList<FeatureGroupType> Package_GetFeatureGroupType(PackageSection pkg){

		ArrayList<FeatureGroupType> alist = new ArrayList<FeatureGroupType>();

		if(pkg.getOwnedElements() != null){
			EList<Element> list1 = pkg.getOwnedElements();

			if(list1 != null)
			{
				for(int i =0; i< list1.size(); i++){
					if(list1.get(i) instanceof FeatureGroupType){
						FeatureGroupType s = (FeatureGroupType) list1.get(i);
						alist.add(s);
					}	
				}  	   		
			}
		}

		return alist;
	}


	/**
	 * Returns string name of a property set ps
	 * @param ps a property set
	 * @return string name
	 */
	public String PropertySet_GetName(PropertySet ps){
		return ps.getName();
	}


	/**
	 * Returns whether a systemImpl (s) is a subsystem of any systemImpl in the list (l_sys)
	 * 
	 * @param s a system implementation component
	 * @param l_sys_impl a list of system implementation components
	 * @return a boolean value that indicates whether s is a subsystem of l_sys_impl
	 */
	public Boolean IsSubsystem(SystemImplementation s, ArrayList<SystemImplementation> l_sys_impl){

		Iterator<SystemImplementation> it = l_sys_impl.iterator();
		//for each sys_impl in l_sys_impl list
		while (it.hasNext())
		{
			SystemImplementation sys_impl = (SystemImplementation) it.next();
			//get subsystems
			ArrayList<SystemSubcomponent> l_sub_sys = System_GetSubcomponentsSystem(sys_impl);

			Iterator<SystemSubcomponent> it1 = l_sub_sys.iterator();
			//for each subsystem of sys_impl
			while (it1.hasNext())
			{
				SystemSubcomponent sys = (SystemSubcomponent) it1.next();
				//get the classifier
				ComponentClassifier sys_classifier = sys.getClassifier();
				if (sys_classifier instanceof SystemImplementation){
					if (sys_classifier == s) return true;
				}			
			}		
		}	
		return false;
	}

	public boolean isInputPort(Port aPort)
	{
		return( Port_GetDirection(aPort) == "in");
	}
	public boolean isOutputPort(Port aPort)
	{
		return( Port_GetDirection(aPort) == "out");
	}

	/**
	 * Get list of in Data Ports of a SystemType (systype)
	 * 
	 * <p>
	 * Get the data ports of a system type, and selects the ones whose direction is "in".
	 * 
	 * @param systype a system type component
	 * @return An ArrayList of DataPort objects
	 */
	public ArrayList<DataPort> System_GetInDataPort(SystemType systype)
	{	
		ArrayList<DataPort> indataports = new ArrayList<DataPort>();

		if ( systype.getOwnedDataPorts() != null){
			EList<DataPort> dataports = systype.getOwnedDataPorts();

			if (dataports.size()>0){
				Iterator<?> it_test = dataports.iterator();
				while (it_test.hasNext())
				{
					DataPort dataPort = (DataPort) it_test.next();
					if (isInputPort(dataPort)) indataports.add(dataPort);
				}
			}
		}
		return indataports;
	}


	/**
	 * Get list of out Data Ports of a SystemType (systype)
	 * 
	 * <p>
	 * Get the data ports of a system type, and selects the ones whose direction is "out".
	 * 
	 * @param systype a system type component
	 * @return An ArrayList of DataPort objects
	 */
	public ArrayList<DataPort> System_GetOutDataPort(SystemType systype){	

		ArrayList<DataPort> outdataports = new ArrayList<DataPort>();

		if(systype.getOwnedDataPorts() != null){
			EList<DataPort> dataports = systype.getOwnedDataPorts();

			if(dataports.size()>0){
				Iterator<?> it = dataports.iterator();
				while (it.hasNext())
				{
					DataPort dataPort = (DataPort) it.next();
					if (isOutputPort(dataPort)) 	outdataports.add(dataPort);
				}
			}	    
		}

		return outdataports;
	}


	/**
	 * Get list of in event Ports of a SystemType (systype)
	 * 
	 * <p>
	 * Get the event ports of a system type, and selects the ones whose direction is "in".
	 * 
	 * @param systype a system type component
	 * @return An ArrayList of EventPort objects
	 */
	public ArrayList<EventPort> System_GetInEventPort(SystemType systype){	

		ArrayList<EventPort> ineventports = new ArrayList<EventPort>();	
		if (systype.getOwnedEventPorts() != null){

			EList<EventPort> eventports = systype.getOwnedEventPorts();		
			if(eventports.size()>0){
				Iterator<?> it = eventports.iterator();
				while (it.hasNext())
				{
					EventPort eventPort = (EventPort) it.next();
					if (isInputPort(eventPort)) ineventports.add(eventPort);
				}
			}		
		}
		return ineventports;
	}


	/**
	 * Get list of out event Ports of a SystemType (systype)
	 * 
	 * <p>
	 * Get the event ports of a system type, and selects the ones whose direction is "out".
	 * 
	 * @param systype a system type component
	 * @return An ArrayList of EventPort objects
	 */
	public ArrayList<EventPort> System_GetOutEventPort(SystemType systype){	

		ArrayList<EventPort> outeventports = new ArrayList<EventPort>();

		if(systype.getOwnedEventPorts() != null){

			EList<EventPort> eventports = systype.getOwnedEventPorts();		
			if(eventports.size()>0){	    	
				Iterator<?> it = eventports.iterator();
				while (it.hasNext())
				{
					EventPort eventPort = (EventPort) it.next();
					if (isOutputPort(eventPort)) outeventports.add(eventPort);
				}
			}
		}

		return outeventports;
	}


	/**
	 * Get list of in event data Ports of a SystemType (systype)
	 * 
	 * <p>
	 * Get the event data ports of a system type, and selects the ones whose direction is "in".
	 * 
	 * @param systype a system type component
	 * @return An ArrayList of EventDataPort objects
	 */
	public ArrayList<EventDataPort> System_GetInEventDataPort(SystemType systype){	

		ArrayList<EventDataPort> ineventdataports = new ArrayList<EventDataPort>();	
		if(systype.getOwnedEventDataPorts() != null){

			EList<EventDataPort> eventdataports = systype.getOwnedEventDataPorts();	    
			if(eventdataports.size()>0){
				Iterator<?> it = eventdataports.iterator();
				while (it.hasNext())
				{
					EventDataPort eventdataPort = (EventDataPort) it.next();
					if (isInputPort(eventdataPort)) 
						ineventdataports.add(eventdataPort);
				}
			}
		}
		return ineventdataports;
	}


	/**
	 * Get list of out event data Ports of a SystemType (systype)
	 * 
	 * <p>
	 * Get the event data ports of a system type, and selects the ones whose direction is "out".
	 * 
	 * @param systype a system type component
	 * @return An ArrayList of EventDataPort objects
	 */
	public ArrayList<EventDataPort> System_GetOutEventDataPort(SystemType systype){	

		ArrayList<EventDataPort> outeventdataports = new ArrayList<EventDataPort>();	
		if (systype.getOwnedEventDataPorts() != null){

			EList<EventDataPort> eventdataports = systype.getOwnedEventDataPorts();	    
			if(eventdataports.size()>0){
				Iterator<?> it = eventdataports.iterator();
				while (it.hasNext())
				{
					EventDataPort eventdataPort = (EventDataPort) it.next();
					if (isOutputPort(eventdataPort)) outeventdataports.add(eventdataPort);
				}
			}
		}
		return outeventdataports;
	}


	/**
	 * Get the system type of a system implementation component (sysimpl)
	 * 
	 * @param sysimpl a system implementation component
	 * @return a SystemType object
	 */
	public SystemType System_GetCompType(SystemImplementation sysimpl)
	{	
		return sysimpl.getType();
	}


	/**
	 * Get a list of sub systems of a SystemImplementation object(sysimpl)
	 * 
	 * @param sysimpl a system implementation component
	 * @return An ArrayList of SystemSubcomponent objects
	 */
	public ArrayList<SystemSubcomponent> System_GetSubcomponentsSystem(SystemImplementation sysimpl){	

		ArrayList<SystemSubcomponent> list_s = new ArrayList<SystemSubcomponent>();

		if(sysimpl.getOwnedSystemSubcomponents() != null){
			EList<SystemSubcomponent> subsys = sysimpl.getOwnedSystemSubcomponents();
			if(subsys.size()>0){
				Iterator<?> it = subsys.iterator();
				while (it.hasNext())
				{
					SystemSubcomponent s = (SystemSubcomponent) it.next();
					list_s.add(s);
				}
			}    
		}

		return list_s;	
	}


	/**
	 * Get a list of sub processes of a SystemImpl (sysimpl)
	 * 
	 * @param sysimple a system implementaton component
	 * @return An ArrayList of ProcessSubcomponent objects
	 */
	public ArrayList<ProcessSubcomponent> System_GetSubcomponentsProcess(SystemImplementation sysimpl){	

		ArrayList<ProcessSubcomponent> list_s = new ArrayList<ProcessSubcomponent>();	
		if (sysimpl.getOwnedProcessSubcomponents() != null){
			EList<ProcessSubcomponent> subs = sysimpl.getOwnedProcessSubcomponents();

			if(subs.size()>0){
				Iterator<?> it = subs.iterator();
				while (it.hasNext())
				{
					ProcessSubcomponent s = (ProcessSubcomponent) it.next();
					list_s.add(s);
				}
			}    
		}

		return list_s;	
	}


	/**
	 * Get a list of sub data components a SystemImpl (sysimpl)
	 * 
	 * @param sysimpl a system implementation component
	 * @return An ArrayList of DataSubcomponent objects
	 */
	public ArrayList<DataSubcomponent> System_GetSubcomponentsData(SystemImplementation sysimpl){	

		ArrayList<DataSubcomponent> list_s = new ArrayList<DataSubcomponent>();   
		if(sysimpl.getOwnedDataSubcomponents() != null){
			EList<DataSubcomponent> subs = sysimpl.getOwnedDataSubcomponents();

			if(subs.size()>0){
				Iterator<?> it = subs.iterator();
				while (it.hasNext())
				{
					DataSubcomponent s = (DataSubcomponent) it.next();
					list_s.add(s);
				}
			}       
		}

		return list_s;	
	}


	/**
	 * Get a list of sub memory components of a SystemImpl (sysimpl)
	 * 
	 * @param sysimpl a system implementation component
	 * @return An ArrayList of MemorySubcomponent objects
	 */
	public ArrayList<MemorySubcomponent> System_GetSubcomponentsMemory(SystemImplementation sysimpl){	

		ArrayList<MemorySubcomponent> list_s = new ArrayList<MemorySubcomponent>();   
		if(sysimpl.getOwnedMemorySubcomponents() != null){
			EList<MemorySubcomponent> subs = sysimpl.getOwnedMemorySubcomponents();

			if(subs.size()>0){
				Iterator<?> it = subs.iterator();
				while (it.hasNext())
				{
					MemorySubcomponent s = (MemorySubcomponent) it.next();
					list_s.add(s);
				}
			}      
		}

		return list_s;	
	}


	/**
	 * Get a list of sub processors of a SystemImpl (sysimpl)
	 * 
	 * @param sysimpl a system implementation component
	 * @return An ArrayList of ProcessorSubcomponent objects
	 */
	public ArrayList<ProcessorSubcomponent> System_GetSubcomponentsProcessor(SystemImplementation sysimpl){	

		ArrayList<ProcessorSubcomponent> list_s = new ArrayList<ProcessorSubcomponent>();	
		if(sysimpl.getOwnedProcessorSubcomponents() != null){
			EList<ProcessorSubcomponent> subs = sysimpl.getOwnedProcessorSubcomponents();

			if(subs.size()>0){
				Iterator<?> it = subs.iterator();
				while (it.hasNext())
				{
					ProcessorSubcomponent s = (ProcessorSubcomponent) it.next();
					list_s.add(s);
				}
			}   
		}

		return list_s;	
	}


	/**
	 * Get a list of sub devices of a SystemImpl (sysimpl)
	 * 
	 * @param sysimpl a system implementation component
	 * @return An ArrayList of DeviceSubcomponent objects
	 */
	public ArrayList<DeviceSubcomponent> System_GetSubcomponentsDevice(SystemImplementation sysimpl){		

		ArrayList<DeviceSubcomponent> list_s = new ArrayList<DeviceSubcomponent>();	
		if(sysimpl.getOwnedDeviceSubcomponents() != null){
			EList<DeviceSubcomponent> subs = sysimpl.getOwnedDeviceSubcomponents();

			if(subs.size()>0){
				Iterator<?> it = subs.iterator();
				while (it.hasNext())
				{
					DeviceSubcomponent s = (DeviceSubcomponent) it.next();
					list_s.add(s);
				}
			}    
		}

		return list_s;	
	}


	/**
	 * Get a list of sub bus components of a SystemImpl (sysimpl)
	 * 
	 * @param sysimpl a system implementation component
	 * @return An ArrayList of BusSubcomponent objects
	 */
	public ArrayList<BusSubcomponent> System_GetSubcomponentsBus(SystemImplementation sysimpl){	

		ArrayList<BusSubcomponent> list_s = new ArrayList<BusSubcomponent>();
		if(sysimpl.getOwnedBusSubcomponents() != null){
			EList<BusSubcomponent> subs = sysimpl.getOwnedBusSubcomponents();

			if(subs.size()>0){
				Iterator<?> it = subs.iterator();
				while (it.hasNext())
				{
					BusSubcomponent s = (BusSubcomponent) it.next();
					list_s.add(s);
				}
			}   
		}

		return list_s;	
	}


	/**
	 * Get a list of data port connections from a SystemImpl (sysimpl)
	 * 
	 * @param sysimpl a system implementation component
	 * @return An ArrayList of Data port Connections
	 */
	public ArrayList<PortConnection> System_GetDataConnection(SystemImplementation sysimpl)
	{	
		ArrayList<PortConnection> list= new ArrayList<PortConnection>();	
		EList<PortConnection> list1 = sysimpl.getOwnedPortConnections();
		if(list1 != null){
			if(list1.size()>0){
				for(int i =0; i<list1.size(); i++){
					PortConnection pc = list1.get(i);
					if(Is_DataPortConnection(pc)){
						list.add(pc);
					}
				}
			}
		}
		return list;
	}


	/**
	 * Get a list of event port connections from a SystemImpl (sysimpl)
	 * 
	 * @param sysimpl a system implementation component
	 * @return An ArrayList of Event port Connections
	 */
	public ArrayList<PortConnection> System_GetEventConnection(SystemImplementation sysimpl)
	{	
		ArrayList<PortConnection> list= new ArrayList<PortConnection>();
		EList<PortConnection> list1 = sysimpl.getOwnedPortConnections();
		if(list1 != null){
			if(list1.size()>0){
				for(int i =0; i<list1.size(); i++){
					PortConnection pc = list1.get(i);
					if(Is_EventPortConnection(pc)){
						list.add(pc);
					}
				}
			}
		}
		return list;
	}


	/**
	 * Get a list of event data port connections from a SystemImpl (sysimpl)
	 * 
	 * @param sysimpl a system implementation component
	 * @return An ArrayList of Event Data port Connections
	 */
	public ArrayList<PortConnection> System_GetEventDataConnection(SystemImplementation sysimpl)
	{	
		ArrayList<PortConnection> list= new ArrayList<PortConnection>();
		EList<PortConnection> list1 = sysimpl.getOwnedPortConnections();
		if(list1 != null){
			if(list1.size()>0){
				for(int i =0; i<list1.size(); i++){
					PortConnection pc = list1.get(i);
					if(Is_EventDataPortConnection(pc)){
						list.add(pc);
					}
				}
			}
		}
		return list;
	}


	/**
	 * Get a BusSubcomponent object that the Connection c is "actual bound" 
	 * <p>
	 * Get the value from "Actual_Connection_Binding" property 
	 * 
	 * @param sysimpl a system implementation component that contains this connection c
	 * @param c a connection 
	 * @return a BusSubcomponent object that the connection c is bound to
	 */
	public BusSubcomponent System_GetConnectionBindingBus(SystemImplementation sysimpl, Connection c)
	{	return null;}


	/**
	 * Get a ProcessorSubcomponent object that the ProcessSubcomponent p is "actual bound to"
	 * <p>
	 * Get the value from "Actual_Processor_Binding" property 
	 * 
	 * @param sysimpl a system implementation component that contains this sub process p
	 * @param p a sub process
	 * @return a ProcessorSubcomponent object that this sub process p is bound to
	 */
	public ProcessorSubcomponent System_GetProcessBindingProcessor(SystemImplementation sysimpl, ProcessSubcomponent p)
	{	return null;}


	/**
	 * Get a list of in data ports of a process
	 * <p>
	 * Get all the data ports of this process (processtype), 
	 * and select the ones whose direction is "in" 
	 * 
	 * @param processtype a process type component
	 * @return An ArrayList of DataPort objects
	 */
	public ArrayList<DataPort> Process_GetInDataPort(ProcessType processtype)
	{	
		ArrayList<DataPort> indataports = new ArrayList<DataPort>();

		if(processtype.getOwnedDataPorts() != null){
			EList<DataPort> dataports = processtype.getOwnedDataPorts();

			if(dataports.size()>0){
				Iterator<?> it_test = dataports.iterator();
				while (it_test.hasNext())
				{
					DataPort dataPort = (DataPort) it_test.next();
					if (isInputPort(dataPort)) indataports.add(dataPort);
				}
			}
		}
		return indataports;	
	}


	/**
	 * Get a list of out data ports of a process
	 * <p>
	 * Get all the data ports of this process (processtype), 
	 * and select the ones whose direction is "out" 
	 * 
	 * @param processtype a process type component
	 * @return An ArrayList of DataPort objects
	 */
	public ArrayList<DataPort> Process_GetOutDataPort(ProcessType processtype){	

		ArrayList<DataPort> outdataports = new ArrayList<DataPort>();

		if (processtype.getOwnedDataPorts() != null){
			EList<DataPort> dataports = processtype.getOwnedDataPorts();

			if(dataports.size()>0){
				Iterator<?> it = dataports.iterator();
				while (it.hasNext())
				{
					DataPort dataPort = (DataPort) it.next();
					if (isOutputPort(dataPort)) outdataports.add(dataPort);
				}
			}
		}

		return outdataports;
	}


	/**
	 * Get a list of in event ports of a process
	 * <p>
	 * Get all the event ports of this process (processtype), 
	 * and select the ones whose direction is "in" 
	 * 
	 * @param processtype a process type component
	 * @return An ArrayList of EventPort objects
	 */
	public ArrayList<EventPort> Process_GetInEventPort(ProcessType processtype){	

		ArrayList<EventPort> ineventports = new ArrayList<EventPort>();	
		if(processtype.getOwnedEventPorts() != null){

			EList<EventPort> eventports = processtype.getOwnedEventPorts();		
			if(eventports.size()>0){
				Iterator<?> it = eventports.iterator();
				while (it.hasNext())
				{
					EventPort eventPort = (EventPort) it.next();
					if (isInputPort(eventPort)) ineventports.add(eventPort);
				}
			}
		}

		return ineventports;
	}


	/**
	 * Get a list of out event ports of a process
	 * <p>
	 * Get all the event ports of this process (processtype), 
	 * and select the ones whose direction is "out" 
	 * 
	 * @param processtype a process type component
	 * @return An ArrayList of EventPort objects
	 */
	public ArrayList<EventPort> Process_GetOutEventPort(ProcessType processtype){	

		ArrayList<EventPort> outeventports = new ArrayList<EventPort>();	
		if(processtype.getOwnedEventPorts() != null){

			EList<EventPort> eventports = processtype.getOwnedEventPorts();		
			if(eventports.size()>0){
				Iterator<?> it = eventports.iterator();
				while (it.hasNext())
				{
					EventPort eventPort = (EventPort) it.next();
					if (isOutputPort(eventPort)) outeventports.add(eventPort);
				}
			}
		}
		return outeventports;
	}


	/**
	 * Get a list of in event data ports of a process
	 * <p>
	 * Get all the event data ports of this process (processtype), 
	 * and select the ones whose direction is "in" 
	 * 
	 * @param processtype a process type component
	 * @return An ArrayList of EventDataPort objects
	 */
	public ArrayList<EventDataPort> Process_GetInEventDataPort(ProcessType processtype){	

		ArrayList<EventDataPort> ineventdataports = new ArrayList<EventDataPort>();	
		if(processtype.getOwnedEventDataPorts() != null){

			EList<EventDataPort> eventdataports = processtype.getOwnedEventDataPorts();	    
			if(eventdataports.size()>0){
				Iterator<?> it = eventdataports.iterator();
				while (it.hasNext())
				{
					EventDataPort eventdataPort = (EventDataPort) it.next();
					if (isInputPort(eventdataPort))  ineventdataports.add(eventdataPort);
				}
			}
		}

		return ineventdataports;
	}


	/**
	 * Get a list of out event data ports of a process
	 * <p>
	 * Get all the event data ports of this process (processtype), 
	 * and select the ones whose direction is "out" 
	 * 
	 * @param processtype a process type component
	 * @return An ArrayList of EventDataPort objects
	 */
	public ArrayList<EventDataPort> Process_GetOutEventDataPort(ProcessType processtype){	

		ArrayList<EventDataPort> outeventdataports = new ArrayList<EventDataPort>();	
		if(processtype.getOwnedEventDataPorts() != null){

			EList<EventDataPort> eventdataports = processtype.getOwnedEventDataPorts();	    
			if(eventdataports.size()>0){
				Iterator<?> it = eventdataports.iterator();
				while (it.hasNext())
				{
					EventDataPort eventdataPort = (EventDataPort) it.next();
					if (isOutputPort(eventdataPort)) outeventdataports.add(eventdataPort);
				}
			}
		}
		return outeventdataports;
	}


	/**
	 * Get the corresponding process type of a ProcessImpl (processimpl)
	 * 
	 * @param processimpl a process implementation component
	 * @return a ProcessType object
	 */
	public ProcessType Process_GetCompType(ProcessImplementation processimpl)
	{
		return processimpl.getType();
	}


	/**
	 * Get a list of sub threads of a ProcessImplementation (processimpl)
	 * 
	 * @param processimpl a process implementation component
	 * @return An ArrayList of ThreadSubcomponent objects
	 */
	public ArrayList<ThreadSubcomponent> Process_GetSubcomponentsThread(ProcessImplementation processimpl){		

		ArrayList<ThreadSubcomponent> list_s = new ArrayList<ThreadSubcomponent>();	
		if (processimpl.getOwnedThreadSubcomponents() != null){		
			EList<ThreadSubcomponent> subs = processimpl.getOwnedThreadSubcomponents();		
			if(subs.size() >0 ){
				Iterator<?> it = subs.iterator();
				while (it.hasNext())
				{
					ThreadSubcomponent s = (ThreadSubcomponent) it.next();
					list_s.add(s);
				}
			}
		}

		return list_s;	
	}


	/**
	 * Get a list of sub thread groups of a ProcessImpl (processimpl)
	 * 
	 * @param processimpl a process implementation component
	 * @return An ArrayList of ThreadGroupSubcomponent objects
	 */
	public ArrayList<ThreadGroupSubcomponent> Process_GetSubcomponentsThreadGroup(ProcessImplementation processimpl){		

		ArrayList<ThreadGroupSubcomponent> list_s = new ArrayList<ThreadGroupSubcomponent>();
		EList<ThreadGroupSubcomponent> list1= processimpl.getOwnedThreadGroupSubcomponents();
		if(list1 != null){
			if(list1.size()>0){
				list_s.addAll(list1);
			}
		}

		return list_s;	
	}


	/**
	 * Get a list of sub data components of a ProcessImpl(processimpl)
	 * 
	 * @param processimpl a process implementation component
	 * @return An ArrayList of DataSubcomponent objects
	 */
	public ArrayList<DataSubcomponent> Process_GetSubcomponentsData(ProcessImplementation processimpl){		

		ArrayList<DataSubcomponent> list_s = new ArrayList<DataSubcomponent>();
		EList<DataSubcomponent> p_subcomponents = processimpl.getOwnedDataSubcomponents();

		if (p_subcomponents == null){
			return list_s;	
		}
		else{ 
			if(p_subcomponents.size() >0 ){
				for(int i =0; i< p_subcomponents.size(); i++)
				{
					DataSubcomponent s = p_subcomponents.get(i);
					list_s.add(s);
				}
			}		
		}

		return list_s;	
	}



	/**
	 * Get a list of sub subprogram components of a ProcessImpl(processimpl)
	 * 
	 * @param processimpl a process implementation component
	 * @return An ArrayList of SubprogramSubcomponent objects
	 */
	public ArrayList<SubprogramSubcomponent> Process_GetSubcomponents_Subprogram(ProcessImplementation processimpl){		

		ArrayList<SubprogramSubcomponent> list_s = new ArrayList<SubprogramSubcomponent>();
		EList<SubprogramSubcomponent> p_subcomponents = processimpl.getOwnedSubprogramSubcomponents();

		if (p_subcomponents == null){
			//Trace("p_subcomponents == null");
			return list_s;	
		}
		else{ 
			if(p_subcomponents.size() >0 ){
				//Trace("p_subcomponents size > 0");
				for(int i =0; i< p_subcomponents.size(); i++)
				{
					SubprogramSubcomponent s = p_subcomponents.get(i);
					list_s.add(s);
				}
			}		
		}

		return list_s;	
	}


	/**
	 * Get the period of a ProcessImpl (processimpl)
	 * <p>
	 * This value is fetched from "Period" property
	 * 
	 * @param processimpl a process implementation component
	 * @return a String value of process period
	 */
	public String Process_GetProcessPeriod(ProcessImplementation processimpl)
	{
		EList<PropertyAssociation> list1 = processimpl.getAllPropertyAssociations();
		EList<PropertyAssociation> list2 = processimpl.getType().getAllPropertyAssociations();		
		ArrayList<PropertyAssociation> list = new ArrayList<PropertyAssociation>();
		if(list1 != null){
			if(list1.size()>0){
				list.addAll(list1);
			}
		}

		if(list2 != null){
			if(list2.size()>0){
				list.addAll(list2);
			}
		}

		for(int i=0; i<list.size(); i++)
		{
			PropertyAssociation pa = list.get(i);
			String propertyDef_name = Property_Get_Name(pa);

			if (propertyDef_name.equals("Period")) 
			{
				IntegerLiteral int_value = Property_Get_Period(pa);   
				if(int_value != null){
					String v1 = IntegerLiteral_Get_Value(int_value);
					String v2 = IntegerLiteral_Get_Unit(int_value);
					return v1 + " " + v2;
				}

			}
		}			

		return null;	
	}


	/**
	 * Get a list of data port connections from a ProcessImpl (processimpl)
	 * 
	 * @param processimpl a process implementation component
	 * @return A List of DataConnection objects
	 */
	public ArrayList<PortConnection> Process_GetDataConnection(ProcessImplementation processimpl)
	{
		ArrayList<PortConnection> list = new ArrayList<PortConnection>();
		EList<PortConnection> list1 = processimpl.getOwnedPortConnections();
		if(list1 != null){
			for(int i =0; i<list1.size(); i++){
				PortConnection c = list1.get(i);
				if(Is_DataPortConnection(c)){
					list.add(c);
				}
			}
		}

		return list;
	}


	/**
	 * Get A list of event port connections from ProcessImpl (processimpl)
	 * 
	 * @param processimpl a process implementation component
	 * @return List of EventConnection objects
	 */
	public ArrayList<PortConnection> Process_GetEventConnection(ProcessImplementation processimpl)
	{
		ArrayList<PortConnection> list = new ArrayList<PortConnection>();
		EList<PortConnection> list1 = processimpl.getOwnedPortConnections();
		if(list1 != null){
			for(int i =0; i<list1.size(); i++){
				PortConnection c = list1.get(i);
				if(Is_EventPortConnection(c)){
					list.add(c);
				}
			}
		}

		return list;
	}


	/**
	 * Get a list of event data connections from a ProcessImpl (processimpl)
	 * 
	 * @param processimpl a process implementation component
	 * @return List of EventDataConnection objects
	 */
	public ArrayList<PortConnection> Process_GetEventDataConnection(ProcessImplementation processimpl)
	{
		ArrayList<PortConnection> list = new ArrayList<PortConnection>();
		EList<PortConnection> list1 = processimpl.getOwnedPortConnections();
		if(list1 != null){
			for(int i =0; i<list1.size(); i++){
				PortConnection c = list1.get(i);
				if(Is_EventDataPortConnection(c)){
					list.add(c);
				}
			}
		}

		return list;
	}



	/**
	 * Get the thread subprograms that requires access subprogram x in process p
	 * @param p a process implementation
	 * @param x a string name of a subprogram (type of implementation)
	 * @return a list of thread subcomponents
	 */
	public ArrayList<ThreadSubcomponent> Process_Get_Threads_Requires_Subprogram_x(ProcessImplementation p, String x)
	{
		ArrayList<ThreadSubcomponent> list1 = new ArrayList<ThreadSubcomponent>();
		ArrayList<ThreadSubcomponent> list2 = Process_GetSubcomponentsThread(p);

		for(int i=0;i<list2.size();i++){
			ThreadSubcomponent t1 = list2.get(i);
			ThreadImplementation ti = (ThreadImplementation) t1.getClassifier();
			ThreadType tt = ti.getType();
			if (ThreadType_Get_SubprogramAccess_x(tt, x) != null){
				list1.add(t1);
			}
		}

		return list1;
	}


	/**
	 * Get the thread subprograms that calls subprogram x in process p
	 * @param p a process implementation
	 * @param x a string name of a subprogram (type of implementation)
	 * @return a list of thread subcomponents
	 */
	public ArrayList<ThreadSubcomponent> Process_Get_Threads_Calls_Subprogram_x(ProcessImplementation p, String x)
	{
		ArrayList<ThreadSubcomponent> list1 = new ArrayList<ThreadSubcomponent>();
		ArrayList<ThreadSubcomponent> list2 = Process_GetSubcomponentsThread(p);

		for(int i=0;i<list2.size();i++){
			ThreadSubcomponent t1 = list2.get(i);
			ThreadImplementation ti = (ThreadImplementation) t1.getClassifier();
			ArrayList<CalledSubprogram> called_sps = Thread_Get_CalledSubprograms(ti);
			for(int j=0; j< called_sps.size(); j++){
				CalledSubprogram called_sp1 = called_sps.get(j);
				if(called_sp1 instanceof SubprogramType){
					SubprogramType spt = (SubprogramType) called_sp1;
					String str1 = spt.getName();
					if(str1.equals(x)){
						list1.add(t1);
					}
				}
				else if(called_sp1 instanceof SubprogramImplementation){
					SubprogramImplementation spi = (SubprogramImplementation) called_sp1;
					String str2 = spi.getName();
					if(str2.equals(x)){
						list1.add(t1);
					}
				}
				else{
					//Trace("called subprogram catefory not implemented");
					throw new IllegalArgumentException("called subprogram category illegal");
				}
			}
		}

		return list1;
	}



	/**
	 * Get a list of in data ports of a DeviceType (devicetype)
	 * 
	 * @param devicetype a device type component
	 * @return An ArratList of DataPort objects
	 */
	public ArrayList<DataPort> Device_GetInDataPort(DeviceType devicetype){

		ArrayList<DataPort> indataports = new ArrayList<DataPort>();

		if(devicetype.getOwnedDataPorts() != null){
			EList<DataPort> dataports = devicetype.getOwnedDataPorts();

			if(dataports.size() >0 ){
				Iterator<?> it_test = dataports.iterator();
				while (it_test.hasNext())
				{
					DataPort dataPort = (DataPort) it_test.next();
					if (isInputPort(dataPort)) indataports.add(dataPort);
				}
			}  
		}

		return indataports;
	}


	/**
	 * Get a list of out data ports of a DeviceType (devicetype)
	 * 
	 * @param devicetype a device type component
	 * @return An ArratList of DataPort objects
	 */
	public ArrayList<DataPort> Device_GetOutDataPort(DeviceType devicetype){	

		ArrayList<DataPort> outdataports = new ArrayList<DataPort>();

		if(devicetype.getOwnedDataPorts() != null ){
			EList<DataPort> dataports = devicetype.getOwnedDataPorts();

			if(dataports.size() >0 ){
				Iterator<?> it = dataports.iterator();
				while (it.hasNext())
				{
					DataPort dataPort = (DataPort) it.next();
					if (isOutputPort(dataPort)) outdataports.add(dataPort);
				}
			}
		}
		return outdataports;
	}


	/**
	 * Get a list of in event ports of a DeviceType (devicetype)
	 * 
	 * @param devicetype a device type component
	 * @return An ArratList of EventPort objects
	 */
	public ArrayList<EventPort> Device_GetInEventPort(DeviceType devicetype){	

		ArrayList<EventPort> ineventports = new ArrayList<EventPort>();

		if(devicetype.getOwnedEventPorts() != null){
			EList<EventPort> eventports = devicetype.getOwnedEventPorts();

			if(eventports.size() >0){
				Iterator<?> it = eventports.iterator();
				while (it.hasNext())
				{
					EventPort eventPort = (EventPort) it.next();
					if (isInputPort(eventPort)) ineventports.add(eventPort);
				}
			}
		}

		return ineventports;
	}


	/**
	 * Get a list of out event ports of a DeviceType (devicetype)
	 * 
	 * @param devicetype a device type component
	 * @return An ArratList of EventPort objects
	 */
	public ArrayList<EventPort> Device_GetOutEventPort(DeviceType devicetype){	

		ArrayList<EventPort> outeventports = new ArrayList<EventPort>();
		if(devicetype.getOwnedEventPorts() != null){
			EList<EventPort> eventports = devicetype.getOwnedEventPorts();

			if(eventports.size()>0){
				Iterator<?> it = eventports.iterator();
				while (it.hasNext())
				{
					EventPort eventPort = (EventPort) it.next();
					if (isOutputPort(eventPort)) outeventports.add(eventPort);
				}
			}
		}
		return outeventports;
	}


	/**
	 * Get a list of in event data ports of a DeviceType (devicetype)
	 * 
	 * @param devicetype a device type component
	 * @return An ArratList of EventDataPort objects
	 */
	public ArrayList<EventDataPort> Device_GetInEventDataPort(DeviceType devicetype){	

		ArrayList<EventDataPort> ineventdataports = new ArrayList<EventDataPort>();

		if(devicetype.getOwnedEventDataPorts() != null){
			EList<EventDataPort> eventdataports = devicetype.getOwnedEventDataPorts();

			if(eventdataports.size() >0 ){
				Iterator<?> it = eventdataports.iterator();
				while (it.hasNext())
				{
					EventDataPort eventdataPort = (EventDataPort) it.next();
					if (isInputPort(eventdataPort)) ineventdataports.add(eventdataPort);
				}
			}
		}
		return ineventdataports;
	}


	/**
	 * Get a list of out event data ports of a DeviceType (devicetype)
	 * 
	 * @param devicetype a device type component
	 * @return An ArratList of EventDataPort objects
	 */
	public ArrayList<EventDataPort> Device_GetOutEventDataPort(DeviceType devicetype){	

		ArrayList<EventDataPort> outeventdataports = new ArrayList<EventDataPort>();

		if(devicetype.getOwnedEventDataPorts() != null){
			EList<EventDataPort> eventdataports = devicetype.getOwnedEventDataPorts();

			if(eventdataports.size() >0){
				Iterator<?> it = eventdataports.iterator();
				while (it.hasNext())
				{
					EventDataPort eventdataPort = (EventDataPort) it.next();
					if (isOutputPort(eventdataPort)) outeventdataports.add(eventdataPort);
				}
			}
		}

		return outeventdataports;
	}


	/**
	 * Returns a list of bus access of a device type dt
	 * @param dt a device type
	 * @return a list of bus access features
	 */
	public ArrayList<BusAccess> Device_GetBusAccess(DeviceType dt)
	{	
		ArrayList<BusAccess> list = new ArrayList<BusAccess>();
		EList<BusAccess> list1 = dt.getOwnedBusAccesses();
		if(list1 != null) if (list1.size()>0) list.addAll(list1);
		return list;
	}


	/**
	 * Get a list of in data ports of a ProcessorType (processortype)
	 * 
	 * @param processortype a processor type component
	 * @return An ArratList of DataPort objects
	 */
	public ArrayList<DataPort> Processor_GetInDataPort(ProcessorType processortype){

		ArrayList<DataPort> indataports = new ArrayList<DataPort>();

		if(processortype.getOwnedDataPorts() != null){
			EList<DataPort> dataports = processortype.getOwnedDataPorts();	    
			if(dataports.size()>0){       	
				Iterator<?> it_test = dataports.iterator();
				while (it_test.hasNext())
				{
					DataPort dataPort = (DataPort) it_test.next();
					if (isInputPort(dataPort)) indataports.add(dataPort);
				}
			}
		}
		return indataports;
	}


	/**
	 * Get a list of out data ports of a ProcessorType (processortype)
	 * 
	 * @param processortype a processor type component
	 * @return An ArratList of DataPort objects
	 */
	public ArrayList<DataPort> Processor_GetOutDataPort(ProcessorType processortype){	

		ArrayList<DataPort> outdataports = new ArrayList<DataPort>();	
		if(processortype.getOwnedDataPorts() != null){
			EList<DataPort> dataports = processortype.getOwnedDataPorts();	    
			if(dataports.size()>0){
				Iterator<?> it = dataports.iterator();
				while (it.hasNext())
				{
					DataPort dataPort = (DataPort) it.next();
					if (isOutputPort(dataPort)) outdataports.add(dataPort);
				}
			}
		}
		return outdataports;
	}


	/**
	 * Get a list of in event ports of a ProcessorType (processortype)
	 * 
	 * @param processortype a processor type component
	 * @return An ArratList of EventPort objects
	 */
	public ArrayList<EventPort> Processor_GetInEventPort(ProcessorType processortype){	

		ArrayList<EventPort> ineventports = new ArrayList<EventPort>();	
		if(processortype.getOwnedEventPorts() != null){
			EList<EventPort> eventports = processortype.getOwnedEventPorts();	    
			if(eventports.size()>0){
				Iterator<?> it = eventports.iterator();
				while (it.hasNext())
				{
					EventPort eventPort = (EventPort) it.next();
					if (isInputPort(eventPort)) ineventports.add(eventPort);
				}
			}
		}

		return ineventports;
	}


	/**
	 * Get a list of out event ports of a ProcessorType (processortype)
	 * 
	 * @param processortype a processor type component
	 * @return An ArratList of EventPort objects
	 */
	public ArrayList<EventPort> Processor_GetOutEventPort(ProcessorType processortype){	

		ArrayList<EventPort> outeventports = new ArrayList<EventPort>();	
		if(processortype.getOwnedEventPorts() != null){
			EList<EventPort> eventports = processortype.getOwnedEventPorts();	    
			if(eventports.size()>0){
				Iterator<?> it = eventports.iterator();
				while (it.hasNext())
				{
					EventPort eventPort = (EventPort) it.next();
					if (isOutputPort(eventPort)) outeventports.add(eventPort);
				}
			}    
		}
		return outeventports;
	}


	/**
	 * Get a list of in event data ports of a ProcessorType (processortype)
	 * 
	 * @param processortype a processor type component
	 * @return An ArratList of EventDataPort objects
	 */
	public ArrayList<EventDataPort> Processor_GetInEventDataPort(ProcessorType processortype){	

		ArrayList<EventDataPort> ineventdataports = new ArrayList<EventDataPort>();	
		if(processortype.getOwnedEventDataPorts() != null){		
			EList<EventDataPort> eventdataports = processortype.getOwnedEventDataPorts();    
			if(eventdataports.size()>0) {
				Iterator<?> it = eventdataports.iterator();
				while (it.hasNext())
				{
					EventDataPort eventdataPort = (EventDataPort) it.next();
					if (isInputPort(eventdataPort)) ineventdataports.add(eventdataPort);
				}
			}
		}
		return ineventdataports;
	}


	/**
	 * Get a list of out event data ports of a ProcessorType (processortype)
	 * 
	 * @param processortype a processor type component
	 * @return An ArratList of EventDataPort objects
	 */
	public ArrayList<EventDataPort> Processor_GetOutEventDataPort(ProcessorType processortype){	

		ArrayList<EventDataPort> outeventdataports = new ArrayList<EventDataPort>();	
		if(processortype.getOwnedEventDataPorts() != null){

			EList<EventDataPort> eventdataports = processortype.getOwnedEventDataPorts();	    
			if(eventdataports.size()>0){
				Iterator<?> it = eventdataports.iterator();
				while (it.hasNext())
				{
					EventDataPort eventdataPort = (EventDataPort) it.next();
					if (isOutputPort(eventdataPort)) outeventdataports.add(eventdataPort);
				}
			}	    
		}
		return outeventdataports;
	}


	/**
	 * Returns all subcomponents of a processor implementation
	 * @param pi processor implementation
	 * @return
	 */
	public ArrayList<Subcomponent> Processor_GetSubcomponents(ProcessorImplementation pi){

		ArrayList<Subcomponent> list = new ArrayList<Subcomponent> ();
		EList<Subcomponent> list1 = pi.getOwnedSubcomponents();
		if (list1 != null) if (list1.size()>0) list.addAll(list1);
		return list;
	}



	/**
	 * Returns all memory subcomponents of a processor implementation
	 * @param pi processor implementation
	 * @return
	 */
	public ArrayList<MemorySubcomponent> Processor_GetSubMemorys(ProcessorImplementation pi){

		ArrayList<MemorySubcomponent> list = new ArrayList<MemorySubcomponent> ();	
		EList<MemorySubcomponent> list1 = pi.getOwnedMemorySubcomponents();
		if(list1 != null) if(list1.size()>0) list.addAll(list1);
		return list;
	}


	/**
	 * Returns all bus subcomponents of a processor implementation
	 * @param pi processor implementation
	 * @return
	 */
	public ArrayList<BusSubcomponent> Processor_GetSubBus(ProcessorImplementation pi){

		ArrayList<BusSubcomponent> list = new ArrayList<BusSubcomponent> ();	
		EList<BusSubcomponent> list1 = pi.getOwnedBusSubcomponents();
		if(list1 != null) if(list1.size()>0) list.addAll(list1);
		return list;
	}


	/**
	 * Returns a list of bus access that declared in a bus type (bt)
	 * 
	 * @param bt a bus component type
	 * @return a list of bus access
	 */
	public ArrayList<BusAccess> Bus_GetBusAccess(BusType bt){

		ArrayList<BusAccess> list = new ArrayList<BusAccess>();
		EList<BusAccess> list1 = bt.getOwnedBusAccesses();
		if (list1 != null) if (list1.size()>0) list.addAll(list1);
		return list;
	}



	/**
	 * Get a list of out event ports of a SubprogramType (subp_type)
	 * 
	 * @param subp_type a subprogram type component
	 * @return An ArratList of EventPort objects
	 */
	public ArrayList<EventPort> Subprogram_GetOutEventPort(SubprogramType subp_type){	

		ArrayList<EventPort> outeventports = new ArrayList<EventPort>();	
		if (subp_type.getOwnedEventPorts() != null){
			EList<EventPort> eventports = subp_type.getOwnedEventPorts();

			if(eventports.size()>0){
				Iterator<?> it = eventports.iterator();
				while (it.hasNext())
				{
					EventPort eventPort = (EventPort) it.next();
					if (isOutputPort(eventPort)) outeventports.add(eventPort);
				}
			}
		}
		return outeventports;
	}


	/**
	 * Get a list of out event data ports of a SubprogramType (subp_type)
	 * 
	 * @param subp_type a subprogram type component
	 * @return An ArratList of EventDataPort objects
	 */
	public ArrayList<EventDataPort> Subprogram_GetOutEventDataPort(SubprogramType subp_type){	

		ArrayList<EventDataPort> outeventdataports = new ArrayList<EventDataPort>();	
		if (subp_type.getOwnedEventDataPorts() != null){
			EList<EventDataPort> eventdataports = subp_type.getOwnedEventDataPorts();

			if(eventdataports.size()>0){
				Iterator<?> it = eventdataports.iterator();
				while (it.hasNext())
				{
					EventDataPort eventdataPort = (EventDataPort) it.next();
					if (isOutputPort(eventdataPort)) outeventdataports.add(eventdataPort);
				}
			}
		}
		return outeventdataports;
	}



	/**
	 * Get a list of requires subprogram access a ThreadType (threadtype)
	 * 
	 * @param threadtype a thread type component
	 * @return An ArratList of subprogram access objects
	 */
	public ArrayList<SubprogramAccess> Thread_GetRequiresSubprogramAccess(ThreadType threadtype){

		ArrayList<SubprogramAccess> list1 = new ArrayList<SubprogramAccess>();	
		if(threadtype.getOwnedSubprogramAccesses() != null){		
			EList<SubprogramAccess> list = threadtype.getOwnedSubprogramAccesses();
			if(list.size() >0 ){
				Iterator<?> it1 = list.iterator();
				while (it1.hasNext())
				{
					SubprogramAccess x = (SubprogramAccess) it1.next();
					if (x.getKind().toString()  == "requires") list1.add(x);
				}
			}
		}
		return list1;
	}


	/**
	 * Get the subprogram access of subprogram type or implementation x
	 * @param t a thread type
	 * @param x a string name of a subprogram (type of implementation)
	 * @return subprogram access
	 */
	public SubprogramAccess ThreadType_Get_SubprogramAccess_x(ThreadType t, String x)
	{
		ArrayList<SubprogramAccess> list1 = Thread_GetRequiresSubprogramAccess(t);

		if (list1.size() > 0){
			for(int i=0; i< list1.size(); i++){
				SubprogramAccess spa1 = list1.get(i);
				SubprogramClassifier spc1 = (SubprogramClassifier) spa1.getClassifier();
				String str;
				if(spc1 instanceof SubprogramType){
					SubprogramType spc2 = (SubprogramType) spc1;
					str = spc2.getName();
				}
				else{
					SubprogramImplementation spc3 = (SubprogramImplementation) spc1;
					str = spc3.getName();
				}
				if(str.equals(x)) return spa1;
			}
		}
		return null;
	}


	/**
	 * Get the  subprogram call of which the called subprogram is named as x
	 * @param t a thread implementation
	 * @param x a string name of the called subprogram (type of implementation)
	 * @return a subprogram call
	 */
	public SubprogramCall Thread_Get_SubprogramCall_CalledSubprogram_x(ThreadImplementation t, String x)
	{
		ArrayList<SubprogramCall> list1 = Thread_Get_SubprogramCalls(t);

		if (list1.size() > 0){
			for(int i=0; i< list1.size(); i++){
				SubprogramCall sp1 = list1.get(i);
				CalledSubprogram csp1 = sp1.getCalledSubprogram();

				String str=null;
				if(csp1 instanceof SubprogramType){
					SubprogramType spc2 = (SubprogramType) csp1;
					str = spc2.getName();
				}
				else if(csp1 instanceof SubprogramImplementation){
					SubprogramImplementation spc3 = (SubprogramImplementation) csp1;
					str = spc3.getName();
				}

				if(str.equals(x)) return sp1;
			}
		}
		return null;
	}




	/**
	 * Returns whether a thread contains ports
	 * 
	 * @param ttype a thread type
	 * @return boolean
	 */
	public boolean Thread_HasPorts(ThreadType ttype){

		if(ttype.getOwnedDataPorts() != null ){
			if (Thread_GetInDataPort(ttype).size()>0 | Thread_GetOutDataPort(ttype).size()>0) return true;
		}
		if(ttype.getOwnedEventPorts() != null ){
			if(Thread_GetInEventPort(ttype).size()>0 | Thread_GetOutEventPort(ttype).size()>0) return true;
		}
		if (ttype.getOwnedEventDataPorts() != null ){
			if(Thread_GetInEventDataPort(ttype).size()>0 | Thread_GetOutEventDataPort(ttype).size()>0) return true;
		}
		return false;
	}



	/**
	 * Get a list of requires data access of a ThreadType (threadtype)
	 * 
	 * @param threadtype a thread type component
	 * @return An ArratList of DataAccess objects
	 */
	public ArrayList<DataAccess> Thread_GetRequiresDataAccess(ThreadType threadtype){

		ArrayList<DataAccess> list1 = new ArrayList<DataAccess>();	
		if(threadtype.getOwnedDataAccesses() != null){		
			EList<DataAccess> list = threadtype.getOwnedDataAccesses();
			if(list.size() >0 ) {
				Iterator<?> it1 = list.iterator();
				while (it1.hasNext())
				{
					DataAccess x = (DataAccess) it1.next();
					if (x.getKind().toString()  == "requires") list1.add(x);
				}
			}
		}
		return list1;
	}



	/**
	 * Get a list of in data ports of a ThreadType (threadtype)
	 * 
	 * @param threadtype a thread type component
	 * @return An ArratList of DataPort objects
	 */
	public ArrayList<DataPort> Thread_GetInDataPort(ThreadType threadtype){

		ArrayList<DataPort> indataports = new ArrayList<DataPort>();
		if(threadtype.getOwnedDataPorts() != null ){

			EList<DataPort> dataports = threadtype.getOwnedDataPorts();  	
			if(dataports.size() >0 ){
				Iterator<?> it_test = dataports.iterator();
				while (it_test.hasNext())
				{
					DataPort dataPort = (DataPort) it_test.next();
					if (isInputPort(dataPort)) indataports.add(dataPort);
				}
			}
		}

		return indataports;
	}


	/**
	 * Get a list of out data ports of a ThreadType (threadtype)
	 * 
	 * @param threadtype a thread type component
	 * @return An ArratList of DataPort objects
	 */
	public ArrayList<DataPort> Thread_GetOutDataPort(ThreadType threadtype){	

		ArrayList<DataPort> outdataports = new ArrayList<DataPort>();   
		if(threadtype.getOwnedDataPorts() != null){
			EList<DataPort> dataports = threadtype.getOwnedDataPorts();		
			if(dataports.size() >0 ){
				Iterator<?> it = dataports.iterator();
				while (it.hasNext())
				{
					DataPort dataPort = (DataPort) it.next();
					if (isOutputPort(dataPort)) outdataports.add(dataPort);
				}
			}
		}
		return outdataports;
	}


	/**
	 * Get a list of in event ports of a ThreadType (threadtype)
	 * 
	 * @param threadtype a thread type component
	 * @return An ArratList of EventPort objects
	 */
	public ArrayList<EventPort> Thread_GetInEventPort(ThreadType threadtype){	

		ArrayList<EventPort> ineventports = new ArrayList<EventPort>();   
		if(threadtype.getOwnedEventPorts() != null){
			EList<EventPort> eventports = threadtype.getOwnedEventPorts();
			if(eventports.size() >0 ){
				for(int i =0; i< eventports.size(); i++){
					EventPort eventPort = (EventPort) eventports.get(i);
					if (isInputPort(eventPort)) ineventports.add(eventPort);
				}
			}
		}
		return ineventports;
	}


	/**
	 * Get a list of out event ports of a ThreadType (threadtype)
	 * 
	 * @param threadtype a thread type component
	 * @return An ArratList of EventPort objects
	 */
	public ArrayList<EventPort> Thread_GetOutEventPort(ThreadType threadtype){	

		ArrayList<EventPort> outeventports = new ArrayList<EventPort>();    
		if(threadtype.getOwnedEventPorts() != null){
			EList<EventPort> eventports = threadtype.getOwnedEventPorts();	
			if(eventports.size()>0){
				Iterator<?> it = eventports.iterator();
				while (it.hasNext())
				{
					EventPort eventPort = (EventPort) it.next();
					if (isOutputPort(eventPort))  outeventports.add(eventPort);
				}
			}   
		}

		return outeventports;
	}


	/**
	 * Get a list of in event data ports of a ThreadType (threadtype)
	 * 
	 * @param threadtype a thread type component
	 * @return An ArratList of EventDataPort objects
	 */
	public ArrayList<EventDataPort> Thread_GetInEventDataPort(ThreadType threadtype){	

		ArrayList<EventDataPort> ineventdataports = new ArrayList<EventDataPort>();   
		if(threadtype.getOwnedEventDataPorts() != null){
			EList<EventDataPort> eventdataports = threadtype.getOwnedEventDataPorts(); 	
			if(eventdataports.size() >0 ){
				Iterator<?> it = eventdataports.iterator();
				while (it.hasNext())
				{
					EventDataPort eventdataPort = (EventDataPort) it.next();
					if (isInputPort(eventdataPort)) ineventdataports.add(eventdataPort);
				}
			}
		}

		return ineventdataports;
	}


	/**
	 * Get a list of out event data ports of a ThreadType (threadtype)
	 * 
	 * @param threadtype a thread type component
	 * @return An ArratList of EventDataPort objects
	 */
	public ArrayList<EventDataPort> Thread_GetOutEventDataPort(ThreadType threadtype){	

		ArrayList<EventDataPort> outeventdataports = new ArrayList<EventDataPort>();  
		if(threadtype.getOwnedEventDataPorts() != null){
			EList<EventDataPort> eventdataports = threadtype.getOwnedEventDataPorts();  	
			if(eventdataports.size() >0){
				Iterator<?> it = eventdataports.iterator();
				while (it.hasNext())
				{
					EventDataPort eventdataPort = (EventDataPort) it.next();
					if (isOutputPort(eventdataPort)) outeventdataports.add(eventdataPort);
				}
			}
		}

		return outeventdataports;
	}


	/**
	 * Get the corresponding thread type of a ThreadImpl (threadimpl)
	 * 
	 * @param threadimpl a thread implementation component
	 * @return a ThreadType object
	 */
	public ThreadType Thread_GetCompType(ThreadImplementation threadimpl)
	{
		return threadimpl.getType();	
	}


	/**
	 * Get a list of sub data components of a ThreadImpl (threadimpl)
	 * 
	 * @param threadimpl a thread implementation component
	 * @return An ArrayList of DataSubcomponent objects
	 */
	public ArrayList<DataSubcomponent> Thread_GetSubcomponentsData(ThreadImplementation threadimpl){		

		ArrayList<DataSubcomponent> list_s = new ArrayList<DataSubcomponent>();
		EList<DataSubcomponent> subs = threadimpl.getOwnedDataSubcomponents();

		if (subs != null){
			if(subs.size()>0)
			{ list_s.addAll(subs);}
		}

		return list_s;	
	}


	/**
	 * Get a list of sub subprogram components of a ThreadImpl (threadimpl)
	 * 
	 * @param threadimpl a thread implementation component
	 * @return An ArrayList of SubprogramSubcomponent objects
	 */
	public ArrayList<SubprogramSubcomponent> Thread_GetSubcomponent_Subprograms(ThreadImplementation threadimpl){		

		ArrayList<SubprogramSubcomponent> list_s = new ArrayList<SubprogramSubcomponent>();
		EList<SubprogramSubcomponent> subs = threadimpl.getOwnedSubprogramSubcomponents();

		if (subs != null){
			if(subs.size()>0)
			{ list_s.addAll(subs);}
		}

		return list_s;	
	}


	/**
	 * Get a list of SubprogramCall objects from callSequences of a ThreadImpl (threadimpl)
	 * 
	 * @param threadimpl a thread implementation component
	 * @return An ArrayList of SubprogramCall objects
	 */
	public ArrayList<SubprogramCall> Thread_Get_SubprogramCalls(ThreadImplementation threadimpl){		

		ArrayList<SubprogramCall> list = new ArrayList<SubprogramCall>();
		ArrayList<SubprogramCallSequence> list1 = BehavioredImpl_Get_CallSequences(threadimpl);

		for(int i=0; i< list1.size(); i++){
			SubprogramCallSequence sp_seq1 = list1.get(i);
			ArrayList<SubprogramCall> list2 = SubprogramCallSequence_Get_SubprogramCalls(sp_seq1);
			if(list2.size()>0){
				list.addAll(list2);
			}
		}

		return list;
	}


	/**
	 * Get a list of CalledSubprogram objects from a ThreadImpl (threadimpl)
	 * 
	 * @param threadimpl a thread implementation component
	 * @return An ArrayList of CalledSubprogram objects
	 */
	public ArrayList<CalledSubprogram> Thread_Get_CalledSubprograms(ThreadImplementation threadimpl){		

		ArrayList<CalledSubprogram> list = new ArrayList<CalledSubprogram>();	
		ArrayList<SubprogramCallSequence> callseqs = BehavioredImpl_Get_CallSequences(threadimpl);

		for(int i=0; i< callseqs.size(); i++){
			SubprogramCallSequence callseq1 = callseqs.get(i);
			ArrayList<SubprogramCall> list1 = SubprogramCallSequence_Get_SubprogramCalls(callseq1);
			for(int j=0; j< list1.size(); j++){
				SubprogramCall call1 = list1.get(j);
				CalledSubprogram sp = call1.getCalledSubprogram();
				list.add(sp);
			}
		}
		return list;
	} 


	/**
	 * Get the property association (which is named as X) of a thread t
	 * 
	 * @param t a thread implementation that contains this port p
	 * @param X the property name
	 * @return
	 */
	public PropertyAssociation Thread_Get_PropertyX(ThreadImplementation t, String X){

		ArrayList<PropertyAssociation> list2 = ComponentImpl_GetPropertyAssociations(t);
		ArrayList<PropertyAssociation> list3 = Component_GetPropertyAssociations(t.getType());
		if(list3.size()>0){
			list2.addAll(list3);
		}

		for(int i =0; i< list2.size(); i++){
			PropertyAssociation pa = list2.get(i);
			if(pa != null){
				//Trace("\n ****** PropertyAssociation = " + pa.toString());
				String propertyDef_name = Property_Get_Name(pa);

				if (propertyDef_name != null && propertyDef_name.equals(X)) {
					return pa;
				}
			}
		}

		Trace("\n ****** Thread_Get_PropertyX() " + X + " is null ");
		return null;
	}



	/**
	 * Get the Dispatch type ("Periodic", "Aperiodic" ... ) of a ThreadImpl (threadimpl)
	 * <p>
	 * Get the value from "Dispatch_Protocol" property
	 * 
	 * @param threadimpl a thread implementation component
	 * @return a String that indicates the dispatch type
	 */
	public String Thread_Get_DispatchProtocol_str(ThreadImplementation timpl)
	{
		/*	String dispatch = "Periodic";

	PropertyAssociation pa = Thread_Get_PropertyX(timpl, "Dispatch_Protocol");
	if(pa != null){
		dispatch = Property_Get_DispatchProtocol_str(pa);
	}

	return dispatch;
		 */

		PropertyAssociation pa = Thread_Get_PropertyX(timpl, "Dispatch_Protocol");
		if(pa != null){
			String dispatch = Property_Get_DispatchProtocol_str(pa);
			return dispatch;
		}

		return null;
	}


	/**
	 * Get the period of a ThreadImpl (threadimpl)
	 * <p>
	 * Get the value from "Period" property, for example, 2 ms
	 * 
	 * @param threadimpl a thread implementation component
	 * @return a IntegerLiteral that indicates the period
	 */
	public IntegerLiteral Thread_GetPeriod(ThreadImplementation timpl)
	{
		IntegerLiteral int_value;
		PropertyAssociation pa = Thread_Get_PropertyX(timpl, "Period");
		if(pa != null){
			int_value = Property_Get_Period(pa);   
			return int_value;
		}

		return null;
	}


	/**
	 * Get the deadline of a ThreadImpl (threadimpl)
	 * <p>
	 * Get the value from "Deadline" property, for example, 2 ms
	 * 
	 * @param threadimpl a thread implementation component
	 * @return a IntegerLiteral that indicates the deadline
	 */
	public IntegerLiteral Thread_GetDeadline(ThreadImplementation timpl)
	{
		IntegerLiteral int_value;
		PropertyAssociation pa = Thread_Get_PropertyX(timpl, "Deadline");
		if(pa != null){
			int_value = Property_Get_Deadline(pa);   
			return int_value;
		}

		return null;
	}


	/**
	 * Get the corresponding data type of a DataImpl (dataimpl)
	 * 
	 * @param dataimpl a data implementation component
	 * @return a DataType object
	 */
	public DataType Data_GetCompType(DataImplementation dataimpl)
	{
		return dataimpl.getType();}


	/**
	 * Get a list of sub data components of a DataImplementation (dataimpl)
	 * 
	 * @param dataimpl a data implementation component
	 * @return An ArratList of DataSubcomponent objects
	 */
	public ArrayList<DataSubcomponent> Data_GetSubcomponentsData(DataImplementation dataimpl)
	{
		ArrayList<DataSubcomponent> list_s = new ArrayList<DataSubcomponent>();

		if(dataimpl.getOwnedDataSubcomponents() != null){
			EList<DataSubcomponent> subs = dataimpl.getOwnedDataSubcomponents();		
			if(subs.size()>0){
				Iterator<?> it = subs.iterator();
				while (it.hasNext())
				{
					DataSubcomponent s = (DataSubcomponent) it.next();
					list_s.add(s);
				}
			}	    
		}

		return list_s;	
	}


	/**
	 * Returns the string name of a data access
	 * @param da a data access
	 * @return string name
	 */
	public String DataAccess_GetName(DataAccess da){
		return da.getName();
	}


	/**
	 * Returns data access kind: "provided" "required"
	 * @param da
	 * @return
	 */
	public String DataAccess_GetKind(DataAccess da){
		return da.getKind().toString();
	}


	/**
	 * Returns the string name of a subprogram call
	 * @param da a subprogram call
	 * @return string name
	 */
	public String SubprogramCall_GetName(SubprogramCall da){
		return da.getName();
	}

	/**
	 * Returns the string name of a subprogram access
	 * @param da a subprogram access
	 * @return string name
	 */
	public String SubprogramAccess_GetName(SubprogramAccess da){
		return da.getName();
	}


	/**
	 * Returns subprogram access kind: "provided" "required"
	 * @param da
	 * @return
	 */
	public String SubprogramAccess_GetKind(SubprogramAccess da){
		return da.getKind().toString();
	}


	/**
	 *  Returns subprogram classfier of a subprogram access (spa)
	 *  
	 * @param spa a subprogram access
	 * @return its classifier
	 */
	public SubprogramClassifier SubprogramAccess_GetClassifier(SubprogramAccess spa){
		return (SubprogramClassifier) spa.getClassifier();
	}


	/**
	 * Get the component type (the classifier) of a SystemSubcomponent (syssubcomponent)
	 * <p>
	 * If classifier of this syssubcomponent is type of systemImpl, return its component type, 
	 * otherwise if this syssubcomponent is type of systemType, return this classifier 
	 * 
	 * @param syssubcomponent a sub system component
	 * @return a SystemType object
	 */
	public SystemType Systemsubcomponent_GetComponentType(SystemSubcomponent syssubcomponent)
	{	
		ComponentClassifier cc = syssubcomponent.getClassifier();
		if(cc instanceof SystemType){
			return (SystemType) cc;
		}
		else{
			SystemImplementation si = (SystemImplementation) cc;
			return si.getType();
		}
	}


	/**
	 * Get the component implementation (the classifier) of a SystemSubcomponent (syssubcomponent)
	 * <p>
	 * If classifier of this syssubcomponent is type of systemImpl, return it, 
	 * otherwise return null
	 * 
	 * @param syssubcomponent a sub system component
	 * @return a SystemImpl object
	 */
	public SystemImplementation Systemsubcomponent_GetComponentImpl(SystemSubcomponent syssubcomponent)
	{	
		ComponentClassifier cc = syssubcomponent.getClassifier();
		if(cc instanceof SystemImplementation){
			return (SystemImplementation) cc;
		}

		return null;
	}


	/**
	 * get the SystemClassifier of a SystemSubcomponent (s)
	 * 
	 * @param s a sub system component
	 * @return a SystemClassifier object 
	 */
	public SystemClassifier Systemsubcomponent_GetClassifier(SystemSubcomponent s){	
		return (SystemClassifier) s.getClassifier();
	}


	/**
	 * Get the component type (the classifier) of a ProcessSubcomponent (processsubcomponent)
	 * <p>
	 * If classifier of this processsubcomponent is type of ProcessImpl, return its component type, 
	 * otherwise if this processsubcomponent is type of ProcessType, return this classifier
	 * 
	 * @param processsubcomponent a sub process component
	 * @return a ProcessType object
	 */
	public ProcessType Processsubcomponent_GetComponentType(ProcessSubcomponent processsubcomponent)
	{	
		ComponentClassifier cc = processsubcomponent.getClassifier();
		if(cc instanceof ProcessType){
			return (ProcessType) cc;
		}
		else{
			ProcessImplementation pi = (ProcessImplementation) cc;
			return pi.getType();
		}
	}


	/**
	 * Get the component implementation (the classifier) of a ProcessSubcomponent (processsubcomponent)
	 * <p>
	 * If classifier of this processsubcomponent is type of  ProcessImpl, return this classifier, 
	 * otherwise return null
	 * 
	 * @param processsubcomponent a sub process component
	 * @return a ProcessType object
	 */
	public ProcessImplementation Processsubcomponent_GetComponentImpl(ProcessSubcomponent processsubcomponent)
	{	
		ComponentClassifier cc = processsubcomponent.getClassifier();
		if(cc instanceof ProcessImplementation){
			return (ProcessImplementation) cc;
		}

		return null;
	}


	/**
	 * Get the ProcessClassifier of a ProcessSubcomponent (p)
	 * 
	 * @param p a sub process component
	 * @return a ProcessClassifier object 
	 */
	public ProcessClassifier Processsubcomponent_GetClassifier(ProcessSubcomponent p){	
		return (ProcessClassifier) p.getClassifier();
	}


	/**
	 * Get the component type (the classifier) of a ThreadSubcomponent (t)
	 * <p>
	 * If classifier of this t is type of ThreadImpl, return its component type, 
	 * otherwise if this t is type of ThreadType, return this classifier
	 * 
	 * @param t a sub thread component
	 * @return a ThreadType object
	 */
	public ThreadType Threadsubcomponent_GetComponentType(ThreadSubcomponent t)
	{	
		ComponentClassifier cc = t.getClassifier();
		if(cc instanceof ThreadType){
			return (ThreadType) cc;
		}
		else{
			ThreadImplementation ti = (ThreadImplementation) cc;
			return ti.getType();
		}
	}


	/**
	 * Get the component implementation (the classifier) of a ThreadSubcomponent (threadsubcomponent)
	 * <p>
	 * If classifier of this threadsubcomponent is type of  ThreadImpl, return this classifier, 
	 * otherwise return null
	 * 
	 * @param threadsubcomponent a sub Thread component
	 * @return a ThreadType object
	 */
	public ThreadImplementation Threadsubcomponent_GetComponentImpl(ThreadSubcomponent threadsubcomponent)
	{	
		ComponentClassifier cc = threadsubcomponent.getClassifier();
		if(cc instanceof ThreadImplementation) return (ThreadImplementation) cc;
		return null;
	}


	/**
	 * Get the ThreadClassifier of a ThreadSubcomponent (threadsubcomponent)
	 * 
	 * @param threadsubcomponent a sub thread component
	 * @return a ThreadClassifier object 
	 */
	public ThreadClassifier Threadsubcomponent_GetClassifier(ThreadSubcomponent threadsubcomponent){	
		// return (ThreadClassifier) threadsubcomponent.getClassifier();
		ComponentClassifier cc = threadsubcomponent.getClassifier();
		if(cc instanceof AbstractTypeImpl) return null;
		return  (ThreadClassifier) cc;
	}


	/**
	 * Get the ThreadGroupClassifier of a ThreadGroupSubcomponent (tg)
	 * 
	 * @param tg a sub thread group component
	 * @return a ThreadGroupClassifier object 
	 */
	public ThreadGroupClassifier ThreadGroupsubcomponent_GetClassifier(ThreadGroupSubcomponent tg){	
		return (ThreadGroupClassifier) tg.getClassifier();
	}


	/**
	 * Get the DeviceClassifier of a DeviceSubcomponent (d)
	 * 
	 * @param d a sub device component
	 * @return a DeviceClassifier object 
	 */
	public DeviceClassifier Devicesubcomponent_GetClassifier(DeviceSubcomponent d){	
		return (DeviceClassifier) d.getClassifier();
	}


	/**
	 * Get the MemoryClassifier of a MemorySubcomponent (m)
	 * 
	 * @param m a sub memory component
	 * @return a MemoryClassifier object 
	 */
	public MemoryClassifier Memorysubcomponent_GetClassifier(MemorySubcomponent m){	
		return (MemoryClassifier) m.getClassifier();
	}


	/**
	 * Get the ProcessorClassifier of a ProcessorSubcomponent (pr)
	 * 
	 * @param pr a sub processor component
	 * @return a ProcessorClassifier object 
	 */
	public ProcessorClassifier Processorsubcomponent_GetClassifier(ProcessorSubcomponent pr){	
		return (ProcessorClassifier) pr.getClassifier();
	}


	/**
	 * Get the BusClassifier of a BusSubcomponent (bus)
	 * 
	 * @param bus a sub bus component
	 * @return a BusClassifier object 
	 */
	public BusClassifier Bussubcomponent_GetClassifier(BusSubcomponent bus){	
		return (BusClassifier) bus.getClassifier();
	}


	/**
	 * Get the SubprogramType of a SubprogramSubcomponent (s)
	 * <p>
	 * If classifier of this s is type of SubprogramImpl, return its component type, 
	 * otherwise if this s is type of SubprogramType, return this classifier
	 * 
	 * @param s a sub subprogram component
	 * @return a SubprogramType object 
	 */
	public SubprogramType Subprogramsubcomponent_GetComponentType(SubprogramSubcomponent s)
	{	
		ComponentClassifier cc = s.getClassifier();
		if(cc instanceof SubprogramType){
			return (SubprogramType) cc;
		}
		else{
			SubprogramImplementation ti = (SubprogramImplementation) cc;
			return ti.getType();
		}
	}


	/**
	 * Get the component implementation (the classifier) of a SubprogramSubcomponent (s)
	 * <p>
	 * If classifier of this s is type of  SubprogramImpl, return this classifier, 
	 * otherwise return null
	 * 
	 * @param s a sub SubprogramImpl component
	 * @return a SubprogramImpl object
	 */
	public SubprogramImplementation Subprogramsubcomponent_GetComponentImpl(SubprogramSubcomponent s)
	{	
		ComponentClassifier cc = s.getClassifier();
		if(cc instanceof SubprogramImplementation){
			return (SubprogramImplementation) cc;
		}

		return null;
	}


	/**
	 * Get the component classifier  of a SubprogramSubcomponent (subp)
	 * 
	 * @param subp a sub subprogram component
	 * @return a SubprogramClassifier object 
	 */
	public SubprogramClassifier Subprogramsubcomponent_GetClassifier(SubprogramSubcomponent subp){	
		return (SubprogramClassifier) subp.getClassifier();
	}


	/**
	 * Get the DataType of a DataSubcomponent (d)
	 * <p>
	 * If classifier of this d is type of DataImpl, return its component type, 
	 * otherwise if this d is type of DataType, return this classifier
	 * 
	 * @param d a sub data component
	 * @return a DataType object 
	 */
	public DataType Datasubcomponent_GetComponentType(DataSubcomponent d)
	{	
		ComponentClassifier cc = d.getClassifier();
		if(cc instanceof DataType){
			return (DataType) cc;
		}
		else{
			DataImplementation ti = (DataImplementation) cc;
			return ti.getType();
		}
	}


	/**
	 * Get the component implementation (the classifier) of a DataSubcomponent (d)
	 * <p>
	 * If classifier of this d is type of  DataImpl, return this classifier, 
	 * otherwise return null
	 * 
	 * @param d a DataSubcomponent component
	 * @return a DataImpl object
	 */
	public DataImplementation Datasubcomponent_GetComponentImpl(DataSubcomponent d)
	{	
		ComponentClassifier cc = d.getClassifier();
		if(cc instanceof DataImplementation){
			return (DataImplementation) cc;
		}

		return null;
	}


	/**
	 * Get the data classifier of a DataSubcomponent (Datasubcomponent)
	 * 
	 * @param d a sub data component
	 * @return a DataClassifier object 
	 */
	public ComponentClassifier Datasubcomponent_GetClassifier(DataSubcomponent d){	

		ComponentClassifier cc = d.getClassifier();	
		Trace(d.getName() + " Datasubcomponent_GetClassifier : " + cc);
		//Trace(d.getClassifier().toString());
		return cc;
	}


	/**
	 * Returns the reference classifier of data sub component d
	 * @param d
	 * @return
	 */
	public DataSubcomponentType DataSubcomponent_GetDataSubcomponentType(DataSubcomponent d){
		return d.getDataSubcomponentType();
	}


	/**
	 * Get the ProcessorType of a ProcessorSubcomponent (p)
	 * <p>
	 * If classifier of this p is type of ProcessorImpl, return its component type, 
	 * otherwise if this p is type of ProcessorType, return this classifier
	 * 
	 * @param p a sub processor component
	 * @return a ProcessorType object 
	 */
	public ProcessorType Processorsubcomponent_GetComponentType(ProcessorSubcomponent p)
	{	
		ComponentClassifier cc = p.getClassifier();
		if(cc instanceof ProcessorType){
			return (ProcessorType) cc;
		}
		else{
			ProcessorImplementation ti = (ProcessorImplementation) cc;
			return ti.getType();
		}
	}


	/**
	 * Get the component implementation (the classifier) of a ProcessorSubcomponent (p)
	 * <p>
	 * If classifier of this p is type of ProcessorImpl, return this classifier, 
	 * otherwise return null
	 * 
	 * @param p a ProcessorSubcomponent component
	 * @return a ProcessorImpl object
	 */
	public ProcessorImplementation Processorsubcomponent_GetComponentImpl(ProcessorSubcomponent p)
	{	
		ComponentClassifier cc = p.getClassifier();
		if(cc instanceof ProcessorImplementation){
			return (ProcessorImplementation) cc;
		}

		return null;
	}


	/**
	 * Get the DeviceType of a DeviceSubcomponent (p)
	 * <p>
	 * If classifier of this p is type of DeviceImpl, return its component type, 
	 * otherwise if this p is type of DeviceType, return this classifier
	 * 
	 * @param p a sub device component
	 * @return a DeviceType object 
	 */
	public DeviceType Devicesubcomponent_GetComponentType(DeviceSubcomponent p)
	{	
		ComponentClassifier cc = p.getClassifier();
		if(cc instanceof DeviceType){
			return (DeviceType) cc;
		}
		else{
			DeviceImplementation ti = (DeviceImplementation) cc;
			return ti.getType();
		}
	}


	/**
	 * Get the component implementation (the classifier) of a DeviceSubcomponent (p)
	 * <p>
	 * If classifier of this p is type of DeviceImpl, return this classifier, 
	 * otherwise return null
	 * 
	 * @param p a DeviceSubcomponent component
	 * @return a DeviceImpl object
	 */
	public DeviceImplementation Devicesubcomponent_GetComponentImpl(DeviceSubcomponent p)
	{	
		ComponentClassifier cc = p.getClassifier();
		if(cc instanceof DeviceImplementation){
			return (DeviceImplementation) cc;
		}

		return null;
	}


	/**
	 * Get the BusType of a BusSubcomponent (p)
	 * <p>
	 * If classifier of this p is type of BusImpl, return its component type, 
	 * otherwise if this p is type of BusType, return this classifier
	 * 
	 * @param p a sub bus component
	 * @return a BusType object 
	 */
	public BusType Bussubcomponent_GetComponentType(BusSubcomponent p)
	{	
		ComponentClassifier cc = p.getClassifier();
		if(cc instanceof BusType){
			return (BusType) cc;
		}
		else{
			BusImplementation ti = (BusImplementation) cc;
			return ti.getType();
		}
	}


	/**
	 * Get the component implementation (the classifier) of a BusSubcomponent (p)
	 * <p>
	 * If classifier of this p is type of BusImpl, return this classifier, 
	 * otherwise return null
	 * 
	 * @param p a BusSubcomponent component
	 * @return a BusImpl object
	 */
	public BusImplementation Bussubcomponent_GetComponentImpl(BusSubcomponent p)
	{	
		ComponentClassifier cc = p.getClassifier();
		if(cc instanceof BusImplementation){
			return (BusImplementation) cc;
		}

		return null;
	}


	/**
	 * Get the value of a IO_Time_Spec property
	 * 
	 * @param pa a property association
	 * @return a list of String, which includes the min offset, max offset, unit1, unit2 and reference time
	 */
	public EList<String> Get_IOTimeSpec(PropertyAssociation  pa){
		return null;
	}


	/**
	 * Returns data classfier of parameter p
	 * @param p
	 * @return
	 */
	public DataClassifier Parameter_GetClassifier(Parameter p){
		return (DataClassifier) p.getClassifier();
	}


	/**
	 * Get the name of a parameter
	 * 
	 * @param p a parameter
	 * @return a string of parameter name
	 */
	public String Parameter_GetName(Parameter p){
		return p.getName();
	}


	/**
	 * Get the name of a port
	 * 
	 * @param port a (data, event or event data) port
	 * @return a string of data name
	 */
	public String Port_GetName(Port port){
		return port.getName();
	}


	/**
	 * Returns the direction of a port p: "in", "out"
	 * @param p a port
	 * @return
	 */
	public String Port_GetDirection(Port p){
		return p.getDirection().toString();
	}


	/**
	 * returns the data classifier of a data port
	 * @param dport
	 * @return
	 */
	public DataClassifier DataPort_GetClassifier(DataPort dport){
		return (DataClassifier) dport.getClassifier();
	}


	/**
	 * returns the data classifier of a event data port
	 * @param edport
	 * @return
	 */
	public DataClassifier EventDataPort_GetClassifier(EventDataPort edport){
		return (DataClassifier) edport.getClassifier();
	}

	/**
	 * Returns a list of property associations of a port
	 * @param p
	 * @return
	 */
	public ArrayList<PropertyAssociation> Port_Get_PropertyAssociation(Port p){
		ArrayList<PropertyAssociation> list = new ArrayList<PropertyAssociation>();
		EList<PropertyAssociation> list1 = p.getOwnedPropertyAssociations();
		if(list1 != null){
			if(list1.size()>0){
				list.addAll(list1);
			}
		}
		return list;
	}


	/**
	 * Get the property association (which is named as X) of a port p
	 * 
	 * @param t a thread implementation that contains this port p
	 * @param p a port
	 * @return
	 */
	public PropertyAssociation Port_Get_PropertyX(ThreadImplementation t, Port p, String X){

		ArrayList<PropertyAssociation> list1 = Port_Get_PropertyAssociation(p);
		for(int i =0; i< list1.size(); i++){
			PropertyAssociation pa = list1.get(i);
			String propertyDef_name = Property_Get_Name(pa);	
			if (propertyDef_name.equals(X)) {
				return pa;
			}
		}

		ArrayList<PropertyAssociation> list2 = ComponentImpl_GetPropertyAssociations(t);
		ArrayList<PropertyAssociation> list3 = Component_GetPropertyAssociations(t.getType());
		if(list3.size()>0){
			list2.addAll(list3);
		}

		for(int j =0; j< list2.size(); j++){
			PropertyAssociation pa = list2.get(j);
			String propertyDef_name = Property_Get_Name(pa);	    
			if (propertyDef_name.equals(X)) {

				EList<ContainedNamedElement> list_applyto = pa.getAppliesTos();
				if(list_applyto !=null){
					for(int n =0; n<list_applyto.size(); n++){
						ContainedNamedElement cne = list_applyto.get(n);
						EList<ContainmentPathElement> cpe_list = cne.getContainmentPathElements();
						if(cpe_list != null){
							for(int k=0; k<cpe_list.size(); k++){
								ContainmentPathElement cpe = cpe_list.get(k);
								String applyto_name = cpe.getNamedElement().getName();

								//if Input_Time_Offset applies to p
								if (applyto_name == p.getName()){
									return pa;
								}
							}
						}
					}
				}
			}
		}

		return null;
	}


	/**
	 * Get the Input_Time property association of a port p
	 * 
	 * @param t a thread implementation that contains this port p
	 * @param p a port
	 * @return
	 */
	public PropertyAssociation Port_Get_InputTime(ThreadImplementation t, Port p){

		return Port_Get_PropertyX(t, p, "Input_Time");
	}



	/**
	 * Get the (first) Reference_Time of a Input_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return a string value of Reference_Time 
	 */
	public String Port_Get_InputTime_Time1_str(ThreadImplementation t, Port p){

		String ref_time = "Dispatch";

		PropertyAssociation pa = Port_Get_InputTime(t,p);
		if(pa != null){

			Trace("Input_Time Time1 != null");
			EnumerationLiteral time1 =  InputTime_Get_Time1(pa);
			if(time1 !=null ){
				ref_time = time1.getName();
				Trace("ref_time = " + ref_time);
			}
		}

		return ref_time;
	}



	/**
	 * Get the min_offset's unit of Offset field in a Input_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the unit of the min_offset of port p
	 */
	public String Port_Get_InputTime_Offset1_MinUnit_str(ThreadImplementation t, Port p){

		String min_unit = "ns";

		PropertyAssociation pa = Port_Get_InputTime(t,p);
		if(pa != null){
			RangeValue range_val = InputTime_Get_Offset1(pa);
			if(range_val != null){
				min_unit = TimeRange_Get_MinUnit(range_val);
			}
		}

		return min_unit;
	}



	/**
	 * Get the min_offset's value of Offset field in a Input_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the value of the min_offset of port p
	 */
	public String Port_Get_InputTime_Offset1_MinValue_str(ThreadImplementation t, Port p){

		String min_value = "0";

		PropertyAssociation pa = Port_Get_InputTime(t,p);
		if(pa != null){
			RangeValue range_val = InputTime_Get_Offset1(pa);
			if(range_val != null){
				min_value = TimeRange_Get_MinValue(range_val);
			}
		}

		return min_value;
	}



	/**
	 * Get the min_offset's value of Offset field in a Input_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the value of the min_offset of port p
	 */
	public long Port_Get_InputTime_Offset1_MinValue_long(ThreadImplementation t, Port p){

		long min_value = 0;

		PropertyAssociation pa = Port_Get_InputTime(t,p);
		if(pa != null){
			RangeValue range_val = InputTime_Get_Offset1(pa);
			if(range_val != null){
				min_value = TimeRange_Get_MinValue_long(range_val);
			}
		}

		return min_value;
	}


	/**
	 * Get the max_offset's unit of Offset field in a Input_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the unit of the max_offset of port p
	 */
	public String Port_Get_InputTime_Offset1_MaxUnit_str(ThreadImplementation t, Port p){

		String max_unit = "ns";

		PropertyAssociation pa = Port_Get_InputTime(t,p);
		if(pa != null){

			Trace("Input_Time != null ");
			RangeValue range_val = InputTime_Get_Offset1(pa);
			if(range_val != null){
				max_unit = TimeRange_Get_MaxUnit(range_val);
				Trace("max_unit = " + max_unit);
			}
		}

		return max_unit;
	}


	/**
	 * Get the max_offset's value of Offset field in a Input_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the value of the max_offset of port p
	 */
	public String Port_Get_InputTime_Offset1_MaxValue_str(ThreadImplementation t, Port p){

		String max_value = "0";

		PropertyAssociation pa = Port_Get_InputTime(t,p);
		if(pa != null){
			RangeValue range_val = InputTime_Get_Offset1(pa);
			if(range_val != null){
				max_value = TimeRange_Get_MaxValue(range_val);
			}
		}

		return max_value;
	}


	/**
	 * Get the max_offset's value of Offset field in a Input_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the value of the max_offset of port p
	 */
	public long Port_Get_InputTime_Offset1_MaxValue_long(ThreadImplementation t, Port p){

		long max_value = 0;

		PropertyAssociation pa = Port_Get_InputTime(t,p);
		if(pa != null){
			RangeValue range_val = InputTime_Get_Offset1(pa);
			if(range_val != null){
				max_value = TimeRange_Get_MaxValue_long(range_val);
			}
		}

		return max_value;
	}


	/**
	 * Get the Output_Time property association of a port p
	 * 
	 * @param t a thread implementation that contains this port p
	 * @param p a port
	 * @return
	 */
	public PropertyAssociation Port_Get_OutputTime(ThreadImplementation t, Port p){

		return Port_Get_PropertyX(t, p, "Output_Time");
	}


	/**
	 * Get the (first) Reference_Time of a Output_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return a string value of Reference_Time 
	 */
	public String Port_Get_OutputTime_Time1_str(ThreadImplementation t, Port p){

		String ref_time = "Completion";

		PropertyAssociation pa = Port_Get_OutputTime(t,p);
		if(pa != null){
			EnumerationLiteral time1 =  OutputTime_Get_Time1(pa);
			if(time1 !=null ){
				ref_time = time1.getName();
			}
		}

		return ref_time;
	}


	/**
	 * Get the min_offset's unit of Offset field in a Output_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the unit of the min_offset of port p
	 */
	public String Port_Get_OutputTime_Offset1_MinUnit_str(ThreadImplementation t, Port p){

		String min_unit = "ns";

		PropertyAssociation pa = Port_Get_OutputTime(t,p);
		if(pa != null){
			RangeValue range_val = OutputTime_Get_Offset1(pa);
			if(range_val != null){
				min_unit = TimeRange_Get_MinUnit(range_val);
			}
		}

		return min_unit;
	}



	/**
	 * Get the min_offset's value of Offset field in a Output_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the value of the min_offset of port p
	 */
	public String Port_Get_OutputTime_Offset1_MinValue_str(ThreadImplementation t, Port p){

		String min_value = "0";

		PropertyAssociation pa = Port_Get_OutputTime(t,p);
		if(pa != null){
			RangeValue range_val = OutputTime_Get_Offset1(pa);
			if(range_val != null){
				min_value = TimeRange_Get_MinValue(range_val);
			}
		}

		return min_value;
	}


	/**
	 * Get the min_offset's value of Offset field in a Output_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the value of the min_offset of port p
	 */
	public long Port_Get_OutputTime_Offset1_MinValue_long(ThreadImplementation t, Port p){

		long min_value = 0;

		PropertyAssociation pa = Port_Get_OutputTime(t,p);
		if(pa != null){
			RangeValue range_val = OutputTime_Get_Offset1(pa);
			if(range_val != null) min_value = TimeRange_Get_MinValue_long(range_val);
		}
		return min_value;
	}


	/**
	 * Get the max_offset's unit of Offset field in a Output_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the unit of the max_offset of port p
	 */
	public String Port_Get_OutputTime_Offset1_MaxUnit_str(ThreadImplementation t, Port p){

		String max_unit = "ns";

		PropertyAssociation pa = Port_Get_OutputTime(t,p);
		if(pa != null){
			RangeValue range_val = OutputTime_Get_Offset1(pa);
			if(range_val != null) max_unit = TimeRange_Get_MaxUnit(range_val);
		}
		return max_unit;
	}


	/**
	 * Get the max_offset's value of Offset field in a Output_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the value of the max_offset of port p
	 */
	public String Port_Get_OutputTime_Offset1_MaxValue_str(ThreadImplementation t, Port p){

		String max_value = "0";

		PropertyAssociation pa = Port_Get_OutputTime(t,p);
		if(pa != null){
			RangeValue range_val = OutputTime_Get_Offset1(pa);
			if (range_val != null) max_value = TimeRange_Get_MaxValue(range_val);
		}
		return max_value;
	}


	/**
	 * Get the max_offset's value of Offset field in a Output_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the value of the max_offset of port p
	 */
	public long Port_Get_OutputTime_Offset1_MaxValue_long(ThreadImplementation t, Port p){

		long max_value = 0;

		PropertyAssociation pa = Port_Get_OutputTime(t,p);
		if(pa != null) {
			RangeValue range_val = OutputTime_Get_Offset1(pa);
			if(range_val != null) max_value = TimeRange_Get_MaxValue_long(range_val);
		}
		return max_value;
	}


	/**
	 * Get the Timing property association of a port p
	 * 
	 * @param t a thread implementation that contains this port p
	 * @param p a port
	 * @return
	 */
	public PropertyAssociation Port_Get_Timing(ThreadImplementation t, Port p){

		return Port_Get_PropertyX(t, p, "Timing");
	}


	/**
	 * Get a string value of a port p's Timing property, default value is "sampled"
	 * 
	 * @param t the thread implementation which contains this port p
	 * @param p a port
	 * @return
	 */
	public String Port_Get_Timing_str(ThreadImplementation t, Port p){

		PropertyAssociation pa = Port_Get_Timing(t,p);
		if(pa != null){
			String timing = Property_Get_Timing_str(pa);
			if(timing !=null ) return timing;
		}
		return "sampled";
	}


	/**
	 * Get the Overflow_Handling_Protocol property association of a port p
	 * 
	 * @param t a thread implementation that contains this port p
	 * @param p a port
	 * @return
	 */
	public PropertyAssociation Port_Get_OverflowHandlingProtocol(ThreadImplementation t, Port p){

		return Port_Get_PropertyX(t, p, "Overflow_Handling_Protocol");
	}


	/**
	 * Get a string value of a port p's Overflow_Handling_Protocol property, 
	 * the default value is "DropOldest"
	 * 
	 * @param t the thread implementation which contains this port p
	 * @param p a port
	 * @return
	 */
	public String Port_Get_Overflow_Handling_Protocol_str(ThreadImplementation t, Port p){

		PropertyAssociation pa = Port_Get_OverflowHandlingProtocol(t,p);
		if(pa != null) {
			String v1 = Property_Get_OverflowHandlingProtocol_str(pa);
			if(v1 !=null ) return v1;
		}
		return "DropOldest";	

	}


	/**
	 * Get the Queue_Processing_Protocol property association of a port p
	 * 
	 * @param t a thread implementation that contains this port p
	 * @param p a port
	 * @return
	 */
	public PropertyAssociation Port_Get_QueueProcessingProtocol(ThreadImplementation t, Port p){

		return Port_Get_PropertyX(t, p, "Queue_Processing_Protocol");
	}


	/**
	 * Get a string value of a port p's Queue_Processing_Protocol property, default value is "FIFO"
	 * 
	 * @param t the thread implementation which contains this port p
	 * @param p a port
	 * @return
	 */
	public String Port_Get_Queue_Processing_Protocol_str(ThreadImplementation t, Port p){

		PropertyAssociation pa = Port_Get_QueueProcessingProtocol(t,p);
		if(pa != null){
			String v1 = Property_Get_QueueProcessingProtocol_str(pa);
			if (v1 !=null) return v1;
		}
		return "Fifo";	
	}


	/**
	 * Get the Queue_Size value of (event data or event) port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the value (in string) of Queue_Size
	 */
	public String Port_Get_QueueSize_str(ThreadImplementation t, Port p){

		PropertyAssociation pa = Port_Get_PropertyX(t, p, "Queue_Size");
		if (pa != null) {
			String size = Property_Get_QueueSize_str(pa);
			if(size != null ) return size;
		}
		return "1";
	}


	/**
	 * Get the Dequeued_Items value of (event data or event) port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the value (in string) of Dequeued_Items
	 */
	public String Port_Get_DequeuedItems_str(ThreadImplementation t, Port p){

		PropertyAssociation pa = Port_Get_PropertyX(t, p, "Dequeued_Items");
		if(pa != null){
			String size = Property_Get_DequeuedItems_str(pa);
			if(size !=null ){
				return size;
			}
		}

		return "1";
	}


	/**
	 * Get the number (in string) of dequeue items of (event data or event) port (p). 
	 * <p>
	 * If the the value of Dequeue_Protocol is OneItem, then return 1; 
	 * if AllItems, then return Queue_Size; 
	 * if MultiItems, then return the value of Dequeue_Items property 
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return the value (in string) of dequeue number
	 */
	public String Port_Get_Dequeue_Number(ThreadImplementation t, Port p){

		String dequeue_protocol = "OneItem";
		String dequeue_number;	

		dequeue_protocol = Port_Get_DequeueProtocol_str(t,p);

		if (dequeue_protocol.equals("OneItem")){
			dequeue_number = "1";
		}
		else if (dequeue_protocol.equals("AllItems")){
			dequeue_number = Port_Get_QueueSize_str(t, p);
		}
		else if (dequeue_protocol.equals("MultipleItems")){
			dequeue_number = Port_Get_DequeuedItems_str(t, p);
		}
		else{
			dequeue_number = "1";
		}

		return dequeue_number;
	}



	/**
	 * Get the Input_Time property association of a port p
	 * 
	 * @param t a thread implementation that contains this port p
	 * @param p a port
	 * @return
	 */
	public PropertyAssociation Port_Get_DequeueProtocol(ThreadImplementation t, Port p){

		return Port_Get_PropertyX(t, p, "Dequeue_Protocol");
	}



	/**
	 * Get the (first) Reference_Time of a Input_Time property that applies a port (p)
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return a string value of Reference_Time 
	 */
	public String Port_Get_DequeueProtocol_str(ThreadImplementation t, Port p){

		PropertyAssociation pa = Port_Get_DequeueProtocol(t,p);
		if(pa != null){
			String dp = Property_Get_DequeueProtocol_str(pa);
			if(dp != null) return dp;
		}
		return "OneItem";
	}


	/**
	 * Returns whether the input_time of a port is default value, 
	 * if [0.0 Ns .. 0.0 Ns] Dispatch
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return a boolean
	 */
	public boolean Port_InputTime_IsDefault(ThreadImplementation t, Port p){

		String reference = Port_Get_InputTime_Time1_str(t,p);

		if (reference.equals("Dispatch")) {
			String minvalue = Port_Get_InputTime_Offset1_MinValue_str(t, p);
			String minunit = Port_Get_InputTime_Offset1_MinUnit_str(t, p);
			String maxvalue = Port_Get_InputTime_Offset1_MaxValue_str(t, p);
			String maxunit = Port_Get_OutputTime_Offset1_MaxUnit_str(t, p);
			if (minvalue.equals("0") && maxvalue.equals("0") && minunit.equals("ns") && maxunit.equals("ns"))return true;
		}
		return false;
	}


	/**
	 * Returns whether the output_time of a port is default value, 
	 * if [0.0 Ns .. 0.0 Ns] Completion
	 * 
	 * @param t a thread implementation that contains this port 
	 * @param p a port
	 * @return a boolean
	 */
	public boolean Port_OutputTime_IsDefault(ThreadImplementation t, Port p){

		String reference = Port_Get_OutputTime_Time1_str(t,p);

		if (reference.equals("Completion")){
			String minvalue = Port_Get_OutputTime_Offset1_MinValue_str(t, p);
			String minunit = Port_Get_OutputTime_Offset1_MinUnit_str(t, p);
			String maxvalue = Port_Get_OutputTime_Offset1_MaxValue_str(t, p);
			String maxunit = Port_Get_OutputTime_Offset1_MaxUnit_str(t, p);
			if (minvalue.equals("0") && maxvalue.equals("0") && minunit.equals("ns") && maxunit.equals("ns")) return true;
		}
		return false;
	}


	/**
	 * Get a list of properties that applies a connection (pconnection)
	 * 
	 * @param compimpl a component implementation that contains this connection (pconnection) 
	 * @param pconnection a connection
	 * @return An ArrayList of properties that specified in compimpl and applies to pconnection 
	 */
	public ArrayList<PropertyAssociation> Get_Properties_AppliesTo_Connection(ComponentImplementation compimpl, Connection pconnection){

		EList<PropertyAssociation> property_association_list = compimpl.getAllPropertyAssociations();
		ArrayList<PropertyAssociation> list_p = new ArrayList<PropertyAssociation>();
		ArrayList<PropertyAssociation> list_connection_property = new ArrayList<PropertyAssociation>();

		Iterator<?> it = property_association_list.iterator();
		while (it.hasNext())
		{
			PropertyAssociation s = (PropertyAssociation) it.next();
			list_p.add(s);
		}

		Iterator<PropertyAssociation> it2 = list_p.iterator();
		while (it2.hasNext())
		{
			PropertyAssociation pa = (PropertyAssociation) it2.next();
			EList<ContainedNamedElement> list_applyto = pa.getAppliesTos();

			for (int i = 0; i < list_applyto.size(); i++) { 

				ContainedNamedElement ne = list_applyto.get(i);
				EList<ContainmentPathElement> list_cpe = ne.getContainmentPathElements();

				for(int k =0; k < list_cpe.size(); k++){
					ContainmentPathElement cpe = list_cpe.get(k);
					String applyto_name = cpe.getNamedElement().getName();
					//if Input_Time_Offset applies to p
					if (applyto_name == pconnection.getName()) list_connection_property.add(pa);
				}         	
			}
		}			
		return list_connection_property;
	}


	/**
	 * Get a list of (component) string name that a property (pa) applies to
	 * 
	 * @param pa a property association
	 * @return An ArrayList of string (property holder's name) that property pa applies to 
	 */
	public ArrayList<String> Property_Get_AppliesTo(PropertyAssociation pa){

		ArrayList<String> property_holder_str_list = new ArrayList<String>();	
		EList<ContainedNamedElement> list_applyto = pa.getAppliesTos();

		if(list_applyto != null){
			for (int i = 0; i < list_applyto.size(); i++) { 
				ContainedNamedElement applyto = list_applyto.get(i);
				EList<ContainmentPathElement> cpe_list = applyto.getContainmentPathElements();

				if(cpe_list != null){
					for(int j =0; j<cpe_list.size(); j++ ){

						ContainmentPathElement cpe = cpe_list.get(j);
						NamedElement ne = cpe.getNamedElement();
						String applyto_name = ne.getName();
						property_holder_str_list.add(applyto_name);
					}   			
				}
			}   
		}       	

		return property_holder_str_list;
	}


	/**
	 * Get a list of string name of a property (pa)'s reference elements
	 * 
	 * @param pa a property association
	 * @return An ArrayList of string (reference element's name) of property pa
	 */
	public ArrayList<String> Property_Get_ReferenceElements(PropertyAssociation pa){


		ArrayList<String> reference_element_str_list = new ArrayList<String>();
		String property_def_name = Property_Get_Name(pa);

		EList<ModalPropertyValue> list_property_value = pa.getOwnedValues();
		if(list_property_value != null){
			for (int i = 0; i < list_property_value.size(); i++) {  

				ModalPropertyValue mpv = list_property_value.get(i);
				if(mpv.getOwnedValue() instanceof ListValue){
					ListValue value1 = (ListValue) mpv.getOwnedValue();
					EList<PropertyExpression> list2 = value1.getOwnedListElements();

					if(list2 != null){
						for(int j =0; j< list2.size(); j++){
							if(list2.get(j) instanceof ReferenceValue){
								ReferenceValue property_value = (ReferenceValue) list2.get(j);
								EList<ContainmentPathElement> listx = property_value.getContainmentPathElements();

								if(listx != null){
									for (int k = 0; k < listx.size(); k++) { 
										ContainmentPathElement cpe2 = listx.get(k);
										NamedElement ne2 = cpe2.getNamedElement();
										if(property_def_name.equals("Actual_Processor_Binding") 
												| property_def_name.equals("Actual_Connection_Binding")){
											Trace("property_def_name = Actual_Processor_Binding ");
											//ProcessorSubcomponent subprocessor = (ProcessorSubcomponent) listx.get(k);
											String reference_element_name = ne2.getName();
											reference_element_str_list.add(reference_element_name);
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return reference_element_str_list;
	}


	/**
	 * Get the EnumerationLiteral value of property named as xname
	 * 
	 * @param pa a property association
	 * @param xname peoperty name
	 * @return A EnumerationLiteral value of property pa if pa is named as xname
	 */
	public EnumerationLiteral Property_Get_X_EnumerationLiteral(PropertyAssociation pa, String xname){

		Property propertyDef = pa.getProperty();
		String propertyDef_name = propertyDef.getName();

		if (propertyDef_name.equals(xname)) 
		{
			EList<ModalPropertyValue> listPropertyValue = pa.getOwnedValues();	

			if(listPropertyValue != null){
				for (int i = 0; i < listPropertyValue.size(); i++) {  
					ModalPropertyValue value1 = listPropertyValue.get(i);

					if (value1.getOwnedValue() instanceof NamedValue) {
						NamedValue value = (NamedValue) value1.getOwnedValue();   
						EnumerationLiteral el1 = (EnumerationLiteral) value.getNamedValue();
						return el1;
					}
				}
			}
		}          		

		Trace("property " + xname + " EnumerationLiteral value not feteched");
		return null;
	}



	/**
	 * Get the IntegerLiteral value of property named as xname
	 * 
	 * @param pa a property association
	 * @param xname peoperty name
	 * @return A IntegerLiteral value of property pa if pa is named as xname
	 */
	public IntegerLiteral Property_Get_X_IntegerLiteral(PropertyAssociation pa, String xname){

		Property propertyDef = pa.getProperty();
		String propertyDef_name = propertyDef.getName();

		if (propertyDef_name.equals(xname)) 
		{
			EList<ModalPropertyValue> listPropertyValue = pa.getOwnedValues();	

			if(listPropertyValue != null){
				for (int i = 0; i < listPropertyValue.size(); i++) {  
					ModalPropertyValue value1 = listPropertyValue.get(i);

					if (value1.getOwnedValue() instanceof IntegerLiteral) {
						IntegerLiteral il = (IntegerLiteral) value1.getOwnedValue();
						return il;
					}
				}
			}
		}          		

		Trace("property " + xname + " IntegerLiteral value not feteched");
		return null;
	}



	/**
	 * Get the StringLiteral value of property named as xname
	 * 
	 * @param pa a property association
	 * @param xname peoperty name
	 * @return A StringLiteral value of property pa if pa is named as xname
	 */
	public StringLiteral Property_Get_X_StringLiteral(PropertyAssociation pa, String xname){

		Property propertyDef = pa.getProperty();
		String propertyDef_name = propertyDef.getName();

		if (propertyDef_name.equals(xname)) 
		{
			EList<ModalPropertyValue> listPropertyValue = pa.getOwnedValues();	

			if(listPropertyValue != null){
				for (int i = 0; i < listPropertyValue.size(); i++) {  
					ModalPropertyValue value1 = listPropertyValue.get(i);

					if (value1.getOwnedValue() instanceof StringLiteral) {
						StringLiteral il = (StringLiteral) value1.getOwnedValue();
						return il;
					}
				}
			}
		}          		

		Trace("property " + xname + " StringLiteral value not feteched");
		return null;
	}


	/**
	 * Get the name of a property
	 * 
	 * @param pa a property association
	 * 
	 * @return A string that represents name of property pa 
	 */
	public String Property_Get_Name(PropertyAssociation pa){

		// Trace("\n -- Property_Get_Name ");
		//		EList<ModalPropertyValue> list1 = pa.getOwnedValues();
		//		if (list1 != null){
		//			for(int i=0; i< list1.size(); i++){
		//				// ModalPropertyValue e1 = list1.get(i);
		//				//Trace("\n ****** pa owned value = " + e1.toString());
		//			}
		//		}
		// Trace("\n -- pa location reference = " + pa.getLocationReference().toString());
		
		String str;
		Property property1 =  pa.getProperty();
		if (property1 != null) {
			str = property1.getName();
			if (str != null) return str;
			java.lang.System.err.print("\n property name null !!! ");
			throw new IllegalArgumentException("Property name not fetched!!  \n");
		}
		else
			java.lang.System.err.print("\n AADLCoLAPI :: Property_Get_Name: property null !!!");
		return null;
	}


	/**
	 * Get the name of a property
	 * 
	 * @param pa a property association
	 * 
	 * @return A string that represents name of property pa 
	 */
	public ArrayList<ModalPropertyValue> Property_Get_Values(PropertyAssociation pa){

		EList<ModalPropertyValue> list1 = pa.getOwnedValues();
		ArrayList<ModalPropertyValue> list = new ArrayList<ModalPropertyValue>();

		if(list1 != null){
			list.addAll(list1);
		}

		return list;
	}


	/**
	 * Get the string value of Device_Dispatch_Protocol property
	 * 
	 * @param pa a property association
	 * @return A string value of Device_Dispatch_Protocol property
	 */
	public String Property_Get_DeviceDispatchProtocol_str(PropertyAssociation pa){

		EnumerationLiteral il = Property_Get_DeviceDispatchProtocol(pa);
		if(il != null) return il.getName();
		return null;
	}


	/**
	 * Get the value of Timing property
	 * 
	 * @param pa a property association
	 * @return A string value of Timing property
	 */
	public String Property_Get_Timing_str(PropertyAssociation pa){

		EnumerationLiteral il = Property_Get_Timing(pa);
		if(il != null) return il.getName();
		return null;
	}


	/**
	 * Get the value of Fan_Out_Policy property
	 * 
	 * @param pa a property association
	 * @return A string value of Fan_Out_Policy property
	 */
	public String Property_Get_FanOutPolicy_str(PropertyAssociation pa){

		EnumerationLiteral il = Property_Get_FanOutPolicy(pa);
		if(il != null) return il.getName();
		return null;
	}


	/**
	 * Get the value of Queue_Size property
	 * 
	 * @param pa a property association
	 * @return A string value of Queue_Size property
	 */
	public String Property_Get_QueueSize_str(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_QueueSize(pa);
		if (il != null) return IntegerLiteral_Get_Value(il);
		return null;
	}

	/**
	 * Get the string value of Priority property
	 * 
	 * @param pa a property association
	 * @return A string value of Priority property
	 */
	public String Property_Get_Priority_str(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_Priority(pa);
		if (il != null) return IntegerLiteral_Get_Value(il);
		return null;
	}


	/**
	 * Get the value of Dequeue_Protocol property
	 * 
	 * @param pa a property association
	 * @return A string value of Dequeue_Protocol property
	 */
	public String Property_Get_DequeueProtocol_str(PropertyAssociation pa){

		EnumerationLiteral il = Property_Get_DequeueProtocol(pa);
		if(il != null) return il.getName();
		return null;
	}


	/**
	 * Get the value of Dequeued_Items property
	 * 
	 * @param pa a property association
	 * @return A string value of Dequeued_Items property
	 */
	public String Property_Get_DequeuedItems_str(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_DequeuedItems(pa);
		if(il != null) return IntegerLiteral_Get_Value(il);
		return null;
	}


	/**
	 * Get the value of Queue_Processing_Protocol property
	 * 
	 * @param pa a property association
	 * @return A string value of Queue_Processing_Protocol property
	 */
	public String Property_Get_QueueProcessingProtocol_str(PropertyAssociation pa){

		EnumerationLiteral il = Property_Get_QueueProcessingProtocol(pa);
		if(il != null) return il.getName();
		return null;
	}


	/**
	 * Get the value of Overflow_Handling_Protocol property
	 * 
	 * @param pa a property association
	 * @return A string value of Overflow_Handling_Protocol property
	 */
	public String Property_Get_OverflowHandlingProtocol_str(PropertyAssociation pa){

		EnumerationLiteral il = Property_Get_OverflowHandlingProtocol(pa);
		if(il != null) return il.getName();
		return null;
	}


	/**
	 * Get the value of Dispatch_Protocol property
	 * 
	 * @param pa a property association
	 * @return A string value of Dispatch_Protocol property
	 */
	public String Property_Get_DispatchProtocol_str(PropertyAssociation pa){

		EnumerationLiteral il = Property_Get_DispatchProtocol(pa);
		if(il == null) return null;
		return il.getName();
	}


	/**
	 * Get the string value of Clock_Period property
	 * 
	 * @param pa a property association
	 * @return A string value of Clock_Period property
	 */
	public String Property_ClockPeriod_Value_str(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_ClockPeriod(pa);
		if(il == null) return null;
		return String.valueOf(il.getValue());
	}


	/**
	 * Get the string unit of Clock_Period property
	 * 
	 * @param pa a property association
	 * @return A string unit of Clock_Period property
	 */
	public String Property_ClockPeriod_Unit_str(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_ClockPeriod(pa);
		if(il == null) return null;
		return il.getUnit().getName();
	}


	/**
	 * Get the string unit of Clock_Period property
	 * 
	 * @param pa a property association
	 * @return A string unit of Clock_Period property
	 */
	public String Property_Get_SourceName_str(PropertyAssociation pa){

		StringLiteral il = Property_Get_X_StringLiteral(pa, "Source_Name");
		if (il == null) return null;
		return StringLiteral_Name(il);
	}



	/**
	 * Get the Period (for example: 2 ms)
	 * 
	 * @param pa a property association
	 * @return Period 
	 */
	public IntegerLiteral Property_Get_Period(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_X_IntegerLiteral(pa, "Period");
		return il;
	}


	/**
	 * Get the Deadline (for example: 2 ms)
	 * 
	 * @param pa a property association
	 * @return Deadline 
	 */
	public IntegerLiteral Property_Get_Deadline(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_X_IntegerLiteral(pa, "Deadline");
		return il;
	}



	/**
	 * Get the Clock_Period (for example: 2 ms)
	 * 
	 * @param pa a property association
	 * @return Clock_Period 
	 */
	public IntegerLiteral Property_Get_ClockPeriod(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_X_IntegerLiteral(pa, "Clock_Period");
		return il;
	}


	/**
	 * Get the Queue_Size
	 * 
	 * @param pa a property association
	 * @return Queue_Size 
	 */
	public IntegerLiteral Property_Get_QueueSize(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_X_IntegerLiteral(pa, "Queue_Size");
		return il;
	}



	/**
	 * Get the Dequeued_Items
	 * 
	 * @param pa a property association
	 * @return Dequeued_Items 
	 */
	public IntegerLiteral Property_Get_DequeuedItems(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_X_IntegerLiteral(pa, "Dequeued_Items");
		return il;
	}



	/**
	 * Get the Priority
	 * 
	 * @param pa a property association
	 * @return Priority 
	 */
	public IntegerLiteral Property_Get_Priority(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_X_IntegerLiteral(pa, "Priority");
		return il;
	}



	/**
	 * Get the Dispatch_Protocol (for example: Periodic)
	 * 
	 * @param pa a property association
	 * @return Dispatch_Protocol 
	 */
	public EnumerationLiteral Property_Get_DispatchProtocol(PropertyAssociation pa){

		EnumerationLiteral el = Property_Get_X_EnumerationLiteral(pa, "Dispatch_Protocol" );	
		return el;
	}



	/**
	 * Get the Device_Dispatch_Protocol (for example: Periodic)
	 * 
	 * @param pa a property association
	 * @return Dispatch_Protocol 
	 */
	public EnumerationLiteral Property_Get_DeviceDispatchProtocol(PropertyAssociation pa){

		EnumerationLiteral el = Property_Get_X_EnumerationLiteral(pa, "Device_Dispatch_Protocol" );	
		return el;
	}




	/**
	 * Get the Timing 
	 * 
	 * @param pa a property association
	 * @return Timing 
	 */
	public EnumerationLiteral Property_Get_Timing(PropertyAssociation pa){

		EnumerationLiteral el = Property_Get_X_EnumerationLiteral(pa, "Timing" );	
		return el;
	}


	/**
	 * Get the Fan_Out_Policy 
	 * 
	 * @param pa a property association
	 * @return Timing 
	 */
	public EnumerationLiteral Property_Get_FanOutPolicy(PropertyAssociation pa){

		EnumerationLiteral el = Property_Get_X_EnumerationLiteral(pa, "Fan_Out_Policy" );	
		return el;
	}


	/**
	 * Get the Overflow_Handling_Protocol 
	 * 
	 * @param pa a property association
	 * @return Overflow_Handling_Protocol 
	 */
	public EnumerationLiteral Property_Get_OverflowHandlingProtocol(PropertyAssociation pa){

		EnumerationLiteral el = Property_Get_X_EnumerationLiteral(pa, "Overflow_Handling_Protocol" );	
		return el;
	}


	/**
	 * Get the Queue_Processing_Protocol 
	 * 
	 * @param pa a property association
	 * @return Queue_Processing_Protocol 
	 */
	public EnumerationLiteral Property_Get_QueueProcessingProtocol(PropertyAssociation pa){

		EnumerationLiteral el = Property_Get_X_EnumerationLiteral(pa, "Queue_Processing_Protocol" );	
		return el;
	}


	/**
	 * Get the Dequeue_Protocol 
	 * 
	 * @param pa a property association
	 * @return Dequeue_Protocol 
	 */
	public EnumerationLiteral Property_Get_DequeueProtocol(PropertyAssociation pa){

		EnumerationLiteral el = Property_Get_X_EnumerationLiteral(pa, "Dequeue_Protocol" );	
		return el;
	}


	/**
	 * Get the Fixed field of a Read_Time property
	 * @param pa
	 * @return
	 */
	public RangeValue Property_Get_ReadTime_Fixed(PropertyAssociation pa){

		String pname = Property_Get_Name(pa);
		if(pname.equals("Read_Time")){

			EList<ModalPropertyValue> list1 = pa.getOwnedValues();
			for (int j = 0; j < list1.size(); j++) {  

				ModalPropertyValue mpv = list1.get(j);
				if(mpv.getOwnedValue() instanceof RecordValue) {
					RecordValue rv1 = (RecordValue) mpv.getOwnedValue();

					//get record value's fields
					EList<BasicPropertyAssociation> list3 = rv1.getOwnedFieldValues();
					if(list3 != null){
						for(int k =0; k< list3.size(); k++){
							BasicPropertyAssociation bpa1 = list3.get(k);
							BasicProperty p1 = bpa1.getProperty();
							if( p1.getName().equals("Fixed")){ 									 
								RangeValue value1 = (RangeValue) bpa1.getOwnedValue();
								return value1;
							}
						}					 
					}
				}
			}		
		}

		return null;
	}


	/**
	 * Get the unit of min offset of the Fixed field of Read_Time property
	 * @param pa
	 * @return
	 */
	public String Property_Get_ReadTime_Fixed_MinUnit_str(PropertyAssociation pa){

		RangeValue rv = Property_Get_ReadTime_Fixed(pa);
		if (rv == null) return "ns";
		Trace("rv != null");
		String u1 = TimeRange_Get_MinUnit(rv);
		return u1;
	}


	/**
	 * Get the value of min offset of the Fixed field of Read_Time property
	 * @param pa
	 * @return
	 */
	public String Property_Get_ReadTime_Fixed_MinValue_str(PropertyAssociation pa){

		RangeValue rv = Property_Get_ReadTime_Fixed(pa);
		if(rv == null) return "0";
		String u1 = TimeRange_Get_MinValue(rv);
		return u1;
	}

	/**
	 * Get the unit of max offset of the Fixed field of Read_Time property
	 * @param pa
	 * @return
	 */
	public String Property_Get_ReadTime_Fixed_MaxUnit_str(PropertyAssociation pa){

		RangeValue rv = Property_Get_ReadTime_Fixed(pa);
		if(rv == null) return "ns";
		String u1 = TimeRange_Get_MaxUnit(rv);
		return u1;
	}


	/**
	 * Get the value of max offset of the Fixed field of Read_Time property
	 * @param pa
	 * @return
	 */
	public String Property_Get_ReadTime_Fixed_MaxValue_str(PropertyAssociation pa){

		RangeValue rv = Property_Get_ReadTime_Fixed(pa);
		if(rv == null) return "0";
		String u1 = TimeRange_Get_MaxValue(rv);
		return u1;
	}

	/**
	 * Get the PerByte field of a Read_Time property
	 * @param pa
	 * @return
	 */
	public RangeValue Property_Get_ReadTime_PerByte(PropertyAssociation pa){

		String pname = Property_Get_Name(pa);
		if(pname.equals("Read_Time")){

			EList<ModalPropertyValue> list1 = pa.getOwnedValues();
			for (int j = 0; j < list1.size(); j++) {  

				ModalPropertyValue mpv = list1.get(j);
				if(mpv.getOwnedValue() instanceof RecordValue) {
					RecordValue rv1 = (RecordValue) mpv.getOwnedValue();

					//get record value's fields
					EList<BasicPropertyAssociation> list3 = rv1.getOwnedFieldValues();
					if(list3 != null){
						for(int k =0; k< list3.size(); k++){
							BasicPropertyAssociation bpa1 = list3.get(k);
							BasicProperty p1 = bpa1.getProperty();
							if( p1.getName().equals("PerByte")){ 									 
								RangeValue value1 = (RangeValue) bpa1.getOwnedValue();
								return value1;
							}
						}					 
					}
				}
			}		
		}

		return null;
	}



	/**
	 * Get the unit of min offset of the PerByte field of Read_Time property
	 * @param pa
	 * @return
	 */
	public String Property_Get_ReadTime_PerByte_MinUnit_str(PropertyAssociation pa){

		RangeValue rv = Property_Get_ReadTime_PerByte(pa);
		if(rv == null) return "ns";
		String u1 = TimeRange_Get_MinUnit(rv);
		return u1;
	}


	/**
	 * Get the value of min offset of the PerByte field of Read_Time property
	 * @param pa
	 * @return
	 */
	public String Property_Get_ReadTime_PerByte_MinValue_str(PropertyAssociation pa){

		RangeValue rv = Property_Get_ReadTime_PerByte(pa);
		if(rv == null) return "0";
		String u1 = TimeRange_Get_MinValue(rv);
		return u1;
	}



	/**
	 * Get the unit of max offset of the PerByte field of Read_Time property
	 * @param pa
	 * @return
	 */
	public String Property_Get_ReadTime_PerByte_MaxUnit_str(PropertyAssociation pa){

		RangeValue rv = Property_Get_ReadTime_PerByte(pa);
		if(rv == null) return "ns";
		String u1 = TimeRange_Get_MaxUnit(rv);
		return u1;
	}


	/**
	 * Get the value of max offset of the PerByte field of Read_Time property
	 * @param pa
	 * @return
	 */
	public String Property_Get_ReadTime_PerByte_MaxValue_str(PropertyAssociation pa){

		RangeValue rv = Property_Get_ReadTime_PerByte(pa);
		if(rv == null) return "0"; 
		String u1 = TimeRange_Get_MaxValue(rv);
		return u1;
	}


	/**
	 * Get the name of a property
	 * 
	 * @param pa a property association
	 * 
	 * @return A string that represents name of property pa 
	 */
	public PropertyExpression ModalPropertyValue_Get_Value(ModalPropertyValue v){

		return v.getOwnedValue();
	} 


	/**
	 * Get the string value of a IntegerLiteral x, 
	 * for example, get value 2 of 2 ms
	 * 
	 * @param x a IntegerLiteral value
	 * @return A string that represents the value 
	 */
	public String IntegerLiteral_Get_Value(IntegerLiteral x){

		return String.valueOf(x.getValue());
	}


	/**
	 * Get the string unit of a IntegerLiteral x, 
	 * for example, get value ms of 2 ms
	 * 
	 * @param x a IntegerLiteral value
	 * @return A string that represents the unit 
	 */
	public String IntegerLiteral_Get_Unit(IntegerLiteral x){

		return x.getUnit().getName();
	}


	/**
	 * Get a IntegerLiteral x's AbsoluteFactor, corresponding to a target unit, 
	 * 
	 * @param x a IntegerLiteral value
	 * @param target a target UnitLiteral
	 * 
	 * @return A double that represents the factor 
	 */
	public double IntegerLiteral_Get_AbsoluteFactor(IntegerLiteral x, UnitLiteral target){

		return x.getUnit().getAbsoluteFactor(target);
	}


	/**
	 * Get the unit of a IntegerLiteral x
	 * 
	 * @param x a IntegerLiteral value
	 * @return A the unit 
	 */
	public UnitLiteral IntegerLiteral_Unit(IntegerLiteral x){

		return x.getUnit();
	}


	/**
	 * Get the string value of lower bound (IntegerLiteral) of a RangeValue x, 
	 * for example, get value 2 of RangeValue 2 ms .. 5 ms
	 * 
	 * @param x a RangeValue value
	 * @return A string that represents the value of lower bound
	 */
	public String IntegerLiteralRange_Get_MinValue(RangeValue x){

		IntegerLiteral min = (IntegerLiteral) x.getMinimum();
		return String.valueOf(min.getValue());
	}


	/**
	 * Get the string unit of lower bound (IntegerLiteral) of a RangeValue x, 
	 * for example, get value ms of RangeValue 2 ms .. 5 ms
	 * 
	 * @param x a RangeValue value
	 * @return A string that represents the unit of lower bound
	 */
	public String IntegerLiteralRange_Get_MinUnit(RangeValue x){

		IntegerLiteral min = (IntegerLiteral) x.getMinimum();
		return min.getUnit().getName();
	}


	/**
	 * Get the string value of upper bound (IntegerLiteral) of a RangeValue x, 
	 * for example, get value 5 of RangeValue 2 ms .. 5 ms
	 * 
	 * @param x a RangeValue value
	 * @return A string that represents the value of upper bound
	 */
	public String IntegerLiteralRange_Get_MaxValue(RangeValue x){

		IntegerLiteral max = (IntegerLiteral) x.getMaximum();
		return String.valueOf(max.getValue());
	}


	/**
	 * Get the string unit of upper bound (IntegerLiteral) of a RangeValue x, 
	 * for example, get value ms of RangeValue 2 ms .. 5 ms
	 * 
	 * @param x a RangeValue value
	 * @return A string that represents the unit of upper bound
	 */
	public String IntegerLiteralRange_Get_MaxUnit(RangeValue x){

		IntegerLiteral max = (IntegerLiteral) x.getMaximum();
		return max.getUnit().getName();
	}


	/**
	 * Get the min_offset's unit of a TimeRange
	 * 
	 * @param range_val a range value
	 * @return the unit of the min_offset of port p
	 */
	public String TimeRange_Get_MinUnit(RangeValue range_val){

		IntegerLiteral min = (IntegerLiteral) range_val.getMinimum();
		String min_unit = IntegerLiteral_Get_Unit(min); 	
		if(min_unit == null) min_unit = "ns";
		return min_unit;
	}


	/**
	 * Get the min_offset's value of a TimeRange
	 * 
	 * @param range_val a range value
	 * @return the value (in string) of the min_offset of port p
	 */
	public String TimeRange_Get_MinValue(RangeValue range_val){

		IntegerLiteral min = (IntegerLiteral) range_val.getMinimum();
		String min_value = IntegerLiteral_Get_Value(min); 	
		if(min_value == null) min_value = "0";
		return min_value;
	}


	/**
	 * Get the min_offset's value of a TimeRange
	 * 
	 * @param range_val a range value
	 * @return the value (in string) of the min_offset of port p
	 */
	public long TimeRange_Get_MinValue_long(RangeValue range_val){

		IntegerLiteral min = (IntegerLiteral) range_val.getMinimum();
		if(min != null){
			long min_value = min.getValue();
			return min_value;
		}

		return 0;
	}


	/**
	 * Get the max_offset's unit of a TimeRange
	 * 
	 * @param range_val a range value
	 * @return the unit of the max_offset of port p
	 */
	public String TimeRange_Get_MaxUnit(RangeValue range_val){

		IntegerLiteral max = (IntegerLiteral) range_val.getMaximum();
		String miax_unit = IntegerLiteral_Get_Unit(max); 	
		if(miax_unit == null) miax_unit = "ns";
		return miax_unit;
	}


	/**
	 * Get the max_offset's value of a TimeRange
	 * 
	 * @param range_val a range value
	 * @return the value (in string) of the max_offset of port p
	 */
	public String TimeRange_Get_MaxValue(RangeValue range_val){

		IntegerLiteral max = (IntegerLiteral) range_val.getMaximum();
		String max_value = IntegerLiteral_Get_Value(max); 	
		if(max_value == null) max_value = "0";
		return max_value;
	}


	/**
	 * Get the max_offset's value of a TimeRange
	 * 
	 * @param range_val a range value
	 * @return the value (in string) of the max_offset of port p
	 */
	public long TimeRange_Get_MaxValue_long(RangeValue range_val){

		IntegerLiteral max = (IntegerLiteral) range_val.getMaximum();
		if(max == null) return 0; 
		return max.getValue(); 
		}


	/**
	 * Returns the (first) Time field of a Input_Time property.
	 * Since Input_Time could have a list of record (Time, Offset), 
	 * this method gets the first Time
	 * 
	 * @param pa a Input_Time property
	 * @return
	 */
	public EnumerationLiteral InputTime_Get_Time1(PropertyAssociation pa){

		EList<ModalPropertyValue> list1 = pa.getOwnedValues();
		for (int j = 0; j < list1.size(); j++) {  

			ModalPropertyValue mpv = list1.get(j);
			if(mpv.getOwnedValue() instanceof ListValue) {
				ListValue lv1 = (ListValue) mpv.getOwnedValue();
				EList<PropertyExpression> list2 = lv1.getOwnedListElements();
				if(list2 != null){
					for(int i=0; i< list2.size(); i++){
						if(list2.get(i) instanceof RecordValue){
							RecordValue rv1 = (RecordValue) list2.get(i);

							//get record value's fields
							EList<BasicPropertyAssociation> list3 = rv1.getOwnedFieldValues();
							if(list3 != null){
								for(int k =0; k< list3.size(); k++){
									BasicPropertyAssociation bpa1 = list3.get(k);
									BasicProperty p1 = bpa1.getProperty();
									if( p1.getName().equals("Time")){ 									 
										NamedValue value1 = (NamedValue) bpa1.getOwnedValue();
										EnumerationLiteral el1 = (EnumerationLiteral) value1.getNamedValue();
										return el1;
									}
								}					 
							}
						}
					}
				}
			}
		}

		return null;
	}


	/**
	 * Returns the (first) Offset field of a Input_Time property.
	 * Since Input_Time could have a list of record (Time, Offset), 
	 * this method gets the first Offset
	 * 
	 * @param pa a Input_Time property
	 * @return
	 */
	public RangeValue InputTime_Get_Offset1(PropertyAssociation pa){

		EList<ModalPropertyValue> list1 = pa.getOwnedValues();
		for (int j = 0; j < list1.size(); j++) {  

			ModalPropertyValue mpv = list1.get(j);
			if(mpv.getOwnedValue() instanceof ListValue) {
				ListValue lv1 = (ListValue) mpv.getOwnedValue();
				EList<PropertyExpression> list2 = lv1.getOwnedListElements();
				if(list2 != null){
					for(int i=0; i< list2.size(); i++){
						if(list2.get(i) instanceof RecordValue){
							RecordValue rv1 = (RecordValue) list2.get(i);

							//get record value's fields
							EList<BasicPropertyAssociation> list3 = rv1.getOwnedFieldValues();
							if(list3 != null){
								for(int k =0; k< list3.size(); k++){
									BasicPropertyAssociation bpa1 = list3.get(k);
									BasicProperty p1 = bpa1.getProperty();
									if( p1.getName().equals("Offset")){ 									 
										RangeValue value1 = (RangeValue) bpa1.getOwnedValue();									 
										return value1;
									}
								}					 
							}
						}
					}
				}
			}
		}

		return null;
	}


	/**
	 * Returns the (first) Time field of a Output_Time property.
	 * Since Output_Time could have a list of record (Time, Offset), 
	 * this method gets the first Time
	 * 
	 * @param pa a Output_Time property
	 * @return
	 */
	public EnumerationLiteral OutputTime_Get_Time1(PropertyAssociation pa){

		EList<ModalPropertyValue> list1 = pa.getOwnedValues();
		for (int j = 0; j < list1.size(); j++) {  

			ModalPropertyValue mpv = list1.get(j);
			if(mpv.getOwnedValue() instanceof ListValue) {
				ListValue lv1 = (ListValue) mpv.getOwnedValue();
				EList<PropertyExpression> list2 = lv1.getOwnedListElements();
				if(list2 != null){
					for(int i=0; i< list2.size(); i++){
						if(list2.get(i) instanceof RecordValue){
							RecordValue rv1 = (RecordValue) list2.get(i);

							//get record value's fields
							EList<BasicPropertyAssociation> list3 = rv1.getOwnedFieldValues();
							if(list3 != null){
								for(int k =0; k< list3.size(); k++){
									BasicPropertyAssociation bpa1 = list3.get(k);
									BasicProperty p1 = bpa1.getProperty();
									if( p1.getName().equals("Time")){ 									 
										NamedValue value1 = (NamedValue) bpa1.getOwnedValue();
										EnumerationLiteral el1 = (EnumerationLiteral) value1.getNamedValue();
										return el1;
									}
								}

							}
						}
					}
				}
			}
		}

		return null;
	}


	/**
	 * Returns the (first) Offset field of a Output_Time property.
	 * Since Output_Time could have a list of record (Time, Offset), 
	 * this method gets the first Offset
	 * 
	 * @param pa a Output_Time property
	 * @return
	 */
	public RangeValue OutputTime_Get_Offset1(PropertyAssociation pa){

		EList<ModalPropertyValue> list1 = pa.getOwnedValues();
		for (int j = 0; j < list1.size(); j++) {  

			ModalPropertyValue mpv = list1.get(j);
			if(mpv.getOwnedValue() instanceof ListValue) {
				ListValue lv1 = (ListValue) mpv.getOwnedValue();
				EList<PropertyExpression> list2 = lv1.getOwnedListElements();
				if(list2 != null){
					for(int i=0; i< list2.size(); i++){
						if(list2.get(i) instanceof RecordValue){
							RecordValue rv1 = (RecordValue) list2.get(i);

							//get record value's fields
							EList<BasicPropertyAssociation> list3 = rv1.getOwnedFieldValues();
							if(list3 != null){
								for(int k =0; k< list3.size(); k++){
									BasicPropertyAssociation bpa1 = list3.get(k);
									BasicProperty p1 = bpa1.getProperty();
									if( p1.getName().equals("Offset")){ 									 
										RangeValue value1 = (RangeValue) bpa1.getOwnedValue();									 
										return value1;
									}
								}					 
							}
						}
					}
				}
			}
		}

		return null;
	}


	/**
	 * Get the string name of a UnitLiteral x
	 * 
	 * @param x a UnitLiteral
	 * @return the string that represents this unit 
	 */
	public String UnitLiteral_Name(UnitLiteral x){

		return x.getName();
	}


	/**
	 * Get the string name of a StringLiteral x
	 * 
	 * @param x a StringLiteral
	 * @return the string that represents this StringLiteral
	 */
	public String StringLiteral_Name(StringLiteral x){

		return x.getValue();
	}


	/**
	 * Get the unit of Period property
	 * 
	 * @param pa a property association
	 * @return A string value of unit of Period property
	 */
	public String Thread_Get_Period_Unit(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_Period(pa);
		if(il != null){
			return il.getUnit().getName();
		}

		return null;
	}


	/**
	 * Get the number value of Period property
	 * 
	 * @param pa a property association
	 * @return A string value of number of Period property
	 */
	public String Thread_Get_Period_Value(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_Period(pa);
		if(il != null){
			return String.valueOf(il.getValue());
		}

		return null;
	}


	/**
	 * Get the unit of Deadline property, for example: ms
	 * 
	 * @param pa a property association
	 */
	public String Thread_Get_Deadline_Unit(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_Deadline(pa);
		if(il != null){
			return il.getUnit().getName();
		}

		return null;
	}


	/**
	 * Get the number value of Deadline property, e.g., 5
	 * 
	 * @param pa a property association
	 * @return A string value of number of Deadline property
	 */
	public String Thread_Get_Deadline_Value(PropertyAssociation pa){

		IntegerLiteral il = Property_Get_Deadline(pa);
		if(il != null){
			return String.valueOf(il.getValue());
		}

		return null;
	}


	/**
	 * Get the string value of a thread's Dispatch_Protocol property
	 * 
	 * @param pa a property association
	 * @return A string value of Dispatch_Protocol property
	 */
	public String Thread_Get_DispatchProtocol_str(PropertyAssociation pa){

		EnumerationLiteral il = Property_Get_DispatchProtocol(pa);
		if(il != null){
			return il.getName();
		}

		return null;
	}


	/**
	 * Get the name of a ComponentType (t)
	 * 
	 * @param t a component type 
	 * @return a string name
	 */
	public String ComponentType_GetName(ComponentType t)
	{	
		return t.getName();
	}


	/**
	 * Returns the name of a abstractType object
	 * @param at
	 * @return
	 */
	public String AbstractType_GetName(AbstractType at){
		return at.getName();
	}

	/**
	 * Get a list of in data ports of a ComponentType (comp)
	 * 
	 * <p>
	 * The ComponentType could be: SystemType, ProcessType, ThreadType, DeviceType and ProcessorType
	 * 
	 * @param comp a component type
	 * @return An ArrayList of DataPort objects
	 */
	public ArrayList<DataPort> Component_GetInDataPort(ComponentType comp)
	{	
		if (comp instanceof SystemType){
			return System_GetInDataPort((SystemType) comp);
		}
		else if (comp instanceof ProcessType){
			return Process_GetInDataPort((ProcessType) comp);
		}
		else if (comp instanceof ThreadType){
			return Thread_GetInDataPort((ThreadType) comp);
		}
		else if (comp instanceof DeviceType){
			return Device_GetInDataPort((DeviceType) comp);
		}
		else if (comp instanceof ProcessorType){
			return Processor_GetInDataPort((ProcessorType) comp);
		}
		else {
			return null;
		}
	}


	/**
	 * Get a list of out data ports of a ComponentType (comp)
	 * 
	 * <p>
	 * The ComponentType could be: SystemType, ProcessType, ThreadType, DeviceType and ProcessorType
	 * 
	 * @param comp a component type
	 * @return An ArrayList of DataPort objects
	 */
	public ArrayList<DataPort> Component_GetOutDataPort(ComponentType comp)
	{	if (comp instanceof SystemType){
		return System_GetOutDataPort((SystemType) comp);
	}
	else if (comp instanceof ProcessType){
		return Process_GetOutDataPort((ProcessType) comp);
	}
	else if (comp instanceof ThreadType){
		return Thread_GetOutDataPort((ThreadType) comp);
	}
	else if (comp instanceof DeviceType){
		return Device_GetOutDataPort((DeviceType) comp);
	}
	else if (comp instanceof ProcessorType){
		return Processor_GetOutDataPort((ProcessorType) comp);
	}
	else {
		return null;
	}
	}


	/**
	 * Get a list of in event ports of a ComponentType (comp)
	 * 
	 * <p>
	 * The ComponentType could be: SystemType, ProcessType, ThreadType, DeviceType and ProcessorType
	 * 
	 * @param comp a component type
	 * @return An ArrayList of EventPort objects
	 */
	public ArrayList<EventPort> Component_GetInEventPort(ComponentType comp)
	{	if (comp instanceof SystemType){
		return System_GetInEventPort((SystemType) comp);
	}
	else if (comp instanceof ProcessType){
		return Process_GetInEventPort((ProcessType) comp);
	}
	else if (comp instanceof ThreadType){
		return Thread_GetInEventPort((ThreadType) comp);
	}
	else if (comp instanceof DeviceType){
		return Device_GetInEventPort((DeviceType) comp);
	}
	else if (comp instanceof ProcessorType){
		return Processor_GetInEventPort((ProcessorType) comp);
	}
	else {
		return null;
	}
	}


	/**
	 * Get a list of out event ports of a ComponentType (comp)
	 * 
	 * <p>
	 * The ComponentType could be: SystemType, ProcessType, ThreadType, DeviceType and ProcessorType
	 * 
	 * @param comp a component type
	 * @return An ArrayList of EventPort objects
	 */
	public ArrayList<EventPort> Component_GetOutEventPort(ComponentType comp)
	{	if (comp instanceof SystemType){
		return System_GetOutEventPort((SystemType) comp);
	}
	else if (comp instanceof ProcessType){
		return Process_GetOutEventPort((ProcessType) comp);
	}
	else if (comp instanceof ThreadType){
		return Thread_GetOutEventPort((ThreadType) comp);
	}
	else if (comp instanceof DeviceType){
		return Device_GetOutEventPort((DeviceType) comp);
	}
	else if (comp instanceof ProcessorType){
		return Processor_GetOutEventPort((ProcessorType) comp);
	}
	else if (comp instanceof SubprogramType){
		return Subprogram_GetOutEventPort((SubprogramType) comp);
	}
	else {
		return null;
	}
	}


	/**
	 * Get a list of in event data ports of a ComponentType (comp)
	 * 
	 * <p>
	 * The ComponentType could be: SystemType, ProcessType, ThreadType, DeviceType and ProcessorType
	 * 
	 * @param comp a component type
	 * @return An ArrayList of EventDataPort objects
	 */
	public ArrayList<EventDataPort> Component_GetInEventDataPort(ComponentType comp)
	{	
		if (comp instanceof SystemType){
			return System_GetInEventDataPort((SystemType) comp);
		}
		else if (comp instanceof ProcessType){
			return Process_GetInEventDataPort((ProcessType) comp);
		}
		else if (comp instanceof ThreadType){
			return Thread_GetInEventDataPort((ThreadType) comp);
		}
		else if (comp instanceof DeviceType){
			return Device_GetInEventDataPort((DeviceType) comp);
		}
		else if (comp instanceof ProcessorType){
			return Processor_GetInEventDataPort((ProcessorType) comp);
		}
		else {
			return null;
		}
	}


	/**
	 * Get a list of out event data ports of a ComponentType (comp)
	 * 
	 * <p>
	 * The ComponentType could be: SystemType, ProcessType, ThreadType, DeviceType and ProcessorType
	 * 
	 * @param comp a component type
	 * @return An ArrayList of EventDataPort objects
	 */
	public ArrayList<EventDataPort> Component_GetOutEventDataPort(ComponentType comp)
	{	
		if (comp instanceof SystemType){
			return System_GetOutEventDataPort((SystemType) comp);
		}
		else if (comp instanceof ProcessType){
			return Process_GetOutEventDataPort((ProcessType) comp);
		}
		else if (comp instanceof ThreadType){
			return Thread_GetOutEventDataPort((ThreadType) comp);
		}
		else if (comp instanceof DeviceType){
			return Device_GetOutEventDataPort((DeviceType) comp);
		}
		else if (comp instanceof ProcessorType){
			return Processor_GetOutEventDataPort((ProcessorType) comp);
		}
		else if (comp instanceof SubprogramType){
			return Subprogram_GetOutEventDataPort((SubprogramType) comp);
		}
		else {
			return null;
		}
	}


	/**
	 * Get a list of requires data access of a ComponentType (comp)
	 * 
	 * <p>
	 * The ComponentType could be: SystemType, ProcessType, ThreadType, DeviceType and ProcessorType
	 * 
	 * @param comp a component type
	 * @return An ArrayList of DataAccess objects
	 */
	public ArrayList<DataAccess> Component_GetRequiresDataAccess(ComponentType comp)
	{	
		if (comp instanceof ThreadType){
			return Thread_GetRequiresDataAccess((ThreadType) comp);
		}
		else {
			Trace("component get requires data access not yet implemented! ");
			return null;
		}
	}



	/**
	 * Get a list of requires subprogram access of a ComponentType (comp)
	 * 
	 * <p>
	 * The ComponentType could be: ProcessType, ThreadType
	 * 
	 * @param comp a component type
	 * @return An ArrayList of SubprogramAccess objects
	 */
	public ArrayList<SubprogramAccess> Component_GetRequiresSubprogramAccess(ComponentType comp)
	{	
		if (comp instanceof ThreadType){
			return Thread_GetRequiresSubprogramAccess((ThreadType) comp);
		}
		else {
			Trace("component get requires subprogram access not yet implemented! ");
			return null;
		}
	}



	/**
	 * Returns a list of flow specifications of a given component type ct
	 * @param ct a component type
	 * @return a list of flow specifications
	 */
	public ArrayList<FlowSpecification> Component_GetFlowSpecification(ComponentType ct){

		ArrayList<FlowSpecification> list = new ArrayList<FlowSpecification>();
		if(ct.getOwnedFlowSpecifications() != null){
			EList<FlowSpecification> list1 = ct.getOwnedFlowSpecifications();
			list.addAll(list1);
		}

		return list;
	}


	/**
	 * Returns a list of property associations of a component type
	 * 
	 * @param ct a component type
	 * @return a list of property associations
	 */
	public ArrayList<PropertyAssociation> Component_GetPropertyAssociations(ComponentType ct){

		ArrayList<PropertyAssociation> list = new ArrayList<PropertyAssociation>();
		if(ct.getAllPropertyAssociations() != null){
			EList<PropertyAssociation> list1 = ct.getAllPropertyAssociations();
			list.addAll(list1);
		}

		return list;
	}


	/**
	 * Returns a list of feature groups of a component type ct
	 * @param ct
	 * @return
	 */
	public ArrayList<FeatureGroup> Component_GetFeatureGroups(ComponentType ct){

		ArrayList<FeatureGroup> list = new ArrayList<FeatureGroup>();
		EList<FeatureGroup> list1 = ct.getOwnedFeatureGroups();
		if(list1 != null){
			if(list1.size()>0){
				list.addAll(list1);
			}
		}

		return list;
	}


	/**
	 * Returns  a list of modes of a component t
	 * 
	 * @param t a component type
	 * @return a list of modes
	 */
	public ArrayList<Mode> Component_GetModes(ComponentType t){

		ArrayList<Mode> list = new ArrayList<Mode>();
		if(t.getAllModes() != null){
			EList<Mode> list1 = t.getAllModes();
			list.addAll(list1);
		}

		return list;
	}


	/**
	 * Returns  a list of mode transitions of a component t
	 * 
	 * @param t a component type
	 * @return a list of modes
	 */
	public ArrayList<ModeTransition> Component_GetModeTransitions(ComponentType t){

		ArrayList<ModeTransition> list = new ArrayList<ModeTransition>();
		if(t.getAllModeTransitions() != null){
			EList<ModeTransition> list1 = t.getAllModeTransitions();
			list.addAll(list1);
		}

		return list;
	}



	/**
	 * Get the name of a ComponentImpl (t)
	 * 
	 * @param t a component implementation
	 * @return a string name 
	 */
	public String ComponentImpl_GetName(ComponentImplementation t)
	{	
		return t.getName();
	}


	/**
	 * Get the port connections of a componentImpl
	 * 
	 * @param t a component implementation
	 * @return a list of port connections
	 */
	public EList<PortConnection> ComponentImpl_GetPortConnections(ComponentImplementation t){
		return t.getOwnedPortConnections();
	}


	/**
	 * Get the data port connections of a componentImpl
	 * 
	 * @param t a component implementation
	 * @return a list of (data) port connections
	 */
	public ArrayList<PortConnection> ComponentImpl_GetDataPortConnections(ComponentImplementation t){

		ArrayList<PortConnection> list = new ArrayList<PortConnection>();
		EList<PortConnection> list1 = t.getOwnedPortConnections();

		if(list1!=null){
			for(int i =0; i< list1.size(); i++){
				PortConnection c1 = list1.get(i);

				if(Is_DataPortConnection(c1)){
					list.add(c1);
				}
			}
		}
		return list;
	}


	/**
	 * Get the event port connections of a componentImpl
	 * 
	 * @param t a component implementation
	 * @return a list of (event) port connections
	 */
	public ArrayList<PortConnection> ComponentImpl_GetEventPortConnections(ComponentImplementation t){

		ArrayList<PortConnection> list = new ArrayList<PortConnection>();
		EList<PortConnection> list1 = t.getOwnedPortConnections();

		if(list1!=null){
			for(int i =0; i< list1.size(); i++){
				PortConnection c1 = list1.get(i);

				if(Is_EventPortConnection(c1)){
					list.add(c1);
				}
			}
		}
		return list;
	}


	/**
	 * Get the event data port connections of a componentImpl
	 * 
	 * @param t a component implementation
	 * @return a list of (event data ) port connections
	 */
	public ArrayList<PortConnection> ComponentImpl_GetEventDataPortConnections(ComponentImplementation t){

		ArrayList<PortConnection> list = new ArrayList<PortConnection>();
		EList<PortConnection> list1 = t.getOwnedPortConnections();

		if(list1!=null){
			for(int i =0; i< list1.size(); i++){
				PortConnection c1 = list1.get(i);

				if(Is_EventDataPortConnection(c1)){
					list.add(c1);
				}
			}
		}
		return list;
	}


	/**
	 * Get the connections of a component t
	 * @param t a component implementation
	 * @return a list of connections
	 */
	public ArrayList<Connection> ComponentImpl_GetConnections(ComponentImplementation t){

		ArrayList<Connection> list = new ArrayList<Connection>();
		if(  	t.getOwnedConnections() != null){
			EList<Connection> list1 = t.getOwnedConnections();
			list.addAll(list1);
		}

		return list;
	}

	/**
	 * Get the parameter connections of a componentImpl
	 * 
	 * @param t a component implementation
	 * @return a list of parameter connections
	 */
	public ArrayList<ParameterConnection> ComponentImpl_GetParameterConnections(ComponentImplementation t){

		ArrayList<ParameterConnection> list = new ArrayList<ParameterConnection>();
		EList<ParameterConnection> list1 = t.getOwnedParameterConnections();

		if(list1!=null){
			if(list1.size()>0){
				list.addAll(list1);
			}
		}
		return list;
	}



	/**
	 * Get the access connections of a componentImpl
	 * 
	 * @param t a component implementation
	 * @return a list of access connections
	 */
	public EList<AccessConnection> ComponentImpl_GetAccessConnections(ComponentImplementation t){
		return t.getOwnedAccessConnections();
	}


	/**
	 * Get the bus access connections of a componentImpl
	 * 
	 * @param t a component implementation
	 * @return a list of (bus) access connections
	 */
	public ArrayList<AccessConnection> ComponentImpl_GetBusAccessConnections(ComponentImplementation t){

		ArrayList<AccessConnection> list = new ArrayList<AccessConnection>();
		EList<AccessConnection> list1 = t.getOwnedAccessConnections();

		if(list1!=null){
			for(int i =0; i< list1.size(); i++){
				AccessConnection c1 = list1.get(i);

				if(Is_BusAccessConnection(c1)){
					list.add(c1);
				}
			}
		}
		return list;
	}


	/**
	 * Get the data access connections of a componentImpl
	 * 
	 * @param t a component implementation
	 * @return a list of (data) access connections
	 */
	public ArrayList<AccessConnection> ComponentImpl_GetDataAccessConnections(ComponentImplementation t){

		ArrayList<AccessConnection> list = new ArrayList<AccessConnection>();
		EList<AccessConnection> list1 = t.getOwnedAccessConnections();

		if(list1!=null){
			for(int i =0; i< list1.size(); i++){
				AccessConnection c1 = list1.get(i);

				if(Is_DataAccessConnection(c1)){
					list.add(c1);
				}
			}
		}
		return list;
	}


	/**
	 * Get the subprogram access connections of a componentImpl
	 * 
	 * @param t a component implementation
	 * @return a list of (subprogram) access connections
	 */
	public ArrayList<AccessConnection> ComponentImpl_GetSubprogramAccessConnections(ComponentImplementation t){

		ArrayList<AccessConnection> list = new ArrayList<AccessConnection>();
		EList<AccessConnection> list1 = t.getOwnedAccessConnections();

		if(list1!=null){
			for(int i =0; i< list1.size(); i++){
				AccessConnection c1 = list1.get(i);

				if(Is_SubprogramAccessConnection(c1)){
					list.add(c1);
				}
			}
		}
		return list;
	}


	/**
	 * Get the subprogram group access connections of a componentImpl
	 * 
	 * @param t a component implementation
	 * @return a list of (subprogram group) access connections
	 */
	public ArrayList<AccessConnection> ComponentImpl_GetSubprogramGroupAccessConnections(ComponentImplementation t){

		ArrayList<AccessConnection> list = new ArrayList<AccessConnection>();
		EList<AccessConnection> list1 = t.getOwnedAccessConnections(); 

		if(list1!=null){
			for(int i =0; i< list1.size(); i++){
				AccessConnection c1 = list1.get(i);

				if(Is_SubprogramGroupAccessConnection(c1)){
					list.add(c1);
				}
			}
		}
		return list;
	}


	/**
	 * Get the FeatureGroupConnection of a componentImpl
	 * 
	 * @param t a component implementation
	 * @return a list of FeatureGroupConnection
	 */
	public ArrayList<FeatureGroupConnection> ComponentImpl_GetFeatureGroupConnections(ComponentImplementation t){

		ArrayList<FeatureGroupConnection> list = new ArrayList<FeatureGroupConnection>();
		EList<FeatureGroupConnection> list1 = t.getOwnedFeatureGroupConnections();

		if(list1!=null){

			if(list1.size()>0){
				list.addAll(list1);
			}
		}
		return list;
	}



	/**
	 * Get the component type of a given component implementation component (t)
	 * 
	 * @param t a component implementation
	 * @return a component type
	 */
	public ComponentType ComponentImpl_GetComponentType(ComponentImplementation t)
	{	
		return t.getType();
	}


	/**
	 * Returns  a list of modes of a component t
	 * 
	 * @param t a component implementation
	 * @return a list of modes
	 */
	public ArrayList<Mode> ComponentImpl_GetModes(ComponentImplementation t){

		ArrayList<Mode> list = new ArrayList<Mode>();
		if(t.getAllModes() != null){
			EList<Mode> list1 = t.getAllModes();
			list.addAll(list1);
		}

		return list;
	}


	/**
	 * Returns  a list of mode transitions of a component t
	 * 
	 * @param t a component implementation
	 * @return a list of modes
	 */
	public ArrayList<ModeTransition> ComponentImpl_GetModeTransitions(ComponentImplementation t){

		ArrayList<ModeTransition> list = new ArrayList<ModeTransition>();
		if(t.getAllModeTransitions() != null){
			EList<ModeTransition> list1 = t.getAllModeTransitions();
			list.addAll(list1);
		}

		return list;
	}


	/**
	 * Returns a list of property associations of a component implementation
	 * 
	 * @param ct a component implementation
	 * @return a list of property associations
	 */
	public ArrayList<PropertyAssociation> ComponentImpl_GetPropertyAssociations(ComponentImplementation ct){

		ArrayList<PropertyAssociation> list = new ArrayList<PropertyAssociation>();
		if(ct.getAllPropertyAssociations() != null){
			EList<PropertyAssociation> list1 = ct.getAllPropertyAssociations();
			list.addAll(list1);
		}

		return list;
	}


	/**
	 * Returns a list of flow implementation of a given component implementation ct
	 * @param ct a implementation type
	 * @return a list of flow implementations
	 */
	public ArrayList<FlowImplementation> ComponentImpl_GetFlowImplementations(ComponentImplementation ct){

		ArrayList<FlowImplementation> list = new ArrayList<FlowImplementation>();
		if(ct.getOwnedFlowImplementations() != null){
			EList<FlowImplementation> list1 = ct.getOwnedFlowImplementations();
			list.addAll(list1);
		}

		return list;
	}


	/**
	 * Get the name of a Component classifier (t)
	 * 
	 * @param t a classifier component
	 * @return a string name
	 */
	public String Classifier_GetName(Classifier t)
	{	
		if (t instanceof ComponentType){
			return ComponentType_GetName((ComponentType) t);
		}
		else if (t instanceof ComponentImplementation) {
			return ComponentImpl_GetName((ComponentImplementation) t);
		}
		else {
			return null;
		}
	}


	/**
	 * Get the name of a sub component (sub)
	 * 
	 * @param sub a sub component
	 * @return a string name
	 */
	public String Subcomponent_GetName(Subcomponent sub)
	{	
		return sub.getName();
	}



	/**
	 * Get a list of in paramters of a subprogram type (subp_type)
	 * 
	 * @param subp_type a subprogram type
	 * @return An ArrayList of in Parameter objects
	 */
	public ArrayList<Parameter> Subprogram_Get_InParameter(SubprogramType subp_type)
	{	
		ArrayList<Parameter> list_in_p = new ArrayList<Parameter>();

		if (subp_type.getOwnedParameters() != null){
			EList<Parameter> list_p = subp_type.getOwnedParameters();

			Iterator<?> it1 = list_p.iterator();
			while (it1.hasNext())
			{
				Parameter p1 = (Parameter) it1.next();
				if (p1.getDirection().toString()  == "in") list_in_p.add(p1);
			}
		}

		return list_in_p;
	}


	/**
	 * Get a list of out paramters of a subprogram type (subp_type)
	 * 
	 * @param subp_type a subprogram type
	 * @return An ArrayList of in Parameter objects
	 */
	public ArrayList<Parameter> Subprogram_Get_OutParameter(SubprogramType subp_type)
	{	
		ArrayList<Parameter> list_in_p = new ArrayList<Parameter>();

		if (subp_type.getOwnedParameters() != null){
			EList<Parameter> list_p = subp_type.getOwnedParameters();

			Iterator<?> it1 = list_p.iterator();
			while (it1.hasNext())
			{
				Parameter p1 = (Parameter) it1.next();
				if (p1.getDirection().toString()  == "out"){
					list_in_p.add(p1);
				}
			}
		}

		return list_in_p;
	}


	/**
	 * Get a list of processes that bound to a processor (subprocessor). 
	 * If it is not explicitly specified, we suppose that all the sub-processes of this system is running on this processor by default.
	 * 
	 * @param sysimpl a system implementation that contains this subprocessor
	 * @param subprocessor a sub processor component
	 * @return An ArrayList of process subcomponents that bound to this subprocessor
	 */
	public ArrayList<ProcessSubcomponent> Processor_Get_Binding_Process(SystemImplementation sysimpl, ProcessorSubcomponent subprocessor){

		ArrayList<ProcessSubcomponent> list_process_subcomponent = new ArrayList<ProcessSubcomponent>();
		String subprocessor_name = subprocessor.getName();
		Trace("------AADLColAPI: Processor_Get_Binding_Process: " + subprocessor_name);   

		if (sysimpl.getAllPropertyAssociations() != null){
			EList<PropertyAssociation> listPropertyAssociation = sysimpl.getAllPropertyAssociations();		
			Iterator<?> it1 = listPropertyAssociation.iterator();

			while (it1.hasNext())
			{
				PropertyAssociation property_association = (PropertyAssociation) it1.next();
				Property propertyDef = property_association.getProperty();

				if (propertyDef.getName().equals("Actual_Processor_Binding") ){
					//Trace("propertyDef.getName() == Actual_Processor_Binding");

					EList<ModalPropertyValue> list_property_value = property_association.getOwnedValues();

					for (int i = 0; i < list_property_value.size(); i++) {  
						ModalPropertyValue value1 = list_property_value.get(i);

						if(value1.getOwnedValue() instanceof ListValue) {
							ListValue v1 = (ListValue) value1.getOwnedValue();
							EList<PropertyExpression> list1 =  v1.getOwnedListElements();

							if(list1 != null){

								for(int n=0; n<list1.size(); n++){
									if(list1.get(n) instanceof ReferenceValue){
										ReferenceValue property_value = (ReferenceValue) list1.get(n);
										EList<ContainmentPathElement>  list_reference_element = property_value.getContainmentPathElements();

										for (int j = 0; j < list_reference_element.size(); j++) { 
											ContainmentPathElement e1=  list_reference_element.get(j);

											if (e1.getNamedElement() instanceof ProcessorSubcomponent) {
												ProcessorSubcomponent a_subprocessor = (ProcessorSubcomponent) e1.getNamedElement();
												String a_subprocessor_name = a_subprocessor.getName();

												if (a_subprocessor_name.equals(subprocessor_name)){							
													EList<ContainedNamedElement> list_applyto = property_association.getAppliesTos();

													if (list_applyto != null){
														for (int k = 0; k < list_applyto.size(); k++) { 
															ContainedNamedElement e2 = list_applyto.get(k);
															EList<ContainmentPathElement> list2 = e2.getContainmentPathElements();

															if(list2 != null){
																for (int m =0; m< list2.size(); m++){
																	ContainmentPathElement e3 = list2.get(m);

																	if (e3.getNamedElement() instanceof ProcessSubcomponent) {				        							
																		ProcessSubcomponent a_process_subcomponent =  (ProcessSubcomponent) e3.getNamedElement();
																		list_process_subcomponent.add(a_process_subcomponent);
																	}	
																}//end for m
															}//end list2 != null

														}//end for k
													}//end list_applyto != null

												}//end equals
											}//end instanceof ProcessorSubcomponent
										}//end for j			    	
									}	//end for ReferenceValue
								} //end for n loop
							} //end list1!=null

						}//end of ListValue
					}//end of i loop
				}	//end of equals
			}//end of while
		}	//end of if

		if (list_process_subcomponent.size()== 0){
			if (sysimpl.getOwnedProcessSubcomponents() != null){
				EList <ProcessSubcomponent> l1 = sysimpl.getOwnedProcessSubcomponents();
				if(l1.size()>0){
					for(int m =0; m< l1.size(); m++){
						ProcessSubcomponent s1 = l1.get(m);
						list_process_subcomponent.add(s1);
					}
				}			
			}		
		}

		return list_process_subcomponent;
	}



	/**
	 * Get a processor that a process (subprocess) is bound to
	 * 
	 * @param sysimpl a system implementation that contains this process
	 * @param subprocess a process subcomponent
	 * @return A processor subcomponent that bound this subprocess is bound to
	 */
	public ProcessorSubcomponent Process_Get_Binding_Processor(SystemImplementation sysimpl, ProcessSubcomponent subprocess){

		ProcessorSubcomponent subprocessor = null;
		// String subprocess_name = subprocess.getName();

		ArrayList<PropertyAssociation> list1 = ComponentImpl_GetPropertyAssociations(sysimpl);
		for(int i=0; i<list1.size(); i++){
			PropertyAssociation pa = list1.get(i);
			String pa_name = Property_Get_Name(pa);

			if (pa_name.equals("Actual_Processor_Binding") ){			
				EList<ContainedNamedElement> cne_list = pa.getAppliesTos();

				if(cne_list != null){
					for(int j =0; j< cne_list.size(); j++){
						ContainedNamedElement cne = cne_list.get(j);
						EList<ContainmentPathElement> cpe_list = cne.getContainmentPathElements();

						if(cpe_list != null){
							for(int k =0; k< cpe_list.size(); k++){
								ContainmentPathElement cpe = cpe_list.get(k);
								// NamedElement ne = cpe.getNamedElement();

								if (cpe instanceof ProcessSubcomponent) {				        							
									ProcessSubcomponent a_process =  (ProcessSubcomponent) cpe;

									if (a_process == subprocess){
										EList<ModalPropertyValue> mpv_list = pa.getOwnedValues();	

										if(mpv_list != null){
											for(int m=0; m< mpv_list.size(); m++){
												ModalPropertyValue mpv= mpv_list.get(m);
												if (mpv.getOwnedValue() instanceof ReferenceValue) {
													ReferenceValue property_value = (ReferenceValue) mpv.getOwnedValue();
													EList<ContainmentPathElement> cpe_list2 = property_value.getContainmentPathElements();

													if(cpe_list2 != null){
														for (int n = 0; n < cpe_list2.size(); n++) { 

															ContainmentPathElement cpe2 = cpe_list2.get(n);
															if (cpe2.getNamedElement() instanceof ProcessorSubcomponent) {
																ProcessorSubcomponent a_subprocessor = (ProcessorSubcomponent) cpe2.getNamedElement();
																subprocessor = a_subprocessor;   									    
															}
														}
													}								    		
												}	
											}
										}				 
									}   				    
								}
							}
						}
					}				    
				}
			}
		}

		return subprocessor;
	}


	/**
	 * Get a list of (data) port connections between processes, where the two processes are bound to a same processor (subprocessor)
	 * 
	 * @param sysimpl a system implemenation that contains this subprocessor
	 * @param subprocessor a sub processor component
	 * @return An ArrayList of data port connections
	 */
	public ArrayList<PortConnection> Get_ProcessDataConnection_SameProcessor(SystemImplementation sysimpl, ProcessorSubcomponent subprocessor){

		ArrayList<PortConnection> connections = ComponentImpl_GetDataPortConnections(sysimpl);
		ArrayList<PortConnection> list = new ArrayList<PortConnection>();

		if(connections.size()>0){		
			ArrayList<ProcessSubcomponent> lis_subprocess = Processor_Get_Binding_Process(sysimpl, subprocessor);

			for(int i=0; i< connections.size(); i++){
				PortConnection conn1 = connections.get(i);

				if((PortConnection_GetSrcContext(conn1) instanceof ProcessSubcomponent) 
						&& (PortConnection_GetDstContext(conn1) instanceof ProcessSubcomponent)){
					ProcessSubcomponent src_context = (ProcessSubcomponent) PortConnection_GetSrcContext(conn1);
					ProcessSubcomponent dst_context = (ProcessSubcomponent) PortConnection_GetDstContext(conn1);

					if(lis_subprocess.contains(src_context) && lis_subprocess.contains(dst_context)){
						list.add(conn1);
					}
				}
			}

		}

		return list;
	}


	/**
	 * Get a list of event port connections between processes, 
	 * where the two processes are bound to a same processor (subprocessor)
	 * 
	 * @param sysimpl a system implementation that contains this subprocessor
	 * @param subprocessor a sub processor component
	 * @return An ArrayList of (event) port connections that 
	 * the source and destination context are sub-processes bound to processor (subprocessor)
	 */
	public ArrayList<PortConnection> Get_ProcessEventConnection_SameProcessor(SystemImplementation sysimpl, ProcessorSubcomponent subprocessor){

		ArrayList<PortConnection> list = new ArrayList<PortConnection>();	
		if( ComponentImpl_GetEventPortConnections(sysimpl) !=null){
			ArrayList<PortConnection> list_EventConnections = ComponentImpl_GetEventPortConnections(sysimpl);

			if(list_EventConnections.size()>0){
				ArrayList<ProcessSubcomponent> lis_subprocess = Processor_Get_Binding_Process(sysimpl, subprocessor);

				for(int i=0; i< list_EventConnections.size(); i++){
					PortConnection conn1 = list_EventConnections.get(i);
					if((PortConnection_GetSrcContext(conn1) instanceof ProcessSubcomponent)
							&& (PortConnection_GetDstContext(conn1) instanceof ProcessSubcomponent)){
						ProcessSubcomponent src_context = (ProcessSubcomponent) PortConnection_GetSrcContext(conn1);
						ProcessSubcomponent dst_context = (ProcessSubcomponent) PortConnection_GetDstContext(conn1);

						if(lis_subprocess.contains(src_context) && lis_subprocess.contains(dst_context)){
							list.add(conn1);
						}
					}
				}
			}

		}

		return list;
	}


	/**
	 * Get a list of event data port connections between processes, 
	 * where the two processes are bound to a same processor (subprocessor)
	 * 
	 * @param sysimpl a system implementation that contains this subprocessor
	 * @param subprocessor a sub processor component
	 * @return An ArrayList of (event data) port connections that 
	 * the source and destination context are sub-processes bound to processor (subprocessor)
	 */
	public ArrayList<PortConnection> Get_ProcessEventDataConnection_SameProcessor(SystemImplementation sysimpl, ProcessorSubcomponent subprocessor){

		ArrayList<PortConnection> list = new ArrayList<PortConnection>();    
		if(ComponentImpl_GetEventDataPortConnections(sysimpl) != null){

			ArrayList<PortConnection> list_EventDataConnections = ComponentImpl_GetEventDataPortConnections(sysimpl);

			if(list_EventDataConnections.size()>0){
				ArrayList<ProcessSubcomponent> lis_subprocess = Processor_Get_Binding_Process(sysimpl, subprocessor);

				for(int i=0; i< list_EventDataConnections.size(); i++){
					PortConnection conn1 = list_EventDataConnections.get(i);

					if((PortConnection_GetSrcContext(conn1) instanceof ProcessSubcomponent) 
							&& (PortConnection_GetDstContext(conn1) instanceof ProcessSubcomponent)){
						ProcessSubcomponent src_context = (ProcessSubcomponent) PortConnection_GetSrcContext(conn1);
						ProcessSubcomponent dst_context = (ProcessSubcomponent) PortConnection_GetDstContext(conn1);

						if(lis_subprocess.contains(src_context) && lis_subprocess.contains(dst_context)){
							list.add(conn1);
						}
					}
				}
			}   	
		}

		return list;
	}


	/**
	 * Determine whether a port connection (pc) is data port connection. 
	 * If both the source and destination end are data ports, then return true, else false.
	 * 
	 * @param pc a port connection
	 * @return a boolean 
	 */
	public boolean Is_DataPortConnection(PortConnection pc){

		// Trace("========= AADLCoLAPI: Is_DataPortConnection() : " + pc.getName());
		ConnectedElement src_element = (ConnectedElement) pc.getSource();
		ConnectedElement dst_element = (ConnectedElement) pc.getDestination();
		ConnectionEnd src = src_element.getConnectionEnd();
		ConnectionEnd dst = dst_element.getConnectionEnd();
		return ((src instanceof DataPort) && (dst instanceof DataPort));
	}


	/**
	 * Determine whether a port connection (pc) is event port connection. 
	 * If both the source and destination end are event ports, then return true, else false.
	 * 
	 * @param pc a port connection
	 * @return a boolean 
	 */
	public boolean Is_EventPortConnection(PortConnection pc){

		ConnectedElement src_element = (ConnectedElement) pc.getSource();
		ConnectedElement dst_element = (ConnectedElement) pc.getDestination();
		ConnectionEnd src = src_element.getConnectionEnd();
		ConnectionEnd dst = dst_element.getConnectionEnd();
		return ((src instanceof EventPort) && (dst instanceof EventPort));
	}


	/**
	 * Determine whether a port connection (pc) is event data port connection. 
	 * If both the source and destination end are event data ports, then return true, else false.
	 * 
	 * @param pc a port connection
	 * @return a boolean 
	 */
	public boolean Is_EventDataPortConnection(PortConnection pc){

		// Trace("========= AADLCoLAPI: Is_EventDataPortConnection() : " + pc.getName()  );
		ConnectedElement src_element = (ConnectedElement) pc.getSource();
		ConnectedElement dst_element = (ConnectedElement) pc.getDestination();
		ConnectionEnd src = src_element.getConnectionEnd();
		ConnectionEnd dst = dst_element.getConnectionEnd();
		return  ((src instanceof EventDataPort) && (dst instanceof EventDataPort));
	}


	/**
	 * Determine whether a access connection (pc) is bus access connection. 
	 * 
	 * @param ac a access connection
	 * @return a boolean 
	 */
	public boolean Is_BusAccessConnection(AccessConnection ac){

		// Trace("========= AADLCoLAPI: Is_BusAccessConnection() : " + ac.getName());
		AccessCategory cat = ac.getAccessCategory();
		String cat_str = cat.getName();
		return cat_str.equals("bus"); 
	}


	/**
	 * Determine whether a access connection (pc) is data access connection. 
	 * 
	 * @param ac a access connection
	 * @return a boolean 
	 */
	public boolean Is_DataAccessConnection(AccessConnection ac){

		//Trace("========= AADLCoLAPI: Is_BusAccessConnection() : " +  ac.getName() );
		AccessCategory cat = ac.getAccessCategory();
		String cat_str = cat.getName();
		return cat_str.equals("data");
	}


	/**
	 * Determine whether a access connection (pc) is subprogram access connection. 
	 * 
	 * @param ac a access connection
	 * @return a boolean 
	 */
	public boolean Is_SubprogramAccessConnection(AccessConnection ac){

		//Trace("========= AADLCoLAPI: Is_BusAccessConnection() : " + ac.getName() );
		AccessCategory cat = ac.getAccessCategory();
		String cat_str = cat.getName();
		return cat_str.equals("subprogram");
	}


	/**
	 * Determine whether a access connection (pc) is subprogram group access connection. 
	 * 
	 * @param ac a access connection
	 * @return a boolean 
	 */
	public boolean Is_SubprogramGroupAccessConnection(AccessConnection ac){

		AccessCategory cat = ac.getAccessCategory();
		String cat_str = cat.getName();
		return cat_str.equals("subprogramGroup");
	}


	/**
	 * Returns whether a subprogram access (sp_access) is remote. 
	 * If sp_access is a subcomponent of other thread rather than ct (supposes that ct is a thread type), 
	 * then it is considered as a remote call, 
	 * otherwise, it is a local call.
	 * 
	 * @param sp_access a subprogram access feature
	 * @param ct a component type that contains this subprogram access feature
	 * @param ap a package
	 * @return
	 */
	public boolean Is_SubprogramAccess_Remote(SubprogramAccess sp_access, ComponentType ct, AadlPackage ap){

		ArrayList<ThreadImplementation> ti_list = AadlPackage_GetThreadImpl(ap);
		/*Component LOIC OCTOBER 2013*/Classifier spc = sp_access.getClassifier();

		//if sp_access is a subcomponent of thread t1
		for(int i=0; i<ti_list.size(); i++){
			ThreadImplementation t1 = ti_list.get(i);
			ArrayList<SubprogramSubcomponent> sub_sps = Thread_GetSubcomponent_Subprograms(t1);

			for(int j=0; j< sub_sps.size(); j++){
				SubprogramSubcomponent sub_sp = sub_sps.get(j);

				if(spc instanceof SubprogramType){
					SubprogramType spc1 = (SubprogramType) spc;
					if((sub_sp.getClassifier() instanceof SubprogramType) ){
						SubprogramType spt1 = (SubprogramType) sub_sp.getClassifier();
						// if spc1 is a subcomponent of t1, and t1's type is not ct
						if(spt1.equals(spc1) & !t1.getType().equals(ct)){
							return true;
						}
					}
				}
				else if(spc instanceof SubprogramImplementation){
					SubprogramImplementation spc2 = (SubprogramImplementation) spc;
					if((sub_sp.getClassifier() instanceof SubprogramImplementation) ){
						SubprogramImplementation spt2 = (SubprogramImplementation) sub_sp.getClassifier();
						// if spc2 is a subcomponent of t1, and t1's type is not ct
						if(spt2.equals(spc2) & !t1.getType().equals(ct)){
							return true;
						}
					}
				}
			}//end for j
		}//end for i

		return false;
	}


	/**
	 * Returns whether a subprogram call (sp_call) is a remote call. 
	 * If sp_call is a subcomponent of other thread rather than t, then it is considered as a remote call, 
	 * otherwise, it is a local call
	 * 
	 * @param sp_call
	 * @param t
	 * @param ap a package
	 * @return
	 */
	public boolean Is_SubprogramCall_Remote(SubprogramCall sp_call, BehavioredImplementation t, AadlPackage ap){

		CalledSubprogram called_sp = sp_call.getCalledSubprogram();
		ArrayList<ThreadImplementation> ti_list = AadlPackage_GetThreadImpl(ap);

		//if called_sp is a subcomponent of thread t1
		for(int i=0; i<ti_list.size(); i++){
			ThreadImplementation t1 = ti_list.get(i);
			ArrayList<SubprogramSubcomponent> sub_sps = Thread_GetSubcomponent_Subprograms(t1);

			for(int j=0; j< sub_sps.size(); j++){
				SubprogramSubcomponent sub_sp = sub_sps.get(j);

				if(called_sp instanceof SubprogramType){
					SubprogramType spc1 = (SubprogramType) called_sp;
					if((sub_sp.getClassifier() instanceof SubprogramType) ){
						SubprogramType spt1 = (SubprogramType) sub_sp.getClassifier();
						// if spc1 is a subcomponent of t1, and t1 is not t
						if(spt1.equals(spc1) & !t1.equals(t)){
							return true;
						}
					}
				}
				else if(called_sp instanceof SubprogramImplementation){
					SubprogramImplementation spc2 = (SubprogramImplementation) called_sp;
					if((sub_sp.getClassifier() instanceof SubprogramImplementation) ){
						SubprogramImplementation spt2 = (SubprogramImplementation) sub_sp.getClassifier();
						// if spc2 is a subcomponent of t1, and t1 is not t
						if(spt2.equals(spc2) & !t1.equals(t)){
							return true;
						}
					}
				}
			}//end for j
		}//end for i
		return false;
	}


	/**
	 * Returns whether a subprogram type spt is a subcomponent of component implementation ci
	 * 
	 * @param spt
	 * @param ci
	 * @return
	 */
	public boolean Is_SubprogramType_Sub_Of_ComponentImpl(SubprogramType spt, ComponentImplementation ci){

		ArrayList<SubprogramSubcomponent> sub_sps = new ArrayList<SubprogramSubcomponent>();

		if(ci instanceof ProcessImplementation){
			sub_sps = Process_GetSubcomponents_Subprogram((ProcessImplementation) ci);
		}
		else if(ci instanceof ThreadImplementation){
			sub_sps = Thread_GetSubcomponent_Subprograms((ThreadImplementation) ci);
		}

		for(int i=0; i< sub_sps.size(); i++){
			SubprogramSubcomponent sub_sp1 = sub_sps.get(i);
			ComponentClassifier spc1 = sub_sp1.getClassifier();
			if((spc1 instanceof SubprogramType) & ( ((SubprogramType) spc1).equals(spt))){
				return true;
			}
		}

		return false;
	}


	/**
	 * Returns whether a subprogram implementation spi is a subcomponent of component implementation ci
	 * 
	 * @param spi
	 * @param ci
	 * @return
	 */
	public boolean Is_SubprogramImpl_Sub_Of_ComponentImpl(SubprogramImplementation spi, ComponentImplementation ci){

		ArrayList<SubprogramSubcomponent> sub_sps = new ArrayList<SubprogramSubcomponent>();

		if(ci instanceof ProcessImplementation){
			sub_sps = Process_GetSubcomponents_Subprogram((ProcessImplementation) ci);
		}
		else if(ci instanceof ThreadImplementation){
			sub_sps = Thread_GetSubcomponent_Subprograms((ThreadImplementation) ci);
		}

		for(int i=0; i< sub_sps.size(); i++){
			SubprogramSubcomponent sub_sp1 = sub_sps.get(i);
			ComponentClassifier spc1 = sub_sp1.getClassifier();
			if((spc1 instanceof SubprogramImplementation) & ( ((SubprogramImplementation) spc1).equals(spi))){
				return true;
			}
		}

		return false;
	}



	/**
	 * Returns whether a subprogram calssifier sp is a subcomponent of a component implementation ci
	 * 
	 * @param sp
	 * @param ci
	 * @return
	 */
	public boolean Is_SubprogramClassifier_Sub_Of_ComponentImpl(SubprogramClassifier sp, ComponentImplementation ci){

		if(sp instanceof SubprogramType){
			return Is_SubprogramType_Sub_Of_ComponentImpl((SubprogramType) sp, ci);
		}
		else if(sp instanceof SubprogramImplementation){
			return Is_SubprogramImpl_Sub_Of_ComponentImpl((SubprogramImplementation) sp, ci);
		}

		return false;
	}

	/**
	 * Returns whether a called subprogram call_sp is a subcomponent of component implementation ci
	 * 
	 * @param call_sp
	 * @param ci
	 * @return
	 */
	public boolean Is_CalledSubprogram_Sub_Of_ComponentImpl(CalledSubprogram call_sp, ComponentImplementation ci){

		if(call_sp instanceof SubprogramType){
			return Is_SubprogramType_Sub_Of_ComponentImpl((SubprogramType) call_sp, ci);
		}
		else if(call_sp instanceof SubprogramImplementation){
			return Is_SubprogramImpl_Sub_Of_ComponentImpl((SubprogramImplementation) call_sp, ci);
		}

		return false;   	
	}


	/**
	 * Returns whether a subprogram type spt is a subcomponent of any process in AADL package aadlpkg
	 * 
	 * @param spt a subprogram type
	 * @param aadlpkg an AADL package
	 * 
	 * @return true if it is a subcomponent of a process, else false
	 */
	public boolean Is_SubprogramType_Sub_Of_Any_Process(SubprogramType spt, AadlPackage aadlpkg){

		ArrayList<ProcessImplementation> processimpls = AadlPackage_GetProcessImpl(aadlpkg);

		for(int i =0; i< processimpls.size(); i++){
			ProcessImplementation p1 = processimpls.get(i);
			if(Is_SubprogramType_Sub_Of_ComponentImpl(spt, p1)){//if spt is a sub component of process p1
				return true;
			}
		}
		return false;
	}


	/**
	 * Returns whether a subprogram impl sp is a subcomponent of any process in AADL package aadlpkg
	 * 
	 * @param sp a subprogram impl
	 * @param aadlpkg an AADL package
	 * 
	 * @return true if it is a subcomponent of a process, else false
	 */
	public boolean Is_SubprogramImpl_Sub_Of_Any_Process(SubprogramImplementation sp, AadlPackage aadlpkg){

		ArrayList<ProcessImplementation> processimpls = AadlPackage_GetProcessImpl(aadlpkg);
		for(int i =0; i< processimpls.size(); i++){
			ProcessImplementation p1 = processimpls.get(i);
			if(Is_SubprogramImpl_Sub_Of_ComponentImpl(sp, p1)){//if spt is a sub component of process p1
				return true;
			}
		}
		return false;
	}


	/**
	 * Returns whether a subprogram type spt is a subcomponent of any thread in AADL package aadlpkg
	 * 
	 * @param spt a subprogram type
	 * @param aadlpkg an AADL package
	 * 
	 * @return true if it is a subcomponent of a thread, else false
	 */
	public boolean Is_SubprogramType_Sub_Of_Any_Thread(SubprogramType spt, AadlPackage aadlpkg){

		ArrayList<ThreadImplementation> threadimpls = AadlPackage_GetThreadImpl(aadlpkg);
		for(int i =0; i< threadimpls.size(); i++){
			ThreadImplementation p1 = threadimpls.get(i);
			if(Is_SubprogramType_Sub_Of_ComponentImpl(spt, p1)){//if spt is a sub component of process p1
				return true;
			}
		}
		return false;
	}


	/**
	 * Returns whether a subprogram impl sp is a subcomponent of any thread in AADL package aadlpkg
	 * 
	 * @param sp a subprogram impl
	 * @param aadlpkg an AADL package
	 * 
	 * @return true if it is a subcomponent of a thread, else false
	 */
	public boolean Is_SubprogramImpl_Sub_Of_Any_Thread(SubprogramImplementation sp, AadlPackage aadlpkg){

		ArrayList<ThreadImplementation> threadimpls = AadlPackage_GetThreadImpl(aadlpkg);
		for(int i =0; i< threadimpls.size(); i++){
			ThreadImplementation p1 = threadimpls.get(i);
			if(Is_SubprogramImpl_Sub_Of_ComponentImpl(sp, p1)){//if spt is a sub component of process p1
				return true;
			}
		}
		return false;
	}


	/**
	 * Returns whether there exists a connection from sub process ps1's port p1  to system sysimpl in sysimpl
	 * 
	 * @param sysimpl a system implementation
	 * @param ps1 a sub process of system sysimpl
	 * @param p1 a port of sub process ps1
	 * @return whether there is a connection from port p1 of sub process ps1 to system sysimpl
	 */
	public boolean Exist_Connection_SubProcess_to_System(SystemImplementation sysimpl, ProcessSubcomponent ps1, Port p1){

		EList<PortConnection> list1 = ComponentImpl_GetPortConnections(sysimpl);
		if( list1!= null){

			for(int i =0; i< list1.size(); i++){
				PortConnection pc1 = list1.get(i);
				Context src_context = Connection_GetSrcContext(pc1);
				Context dst_context = Connection_GetDstContext(pc1);
				if((src_context == ps1) && (dst_context == null)){ // if connection ps1 -> sysimpl
					Port src_element = (Port) PortConnection_GetSrc(pc1);
					if(src_element == p1){
						return true;
					}
				}   			
			}
		}

		return false;
	}



	/**
	 * Returns string name of a connection
	 * @param cc
	 * @return
	 */
	public String Connection_GetName(Connection cc){
		return cc.getName(); 
	}


	/**
	 * Get the source of a connection
	 * 
	 * @param cc a  connection
	 * @return 
	 */
	public ConnectionEnd Connection_GetSrc(Connection cc){
		//return  cc.getAllSource();
		return ( (ConnectedElement) cc.getSource()).getConnectionEnd();
	}


	/**
	 * Get the destination  of a connection
	 * 
	 * @param cc 
	 * @return 
	 */
	public ConnectionEnd Connection_GetDst(Connection cc){
		//return cc.getAllDestination();
		return ( (ConnectedElement) cc.getDestination()).getConnectionEnd();
	}


	/**
	 * Get the source context of a  connection
	 * 
	 * @param cc a  connection
	 * @return 
	 */
	public  Context Connection_GetSrcContext(Connection cc){
		return  cc.getAllSourceContext();
	}


	/**
	 * Get the destination context of a connection
	 * 
	 * @param cc a  connection
	 * @return 
	 */
	public  Context Connection_GetDstContext(Connection cc){
		return  cc.getAllDestinationContext();
	}

	/**
	 * Get the string value of some part of a Latency property that applies a connection (pconnection)
	 * 
	 * @param compimpl a component implementation that contains this connection (pconnection) 
	 * @param pconnection a (data port) connection
	 * @param part a string that represents which part of Latency, it could be MinUnit, MaxUnit, MinValue, MaxValue
	 * 
	 * @return a string value of Latency's min unit 
	 */
	public String Connection_Get_Latency_X(ComponentImplementation compimpl, Connection pconnection, String X){

		EList<PropertyAssociation> property_association_list = compimpl.getAllPropertyAssociations();
		ArrayList<PropertyAssociation> list_p = new ArrayList<PropertyAssociation>();

		Iterator<?> it = property_association_list.iterator();
		while (it.hasNext())
		{
			PropertyAssociation s = (PropertyAssociation) it.next();
			list_p.add(s);
		}

		Iterator<PropertyAssociation> it2 = list_p.iterator();
		while (it2.hasNext())
		{
			PropertyAssociation pa = (PropertyAssociation) it2.next();
			Property propertyDef = pa.getProperty();
			String propertyDef_name = propertyDef.getName();
			// Trace("---Property Name: " + propertyDef_name);

			if (propertyDef_name.equals("Latency")) 
			{
				EList<ContainedNamedElement> list_applyto = pa.getAppliesTos();
				for (int i = 0; i < list_applyto.size(); i++) { 
					ContainedNamedElement applyto = list_applyto.get(i);
					EList<ContainmentPathElement> cpe_list =applyto.getContainmentPathElements();

					if(cpe_list != null){
						for(int k =0; k< cpe_list.size(); k++){
							ContainmentPathElement cpe = cpe_list.get(k);
							String name1 = cpe.getNamedElement().getName();

							//if Latency applies to pconnection
							if (name1 == pconnection.getName()){
								EList<ModalPropertyValue> listPropertyValue = pa.getOwnedValues();

								if(listPropertyValue != null){
									for (int ii = 0; ii < listPropertyValue.size(); i++) {  
										ModalPropertyValue value1 = listPropertyValue.get(ii);

										if (value1.getOwnedValue() instanceof RangeValue) {
											RangeValue propertyValue = (RangeValue) value1.getOwnedValue();

											if(X.equals("MinUnit")){
												return IntegerLiteralRange_Get_MinUnit(propertyValue);
											}
											else if(X.equals("MaxUnit")){
												return IntegerLiteralRange_Get_MaxUnit(propertyValue);
											}
											else if(X.equals("MinValue")){
												return IntegerLiteralRange_Get_MinValue(propertyValue);
											}
											else if(X.equals("MaxValue")){
												return IntegerLiteralRange_Get_MaxValue(propertyValue);
											}

										}
									}
								}	    	        				
							}
						}
					}	        			
				}           	
			}
		}			

		if(X.equals("MinUnit") | X.equals("MaxUnit")){
			return "Ns";
		}
		else if(X.equals("MinValue") | X.equals("MaxValue")){
			return "0";
		}
		else {
			return null;
		}
	}


	/**
	 * Get the unit of minimum of a Latency property that applies a connection (pconnection), "Ns" by default
	 * 
	 * @param compimpl a component implementation that contains this connection (pconnection) 
	 * @param pconnection a (data port) connection
	 * @return a string value of Latency's min unit 
	 */
	public String Connection_Get_Latency_MinUnit(ComponentImplementation compimpl, Connection pconnection){

		return Connection_Get_Latency_X(compimpl, pconnection, "MinUnit");
	}


	/**
	 * Get the unit of maximum of a Latency property that applies a connection (pconnection), "Ns" by default
	 * 
	 * @param compimpl a component implementation that contains this connection (pconnection) 
	 * @param pconnection a (data port) connection
	 * @return a string value of Latency's max unit 
	 */
	public String Connection_Get_Latency_MaxUnit(ComponentImplementation compimpl, Connection pconnection){
		return Connection_Get_Latency_X(compimpl, pconnection, "MaxUnit");
	}


	/**
	 * Get the minimum value of a Latency property that applies a connection (pconnection)
	 * 
	 * @param compimpl a component implementation that contains this connection (pconnection) 
	 * @param pconnection a (data port) connection
	 * @return a string value of Latency's minimum number 
	 */
	public String Connection_Get_Latency_MinOffset(ComponentImplementation compimpl, Connection pconnection){
		return Connection_Get_Latency_X(compimpl, pconnection, "MinValue");
	}


	/**
	 * Get the maximum value of a Latency property that applies a connection (pconnection)
	 * 
	 * @param compimpl a component implementation that contains this connection (pconnection) 
	 * @param pconnection a (data port) connection
	 * @return a string value of Latency's maximum number 
	 */
	public String Connection_Get_Latency_MaxOffset(ComponentImplementation compimpl, Connection pconnection){
		return Connection_Get_Latency_X(compimpl, pconnection, "MaxValue");
	}




	/**
	 * Get the source port of a port connection
	 * 
	 * @param cc a port connection
	 * @return source port
	 */
	public Port PortConnection_GetSrc(PortConnection cc){
		return (Port) ( (ConnectedElement) cc.getSource()).getConnectionEnd();
	}


	/**
	 * Get the destination port of a port connection
	 * 
	 * @param cc a port connection
	 * @return destination port
	 */
	public Port PortConnection_GetDst(PortConnection cc){
		return (Port) ( (ConnectedElement) cc.getDestination()).getConnectionEnd();
	}


	/**
	 * Get the source context of a port connection
	 * 
	 * @param cc a port connection
	 * @return source context: a subcomponent or a component implementation 
	 */
	public  Context PortConnection_GetSrcContext(PortConnection cc){
		return  cc.getAllSourceContext();
	}


	/**
	 * Get the destination context of a port connection
	 * 
	 * @param cc a port connection
	 * @return dst context: a subcomponent or a component implementation 
	 */
	public  Context PortConnection_GetDstContext(PortConnection cc){
		return  cc.getAllDestinationContext();
	}


	/**
	 * Get the source of a access connection
	 * 
	 * @param cc a access connection
	 * @return source 
	 */
	public ConnectionEnd AccessConnection_GetSrc(AccessConnection cc){
		return  ((ConnectedElement) cc.getSource()).getConnectionEnd();
	}


	/**
	 * Get the destination of a access connection
	 * 
	 * @param cc a access connection
	 * @return destination 
	 */
	public ConnectionEnd AccessConnection_GetDst(AccessConnection cc){
		return  ((ConnectedElement) cc.getDestination()).getConnectionEnd();
	}


	/**
	 * Get the source context of a access connection
	 * 
	 * @param cc a access connection
	 * @return source context: a subcomponent or a component implementation 
	 */
	public  Context AccessConnection_GetSrcContext(AccessConnection cc){
		return  cc.getAllSourceContext();
	}


	/**
	 * Get the destination context of a access connection
	 * 
	 * @param cc a access connection
	 * @return dst context: a subcomponent or a component implementation 
	 */
	public  Context AccessConnection_GetDstContext(AccessConnection cc){
		return  cc.getAllDestinationContext();
	}


	/**
	 * Get the access category: bus, data, ...
	 * 
	 * @param cc access connection
	 * @return a string that represents the access connection category
	 */
	public String AccessConnection_GetCategory(AccessConnection cc){
		return cc.getAccessCategory().toString();
	}


	/**
	 * Returns string name of a flow specification
	 * @param fs
	 * @return
	 */
	public String FlowSpec_GetName(FlowSpecification fs){
		return fs.getName();
	}


	/**
	 * Returns a list of property associations of a flow specification
	 * @param fs
	 * @return
	 */
	public ArrayList<PropertyAssociation> FlowSpec_GetPropertyAssociation(FlowSpecification fs){
		ArrayList<PropertyAssociation> list= new ArrayList<PropertyAssociation>();
		EList<PropertyAssociation> list1 = fs.getOwnedPropertyAssociations();
		if(list1 != null){
			if(list1.size()>0){
				list.addAll(list1);
			}
		}

		return list;
	}


	/**
	 * Returns string name of a end to end flow
	 * @param fs
	 * @return
	 */
	public String EndToEndFlow_GetName(EndToEndFlow fs){
		return fs.getName();
	}


	/**
	 * Returns a list of property associations of a end to end flow
	 * @param fs
	 * @return
	 */
	public ArrayList<PropertyAssociation> EndToEndFlow_GetPropertyAssociation(EndToEndFlow fs){
		ArrayList<PropertyAssociation> list= new ArrayList<PropertyAssociation>();
		EList<PropertyAssociation> list1 = fs.getOwnedPropertyAssociations();
		if(list1 != null){
			if(list1.size()>0){
				list.addAll(list1);
			}
		}

		return list;
	}


	/**
	 * Returns string name of a FlowImplementation
	 * @param fs
	 * @return
	 */
	public String FlowImplementation_GetName(FlowImplementation fs){
		return fs.getName();
	}


	/**
	 * Returns a list of property associations of a FlowImplementation
	 * @param fs
	 * @return
	 */
	public ArrayList<PropertyAssociation> FlowImplementation_GetPropertyAssociation(FlowImplementation fs){
		ArrayList<PropertyAssociation> list= new ArrayList<PropertyAssociation>();
		EList<PropertyAssociation> list1 = fs.getOwnedPropertyAssociations();
		if(list1 != null){
			if(list1.size()>0){
				list.addAll(list1);
			}
		}

		return list;
	}


	/**
	 * Return kind (source, sink, path) of a flow implementation
	 * @param f
	 * @return
	 */
	public String FlowImplementation_GetKind(FlowImplementation f){
		return f.getKind().toString();
	}


	/**
	 * Returns a list of ports that are defined in a feature group type fgt
	 * @param fgt
	 * @return
	 */
	public ArrayList<Port> FeatureGroupType_GetPorts(FeatureGroupType fgt){

		ArrayList<Port> list = new ArrayList<Port> ();
		EList<DataPort> list1 = fgt.getOwnedDataPorts();
		EList<EventPort> list2 = fgt.getOwnedEventPorts();
		EList<EventDataPort> list3 = fgt.getOwnedEventDataPorts();

		if(list1 != null){
			if(list1.size()>0){
				list.addAll(list1);
			}
		}

		if(list2 != null){
			if(list2.size()>0){
				list.addAll(list2);
			}
		}

		if(list3 != null){
			if(list3.size()>0){
				list.addAll(list3);
			}
		}

		return list;

	}


	/**
	 * Returns the inverse feature group type of a feature group type fgt
	 * @param fgt
	 * @return
	 */
	public FeatureGroupType FeatureGroupType_GetInverse(FeatureGroupType fgt){

		return fgt.getInverse();
	}


	/**
	 * Returns a list of subprogram call sequences of a behavioredImplementation component t, that is of type thread or subprogram
	 * @param t a behaviored implementaton component
	 * @return a list of subprogram call sequences
	 */
	public ArrayList<SubprogramCallSequence> BehavioredImpl_Get_CallSequences(BehavioredImplementation t){

		ArrayList<SubprogramCallSequence> list = new ArrayList<SubprogramCallSequence>();
		EList<SubprogramCallSequence> list1 = t.getOwnedSubprogramCallSequences();
		if(list1 != null){
			list.addAll(list1);
		}

		return list;
	}


	/**
	 * Returns a list of subprogram calls of a subprogram call sequence
	 * 
	 * @param callseq a subprogram call sequence
	 * @return a list of subprogram calls
	 */
	public ArrayList<SubprogramCall> SubprogramCallSequence_Get_SubprogramCalls(SubprogramCallSequence callseq){

		ArrayList<SubprogramCall> list = new ArrayList<SubprogramCall>();
		//	EList< CallSpecification > list1 = callseq.getOwnedCallSpecifications(); modif LBesnard Fev 2015. replaced by:
		EList<SubprogramCall> list1 = callseq.getOwnedSubprogramCalls();

		if(list1 != null){
			for(int i =0; i< list1.size(); i++){
				if(list1.get(i) instanceof SubprogramCall){
					SubprogramCall call1 = (SubprogramCall) list1.get(i);
					list.add(call1);
				}
			}
		}

		return list;
	}


	/**
	 * Returns the called subprogram of a subprogram call
	 * 
	 * @param sp_call
	 * @return
	 */
	public CalledSubprogram SubprogramCall_Get_CalledSubprogram(SubprogramCall sp_call){
		return sp_call.getCalledSubprogram();
	}


	public String CalledSubprogram_Get_Name(CalledSubprogram called_sp){

		if(called_sp instanceof SubprogramType){
			SubprogramType spt = (SubprogramType) called_sp;
			return spt.getName();
		}
		else{ //called_sp instanceof SubprogramImplementation
			SubprogramImplementation spi = (SubprogramImplementation) called_sp;
			return spi.getName();
		}
	}


}
